Package: ggpubr
Type: Package
Title: 'ggplot2' Based Publication Ready Plots
Version: 1.0.0
Date: 2026-07-06
Authors@R: c(
    person("Alboukadel", "Kassambara", role = c("aut", "cre"), email = "alboukadel.kassambara@gmail.com"),
    person("Laszlo", "Erdey", role = "ctb", email = "erdey.laszlo@econ.unideb.hu",
           comment = "Faculty of Economics and Business, University of Debrecen, Hungary"))
Author: Alboukadel Kassambara [aut, cre],
  Laszlo Erdey [ctb] (Faculty of Economics and Business, University of
    Debrecen, Hungary)
Maintainer: Alboukadel Kassambara <alboukadel.kassambara@gmail.com>
Description: The 'ggplot2' package is excellent and flexible for elegant data
    visualization in R. However the default generated plots require some formatting
    before we can send them for publication. Furthermore, to customize a 'ggplot',
    the syntax is opaque and this raises the level of difficulty for researchers
    with no advanced R programming skills. 'ggpubr' provides some easy-to-use
    functions for creating and customizing 'ggplot2'-based publication ready plots.
    This version includes modern R ecosystem compatibility updates and customizable
    p-value formatting presets (APA, AMA, NEJM, Lancet, GraphPad, and scientific
    notation) for publication workflows, plus robust sparse-subset handling in
    statistical annotation layers such as 'stat_compare_means()' and 'geom_pwc()',
    with informative per-group skip diagnostics for non-comparable subsets.
License: GPL (>= 2)
LazyData: TRUE
Encoding: UTF-8
Depends: R (>= 4.1.0), ggplot2 (>= 3.5.2)
Imports: ggrepel (>= 0.9.2), grid, ggsci, stats, utils, tidyr (>=
        1.3.2), purrr, dplyr (>= 1.2.0), cowplot (>= 1.2.0), ggsignif,
        scales, gridExtra, glue, polynom, rlang (>= 1.1.7), rstatix (>=
        1.0.0), tibble, magrittr
Suggests: grDevices, knitr, RColorBrewer, gtable, testthat
URL: https://rpkgs.datanovia.com/ggpubr/,
        https://github.com/kassambara/ggpubr
BugReports: https://github.com/kassambara/ggpubr/issues
RoxygenNote: 7.3.3
Collate: 'utilities_color.R' 'utilities_base.R' 'desc_statby.R'
        'utilities.R' 'add_summary.R' 'annotate_figure.R' 'as_ggplot.R'
        'as_npc.R' 'axis_scale.R' 'background_image.R' 'bgcolor.R'
        'border.R' 'p_format_utils.R' 'compare_means.R' 'create_aes.R'
        'diff_express.R' 'facet.R' 'font.R' 'gene_citation.R'
        'gene_expression.R' 'geom_bracket.R' 'geom_exec.R'
        'utils-aes.R' 'utils_stat_test_label.R' 'geom_pwc.R'
        'get_breaks.R' 'get_coord.R' 'get_legend.R' 'get_palette.R'
        'ggadd.R' 'ggadjust_pvalue.R' 'ggarrange.R' 'ggballoonplot.R'
        'ggpar.R' 'ggbarplot.R' 'ggboxplot.R' 'ggdensity.R' 'ggpie.R'
        'ggdonutchart.R' 'stat_conf_ellipse.R' 'stat_chull.R'
        'ggdotchart.R' 'ggdotplot.R' 'ggecdf.R' 'ggerrorplot.R'
        'ggexport.R' 'gghistogram.R' 'ggline.R' 'ggmaplot.R'
        'ggpaired.R' 'ggparagraph.R' 'ggpubr-package.R' 'ggpubr_args.R'
        'ggpubr_options.R' 'ggqqplot.R' 'utilities_label.R'
        'stat_cor.R' 'stat_stars.R' 'ggscatter.R' 'ggscatterhist.R'
        'ggstripchart.R' 'ggsummarystats.R' 'ggtext.R' 'ggtexttable.R'
        'ggviolin.R' 'gradient_color.R' 'grids.R' 'npc_to_data_coord.R'
        'reexports.R' 'rotate.R' 'rotate_axis_text.R' 'rremove.R'
        'set_palette.R' 'shared_docs.R' 'show_line_types.R'
        'show_point_shapes.R' 'stat_anova_test.R'
        'stat_central_tendency.R' 'stat_compare_means.R'
        'stat_friedman_test.R' 'stat_kruskal_test.R' 'stat_mean.R'
        'stat_overlay_normal_density.R' 'stat_pvalue_manual.R'
        'stat_regline_equation.R' 'stat_welch_anova_test.R'
        'text_grob.R' 'theme_pubr.R' 'theme_transparent.R'
        'utils-geom-signif.R' 'utils-pipe.R' 'utils-tidyr.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-07-06 10:18:45 UTC; kassambara
Repository: CRAN
Date/Publication: 2026-07-06 15:00:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(drawDetails,splitText)
S3method(print,ggscatterhist)
S3method(print,ggsummarystats)
S3method(print,ggsummarystats_list)
export("%>%")
export(add_summary)
export(annotate_figure)
export(as_ggplot)
export(background_image)
export(bgcolor)
export(border)
export(change_palette)
export(clean_table_theme)
export(clean_theme)
export(colnames_style)
export(color_palette)
export(compare_means)
export(create_aes)
export(desc_statby)
export(facet)
export(fill_palette)
export(font)
export(geom_bracket)
export(geom_exec)
export(geom_signif)
export(get_breaks)
export(get_legend)
export(get_palette)
export(get_summary_stats)
export(ggadd)
export(ggarrange)
export(ggballoonplot)
export(ggbarplot)
export(ggboxplot)
export(ggdensity)
export(ggdonutchart)
export(ggdotchart)
export(ggdotplot)
export(ggecdf)
export(ggerrorplot)
export(ggexport)
export(gghistogram)
export(ggline)
export(ggmaplot)
export(ggpaired)
export(ggpar)
export(ggparagraph)
export(ggpie)
export(ggqqplot)
export(ggscatter)
export(ggscatterhist)
export(ggstripchart)
export(ggsummarystats)
export(ggsummarytable)
export(ggtext)
export(ggtexttable)
export(ggviolin)
export(gradient_color)
export(gradient_fill)
export(grids)
export(group_by)
export(labs_pubr)
export(mean_ci)
export(mean_range)
export(mean_sd)
export(mean_se_)
export(median_hilow_)
export(median_iqr)
export(median_mad)
export(median_q1q3)
export(median_range)
export(mutate)
export(rotate)
export(rotate_x_text)
export(rotate_y_text)
export(rownames_style)
export(rremove)
export(set_palette)
export(show_line_types)
export(show_point_shapes)
export(stat_bracket)
export(stat_central_tendency)
export(stat_chull)
export(stat_compare_means)
export(stat_conf_ellipse)
export(stat_cor)
export(stat_mean)
export(stat_overlay_normal_density)
export(stat_pvalue_manual)
export(stat_regline_equation)
export(stat_stars)
export(tab_add_border)
export(tab_add_footnote)
export(tab_add_hline)
export(tab_add_title)
export(tab_add_vline)
export(tab_cell_crossout)
export(tab_ncol)
export(tab_nrow)
export(table_cell_bg)
export(table_cell_font)
export(tbody_add_border)
export(tbody_style)
export(text_grob)
export(thead_add_border)
export(theme_classic2)
export(theme_cleveland)
export(theme_pubclean)
export(theme_pubr)
export(theme_transparent)
export(ttheme)
export(xscale)
export(yscale)
import(ggplot2)
importFrom(dplyr,arrange)
importFrom(dplyr,desc)
importFrom(dplyr,do)
importFrom(dplyr,everything)
importFrom(dplyr,group_by)
importFrom(dplyr,group_nest)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(ggsignif,geom_signif)
importFrom(glue,glue)
importFrom(grid,drawDetails)
importFrom(magrittr,"%>%")
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,.data)
importFrom(rlang,syms)
importFrom(rstatix,df_arrange)
importFrom(rstatix,df_group_by)
importFrom(rstatix,df_nest_by)
importFrom(rstatix,df_select)
importFrom(rstatix,df_split_by)
importFrom(rstatix,get_summary_stats)
importFrom(tibble,as_tibble)
importFrom(tidyr,unite)

<!-- README.md is generated from README.Rmd. Please edit that file -->

[![R-CMD-check](https://github.com/kassambara/ggpubr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kassambara/ggpubr/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ggpubr)](https://cran.r-project.org/package=ggpubr)
[![Downloads](https://cranlogs.r-pkg.org/badges/ggpubr)](https://cran.r-project.org/package=ggpubr)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/ggpubr?color=orange)](https://cran.r-project.org/package=ggpubr)

# ggpubr: ‘ggplot2’ Based Publication Ready Plots

[ggplot2, by Hadley Wickham,](https://ggplot2.tidyverse.org/) is an
excellent and flexible package for elegant data visualization in R.
However the default generated plots requires some formatting before we
can send them for publication. Furthermore, to customize a ggplot, the
syntax is opaque and this raises the level of difficulty for researchers
with no advanced R programming skills.

The ‘ggpubr’ package provides some easy-to-use functions for creating
and customizing ‘ggplot2’- based publication ready plots.

Find out more at <https://rpkgs.datanovia.com/ggpubr/>.

## Installation and loading

- Install from [CRAN](https://cran.r-project.org/package=ggpubr) as
  follow:

``` r
install.packages("ggpubr")
```

- Or, install the latest version from
  [GitHub](https://github.com/kassambara/ggpubr) as follow:

``` r
# Install
if(!require(devtools)) install.packages("devtools")
devtools::install_github("kassambara/ggpubr")
```

## Distribution

``` r
library(ggpubr)
#> Loading required package: ggplot2
# Create some data format
# :::::::::::::::::::::::::::::::::::::::::::::::::::
set.seed(1234)
wdata = data.frame(
   sex = factor(rep(c("F", "M"), each=200)),
   weight = c(rnorm(200, 55), rnorm(200, 58)))
head(wdata, 4)
#>   sex   weight
#> 1   F 53.79293
#> 2   F 55.27743
#> 3   F 56.08444
#> 4   F 52.65430

# Density plot with mean lines and marginal rug
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change outline and fill colors by groups ("sex")
# Use custom palette
ggdensity(wdata, x = "weight",
   add = "mean", rug = TRUE,
   color = "sex", fill = "sex",
   palette = c("#00AFBB", "#E7B800"))
```

![](tools/README-ggpubr-1.png)<!-- -->

``` r
# Histogram plot with mean lines and marginal rug
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change outline and fill colors by groups ("sex")
# Use custom color palette
gghistogram(wdata, x = "weight",
   add = "mean", rug = TRUE,
   color = "sex", fill = "sex",
   palette = c("#00AFBB", "#E7B800"))
```

![](tools/README-ggpubr-2.png)<!-- -->

## Box plots and violin plots

``` r
# Load data
data("ToothGrowth")
df <- ToothGrowth
head(df, 4)
#>    len supp dose
#> 1  4.2   VC  0.5
#> 2 11.5   VC  0.5
#> 3  7.3   VC  0.5
#> 4  5.8   VC  0.5

# Box plots with jittered points
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change outline colors by groups: dose
# Use custom color palette
# Add jitter points and change the shape by groups
 p <- ggboxplot(df, x = "dose", y = "len",
                color = "dose", palette =c("#00AFBB", "#E7B800", "#FC4E07"),
                add = "jitter", shape = "dose")
 p
```

![](tools/README-ggpubr-box-plot-dot-plots-strip-charts-1.png)<!-- -->

``` r
 
 # Add p-values comparing groups
 # Specify the comparisons you want
my_comparisons <- list( c("0.5", "1"), c("1", "2"), c("0.5", "2") )
p + stat_compare_means(comparisons = my_comparisons)+ # Add pairwise comparisons p-value
  stat_compare_means(label.y = 50)                   # Add global p-value
```

![](tools/README-ggpubr-box-plot-dot-plots-strip-charts-2.png)<!-- -->

``` r

 
# Violin plots with box plots inside
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change fill color by groups: dose
# add boxplot with white fill color
ggviolin(df, x = "dose", y = "len", fill = "dose",
         palette = c("#00AFBB", "#E7B800", "#FC4E07"),
         add = "boxplot", add.params = list(fill = "white"))+
  stat_compare_means(comparisons = my_comparisons, label = "p.signif")+ # Add significance levels
  stat_compare_means(label.y = 50)                                      # Add global the p-value 
```

![](tools/README-ggpubr-box-plot-dot-plots-strip-charts-3.png)<!-- -->

## Bar plots

### Demo data set

Load and prepare data:

``` r
# Load data
data("mtcars")
dfm <- mtcars
# Convert the cyl variable to a factor
dfm$cyl <- as.factor(dfm$cyl)
# Add the name colums
dfm$name <- rownames(dfm)
# Inspect the data
head(dfm[, c("name", "wt", "mpg", "cyl")])
#>                                name    wt  mpg cyl
#> Mazda RX4                 Mazda RX4 2.620 21.0   6
#> Mazda RX4 Wag         Mazda RX4 Wag 2.875 21.0   6
#> Datsun 710               Datsun 710 2.320 22.8   4
#> Hornet 4 Drive       Hornet 4 Drive 3.215 21.4   6
#> Hornet Sportabout Hornet Sportabout 3.440 18.7   8
#> Valiant                     Valiant 3.460 18.1   6
```

### Ordered bar plots

Change the fill color by the grouping variable “cyl”. Sorting will be
done globally, but not by groups.

``` r
ggbarplot(dfm, x = "name", y = "mpg",
          fill = "cyl",               # change fill color by cyl
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palett. see ?ggpar
          sort.val = "desc",          # Sort the value in dscending order
          sort.by.groups = FALSE,     # Don't sort inside each group
          x.text.angle = 90           # Rotate vertically x axis texts
          )
```

![](tools/README-ordered-bar-plots-1.png)<!-- -->

Sort bars inside each group. Use the argument **sort.by.groups = TRUE**.

``` r
ggbarplot(dfm, x = "name", y = "mpg",
          fill = "cyl",               # change fill color by cyl
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palett. see ?ggpar
          sort.val = "asc",           # Sort the value in dscending order
          sort.by.groups = TRUE,      # Sort inside each group
          x.text.angle = 90           # Rotate vertically x axis texts
          )
```

![](tools/README-ordered-bar-plots-by-groups-1.png)<!-- -->

### Deviation graphs

The deviation graph shows the deviation of quantitatives values to a
reference value. In the R code below, we’ll plot the mpg z-score from
the mtcars dataset.

Calculate the z-score of the mpg data:

``` r
# Calculate the z-score of the mpg data
dfm$mpg_z <- (dfm$mpg -mean(dfm$mpg))/sd(dfm$mpg)
dfm$mpg_grp <- factor(ifelse(dfm$mpg_z < 0, "low", "high"), 
                     levels = c("low", "high"))
# Inspect the data
head(dfm[, c("name", "wt", "mpg", "mpg_z", "mpg_grp", "cyl")])
#>                                name    wt  mpg      mpg_z mpg_grp cyl
#> Mazda RX4                 Mazda RX4 2.620 21.0  0.1508848    high   6
#> Mazda RX4 Wag         Mazda RX4 Wag 2.875 21.0  0.1508848    high   6
#> Datsun 710               Datsun 710 2.320 22.8  0.4495434    high   4
#> Hornet 4 Drive       Hornet 4 Drive 3.215 21.4  0.2172534    high   6
#> Hornet Sportabout Hornet Sportabout 3.440 18.7 -0.2307345     low   8
#> Valiant                     Valiant 3.460 18.1 -0.3302874     low   6
```

Create an ordered barplot, colored according to the level of mpg:

``` r
ggbarplot(dfm, x = "name", y = "mpg_z",
          fill = "mpg_grp",           # change fill color by mpg_level
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palett. see ?ggpar
          sort.val = "asc",           # Sort the value in ascending order
          sort.by.groups = FALSE,     # Don't sort inside each group
          x.text.angle = 90,          # Rotate vertically x axis texts
          ylab = "MPG z-score",
          xlab = FALSE,
          legend.title = "MPG Group"
          )
```

![](tools/README-deviation-graphs-1.png)<!-- -->

Rotate the plot: use rotate = TRUE and sort.val = “desc”

``` r
ggbarplot(dfm, x = "name", y = "mpg_z",
          fill = "mpg_grp",           # change fill color by mpg_level
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palett. see ?ggpar
          sort.val = "desc",          # Sort the value in descending order
          sort.by.groups = FALSE,     # Don't sort inside each group
          x.text.angle = 90,          # Rotate vertically x axis texts
          ylab = "MPG z-score",
          legend.title = "MPG Group",
          rotate = TRUE,
          ggtheme = theme_minimal()
          )
```

![](tools/README-deviation-graphs-horizontal-1.png)<!-- -->

## Dot charts

### Lollipop chart

Lollipop chart is an alternative to bar plots, when you have a large set
of values to visualize.

Lollipop chart colored by the grouping variable “cyl”:

``` r
ggdotchart(dfm, x = "name", y = "mpg",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "ascending",                        # Sort value in descending order
           add = "segments",                             # Add segments from y = 0 to dots
           ggtheme = theme_pubr()                        # ggplot2 theme
           )
```

![](tools/README-lollipop-chart-1.png)<!-- -->

- Sort in decending order. **sorting = “descending”**.
- Rotate the plot vertically, using **rotate = TRUE**.
- Sort the mpg value inside each group by using **group = “cyl”**.
- Set **dot.size** to 6.
- Add mpg values as label. **label = “mpg”** or **label =
  round(dfm\$mpg)**.

``` r
ggdotchart(dfm, x = "name", y = "mpg",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "descending",                       # Sort value in descending order
           add = "segments",                             # Add segments from y = 0 to dots
           rotate = TRUE,                                # Rotate vertically
           group = "cyl",                                # Order by groups
           dot.size = 6,                                 # Large dot size
           label = round(dfm$mpg),                        # Add mpg values as dot labels
           font.label = list(color = "white", size = 9, 
                             vjust = 0.5),               # Adjust label parameters
           ggtheme = theme_pubr()                        # ggplot2 theme
           )
```

![](tools/README-lollipop-chart-rotate-1.png)<!-- -->

Deviation graph:

- Use y = “mpg_z”
- Change segment color and size: add.params = list(color = “lightgray”,
  size = 2)

``` r
ggdotchart(dfm, x = "name", y = "mpg_z",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "descending",                       # Sort value in descending order
           add = "segments",                             # Add segments from y = 0 to dots
           add.params = list(color = "lightgray", size = 2), # Change segment color and size
           group = "cyl",                                # Order by groups
           dot.size = 6,                                 # Large dot size
           label = round(dfm$mpg_z,1),                        # Add mpg values as dot labels
           font.label = list(color = "white", size = 9, 
                             vjust = 0.5),               # Adjust label parameters
           ggtheme = theme_pubr()                        # ggplot2 theme
           )+
  geom_hline(yintercept = 0, linetype = 2, color = "lightgray")
```

![](tools/README-lollipop-chart-deviation-1.png)<!-- -->

### Cleveland’s dot plot

Color y text by groups. Use y.text.col = TRUE.

``` r
ggdotchart(dfm, x = "name", y = "mpg",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "descending",                       # Sort value in descending order
           rotate = TRUE,                                # Rotate vertically
           dot.size = 2,                                 # Large dot size
           y.text.col = TRUE,                            # Color y text by groups
           ggtheme = theme_pubr()                        # ggplot2 theme
           )+
  theme_cleveland()                                      # Add dashed grids
```

![](tools/README-cleveland-dot-plots-1.png)<!-- -->

## More

Find out more at <https://rpkgs.datanovia.com/ggpubr/>.

## Blog posts

- A. Kassambara. [ggpubr R Package: ggplot2-Based Publication Ready
  Plots](https://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/)

<!-- README.md is generated from README.Rmd. Please edit that file -->

[![R-CMD-check](https://github.com/kassambara/ggpubr/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/kassambara/ggpubr/actions/workflows/R-CMD-check.yaml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ggpubr)](https://cran.r-project.org/package=ggpubr)
[![Downloads](https://cranlogs.r-pkg.org/badges/ggpubr)](https://cran.r-project.org/package=ggpubr)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/ggpubr?color=orange)](https://cran.r-project.org/package=ggpubr)

# ggpubr: 'ggplot2' Based Publication Ready Plots

[ggplot2, by Hadley Wickham,](https://ggplot2.tidyverse.org/) is an
excellent and flexible package for elegant data visualization in R.
However the default generated plots require some formatting before we
can send them for publication. Furthermore, to customize a ggplot, the
syntax is opaque and this raises the level of difficulty for researchers
with no advanced R programming skills.

The 'ggpubr' package provides some easy-to-use functions for creating
and customizing 'ggplot2'-based publication ready plots.

Find out more at <https://rpkgs.datanovia.com/ggpubr/>.

## Highlights in the development version

- Modern compatibility updates for current `ggplot2`, `dplyr`, and
  `tidyr`.
- New p-value formatting helpers: `format_p_value()`,
  `get_p_format_style()`, `list_p_format_styles()`.
- Extended p-value formatting parameters across statistical layers:
  `p.format.style`, `p.digits`, `p.leading.zero`, `p.min.threshold`,
  `p.decimal.mark`.
- Sparse grouped subsets are handled robustly in `stat_compare_means()`
  and `geom_pwc()`: non-comparable subsets are skipped while valid
  comparisons are kept (#663).

## Installation and loading

- Install from [CRAN](https://cran.r-project.org/package=ggpubr) as
  follows:

``` r
install.packages("ggpubr")
```

- Or, install the latest version from
  [GitHub](https://github.com/kassambara/ggpubr) as follows:

``` r
# Install
if(!require(devtools)) install.packages("devtools")
devtools::install_github("kassambara/ggpubr")
```

## Distribution

``` r
library(ggpubr)
#> Loading required package: ggplot2
# Create some data format
# :::::::::::::::::::::::::::::::::::::::::::::::::::
set.seed(1234)
wdata = data.frame(
   sex = factor(rep(c("F", "M"), each=200)),
   weight = c(rnorm(200, 55), rnorm(200, 58)))
head(wdata, 4)
#>   sex   weight
#> 1   F 53.79293
#> 2   F 55.27743
#> 3   F 56.08444
#> 4   F 52.65430

# Density plot with mean lines and marginal rug
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change outline and fill colors by groups ("sex")
# Use custom palette
ggdensity(wdata, x = "weight",
   add = "mean", rug = TRUE,
   color = "sex", fill = "sex",
   palette = c("#00AFBB", "#E7B800"))
```

![](tools/README-ggpubr-1.png)<!-- -->

``` r
# Histogram plot with mean lines and marginal rug
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change outline and fill colors by groups ("sex")
# Use custom color palette
gghistogram(wdata, x = "weight",
   add = "mean", rug = TRUE,
   color = "sex", fill = "sex",
   palette = c("#00AFBB", "#E7B800"))
```

![](tools/README-ggpubr-2.png)<!-- -->

## Box plots and violin plots

``` r
# Load data
data("ToothGrowth")
df <- ToothGrowth
head(df, 4)
#>    len supp dose
#> 1  4.2   VC  0.5
#> 2 11.5   VC  0.5
#> 3  7.3   VC  0.5
#> 4  5.8   VC  0.5

# Box plots with jittered points
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change outline colors by groups: dose
# Use custom color palette
# Add jitter points and change the shape by groups
 p <- ggboxplot(df, x = "dose", y = "len",
                color = "dose", palette =c("#00AFBB", "#E7B800", "#FC4E07"),
                add = "jitter", shape = "dose")
 p
```

![](tools/README-ggpubr-box-plot-dot-plots-strip-charts-1.png)<!-- -->

``` r

 # Add p-values comparing groups
 # Specify the comparisons you want
my_comparisons <- list( c("0.5", "1"), c("1", "2"), c("0.5", "2") )
p + stat_compare_means(comparisons = my_comparisons)+ # Add pairwise comparisons p-value
  stat_compare_means(label.y = 50)                   # Add global p-value
```

![](tools/README-ggpubr-box-plot-dot-plots-strip-charts-2.png)<!-- -->

``` r


# Violin plots with box plots inside
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Change fill color by groups: dose
# add boxplot with white fill color
ggviolin(df, x = "dose", y = "len", fill = "dose",
         palette = c("#00AFBB", "#E7B800", "#FC4E07"),
         add = "boxplot", add.params = list(fill = "white"))+
  stat_compare_means(comparisons = my_comparisons, label = "p.signif")+ # Add significance levels
  stat_compare_means(label.y = 50)                                      # Add global the p-value
```

![](tools/README-ggpubr-box-plot-dot-plots-strip-charts-3.png)<!-- -->

## Bar plots

### Demo data set

Load and prepare data:

``` r
# Load data
data("mtcars")
dfm <- mtcars
# Convert the cyl variable to a factor
dfm$cyl <- as.factor(dfm$cyl)
# Add the name colums
dfm$name <- rownames(dfm)
# Inspect the data
head(dfm[, c("name", "wt", "mpg", "cyl")])
#>                                name    wt  mpg cyl
#> Mazda RX4                 Mazda RX4 2.620 21.0   6
#> Mazda RX4 Wag         Mazda RX4 Wag 2.875 21.0   6
#> Datsun 710               Datsun 710 2.320 22.8   4
#> Hornet 4 Drive       Hornet 4 Drive 3.215 21.4   6
#> Hornet Sportabout Hornet Sportabout 3.440 18.7   8
#> Valiant                     Valiant 3.460 18.1   6
```

### Ordered bar plots

Change the fill color by the grouping variable "cyl". Sorting will be
done globally, but not by groups.

``` r
ggbarplot(dfm, x = "name", y = "mpg",
          fill = "cyl",               # change fill color by cyl
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palette. see ?ggpar
          sort.val = "desc",          # Sort the value in descending order
          sort.by.groups = FALSE,     # Don't sort inside each group
          x.text.angle = 90           # Rotate vertically x axis texts
          )
```

![](tools/README-ordered-bar-plots-1.png)<!-- -->

Sort bars inside each group. Use the argument **sort.by.groups = TRUE**.

``` r
ggbarplot(dfm, x = "name", y = "mpg",
          fill = "cyl",               # change fill color by cyl
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palette. see ?ggpar
          sort.val = "asc",           # Sort the value in ascending order
          sort.by.groups = TRUE,      # Sort inside each group
          x.text.angle = 90           # Rotate vertically x axis texts
          )
```

![](tools/README-ordered-bar-plots-by-groups-1.png)<!-- -->

### Deviation graphs

The deviation graph shows the deviation of quantitative values to a
reference value. In the R code below, we'll plot the mpg z-score from
the mtcars dataset.

Calculate the z-score of the mpg data:

``` r
# Calculate the z-score of the mpg data
dfm$mpg_z <- (dfm$mpg -mean(dfm$mpg))/sd(dfm$mpg)
dfm$mpg_grp <- factor(ifelse(dfm$mpg_z < 0, "low", "high"),
                     levels = c("low", "high"))
# Inspect the data
head(dfm[, c("name", "wt", "mpg", "mpg_z", "mpg_grp", "cyl")])
#>                                name    wt  mpg      mpg_z mpg_grp cyl
#> Mazda RX4                 Mazda RX4 2.620 21.0  0.1508848    high   6
#> Mazda RX4 Wag         Mazda RX4 Wag 2.875 21.0  0.1508848    high   6
#> Datsun 710               Datsun 710 2.320 22.8  0.4495434    high   4
#> Hornet 4 Drive       Hornet 4 Drive 3.215 21.4  0.2172534    high   6
#> Hornet Sportabout Hornet Sportabout 3.440 18.7 -0.2307345     low   8
#> Valiant                     Valiant 3.460 18.1 -0.3302874     low   6
```

Create an ordered barplot, colored according to the level of mpg:

``` r
ggbarplot(dfm, x = "name", y = "mpg_z",
          fill = "mpg_grp",           # change fill color by mpg_level
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palette. see ?ggpar
          sort.val = "asc",           # Sort the value in ascending order
          sort.by.groups = FALSE,     # Don't sort inside each group
          x.text.angle = 90,          # Rotate vertically x axis texts
          ylab = "MPG z-score",
          xlab = FALSE,
          legend.title = "MPG Group"
          )
```

![](tools/README-deviation-graphs-1.png)<!-- -->

Rotate the plot: use rotate = TRUE and sort.val = "desc"

``` r
ggbarplot(dfm, x = "name", y = "mpg_z",
          fill = "mpg_grp",           # change fill color by mpg_level
          color = "white",            # Set bar border colors to white
          palette = "jco",            # jco journal color palette. see ?ggpar
          sort.val = "desc",          # Sort the value in descending order
          sort.by.groups = FALSE,     # Don't sort inside each group
          x.text.angle = 90,          # Rotate vertically x axis texts
          ylab = "MPG z-score",
          legend.title = "MPG Group",
          rotate = TRUE,
          ggtheme = theme_minimal()
          )
```

![](tools/README-deviation-graphs-horizontal-1.png)<!-- -->

## Dot charts

### Lollipop chart

Lollipop chart is an alternative to bar plots, when you have a large set
of values to visualize.

Lollipop chart colored by the grouping variable "cyl":

``` r
ggdotchart(dfm, x = "name", y = "mpg",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "ascending",                        # Sort value in descending order
           add = "segments",                             # Add segments from y = 0 to dots
           ggtheme = theme_pubr()                        # ggplot2 theme
           )
```

![](tools/README-lollipop-chart-1.png)<!-- -->

- Sort in descending order. **sorting = "descending"**.
- Rotate the plot vertically, using **rotate = TRUE**.
- Sort the mpg value inside each group by using **group = "cyl"**.
- Set **dot.size** to 6.
- Add mpg values as label. **label = "mpg"** or **label =
  round(dfm\$mpg)**.

``` r
ggdotchart(dfm, x = "name", y = "mpg",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "descending",                       # Sort value in descending order
           add = "segments",                             # Add segments from y = 0 to dots
           rotate = TRUE,                                # Rotate vertically
           group = "cyl",                                # Order by groups
           dot.size = 6,                                 # Large dot size
           label = round(dfm$mpg),                        # Add mpg values as dot labels
           font.label = list(color = "white", size = 9,
                             vjust = 0.5),               # Adjust label parameters
           ggtheme = theme_pubr()                        # ggplot2 theme
           )
```

![](tools/README-lollipop-chart-rotate-1.png)<!-- -->

Deviation graph:

- Use y = "mpg_z"
- Change segment color and size: add.params = list(color = "lightgray",
  size = 2)

``` r
ggdotchart(dfm, x = "name", y = "mpg_z",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "descending",                       # Sort value in descending order
           add = "segments",                             # Add segments from y = 0 to dots
           add.params = list(color = "lightgray", size = 2), # Change segment color and size
           group = "cyl",                                # Order by groups
           dot.size = 6,                                 # Large dot size
           label = round(dfm$mpg_z,1),                        # Add mpg values as dot labels
           font.label = list(color = "white", size = 9,
                             vjust = 0.5),               # Adjust label parameters
           ggtheme = theme_pubr()                        # ggplot2 theme
           )+
  geom_hline(yintercept = 0, linetype = 2, color = "lightgray")
```

![](tools/README-lollipop-chart-deviation-1.png)<!-- -->

### Cleveland's dot plot

Color y text by groups. Use y.text.col = TRUE.

``` r
ggdotchart(dfm, x = "name", y = "mpg",
           color = "cyl",                                # Color by groups
           palette = c("#00AFBB", "#E7B800", "#FC4E07"), # Custom color palette
           sorting = "descending",                       # Sort value in descending order
           rotate = TRUE,                                # Rotate vertically
           dot.size = 2,                                 # Large dot size
           y.text.col = TRUE,                            # Color y text by groups
           ggtheme = theme_pubr()                        # ggplot2 theme
           )+
  theme_cleveland()                                      # Add dashed grids
```

![](tools/README-cleveland-dot-plots-1.png)<!-- -->

## More

Find out more at <https://rpkgs.datanovia.com/ggpubr/>.

## Blog posts

- A. Kassambara. [ggpubr R Package: ggplot2-Based Publication Ready
  Plots](https://www.sthda.com/english/articles/24-ggpubr-publication-ready-plots/)

#' @include utilities.R ggpar.R
NULL
#' Stripcharts
#' @description Create a stripchart, also known as one dimensional scatter
#'   plots. These plots are suitable compared to box plots when sample sizes are
#'   small.
#' @inheritParams ggboxplot
#' @param shape point shape
#' @param jitter the amount of jitter.
#' @param position position adjustment, either as a string, or the result of a
#'   call to a position adjustment function. Used to adjust position for
#'   multiple groups.
#' @param ... other arguments to be passed to
#'   \code{\link[ggplot2]{geom_jitter}}, \code{\link{ggpar}} and
#'   \code{\link{facet}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#'@seealso \code{\link{ggpar}}, \code{\link{ggviolin}}, \code{\link{ggdotplot}}
#'  and \code{\link{ggboxplot}}.
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot with summary statistics: mean_se
#' # +++++++++++++++++++++++++++
#' # Change point shapes by groups: "dose"
#' ggstripchart(df, x = "dose", y = "len",
#'    shape = "dose", size = 3,
#'    add = "mean_se")
#'
#' # Use mean_sd
#' # Change error.plot to "crossbar"
#' ggstripchart(df, x = "dose", y = "len",
#'    shape = "dose", size = 3,
#'    add = "mean_sd", add.params = list(width = 0.5),
#'    error.plot = "crossbar")
#'
#'
#'
#' # Add summary statistics
#' # ++++++++++++++++++++++++++
#'
#' # Add box plot
#' ggstripchart(df, x = "dose", y = "len",
#'  shape = "dose", add = "boxplot")
#'
#' # Add violin + mean_sd
#' ggstripchart(df, x = "dose", y = "len",
#'  shape = "dose", add = c("violin", "mean_sd"))
#'
#'
#' # Change colors
#' # +++++++++++++++++++++++++++
#' # Change colors by groups: dose
#' # Use custom color palette
#'  ggstripchart(df, "dose", "len",  shape = "dose",
#'    color = "dose", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'    add = "mean_sd")
#'
#'
#'
#' # Plot with multiple groups
#' # +++++++++++++++++++++
#' # Change shape and color by a second group : "supp"
#' ggstripchart(df, "dose", "len", shape = "supp",
#'   color = "supp", palette = c("#00AFBB", "#E7B800"))
#'
#' # Adjust point position
#' ggstripchart(df, "dose", "len", shape = "supp",
#'   color = "supp", palette = c("#00AFBB", "#E7B800"),
#'   position = position_dodge(0.8) )
#'
#' # You can also use position_jitterdodge()
#' # but fill aesthetic is required
#' ggstripchart(df, "dose", "len",  shape = "supp",
#'    color = "supp", palette = c("#00AFBB", "#E7B800"),
#'    position = position_jitterdodge() )
#'
#' # Add boxplot
#' ggstripchart(df, "dose", "len", shape = "supp",
#'  color = "supp", palette = c("#00AFBB", "#E7B800"),
#'  add = "boxplot", add.params = list(color = "black") )
#'
#' @export
ggstripchart <- function(data, x, y, combine = FALSE, merge = FALSE,
                         color = "black", fill = "white", palette = NULL,
                         title = NULL, xlab = NULL, ylab = NULL,
                         facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                         shape = 19, size = NULL,
                         select = NULL, remove = NULL, order = NULL,
                         add = "mean_se",
                         add.params = list(),
                         error.plot = "pointrange",
                         label = NULL, font.label = list(size = 11, color = "black"),
                         label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                         jitter = 0.2,
                         position = position_jitter(jitter),
                         ggtheme = theme_pubr(),
                         ...)
{

  # Default options
  #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    shape = shape, size = size,
    select = select , remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle,
    jitter = jitter, position = position, ggtheme = ggtheme, ...)
  if(!missing(data)) .opts$data <- data
  if(!missing(x)) .opts$x <- x
  if(!missing(y)) .opts$y <- y

  # User options
  #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for(opt.name in names(.opts)){
    if(is.null(.user.opts[[opt.name]]))
      .opts[[opt.name]] <- NULL
  }

  .opts$fun <- ggstripchart_core
  if(missing(ggtheme) & (!is.null(facet.by) | combine))
    .opts$ggtheme <- theme_pubr(border = TRUE)
  p <- do.call(.plotter, .opts)

  if(.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)

}

ggstripchart_core <- function(data, x, y,
                      color = "black", fill = "white", palette = NULL,
                      title = NULL, xlab = NULL, ylab = NULL,
                      shape = 19, size = NULL,
                      add = "mean_se",
                      add.params = list(),
                      error.plot = "pointrange",
                      jitter = 0.2,
                      position = position_jitter(jitter),
                      ggtheme = theme_pubr(),
                      ...)
{

  if(!is.factor(data[, x])) data[, x] <- as.factor(data[, x])
  . <- NULL

  p <- ggplot(data, create_aes(list(x = x, y = y)))
  if("none" %in% add) add <- "none"

  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill, ...)
  # plot boxplot | violin | crossbar before jitter
  if( any( c("boxplot", "violin") %in% add)){
    p <- add.params %>%
      .add_item(p = p, add = intersect(add, c("boxplot", "violin") )) %>%
      do.call(ggadd, .)
  }
  if(error.plot == "crossbar"){
    p <- add.params %>%
      .add_item(p = p, error.plot = error.plot,
                add = setdiff(add, c("boxplot", "violin", "jitter"))) %>%
      do.call(ggadd, .)
  }
  # Plot jitter
  set.seed(123)
  p <- p +
      geom_exec(geom_jitter, data = data,
                color = color, fill = fill, shape = shape,
                position = position, size = size, ...)
  # Add errors
  if(error.plot == "crossbar"){}
  else p <- add.params %>%
    .add_item(p = p, error.plot = error.plot,
              add = setdiff(add, c("boxplot", "violin", "jitter"))) %>%
    do.call(ggadd, .)

   p <- ggpar(p, palette = palette, ggtheme = ggtheme,
              title = title, xlab = xlab, ylab = ylab,...)

  p
}

context("test-colour-alias")

# #317: accept the British spelling `colour` as an alias for `color` in the
# core plotting functions (those routed through .plotter). Previously `colour`
# was silently ignored.

ng <- function(p) length(unique(ggplot2::ggplot_build(p)$data[[1]]$colour))

test_that("colour is an alias for color (grouping) across core functions (#317)", {
  tg <- ToothGrowth
  tg$dose <- factor(tg$dose)
  expect_equal(ng(ggscatter(mtcars, "wt", "mpg", colour = "cyl")),
               ng(ggscatter(mtcars, "wt", "mpg", color = "cyl")))
  expect_equal(ng(ggboxplot(tg, "dose", "len", colour = "dose")),
               ng(ggboxplot(tg, "dose", "len", color = "dose")))
  expect_equal(ng(ggviolin(tg, "dose", "len", colour = "dose")), 3L)
  expect_equal(ng(ggstripchart(tg, "dose", "len", colour = "dose")), 3L)
})

test_that("colour works as a static color and does not break color (#317)", {
  d <- ggplot2::ggplot_build(ggscatter(mtcars, "wt", "mpg", colour = "red"))$data[[1]]
  expect_true(all(d$colour == "red"))
  # color = still works and default is unchanged
  d2 <- ggplot2::ggplot_build(ggscatter(mtcars, "wt", "mpg"))$data[[1]]
  expect_true(all(d2$colour == "black"))
  d3 <- ggplot2::ggplot_build(ggscatter(mtcars, "wt", "mpg", color = "blue"))$data[[1]]
  expect_true(all(d3$colour == "blue"))
})

context("test-show-n")

# #598 / #627: ggboxplot(), ggviolin() and ggstripchart() gain an opt-in
# `show.n` argument that prints the per-group number of observations at the top
# of each group. Default (show.n = FALSE) adds nothing (byte-identical output).

df <- ToothGrowth
df$dose <- factor(df$dose)

.text_layer <- function(p) {
  which(vapply(p$layers, function(l) inherits(l$geom, "GeomText"), logical(1)))
}
.n_labels <- function(p) {
  b <- ggplot2::ggplot_build(p)
  i <- .text_layer(p)[1]
  if (is.na(i)) return(character(0))
  as.character(b$data[[i]]$label)
}

test_that("default show.n = FALSE adds no text layer (unchanged output)", {
  for (fn in list(ggboxplot, ggviolin, ggstripchart)) {
    p <- fn(df, "dose", "len")
    expect_length(.text_layer(p), 0)
  }
})

test_that("show.n = TRUE shows one count per x tick when ungrouped", {
  for (fn in list(ggboxplot, ggviolin, ggstripchart)) {
    p <- fn(df, "dose", "len", show.n = TRUE)
    expect_equal(sort(.n_labels(p)), c("n = 20", "n = 20", "n = 20"))
    expect_silent(ggplot2::ggplotGrob(p))
  }
})

test_that("show.n = TRUE shows per-group counts for dodged box/violin", {
  for (fn in list(ggboxplot, ggviolin)) {
    p <- fn(df, "dose", "len", color = "supp", show.n = TRUE)
    labs <- .n_labels(p)
    expect_length(labs, 6)                 # 3 doses x 2 supp
    expect_true(all(labs == "n = 10"))
  }
})

test_that("stripchart adapts to its position (total when not dodged, per-group when jitterdodged)", {
  # default position_jitter (points not dodged) -> total per x
  p1 <- ggstripchart(df, "dose", "len", color = "supp", show.n = TRUE)
  expect_equal(sort(.n_labels(p1)), c("n = 20", "n = 20", "n = 20"))
  # position_jitterdodge -> per-group
  p2 <- ggstripchart(df, "dose", "len", color = "supp",
                     position = ggplot2::position_jitterdodge(), show.n = TRUE)
  expect_length(.n_labels(p2), 6)
  expect_true(all(.n_labels(p2) == "n = 10"))
})

test_that("counts respect select/remove (taken from the plotted data)", {
  p <- ggboxplot(df, "dose", "len", select = c("0.5", "1"), show.n = TRUE)
  expect_equal(sort(.n_labels(p)), c("n = 20", "n = 20"))

  d2 <- rbind(df, data.frame(len = c(1, 2), supp = "OJ", dose = factor("0.5")))
  p2 <- ggboxplot(d2, "dose", "len", show.n = TRUE)
  # dose 0.5 now has 22 observations
  b <- ggplot2::ggplot_build(p2)
  labs <- as.character(b$data[[.text_layer(p2)[1]]]$label)
  expect_true("n = 22" %in% labs)
})

test_that("show.n works with facets (one set of counts per panel)", {
  p <- ggboxplot(df, "dose", "len", color = "supp", facet.by = "supp", show.n = TRUE)
  expect_length(.n_labels(p), 6)
  expect_silent(ggplot2::ggplotGrob(p))
})

test_that("show.n handles combine = TRUE per response variable (not double-counted)", {
  df2 <- df
  df2$len2 <- df2$len * 1.1
  p <- ggboxplot(df2, "supp", c("len", "len2"), combine = TRUE, show.n = TRUE)
  labs <- .n_labels(p)
  expect_length(labs, 4)                       # 2 supp x 2 response panels
  expect_true(all(labs == "n = 30"))           # not n = 60
  expect_silent(ggplot2::ggplotGrob(p))
})

test_that("show.n with an empty selection does not error and adds no labels", {
  p <- ggboxplot(df, "dose", "len", select = "does-not-exist", show.n = TRUE)
  expect_length(.text_layer(p), 0)
  expect_error(ggplot2::ggplot_build(p), NA)
})

test_that("show.n counts per box when color and fill map to different columns", {
  d <- data.frame(
    x = rep(c("A", "B"), each = 8),
    s = rep(c("p", "q"), 8),
    b = rep(rep(c("m", "n"), each = 2), 4),
    y = 1:16
  )
  p <- ggboxplot(d, "x", "y", color = "s", fill = "b", show.n = TRUE)
  labs <- .n_labels(p)
  expect_length(labs, 8)                        # 2 x * (2 s * 2 b)
  expect_true(all(labs == "n = 2"))
})

test_that("show.n counts each group correctly on an unbalanced dataset", {
  d <- data.frame(
    g = c("a", "a", "a", "b", "b"),
    y = c(1, 2, 3, 4, 5)
  )
  p <- ggboxplot(d, "g", "y", show.n = TRUE)
  expect_equal(sort(.n_labels(p)), c("n = 2", "n = 3"))
})

#' Remove a ggplot Component
#'
#' @description Remove a specific component from a ggplot.
#'
#' @param object character string specifying the plot components. Allowed values
#'  include: \itemize{
#'  \item \code{"grid"} for both x and y grids
#'  \item \code{"x.grid"} for x axis grids
#'  \item \code{"y.grid"} for y axis grids
#'  \item \code{"axis"} for both x and y axes
#'  \item \code{"x.axis"} for x axis
#'  \item \code{"y.axis"} for y axis
#'
#'  \item \code{"xlab", or "x.title"}
#'  for x axis label
#'  \item \code{"ylab", or "y.title"} for y axis label
#'  \item \code{"xylab", "xy.title" or "axis.title"} for both x and y axis
#'  labels
#'  \item \code{"x.text"} for x axis texts (x axis tick labels)
#'  \item \code{"y.text"} for y axis texts (y axis tick labels)
#'  \item \code{"xy.text"} or \code{"axis.text"} for both x and y axis texts
#'
#'  \item \code{"ticks"} for both x and y ticks
#'  \item \code{"x.ticks"} for x ticks
#'  \item \code{"y.ticks"} for y ticks
#'
#'  \item \code{"legend.title"} for the legend title
#'  \item \code{"legend"} for the legend
#'  }
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#'
#' # Basic plot
#' p <- ggboxplot(ToothGrowth,
#'   x = "dose", y = "len",
#'   ggtheme = theme_gray()
#' )
#' p
#'
#' # Remove all grids
#' p + rremove("grid")
#'
#' # Remove only x grids
#' p + rremove("x.grid")
#' @export
rremove <- function(object) {
  blank <- element_blank()

  switch(object,
    grid = theme(panel.grid.minor = blank, panel.grid.major = blank),
    x.grid = theme(panel.grid.minor.x = blank, panel.grid.major.x = blank),
    y.grid = theme(panel.grid.minor.y = blank, panel.grid.major.y = blank),
    axis = theme(axis.line = blank),
    x.axis = theme(axis.line.x = blank),
    y.axis = theme(axis.line.y = blank),
    xlab = theme(axis.title.x = blank),
    x.title = theme(axis.title.x = blank),
    ylab = theme(axis.title.y = blank),
    y.title = theme(axis.title.y = blank),
    xy.title = theme(axis.title.x = blank, axis.title.y = blank),
    xylab = theme(axis.title.x = blank, axis.title.y = blank),
    axis.title = theme(axis.title.x = blank, axis.title.y = blank),
    x.ticks = theme(axis.ticks.x = blank),
    y.ticks = theme(axis.ticks.y = blank),
    ticks = theme(axis.ticks.x = blank, axis.ticks.y = blank),
    x.text = theme(axis.text.x = blank),
    y.text = theme(axis.text.y = blank),
    xy.text = theme(axis.text.x = blank, axis.text.y = blank),
    axis.text = theme(axis.text.x = blank, axis.text.y = blank),
    legend.title = theme(legend.title = blank),
    legend = theme(legend.position = "none"),
    stop("Don't support ", object)
  )
}

#' @include utilities.R
NULL

#' Create a Text Graphical object
#' @description Create easily a customized text grob (graphical object). Wrapper around
#'   \code{\link[grid:grid.text]{textGrob}}.
#' @inheritParams grid::grid.text
#' @param color text font color.
#' @param face font face. Allowed values include one of \code{"plain", "bold",
#'   "italic", "bold.italic"}.
#' @param size font size (e.g.: size = 12)
#' @param lineheight line height (e.g.: lineheight = 2).
#' @param family font family.
#' @param ... other arguments passed to \link[grid:grid.text]{textGrob}.
#' @return  a text grob.
#' @examples
#' text <- paste("iris data set gives the measurements in cm",
#'   "of the variables sepal length and width",
#'   "and petal length and width, respectively,",
#'   "for 50 flowers from each of 3 species of iris.",
#'   "The species are Iris setosa, versicolor, and virginica.",
#'   sep = "\n"
#' )
#'
#' # Create a text grob
#' tgrob <- text_grob(text, face = "italic", color = "steelblue")
#' # Draw the text
#' as_ggplot(tgrob)
#'
#' @export
text_grob <- function(label, just = "centre", hjust = NULL, vjust = NULL, rot = 0,
                      color = "black", face = "plain", size = NULL, lineheight = NULL,
                      family = NULL, ...) {
  gp <- grid::gpar(
    col = color, fontface = face, fontsize = size,
    lineheight = lineheight, fontfamily = family
  )

  tgrob <- grid::textGrob(
    label = label, just = just, hjust = hjust, vjust = vjust,
    rot = rot, gp = gp, ...
  )
  tgrob
}

#' ggpubr General Arguments Description
#'
#' @param data a data frame
#' @param x character string containing the name of x variable.
#' @param y character vector containing one or more variables to plot
#' @param combine logical value. Default is FALSE. Used only when y is a vector
#'  containing multiple variables to plot. If TRUE, create a multi-panel plot by
#'  combining the plot of y variables.
#' @param merge logical or character value. Default is FALSE. Used only when y is
#'  a vector containing multiple variables to plot. If TRUE, merge multiple y
#'  variables in the same plotting area. Allowed values include also "asis"
#'  (TRUE) and "flip". If merge = "flip", then y variables are used as x tick
#'  labels and the x variable is used as grouping variable.
#' @param color outline color.
#' @param fill fill color.
#' @param palette the color palette to be used for coloring or filling by groups.
#'  Allowed values include "grey" for grey color palettes; brewer palettes e.g.
#'  "RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
#'  scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
#'  "lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".
#' @param linetype line types.
#' @param size Numeric value (e.g.: size = 1). change the size of points and
#'  outlines.
#' @param select character vector specifying which items to display.
#' @param remove character vector specifying which items to remove from the plot.
#' @param order character vector specifying the order of items.
#' @param add character vector for adding another plot element (e.g.: dot plot or
#'  error bars). Allowed values are one or the combination of: "none",
#'  "dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
#'  "mean_ci", "mean_range", "median", "median_iqr", "median_mad",
#'  "median_range"; see ?desc_statby for more details.
#' @param add.params parameters (color, shape, size, fill, linetype) for the
#'  argument 'add'; e.g.: add.params = list(color = "red").
#' @param error.plot plot type used to visualize error. Allowed values are one of
#'  c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
#'  "lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
#'  "lower_linerange"). Default value is "pointrange" or "errorbar". Used only
#'  when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
#'  se, ....
#' @param font.label a list which can contain the combination of the following elements: the size
#'  (e.g.: 14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and
#'  the color (e.g.: "red") of labels. For example font.label = list(size = 14,
#'  face = "bold", color ="red"). To specify only the size and the style, use font.label =
#'  list(size = 14, face = "plain").
#' @param title plot main title.
#' @param xlab character vector specifying x axis labels. Use xlab
#'   = FALSE to hide xlab.
#' @param ylab character vector specifying y axis labels. Use ylab = FALSE to
#'   hide ylab.
#' @param ggtheme function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
#'  Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
#'  theme_minimal(), theme_classic(), theme_void(), ....
#' @name ggpubr_args
#' @rdname ggpubr_args
NULL

#' Add Grids to a ggplot
#'
#' @description Add grids to ggplot.
#' @param axis axis for which grid should be added. Allowed values include \code{c("xy", "x", "y")}.
#' @param color grid line color.
#' @param size numeric value specifying grid line size.
#' @param linetype line type. An integer (0:8), a name (blank, solid, dashed,
#'  dotted, dotdash, longdash, twodash). Sess \code{\link{show_line_types}}.
#'
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#'
#' # Basic plot
#' p <- ggboxplot(ToothGrowth, x = "dose", y = "len")
#' p
#'
#' # Add border
#' p + grids(linetype = "dashed")
#' @export
grids <- function(axis = c("xy", "x", "y"), color = "grey92", size = NULL, linetype = NULL) {
  axis <- match.arg(axis)
  grid.major <- element_line(
    color = color, linewidth = size,
    linetype = linetype
  )
  grid.minor <- element_line(
    color = color, linewidth = 0.25,
    linetype = linetype
  )

  switch(axis,
    xy = theme(panel.grid.major = grid.major, panel.grid.minor = grid.minor),
    x = theme(panel.grid.major.x = grid.major, panel.grid.minor.x = grid.minor),
    y = theme(panel.grid.major.y = grid.major, panel.grid.minor.y = grid.minor)
  )
}

#' Differential gene expression analysis results
#'
#' @description Differential gene expression analysis results obtained from
#'   comparing the RNAseq data of two different cell populations using DESeq2
#' @name diff_express
#' @docType data
#' @usage data("diff_express")
#' @format A data frame with 36028 rows and 5 columns. \describe{
#'   \item{\code{name}}{gene names} \item{\code{baseMean}}{mean expression
#'   signal across all samples} \item{\code{log2FoldChange}}{log2 fold change}
#'   \item{\code{padj}}{Adjusted p-value}\item{\code{detection_call}}{a numeric
#'   vector specifying whether the genes is expressed (value = 1) or not (value = 0).}}
#'
#' @examples
#' data(diff_express)
#'
#' # Default plot
#' ggmaplot(diff_express,
#'   main = expression("Group 1" %->% "Group 2"),
#'   fdr = 0.05, fc = 2, size = 0.4,
#'   palette = c("#B31B21", "#1465AC", "darkgray"),
#'   genenames = as.vector(diff_express$name),
#'   legend = "top", top = 20,
#'   font.label = c("bold", 11),
#'   font.legend = "bold",
#'   font.main = "bold",
#'   ggtheme = ggplot2::theme_minimal()
#' )
#'
#' # Add rectangle around labesl
#' ggmaplot(diff_express,
#'   main = expression("Group 1" %->% "Group 2"),
#'   fdr = 0.05, fc = 2, size = 0.4,
#'   palette = c("#B31B21", "#1465AC", "darkgray"),
#'   genenames = as.vector(diff_express$name),
#'   legend = "top", top = 20,
#'   font.label = c("bold", 11), label.rectangle = TRUE,
#'   font.legend = "bold",
#'   font.main = "bold",
#'   ggtheme = ggplot2::theme_minimal()
#' )
#'
NULL

#' @include utilities.R ggpar.R stat_chull.R stat_conf_ellipse.R
NULL
#' Text
#' @description Add text to a plot.
#' @inheritParams ggscatter
#' @param data a data frame
#' @param x,y x and y variables for drawing.
#' @param label the name of the column containing point labels. Can be also a
#'   character vector with length = nrow(data).
#' @param color text font color.
#' @param size text font size.
#' @param face text font style. Allowed values are one of c("plain", "bold",
#'   "italic", "bold.italic").
#' @param parse If \code{TRUE}, the labels will be parsed into expressions and
#'   displayed as described in \code{?plotmath}.
#' @param family character vector specifying font family.
#' @param show.legend logical. Should text be included in the legends? NA, the
#'   default, includes if any aesthetics are mapped. FALSE never includes, and
#'   TRUE always includes.
#' @param label.select can be of two formats: \itemize{ \item a character vector
#'   specifying some labels to show. \item a list containing one or the
#'   combination of the following components: \itemize{ \item \code{top.up} and
#'   \code{top.down}: to display the labels  of the top up/down points. For
#'   example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
#'   \code{criteria}: to filter, for example, by x and y variables values, use
#'   this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
#'   c('A', 'B')")}. } }
#' @param repel a logical value, whether to use ggrepel to avoid overplotting
#'   text labels or not.
#' @param label.rectangle logical value. If TRUE, add rectangle underneath the
#'   text, making it easier to read.
#' @param grouping.vars grouping variables to sort the data by, when the user
#'   wants to display the top n up/down labels.
#' @param position Position adjustment, either as a string, or the result of a
#'   call to a position adjustment function.
#' @param ggp a ggplot. If not NULL, points are added to an existing plot.
#' @param ... other arguments to be passed to \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right"  }
#' @seealso \code{\link{ggpar}}
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#' df$name <- rownames(df)
#' head(df[, c("wt", "mpg", "cyl")], 3)
#'
#' # Textual annotation
#' # +++++++++++++++++
#' ggtext(df,
#'   x = "wt", y = "mpg",
#'   color = "cyl", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   label = "name", repel = TRUE
#' )
#'
#' # Add rectangle around label
#' ggtext(df,
#'   x = "wt", y = "mpg",
#'   color = "cyl", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   label = "name", repel = TRUE, label.rectangle = TRUE
#' )
#'
#' @export
ggtext <- function(data, x = NULL, y = NULL, label = NULL,
                   color = "black", palette = NULL,
                   size = 11, face = "plain", family = "", show.legend = NA,
                   label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                   parse = FALSE,
                   grouping.vars = NULL,
                   position = "identity",
                   ggp = NULL, ggtheme = theme_pubr(),
                   ...) {
  . <- NULL
  .dots <- list(...)
  data <- as.data.frame(data)
  if (length(label) > 1) {
    if (length(label) != nrow(data)) {
      stop(
        "The argument label should be a column name or a vector of length = nrow(data). ",
        "It seems that length(label) != nrow(data)"
      )
    } else {
      data$label.xx <- label
    }
    label <- "label.xx"
  }

  if (is.null(ggp)) {
    p <- ggplot(data, create_aes(list(x = x, y = y)))
  } else {
    p <- ggp
  }

  # Add textual annotation
  # ++++++
  alpha <- 1
  if (!is.null(list(...)$alpha)) alpha <- list(...)$alpha

  if (is.null(label)) {
    return(p)
  }

  lab_data <- data

  # Special case for density plot and histogram
  # y is calculated as ..count.. or ..density..
  # we should estimate label y from ggplot2 output
  .is_density_plot <- y[1] %in% c("..count..", "..density..")
  if (.is_density_plot) {
    lab_data <- .hist_label_data(p, grouping.vars = list(...)$facet.by, x = x)
    y <- "lab.y"

    # hist.data <- ggplot_build(p)$data[[1]][, c("x", "y", "count", "density")]
    # hist.x <- hist.data$x
    # hist.y <- hist.data$y
    # break.x <- c(0, hist.x) %>% unique()
    # label.break <- seq_len(length(break.x) - 1)
    # lab.y <- .select_vec(data, x) %>%
    #   cut(breaks = break.x, labels = label.break) %>%
    #   hist.y[.]
    # lab_data$lab.y <- lab.y
    # y <- "lab.y"
    # lab_data <- lab_data %>% dplyr::filter(!is.na(lab.y))
  }

  # Select some labels to show
  if (!is.null(label.select)) {
    lab_data <- .get_label_data(lab_data, x, y,
      label = label,
      label.select = label.select,
      grouping.vars = grouping.vars
    )
  }

  if (repel) {
    max.overlaps <- getOption("ggrepel.max.overlaps", default = Inf)
    ggfunc <- ggrepel::geom_text_repel
    if (label.rectangle) ggfunc <- ggrepel::geom_label_repel
    p <- p + geom_exec(ggfunc,
      data = lab_data, x = x, y = y,
      label = label, fontface = face,
      family = family, show.legend = show.legend,
      size = size / 3, color = color,
      alpha = alpha, parse = parse,
      box.padding = unit(0.35, "lines"),
      point.padding = unit(0.3, "lines"),
      force = 1, segment.size = 0.2, seed = 123,
      max.overlaps = max.overlaps
    )
  } else {
    ggfunc <- geom_text
    vjust <- -0.7
    hjust <- NULL
    if (label.rectangle) {
      ggfunc <- geom_label
      vjust <- -0.4
    }
    vjust <- ifelse(is.null(.dots$vjust), vjust, .dots$vjust)
    if (!is.null(.dots$hjust)) hjust <- .dots$hjust
    p <- p + geom_exec(ggfunc,
      data = lab_data, x = x, y = y, color = color,
      label = label, fontface = face, family = family, show.legend = show.legend,
      size = size / 3, color = color, parse = parse,
      vjust = vjust, hjust = hjust, alpha = alpha, position = position
    )
  }

  # p <- ggpar(p, palette = palette, ggtheme = ggtheme, ...)
  if (family != "") {
    p <- p + theme(text = element_text(family = family))
  }

  p
}


# data: data frame
# x, y: x and y variables
# label: label columns
# label.select: select some labels. Can be a character vector, or a list
#   with the following components (top.up, top.down)
# grouping.vars grouping variables
.get_label_data <- function(data, x, y, label = NULL,
                            label.select = NULL, grouping.vars = NULL) {
  if (.is_list(label.select)) {
    expected.components <- c("top.up", "top.down", "criteria")
    if (!any(expected.components %in% names(label.select))) {
      stop(
        "If label.select is a list, it should contain one or the combination ",
        "of the following element: ", .collapse(expected.components, sep = ", ")
      )
    }
  }

  data <- as.data.frame(data)

  if (is.null(label)) {
    lab_data <- NULL
  } else if (is.null(label.select)) {
    lab_data <- data
  } else if (.is_list(label.select)) {
    lab_data <- data
    top_up <- top_down <- . <- NULL

    if (!is.null(label.select$top.up)) {
      top_up <- .top_up(data, x, y,
        n = label.select$top.up,
        grouping.vars = grouping.vars
      )
    }

    if (!is.null(label.select$top.down)) {
      top_down <- .top_down(data, x, y,
        n = label.select$top.down,
        grouping.vars = grouping.vars
      )
    }

    if (!is.null(top_up) | !is.null(top_down)) {
      lab_data <- rbind(top_up, top_down)
    }

    if (!is.null(label.select$criteria)) {
      criteria <- gsub("`y`", y, label.select$criteria) %>%
        gsub("`x`", x, .)
      lab_data <- dplyr::filter(lab_data, !!rlang::parse_expr(criteria))
    }
  } else {
    lab_data <- subset(data, data[[label]] %in% label.select,
      drop = FALSE
    )
  }

  return(lab_data)
}


# Get histogram/density label y coord from ggplot output
# grouping.vars : facet variables
# x: x variable name
.hist_label_data <- function(p, grouping.vars = NULL, x = NULL) {
  . <- NULL
  # x <- .mapping(p) %>%.$x
  hist.data <- ggplot_build(p)$data[[1]]
  data <- p$data

  if (is.null(grouping.vars)) {
    return(.hist_label_y(hist.data, data, x))
  }


  data <- p$data %>%
    df_nest_by(vars = grouping.vars)

  hist.data <- hist.data %>%
    df_nest_by(vars = "PANEL") %>%
    .$data

  data <- data %>% mutate(hist.data = hist.data)
  lab.data <- purrr::map2(data$hist.data, data$data, .hist_label_y, x)

  data <- data %>%
    mutate(lab.data = lab.data) %>%
    df_select(vars = c("lab.data", grouping.vars)) %>%
    tidyr::unnest()

  data
}

# Get histogram/density label y coord from ggplot output
# hist.data: histogram data. ggplot_build(p)$data[[1]]
# data: data frame
# x: x variable name
.hist_label_y <- function(hist.data, data, x) {
  . <- NULL
  hist.x <- hist.data$x
  hist.y <- hist.data$y
  break.x <- c(0, hist.x) %>% unique()
  label.break <- seq_len(length(break.x) - 1)
  lab.y <- .select_vec(data, x) %>%
    cut(breaks = break.x, labels = label.break) %>%
    hist.y[.]

  data$lab.y <- lab.y
  data <- data %>% dplyr::filter(!is.na(lab.y))
  data
}

data("gene_expression", package = "ggpubr")

test_that(".get_label_data works when label.select specified as a list", {
  results <- .get_label_data(
    data = gene_expression, x = "dataset", y = "XBP1", label = "bcr_patient_barcode",
    label.select = list(criteria = "`y` > 4.4 & `x` %in% c('BRCA', 'OV')"),
  )
  expected <- tibble::tribble(
    ~bcr_patient_barcode, ~dataset, ~GATA3, ~PTEN, ~XBP1,
    "BRCA149", "BRCA", 2.345125, 1.13357142857143, 4.48758333333333,
    "BRCA536", "BRCA", 3.1135, 0.846142857142857, 4.73675,
    "BRCA581", "BRCA", 3.38675, 0.152928571428571, 4.5725
  ) %>%
    data.frame(stringsAsFactors = FALSE)
  expect_equal(results, expected, tolerance = 1e-4)
})

#' Change the Appearance of Titles and Axis Labels
#'
#' @description Change the appearance of the main title, subtitle, caption, axis
#'  labels and text, as well as the legend title and texts. Wrapper around
#'  \code{\link[ggplot2:element]{element_text}()}.
#'
#' @param object character string specifying the plot components. Allowed values
#'  include: \itemize{ \item \code{"title"} for the main title \item
#'  \code{"subtitle"} for the plot subtitle \item \code{"caption"} for the plot
#'  caption \item \code{"legend.title"} for the legend title \item
#'  \code{"legend.text"} for the legend text \item \code{"x", "xlab", or "x.title"}
#'  for x axis label \item \code{"y", "ylab", or "y.title"} for y axis label \item
#'  \code{"xy", "xylab", "xy.title" or "axis.title"} for both x and y axis
#'  labels \item \code{"x.text"} for x axis texts (x axis tick labels) \item
#'  \code{"y.text"} for y axis texts (y axis tick labels) \item \code{"xy.text"}
#'  or \code{"axis.text"} for both x and y axis texts }
#'
#' @param size numeric value specifying the font size, (e.g.: \code{size = 12}).
#' @param color character string specifying the font color, (e.g.: \code{color =
#'  "red"}).
#' @param face the font face or style. Allowed values include one of
#'  \code{"plain", "bold", "italic", "bold.italic"}, (e.g.: \code{face =
#'  "bold.italic"}).
#' @param family the font family.
#' @param ... other arguments to pass to the function
#'  \code{\link[ggplot2:element]{element_text}()}.
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#'
#' # Basic plot
#' p <- ggboxplot(ToothGrowth,
#'   x = "dose", y = "len", color = "dose",
#'   title = "Box Plot created with ggpubr",
#'   subtitle = "Length by dose",
#'   caption = "Source: ggpubr",
#'   xlab = "Dose (mg)", ylab = "Teeth length"
#' )
#' p
#'
#' # Change the appearance of titles and labels
#' p +
#'   font("title", size = 14, color = "red", face = "bold.italic") +
#'   font("subtitle", size = 10, color = "orange") +
#'   font("caption", size = 10, color = "orange") +
#'   font("xlab", size = 12, color = "blue") +
#'   font("ylab", size = 12, color = "#993333") +
#'   font("xy.text", size = 12, color = "gray", face = "bold")
#'
#' # Change the appearance of legend title and texts
#' p +
#'   font("legend.title", color = "blue", face = "bold") +
#'   font("legend.text", color = "red")
#'
#' @export
font <- function(object, size = NULL, color = NULL, face = NULL, family = NULL, ...) {
  elmt <- element_text(
    size = size, color = color,
    face = face, family = family, ...
  )
  switch(object,
    title = theme(plot.title = elmt),
    subtitle = theme(plot.subtitle = elmt),
    caption = theme(plot.caption = elmt),
    x = theme(axis.title.x = elmt),
    xlab = theme(axis.title.x = elmt),
    x.title = theme(axis.title.x = elmt),
    y = theme(axis.title.y = elmt),
    ylab = theme(axis.title.y = elmt),
    y.title = theme(axis.title.y = elmt),
    xy = theme(axis.title.x = elmt, axis.title.y = elmt),
    xylab = theme(axis.title.x = elmt, axis.title.y = elmt),
    xy.title = theme(axis.title.x = elmt, axis.title.y = elmt),
    axis.title = theme(axis.title.x = elmt, axis.title.y = elmt),
    legendtitle = theme(legend.title = elmt),
    legend.title = theme(legend.title = elmt),
    legendtext = theme(legend.text = elmt),
    legend.text = theme(legend.text = elmt),

    # Axis tick labels
    x.text = theme(axis.text.x = elmt),
    y.text = theme(axis.text.y = elmt),
    xy.text = theme(axis.text.x = elmt, axis.text.y = elmt),
    yxtext = theme(axis.text.x = elmt, axis.text.y = elmt),
    axis.text = theme(axis.text.x = elmt, axis.text.y = elmt),
    stop("Don't support ", object)
  )
}

#' Generate Color Palettes
#'
#' @description Generate a palette of k colors from ggsci palettes, RColorBrewer
#'  palettes and custom color palettes. Useful to extend RColorBrewer and ggsci to support more colors.
#' @param palette Color palette. Allowed values include: \itemize{ \item
#'  \strong{Grey color palettes}: "grey" or "gray"; \item \strong{RColorBrewer
#'  palettes}, see \code{\link[RColorBrewer:ColorBrewer]{brewer.pal}} and details section.
#'  Examples of palette names include: "RdBu", "Blues", "Dark2", "Set2", ...;
#'  \item \strong{Custom color palettes}. For example, palette = c("#00AFBB",
#'  "#E7B800", "#FC4E07"); \item \strong{ggsci scientific journal palettes},
#'  e.g.: "npg", "aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
#'  "rickandmorty". }
#' @param k the number of colors to generate.
#' @details
#' \strong{RColorBrewer palettes}: To display all available color
#'  palettes, type this in R:RColorBrewer::display.brewer.all(). Color palette
#'  names include:
#'
#'  \itemize{ \item \strong{Sequential palettes}, suited to ordered data that
#'  progress from low to high. Palette names include: Blues BuGn BuPu GnBu
#'  Greens Greys Oranges OrRd PuBu PuBuGn PuRd Purples RdPu Reds YlGn YlGnBu
#'  YlOrBr YlOrRd. \item \strong{Diverging palettes}:Gradient colors. Names
#'  include: BrBG PiYG PRGn PuOr RdBu RdGy RdYlBu RdYlGn Spectral. \item
#'  \strong{Qualitative palettes}:  Best suited to representing nominal or
#'  categorical data. Names include: Accent, Dark2, Paired, Pastel1, Pastel2,
#'  Set1, Set2, Set3.
#'  }
#'
#' @return Returns a vector of color palettes.
#'
#' @examples
#' data("iris")
#' iris$Species2 <- factor(rep(c(1:10), each = 15))
#'
#' # Generate a gradient of 10 colors
#' ggscatter(iris,
#'   x = "Sepal.Length", y = "Petal.Length",
#'   color = "Species2",
#'   palette = get_palette(c("#00AFBB", "#E7B800", "#FC4E07"), 10)
#' )
#'
#' # Scatter plot with default color palette
#' ggscatter(iris,
#'   x = "Sepal.Length", y = "Petal.Length",
#'   color = "Species"
#' )
#'
#' # RColorBrewer color palettes
#' ggscatter(iris,
#'   x = "Sepal.Length", y = "Petal.Length",
#'   color = "Species", palette = get_palette("Dark2", 3)
#' )
#'
#' # ggsci color palettes
#' ggscatter(iris,
#'   x = "Sepal.Length", y = "Petal.Length",
#'   color = "Species", palette = get_palette("npg", 3)
#' )
#'
#' # Custom color palette
#' ggscatter(iris,
#'   x = "Sepal.Length", y = "Petal.Length",
#'   color = "Species",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#' # Or use this
#' ggscatter(iris,
#'   x = "Sepal.Length", y = "Petal.Length",
#'   color = "Species",
#'   palette = get_palette(c("#00AFBB", "#FC4E07"), 3)
#' )
#'
#' @export
get_palette <- function(palette = "default", k) {
  # Check if RColorBrewer, ggsci, hue or grey/gray color palettes
  if (.is_col_palette(palette)) {
    if (palette %in% .brewerpal()) {
      .get_brewer_pal(palette, k)
    } else if (palette %in% .ggscipal()) {
      .get_ggsci_pal(palette, k)
    } else if (palette %in% c("default", "hue")) {
      hues <- seq(15, 375, length = k + 1)
      grDevices::hcl(h = hues, l = 65, c = 100, alpha = 1)[seq_len(k)]
    }
    # Grey color palette
    else if (palette %in% c("grey", "gray")) {
      grDevices::grey.colors(k, start = 0.2, end = 0.8)
    }
  } else {
    grDevices::colorRampPalette(palette)(k)
  }
}

#' @include utilities.R
NULL
#' Draw group mean points
#' @description Draw the mean point of each group.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @param ... other arguments to pass to \code{\link[ggplot2]{geom_point}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning.
#'   If TRUE silently removes missing values.
#' @seealso \code{\link{stat_conf_ellipse}}, \code{\link{stat_chull}} and
#'   \code{\link{ggscatter}}
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#'
#' # Scatter plot with ellipses and group mean points
#' ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "cyl", shape = "cyl", ellipse = TRUE
#' ) +
#'   stat_mean(aes(color = cyl, shape = cyl), size = 4)
#'
#' @export
stat_mean <- function(mapping = NULL, data = NULL, geom = "point",
                      position = "identity", na.rm = FALSE, show.legend = NA,
                      inherit.aes = TRUE, ...) {
  layer(
    stat = StatMean, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, ...)
  )
}


StatMean <- ggproto("StatMean", Stat,
  required_aes = c("x", "y"),
  compute_group = function(data, scales) {
    res <- data.frame(
      x = mean(data$x, na.rm = TRUE),
      y = mean(data$y, na.rm = TRUE)
    )
    res
  }
)

#' Change Axis Scale: log2, log10 and more
#'
#' @description Change axis scale.
#' \itemize{
#' \item \code{xscale}: change x axis scale.
#' \item \code{yscale}: change y axis scale.
#' }
#' @param .scale axis scale. Allowed values are one of c("none", "log2", "log10",
#'  "sqrt", "percent", "dollar", "scientific"); e.g.: .scale="log2".
#' @param .format logical value. If TRUE, axis tick mark labels will be formatted
#'  when .scale  = "log2" or "log10".
#'
#'
#' @examples
#' # Basic scatter plots
#' data(cars)
#' p <- ggscatter(cars, x = "speed", y = "dist")
#' p
#'
#' # Set log scale
#' p + yscale("log2", .format = TRUE)

#' @name axis_scale
#' @rdname axis_scale
#' @export
xscale <- function(.scale, .format = FALSE) {
  .x <- NULL

  if (.format & .scale %in% c("percent", "dollar", "scientific")) {
    .format <- FALSE
  }


  if (.format) {
    if (.scale == "log2") {
      scale_x_continuous(
        trans = scales::log2_trans(),
        breaks = scales::trans_breaks("log2", function(x) 2^x),
        labels = scales::trans_format("log2", scales::math_format(2^.x))
      )
    } else if (.scale == "log10") {
      scale_x_continuous(
        trans = scales::log10_trans(),
        breaks = scales::trans_breaks("log10", function(x) 10^x),
        labels = scales::trans_format("log10", scales::math_format(10^.x))
      )
    }
  } else if (.scale %in% c("log2", "log10")) {
    scale_x_continuous(trans = .scale)
  } else {
    switch(.scale,
      percent = scale_x_continuous(labels = scales::percent),
      dollar = scale_x_continuous(labels = scales::dollar),
      scientific = scale_x_continuous(labels = scales::scientific)
    )
  }
}

#' @rdname axis_scale
#' @export
yscale <- function(.scale, .format = FALSE) {
  .x <- NULL

  if (.format & .scale %in% c("percent", "dollar", "scientific")) {
    .format <- FALSE
  }

  if (.format) {
    if (.scale == "log2") {
      scale_y_continuous(
        trans = scales::log2_trans(),
        breaks = scales::trans_breaks("log2", function(x) 2^x),
        labels = scales::trans_format("log2", scales::math_format(2^.x))
      )
    } else if (.scale == "log10") {
      scale_y_continuous(
        trans = scales::log10_trans(),
        breaks = scales::trans_breaks("log10", function(x) 10^x),
        labels = scales::trans_format("log10", scales::math_format(10^.x))
      )
    }
  } else if (.scale %in% c("log2", "log10")) {
    scale_y_continuous(trans = .scale)
  } else {
    switch(.scale,
      percent = scale_y_continuous(labels = scales::percent),
      dollar = scale_y_continuous(labels = scales::dollar),
      scientific = scale_y_continuous(labels = scales::scientific)
    )
  }
}

#' @include utilities.R utilities_label.R p_format_utils.R
NULL

#' Add Correlation Coefficients with P-values to a Scatter Plot
#' @description Add correlation coefficients with p-values to a scatter plot. Can
#'  be also used to add `R2`.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @param method a character string indicating which correlation coefficient (or
#'  covariance) is to be computed. One of "pearson" (default), "kendall", or
#'  "spearman".
#' @param alternative a character string specifying the alternative hypothesis,
#'  must be one of "two.sided" (default), "greater" or "less". You can specify
#'  just the initial letter.
#' @param cor.coef.name character. Can be one of \code{"R"} (pearson coef),
#'  \code{"rho"} (spearman coef) and \code{"tau"} (kendall coef). Uppercase and
#'  lowercase are allowed.
#' @param label.sep a character string to separate the terms. Default is ", ", to
#'  separate the correlation coefficient and the p.value.
#' @param label.x.npc,label.y.npc can be \code{numeric} or \code{character}
#'  vector of the same length as the number of groups and/or panels. If too
#'  short they will be recycled. \itemize{ \item If \code{numeric}, value should
#'  be between 0 and 1. Coordinates to be used for positioning the label,
#'  expressed in "normalized parent coordinates". \item If \code{character},
#'  allowed values include: i) one of c('right', 'left', 'center', 'centre',
#'  'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
#'  'middle') for y-axis.}
#'
#'  If too short they will be recycled.
#' @param label.x,label.y \code{numeric} Coordinates (in data units) to be used
#'  for absolute positioning of the label. If too short they will be recycled.
#' @param label.y.step numeric value giving the vertical spacing (in text-line
#'  units) between the labels of successive groups when several groups are present.
#'  Default is 1.4 (unchanged behavior). Set \code{label.y.step = 0} to stop the
#'  per-group vertical shift, so that labels align across facet panels when a
#'  factor is mapped to an aesthetic that also defines the facets. This places the
#'  labels flush with the top of each panel; it is similar in spirit to the
#'  \code{aes(group = 1)} workaround, which instead leaves them one text line lower.
#' @param label.anchor character. How \code{label.x.npc}/\code{label.y.npc} are
#'  interpreted. \code{"data"} (default) converts them to data coordinates using
#'  the data range, so the label follows the data; the output is unchanged from
#'  previous versions. \code{"panel"} places the label at the true panel-relative
#'  position (npc), so the labels stay aligned across panels/facets whose axis
#'  ranges differ - e.g. with \code{facet_wrap(scales = "free_y")}, with
#'  \code{geom_smooth()} extending each panel by a different amount, or across
#'  separate plots combined with \code{\link{ggarrange}()}. An explicit
#'  \code{label.x}/\code{label.y} always stays in data units regardless of
#'  \code{label.anchor}. Requires ggplot2 >= 3.5.0 (already a dependency).
#' @param output.type character One of "expression", "latex", "tex" or "text".
#' @param digits,r.digits,p.digits integer indicating the number of decimal
#'  places (round) or significant digits (signif) to be used for the correlation
#'  coefficient and the p-value, respectively. In the default p-value label style,
#'  the displayed p-value keeps the legacy raw value unless \code{p.accuracy} is
#'  set; \code{p.digits} is used by non-default \code{p.format.style} outputs and
#'  the computed \code{p} variable.
#' @param r.accuracy a real value specifying the number of decimal places of
#'  precision for the correlation coefficient. Default is NULL. Use (e.g.) 0.01
#'  to show 2 decimal places of precision. If specified, then \code{r.digits} is
#'  ignored.
#' @param p.accuracy a real value specifying the number of decimal places of
#'  precision for the p-value. Default is NULL. Use (e.g.) 0.0001 to show 4
#'  decimal places of precision. If specified, then \code{p.digits} is ignored.
#' @param r.leading.zero logical. Whether to include the leading zero before the
#'  decimal point in the correlation coefficient (e.g., \code{"0.73"} vs
#'  \code{".73"}). Default (NULL) keeps the leading zero; set to \code{FALSE}
#'  for APA-style reporting.
#' @param p.coef.name character. Symbol used for the p-value label. Default is
#'  \code{"p"}; use \code{"P"} for an uppercase p-value label. For
#'  \code{output.type = "expression"} this should be a single valid plotmath
#'  symbol (e.g. \code{"P"}), since it is parsed as an expression.
#' @param p.format.style character specifying the p-value formatting style.
#'  One of "default", "apa", "nejm", "lancet", "ama", "graphpad", "scientific".
#'  Default is "default" for backward compatibility.
#' @param p.leading.zero logical. Whether to include leading zero before decimal
#'  point (e.g., "0.05" vs ".05"). If NULL, uses the style's default setting.
#' @param p.decimal.mark character string to use as the decimal mark. If NULL,
#'  uses \code{getOption("OutDec")}.
#' @param conf.level confidence level for the confidence interval of the
#'  correlation coefficient, used to compute the \code{conf.int.low},
#'  \code{conf.int.high} and \code{conf.int.label} computed variables (see the
#'  \strong{Computed variables} section). Default is 0.95. A confidence interval
#'  is only available for \code{method = "pearson"} (with at least 4 complete
#'  observations); for \code{"spearman"}/\code{"kendall"} the confidence-interval
#'  variables are \code{NA}.
#' @param ... other arguments to pass to \code{\link[ggplot2]{geom_text}} or
#'  \code{\link[ggplot2:geom_text]{geom_label}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning. If
#'  TRUE silently removes missing values.
#' @seealso \code{\link{ggscatter}}. For an alternative implementation with more
#'  control over label positioning (native NPC coordinates, per-group vertical and
#'  horizontal steps), see \code{ggpmisc::stat_correlation()}.
#' @section Computed variables: \describe{ \item{r}{correlation coefficient}
#'  \item{rr}{correlation coefficient squared} \item{rmse}{root mean square
#'  deviation (RMSE/RMSD) between \code{x} and \code{y}, computed on the complete
#'  pairs; meaningful when \code{x} and \code{y} are on the same scale (e.g.
#'  predicted vs. reference values). Rounded and formatted with the coefficient
#'  settings (\code{r.digits} / \code{r.accuracy}).} \item{r.label}{formatted
#'  label for the correlation coefficient} \item{rr.label}{formatted label for
#'  the squared correlation coefficient} \item{p.label}{label for the p-value}
#'  \item{rmse.label}{formatted label for the RMSE/RMSD}
#'  \item{conf.int.low, conf.int.high}{lower and upper bounds of the confidence
#'  interval of the correlation coefficient (Pearson only; \code{NA} for
#'  Spearman/Kendall), at the level given by \code{conf.level}}
#'  \item{conf.int.label}{formatted label for the confidence interval, e.g.
#'  \code{"95\% CI [0.21, 0.75]"} (\code{NA} when the interval is unavailable)}
#'  \item{label}{default label displayed by \code{stat_cor()}} }
#'
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#'
#' # Scatter plot with correlation coefficient
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' sp <- ggscatter(df,
#'   x = "wt", y = "mpg",
#'   add = "reg.line", # Add regressin line
#'   add.params = list(color = "blue", fill = "lightgray"), # Customize reg. line
#'   conf.int = TRUE # Add confidence interval
#' )
#' # Add correlation coefficient
#' sp + stat_cor(method = "pearson", label.x = 3, label.y = 30)
#'
#' # Specify the number of decimal places of precision for p and r
#' # Using 3 decimal places for the p-value and
#' # 2 decimal places for the correlation coefficient (r)
#' sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01)
#'
#' # Show only the r.label but not the p.label
#' sp + stat_cor(aes(label = after_stat(r.label)), label.x = 3)
#'
#' # Use R2 instead of R
#' ggscatter(df, x = "wt", y = "mpg", add = "reg.line") +
#'   stat_cor(
#'     aes(label = paste(after_stat(rr.label), after_stat(p.label), sep = "~`,`~")),
#'     label.x = 3
#'   )
#'
#' # Show the RMSE/RMSD (root mean square deviation) between x and y
#' # (useful for agreement between paired measurements on the same scale)
#' sp + stat_cor(aes(label = after_stat(rmse.label)), label.x = 3)
#'
#' # Combine the correlation coefficient and the RMSE (comma-separated)
#' sp + stat_cor(
#'   aes(label = paste(after_stat(r.label), after_stat(rmse.label), sep = "~`,`~")),
#'   label.x = 3
#' )
#'
#' # Show the confidence interval of the correlation coefficient (Pearson)
#' sp + stat_cor(aes(label = after_stat(conf.int.label)), label.x = 3)
#'
#' # Correlation coefficient with its confidence interval
#' sp + stat_cor(
#'   aes(label = paste(after_stat(r.label), after_stat(conf.int.label), sep = "~`,`~")),
#'   label.x = 1.5
#' )
#'
#' # Color by groups and facet
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::::
#' sp <- ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "cyl", palette = "jco",
#'   add = "reg.line", conf.int = TRUE
#' )
#' sp + stat_cor(aes(color = cyl), label.x = 3)
#'
#' @export
stat_cor <- function(mapping = NULL, data = NULL,
                     method = "pearson", alternative = "two.sided",
                     cor.coef.name = c("R", "rho", "tau"), label.sep = ", ",
                     label.x.npc = "left", label.y.npc = "top",
                     label.x = NULL, label.y = NULL, label.y.step = 1.4,
                     label.anchor = c("data", "panel"),
                     output.type = "expression",
                     digits = 2, r.digits = digits, p.digits = digits,
                     r.accuracy = NULL, p.accuracy = NULL,
                     r.leading.zero = NULL,
                     p.format.style = "default", p.leading.zero = NULL,
                     p.decimal.mark = NULL, p.coef.name = "p", conf.level = 0.95,
                     geom = "text", position = "identity", na.rm = FALSE, show.legend = NA,
                     inherit.aes = TRUE, ...) {
  parse <- ifelse(output.type == "expression", TRUE, FALSE)
  cor.coef.name <- cor.coef.name[1]
  label.anchor <- match.arg(label.anchor)
  if (!is.numeric(conf.level) || length(conf.level) != 1L || is.na(conf.level) ||
      conf.level <= 0 || conf.level >= 1) {
    stop("`conf.level` must be a single number between 0 and 1.", call. = FALSE)
  }
  layer(
    stat = StatCor, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y, label.y.step = label.y.step,
      label.anchor = label.anchor,
      label.sep = label.sep,
      method = method, alternative = alternative, output.type = output.type, digits = digits,
      r.digits = r.digits, p.digits = p.digits, r.accuracy = r.accuracy,
      p.accuracy = p.accuracy, r.leading.zero = r.leading.zero,
      p.format.style = p.format.style,
      p.leading.zero = p.leading.zero, p.decimal.mark = p.decimal.mark,
      cor.coef.name = cor.coef.name, p.coef.name = p.coef.name,
      conf.level = conf.level,
      parse = parse, na.rm = na.rm, ...
    )
  )
}


StatCor <- ggproto("StatCor", Stat,
  required_aes = c("x", "y"),
  default_aes = aes(hjust = after_stat(hjust), vjust = after_stat(vjust)),
  compute_group = function(data, scales, method, alternative, label.x.npc, label.y.npc,
                           label.x, label.y, label.y.step = 1.4, label.anchor = "data",
                           label.sep, output.type, digits,
                           r.digits, p.digits, r.accuracy, p.accuracy, r.leading.zero,
                           p.format.style, p.leading.zero, p.decimal.mark, cor.coef.name,
                           p.coef.name, conf.level = 0.95) {
    if (length(unique(data$x)) < 2) {
      # Not enough data to perform test
      return(data.frame())
    }
    # Returns a data frame with estimate, p.value, label, method
    .test <- .cor_test(
      data$x, data$y,
      method = method, alternative = alternative,
      label.sep = label.sep, output.type = output.type, digits = digits,
      r.digits = r.digits, p.digits = p.digits,
      r.accuracy = r.accuracy, p.accuracy = p.accuracy,
      r.leading.zero = r.leading.zero,
      p.format.style = p.format.style, p.leading.zero = p.leading.zero,
      p.decimal.mark = p.decimal.mark,
      cor.coef.name = cor.coef.name, p.coef.name = p.coef.name,
      conf.level = conf.level
    )
    # Returns a data frame with label: x, y, hjust, vjust
    .label.pms <- .label_params(
      data = data, scales = scales,
      label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y, label.y.step = label.y.step,
      label.anchor = label.anchor
    ) %>%
      mutate(hjust = 0)
    cbind(.test, .label.pms)
  }
)


# Correlation test
# ::::::::::::::::::::::::::::::::::::::::
# Returns a data frame: estimatel|p.value|method|label
.cor_test <- function(x, y, method = "pearson", alternative = "two.sided",
                      label.sep = ", ", output.type = "expression",
                      digits = 2, r.digits = digits, p.digits = digits,
                      r.accuracy = NULL, p.accuracy = NULL, r.leading.zero = NULL,
                      p.format.style = "default", p.leading.zero = NULL,
                      p.decimal.mark = NULL,
                      cor.coef.name = "R", p.coef.name = "p", conf.level = 0.95) {
  # Overwritting digits by accuracy, if specified
  if (!is.null(p.accuracy)) {
    nb_decimal_places <- round(abs(log10(p.accuracy)))
    p.digits <- nb_decimal_places
  }
  if (!is.null(r.accuracy)) {
    nb_decimal_places <- round(abs(log10(r.accuracy)))
    r.digits <- nb_decimal_places
  }

  # Correlation analyses
  .cor <- suppressWarnings(stats::cor.test(
    x, y,
    method = method, alternative = alternative,
    conf.level = conf.level,
    use = "complete.obs"
  ))
  # Confidence interval of the correlation coefficient (#418). Only Pearson's
  # cor.test() returns a conf.int (and only for n >= 4); Spearman/Kendall do not,
  # so the bounds are NA in those cases (never an error).
  ci <- .cor$conf.int
  if (is.null(ci) || length(ci) < 2) {
    conf.low.value <- NA_real_
    conf.high.value <- NA_real_
  } else {
    conf.low.value <- ci[1]
    conf.high.value <- ci[2]
  }
  # Root Mean Square Deviation (RMSE/RMSD) between x and y, computed on the same
  # complete pairs used by cor.test() (#458). Meaningful when x and y are on the
  # same scale (e.g. predicted vs. reference values).
  ok <- stats::complete.cases(x, y)
  rmse.value <- sqrt(mean((x[ok] - y[ok])^2))

  estimate <- p.value <- p <- r <- rr <- rmse <- NULL
  conf.int.low <- conf.int.high <- NULL
  z <- data.frame(estimate = .cor$estimate, p.value = .cor$p.value, method = method) %>%
    mutate(
      r = signif(estimate, r.digits),
      rr = signif(estimate^2, r.digits),
      p = signif(p.value, p.digits),
      rmse = signif(rmse.value, r.digits),
      conf.int.low = signif(conf.low.value, r.digits),
      conf.int.high = signif(conf.high.value, r.digits)
    )

  # Defining p and r labels
  pval <- .cor$p.value
  z <- z %>%
    dplyr::mutate(
      r.label = get_corcoef_label(
        r,
        accuracy = r.accuracy, prefix = "R",
        cor.coef.name = cor.coef.name, type = output.type,
        leading.zero = r.leading.zero
      ),
      rr.label = get_corcoef_label(
        rr,
        accuracy = r.accuracy, prefix = "R2",
        cor.coef.name = cor.coef.name, type = output.type,
        leading.zero = r.leading.zero
      ),
      p.label = get_p_label(
        pval,
        p.digits = p.digits, accuracy = p.accuracy, type = output.type,
        p.format.style = p.format.style, p.leading.zero = p.leading.zero,
        p.decimal.mark = p.decimal.mark, p.coef.name = p.coef.name
      ),
      rmse.label = get_rmse_label(
        rmse.value, accuracy = r.accuracy, digits = r.digits, type = output.type
      ),
      conf.int.label = get_conf_int_label(
        conf.low.value, conf.high.value, conf.level = conf.level,
        accuracy = r.accuracy, digits = r.digits,
        leading.zero = r.leading.zero, type = output.type
      )
    )

  # Defining correlation labels
  if (output.type == "expression") {
    if (label.sep == "\n") {
      # Line break at each comma
      cortxt <- paste0("atop(", z$r.label, ",", z$p.label, ")")
    } else {
      label.sep <- trimws(label.sep)
      if (label.sep == "") {
        label.sep <- "~"
      } #  Using "*" to avoid the space between the R2 value and comma
      else {
        label.sep <- paste0("*`", label.sep, "`~")
      }
      cortxt <- paste0(z$r.label, label.sep, z$p.label)
    }
  } else if (output.type %in% c("latex", "tex", "text")) {
    cortxt <- paste0(z$r.label, label.sep, z$p.label)
  }
  z$label <- cortxt
  z
}


# Formatting R and P ----------------------
get_p_label <- function(x, p.digits = 2, accuracy = 0.0001, type = "expression",
                        p.format.style = "default", p.leading.zero = NULL,
                        p.decimal.mark = NULL, p.coef.name = "p") {
  if (is.null(p.decimal.mark)) {
    p.decimal.mark <- getOption("OutDec")
  }

  if (!is.null(p.format.style) && p.format.style != "default") {
    p_digits <- p.digits
    if (!is.null(accuracy)) {
      p_digits <- round(abs(log10(accuracy)))
    }
    digits_by_value <- .signif_to_decimal_digits(x, p_digits)
    p_formatted <- vapply(seq_along(x), function(i) {
      format_p_value(
        x[i],
        style = p.format.style,
        digits = digits_by_value[i],
        leading.zero = p.leading.zero,
        min.threshold = NULL,
        decimal.mark = p.decimal.mark
      )
    }, character(1))
    label <- create_p_label(p_formatted)
  } else {
    # Backward compatible behavior (scales::pvalue + accuracy)
    if (is.null(accuracy)) {
      label <- ifelse(x < 2.2e-16, "p < 2.2e-16", paste0("p = ", x))
    } else if (!(accuracy < 1)) {
      stop(
        "Accuracy should be < 1; For example use 0.01, 0.001, 0.0001, etc.",
        call. = FALSE
      )
    } else {
      label <- scales::pvalue(x, accuracy = accuracy, add_p = TRUE)
      # Add space before and after: = or <
      label <- gsub(pattern = "(=|<)", replacement = " \\1 ", x = label)
    }

    if (!is.null(p.leading.zero) && !p.leading.zero) {
      label <- sub("^((?:p\\s*[=<]\\s*))0\\.", "\\1.", label, perl = TRUE)
      label <- sub("^((?:p\\s*[=<]\\s*))-0\\.", "\\1-.", label, perl = TRUE)
    }
    if (!is.null(p.decimal.mark) && p.decimal.mark != ".") {
      label <- sub("^((?:p\\s*[=<]\\s*)-?[0-9]+)\\.", paste0("\\1", p.decimal.mark), label, perl = TRUE)
    }
  }

  if (type == "expression") {
    label <- gsub(pattern = "p = ", replacement = "italic(p)~`=`~", x = label, fixed = TRUE)
    label <- gsub(pattern = "p < ", replacement = "italic(p)~`<`~", x = label, fixed = TRUE)
    # Quote a value whose leading zero was explicitly dropped (p.leading.zero =
    # FALSE) so plotmath renders it literally instead of re-adding the zero
    # (e.g. .001 -> "0.001"). Scoped to the explicit flag so p.format.style
    # presets keep their existing numeric rendering (#540).
    if (!is.null(p.leading.zero) && !p.leading.zero) {
      label <- .quote_leading_zero_dropped(label)
    }
  }
  # Optionally use a custom symbol for the p-value label, e.g. uppercase "P" (#541).
  if (!identical(p.coef.name, "p")) {
    if (type == "expression") {
      label <- gsub("italic(p)", paste0("italic(", p.coef.name, ")"), label, fixed = TRUE)
    } else {
      label <- sub("^p", p.coef.name, label)
    }
  }
  label
}

.signif_to_decimal_digits <- function(x, sig_digits) {
  if (is.null(sig_digits)) {
    sig_digits <- 2L
  }
  x_abs <- abs(x)
  out <- ifelse(
    is.na(x_abs) | x_abs == 0,
    sig_digits,
    pmax(0, sig_digits - 1L - floor(log10(x_abs)))
  )
  as.integer(out)
}

# Prefix can be R or R^2.
# cor.coef.name: R, rho, tau
get_corcoef_label <- function(x, accuracy = 0.01, prefix = "R", cor.coef.name = "R", type = "expression",
                              leading.zero = NULL) {
  if (is.null(accuracy)) {
    label <- paste0(prefix, " = ", x)
  } else if (!(accuracy < 1)) {
    stop(
      "Accuracy should be < 1; For example use 0.01, 0.001, 0.0001, etc.",
      call. = FALSE
    )
  } else {
    nb_decimal_places <- round(abs(log10(accuracy)))
    label <- formatC(x, digits = nb_decimal_places, format = "f", decimal.mark = ".")
    label <- paste0(prefix, " = ", label)
  }
  if (!is.null(leading.zero) && !leading.zero) {
    # Drop the leading zero of the coefficient value (e.g. "R = 0.20" -> "R = .20",
    # "R = -0.87" -> "R = -.87") for APA-style reporting (#540).
    label <- sub("= (-?)0\\.", "= \\1.", label)
  }
  if (type == "expression") {
    label <- gsub(pattern = "R2 = ", replacement = "italic(R)^2~`=`~", x = label, fixed = TRUE)
    label <- gsub(pattern = "R = ", replacement = "italic(R)~`=`~", x = label, fixed = TRUE)
    if (!is.null(leading.zero) && !leading.zero) {
      label <- .quote_leading_zero_dropped(label)
    }
  }
  label <- gsub(pattern = "R", cor.coef.name, x = label, fixed = TRUE)
  label
}

# Format the RMSE/RMSD label (#458). Kept separate from get_corcoef_label()
# because the latter substitutes "R" -> cor.coef.name, which would corrupt
# "RMSE". `x` is the RAW rmse value: unlike the correlation coefficient it is
# not bounded to [-1, 1], so it is formatted from the raw value (significant
# digits when `accuracy` is NULL, else fixed decimal places) rather than a
# pre-rounded one, to avoid losing precision for values > 1.
get_rmse_label <- function(x, accuracy = NULL, digits = 2, type = "expression") {
  if (is.null(accuracy)) {
    value <- signif(x, digits)
    label <- paste0("RMSE = ", value)
  } else if (!(accuracy < 1)) {
    stop(
      "Accuracy should be < 1; For example use 0.01, 0.001, 0.0001, etc.",
      call. = FALSE
    )
  } else {
    nb_decimal_places <- round(abs(log10(accuracy)))
    value <- formatC(x, digits = nb_decimal_places, format = "f", decimal.mark = ".")
    label <- paste0("RMSE = ", value)
  }
  if (type == "expression") {
    label <- gsub(pattern = "RMSE = ", replacement = "italic(RMSE)~`=`~", x = label, fixed = TRUE)
  }
  label
}

# Format the confidence interval label of the correlation coefficient (#418),
# e.g. "95% CI [0.21, 0.75]". Returns NA when the CI is unavailable (Spearman /
# Kendall / n < 4). For the expression output the whole label is a single quoted
# plotmath string, so the percent sign and the brackets render literally and the
# bounds are not re-normalized (e.g. a dropped leading zero survives).
get_conf_int_label <- function(low, high, conf.level = 0.95, accuracy = NULL,
                               digits = 2, leading.zero = NULL, type = "expression") {
  if (length(low) == 0 || length(high) == 0 || is.na(low) || is.na(high)) {
    return(NA_character_)
  }
  fmt <- function(x) {
    if (is.null(accuracy)) {
      value <- as.character(signif(x, digits))
    } else {
      nb_decimal_places <- round(abs(log10(accuracy)))
      value <- formatC(x, digits = nb_decimal_places, format = "f", decimal.mark = ".")
    }
    if (!is.null(leading.zero) && !leading.zero) {
      value <- sub("^(-?)0\\.", "\\1.", value)
    }
    value
  }
  ci.prefix <- paste0(format(conf.level * 100), "% CI")
  body <- paste0(ci.prefix, " [", fmt(low), ", ", fmt(high), "]")
  if (type == "expression") {
    # Quote the whole label so plotmath renders it literally.
    body <- paste0("\"", body, "\"")
  }
  body
}

# In plotmath, a bare value whose leading zero was dropped (e.g. .87) is parsed
# as a number and rendered back WITH the zero (0.87). To make APA-style
# leading-zero suppression survive rendering, quote such a value so plotmath
# treats it as a literal string, keeping any minus sign outside the quotes
# (e.g. italic(R)~`=`~-.87  ->  italic(R)~`=`~-".87"). Only values that start
# with "." (i.e. already stripped) are affected; "0.87", "1.00", scientific
# notation are left untouched (#540).
.quote_leading_zero_dropped <- function(label) {
  sub("~(-?)(\\.[0-9]+)$", '~\\1"\\2"', label)
}

context("test-stat-cor-label-anchor")

# #248 (Cause 2): stat_cor()/stat_regline_equation() gain label.anchor. With
# label.anchor = "panel" the label is placed at true panel-relative npc (via
# AsIs), so labels align across panels/facets with different axis ranges (e.g.
# scales = "free_y" or geom_smooth extending each panel differently). The default
# label.anchor = "data" is byte-identical to the historical behavior.

.cor_label <- function(p) {
  b <- suppressMessages(ggplot2::ggplot_build(p))
  i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomText"), logical(1)))[1]
  d <- b$data[[i]]
  d[order(d$PANEL), ]
}

d1 <- data.frame(x = c(1, 2, 3, 4, 5, 6), y = c(2, 4, 5, 4, 6, 7))

# ---- Characterization: the default "data" positions must not change ----

test_that("default stat_cor label position (data anchor) is unchanged", {
  d <- .cor_label(ggscatter(d1, "x", "y") + stat_cor())
  # "left" -> x = data min (1); "top" -> y = data max (7); vjust = label.y.step (1.4).
  # expect_identical (not just equal) pins the exact endpoint values, so a
  # regression that changes the anchor formula (wrong fraction, wrong endpoint,
  # or accidentally emitting an npc/AsIs value) fails here.
  expect_identical(as.numeric(d$x), 1)
  expect_identical(as.numeric(d$y), 7)
  expect_identical(d$vjust, 1.4)
  expect_false(inherits(d$x, "AsIs"))    # data coords, not npc
})

test_that("numeric and character npc positions (data anchor) are unchanged", {
  dn <- .cor_label(ggscatter(d1, "x", "y") + stat_cor(label.x.npc = 0.1, label.y.npc = 0.9))
  expect_equal(as.numeric(dn$x), 1.5)    # 1 + 0.1*(6-1)
  expect_equal(as.numeric(dn$y), 6.5)    # 2 + 0.9*(7-2)
  dc <- .cor_label(ggscatter(d1, "x", "y") + stat_cor(label.x.npc = "center", label.y.npc = "center"))
  expect_identical(as.numeric(dc$x), mean(c(1, 6)))  # center uses mean(range) exactly
  expect_identical(as.numeric(dc$y), mean(c(2, 7)))
  db <- .cor_label(ggscatter(d1, "x", "y") + stat_cor(label.x.npc = "right", label.y.npc = "bottom"))
  expect_identical(as.numeric(db$x), 6)  # data max x
  expect_identical(as.numeric(db$y), 2)  # data min y
})

test_that("per-group vjust formulas (top/bottom/center) are unchanged", {
  set.seed(1)
  dg <- data.frame(x = rnorm(30), y = rnorm(30), g = rep(c("a", "b", "c"), 10))
  vj <- function(npc) {
    p <- ggplot(dg, aes(x, y, color = g)) + geom_point() +
      stat_cor(aes(color = g), label.y.npc = npc)
    sort(.cor_label(p)$vjust)
  }
  # group ids 1,2,3 with label.y.step = 1.4 (.label_params runs per group, so
  # length(group.id) == 1 in each call)
  expect_equal(vj("top"),    c(1.4, 2.8, 4.2))            # step * id
  expect_equal(vj("bottom"), c(-4.2, -2.8, -1.4))         # -step * id
  expect_equal(vj("center"), sort(1.4 * (1:3) - 1.4 / 2))  # step*id - step/2
})

test_that("stat_compare_means labels are byte-identical (shared helper untouched)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  scm <- function(p) {
    b <- suppressMessages(ggplot2::ggplot_build(p))
    i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomText"), logical(1)))[1]
    d <- b$data[[i]]
    data.frame(x = as.numeric(d$x), y = as.numeric(d$y), vjust = d$vjust)
  }
  s1 <- scm(ggboxplot(df, "dose", "len") + stat_compare_means())
  expect_identical(as.numeric(s1$x), 1)
  expect_identical(s1$y, 33.9)                # data max area (dimension()[2]) - exact
  expect_false(inherits(s1$y, "AsIs"))        # never npc/AsIs
  s2 <- scm(ggboxplot(df, "dose", "len", color = "supp") + stat_compare_means(aes(group = supp)))
  expect_equal(as.numeric(s2$x), c(1, 2, 3))
  expect_true(all(s2$y == 33.9))
})

test_that("explicit label.x/label.y stay in data coordinates in both modes", {
  dd <- .cor_label(ggscatter(d1, "x", "y") + stat_cor(label.x = 2, label.y = 5))
  expect_equal(as.numeric(dd$x), 2); expect_equal(as.numeric(dd$y), 5)
  # even with label.anchor = "panel", explicit coords win and stay data coords
  dp <- .cor_label(ggscatter(d1, "x", "y") + stat_cor(label.x = 2, label.y = 5, label.anchor = "panel"))
  expect_equal(as.numeric(dp$x), 2); expect_equal(as.numeric(dp$y), 5)
  expect_false(inherits(dp$x, "AsIs"))
})

# ---- The fix: panel anchor aligns labels across free-scale facets ----

.free_facet <- function(anchor) {
  d <- data.frame(x = c(1, 2, 3, 1, 1, 2, 3, 1), y = c(1, 2, 3, 3, 1, 2, 2, 2),
                  z = c("A", "A", "A", "A", "B", "B", "B", "B"))
  p <- ggplot(d, aes(x, y)) + geom_point() +
    ggplot2::geom_smooth(method = "lm", se = FALSE, formula = y ~ x) +
    stat_cor(label.anchor = anchor) +
    ggplot2::facet_wrap(ggplot2::vars(z), scales = "free_y")
  b <- suppressMessages(ggplot2::ggplot_build(p))
  i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomText"), logical(1)))[1]
  td <- b$data[[i]]
  td[order(td$PANEL), ]
}

test_that("label.anchor = 'panel' emits AsIs npc equal across free_y panels", {
  tp <- .free_facet("panel")
  expect_true(inherits(tp$y, "AsIs"))
  expect_equal(length(unique(as.numeric(tp$y))), 1L)   # same npc in every panel
  # default 'data' anchor gives DIFFERENT y per panel (the bug being fixed)
  tdd <- .free_facet("data")
  expect_false(inherits(tdd$y, "AsIs"))
  expect_gt(length(unique(round(as.numeric(tdd$y), 4))), 1L)
})

test_that("label.anchor = 'panel' renders for stat_cor and stat_regline_equation", {
  d <- data.frame(x = c(1, 2, 3, 1, 1, 2, 3, 1), y = c(1, 2, 3, 3, 1, 2, 2, 2),
                  z = c("A", "A", "A", "A", "B", "B", "B", "B"))
  base <- ggplot(d, aes(x, y)) + geom_point() +
    ggplot2::geom_smooth(method = "lm", se = FALSE, formula = y ~ x) +
    ggplot2::facet_wrap(ggplot2::vars(z), scales = "free_y")
  expect_error(suppressMessages(ggplot2::ggplotGrob(base + stat_cor(label.anchor = "panel"))), NA)
  expect_error(suppressMessages(ggplot2::ggplotGrob(base + stat_regline_equation(label.anchor = "panel"))), NA)
})

test_that("multiple grouped labels still stack under panel anchor (label.y.step)", {
  set.seed(1)
  dg <- data.frame(x = rnorm(30), y = rnorm(30), g = rep(c("a", "b", "c"), 10))
  p <- ggplot(dg, aes(x, y, color = g)) + geom_point() +
    stat_cor(aes(color = g), label.anchor = "panel")
  td <- .cor_label(p)
  expect_equal(nrow(td), 3)
  expect_true(inherits(td$y, "AsIs"))
  expect_equal(length(unique(as.numeric(td$y))), 1L)   # same npc anchor...
  expect_equal(length(unique(td$vjust)), 3L)            # ...offset by vjust per group
})

test_that("label.anchor rejects invalid values", {
  # message is locale-dependent (match.arg); just assert it errors
  expect_error(stat_cor(label.anchor = "nope"))
  expect_error(stat_regline_equation(label.anchor = "nope"))
})

context("test-stat-cor-conf-int")

# stat_cor() exposes the confidence interval of the correlation coefficient as
# computed variables conf.int.low, conf.int.high (numeric) and conf.int.label
# (formatted), plus a conf.level argument (#418). Pearson only; NA otherwise.
# Purely additive: the default label is unchanged.

.cor_label <- function(p) {
  b <- ggplot2::ggplot_build(p)
  d <- b$data[[which(vapply(b$data, function(x) "label" %in% names(x), logical(1)))[1]]]
  as.character(d$label)[1]
}

set.seed(1)
.df <- data.frame(x = rnorm(30))
.df$y <- .df$x * 0.5 + rnorm(30)
.sp <- ggscatter(.df, "x", "y")

test_that("the default stat_cor label is unchanged (no regression, #418)", {
  # The default composed label is R + p only; the CI variables never join it.
  lab <- .cor_label(.sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01))
  expect_match(lab, "^italic\\(R\\)~`=`~-?[0-9.]+\\*`,`~italic\\(p\\)~`=`~")
  expect_false(grepl("CI", lab))
})

test_that("conf.int.label reports the Pearson CI (text output)", {
  ci <- cor.test(.df$x, .df$y)$conf.int
  lab <- .cor_label(
    .sp + stat_cor(aes(label = after_stat(conf.int.label)), output.type = "text")
  )
  expect_equal(lab, paste0("95% CI [", signif(ci[1], 2), ", ", signif(ci[2], 2), "]"))
})

test_that("conf.int.label is valid plotmath and parses", {
  lab <- .cor_label(.sp + stat_cor(aes(label = after_stat(conf.int.label))))
  expect_match(lab, "^\"95% CI \\[")
  expect_error(parse(text = lab), NA)
})

test_that("conf.level controls the interval and the label prefix", {
  ci99 <- cor.test(.df$x, .df$y, conf.level = 0.99)$conf.int
  lab <- .cor_label(
    .sp + stat_cor(aes(label = after_stat(conf.int.label)),
      output.type = "text", conf.level = 0.99)
  )
  expect_equal(lab, paste0("99% CI [", signif(ci99[1], 2), ", ", signif(ci99[2], 2), "]"))
})

test_that("conf.int.low/high numeric match cor.test bounds (Pearson)", {
  ci <- cor.test(.df$x, .df$y)$conf.int
  b <- ggplot2::ggplot_build(
    .sp + stat_cor(aes(label = after_stat(paste(conf.int.low, conf.int.high))))
  )
  d <- b$data[[which(vapply(b$data, function(x) "label" %in% names(x), logical(1)))[1]]]
  expect_equal(as.character(d$label)[1],
               paste(signif(ci[1], 2), signif(ci[2], 2)))
})

test_that("conf.int variables are NA for Spearman and Kendall (no error)", {
  for (m in c("spearman", "kendall")) {
    lab <- .cor_label(
      .sp + stat_cor(aes(label = after_stat(conf.int.label)),
        method = m, output.type = "text")
    )
    expect_true(is.na(lab))
  }
})

test_that("invalid conf.level is rejected with a clear error", {
  expect_error(stat_cor(conf.level = 95), "between 0 and 1")
  expect_error(stat_cor(conf.level = 2), "between 0 and 1")
  expect_error(stat_cor(conf.level = -0.1), "between 0 and 1")
  expect_error(stat_cor(conf.level = c(0.9, 0.95)), "single number")
  expect_error(stat_cor(conf.level = NA), "between 0 and 1")
  # valid values are accepted
  expect_error(stat_cor(conf.level = 0.9), NA)
})

test_that("conf.int.label honors r.accuracy and r.leading.zero", {
  ci <- cor.test(.df$x, .df$y)$conf.int
  lab_acc <- .cor_label(
    .sp + stat_cor(aes(label = after_stat(conf.int.label)),
      output.type = "text", r.accuracy = 0.001)
  )
  expect_equal(lab_acc, paste0("95% CI [",
    formatC(ci[1], digits = 3, format = "f"), ", ",
    formatC(ci[2], digits = 3, format = "f"), "]"))
})

context("test-stat-cor-rmse")

# stat_cor() exposes RMSE/RMSD (root mean square deviation between x and y) as a
# computed variable `rmse` and a formatted label `rmse.label` (#458). Purely
# additive: the default label is unchanged.

.cor_label <- function(p) {
  b <- ggplot2::ggplot_build(p)
  d <- b$data[[which(vapply(b$data, function(x) "label" %in% names(x), logical(1)))[1]]]
  as.character(d$label)[1]
}

set.seed(1)
.df <- data.frame(x = rnorm(20))
.df$y <- .df$x * 0.4 + rnorm(20)
.sp <- ggscatter(.df, "x", "y")

.expected_rmse <- function(x, y, r.digits = 2) {
  ok <- stats::complete.cases(x, y)
  signif(sqrt(mean((x[ok] - y[ok])^2)), r.digits)
}

test_that("rmse.label reports the RMSE between x and y (text output)", {
  expected <- .expected_rmse(.df$x, .df$y)
  lab <- .cor_label(
    .sp + stat_cor(aes(label = after_stat(rmse.label)), output.type = "text")
  )
  expect_equal(lab, paste0("RMSE = ", expected))
})

test_that("rmse.label is a valid plotmath expression by default", {
  expected <- .expected_rmse(.df$x, .df$y)
  lab <- .cor_label(.sp + stat_cor(aes(label = after_stat(rmse.label))))
  expect_equal(lab, paste0("italic(RMSE)~`=`~", expected))
  # must parse as plotmath (draw-time safety, not just build)
  expect_error(parse(text = lab), NA)
})

test_that("rmse.label honors r.accuracy for decimal places", {
  ok <- stats::complete.cases(.df$x, .df$y)
  raw <- sqrt(mean((.df$x[ok] - .df$y[ok])^2))
  lab <- .cor_label(
    .sp + stat_cor(aes(label = after_stat(rmse.label)),
      output.type = "text", r.accuracy = 0.001
    )
  )
  expect_equal(lab, paste0("RMSE = ", formatC(raw, digits = 3, format = "f")))
})

test_that("the default stat_cor label is unchanged (no regression, #458)", {
  lab <- .cor_label(.sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01))
  expect_equal(lab, "italic(R)~`=`~0.20*`,`~italic(p)~`=`~0.392")
  expect_false(grepl("RMSE", lab))
})

test_that("rmse is computed per group", {
  d <- .df
  d$g <- rep(c("a", "b"), each = 10)
  sp <- ggscatter(d, "x", "y", color = "g")
  b <- ggplot2::ggplot_build(sp + stat_cor(aes(color = g, label = after_stat(rmse.label)), output.type = "text"))
  dd <- b$data[[which(vapply(b$data, function(x) "label" %in% names(x), logical(1)))[1]]]
  labs <- as.character(dd$label)
  exp_a <- .expected_rmse(d$x[d$g == "a"], d$y[d$g == "a"])
  exp_b <- .expected_rmse(d$x[d$g == "b"], d$y[d$g == "b"])
  expect_setequal(labs, c(paste0("RMSE = ", exp_a), paste0("RMSE = ", exp_b)))
})

context("test-stat_cor")

.cor_label <- function(p) {
  b <- ggplot2::ggplot_build(p)
  d <- b$data[[which(vapply(b$data, function(x) "label" %in% names(x), logical(1)))[1]]]
  as.character(d$label)[1]
}

set.seed(1)
.df <- data.frame(x = rnorm(20))
.df$y <- .df$x * 0.4 + rnorm(20)
.sp <- ggscatter(.df, "x", "y")

test_that("stat_cor default label is unchanged (no regression, #540/#541)", {
  lab <- .cor_label(.sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01))
  expect_equal(lab, "italic(R)~`=`~0.20*`,`~italic(p)~`=`~0.392")
})

test_that("stat_cor r.leading.zero = FALSE drops the coefficient leading zero (#540)", {
  # For expression output the stripped value is QUOTED so plotmath renders it
  # literally (a bare .20 would be re-normalized to 0.20 at draw time). The
  # quoted form is what actually renders as ".20"/"-.87".
  lab <- .cor_label(.sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01, r.leading.zero = FALSE))
  expect_true(grepl('italic(R)~`=`~".20"', lab, fixed = TRUE))
  # negative coefficient keeps the minus sign outside the quotes: -0.87 -> -".87"
  lab.neg <- .cor_label(
    ggscatter(mtcars, "wt", "mpg") + stat_cor(r.accuracy = 0.01, r.leading.zero = FALSE)
  )
  expect_true(grepl('`=`~-".87"', lab.neg, fixed = TRUE))
})

test_that("stat_cor p.coef.name = 'P' uses an uppercase p-value symbol (#541)", {
  lab <- .cor_label(.sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01, p.coef.name = "P"))
  expect_true(grepl("italic(P)~`=`~", lab, fixed = TRUE))
  expect_false(grepl("italic(p)", lab, fixed = TRUE))
})

test_that("stat_cor APA style: both leading-zero drops + uppercase P together (#540/#541)", {
  lab <- .cor_label(.sp + stat_cor(
    p.accuracy = 0.001, r.accuracy = 0.01,
    r.leading.zero = FALSE, p.leading.zero = FALSE, p.coef.name = "P"
  ))
  # Both stripped values are quoted (render as .20 / .392 respectively).
  expect_equal(lab, 'italic(R)~`=`~".20"*`,`~italic(P)~`=`~".392"')
})

test_that("stat_cor leading-zero drop survives plotmath rendering (#540)", {
  # Guard against the plotmath re-normalization bug: the stripped value must be
  # quoted in the expression so it renders literally. A bare `.20` would draw
  # as `0.20`. Rendering the plot must not error.
  p <- .sp + stat_cor(r.accuracy = 0.01, r.leading.zero = FALSE, label.x = -1)
  lab <- .cor_label(p)
  expect_true(grepl('"', lab, fixed = TRUE))                   # value is quoted
  # the R coefficient itself is not a bare 0.-prefixed number
  expect_false(grepl('italic(R)~`=`~0.', lab, fixed = TRUE))
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

test_that("stat_cor new label options work with text output (#540/#541)", {
  lab <- .cor_label(.sp + stat_cor(
    p.accuracy = 0.001, r.accuracy = 0.01, output.type = "text",
    r.leading.zero = FALSE, p.coef.name = "P"
  ))
  expect_true(grepl("R = .20", lab, fixed = TRUE))
  expect_true(grepl("P = ", lab, fixed = TRUE))
})

# label.y.step: per-group vertical spacing / facet alignment (#248) -----------

.cor_vjust <- function(p) {
  b <- ggplot2::ggplot_build(p)
  d <- b$data[[which(vapply(b$data, function(x) "label" %in% names(x), logical(1)))[1]]]
  d$vjust
}

.facet_df <- transform(mtcars, cyl = factor(cyl))

test_that("stat_cor label.y.step default keeps the per-group vjust shift (#248)", {
  p <- ggplot2::ggplot(.facet_df, ggplot2::aes(mpg, hp)) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~cyl, scales = "free_y") +
    stat_cor(ggplot2::aes(color = cyl))
  # historical behavior: vjust = 1.4 * group.id -> 1.4, 2.8, 4.2 (the "stairs")
  expect_equal(sort(.cor_vjust(p)), c(1.4, 2.8, 4.2), tolerance = 1e-6)
})

test_that("stat_cor default equals explicitly passing label.y.step = 1.4 (byte-identical)", {
  mk <- function(step) {
    g <- ggplot2::ggplot(.facet_df, ggplot2::aes(mpg, hp)) +
      ggplot2::geom_point() +
      ggplot2::facet_wrap(~cyl, scales = "free_y")
    if (is.null(step)) g + stat_cor(ggplot2::aes(color = cyl))
    else g + stat_cor(ggplot2::aes(color = cyl), label.y.step = step)
  }
  expect_equal(sort(.cor_vjust(mk(NULL))), sort(.cor_vjust(mk(1.4))), tolerance = 1e-9)
})

test_that("stat_cor label.y.step = 0 aligns labels across facets (#248)", {
  p <- ggplot2::ggplot(.facet_df, ggplot2::aes(mpg, hp)) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~cyl, scales = "free_y") +
    stat_cor(ggplot2::aes(color = cyl), label.y.step = 0)
  vj <- .cor_vjust(p)
  expect_equal(vj, rep(0, length(vj)), tolerance = 1e-9)
})

test_that("stat_regline_equation gains the same label.y.step control (#248)", {
  base <- ggplot2::ggplot(.facet_df, ggplot2::aes(mpg, hp)) +
    ggplot2::geom_point() +
    ggplot2::facet_wrap(~cyl, scales = "free_y")
  default <- .cor_vjust(base + stat_regline_equation(ggplot2::aes(color = cyl)))
  aligned <- .cor_vjust(base + stat_regline_equation(ggplot2::aes(color = cyl), label.y.step = 0))
  expect_equal(sort(default), c(1.4, 2.8, 4.2), tolerance = 1e-6)
  expect_equal(aligned, rep(0, length(aligned)), tolerance = 1e-9)
})

test_that("label.y.step default is byte-identical for numeric and center npc branches (#248)", {
  # These branches use the recentering term (label.y.step/2 * length(group.id)),
  # which must reduce to the historical 0.7 at the default 1.4.
  mk <- function(npc, step) {
    g <- ggplot2::ggplot(.facet_df, ggplot2::aes(mpg, hp)) +
      ggplot2::geom_point() +
      ggplot2::facet_wrap(~cyl, scales = "free_y")
    if (is.null(step)) g + stat_cor(ggplot2::aes(color = cyl), label.y.npc = npc)
    else g + stat_cor(ggplot2::aes(color = cyl), label.y.npc = npc, label.y.step = step)
  }
  for (npc in list(0.5, "center", "bottom")) {
    expect_equal(sort(.cor_vjust(mk(npc, NULL))), sort(.cor_vjust(mk(npc, 1.4))),
                 tolerance = 1e-9, info = paste("npc =", npc))
    # step = 0 collapses every branch to a single aligned value
    expect_equal(.cor_vjust(mk(npc, 0)), rep(0, length(.cor_vjust(mk(npc, 0)))),
                 tolerance = 1e-9, info = paste("npc =", npc))
  }
})

#' Change ggplot Panel Background Color
#'
#' @description Change ggplot panel background color.
#' @param color background color.
#'
#' @seealso \link{border}().
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' p <- ggboxplot(df, x = "dose", y = "len")
#' p
#'
#' # Change panel background color
#' p +
#'   bgcolor("#BFD5E3") +
#'   border("#BFD5E3")
#' @export
bgcolor <- function(color) {
  theme(panel.background = element_rect(fill = color))
}

#' @include utilities.R
NULL
#' Add Summary Statistics onto a ggplot.
#' @description add summary statistics onto a ggplot.
#' @param p a ggplot on which you want to add summary statistics.
#' @param fun a function that is given the complete data and should return a data
#'  frame with variables ymin, y, and ymax. Allowed values are one of: "mean",
#'  "mean_se", "mean_sd", "mean_ci", "mean_range", "median", "median_iqr",
#'  "median_hilow", "median_q1q3", "median_mad", "median_range".
#' @param error.plot plot type used to visualize error. Allowed values are one of
#'  \code{c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
#'  "lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
#'  "lower_linerange")}. Default value is "pointrange".
#' @param color point or outline color.
#' @param fill fill color. Used only when \code{error.plot = "crossbar"}.
#' @param group grouping variable. Allowed values are 1 (for one group) or a
#'  character vector specifying the name of the grouping variable. Used only for
#'  adding statistical summary per group.
#' @param width numeric value between 0 and 1 specifying bar or box width.
#'  Example width = 0.8. Used only when \code{error.plot} is one of
#'  c("crossbar", "errorbar").
#' @param shape point shape. Allowed values can be displayed using the function
#'  \code{\link{show_point_shapes}()}.
#' @param size numeric value in [0-1] specifying point and line size.
#' @param linewidth numeric value in [0-1] specifying line width.
#' @param linetype line type.
#' @param show.legend logical. Should this layer be included in the legends? NA,
#'  the default, includes if any aesthetics are mapped. \code{FALSE} never includes,
#'  and TRUE always includes. It can also be a named logical vector to finely
#'  select the aesthetics to display.
#' @param data a \code{data.frame} to be displayed. If \code{NULL}, the default,
#'  the data is inherited from the plot data as specified in the call to
#'  \link[ggplot2]{ggplot}.
#' @param position position adjustment, either as a string, or the result of a
#'  call to a position adjustment function. Used to adjust position for multiple
#'  groups.
#' @param x a numeric vector.
#' @param ci the percent range of the confidence interval (default is 0.95).
#' @param error.limit allowed values are one of ("both", "lower", "upper",
#'  "none") specifying whether to plot the lower and/or the upper limits of
#'  error interval.
#' @examples
#'
#' # Basic violin plot
#' p <- ggviolin(ToothGrowth, x = "dose", y = "len", add = "none")
#' p
#'
#' # Add mean_sd
#' add_summary(p, "mean_sd")
#'
#' @describeIn add_summary add summary statistics onto a ggplot.
#' @export
add_summary <- function(p, fun = "mean_se", error.plot = "pointrange",
                        color = "black", fill = "white", group = 1,
                        width = NULL, shape = 19, size = 1, linewidth = size, linetype = 1,
                        show.legend = NA, ci = 0.95,
                        data = NULL, position = position_dodge(0.8)) {
  if (is.null(data)) data <- p$data
  if (fun == "mean_se") {
    fun <- "mean_se_"
  } else if (fun == "median_hilow") fun <- "median_hilow_"

  allowed.fun <- .summary_functions()
  if (!(fun %in% allowed.fun)) {
    stop(
      "Don't support ", fun, ". Possibilities for the argument fun are: ",
      .collapse(allowed.fun, sep = ", ")
    )
  }

  line_geoms <- c(
    "errorbar", "linerange", "upper_errorbar", "lower_errorbar",
    "upper_linerange", "lower_linerange", "crossbar"
  )

  allowed.error.plot <- c(
    "pointrange", "linerange", "crossbar", "errorbar",
    "upper_errorbar", "lower_errorbar", "upper_pointrange", "lower_pointrange",
    "upper_linerange", "lower_linerange"
  )

  if (!(error.plot %in% allowed.error.plot)) {
    stop(
      "Don't support ", error.plot, ". Possibilities for the argument error.plot are: ",
      .collapse(allowed.error.plot, sep = ", ")
    )
  }


  if (missing(width)) width <- 0.8

  .map <- .mapping(p)
  if (missing(color) & !is.null(.map$colour)) {
    color <- .map$colour
  }
  if (missing(fill) & !is.null(.map$fill)) {
    fill <- .map$fill
  }

  # Error limits
  # ::::::::::::::::::::::::::::::::::::::::::::::::::
  . <- NULL
  error.limit <- strsplit(error.plot, "_") %>%
    unlist() %>%
    .[1]
  if (!(error.limit %in% c("upper", "lower"))) {
    error.limit <- "both"
  }
  if (fun %in% c("mean", "median")) error.limit <- "none"

  # Defining plot geometry
  # ::::::::::::::::::::::::::::::::::::::::::::::::::
  geom <- error.plot
  if (error.plot %in% c("pointrange", "lower_pointrange", "upper_pointrange")) {
    geom <- "pointrange"
  } else if (error.plot %in% c("linerange", "lower_linerange", "upper_linerange")) {
    geom <- "linerange"
  } else if (error.plot %in% c("errorbar", "lower_errorbar", "upper_errorbar")) {
    geom <- "errorbar"
  }

  fun.data <- fun.y <- fun.ymin <- fun.ymax <- NULL
  if (fun %in% c("mean", "median")) {
    fun.y <- fun.ymin <- fun.ymax <- fun
  } else {
    fun.data <- fun
  }


  # General option
  # ::::::::::::::::::::::::::::::::::::::::::::::::::
  opts <- list(
    geomfunc = "stat_summary", fun.data = fun.data, fun = fun.y,
    fun.min = fun.ymin, fun.max = fun.ymax,
    color = color, geom = geom, size = size, linewidth = linewidth, linetype = linetype,
    show.legend = show.legend, data = data, position = position,
    fun.args = list(error.limit = error.limit), group = group
  )
  if (error.plot %in% line_geoms) {
    opts <- opts %>% .remove_item("size")
  }
  if (fun %in% c("mean_ci", "median_hilow_")) {
    opts$fun.args$ci <- ci
  }

  # Specific option
  # ::::::::::::::::::::::::::::::::::::::::::::::::::
  if (geom == "crossbar") {
    opts <- opts %>%
      .add_item(fill = fill, width = width)
  } else if (geom == "errorbar") {
    if (missing(width)) {
      opts$width <- 0.1
    } else {
      opts$width <- width
    }
  }

  opts %>% .update_plot(p)
}


#' @describeIn add_summary returns the \code{mean} and the error limits defined by the
#'   \code{standard error}. We used the name \code{mean_se_}() to avoid masking \code{\link[ggplot2]{mean_se}}().
#' @export
mean_se_ <- function(x, error.limit = "both") {
  length <- base::sum(!is.na(x))
  sd <- stats::sd(x, na.rm = TRUE)
  se <- sd / sqrt(length)
  .mean <- base::mean(x, na.rm = TRUE)
  data.frame(
    y =  .mean,
    ymin = .mean - se,
    ymax = .mean + se
  ) %>% .format_error(error.limit)
}

#' @describeIn add_summary returns the \code{mean} and the error limits defined by the
#'   \code{standard deviation}.
#' @export
mean_sd <- function(x, error.limit = "both") {
  sd <- stats::sd(x, na.rm = TRUE)
  .mean <- base::mean(x, na.rm = TRUE)
  data.frame(
    y =  .mean,
    ymin = .mean - sd,
    ymax = .mean + sd
  ) %>% .format_error(error.limit)
}


#' @describeIn add_summary returns the \code{mean} and the error limits defined by the
#'   \code{confidence interval}.
#' @export
mean_ci <- function(x, ci = 0.95, error.limit = "both") {
  length <- base::sum(!is.na(x))
  sd <- stats::sd(x, na.rm = TRUE)
  se <- sd / sqrt(length)
  .mean <- base::mean(x, na.rm = TRUE)
  df <- length - 1
  ci <- if (is.finite(df) && df > 0 && is.finite(se)) stats::qt(ci / 2 + .5, df) * se else NA_real_
  data.frame(
    y =  .mean,
    ymin = .mean - ci,
    ymax = .mean + ci
  ) %>% .format_error(error.limit)
}


#' @describeIn add_summary returns the \code{mean} and the error limits defined by the
#'   \code{range = max - min}.
#' @export
mean_range <- function(x, error.limit = "both") {
  .mean <- base::mean(x, na.rm = TRUE)
  .min <- base::min(x, na.rm = TRUE)
  .max <- base::max(x, na.rm = TRUE)
  .range <- .max - .min
  data.frame(
    y =  .mean,
    ymin = .mean - .range,
    ymax = .mean + .range
  ) %>% .format_error(error.limit)
}


#' @describeIn add_summary returns the \code{median} and the error limits
#'   defined by the \code{interquartile range}.
#' @export
median_iqr <- function(x, error.limit = "both") {
  .median <- stats::median(x, na.rm = TRUE)
  .iqr <- stats::IQR(x, na.rm = TRUE)
  data.frame(
    y =  .median,
    ymin = .median - .iqr,
    ymax = .median + .iqr
  ) %>% .format_error(error.limit)
}

#' @describeIn add_summary computes the sample median and a selected pair of
#'   outer quantiles having equal tail areas. This function is a reformatted
#'   version of \code{Hmisc::smedian.hilow()}. The confidence limits are computed
#'   as follows: \code{lower.limits = (1-ci)/2} percentiles; \code{upper.limits =
#'   (1+ci)/2} percentiles. By default (\code{ci = 0.95}), the 2.5th and the
#'   97.5th percentiles are used as the lower and the upper confidence limits,
#'   respectively. If you want to use the 25th and the 75th percentiles as the
#'   confidence limits, then specify \code{ci = 0.5} or use the function
#'   \code{median_q1q3()}.
#' @export
median_hilow_ <- function(x, ci = 0.95, error.limit = "both") {
  quant <- stats::quantile(
    x,
    probs = c(0.5, (1 - ci) / 2, (1 + ci) / 2),
    na.rm = TRUE
  )
  names(quant) <- c("median", "lower", "upper")
  data.frame(
    y = quant["median"],
    ymin = quant["lower"],
    ymax = quant["upper"]
  ) %>%
    .format_error(error.limit)
}

#' @describeIn add_summary computes the sample median and, the 25th and 75th
#'   percentiles. Wrapper around the function \code{median_hilow_()} using
#'   \code{ci = 0.5}.
#' @export
median_q1q3 <- function(x, error.limit = "both") {
  median_hilow_(x, ci = 0.5, error.limit = error.limit)
}


#' @describeIn add_summary returns the \code{median} and the error limits
#'   defined by the \code{median absolute deviation}.
#' @export
median_mad <- function(x, error.limit = "both") {
  .median <- stats::median(x, na.rm = TRUE)
  .mad <- stats::mad(x, na.rm = TRUE)
  data.frame(
    y =  .median,
    ymin = .median - .mad,
    ymax = .median + .mad
  ) %>% .format_error(error.limit)
}

#' @describeIn add_summary returns the \code{median} and the error limits
#'   defined by the \code{range = max - min}.
#' @export
median_range <- function(x, error.limit = "both") {
  .median <- stats::median(x, na.rm = TRUE)
  .min <- base::min(x, na.rm = TRUE)
  .max <- base::max(x, na.rm = TRUE)
  .range <- .max - .min
  data.frame(
    y =  .median,
    ymin = .median - .range,
    ymax = .median + .range
  ) %>% .format_error(error.limit)
}


# Format error
.format_error <- function(d, error.limit = "both") {
  if (error.limit == "upper") {
    d$ymin <- d$y
  } else if (error.limit == "lower") {
    d$ymax <- d$y
  } else if (error.limit == "none") d$ymin <- d$ymax <- d$y
  d
}

#' @include utilities.R
NULL
#' Create Aes Mapping from a List
#'
#' @description Create aes mapping to make programming easy with ggplot2.
#' @param .list a list of aesthetic arguments; for example .list = list(x = "dose", y = "len", color = "dose").
#' @param parse logical. If TRUE, parse the input as an expression.
#'
#' @examples
#' # Simple aes creation
#' create_aes(list(x = "Sepal.Length", y = "Petal.Length"))
#'
#' # Parse an expression
#' x <- "log2(Sepal.Length)"
#' y <- "log2(Petal.Length)"
#' create_aes(list(x = x, y = y), parse = TRUE)
#'
#' # Create a ggplot
#' mapping <- create_aes(list(x = x, y = y), parse = TRUE)
#' ggplot(iris, mapping) +
#'   geom_point()
#'
#' @export
create_aes <- function(.list, parse = TRUE) {
  if (missing(parse)) {
    parse <- base::getOption("ggpubr.parse_aes", default = TRUE)
  }
  if (parse) {
    return(create_aes.parse(.list))
  } else {
    return(create_aes.name(.list))
  }
}


# Ref:  https://rpubs.com/hadley/97970
# Parse as name. If x_var is "a + b", as.name() will turn it into a variable called `a + b`.
create_aes.name <- function(.list) {
  .list <- .list %>% purrr::map(function(x) to_name(x))
  do.call(ggplot2::aes, .list)
}

# Parse an expression. If x_var is "a + b", parse() will turn it into the function call a + b
create_aes.parse <- function(.list) {
  .list <- .list %>% purrr::map(function(x) parse_expression(x))
  do.call(ggplot2::aes, .list)
}

parse_expression <- function(x) {
  if (is_parsable_aes(x)) {
    # if contains space, just add backsticks using as.name()
    if (contains_space(x)) {
      if (!is_math_string(x)) {
        return(as.name(x))
      }
    }
    x <- parse(text = x)[[1]]
  }
  x
}

to_name <- function(x) {
  if (is_parsable_aes(x)) {
    x <- as.name(x)
  }
  x
}

# Check if parsable aesthetic
is_parsable_aes <- function(x) {
  is.character(x) & (!is_numeric_char(x)) & (length(x) == 1)
}

# Check if x is a numeric string "1", "2"
# return TRUE for "1", "2", etc
is_numeric_char <- function(x) {
  if (is.character(x)) {
    res <- grepl("^[[:digit:]]+$", x)
  } else {
    res <- FALSE
  }
  res
}


# Fortify variable name----------------------------
fortify_variable_name <- function(x) {
  if (contains_space(x)) {
    if (!is_math_string(x)) {}
  }
}

# Check if string contains space
contains_space <- function(x) {
  grepl("\\s", x)
}

extract_text_inside_parenthesis <- function(x) {
  regmatches(x, gregexpr("(?<=\\().*?(?=\\))", x, perl = TRUE))[[1]]
}
# Check if text contains mathematical operators
is_math_string <- function(x) {
  # operators <- unlist(lapply( c("Arith","Compare","Math"), methods::getGroupMembers ))
  operators <- c("+", "-", "*", "^", "%%", "%/%", "/", "==", ">", "<", "!=", "<=", ">=")
  contains_math_operators <- unlist(lapply(operators, grepl, x, fixed = TRUE))
  contains_parentheses <- grepl(pattern = "\\(.*\\)", x)
  any(c(contains_math_operators, contains_parentheses))
}

context("test-create_aes")

test_that("parse_expression works for a valid variable name", {
  observed <- parse_expression("a")
  expected <- parse(text = "a")[[1]]
  expect_equal(observed, expected)
})


test_that("parse_expression works for a variable name containing space", {
  observed <- parse_expression("a b")
  expected <- as.name("a b")
  expect_equal(observed, expected)
})

test_that("parse_expression works for a mathetical expression", {
  observed <- parse_expression("log2( a + b )")
  expected <- parse(text = "log2( a + b )")[[1]]
  expect_equal(observed, expected)
})

# Extract group name from mapping, aes
aes_get_group <- function(mapping) {
  group <- NULL
  if (!is.null(mapping)) {
    if (!is.null(mapping$group)) group <- rlang::as_label(mapping$group)
  }
  group
}

#' @include utilities.R
NULL
#' Overlay Normal Density Plot
#'
#' @description Overlay normal density plot (with the same mean and SD) to the
#'   density distribution of 'x'. This is useful for visually inspecting the
#'   degree of deviance from normality.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @param ... other arguments to pass to \code{\link[ggplot2:geom_path]{geom_line}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning.
#'   If TRUE silently removes missing values.
#' @seealso \code{\link{ggdensity}}
#' @examples
#' # Simpledensity plot
#' data("mtcars")
#' ggdensity(mtcars, x = "mpg", fill = "red") +
#'   scale_x_continuous(limits = c(-1, 50)) +
#'   stat_overlay_normal_density(color = "red", linetype = "dashed")
#'
#' # Color by groups
#' data(iris)
#' ggdensity(iris, "Sepal.Length", color = "Species") +
#'   stat_overlay_normal_density(aes(color = Species), linetype = "dashed")
#'
#'
#' # Facet
#' ggdensity(iris, "Sepal.Length", facet.by = "Species") +
#'   stat_overlay_normal_density(color = "red", linetype = "dashed")
#'
#' @export
stat_overlay_normal_density <- function(mapping = NULL, data = NULL, geom = "line",
                                        position = "identity", na.rm = FALSE, show.legend = NA,
                                        inherit.aes = TRUE, ...) {
  if (is.null(mapping)) {
    mapping <- ggplot2::aes(y = NULL)
  } else {
    mapping["y"] <- list(NULL)
  }
  layer(
    stat = StatOverlayNormalDensity, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, ...)
  )
}


StatOverlayNormalDensity <- ggproto("StatOverlayNormalDensity", Stat,
  required_aes = c("x"),
  compute_group = function(data, scales) {
    x <- data$x
    .mean <- mean(x, na.rm = TRUE)
    .sd <- stats::sd(x, na.rm = TRUE)
    probability.points <- stats::ppoints(length(x[!(is.na(x))]))
    res.density <- stats::density(stats::qnorm(probability.points, .mean, .sd))
    res.density <- data.frame(x = res.density$x, y = res.density$y)
    res.density
  }
)

#' @include utilities.R ggpar.R
NULL
#' Dot plot
#' @description Create a dot plot.
#' @inheritParams ggboxplot
#' @param binwidth numeric value specifying bin width. use value between 0 and 1
#'   when you have a strong dense dotplot. For example binwidth = 0.2.
#' @param ... other arguments to be passed to
#'   \code{\link[ggplot2]{geom_dotplot}}, \code{\link{ggpar}} and
#'   \code{\link{facet}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}, \code{\link{ggviolin}}, \code{\link{ggboxplot}}
#'  and \code{\link{ggstripchart}}.
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot with summary statistics : mean_sd
#' # +++++++++++++++++++++++++++
#' ggdotplot(df,
#'   x = "dose", y = "len",
#'   add = "mean_sd"
#' )
#'
#' # Change error.plot to "crossbar"
#' ggdotplot(df,
#'   x = "dose", y = "len",
#'   add = "mean_sd", add.params = list(width = 0.5),
#'   error.plot = "crossbar"
#' )
#'
#'
#' # Add box plot
#' ggdotplot(df,
#'   x = "dose", y = "len",
#'   add = "boxplot"
#' )
#'
#' # Add violin + mean_sd
#' ggdotplot(df,
#'   x = "dose", y = "len",
#'   add = c("violin", "mean_sd")
#' )
#'
#'
#' # Change colors
#' # +++++++++++++++++++++++++++
#' # Change fill and outline colors by groups: dose
#' # Use custom color palette
#' ggdotplot(df, "dose", "len",
#'   add = "boxplot",
#'   color = "dose", fill = "dose",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#'
#' # Plot with multiple groups
#' # +++++++++++++++++++++
#' # Change color by a second group : "supp"
#' ggdotplot(df, "dose", "len",
#'   fill = "supp", color = "supp",
#'   palette = c("#00AFBB", "#E7B800")
#' )
#'
#' @export
ggdotplot <- function(data, x, y, combine = FALSE, merge = FALSE,
                      color = "black", fill = "lightgray", palette = NULL,
                      title = NULL, xlab = NULL, ylab = NULL,
                      facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                      size = NULL, binwidth = NULL,
                      select = NULL, remove = NULL, order = NULL,
                      add = "mean_se",
                      add.params = list(),
                      error.plot = "pointrange",
                      label = NULL, font.label = list(size = 11, color = "black"),
                      label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                      ggtheme = theme_pubr(),
                      ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    size = size, binwidth = binwidth,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle, ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  .opts$fun <- ggdotplot_core
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


ggdotplot_core <- function(data, x, y,
                           color = "black", fill = "lightgray", palette = NULL,
                           title = NULL, xlab = NULL, ylab = NULL,
                           size = NULL, dotsize = size,
                           binwidth = NULL,
                           add = "mean_se",
                           add.params = list(),
                           error.plot = "pointrange",
                           position = position_dodge(0.8),
                           ggtheme = theme_pubr(),
                           ...) {
  if (!is.factor(data[[x]])) data[[x]] <- as.factor(data[[x]])
  . <- NULL

  p <- ggplot(data, create_aes(list(x = x, y = y)))
  if ("none" %in% add) add <- "none"

  if (is.null(add.params$fill)) add.params$fill <- "white"
  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill, ...)
  # plot boxplot | violin | crossbar before jitter
  if (any(c("boxplot", "violin") %in% add)) {
    p <- add.params %>%
      .add_item(p = p, add = intersect(add, c("boxplot", "violin"))) %>%
      do.call(ggadd, .)
  }
  if (error.plot == "crossbar") {
    p <- add.params %>%
      .add_item(
        p = p, error.plot = error.plot,
        add = setdiff(add, c("boxplot", "violin", "jitter"))
      ) %>%
      do.call(ggadd, .)
  }
  # Plot jitter
  p <- p +
    geom_exec(geom_dotplot,
      data = data,
      binaxis = "y", stackdir = "center",
      color = color, fill = fill,
      position = position, stackratio = 1,
      dotsize = dotsize, binwidth = binwidth, ...
    )

  # Add errors
  if (error.plot == "crossbar") {} else {
    p <- add.params %>%
      .add_item(
        p = p, error.plot = error.plot,
        add = setdiff(add, c("boxplot", "violin", "jitter"))
      ) %>%
      do.call(ggadd, .)
  }

  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )

  p
}

#' @include utilities_color.R
NULL
#' Set Color Palette
#'
#' @description \itemize{ \item \code{change_palette(), set_palette()}: Change
#' both color and fill palettes. \item \code{color_palette()}: change color
#' palette only. \item \code{fill_palette()}: change fill palette only.
#'
#' }
#' @inheritParams get_palette
#' @param p a ggplot
#' @param ... other arguments passed to ggplot2 scale_color_xxx() and
#'  scale_fill_xxx() functions.
#'
#' @seealso \link{get_palette}.
#'
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' p <- ggboxplot(df,
#'   x = "dose", y = "len",
#'   color = "dose"
#' )
#' p
#'
#' # Change the color palette
#' set_palette(p, "jco")
#' @name set_palette
#' @rdname set_palette
#' @export
set_palette <- function(p, palette) {
  p + .ggcolor(palette) +
    .ggfill(palette)
}

#' @rdname set_palette
#' @export
change_palette <- function(p, palette) {
  set_palette(p, palette)
}

#' @rdname set_palette
#' @export
color_palette <- function(palette = NULL, ...) {
  brewerpal <- .brewerpal()
  ggscipal <- .ggscipal()

  res <- NULL
  if (is.null(palette)) {
    palette <- ""
  }
  if (length(palette) == 1) {
    if (palette %in% brewerpal) {
      ggplot2::scale_color_brewer(..., palette = palette)
    } else if (palette %in% ggscipal) {
      .scale_color_ggsci(palette = palette)
    } else if (palette == "grey") {
      ggplot2::scale_color_grey(..., start = 0.8, end = 0.2)
    } else if (palette == "hue") {
      ggplot2::scale_color_hue(...)
    } else if (.is_color(palette)) {
      ggplot2::scale_color_manual(..., values = palette)
    }
  } else if (palette[1] != "") {
    ggplot2::scale_color_manual(..., values = palette)
  }
}

#' @rdname set_palette
#' @export
fill_palette <- function(palette = NULL, ...) {
  brewerpal <- .brewerpal()
  ggscipal <- .ggscipal()

  res <- NULL
  if (is.null(palette)) {
    palette <- ""
  }
  if (length(palette) == 1) {
    if (palette %in% brewerpal) {
      ggplot2::scale_fill_brewer(..., palette = palette)
    } else if (palette %in% ggscipal) {
      .scale_fill_ggsci(palette = palette)
    } else if (palette == "grey") {
      ggplot2::scale_fill_grey(..., start = 0.8, end = 0.2)
    } else if (palette == "hue") {
      ggplot2::scale_fill_hue(...)
    } else if (.is_color(palette)) {
      ggplot2::scale_fill_manual(..., values = palette)
    }
  } else if (palette[1] != "") {
    ggplot2::scale_fill_manual(..., values = palette)
  }
}

#' @include utilities.R
NULL
#' Add Stars to a Scatter Plot
#' @description Create a star plot by drawing segments from group centroid to each points.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @param ... other arguments to pass to \code{\link[ggplot2]{geom_segment}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning.
#'   If TRUE silently removes missing values.
#' @seealso \code{\link{ggscatter}}
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#'
#' # Scatter plot with ellipses and group mean points
#' ggscatter(df, x = "wt", y = "mpg",
#'    color = "cyl", shape = "cyl",
#'    mean.point = TRUE, ellipse = TRUE)+
#'  stat_stars(aes(color = cyl))
#'
#' @export
stat_stars <- function(mapping = NULL, data = NULL, geom = "segment",
                    position = "identity",  na.rm = FALSE, show.legend = NA,
                    inherit.aes = TRUE, ...) {
  layer(
    stat = StatStars, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, ...)
  )
}


StatStars<- ggproto("StatStars", Stat,
                  required_aes = c("x", "y"),
                  compute_group = function(data, scales) {
                    x.mean <- mean(data$x, na.rm = TRUE)
                    y.mean <- mean(data$y, na.rm = TRUE)
                    res <- data %>% mutate(x = x.mean, y = y.mean)
                    res$xend <- data$x
                    res$yend <- data$y
                    res
                  }
)

#' @include utilities.R ggpar.R
NULL
#' Histogram plot
#' @description Create a histogram plot.
#' @inheritParams ggboxplot
#' @param x variable to be drawn.
#' @param y one of "..density.." or "..count..".
#' @param weight a variable name available in the input data for creating a weighted histogram.
#' @param color,fill histogram line color and fill color.
#' @param linetype line type. See \code{\link{show_line_types}}.
#' @param alpha numeric value specifying fill color transparency. Value should
#'   be in [0, 1], where 0 is full transparency and 1 is no transparency.
#' @param bins Number of bins. Defaults to 30.
#' @param binwidth numeric value specifying bin width. use value between 0 and 1
#'   when you have a strong dense dotplot. For example binwidth = 0.2.
#' @param add allowed values are one of "mean" or "median" (for adding mean or
#'   median line, respectively).
#' @param add.params parameters (color, size, linetype) for the argument 'add';
#'   e.g.: add.params = list(color = "red").
#' @param rug logical value. If TRUE, add marginal rug.
#' @param add_density logical value. If TRUE, add density curves.
#' @param position Position adjustment, either as a string, or the result of a
#'   call to a position adjustment function. Allowed values include "identity",
#'   "stack", "dodge".
#' @param ... other arguments to be passed to
#'   \code{\link[ggplot2]{geom_histogram}} and \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggdensity}} and \code{\link{ggpar}}
#'
#' @examples
#' # Create some data format
#' set.seed(1234)
#' wdata = data.frame(
#'    sex = factor(rep(c("F", "M"), each=200)),
#'    weight = c(rnorm(200, 55), rnorm(200, 58)))
#'
#' head(wdata, 4)
#'
#' # Basic density plot
#' # Add mean line and marginal rug
#' gghistogram(wdata, x = "weight", fill = "lightgray",
#'    add = "mean", rug = TRUE)
#'
#' # Change outline colors by groups ("sex")
#' # Use custom color palette
#' gghistogram(wdata, x = "weight",
#'    add = "mean", rug = TRUE,
#'    color = "sex", palette = c("#00AFBB", "#E7B800"))
#'
#' # Change outline and fill colors by groups ("sex")
#' # Use custom color palette
#' gghistogram(wdata, x = "weight",
#'    add = "mean", rug = TRUE,
#'    color = "sex", fill = "sex",
#'    palette = c("#00AFBB", "#E7B800"))
#'
#'
#'
#' # Combine histogram and density plots
#' gghistogram(wdata, x = "weight",
#'    add = "mean", rug = TRUE,
#'    fill = "sex", palette = c("#00AFBB", "#E7B800"),
#'    add_density = TRUE)
#'
#' # Weighted histogram
#' gghistogram(iris, x = "Sepal.Length", weight = "Petal.Length")
#' @export
gghistogram <- function(data, x, y = "..count..", combine = FALSE, merge = FALSE,
                        weight = NULL, color = "black", fill = NA, palette = NULL,
                        size = NULL, linetype = "solid", alpha = 0.5,
                        bins = NULL, binwidth = NULL,
                        title = NULL, xlab = NULL, ylab = NULL,
                        facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                        add = c("none", "mean", "median"),
                        add.params = list(linetype = "dashed"),
                        rug = FALSE, add_density = FALSE,
                        label = NULL, font.label = list(size = 11, color = "black"),
                        label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                        position = position_identity(),
                        ggtheme = theme_pubr(),
                        ...)
{

  # Default options
  #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    linetype = linetype, size = size, alpha = alpha, bins = bins, binwidth = binwidth,
    weight = weight, title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    add = add, add.params = add.params, rug = rug, add_density = add_density,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle,
    position = position, ggtheme = ggtheme, ...)
  if(!missing(data)) .opts$data <- data
  if(!missing(x)) .opts$x <- x
  if(!missing(y)) .opts$y <- y

  # User options
  #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for(opt.name in names(.opts)){
    if(is.null(.user.opts[[opt.name]]))
      .opts[[opt.name]] <- NULL
  }

  .opts$fun <- gghistogram_core
  if(missing(ggtheme) & (!is.null(facet.by) | combine))
    .opts$ggtheme <- theme_pubr(border = TRUE)
  if(missing(y)) .opts$y <- y
  if(missing(add.params)) .opts$add.params <- add.params
  p <- do.call(.plotter, .opts)

  if(.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)

}

gghistogram_core <- function(data, x, y = "..count..", weight = NULL,
                      color = "black", fill = NA, palette = NULL,
                      size = NULL, linetype = "solid", alpha = 0.5,
                      bins = NULL, binwidth = NULL,
                      title = NULL, xlab = NULL, ylab = NULL,
                      facet.by = NULL,
                      add = c("none", "mean", "median"),
                      add.params = list(linetype = "dashed"),
                      rug = FALSE, add_density = FALSE,
                      position = position_identity(),
                      ggtheme = theme_classic(),
                      ...)
{

  grouping.vars <- grp <- c(color, fill, linetype, size, alpha, facet.by) %>%
    unique() %>%
    intersect(colnames(data))
  color2 <-  c(color, fill) %>% unique() %>% intersect(colnames(data))
  if(.is_empty(color2)) color2 <- color

  # Check bins
  if(is.null(bins) & is.null(binwidth)){
    bins <- 30
    warning("Using `bins = 30` by default. Pick better value with the argument `bins`.",
            call.= FALSE)
  }

  add <- match.arg(add)
  if(is.null(add.params$color)){
    if(!.is_empty(color2)) add.params$color <- color2
  }
  add.params <- .check_add.params(add, add.params, error.plot = "", data, color, fill, ...)
  if(is.null(add.params$size)) add.params$size <- size
  if(is.null(add.params$linetype)) add.params$linetype <- linetype
  # if(add_density) y <- "..density.."

  p <- ggplot(data, create_aes(list(x = x, y = y)))

  p <- p +
      geom_exec(geom_histogram, data = data,
                 color = color, fill = fill, size = size,
                 linetype = linetype, alpha = alpha, bins = bins, binwidth = binwidth,
                 weight = weight, position = position, ...)

  # Add mean/median
  if(add %in% c("mean", "median")){
    p <- p %>% .add_center_line(add = add, grouping.vars = grouping.vars, color = add.params$color,
                                linetype = add.params$linetype, size = add.params$size)
  }

  # Add marginal rug
  if(rug) {

    grps <- c(color, fill, linetype, size, alpha) %>%
      unique() %>% intersect(colnames(data))
    alpha <- ifelse(.is_empty(grps), 1, alpha)
    .args <- geom_exec(NULL, data = data,
                       color = color2, sides = "b", alpha = alpha)
    mapping <- .args$mapping
    mapping[["y"]] <- 0
    option <- .args$option
    option[["mapping"]] <- create_aes(mapping)
    p <- p + do.call(geom_rug, option)
  }

  # Add density curve
  if(add_density) p <- p + geom_exec(geom_density, data = data,
                                      color = add.params$color,
                                      linetype = linetype, alpha = alpha,
                                      size = add.params$size)


  p <- ggpar(p, palette = palette, ggtheme = ggtheme,
             title = title, xlab = xlab, ylab = ylab,...)
  p
}

context("gghistogram")

test_that("gghistogram works when using after_stat()", {
  p <- gghistogram(iris, x = "Sepal.Length", y = "after_stat(density * width)", bins = 30)
  p_build <- ggplot2::ggplot_build(p)
  df <- p_build$data[[1]]
  observed_count <- df$count
  if (is_pkg_version_sup("ggplot2", "3.5.2")) {
    expected_count <- c(1, 3, 5, 2, 5, 6, 19, 4, 1, 6, 13, 8, 7, 3, 12, 4, 9, 7, 5, 10, 3, 4, 1, 4, 1, 1, 0, 5, 0, 1)
  } else {
    expected_count <- c(4, 1, 4, 2, 11, 10, 9, 4, 7, 7, 6, 8, 7, 9, 6, 4, 9, 12, 2, 8, 3, 5, 1, 3, 1, 1, 1, 4, 0, 1)
  }
  expect_equal(observed_count, expected_count)
})


test_that("gghistogram works when using after_stat() with trailing space inside parentheses", {
  p <- gghistogram(iris, x = "Sepal.Length", y = "after_stat(density  )", bins = 30)
  p_build <- ggplot2::ggplot_build(p)
  df <- p_build$data[[1]]
  observed_count <- df$count
  if (is_pkg_version_sup("ggplot2", "3.5.2")) {
    expected_count <- c(1, 3, 5, 2, 5, 6, 19, 4, 1, 6, 13, 8, 7, 3, 12, 4, 9, 7, 5, 10, 3, 4, 1, 4, 1, 1, 0, 5, 0, 1)
  } else {
    expected_count <- c(4, 1, 4, 2, 11, 10, 9, 4, 7, 7, 6, 8, 7, 9, 6, 4, 9, 12, 2, 8, 3, 5, 1, 3, 1, 1, 1, 4, 0, 1)
  }
  expect_equal(observed_count, expected_count)
})

context("test-ggtheme-null")

# #561: `ggtheme = NULL` should skip applying a ggpubr theme, so the plot keeps
# ggplot2's default theme (or the theme set globally with theme_set()). Passing
# nothing or a real theme must stay byte-identical (no-regression).

# helper: is a *complete* theme attached to the plot?
.has_complete_theme <- function(p) isTRUE(attr(p$theme, "complete"))

test_that("default (ggtheme unset) still applies theme_pubr (#561 no-regression)", {
  p <- ggscatter(mtcars, "wt", "mpg")
  expect_true(.has_complete_theme(p))
  # theme_pubr panel background is white
  expect_equal(ggplot2::calc_element("panel.background", p$theme)$fill, "white")
})

test_that("explicit ggtheme = NULL skips theming across wrappers (#561)", {
  wrappers <- list(
    ggscatter(mtcars, "wt", "mpg", ggtheme = NULL),
    ggboxplot(ToothGrowth, "dose", "len", ggtheme = NULL),
    ggline(ToothGrowth, "dose", "len", ggtheme = NULL),
    gghistogram(ToothGrowth, "len", ggtheme = NULL),
    ggbarplot(ToothGrowth[1:6, ], "supp", "len", ggtheme = NULL)
  )
  for (p in wrappers) {
    # no complete ggpubr theme attached -> ggplot2 default governs at draw time
    expect_false(.has_complete_theme(p))
    # and it still renders without error
    expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)), NA)
  }
})

test_that("explicit ggtheme = theme_bw() is honored and unchanged (#561 no-regression)", {
  p <- ggscatter(mtcars, "wt", "mpg", ggtheme = ggplot2::theme_bw())
  expect_true(.has_complete_theme(p))
  # theme_bw has a black axis-less panel border and grey grid; distinguishable
  # from theme_pubr by the panel.grid.major being drawn (grey), not blank
  expect_false(inherits(ggplot2::calc_element("panel.grid.major", p$theme), "element_blank"))
})

test_that("faceted default still gets the bordered theme_pubr (#561 no-regression)", {
  p <- ggboxplot(ToothGrowth, "dose", "len", facet.by = "supp")
  expect_true(.has_complete_theme(p))
  expect_equal(ggplot2::calc_element("panel.background", p$theme)$fill, "white")
})

test_that("ggtheme = NULL lets a session-global theme govern (#561)", {
  old <- ggplot2::theme_set(ggplot2::theme_minimal())
  on.exit(ggplot2::theme_set(old), add = TRUE)
  # NULL means 'no ggpubr theme' -> the global theme (theme_minimal) is used at
  # draw time; the plot itself carries no competing complete theme.
  p <- ggscatter(mtcars, "wt", "mpg", ggtheme = NULL)
  expect_false(.has_complete_theme(p))
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)), NA)
})

#' @include utilities.R ggpar.R
NULL
#' Density plot
#' @description Create a density plot.
#' @inheritParams ggboxplot
#' @param x variable to be drawn.
#' @param y one of "density" or "count".
#' @param color,fill density line color and fill color.
#' @param linetype line type. See \code{\link{show_line_types}}.
#' @param linewidth numeric value specifying line width. This is the preferred
#'   parameter for ggplot2 3.4.0+. If \code{size} is also specified, \code{linewidth}
#'   takes precedence.
#' @param alpha numeric value specifying fill color transparency. Value should
#'   be in [0, 1], where 0 is full transparency and 1 is no transparency.
#' @param add allowed values are one of "mean" or "median" (for adding mean or
#'   median line, respectively).
#' @param add.params parameters (color, size, linetype) for the argument 'add';
#'   e.g.: add.params = list(color = "red").
#' @param rug logical value. If TRUE, add marginal rug.
#' @param ... other arguments to be passed to
#'   \code{\link[ggplot2]{geom_density}} and \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{gghistogram}} and \code{\link{ggpar}}.
#'
#' @examples
#' # Create some data format
#' set.seed(1234)
#' wdata <- data.frame(
#'   sex = factor(rep(c("F", "M"), each = 200)),
#'   weight = c(rnorm(200, 55), rnorm(200, 58))
#' )
#'
#' head(wdata, 4)
#'
#' # Basic density plot
#' # Add mean line and marginal rug
#' ggdensity(wdata,
#'   x = "weight", fill = "lightgray",
#'   add = "mean", rug = TRUE
#' )
#'
#' # Change outline colors by groups ("sex")
#' # Use custom palette
#' ggdensity(wdata,
#'   x = "weight",
#'   add = "mean", rug = TRUE,
#'   color = "sex", palette = c("#00AFBB", "#E7B800")
#' )
#'
#'
#' # Change outline and fill colors by groups ("sex")
#' # Use custom palette
#' ggdensity(wdata,
#'   x = "weight",
#'   add = "mean", rug = TRUE,
#'   color = "sex", fill = "sex",
#'   palette = c("#00AFBB", "#E7B800")
#' )
#'
#' @export
ggdensity <- function(data, x, y = "density", combine = FALSE, merge = FALSE,
                      color = "black", fill = NA, palette = NULL,
                      size = NULL, linewidth = NULL, linetype = "solid", alpha = 0.5,
                      title = NULL, xlab = NULL, ylab = NULL,
                      facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                      add = c("none", "mean", "median"),
                      add.params = list(linetype = "dashed"),
                      rug = FALSE,
                      label = NULL, font.label = list(size = 11, color = "black"),
                      label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                      ggtheme = theme_pubr(),
                      ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    linetype = linetype, size = size, alpha = alpha,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    add = add, add.params = add.params, rug = rug,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle, ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  # Handle size vs linewidth parameter compatibility
  # size deprecated in ggplot2 v >= 3.4.0 - convert after .user.opts processing
  if (!is.null(.opts$size)) {
    .opts$linewidth <- .opts$size
    .opts$size <- NULL
  }
  if (!is.null(linewidth)) {
    .opts$linewidth <- linewidth
  }

  .opts$fun <- ggdensity_core
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  if (missing(y)) .opts$y <- y
  if (missing(add.params)) .opts$add.params <- add.params
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}

ggdensity_core <- function(data, x, y = "density",
                           color = "black", fill = NA, palette = NULL,
                           size = NULL, linewidth = NULL, linetype = "solid", alpha = 0.5,
                           title = NULL, xlab = NULL, ylab = NULL,
                           add = c("none", "mean", "median"),
                           add.params = list(linetype = "dashed"),
                           rug = FALSE,
                           facet.by = NULL,
                           ggtheme = theme_classic(),
                           ...) {
  # Handle size vs linewidth parameter compatibility
  # size deprecated in ggplot2 v >= 3.4.0
  extra_args <- list(...)
  if ("size" %in% names(extra_args) && is.null(linewidth)) {
    linewidth <- extra_args$size
    extra_args$size <- NULL
  }
  if (!is.null(size) && is.null(linewidth)) {
    linewidth <- size
  }

  grouping.vars <- grp <- c(color, fill, linetype, size, alpha, facet.by) %>%
    unique() %>%
    intersect(colnames(data))

  add <- match.arg(add)
  add.params <- .check_add.params(add, add.params, error.plot = "", data, color, fill, ...)
  if (is.null(add.params$size)) add.params$size <- linewidth
  if (is.null(add.params$linetype)) add.params$linetype <- linetype
  if (y %in% c("..density..", "density")) {
    y <- "after_stat(density)"
  } else if (y %in% c("..count..", "count")) y <- "after_stat(count)"

  p <- ggplot(data, create_aes(list(x = x, y = y)))

  # Build geom_density args without passing size through ...
  geom_args <- c(
    list(
      geomfunc = geom_density, data = data,
      color = color, fill = fill, linewidth = linewidth,
      linetype = linetype, alpha = alpha
    ),
    extra_args
  )
  p <- p + do.call(geom_exec, geom_args)

  # Add mean/median
  if (add %in% c("mean", "median")) {
    p <- p %>% .add_center_line(
      add = add, grouping.vars = grouping.vars, color = add.params$color,
      linetype = add.params$linetype, linewidth = add.params$size
    )
  }

  # Add marginal rug
  if (rug) {
    grps <- c(color, fill, linetype, size, alpha) %>%
      unique() %>%
      intersect(colnames(data))
    alpha <- ifelse(.is_empty(grps), 1, alpha)
    .args <- geom_exec(NULL,
      data = data,
      color = color, sides = "b", alpha = alpha
    )
    mapping <- .args$mapping
    mapping[["y"]] <- 0
    option <- .args$option
    option[["mapping"]] <- create_aes(mapping)
    p <- p + do.call(geom_rug, option)
  }


  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )
  p
}

# Package load and attach hooks

.onAttach <- function(libname, pkgname) {
  # Check for stale lock files and warn user
  lib_path <- .libPaths()[1]
  lock_dir <- file.path(lib_path, paste0("00LOCK-", pkgname))


  if (dir.exists(lock_dir)) {
    packageStartupMessage(
      "Note: A stale lock directory was detected: ", lock_dir, "\n",
      "This may cause installation issues. Run ggpubr::clean_lock_files() to remove it."
    )
  }
}

#' Clean up stale package lock files
#'
#' @description Removes stale lock directories that can prevent package installation.
#'
#' Lock files are created during package installation and should be automatically
#' removed when installation completes. If installation is interrupted (e.g., by
#' closing R or a crash), these lock files may remain and block future installations.
#'
#' @param package Character string specifying which package lock to remove.
#'   Default is "ggpubr". Use "all" to remove all lock files.
#' @param lib Library path to check. Default is the first library in .libPaths().
#' @param ask Logical. If TRUE (default), asks for confirmation before removing.
#'
#' @return Invisibly returns TRUE if files were removed, FALSE otherwise.
#'
#' @examples
#' \dontrun{
#' # Remove ggpubr lock file
#' clean_lock_files()
#'
#' # Remove all lock files
#' clean_lock_files("all")
#'
#' # Remove without confirmation
#' clean_lock_files(ask = FALSE)
#' }
#'
#' @export
clean_lock_files <- function(package = "ggpubr", lib = .libPaths()[1], ask = TRUE) {
  stale_cutoff_mins <- 10

  if (package == "all") {
    # Find all lock directories
    lock_dirs <- list.dirs(lib, recursive = FALSE, full.names = TRUE)
    lock_dirs <- lock_dirs[grepl("^00LOCK-", basename(lock_dirs))]
  } else {
    lock_dirs <- file.path(lib, paste0("00LOCK-", package))
    lock_dirs <- lock_dirs[dir.exists(lock_dirs)]
  }


  if (length(lock_dirs) == 0) {
    message("No lock files found.")
    return(invisible(FALSE))
  }

  # In non-interactive all-lock cleanup, skip very recent locks
  # to reduce risk of removing active package installation locks.
  if (package == "all" && !ask) {
    lock_info <- file.info(lock_dirs)
    lock_age_mins <- as.numeric(difftime(Sys.time(), lock_info$mtime, units = "mins"))
    recent_locks <- is.na(lock_age_mins) | lock_age_mins < stale_cutoff_mins
    if (any(recent_locks)) {
      warning(
        "Skipping lock directories modified within the last ", stale_cutoff_mins,
        " minutes for clean_lock_files(package = 'all', ask = FALSE) ",
        "to avoid removing active installation locks.",
        call. = FALSE
      )
      lock_dirs <- lock_dirs[!recent_locks]
    }
    if (length(lock_dirs) == 0) {
      message("No stale lock files found for non-interactive all-lock cleanup.")
      return(invisible(FALSE))
    }
  }


  message("Found lock directories:")
  for (d in lock_dirs) {
    message("  ", d)
  }

  if (ask) {
    response <- readline("Remove these directories? (y/n): ")
    if (!tolower(response) %in% c("y", "yes")) {
      message("Aborted.")
      return(invisible(FALSE))
    }
  }

  # Remove lock directories
  success <- TRUE
  for (d in lock_dirs) {
    tryCatch(
      {
        unlink(d, recursive = TRUE, force = FALSE)
        if (dir.exists(d)) {
          # unlink doesn't always error on failure, check if still exists
          warning(
            "Failed to remove ", d, ": directory still exists. ",
            "You may need to remove it manually or check permissions."
          )
          success <- FALSE
        } else {
          message("Removed: ", d)
        }
      },
      error = function(e) {
        warning(
          "Failed to remove ", d, ": ", e$message,
          "\nIf this is a stale lock from a stopped install, remove it manually after confirming no package installation is running: 'rm -rf \"", d, "\"'"
        )
        success <<- FALSE
      }
    )
  }

  if (success) {
    message("Lock files cleaned up successfully.")
    message("You can now reinstall the package.")
  }

  invisible(success)
}

context("test-ggbarplot-alpha-errorbar")

# #404: ggbarplot(alpha = <discrete var>, add = "mean_ci", position = position_dodge())
# used to error at draw time ("alpha * 255": non-numeric argument) and, once that was
# worked around, collapsed the summary across the alpha subgroup. The summary now groups
# by the alpha variable too, so the bars are drawn per subgroup with the alpha aesthetic
# preserved, and the error layer dodges by the same subgroup so error bars stay aligned.

suppressPackageStartupMessages({
  library(ggplot2)
  library(dplyr)
})

# small, deterministic 2x2 grouped data
df <- ToothGrowth
df$dose <- factor(df$dose)
df2 <- subset(df, dose %in% c("0.5", "1"))
df2$dose <- droplevels(df2$dose)

test_that("ggbarplot(alpha=var, add='mean_ci', dodge) renders without the alpha*255 error (#404)", {
  p <- ggbarplot(df2, x = "dose", y = "len", fill = "dose", alpha = "supp",
                 add = "mean_ci", position = position_dodge())
  # full render (the crash was draw-time, not build-time)
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)), NA)
})

test_that("alpha subgroup gives per-subgroup bars, aligned error bars, and a real alpha aesthetic (#404)", {
  p <- ggbarplot(df2, x = "dose", y = "len", fill = "dose", alpha = "supp",
                 add = "mean_ci", position = position_dodge())
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  bar <- b$data[[1]]
  err <- b$data[[2]]
  # 2 dose x 2 supp = 4 bars and 4 error bars
  expect_equal(nrow(bar), 4)
  expect_equal(nrow(err), 4)
  # error bars sit exactly on the bar x-positions (aligned, not centered)
  expect_equal(sort(round(bar$x, 6)), sort(round(err$x, 6)))
  # x is genuinely dodged (not all at integer centers)
  expect_false(all(bar$x %in% c(1, 2)))
  # alpha is a mapped aesthetic (two distinct levels), not a static/NA value
  expect_equal(length(unique(round(bar$alpha, 4))), 2)
})

test_that("bar heights equal the per-subgroup means and CIs bracket them (#404)", {
  p <- ggbarplot(df2, x = "dose", y = "len", fill = "dose", alpha = "supp",
                 add = "mean_ci", position = position_dodge())
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  bar <- b$data[[1]][order(b$data[[1]]$x), ]
  err <- b$data[[2]][order(b$data[[2]]$x), ]
  expected <- df2 %>%
    group_by(dose, supp) %>%
    summarise(m = mean(len), .groups = "drop")
  expect_equal(sort(round(bar$y, 6)), sort(round(expected$m, 6)))
  # each CI brackets its bar's mean
  expect_true(all(err$ymin < bar$y & bar$y < err$ymax))
})

test_that("no-regression: ggbarplot summaries without alpha are unchanged (#404)", {
  # A plain grouped summary bar (no alpha) must be untouched by the fix: 2 dose x 2 supp
  # bars, dodged, mean_se error bars centered on each bar as before.
  p <- ggbarplot(df, x = "dose", y = "len", fill = "supp",
                 add = "mean_se", position = position_dodge())
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  expect_equal(nrow(b$data[[1]]), 6)                 # 3 dose x 2 supp
  # bar heights are the per-(dose,supp) means
  expected <- df %>% group_by(dose, supp) %>% summarise(m = mean(len), .groups = "drop")
  expect_equal(sort(round(b$data[[1]]$y, 6)), sort(round(expected$m, 6)))
  # error x aligns with bar x (this always worked for the single fill dimension)
  expect_equal(sort(round(b$data[[1]]$x, 6)), sort(round(b$data[[2]]$x, 6)))
})

test_that("the alpha subgroup also aligns under options(ggpubr.parse_aes = FALSE) (#404)", {
  # The error-layer dodge group is a real, safe-named column, so it works even when
  # aes parsing is disabled (used for column names with special characters).
  old <- options(ggpubr.parse_aes = FALSE)
  on.exit(options(old), add = TRUE)
  p <- ggbarplot(df2, x = "dose", y = "len", fill = "dose", alpha = "supp",
                 add = "mean_ci", position = position_dodge())
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  expect_error(ggplot2::ggplot_gtable(b), NA)
  expect_equal(nrow(b$data[[1]]), 4)
  expect_equal(sort(round(b$data[[1]]$x, 6)), sort(round(b$data[[2]]$x, 6)))
})

test_that("a static numeric alpha is still treated as a constant, not a grouping var (#404)", {
  # alpha = 0.5 is a fixed transparency, not a column -> must NOT trigger subgrouping.
  p <- ggbarplot(df, x = "dose", y = "len", fill = "supp", alpha = 0.5,
                 add = "mean_se", position = position_dodge())
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  expect_equal(nrow(b$data[[1]]), 6)                 # unchanged: 3 dose x 2 supp
  expect_true(all(abs(b$data[[1]]$alpha - 0.5) < 1e-9))
})

test_that("The position of jittered points is reproducible in ggboxplot", {
  p <- ggboxplot(PlantGrowth, x = "group", y = "weight", add = "jitter")
  jitter_data <- layer_data(p, 2) %>%
    dplyr::select(x, y) %>%
    dplyr::mutate(x = as.numeric(round(x, 7)), y = as.numeric(round(y, 7)))
  expected_data <- tibble::tribble(
    ~x, ~y,
    0.915031, 4.1737042,
    1.1153221, 5.5832184,
    0.9635908, 5.1815256,
    1.153207, 6.1123637,
    1.1761869, 4.4961969,
    0.8182226, 4.6098224,
    1.0112422, 5.1720677,
    1.1569676, 4.5277313,
    1.020574, 5.3285454,
    0.9826459, 5.137853,
    2.1827333, 4.8071424,
    1.9813337, 4.1693164,
    2.0710283, 4.4093098,
    2.0290534, 3.5889508,
    1.8411699, 5.8672196,
    2.15993, 3.8271104,
    1.8984351, 6.0278643,
    1.8168238, 4.8897277,
    1.9311683, 4.3181278,
    2.1818015, 4.6928626,
    3.1558157, 6.3063666,
    3.0771214, 5.1195376,
    3.0562027, 5.5423914,
    3.1977079, 5.4969752,
    3.0622823, 5.3704876,
    3.0834122, 5.2876523,
    3.0176264, 4.9170203,
    3.0376568, 6.1520265,
    2.9156639, 5.8031604,
    2.8588455, 5.2589957
  ) %>%
    as.data.frame()
  expect_equal(jitter_data, expected_data)
})

# #615: ggboxplot() crashed when a `position` was supplied (the internal
# position_dodge(0.8) was hardcoded and collided in geom_exec). `position` is
# now an argument, so the dodge width is controllable and defaults are unchanged.
test_that("ggboxplot accepts a position argument without error (#615)", {
  tg <- ToothGrowth
  tg$dose <- factor(tg$dose)
  expect_no_error(
    ggplot2::ggplot_build(
      ggboxplot(tg, "dose", "len", color = "supp", position = ggplot2::position_dodge(0.5))
    )
  )
  # bxp.errorbar path too
  expect_no_error(
    ggplot2::ggplot_build(
      ggboxplot(tg, "dose", "len", color = "supp", bxp.errorbar = TRUE,
                position = ggplot2::position_dodge(0.9))
    )
  )
})

test_that("ggboxplot default dodge is unchanged and width is controllable (#615)", {
  tg <- ToothGrowth
  tg$dose <- factor(tg$dose)
  box_x <- function(p) ggplot2::ggplot_build(p)$data[[1]]$xmin
  default <- box_x(ggboxplot(tg, "dose", "len", color = "supp"))
  explicit08 <- box_x(ggboxplot(tg, "dose", "len", color = "supp", position = ggplot2::position_dodge(0.8)))
  narrow <- box_x(ggboxplot(tg, "dose", "len", color = "supp", position = ggplot2::position_dodge(0.4)))
  expect_equal(default, explicit08)          # default == dodge(0.8), no regression
  expect_false(isTRUE(all.equal(default, narrow)))  # width actually changes spacing
})

context("test-ggline-jitter-dodge")

# #436: in a grouped ggline() with add = "jitter", passing position_dodge() used
# to dodge only the line / error bars, leaving the jitter points centered (and
# thus misaligned from their group). The jitter now dodges with the same width,
# so points sit under their group's line. Default (position = "identity") is
# unchanged (jitter stays centered).

df <- ToothGrowth
df$dose <- as.factor(df$dose)

test_that("position_dodge dodges the jitter to align with the line (#436)", {
  w <- 0.4
  p <- ggline(df, x = "dose", y = "len", color = "supp",
              add = c("mean_se", "jitter"), position = position_dodge(w))
  b <- ggplot2::ggplot_build(p)
  # jitter layer (60 rows) mean x per (dose, group); error-bar layer x per (dose, group)
  jit <- eb <- NULL
  for (d in b$data) {
    if (nrow(d) == 60 && "group" %in% names(d)) jit <- d
    if ("ymin" %in% names(d) && nrow(d) <= 8) eb <- d
  }
  expect_false(is.null(jit)); expect_false(is.null(eb))
  jit.center <- as.numeric(aggregate(x ~ interaction(round(x), group), data = jit, FUN = mean)$x)
  eb.center <- as.numeric(sort(eb$x))
  # each error bar has a jitter cluster centered on it (within jitter noise)
  # -> the sorted per-cluster jitter means match the sorted error-bar centers
  expect_equal(sort(jit.center), eb.center, tolerance = 0.05)
  # and the two groups are actually separated (dodged), not centered on integers
  seps <- tapply(jit$x, round(jit$x), function(v) diff(range(tapply(v, v > median(v), mean))))
  expect_true(mean(seps) > 0.1)
})

test_that("default (position = identity) leaves the jitter centered, not dodged", {
  p_def <- ggline(df, x = "dose", y = "len", color = "supp", add = c("mean_se", "jitter"))
  d <- ggplot2::ggplot_build(p_def)$data[[1]]
  # not dodged -> the two groups' x-ranges OVERLAP at each dose (share a center)
  overlap_ok <- TRUE
  for (xc in unique(round(d$x))) {
    sub <- d[round(d$x) == xc, ]
    r1 <- range(sub$x[sub$group == min(sub$group)])
    r2 <- range(sub$x[sub$group == max(sub$group)])
    if (!(r1[1] <= r2[2] && r2[1] <= r1[2])) overlap_ok <- FALSE
  }
  expect_true(overlap_ok)
})

test_that("default jitter is byte-identical to the pre-fix path (seed = 123)", {
  # the fix only fires for a PositionDodge; default position = 'identity' must be
  # untouched. Compare the built jitter x to a manual position_jitter(seed=123).
  p_def <- ggline(df, x = "dose", y = "len", color = "supp", add = "jitter")
  got <- sort(ggplot2::ggplot_build(p_def)$data[[1]]$x)
  ref <- ggplot2::ggplot_build(
    ggplot2::ggplot(df, ggplot2::aes(dose, len)) +
      ggplot2::geom_jitter(position = ggplot2::position_jitter(0.2, seed = 123))
  )$data[[1]]$x
  expect_equal(got, sort(ref), tolerance = 1e-9)
})

test_that("non-grouped ggline jitter is unchanged", {
  expect_error(
    ggplot2::ggplot_build(ggline(df, x = "dose", y = "len", add = "jitter")), NA
  )
})

test_that("position_dodge() with no width does not error (jitter stays centered)", {
  expect_error(
    suppressWarnings(ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      ggline(df, x = "dose", y = "len", color = "supp", add = "jitter",
             position = position_dodge())
    ))),
    NA
  )
})

test_that("position_dodge2 leaves the jitter centered (unchanged, avoids overshoot)", {
  # dodge2 packs zero-width summary geoms differently; the fix is scoped to plain
  # position_dodge, so dodge2 keeps the centered (position_jitter) behavior.
  p_d2 <- ggline(df, x = "dose", y = "len", color = "supp", add = "jitter",
                 position = ggplot2::position_dodge2(0.5))
  p_j <- ggline(df, x = "dose", y = "len", color = "supp", add = "jitter")
  expect_equal(
    sort(ggplot2::ggplot_build(p_d2)$data[[1]]$x),
    sort(ggplot2::ggplot_build(p_j)$data[[1]]$x)
  )
})

test_that("non-line geoms with jitter are unaffected", {
  # ggboxplot uses the else-branch (already jitterdodge) -- must still build
  expect_error(
    ggplot2::ggplot_build(ggboxplot(df, x = "dose", y = "len", color = "supp", add = "jitter")),
    NA
  )
})

context("test-ggpie-donut-repel")

# #655: opt-in `label.repel = TRUE` on ggdonutchart() places the slice labels with
# ggrepel::geom_text_repel() so labels of many small slices no longer overlap. The
# default (label.repel = FALSE) is unchanged / byte-identical. (ggpie() is a solid
# disc where ggrepel in coord_polar cannot reliably clear the face, so the option is
# donut-only.)

suppressPackageStartupMessages(library(ggplot2))

df <- data.frame(
  group = paste0("grp", 1:8),
  value = c(30, 20, 15, 10, 8, 5, 2, 1)
)
df$labs <- paste0(df$group, " (", df$value, "%)")

.has_repel_layer <- function(p) {
  any(vapply(p$layers, function(l) inherits(l$geom, "GeomTextRepel"), logical(1)))
}

test_that("default donut chart has no ggrepel layer (unchanged)", {
  expect_false(.has_repel_layer(ggdonutchart(df, "value", label = "labs", fill = "group")))
})

test_that("label.repel = TRUE adds a ggrepel layer with one label per slice", {
  p <- ggdonutchart(df, "value", label = "labs", fill = "group", label.repel = TRUE)
  expect_true(.has_repel_layer(p))
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  ri <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomTextRepel"), logical(1)))[1]
  expect_equal(nrow(b$data[[ri]]), nrow(df))            # one repelled label per slice
  expect_setequal(as.character(b$data[[ri]]$label), df$labs)
})

test_that("label.repel = TRUE renders without error (draw-time)", {
  expect_error(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      ggdonutchart(df, "value", label = "labs", fill = "group", color = "white",
                   label.repel = TRUE)
    )),
    NA
  )
})

test_that("default label positioning is preserved when label.repel = FALSE", {
  # lab.pos = 'out' (default): labels are axis text -> no text/repel layer
  d_out <- ggdonutchart(df, "value", label = "labs", fill = "group")
  expect_false(.has_repel_layer(d_out))
  expect_false(any(vapply(d_out$layers, function(l) inherits(l$geom, "GeomText"),
                          logical(1))))
  b <- suppressWarnings(ggplot2::ggplot_build(d_out))
  expect_true(any(df$labs %in% b$layout$panel_scales_y[[1]]$get_labels()))
  # lab.pos = 'in' -> a GeomText (not repel) layer
  d_in <- ggdonutchart(df, "value", label = "labs", fill = "group", lab.pos = "in")
  expect_true(any(vapply(d_in$layers, function(l) inherits(l$geom, "GeomText") &&
                           !inherits(l$geom, "GeomTextRepel"), logical(1))))
})

test_that("clean_lock_files skips recent locks in non-interactive all mode", {
  lib <- file.path(tempdir(), "ggpubr-clean-lock-test")
  dir.create(lib, recursive = TRUE, showWarnings = FALSE)

  old_lock <- file.path(lib, "00LOCK-oldpkg")
  recent_lock <- file.path(lib, "00LOCK-recentpkg")
  dir.create(old_lock, showWarnings = FALSE)
  dir.create(recent_lock, showWarnings = FALSE)

  # Emulate stale and active-ish lock timestamps.
  Sys.setFileTime(old_lock, Sys.time() - 20 * 60)
  Sys.setFileTime(recent_lock, Sys.time() - 60)

  expect_warning(
    suppressMessages(clean_lock_files(package = "all", lib = lib, ask = FALSE)),
    "Skipping lock directories modified within the last"
  )

  expect_false(dir.exists(old_lock))
  expect_true(dir.exists(recent_lock))

  unlink(lib, recursive = TRUE, force = TRUE)
})

#' @include utilities.R
NULL

#' P-Value Formatting Styles
#'
#' @description Predefined p-value formatting styles based on major scientific
#'   publication standards and citation styles.
#'
#' @details The following styles are available:
#' \itemize{
#'   \item \code{default}: Current behavior with scientific notation (backward compatible)
#'   \item \code{apa}: APA Style - no leading zero, 3 decimals, threshold at 0.001
#'   \item \code{nejm}: New England Journal of Medicine - leading zero, 3 decimals, threshold at 0.001
#'   \item \code{lancet}: The Lancet - leading zero, 4 decimals, threshold at 0.0001
#'   \item \code{ama}: American Medical Association - no leading zero, 3 decimals, threshold at 0.001
#'   \item \code{graphpad}: GraphPad Prism - leading zero, 4 decimals, threshold at 0.0001
#'   \item \code{scientific}: Scientific notation for genomics/GWAS studies
#' }
#'
#' @name p_format_styles
#' @rdname p_format_styles
#' @keywords internal
.p_format_styles <- list(
  default = list(
    digits = 2,
    leading.zero = TRUE,
    min.threshold = NULL,
    use.scientific = TRUE,
    description = "Current behavior (backward compatible)"
  ),
  apa = list(
    digits = 3,
    leading.zero = FALSE,
    min.threshold = 0.001,
    use.scientific = FALSE,
    description = "APA Style (Psychology, Social Sciences)"
  ),
  nejm = list(
    digits = 3,
    leading.zero = TRUE,
    min.threshold = 0.001,
    use.scientific = FALSE,
    description = "New England Journal of Medicine"
  ),
  lancet = list(
    digits = 4,
    leading.zero = TRUE,
    min.threshold = 0.0001,
    use.scientific = FALSE,
    description = "The Lancet"
  ),
  ama = list(
    digits = 3,
    leading.zero = FALSE,
    min.threshold = 0.001,
    use.scientific = FALSE,
    description = "American Medical Association"
  ),
  graphpad = list(
    digits = 4,
    leading.zero = TRUE,
    min.threshold = 0.0001,
    use.scientific = FALSE,
    description = "GraphPad Prism Style"
  ),
  scientific = list(
    digits = 2,
    leading.zero = TRUE,
    min.threshold = NULL,
    use.scientific = TRUE,
    description = "Scientific notation (for GWAS, genomics)"
  )
)


#' Get P-Value Format Style Settings
#'
#' @description Returns the settings for a given p-value format style.
#'
#' @param style Character string specifying the style. One of: "default", "apa",
#'   "nejm", "lancet", "ama", "graphpad", "scientific".
#'
#' @return A list containing: digits, leading.zero, min.threshold, use.scientific, description.
#'
#' @examples
#' get_p_format_style("apa")
#' get_p_format_style("nejm")
#'
#' @export
get_p_format_style <- function(style = "default") {
  style <- match.arg(style, names(.p_format_styles))
  .p_format_styles[[style]]
}


#' List Available P-Value Format Styles
#'
#' @description Returns a data frame describing all available p-value formatting styles.
#'
#' @return A data frame with columns: style, digits, leading.zero, min.threshold,
#'   use.scientific, description.
#'
#' @examples
#' list_p_format_styles()
#'
#' @export
list_p_format_styles <- function() {
  styles_df <- do.call(rbind, lapply(names(.p_format_styles), function(name) {
    s <- .p_format_styles[[name]]
    data.frame(
      style = name,
      digits = s$digits,
      leading.zero = s$leading.zero,
      min.threshold = ifelse(is.null(s$min.threshold), NA, s$min.threshold),
      use.scientific = s$use.scientific,
      description = s$description,
      stringsAsFactors = FALSE
    )
  }))
  rownames(styles_df) <- NULL
  styles_df
}


#' Format P-Values According to Style or Custom Settings
#'
#' @description Formats p-values according to major scientific publication standards
#'   (APA, AMA, NEJM, Lancet, etc.) or custom user-defined settings. This function
#'   provides flexible control over decimal places, leading zeros, and threshold
#'   notation for very small p-values.
#'
#' @param p Numeric vector of p-values to format.
#' @param style Character string specifying the formatting style. One of:
#'   \itemize{
#'     \item \code{"default"}: Current behavior with scientific notation (backward compatible)
#'     \item \code{"apa"}: APA Style - no leading zero, 3 decimals, "< .001" threshold
#'     \item \code{"nejm"}: NEJM Style - leading zero, 3 decimals, "< 0.001" threshold
#'     \item \code{"lancet"}: Lancet Style - leading zero, 4 decimals, "< 0.0001" threshold
#'     \item \code{"ama"}: AMA Style - no leading zero, 3 decimals, "< .001" threshold
#'     \item \code{"graphpad"}: GraphPad Style - leading zero, 4 decimals, "< 0.0001" threshold
#'     \item \code{"scientific"}: Scientific notation for genomics/GWAS studies
#'   }
#' @param digits Integer specifying the number of decimal places. If provided,
#'   overrides the style default.
#' @param leading.zero Logical indicating whether to include leading zero before
#'   decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.
#' @param min.threshold Numeric specifying the minimum p-value to display exactly.
#'   Values below this threshold are shown as "< threshold" (e.g., "< 0.001").
#'   If NULL, the selected style's default threshold is used; styles without a
#'   threshold show exact values. If provided, overrides the style default.
#'   Must be a single positive finite number.
#' @param decimal.mark Character string to use as the decimal mark. If NULL,
#'   uses \code{getOption("OutDec")}.
#' @param use.scientific Logical indicating whether to force scientific notation.
#'   If NULL, uses the style default.
#'
#' @return Character vector of formatted p-values.
#'
#' @details
#' P-value formatting conventions vary across scientific disciplines and journals:
#'
#' \strong{APA Style} (Psychology, Social Sciences):
#' \itemize{
#'   \item No leading zero (write ".05" not "0.05")
#'   \item 3 decimal places
#'   \item Report as "p < .001" for very small values
#' }
#'
#' \strong{NEJM/Medical Journals}:
#' \itemize{
#'   \item Leading zero (write "0.05" not ".05")
#'   \item 3 decimal places
#'   \item P < 0.001: report as "< 0.001"
#' }
#'
#' \strong{Scientific Notation} (GWAS, Genomics):
#' \itemize{
#'   \item Used when very small p-values are meaningful (e.g., 5e-8 threshold)
#'   \item Appropriate for high-dimensional data analyses
#' }
#'
#' @examples
#' # Test p-values
#' p_vals <- c(0.76404, 0.0432, 0.0043, 0.00018, 1.7e-11)
#'
#' # Different styles
#' format_p_value(p_vals, style = "default")
#' format_p_value(p_vals, style = "apa")
#' format_p_value(p_vals, style = "nejm")
#' format_p_value(p_vals, style = "lancet")
#'
#' # Custom formatting
#' format_p_value(p_vals, digits = 2, leading.zero = FALSE, min.threshold = 0.01)
#'
#' # Override style defaults
#' format_p_value(p_vals, style = "nejm", digits = 4)
#'
#' @seealso \code{\link{get_p_format_style}}, \code{\link{list_p_format_styles}}
#'
#' @export
format_p_value <- function(p,
                           style = "default",
                           digits = NULL,
                           leading.zero = NULL,
                           min.threshold = NULL,
                           decimal.mark = NULL,
                           use.scientific = NULL) {
  # Validate style

  style <- match.arg(style, names(.p_format_styles))


  # Get style defaults

  style_settings <- .p_format_styles[[style]]


  # Apply overrides if provided

  if (is.null(digits)) digits <- style_settings$digits

  if (is.null(leading.zero)) leading.zero <- style_settings$leading.zero

  if (is.null(min.threshold)) min.threshold <- style_settings$min.threshold

  if (is.null(decimal.mark)) decimal.mark <- getOption("OutDec")

  if (is.null(use.scientific)) use.scientific <- style_settings$use.scientific

  if (!is.null(min.threshold)) {
    is_valid_threshold <- is.numeric(min.threshold) &&
      length(min.threshold) == 1L &&
      !is.na(min.threshold) &&
      is.finite(min.threshold) &&
      min.threshold > 0
    if (!is_valid_threshold) {
      stop(
        "`min.threshold` must be a single positive finite numeric value or NULL.",
        call. = FALSE
      )
    }
  }


  # Handle NA values
  result <- rep(NA_character_, length(p))
  valid_idx <- !is.na(p)
  invalid_idx <- valid_idx & (!is.finite(p) | p < 0 | p > 1)
  if (any(invalid_idx)) {
    warning(
      "Invalid p-values detected (must be finite and in [0, 1]); returning NA for those entries.",
      call. = FALSE
    )
    valid_idx <- valid_idx & !invalid_idx
  }

  if (!any(valid_idx)) {
    return(result)
  }

  p_valid <- p[valid_idx]

  if (use.scientific) {
    # Use vectorized format.pval for consistent formatting across values
    # This preserves the original behavior where format.pval formats all values
    # together to maintain consistent decimal places
    formatted <- format.pval(p_valid, digits = digits, decimal.mark = decimal.mark)
    # Keep integer-looking values compact (e.g., "1" instead of "1.0000")
    formatted <- sub("^(-?\\d+)\\.0+$", "\\1", formatted)

    # Apply threshold if specified
    if (!is.null(min.threshold)) {
      below_threshold <- p_valid < min.threshold
      if (any(below_threshold)) {
        threshold_digits <- max(digits, -floor(log10(min.threshold)))
        threshold_str <- format_single_p(min.threshold, threshold_digits, leading.zero, decimal.mark)
        formatted[below_threshold] <- paste0("< ", threshold_str)
      }
    }

    # Apply leading zero removal if requested
    if (!leading.zero) {
      formatted <- sub("^0\\.", ".", formatted)
      formatted <- sub("^-0\\.", "-.", formatted)
    }
  } else {
    # Non-scientific styles: format individually
    formatted <- character(length(p_valid))
    for (i in seq_along(p_valid)) {
      pval <- p_valid[i]

      # Check if below threshold
      if (!is.null(min.threshold) && pval < min.threshold) {
        # Format threshold value - use enough digits to show the threshold
        threshold_digits <- max(digits, -floor(log10(min.threshold)))
        threshold_str <- format_single_p(min.threshold, threshold_digits, leading.zero, decimal.mark)
        formatted[i] <- paste0("< ", threshold_str)
      } else {
        # Format exact value
        formatted[i] <- format_single_p(pval, digits, leading.zero, decimal.mark)
      }
    }
  }

  result[valid_idx] <- formatted
  result
}


#' Format a Single P-Value
#'
#' @description Internal helper function to format a single p-value.
#'
#' @param p Single numeric p-value.
#' @param digits Number of decimal places.
#' @param leading.zero Whether to include leading zero.
#' @param decimal.mark Character string to use as the decimal mark.
#'
#' @return Formatted character string.
#'
#' @keywords internal
format_single_p <- function(p, digits, leading.zero, decimal.mark) {
  if (is.na(p)) {
    return(NA_character_)
  }


  # Round to specified decimal places
  formatted <- formatC(p, digits = digits, format = "f", decimal.mark = decimal.mark)

  # Remove leading zero if requested

  if (!leading.zero) {
    formatted <- sub("^0\\.", ".", formatted)
    formatted <- sub("^-0\\.", "-.", formatted)
  }

  formatted
}


#' Resolve P-Format Parameters
#'
#' @description Internal helper function to resolve p-value formatting parameters
#'   from style and individual overrides. Used by other functions to maintain
#'   consistent parameter handling.
#'
#' @param style Character string specifying the style.
#' @param digits Override for digits parameter.
#' @param leading.zero Override for leading.zero parameter.
#' @param min.threshold Override for min.threshold parameter.
#' @param decimal.mark Override for decimal.mark parameter.
#' @param use.scientific Override for use.scientific parameter.
#'
#' @return A list with resolved parameters: digits, leading.zero, min.threshold, use.scientific.
#'
#' @keywords internal
resolve_p_format_params <- function(style = "default",
                                    digits = NULL,
                                    leading.zero = NULL,
                                    min.threshold = NULL,
                                    decimal.mark = NULL,
                                    use.scientific = NULL) {
  style <- match.arg(style, names(.p_format_styles))
  style_settings <- .p_format_styles[[style]]

  list(
    digits = if (is.null(digits)) style_settings$digits else digits,
    leading.zero = if (is.null(leading.zero)) style_settings$leading.zero else leading.zero,
    min.threshold = if (is.null(min.threshold)) style_settings$min.threshold else min.threshold,
    decimal.mark = if (is.null(decimal.mark)) getOption("OutDec") else decimal.mark,
    use.scientific = if (is.null(use.scientific)) style_settings$use.scientific else use.scientific
  )
}


#' Create P-Value Label with Proper Prefix
#'
#' @description Creates a p-value label string with proper handling of inequality
#'   symbols. When the formatted p-value starts with "<" or ">", uses "p <value"
#'   or "p >value" format. Otherwise uses "p = value" format.
#'
#' @param p.format Character string of the formatted p-value (e.g., "0.05", "< 0.001").
#' @param p.signif Optional character string of significance symbol (e.g., "*", "**", "ns").
#'   If provided, it will be appended after the p-value.
#'
#' @return A character string with the properly formatted p-value label. Missing
#'   values in \code{p.format} are preserved as \code{NA_character_}.
#'
#' @examples
#' \dontrun{
#' create_p_label("0.05")
#' # Returns: "p = 0.05"
#'
#' create_p_label("< 0.001")
#' # Returns: "p < 0.001"
#'
#' create_p_label("0.01", "**")
#' # Returns: "p = 0.01 **"
#'
#' create_p_label("< 0.001", "****")
#' # Returns: "p < 0.001 ****"
#'
#' create_p_label(c("0.05", NA_character_))
#' # Returns: c("p = 0.05", NA)
#' }
#'
#' @export
create_p_label <- function(p.format, p.signif = NULL) {
  label <- rep(NA_character_, length(p.format))
  valid_idx <- !is.na(p.format)
  if (!any(valid_idx)) return(label)

  # Check if p.format starts with < or > for non-missing values
  has_inequality <- grepl("^[<>]", p.format[valid_idx])

  # Create base label
  label_valid <- ifelse(has_inequality,
    paste("p", p.format[valid_idx]),
    paste("p =", p.format[valid_idx])
  )

  # Normalize spacing around inequality symbols
  label_valid <- gsub("([<>])\\s*", "\\1 ", label_valid)

  # Append significance if provided
  if (!is.null(p.signif)) {
    p_signif_vec <- rep_len(p.signif, length(p.format))
    label_valid <- paste(label_valid, p_signif_vec[valid_idx])
    label_valid <- trimws(label_valid)
  }

  label[valid_idx] <- label_valid
  label
}

context("test-stat-pvalue-manual-pdigits")

# p.digits formats numeric p-value label columns for display (restores clean
# labels with rstatix >= 1.0.0 full-precision pairwise p-values), while leaving
# character labels (significance symbols, pre-formatted strings, glue output)
# unchanged. Opt out with p.digits = NULL.

# Build a stat.test with deterministic p-values covering a mid-range value,
# a very small value, and a large value.
.make_stat <- function(p = c(0.015625, 0.0000123, 0.7)) {
  data.frame(
    group1 = c("0.5", "0.5", "1"),
    group2 = c("1", "2", "2"),
    p = p,
    p.signif = c("*", "****", "ns"),
    y.position = c(30, 36, 40),
    stringsAsFactors = FALSE
  )
}

# Extract the plotted label aesthetic (one per comparison, in stat.test row
# order) from a built ggplot. The bracket layer stores several rows per bracket
# (the bracket corners), all sharing the same `group` (= stat.test row index),
# so we keep the first label of each group and order by group.
.plotted_labels <- function(layer) {
  p <- ggboxplot(ToothGrowth, x = "dose", y = "len") + layer
  b <- ggplot2::ggplot_build(p)
  for (d in b$data) {
    if (all(c("label", "group") %in% names(d))) {
      first <- d[!duplicated(d$group), ]
      first <- first[order(first$group), ]
      return(as.character(first$label))
    }
  }
  stop("no label aesthetic found in built plot")
}

test_that("numeric p label is rounded by default (p.digits = 3)", {
  st <- .make_stat()
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st, label = "p")),
    format_p_value(st$p, digits = 3)
  )
  # concrete expectation: 0.015625 -> 0.0156
  expect_true("0.0156" %in% .plotted_labels(stat_pvalue_manual(st, label = "p")))
})

test_that("p.digits controls the number of significant digits", {
  st <- .make_stat(p = c(0.012345, 0.5, 0.9))
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st, label = "p", p.digits = 2)),
    format_p_value(st$p, digits = 2)
  )
})

test_that("p.digits = NULL prints the raw numeric value (opt-out)", {
  st <- .make_stat()
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st, label = "p", p.digits = NULL)),
    as.character(st$p)
  )
})

test_that("character significance labels are unaffected by p.digits", {
  st <- .make_stat()
  # explicit character column
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st, label = "p.signif")),
    st$p.signif
  )
  # default label = NULL resolves to the significance column
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st)),
    st$p.signif
  )
})

test_that("numeric NON-p label columns are left untouched (no NA, no warning)", {
  # stat_pvalue_manual is generic: the label column need not be a p-value.
  # Columns outside [0, 1] (statistic, n, effect size, ...) must NOT be routed
  # through format_p_value(), which would coerce them to NA with a warning.
  st <- .make_stat()
  st$statistic <- c(5.3, -2.1, 12.0)
  st$n <- c(20L, 20L, 20L)

  expect_warning(
    labs_stat <- .plotted_labels(stat_pvalue_manual(st, label = "statistic")),
    NA
  )
  expect_equal(labs_stat, as.character(st$statistic))

  expect_warning(
    labs_n <- .plotted_labels(stat_pvalue_manual(st, label = "n")),
    NA
  )
  expect_equal(labs_n, as.character(st$n))
})

test_that("a p-named column holding out-of-[0,1] values is left untouched", {
  # A numeric column merely NAMED like a p-value but holding other quantities
  # must NOT be routed through format_p_value() (which would return NA + warn).
  # The range guard leaves the whole column as-is if any value is outside [0, 1].
  st <- .make_stat(p = c(5.3, 0.2, 0.7))
  expect_warning(
    labs <- .plotted_labels(stat_pvalue_manual(st, label = "p")),
    NA
  )
  expect_equal(labs, as.character(st$p))
})

test_that("glue label expressions are unaffected (documented limitation)", {
  st <- .make_stat()
  # glue is evaluated on the raw numeric column, so it keeps full precision;
  # users round inside the expression, e.g. {signif(p, 3)}.
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st, label = "{p}")),
    as.character(st$p)
  )
})

test_that("NA p-values do not error and render as NA", {
  st <- .make_stat(p = c(0.015625, NA, 0.7))
  labs <- .plotted_labels(stat_pvalue_manual(st, label = "p"))
  expect_equal(labs[2], NA_character_)
  expect_equal(labs[c(1, 3)], format_p_value(c(0.015625, 0.7), digits = 3))
})

context("test-stat-pvalue-manual-pformat")

# stat_pvalue_manual() exposes the same p-value formatting knobs as its sibling
# bracket layers (stat_compare_means / geom_pwc / stat_anova_test):
# p.format.style, p.leading.zero, p.min.threshold, p.decimal.mark. All default
# so that the rendered labels are byte-identical to before (opt-in only).

.make_stat <- function(p = c(0.015625, 0.0000000004, 0.7)) {
  data.frame(
    group1 = c("0.5", "0.5", "1"),
    group2 = c("1", "2", "2"),
    p = p,
    y.position = c(30, 36, 40),
    stringsAsFactors = FALSE
  )
}

.plotted_labels <- function(layer) {
  p <- ggboxplot(ToothGrowth, x = "dose", y = "len") + layer
  b <- ggplot2::ggplot_build(p)
  for (d in b$data) {
    if (all(c("label", "group") %in% names(d))) {
      first <- d[!duplicated(d$group), ]
      first <- first[order(first$group), ]
      return(as.character(first$label))
    }
  }
  stop("no label aesthetic found in built plot")
}

test_that("defaults leave the rendered p labels byte-identical (opt-in only)", {
  st <- .make_stat()
  # The style arguments at their defaults must reproduce the plain
  # format_p_value(p, digits = 3) output exactly.
  expect_identical(
    .plotted_labels(stat_pvalue_manual(st, label = "p")),
    format_p_value(st$p, digits = 3)
  )
  # Explicitly passing the defaults is a no-op vs omitting them.
  expect_identical(
    .plotted_labels(stat_pvalue_manual(st, label = "p")),
    .plotted_labels(stat_pvalue_manual(
      st, label = "p", p.format.style = "default",
      p.leading.zero = NULL, p.min.threshold = NULL, p.decimal.mark = NULL
    ))
  )
})

test_that("p.min.threshold renders very small p-values as '< threshold'", {
  st <- .make_stat()
  labs <- .plotted_labels(stat_pvalue_manual(st, label = "p", p.min.threshold = 0.001))
  expect_equal(labs[2], "< 0.001")
  # values above the threshold are untouched
  expect_equal(labs[c(1, 3)], format_p_value(st$p[c(1, 3)], digits = 3))
})

test_that("p.format.style selects a journal style", {
  st <- .make_stat()
  expect_equal(
    .plotted_labels(stat_pvalue_manual(st, label = "p", p.format.style = "nejm")),
    format_p_value(st$p, style = "nejm", digits = 3)
  )
})

test_that("p.leading.zero = FALSE drops the leading zero", {
  st <- .make_stat()
  labs <- .plotted_labels(stat_pvalue_manual(st, label = "p", p.leading.zero = FALSE))
  expect_equal(labs[1], ".0156")
})

test_that("p.decimal.mark controls the decimal separator", {
  st <- .make_stat()
  labs <- .plotted_labels(stat_pvalue_manual(st, label = "p", p.decimal.mark = ","))
  expect_equal(labs[1], "0,0156")
})

test_that("p.digits = NULL disables all formatting, including the style args", {
  st <- .make_stat()
  # NULL is the documented raw opt-out; the style args must then be ignored.
  expect_equal(
    .plotted_labels(stat_pvalue_manual(
      st, label = "p", p.digits = NULL,
      p.format.style = "nejm", p.min.threshold = 0.001
    )),
    as.character(st$p)
  )
})

test_that("style args are a no-op on character, significance and glue labels", {
  st <- .make_stat()
  st$p.signif <- c("*", "****", "ns")
  # character significance column
  expect_equal(
    .plotted_labels(stat_pvalue_manual(
      st, label = "p.signif", p.format.style = "nejm", p.min.threshold = 0.001
    )),
    st$p.signif
  )
  # glue expression is evaluated on the raw column, unaffected by style args
  expect_equal(
    .plotted_labels(stat_pvalue_manual(
      st, label = "p = {p}", p.format.style = "nejm", p.leading.zero = FALSE
    )),
    paste0("p = ", as.character(st$p))
  )
})

test_that("style args do not touch non-p numeric columns or out-of-range values", {
  st <- .make_stat()
  st$statistic <- c(5.3, -2.1, 12.0)
  # a numeric column named like a p-value but out of [0, 1]
  st_bad <- .make_stat(p = c(0.5, 5.3, 0.7))

  expect_warning(
    labs_stat <- .plotted_labels(stat_pvalue_manual(
      st, label = "statistic", p.min.threshold = 0.001, p.format.style = "nejm"
    )),
    NA
  )
  expect_equal(labs_stat, as.character(st$statistic))

  expect_warning(
    labs_bad <- .plotted_labels(stat_pvalue_manual(
      st_bad, label = "p", p.min.threshold = 0.001
    )),
    NA
  )
  expect_equal(labs_bad, as.character(st_bad$p))
})

test_that("format_p_value returns correct format for default style", {
  # Default style should use format.pval with 2 significant digits
  expect_equal(format_p_value(0.001234), "0.0012")
  expect_equal(format_p_value(0.000001234), "1.2e-06")
  expect_equal(format_p_value(0.05), "0.05")
  expect_equal(format_p_value(0.5), "0.5")
})

test_that("format_p_value returns correct format for APA style", {
  # APA: 3 decimals, no leading zero, < .001 threshold
  p <- format_p_value(0.0456, style = "apa")
  expect_true(grepl("046", p))
  expect_false(grepl("^0\\.", p))

  # Very small p-values should use < .001
  p_small <- format_p_value(0.0001, style = "apa")
  expect_true(grepl("<", p_small))
  expect_true(grepl("\\.001", p_small))
})

test_that("format_p_value returns correct format for NEJM style", {
  # NEJM: 3 decimals, leading zero, < 0.001 threshold
  p <- format_p_value(0.0456, style = "nejm")
  expect_true(grepl("0\\.046", p))

  # Very small p-values should use < 0.001
  p_small <- format_p_value(0.0001, style = "nejm")
  expect_true(grepl("<", p_small))
  expect_true(grepl("0\\.001", p_small))
})

test_that("format_p_value returns correct format for Lancet style", {
  # Lancet: 4 decimals, leading zero
  p <- format_p_value(0.04567, style = "lancet")
  expect_true(grepl("0\\.0457", p))
})

test_that("format_p_value respects custom digits parameter", {
  # Custom digits should override style defaults
  # Using a non-scientific style to test digits more clearly
  p <- format_p_value(0.04567, style = "nejm", digits = 4)
  expect_true(grepl("0457", p))
})

test_that("format_p_value respects custom leading.zero parameter", {
  # Override leading zero setting - use non-scientific style
  p_no_zero <- format_p_value(0.05, style = "nejm", leading.zero = FALSE)
  expect_false(grepl("^0\\.", p_no_zero))
  expect_true(grepl("^\\.05", p_no_zero))

  p_with_zero <- format_p_value(0.05, style = "apa", leading.zero = TRUE)
  expect_true(grepl("^0\\.", p_with_zero))
})

test_that("format_p_value respects custom min.threshold parameter", {
  # Custom minimum threshold with non-scientific style
  p <- format_p_value(0.00001, style = "nejm", min.threshold = 0.0001)
  expect_true(grepl("<", p))
  expect_true(grepl("0\\.0001", p))
})

test_that("format_p_value respects decimal.mark parameter", {
  p <- format_p_value(0.0047, style = "nejm", decimal.mark = ",")
  expect_true(grepl(",", p))
  expect_false(grepl("\\.", p))
})

test_that("get_p_format_style returns correct style parameters", {
  apa <- get_p_format_style("apa")
  expect_equal(apa$digits, 3)
  expect_false(apa$leading.zero)
  expect_equal(apa$min.threshold, 0.001)

  nejm <- get_p_format_style("nejm")
  expect_equal(nejm$digits, 3)
  expect_true(nejm$leading.zero)
  expect_equal(nejm$min.threshold, 0.001)

  lancet <- get_p_format_style("lancet")
  expect_equal(lancet$digits, 4)
  expect_true(lancet$leading.zero)
  expect_equal(lancet$min.threshold, 0.0001)
})

test_that("list_p_format_styles returns all available styles", {
  styles <- list_p_format_styles()
  expect_true(is.data.frame(styles))
  expect_true("default" %in% styles$style)
  expect_true("apa" %in% styles$style)
  expect_true("nejm" %in% styles$style)
  expect_true("lancet" %in% styles$style)
  expect_true("ama" %in% styles$style)
  expect_true("graphpad" %in% styles$style)
  expect_true("scientific" %in% styles$style)
})

test_that("format_p_value handles NA values", {
  expect_true(is.na(format_p_value(NA)))
})

test_that("format_p_value handles vector input", {
  p_values <- c(0.001, 0.05, 0.5)
  results <- format_p_value(p_values)
  expect_length(results, 3)
})

test_that("format_p_value returns scientific style correctly", {
  # Scientific style: uses format.pval which may use scientific notation for very small values
  p <- format_p_value(0.0000001234, style = "scientific")
  expect_true(grepl("e-", p))
})

test_that("format_p_value for GraphPad style", {
  # GraphPad: 4 decimals, leading zero, < 0.0001 threshold
  p <- format_p_value(0.04567, style = "graphpad")
  expect_true(grepl("0\\.0457", p))
})

test_that("format_p_value handles edge cases", {
  # P-value of exactly 1
  expect_equal(format_p_value(1), "1")

  # Very small p-value with scientific style
  p_tiny <- format_p_value(1e-20, style = "scientific")
  expect_true(grepl("e-", p_tiny) || grepl("<", p_tiny))

  # P-value of exactly 0 (edge case)
  p_zero <- format_p_value(0)
  expect_true(grepl("0", p_zero) || grepl("<", p_zero))
})

test_that("format_p_value leading.zero works with scientific styles", {
  # Test leading zero removal with default style
  p <- format_p_value(0.05, style = "default", leading.zero = FALSE)
  expect_true(grepl("^\\.05", p))
})

test_that("format_p_value min.threshold works with scientific styles", {
  # Test min.threshold with default style
  p <- format_p_value(0.00001, style = "default", min.threshold = 0.0001)
  expect_true(grepl("<", p))
})

test_that("format_p_value handles invalid p-values safely", {
  expect_warning(
    out <- format_p_value(c(-0.1, 0.5, 1.2, Inf, NaN)),
    "Invalid p-values detected"
  )
  expect_true(is.na(out[1]))
  expect_false(is.na(out[2]))
  expect_true(is.na(out[3]))
  expect_true(is.na(out[4]))
  expect_true(is.na(out[5]))
})

test_that("format_p_value allows overriding scientific notation", {
  p <- format_p_value(1e-6, style = "default", use.scientific = FALSE, digits = 6)
  expect_equal(p, "0.000001")
})

test_that("create_p_label builds labels correctly", {
  expect_equal(create_p_label("0.05"), "p = 0.05")
  expect_equal(create_p_label("< 0.001"), "p < 0.001")
  expect_equal(create_p_label("0.01", "**"), "p = 0.01 **")
  expect_equal(create_p_label("< 0.001", "****"), "p < 0.001 ****")
})

test_that("format_p_value validates min.threshold input", {
  expect_error(
    format_p_value(0.01, style = "nejm", min.threshold = 0),
    "`min.threshold` must be a single positive finite numeric value or NULL\\."
  )
  expect_error(
    format_p_value(0.01, style = "nejm", min.threshold = -1),
    "`min.threshold` must be a single positive finite numeric value or NULL\\."
  )
  expect_error(
    format_p_value(0.01, style = "nejm", min.threshold = Inf),
    "`min.threshold` must be a single positive finite numeric value or NULL\\."
  )
  expect_error(
    format_p_value(0.01, style = "nejm", min.threshold = c(0.001, 0.01)),
    "`min.threshold` must be a single positive finite numeric value or NULL\\."
  )
})

test_that("format_p_value handles common reporting boundaries", {
  expect_equal(format_p_value(0.050, style = "nejm"), "0.050")
  expect_equal(format_p_value(0.010, style = "nejm"), "0.010")
  expect_equal(format_p_value(0.001, style = "nejm"), "0.001")
  expect_equal(format_p_value(0.0009, style = "nejm"), "< 0.001")
})

test_that("create_p_label preserves missing formatted p-values", {
  labels <- create_p_label(c("< 0.001", NA_character_), c("****", "ns"))
  expect_equal(labels[1], "p < 0.001 ****")
  expect_true(is.na(labels[2]))
  expect_true(is.na(create_p_label(NA_character_)))
})

#' @include utilities_color.R
NULL
#' Set Gradient Color
#'
#' @description Change gradient color.
#' \itemize{
#' \item \code{gradient_color()}: Change gradient color.
#' \item \code{gradient_fill()}: Change gradient fill.
#' }
#' @param palette the color palette to be used for coloring or filling by
#'   groups. Allowed values include "grey" for grey color palettes; brewer
#'   palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
#'   "red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
#'   "aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
#'   "rickandmorty". Can be also a numeric vector; in this
#'   case a basic color palette is created using the function
#'   \link[grDevices]{palette}.
#'
#' @seealso \link{set_palette}.
#'
#'
#' @examples
#' df <- mtcars
#' p <- ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "mpg"
#' )
#'
#' # Change gradient color
#' # Use one custom color
#' p + gradient_color("red")
#'
#' # Two colors
#' p + gradient_color(c("blue", "red"))
#'
#' # Three colors
#' p + gradient_color(c("blue", "white", "red"))
#'
#' # Use RColorBrewer palette
#' p + gradient_color("RdYlBu")
#'
#' # Use ggsci color palette
#' p + gradient_color("npg")
#' @rdname gradient_color
#' @export
gradient_color <- function(palette) {
  if (.is_col_palette(palette)) palette <- .get_pal(palette, k = 3)
  n_palette <- length(palette)

  if (n_palette == 1) {
    palette <- grDevices::colorRampPalette(c("white", palette))(10)
    palette <- palette[c(1, 10)]
    n_palette <- 2
  }
  ggplot2::scale_color_gradientn(colours = palette)
}

#' @rdname gradient_color
#' @export
gradient_fill <- function(palette) {
  if (.is_col_palette(palette)) palette <- .get_pal(palette, k = 3)
  n_palette <- length(palette)

  if (n_palette == 1) {
    palette <- grDevices::colorRampPalette(c("white", palette))(10)
    palette <- palette[c(1, 10)]
    n_palette <- 2
  }
  ggplot2::scale_fill_gradientn(colours = palette)
}

#' @include utilities.R
NULL

# Return the y-scale transformation function (e.g. log10) in a way that is robust
# across ggplot2 versions, or NULL if unavailable. Used to place brackets in the
# transformed space of the scale (#342). For an identity scale this is the
# identity function, so applying it leaves positions unchanged.
.stat_bracket_y_transform <- function(scale) {
  trans <- NULL
  # ggplot2 >= 3.5: get_transformation() method / transformation field
  if (is.function(scale$get_transformation)) {
    trans <- tryCatch(scale$get_transformation(), error = function(e) NULL)
  }
  if (is.null(trans)) trans <- scale$transformation
  # ggplot2 < 3.5: trans field
  if (is.null(trans)) trans <- scale$trans
  if (is.null(trans) || !is.function(trans$transform)) return(NULL)
  trans$transform
}

# Companion to .stat_bracket_y_transform: the INVERSE of a scale's transformation
# (e.g. 10^x for a log10 scale), or NULL. Used by vertical brackets to undo the
# x-scale transform that ggplot2 automatically applies to the xmin/xmax columns
# (which, for a vertical bracket, actually carry y-axis values) before those are
# re-expressed in the y scale (#456).
.stat_bracket_scale_inverse <- function(scale) {
  trans <- NULL
  if (is.function(scale$get_transformation)) {
    trans <- tryCatch(scale$get_transformation(), error = function(e) NULL)
  }
  if (is.null(trans)) trans <- scale$transformation
  if (is.null(trans)) trans <- scale$trans
  if (is.null(trans) || !is.function(trans$inverse)) return(NULL)
  trans$inverse
}

StatBracket <- ggplot2::ggproto("StatBracket", ggplot2::Stat,
  required_aes = c("x", "y", "group"),
  setup_params = function(data, params) {
    if (length(params$tip.length) == 1) params$tip.length <- rep(params$tip.length, max(length(params$xmin), 1) * 2)
    if (length(params$tip.length) == length(params$xmin)) params$tip.length <- rep(params$tip.length, each = 2)
    return(params)
  },
  compute_group = function(data, scales, tip.length, tip.length.ref = "data",
                           orientation = "horizontal") {
    # A bracket has a SPAN axis (where its two ends sit) and a POSITION axis
    # (where the connecting bar sits, with the tips extending toward the data).
    # For the default horizontal bracket the span is x and the position is y.
    # For a vertical bracket (#456) the axes swap: the span is y and the bar sits
    # along x. Selecting the two scales here keeps the geometry identical for the
    # horizontal (default) path while enabling the vertical one; every quantity
    # that used to be tied to the y scale (tip length, step increase, the log
    # transform) follows the POSITION axis, and the character/level mapping and
    # the log transform of the ENDS follow the SPAN axis.
    vertical <- identical(orientation, "vertical")
    span.scale <- if (vertical) scales$y else scales$x
    pos.scale <- if (vertical) scales$x else scales$y
    pos.range <- pos.scale$range$range
    if (is.numeric(pos.range)) {
      pos.scale.range <- pos.range[2] - pos.range[1]
    } else {
      # discrete position axis (e.g. a vertical bracket over a categorical x):
      # the numeric positions are 1..n_levels, so the usable range is n-1.
      pos.scale.range <- max(length(pos.range) - 1, 1)
    }
    # Reference range used to convert the fractional `tip.length` into data units.
    # Default ("data"): fraction of the trained data range (historical behavior) -
    # tips scale with the data, so plots with different data ranges get different
    # absolute tips. "axis": fraction of the position-axis range (limits set via
    # ylim / scale_*), which renders at the same physical fraction across plots
    # and so gives visually constant tips regardless of the data (#362).
    tip.scale.range <- pos.scale.range
    if (identical(tip.length.ref, "axis")) {
      axis.limits <- pos.scale$get_limits()
      if (length(axis.limits) == 2 && all(is.finite(axis.limits))) {
        tip.scale.range <- axis.limits[2] - axis.limits[1]
      }
    }
    # Place the bracket in the transformed space of the position scale, so a
    # user-supplied position given in data units lands correctly on e.g.
    # scale_y_log10(). The trained range/tips above are already in transformed
    # units. For an identity scale the transform is a no-op, so default output is
    # unchanged (#342).
    pos.transform <- .stat_bracket_y_transform(pos.scale)
    if (!is.null(pos.transform)) {
      data$y.position <- pos.transform(data$y.position)
    }
    bracket.shorten <- data$bracket.shorten / 2
    span.min <- data$xmin
    span.max <- data$xmax
    # A vertical bracket's ends carry Y-axis values, but they travel in the
    # xmin/xmax columns, which ggplot2 has already transformed with the X scale
    # (the position scale here). Undo that x transform so the ends are back in
    # raw data units before we re-express them in the Y (span) scale below; this
    # keeps a non-identity x scale (log/sqrt/reverse) from corrupting the span
    # (#456). The horizontal path skips this entirely and is unchanged.
    if (vertical) {
      pos.inverse <- .stat_bracket_scale_inverse(pos.scale)
      if (!is.null(pos.inverse)) {
        if (is.numeric(span.min)) span.min <- pos.inverse(span.min)
        if (is.numeric(span.max)) span.max <- pos.inverse(span.max)
      }
    }
    span.min <- span.min + bracket.shorten
    span.max <- span.max - bracket.shorten
    position <- data$y.position + (pos.scale.range * data$step.increase) + data$bracket.nudge.y
    label <- data$label
    if (is.character(span.min)) {
      span.min <- span.scale$map(span.min)
    }
    if (is.character(span.max)) {
      span.max <- span.scale$map(span.max)
    }
    # For a vertical bracket the ends run along the (value) y axis; express them
    # in that scale's transformed space so a log-y span lands correctly.
    # Horizontal keeps the ends on x with no transform, exactly as before.
    if (vertical) {
      span.transform <- .stat_bracket_y_transform(span.scale)
      if (!is.null(span.transform)) {
        if (is.numeric(span.min)) span.min <- span.transform(span.min)
        if (is.numeric(span.max)) span.max <- span.transform(span.max)
      }
    }
    if ("tip.length" %in% colnames(data)) {
      tip.length <- rep(data$tip.length, each = 2)
    }
    # Preparing bracket data: three segments (tip, bar, tip) in (span, position)
    # space, then assigned to x/y according to the orientation.
    data <- dplyr::bind_rows(data, data, data)
    span <- c(span.min, span.min, span.max)
    span.end <- c(span.min, span.max, span.max)
    pos.v <- c(position - tip.scale.range * tip.length[seq_along(tip.length) %% 2 == 1], position, position)
    pos.end <- c(position, position, position - tip.scale.range * tip.length[seq_along(tip.length) %% 2 == 0])
    if (vertical) {
      data$x <- pos.v
      data$xend <- pos.end
      data$y <- span
      data$yend <- span.end
    } else {
      data$x <- span
      data$xend <- span.end
      data$y <- pos.v
      data$yend <- pos.end
    }
    data$annotation <- rep(label, 3)
    # #631: when the POSITION axis is trained to a zero-width range - e.g. a single
    # bracket over a stat-computed y such as geom_bar()/geom_histogram(), whose bar
    # counts do not train the y scale by the time this stat runs - tip.scale.range
    # is 0 and the tips above collapse to zero length (an invisible flat bar).
    # Record that so GeomBracket$draw_group can re-derive the tips against the
    # fully-trained panel range at draw time (the only place the real range,
    # including coord_cartesian(ylim=), is known). These internal columns are
    # ignored by rendering; for a normal, non-zero range the flag is FALSE and
    # nothing downstream changes, so existing plots are byte-identical.
    data$.bracket.tip.zero. <- !is.finite(tip.scale.range) || isTRUE(tip.scale.range == 0)
    data$.bracket.tip.left. <- tip.length[seq_along(tip.length) %% 2 == 1][1]
    data$.bracket.tip.right. <- tip.length[seq_along(tip.length) %% 2 == 0][1]
    data
  }
)


#' Add Brackets with Labels to a GGPlot
#' @description add brackets with label annotation to a ggplot. Helpers for
#'   adding p-value or significance levels to a plot.
#' @param label character vector with alternative label, if not null test is
#'   ignored
#' @param type the label type. Can be one of "text" and "expression" (for
#'   parsing plotmath expression).
#' @param xmin numeric vector with the positions of the left sides of the
#'   brackets
#' @param xmax numeric vector with the positions of the right sides of the
#'   brackets
#' @param y.position numeric vector with the y positions of the brackets
#' @param size change the width of the lines of the bracket. Deprecated, use \code{linewidth} instead.
#' @param linewidth change the width of the lines of the bracket
#' @param label.size change the size of the label text
#' @param family change the font used for the text
#' @param vjust move the text up or down relative to the bracket
#' @param step.increase numeric vector with the increase in fraction of total
#'   height for every additional comparison to minimize overlap.
#' @param bracket.nudge.y Vertical adjustment to nudge brackets by. Useful to
#'   move up or move down the bracket. If positive value, brackets will be moved
#'   up; if negative value, brackets are moved down.
#' @param bracket.shorten a small numeric value in [0-1] for shortening the with
#'   of bracket.
#' @param step.group.by a variable name for grouping brackets before adding
#'   step.increase. Useful to group bracket by facet panel.
#' @param tip.length numeric vector with the fraction that the bracket tips go
#'   down to indicate the precise column. Interpreted relative to the reference
#'   set by \code{tip.length.ref}. Default is 0.03. Can be of the same length as
#'   the number of brackets to adjust each tip specifically, e.g.
#'   \code{tip.length = c(0.01, 0.03)}. If too short it is recycled.
#' @param tip.length.ref character string specifying what \code{tip.length} is a
#'   fraction of. Allowed values are: \itemize{ \item \code{"data"} (default):
#'   fraction of the trained data range. Tips scale with the data, so plots with
#'   different data ranges get different absolute tip lengths (historical
#'   behavior; keeps existing plots unchanged). \item \code{"axis"}: fraction of
#'   the y-axis range (the limits set via \code{ylim}/\code{scale_y_*}). This
#'   renders at the same physical fraction across plots and therefore gives
#'   visually constant tip lengths regardless of the data - useful to keep tips
#'   consistent across facets or across separate plots with different scales
#'   (#362).}
#' @param na.rm If \code{FALSE} (the default), removes missing values with a
#'   warning.  If \code{TRUE} silently removes missing values.
#' @param coord.flip logical. If \code{TRUE}, flip x and y coordinates so that
#'   horizontal becomes vertical, and vertical, horizontal. When adding the
#'   p-values to a horizontal ggplot (generated using
#'   \code{\link[ggplot2]{coord_flip}()}), you need to specify the option
#'   \code{coord.flip = TRUE}.
#' @param orientation the bracket orientation. Either \code{"horizontal"} (the
#'   default, a bracket spanning the x axis with the bar at \code{y.position}) or
#'   \code{"vertical"} (a bracket spanning the y axis with the bar at
#'   \code{x.position}, tips pointing left and the label rotated to the side).
#'   Vertical brackets are useful to annotate plots where the comparison runs
#'   along the y axis, e.g. Kaplan-Meier curves. They are designed for a
#'   continuous x axis; specify the span with \code{ymin}/\code{ymax} and the
#'   position with \code{x.position}. Cannot be combined with
#'   \code{coord.flip = TRUE}.
#' @param ymin,ymax numeric vectors with the positions, along the y axis, of the
#'   two ends of each vertical bracket. Used only when
#'   \code{orientation = "vertical"} (the vertical-bracket analogue of
#'   \code{xmin}/\code{xmax}).
#' @param x.position numeric vector with the x positions of the vertical
#'   brackets. Used only when \code{orientation = "vertical"} (the
#'   vertical-bracket analogue of \code{y.position}).
#' @param ... other arguments passed on to \code{\link[ggplot2]{layer}()}). These are often
#'   aesthetics, used to set an aesthetic to a fixed value, like \code{color =
#'   "red"} or \code{size = 3}. They may also be parameters to the paired
#'   geom/stat.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @examples
#' df <- ToothGrowth
#' df$dose <- factor(df$dose)
#'
#' # Add bracket with labels
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     xmin = "0.5", xmax = "1", y.position = 30,
#'     label = "t-test, p < 0.05"
#'   )
#'
#' # Customize bracket tip.length tip.length
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     xmin = "0.5", xmax = "1", y.position = 30,
#'     label = "t-test, p < 0.05", tip.length = c(0.2, 0.02)
#'   )
#'
#' # Using plotmath expression
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     xmin = "0.5", xmax = "1", y.position = 30,
#'     label = "list(~italic(p)<=0.001)", type = "expression",
#'     tip.length = c(0.2, 0.02)
#'   )
#'
#' # Specify multiple brackets manually
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     xmin = c("0.5", "1"), xmax = c("1", "2"),
#'     y.position = c(30, 35), label = c("***", "**"),
#'     tip.length = 0.01
#'   )
#'
#' # Compute statistical tests and add p-values
#' stat.test <- compare_means(len ~ dose, ToothGrowth, method = "t.test")
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     aes(xmin = group1, xmax = group2, label = signif(p, 2)),
#'     data = stat.test, y.position = 35
#'   )
#'
#' # Increase step length between brackets
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     aes(xmin = group1, xmax = group2, label = signif(p, 2)),
#'     data = stat.test, y.position = 35, step.increase = 0.1
#'   )
#'
#' # Or specify the positions of each comparison
#' ggboxplot(df, x = "dose", y = "len") +
#'   geom_bracket(
#'     aes(xmin = group1, xmax = group2, label = signif(p, 2)),
#'     data = stat.test, y.position = c(32, 35, 38)
#'   )
#'
#' # Vertical bracket (e.g. to annotate a plot with a continuous x axis)
#' ggscatter(df, x = "len", y = "len") +
#'   geom_bracket(
#'     orientation = "vertical",
#'     ymin = 10, ymax = 25, x.position = 34,
#'     label = "p < 0.05"
#'   )
#' @rdname geom_bracket
#' @export
stat_bracket <- function(mapping = NULL, data = NULL,
                         position = "identity", na.rm = FALSE, show.legend = NA,
                         inherit.aes = TRUE,
                         label = NULL, type = c("text", "expression"), y.position = NULL, xmin = NULL, xmax = NULL,
                         step.increase = 0, step.group.by = NULL, tip.length = 0.03,
                         tip.length.ref = c("data", "axis"),
                         bracket.nudge.y = 0, bracket.shorten = 0,
                         size = 0.3, linewidth = size, label.size = 3.88, family = "", vjust = 0,
                         ...) {
  if (!is.null(data) & !is.null(mapping)) {
    if (!"x" %in% names(data)) mapping$x <- 1
    if (!"y" %in% names(data)) mapping$y <- 1
  }
  type <- match.arg(type)
  tip.length.ref <- match.arg(tip.length.ref)
  ggplot2::layer(
    stat = StatBracket, data = data, mapping = mapping, geom = "bracket",
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      label = label, type = type,
      y.position = y.position, xmin = xmin, xmax = xmax,
      step.increase = step.increase, bracket.nudge.y = bracket.nudge.y,
      bracket.shorten = bracket.shorten, step.group.by = step.group.by,
      tip.length = tip.length, tip.length.ref = tip.length.ref,
      size = linewidth, label.size = label.size,
      family = family, vjust = vjust, na.rm = na.rm, ...
    )
  )
}


GeomBracket <- ggplot2::ggproto("GeomBracket", ggplot2::Geom,
  required_aes = c("x", "xend", "y", "yend", "annotation"),
  default_aes = ggplot2::aes(
    shape = 19, colour = "black", label.size = 3.88, angle = NULL, hjust = 0.5,
    vjust = 0, alpha = NA, family = "", fontface = 1, lineheight = 1.2, linetype = 1, size = 0.3, linewidth = NULL,
    xmin = NULL, xmax = NULL, label = NULL, y.position = NULL, step.increase = 0,
    bracket.nudge.y = 0, bracket.shorten = 0 # Added to avoid aesthetics warning
  ),
  # draw_key = function(...){grid::nullGrob()},
  # for legend:
  draw_key = draw_key_path,
  draw_group = function(data, panel_params, coord, type = "text",
                        coord.flip = FALSE, orientation = "horizontal") {
    lab <- as.character(data$annotation)
    if (type == "expression") {
      lab <- parse_as_expression(lab)
    }
    coords <- coord$transform(data, panel_params)
    # #631: rescue tips that collapsed because the position axis trained to a
    # zero-width data range (StatBracket flagged it via .bracket.tip.zero.). Here,
    # after transform, the panel maps to npc [0, 1] over its FULLY-trained range
    # (which includes coord_cartesian(ylim=) and scale limits), so a tip length
    # given as a fraction of that range is simply the fraction itself in npc units.
    # Rebuild the two tips against that range. Gated to the default horizontal,
    # non-flipped bracket and to the flagged degenerate case, so every normal plot
    # (tip.scale.range > 0 -> flag FALSE) keeps its exact current tips.
    if (identical(orientation, "horizontal") && !isTRUE(coord.flip) &&
        nrow(coords) == 3 && isTRUE(coords$.bracket.tip.zero.[1])) {
      bar.npc <- coords$y[2]                       # flat bar level (row 2)
      tip.left <- coords$.bracket.tip.left.[1]     # untransformed fraction
      tip.right <- coords$.bracket.tip.right.[1]
      if (is.finite(bar.npc)) {
        if (is.finite(tip.left) && tip.left > 0) coords$y[1] <- bar.npc - tip.left
        if (is.finite(tip.right) && tip.right > 0) coords$yend[3] <- bar.npc - tip.right
      }
    }
    label.x <- mean(c(coords$x[1], tail(coords$xend, n = 1)))
    label.y <- max(c(coords$y, coords$yend)) + 0.01
    label.angle <- coords$angle
    if (identical(orientation, "vertical")) {
      # native vertical bracket (#456): the bar spans y, tips point left, so the
      # label sits to the RIGHT of the bar, centered on the span and rotated.
      label.x <- max(c(coords$x, coords$xend)) + 0.02
      label.y <- mean(c(coords$y[1], tail(coords$yend, n = 1)))
      if (is.null(label.angle)) label.angle <- -90
    } else if (coord.flip) {
      label.y <- mean(c(coords$y[1], tail(coords$yend, n = 1)))
      label.x <- max(c(coords$x, coords$xend)) + 0.01
      if (is.null(label.angle)) label.angle <- -90
    }
    if (is.null(label.angle)) label.angle <- 0

    grid::gList(
      grid::textGrob(
        label = lab,
        x = label.x,
        y = label.y,
        default.units = "native",
        hjust = coords$hjust, vjust = coords$vjust,
        rot = label.angle,
        gp = grid::gpar(
          col = scales::alpha(coords$colour, coords$alpha),
          fontsize = coords$label.size * ggplot2::.pt,
          fontfamily = coords$family,
          fontface = coords$fontface,
          lineheight = coords$lineheight
        )
      ),
      grid::segmentsGrob(
        coords$x, coords$y,
        default.units = "native",
        coords$xend, coords$yend,
        gp = grid::gpar(
          col = scales::alpha(coords$colour, coords$alpha),
          lty = coords$linetype,
          lwd = (coords$linewidth %||% coords$size) * ggplot2::.pt
        )
      )
    )
  }
)

#' @rdname geom_bracket
#' @export
geom_bracket <- function(mapping = NULL, data = NULL, stat = "bracket",
                         position = "identity", na.rm = FALSE, show.legend = NA,
                         inherit.aes = TRUE,
                         label = NULL, type = c("text", "expression"), y.position = NULL, xmin = NULL, xmax = NULL,
                         step.increase = 0, step.group.by = NULL, tip.length = 0.03,
                         tip.length.ref = c("data", "axis"),
                         bracket.nudge.y = 0, bracket.shorten = 0,
                         size = 0.3, linewidth = size, label.size = 3.88, family = "", vjust = 0,
                         coord.flip = FALSE,
                         orientation = c("horizontal", "vertical"),
                         ymin = NULL, ymax = NULL, x.position = NULL,
                         ...) {
  type <- match.arg(type)
  tip.length.ref <- match.arg(tip.length.ref)
  orientation <- match.arg(orientation)
  vertical <- identical(orientation, "vertical")
  if (vertical) {
    # A vertical bracket takes its span from the value (y) axis and its bar
    # position from the x axis. Accept the axis-appropriate argument names and
    # feed them through the same internal slots the horizontal path uses.
    if (coord.flip) {
      stop("`orientation = 'vertical'` cannot be combined with `coord.flip = TRUE`. ",
           "Use `orientation = 'vertical'` for a native vertical bracket on an ",
           "unflipped plot, or `coord.flip = TRUE` for a plot flipped with coord_flip().",
           call. = FALSE)
    }
    if (!is.null(ymin)) xmin <- ymin
    if (!is.null(ymax)) xmax <- ymax
    if (!is.null(x.position)) y.position <- x.position
  }
  data <- build_signif_data(
    data = data, label = label, y.position = y.position,
    xmin = xmin, xmax = xmax, step.increase = step.increase,
    bracket.nudge.y = bracket.nudge.y, bracket.shorten = bracket.shorten,
    step.group.by = step.group.by, vjust = vjust
  )
  mapping <- build_signif_mapping(mapping, data, orientation)
  ggplot2::layer(
    stat = stat, geom = GeomBracket, mapping = mapping, data = data,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      type = type,
      tip.length = tip.length, tip.length.ref = tip.length.ref,
      size = linewidth, label.size = label.size,
      family = family, na.rm = na.rm, coord.flip = coord.flip,
      orientation = orientation,
      ...
    )
  )
}


# Guess column to be used as significance labem
guess_signif_label_column <- function(data) {
  potential.label <- c(
    "label", "labels", "p.adj.signif", "p.adj", "padj",
    "p.signif", "p.format.signif", "p.value", "pval", "p.val", "p",
    "p.format", "p.adj.format"
  )
  res <- intersect(potential.label, colnames(data))
  if (length(res) > 0) {
    res <- res[1]
  } else {
    stop("label is missing")
  }
  res
}

build_signif_data <- function(data = NULL, label = NULL, y.position = NULL,
                              xmin = NULL, xmax = NULL, step.increase = 0,
                              bracket.nudge.y = 0, bracket.shorten = 0, step.group.by = NULL, vjust = 0) {
  add_step_increase <- function(data, step.increase) {
    comparisons.number <- 0:(nrow(data) - 1)
    step.increase <- step.increase * comparisons.number
    data <- data %>% mutate(step.increase = !!step.increase)
    data
  }
  if (is.null(data)) {
    data <- data.frame(
      label = label, y.position = y.position,
      xmin = xmin, xmax = xmax
    ) %>%
      mutate(vjust = !!vjust)
  } else {
    if (!is.null(label)) data <- data %>% mutate(label = !!label)
    if (!is.null(y.position)) data <- data %>% mutate(y.position = !!y.position)
    if (!is.null(xmin)) data <- data %>% mutate(xmin = !!xmin)
    if (!is.null(xmax)) data <- data %>% mutate(xmax = !!xmax)
    if (!identical(vjust, 0)) data <- data %>% mutate(vjust = !!vjust)
  }
  # add vjust column if doesn't exist
  if (!("vjust" %in% colnames(data))) data <- data %>% mutate(vjust = !!vjust)
  if (!("bracket.nudge.y" %in% colnames(data))) data <- data %>% mutate(bracket.nudge.y = !!bracket.nudge.y)
  if (!("bracket.shorten" %in% colnames(data))) data <- data %>% mutate(bracket.shorten = !!bracket.shorten)

  if (is.null(step.group.by)) {
    data <- data %>% add_step_increase(step.increase)
  } else {
    data <- data %>%
      dplyr::arrange(!!!syms(c(step.group.by, "y.position"))) %>%
      group_by(!!!syms(step.group.by)) %>%
      tidyr::nest() %>%
      dplyr::mutate(step.increase = purrr::map(data, add_step_increase, !!step.increase)) %>%
      dplyr::select(-data) %>%
      unnest(cols = "step.increase")
  }
  data
}


build_signif_mapping <- function(mapping, data, orientation = "horizontal") {
  if (is.null(mapping)) {
    # Check if required variables are present in data
    required.vars <- c("xmin", "xmax", "y.position")
    missing.required.vars <- setdiff(required.vars, colnames(data))
    if (length(missing.required.vars) > 0) {
      stop(
        "Required variables are missing in the data: ",
        paste(missing.required.vars, collapse = ", ")
      )
    }
    mapping <- ggplot2::aes()
  }
  if (is.null(mapping$label)) {
    label.col <- guess_signif_label_column(data)
    data$label <- data %>% dplyr::pull(!!label.col)
    mapping$label <- data$label
  }
  if (is.null(mapping$xmin)) mapping$xmin <- data$xmin
  if (is.null(mapping$xmax)) mapping$xmax <- data$xmax
  if (is.null(mapping$y.position)) mapping$y.position <- data$y.position
  if (is.null(mapping$group)) mapping$group <- seq_len(nrow(data))
  if (is.null(mapping$step.increase)) mapping$step.increase <- data$step.increase
  if (is.null(mapping$vjust)) mapping$vjust <- data$vjust
  if (is.null(mapping$bracket.nudge.y)) mapping$bracket.nudge.y <- data$bracket.nudge.y
  if (is.null(mapping$bracket.shorten)) mapping$bracket.shorten <- data$bracket.shorten
  # Anchor aesthetics that position the bracket for scale training and the
  # stat's required `x`/`y`. For a vertical bracket the roles swap: the bar sits
  # along x (from y.position, which holds the x position) and the span runs along
  # y (from xmin). Mapping them the horizontal way would send an x position into
  # the y scale and drop the bracket as out-of-range (#456).
  if (identical(orientation, "vertical")) {
    if (!"x" %in% names(mapping)) mapping$x <- mapping$y.position
    if (!"y" %in% names(mapping)) mapping$y <- mapping$xmin
  } else {
    if (!"x" %in% names(mapping)) mapping$x <- mapping$xmin
    if (!"y" %in% names(mapping)) mapping$y <- mapping$y.position
  }
  mapping
}


# Source: https://github.com/tidyverse/ggplot2/issues/2864
parse_as_expression <- function(text) {
  stopifnot(is.character(text))
  out <- vector("expression", length(text))
  for (i in seq_along(text)) {
    expr <- parse(text = text[[i]])
    out[[i]] <- if (length(expr) == 0) NA else expr[[1]]
  }
  out
}

context("test-geom-bracket-vertical")

# #456: geom_bracket() gains `orientation = "vertical"` to draw a native vertical
# bracket (bar spanning y, tips pointing left, rotated label) via ymin/ymax/
# x.position - useful for plots where the comparison runs along the y axis (e.g.
# Kaplan-Meier curves). The default orientation = "horizontal" is byte-identical.

df <- ToothGrowth
df$dose <- factor(df$dose)

.bracket <- function(p) {
  b <- ggplot2::ggplot_build(p)
  i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBracket"), logical(1)))[1]
  b$data[[i]]
}

test_that("default (horizontal) output is unchanged by the refactor", {
  # pin the exact segment coordinates of a representative horizontal bracket
  p <- ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "***")
  d <- .bracket(p)
  expect_equal(as.numeric(d$x),    c(1, 1, 2))
  expect_equal(as.numeric(d$xend), c(1, 2, 2))
  # tips go down from y.position = 30 by tip.length (0.03) * data range
  expect_equal(as.numeric(d$y[2:3]),    c(30, 30))
  expect_equal(as.numeric(d$yend[1:2]), c(30, 30))
  expect_lt(d$y[1], 30)          # left tip points down
  expect_lt(d$yend[3], 30)       # right tip points down
  # passing orientation = "horizontal" explicitly is identical to omitting it
  p2 <- ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "***",
                 orientation = "horizontal")
  expect_equal(.bracket(p2)[, c("x", "xend", "y", "yend")],
               d[, c("x", "xend", "y", "yend")])
})

test_that("vertical bracket swaps the geometry: bar spans y, tips point in -x", {
  p <- ggscatter(df, "len", "len") +
    geom_bracket(orientation = "vertical", ymin = 10, ymax = 25,
                 x.position = 30, label = "p<0.05")
  d <- .bracket(p)
  # the bar is vertical at x = 30, spanning y in [10, 25]
  expect_equal(d$y,    c(10, 10, 25))
  expect_equal(d$yend, c(10, 25, 25))
  expect_equal(d$x[2:3],    c(30, 30))
  expect_equal(d$xend[1:2], c(30, 30))
  # tips point left (toward smaller x)
  expect_lt(d$x[1], 30)
  expect_lt(d$xend[3], 30)
  expect_silent(suppressWarnings(ggplot2::ggplotGrob(p)))
})

test_that("vertical tip length scales to the x range (not the y range)", {
  # x range is len (~4.2..33.9); tip.length default 0.03 => tip ~ 0.03 * xrange
  p <- ggscatter(df, "len", "len") +
    geom_bracket(orientation = "vertical", ymin = 10, ymax = 25,
                 x.position = 30, label = "ns")
  d <- .bracket(p)
  b <- ggplot2::ggplot_build(p)
  xr <- diff(b$layout$panel_params[[1]]$x.range)
  tip <- 30 - d$x[1]
  # the tip is a small positive fraction of the x range, and not of the (equal-here) y
  expect_gt(tip, 0)
  expect_lt(tip, xr)               # sane
})

test_that("the bar position honors the x-scale transform (log-x)", {
  set.seed(1)
  d0 <- data.frame(x = 1:10, y = runif(10, 1, 9))
  p <- ggscatter(d0, "x", "y") +
    geom_bracket(orientation = "vertical", ymin = 3, ymax = 7, x.position = 8, label = "ns") +
    ggplot2::scale_x_log10()
  d <- .bracket(p)
  # the bar (the max x among the segment ends) lands at log10(8) in transformed space
  expect_equal(max(d$xend), log10(8), tolerance = 1e-6)
  expect_equal(max(d$x), log10(8), tolerance = 1e-6)
  # and CRUCIALLY the span y must stay at the data values 3 and 7 (a non-identity
  # x scale must not corrupt the y span, #456)
  expect_equal(sort(unique(round(c(d$y, d$yend), 6))), c(3, 7))
})

test_that("the y span is correct under every combination of x/y scale transforms", {
  set.seed(1)
  d0 <- data.frame(x = 1:10, y = runif(10, 1, 9))
  span_y <- function(...) {
    p <- ggscatter(d0, "x", "y") +
      geom_bracket(orientation = "vertical", ymin = 3, ymax = 7, x.position = 8, label = "ns")
    for (s in list(...)) p <- p + s
    d <- .bracket(suppressWarnings(p))
    sort(unique(round(c(d$y, d$yend), 4)))
  }
  expect_equal(span_y(),                                            c(3, 7))
  expect_equal(span_y(ggplot2::scale_y_log10()),                    round(c(log10(3), log10(7)), 4))
  expect_equal(span_y(ggplot2::scale_x_log10()),                    c(3, 7))            # log-x must NOT touch the y span
  expect_equal(span_y(ggplot2::scale_x_log10(), ggplot2::scale_y_log10()),
               round(c(log10(3), log10(7)), 4))                                        # no double transform
  expect_equal(span_y(ggplot2::scale_x_reverse()),                  c(3, 7))
  expect_equal(span_y(ggplot2::scale_x_sqrt()),                     c(3, 7))
})

test_that("orientation = 'vertical' errors if combined with coord.flip = TRUE", {
  expect_error(
    geom_bracket(orientation = "vertical", coord.flip = TRUE,
                 ymin = 1, ymax = 2, x.position = 1, label = "x"),
    "cannot be combined"
  )
})

test_that("a discrete position axis does not crash the vertical bracket", {
  # not the primary use case (vertical brackets target a continuous x axis),
  # but it must not error
  p <- ggboxplot(df, "supp", "len") +
    geom_bracket(orientation = "vertical", ymin = 10, ymax = 25,
                 x.position = 2.3, label = "*")
  expect_error(suppressWarnings(ggplot2::ggplotGrob(p)), NA)
})

test_that("vertical label sits to the right of the bar and is rotated -90", {
  .label_grob <- function(p) {
    g <- ggplot2::ggplotGrob(p)
    txt <- NULL
    find_txt <- function(gr) {
      if (inherits(gr, "text")) { txt <<- gr; return(invisible()) }
      if (!is.null(gr$children)) for (ch in gr$children) find_txt(ch)
      if (inherits(gr, "gTree") && !is.null(gr$grobs)) for (ch in gr$grobs) find_txt(ch)
    }
    for (pi in which(grepl("panel", g$layout$name))) find_txt(g$grobs[[pi]])
    txt
  }
  t <- .label_grob(ggscatter(df, "len", "len") +
    geom_bracket(orientation = "vertical", ymin = 10, ymax = 25,
                 x.position = 34, label = "V"))
  expect_false(is.null(t))
  expect_equal(t$rot, -90)                                # rotated
  expect_gt(as.numeric(t$x), 0.85)                        # to the right (near the bar)
  expect_equal(as.numeric(t$y), 0.45, tolerance = 0.08)   # centered on the span
})

test_that("expression labels work for vertical brackets", {
  p <- ggscatter(df, "len", "len") +
    geom_bracket(orientation = "vertical", ymin = 10, ymax = 25, x.position = 30,
                 label = "list(~italic(p)<=0.001)", type = "expression")
  expect_error(suppressWarnings(ggplot2::ggplotGrob(p)), NA)
})

context("test-stat_pvalue_manual")
data("ToothGrowth")
df <- ToothGrowth

test_with_fill <- function(inherit.aes = NULL) {
  stat.test <- df %>%
    group_by(dose) %>%
    rstatix::t_test(len ~ supp) %>%
    rstatix::add_xy_position(x = "dose")

  if (is.null(inherit.aes)) {
    layer <- stat_pvalue_manual(stat.test)
  } else {
    layer <- stat_pvalue_manual(stat.test, inherit.aes = inherit.aes)
  }

  p <- ggplot(df, aes(x = dose, y = len, color = supp, fill = supp)) +
    geom_boxplot() +
    layer
  ggplot2::ggplot_build(p)
}

test_that("fill aes works well by default", {
  expect_error(test_with_fill(inherit.aes = TRUE))
  expect_no_error(test_with_fill(inherit.aes = FALSE))
  expect_no_error(test_with_fill())
})

# tip.length.ref: "data" (default) vs "axis" (#362) -----------------------

# Helper: absolute tip depth (in y data units) of the bracket in a built plot
.bracket_tip_depth <- function(p) {
  b <- ggplot2::ggplot_build(p)
  for (d in b$data) {
    if (all(c("y", "yend") %in% names(d)) && "annotation" %in% names(d)) {
      return(max(c(d$y, d$yend)) - min(c(d$y, d$yend)))
    }
  }
  NA_real_
}

.tip_plot <- function(values, ylim, y.position, ref = "data") {
  d <- data.frame(
    treatment = c("a", "a", "a", "b", "b", "b"),
    value = values
  )
  stat <- data.frame(group1 = "a", group2 = "b", p = 0.01, y.position = y.position)
  spm <- if (is.null(ref)) {
    stat_pvalue_manual(stat, label = "p", tip.length = 0.2)  # rely on default arg
  } else {
    stat_pvalue_manual(stat, label = "p", tip.length = 0.2, tip.length.ref = ref)
  }
  ggbarplot(d, "treatment", "value") + spm + ggplot2::ylim(ylim)
}

test_that("tip.length.ref default 'data' is unchanged (tips scale with data range)", {
  small <- .tip_plot(c(1, 2, 4, 3, 5, 6),        c(0, 10),  9,  "data")
  big   <- .tip_plot(c(10, 20, 40, 30, 50, 60),  c(0, 100), 90, "data")
  # Historical behavior: tip = tip.length * data range. big's inputs are exactly
  # 10x small's, so its data-mode tip is exactly 10x as deep, and it is NOT a
  # fixed fraction of the axis (that is what 'axis' mode adds).
  tip_small <- .bracket_tip_depth(small)
  tip_big   <- .bracket_tip_depth(big)
  expect_equal(tip_big / tip_small, 10, tolerance = 1e-6)
  expect_false(isTRUE(all.equal(tip_small, 0.2 * 10)))
})

test_that("tip.length.ref = 'data' (default) is byte-identical to omitting the arg", {
  omitted  <- .tip_plot(c(1, 2, 4, 3, 5, 6), c(0, 10), 9, ref = NULL)
  explicit <- .tip_plot(c(1, 2, 4, 3, 5, 6), c(0, 10), 9, ref = "data")
  expect_equal(.bracket_tip_depth(omitted), .bracket_tip_depth(explicit),
               tolerance = 1e-9)
})

test_that("tip.length.ref = 'axis' gives constant fraction of the y-axis range (#362)", {
  small <- .tip_plot(c(1, 2, 4, 3, 5, 6),        c(0, 10),  9,  "axis")
  big   <- .tip_plot(c(10, 20, 40, 30, 50, 60),  c(0, 100), 90, "axis")
  # tip = 0.2 * axis range: 0.2*10 = 2 and 0.2*100 = 20 -> both 20% of their axis
  expect_equal(.bracket_tip_depth(small), 0.2 * 10, tolerance = 1e-6)
  expect_equal(.bracket_tip_depth(big),   0.2 * 100, tolerance = 1e-6)
})

test_that("geom_bracket honors tip.length.ref and validates it", {
  d <- data.frame(
    treatment = c("a", "a", "a", "b", "b", "b"),
    value = c(1, 2, 4, 3, 5, 6)
  )
  p_data <- ggbarplot(d, "treatment", "value") +
    geom_bracket(xmin = "a", xmax = "b", label = "x", y.position = 9,
                 tip.length = 0.2, tip.length.ref = "data") +
    ggplot2::ylim(0, 10)
  p_axis <- ggbarplot(d, "treatment", "value") +
    geom_bracket(xmin = "a", xmax = "b", label = "x", y.position = 9,
                 tip.length = 0.2, tip.length.ref = "axis") +
    ggplot2::ylim(0, 10)
  # here data range (<10) < axis range (10), so axis tips are deeper than data tips
  expect_gt(.bracket_tip_depth(p_axis), .bracket_tip_depth(p_data))
  expect_equal(.bracket_tip_depth(p_axis), 0.2 * 10, tolerance = 1e-6)
  expect_error(
    geom_bracket(xmin = "a", xmax = "b", label = "x", y.position = 9,
                 tip.length.ref = "bogus")
  )
})

# geom_bracket on a transformed (log) y scale (#342) --------------------------

.bracket_y_range <- function(p) {
  b <- ggplot2::ggplot_build(p)
  for (d in b$data) {
    if ("annotation" %in% names(d)) return(range(c(d$y, d$yend)))
  }
  NA_real_
}

test_that("geom_bracket places y.position in the scale's transformed space on log axes (#342)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  p_log <- ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "p") +
    ggplot2::scale_y_log10()
  yr <- .bracket_y_range(p_log)
  # y = 30 in data units must map to log10(30) ~ 1.477, not stay at 30
  expect_equal(max(yr), log10(30), tolerance = 1e-6)
  expect_lt(max(yr), 2)  # sanity: within the log panel, not at 30
  # full render succeeds (draw-time, not just build)
  expect_no_error(ggplot2::ggplot_gtable(p_log %>% ggplot2::ggplot_build()))
})

test_that("geom_bracket on a linear (identity) y scale is unchanged (#342 no-regression)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  p_lin <- ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "p")
  yr <- .bracket_y_range(p_lin)
  expect_equal(max(yr), 30, tolerance = 1e-9)  # y.position untouched on identity scale
})

test_that("stat_pvalue_manual brackets also respect a log y scale (#342)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)
  stat <- data.frame(group1 = "0.5", group2 = "1", p.signif = "*", y.position = 30)
  p <- ggboxplot(df, "dose", "len") +
    stat_pvalue_manual(stat, label = "p.signif") +
    ggplot2::scale_y_log10()
  expect_equal(max(.bracket_y_range(p)), log10(30), tolerance = 1e-6)
})

context("test-geom-bracket-horizontal")

# Characterization tests that PIN the horizontal geom_bracket() geometry (the
# long-standing default behavior, shared by stat_pvalue_manual() and
# stat_compare_means()). These lock the exact segment coordinates produced by
# StatBracket so that any future change which silently alters horizontal
# brackets - tip length, step increase, bracket.shorten, bracket.nudge.y, the
# log-scale (#342) or axis (#362) handling - fails here instead of shipping.

df <- ToothGrowth
df$dose <- factor(df$dose)

.seg <- function(p) {
  b <- ggplot2::ggplot_build(p)
  i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBracket"), logical(1)))[1]
  d <- b$data[[i]]
  data.frame(
    x = as.numeric(d$x), xend = as.numeric(d$xend),
    y = as.numeric(d$y), yend = as.numeric(d$yend),
    annotation = as.character(d$annotation), stringsAsFactors = FALSE
  )
}

test_that("single bracket with character xmin/xmax is unchanged", {
  d <- .seg(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "***"))
  expect_equal(d$x,    c(1, 1, 2))
  expect_equal(d$xend, c(1, 2, 2))
  expect_equal(d$y,    c(29.109, 30, 30), tolerance = 1e-4)
  expect_equal(d$yend, c(30, 30, 29.109), tolerance = 1e-4)
  expect_equal(d$annotation, rep("***", 3))
})

test_that("multiple brackets with step.increase are unchanged", {
  d <- .seg(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = c("0.5", "1"), xmax = c("1", "2"),
                 y.position = c(30, 35), label = c("***", "**"),
                 tip.length = 0.01, step.increase = 0.1))
  expect_equal(d$x,    c(1, 1, 2, 2, 2, 3))
  expect_equal(d$xend, c(1, 2, 2, 2, 3, 3))
  expect_equal(d$y,    c(29.692, 30, 30, 37.772, 38.08, 38.08), tolerance = 1e-3)
  expect_equal(d$yend, c(30, 30, 29.692, 38.08, 38.08, 37.772), tolerance = 1e-3)
})

test_that("per-tip tip.length vector is unchanged", {
  d <- .seg(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "*",
                 tip.length = c(0.2, 0.02)))
  # asymmetric tips: left tip is long (0.2), right tip is short (0.02)
  expect_equal(d$y,    c(24.06, 30, 30), tolerance = 1e-3)
  expect_equal(d$yend, c(30, 30, 29.406), tolerance = 1e-3)
})

test_that("bracket.shorten and bracket.nudge.y are unchanged", {
  d <- .seg(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "2", y.position = 30, label = "*",
                 bracket.shorten = 0.2, bracket.nudge.y = 2))
  # shorten pulls the ends in by 0.1 each (1->1.1, 3->2.9); nudge lifts y by 2
  expect_equal(d$x,    c(1.1, 1.1, 2.9))
  expect_equal(d$xend, c(1.1, 2.9, 2.9))
  expect_equal(d$y,    c(31.109, 32, 32), tolerance = 1e-3)
  expect_equal(d$yend, c(32, 32, 31.109), tolerance = 1e-3)
})

test_that("y positions land in transformed space on a log y scale (#342)", {
  d <- .seg(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "*") +
    ggplot2::scale_y_log10())
  # y.position = 30 -> log10(30) = 1.4771; tip goes down to ~1.4499
  expect_equal(d$y,    c(1.4499, 1.4771, 1.4771), tolerance = 1e-4)
  expect_equal(d$yend, c(1.4771, 1.4771, 1.4499), tolerance = 1e-4)
})

test_that("tip.length.ref = 'axis' uses the axis range, not the data range (#362)", {
  # with ylim(0, 60) the tip is 0.03 * 60 = 1.8 below y.position
  d <- .seg(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "*",
                 tip.length.ref = "axis") + ggplot2::ylim(0, 60))
  expect_equal(min(d$y), 30 - 0.03 * 60, tolerance = 1e-4)
})

test_that("numeric xmin/xmax on a continuous x axis are unchanged", {
  d <- .seg(ggscatter(df, "len", "len") +
    geom_bracket(xmin = 10, xmax = 20, y.position = 30, label = "ns"))
  expect_equal(d$x,    c(10, 10, 20))
  expect_equal(d$xend, c(10, 20, 20))
  expect_equal(d$y[2:3], c(30, 30))
})

test_that("stat_pvalue_manual() draws the same horizontal bracket segments", {
  stat.test <- compare_means(len ~ dose, df, method = "t.test")
  stat.test$y.position <- c(32, 35, 38)
  d <- .seg(ggboxplot(df, "dose", "len") +
    stat_pvalue_manual(stat.test, label = "p.signif"))
  expect_equal(nrow(d), 9)                       # 3 comparisons x 3 segments
  expect_equal(unique(d$x),    c(1, 2, 3))
  expect_equal(unique(d$xend), c(1, 2, 3))
  # the bars sit at the requested y positions
  expect_true(all(c(32, 35, 38) %in% round(d$y, 3)))
})

test_that("a plain horizontal geom_bracket() draws without warning", {
  expect_silent(
    ggplot2::ggplotGrob(
      ggboxplot(df, "dose", "len") +
        geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "*")
    )
  )
})

# The label is drawn in GeomBracket$draw_group (not in the built layer data), so
# these render-and-inspect tests pin its ANGLE and PANEL position - guarding the
# label control flow (horizontal vs coord.flip vs vertical branches).
.label_grob <- function(p) {
  g <- ggplot2::ggplotGrob(p)
  txt <- NULL
  find_txt <- function(gr) {
    if (inherits(gr, "text")) { txt <<- gr; return(invisible()) }
    if (!is.null(gr$children)) for (ch in gr$children) find_txt(ch)
    if (inherits(gr, "gTree") && !is.null(gr$grobs)) for (ch in gr$grobs) find_txt(ch)
  }
  for (pi in which(grepl("panel", g$layout$name))) find_txt(g$grobs[[pi]])
  txt
}

test_that("horizontal label is centered above the bracket, not rotated", {
  t <- .label_grob(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "H"))
  expect_false(is.null(t))
  expect_equal(t$rot, 0)                                  # horizontal: no rotation
  expect_equal(as.numeric(t$x), 0.344, tolerance = 0.03)  # centered over the span
  expect_equal(as.numeric(t$y), 0.845, tolerance = 0.03)  # above the bar
})

test_that("coord.flip label sits to the side and is rotated -90", {
  t <- .label_grob(ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 30, label = "C",
                 coord.flip = TRUE) + ggplot2::coord_flip())
  expect_false(is.null(t))
  expect_equal(t$rot, -90)
  expect_equal(as.numeric(t$x), 0.845, tolerance = 0.03)
  expect_equal(as.numeric(t$y), 0.344, tolerance = 0.03)
})

context("test-geom-bracket-tip-collapse")

# #631: geom_bracket() tips collapsed to an invisible flat line whenever the
# position axis was trained to a zero-width range - the classic case being a
# SINGLE bracket over a stat-computed y (geom_bar()/geom_histogram()), where the
# bar counts do not train the y scale by the time StatBracket runs, so the only
# y value is the bracket's own y.position. StatBracket now flags that case and
# GeomBracket$draw_group re-derives the tips against the fully-trained panel range
# at draw time (which knows coord_cartesian(ylim=) / scale limits). Normal
# brackets (mapped y, non-zero range) are byte-identical - the flag is FALSE and
# the draw path is untouched.

suppressPackageStartupMessages(library(ggplot2))

# Drawn segment grob (npc coords) for the bracket layer.
.bracket_segments <- function(p) {
  li <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBracket"), logical(1)))[1]
  lg <- ggplot2::layer_grob(p, li)[[1]]
  ch <- lg$children
  seg <- ch[[which(vapply(ch, function(g) inherits(g, "segments"), logical(1)))[1]]]
  list(y0 = as.numeric(seg$y0), y1 = as.numeric(seg$y1))
}

# Built (pre-transform) bracket-layer data.
.bracket_data <- function(p) {
  b <- suppressWarnings(suppressMessages(ggplot2::ggplot_build(p)))
  li <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBracket"), logical(1)))[1]
  b$data[[li]]
}

set.seed(1)
bard <- data.frame(Q3 = sample(c("First", "Second", "Third"), 90, replace = TRUE))

# ---- The fix: a single bracket over geom_bar() now draws visible tips ----

test_that("single bracket over geom_bar draws tips of length tip.length (npc)", {
  p <- ggplot(bard, aes(Q3)) + geom_bar() + coord_cartesian(ylim = c(0, 30)) +
    geom_bracket(xmin = "First", xmax = "Second", y.position = 26,
                 tip.length = 0.2, label = "**")
  s <- .bracket_segments(p)
  # rows: 1 = left tip, 2 = bar (flat), 3 = right tip. The bar npc is y at row 2.
  bar <- s$y1[2]
  expect_equal(s$y0[2], s$y1[2])                 # bar is flat
  # left tip drops from bar down by tip.length in npc (axis reference)
  expect_equal(bar - s$y0[1], 0.2, tolerance = 1e-6)
  # right tip drops from bar down by tip.length in npc
  expect_equal(bar - s$y1[3], 0.2, tolerance = 1e-6)
  expect_lt(s$y0[1], bar)                         # visibly below the bar
})

test_that("the fix also works with scale_y_continuous(limits=) and geom_histogram", {
  p1 <- ggplot(bard, aes(Q3)) + geom_bar() + scale_y_continuous(limits = c(0, 40)) +
    geom_bracket(xmin = "First", xmax = "Second", y.position = 30, tip.length = 0.15, label = "*")
  s1 <- .bracket_segments(p1)
  expect_equal(s1$y1[2] - s1$y0[1], 0.15, tolerance = 1e-6)

  dh <- data.frame(v = rnorm(300))
  p2 <- ggplot(dh, aes(v)) + geom_histogram(bins = 20) +
    geom_bracket(xmin = -1, xmax = 1, y.position = 30, tip.length = 0.1, label = "*")
  s2 <- .bracket_segments(p2)
  expect_equal(s2$y1[2] - s2$y0[1], 0.1, tolerance = 1e-6)
})

test_that("asymmetric tip.length is honored per side in the collapsed case", {
  p <- ggplot(bard, aes(Q3)) + geom_bar() + coord_cartesian(ylim = c(0, 30)) +
    geom_bracket(xmin = "First", xmax = "Second", y.position = 26,
                 tip.length = c(0.05, 0.2), label = "**")
  s <- .bracket_segments(p)
  bar <- s$y1[2]
  expect_equal(bar - s$y0[1], 0.05, tolerance = 1e-6)   # left
  expect_equal(bar - s$y1[3], 0.2, tolerance = 1e-6)    # right
})

test_that("tip.length = 0 stays flat even in the collapsed case (intent respected)", {
  p <- ggplot(bard, aes(Q3)) + geom_bar() + coord_cartesian(ylim = c(0, 30)) +
    geom_bracket(xmin = "First", xmax = "Second", y.position = 26,
                 tip.length = 0, label = "**")
  s <- .bracket_segments(p)
  expect_equal(s$y0[1], s$y1[2], tolerance = 1e-9)      # no tip
  expect_equal(s$y1[3], s$y1[2], tolerance = 1e-9)
})

# ---- Characterization: normal brackets are unchanged (flag FALSE) ----

test_that("StatBracket flags only the zero-width (collapsed) case", {
  # collapsed: single bracket over geom_bar -> zero-width y range -> flag TRUE
  d_bar <- .bracket_data(
    ggplot(bard, aes(Q3)) + geom_bar() +
      geom_bracket(xmin = "First", xmax = "Second", y.position = 26, label = "*")
  )
  expect_true(all(d_bar$.bracket.tip.zero.))

  # normal: mapped y with a real range -> flag FALSE
  df <- ToothGrowth; df$dose <- factor(df$dose)
  d_box <- .bracket_data(
    ggboxplot(df, "dose", "len") +
      geom_bracket(xmin = "0.5", xmax = "1", y.position = 35, tip.length = 0.03, label = "p")
  )
  expect_false(any(d_box$.bracket.tip.zero.))
})

test_that("normal bracket tips are computed in the data range as before (unchanged)", {
  df <- ToothGrowth; df$dose <- factor(df$dose)
  p <- ggboxplot(df, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 35, tip.length = 0.03, label = "p")
  d <- .bracket_data(p)
  # tips are baked into the built data (non-zero, computed against the data range);
  # row 1 y (left tip bottom) sits below the bar at 35, and the fallback did NOT run.
  expect_lt(d$y[1], 35)
  expect_equal(d$y[2], 35)
  expect_true(all(d$.bracket.tip.zero. == FALSE))
})

test_that("two brackets over geom_bar (already-working case) still render with tips", {
  p <- ggplot(bard, aes(Q3)) + geom_bar() +
    geom_bracket(xmin = "First", xmax = "Second", y.position = 19, tip.length = 0.2, label = "*") +
    geom_bracket(xmin = "First", xmax = "Third",  y.position = 22, tip.length = 0.2, label = "*")
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)), NA)
  # first (two-position) layer has a non-zero range -> not flagged
  d <- .bracket_data(p)
  expect_false(any(d$.bracket.tip.zero.))
})

test_that("stat_pvalue_manual (shared draw path) is unaffected and renders", {
  df <- ToothGrowth; df$dose <- factor(df$dose)
  stat.test <- data.frame(group1 = "0.5", group2 = "1", p.adj = 0.01, y.position = 35)
  p <- ggboxplot(df, "dose", "len") +
    stat_pvalue_manual(stat.test, label = "p.adj", tip.length = 0.03)
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)), NA)
  d <- .bracket_data(p)
  expect_false(any(d$.bracket.tip.zero.))   # mapped y, real range -> not flagged
})

# Regression test for #665: geom_bracket()/geom_pwc() use the `%||%` operator,
# which base R only provides since R 4.4, while DESCRIPTION allows R (>= 4.1).
# It must be imported from rlang so it resolves on older R too.

test_that("`%||%` is imported into the ggpubr namespace (#665)", {
  imports_env <- parent.env(asNamespace("ggpubr"))
  # revert-sensitive: this binding only exists because of `importFrom(rlang, "%||%")`
  expect_true("%||%" %in% ls(imports_env, all.names = TRUE))
  expect_error(get("%||%", envir = imports_env, inherits = FALSE), NA)
  # rlang genuinely exports it (the import source)
  expect_true("%||%" %in% getNamespaceExports("rlang"))
})

test_that("geom_bracket()/geom_pwc() (which use %||%) still render (#665)", {
  p1 <- ggboxplot(ToothGrowth, "dose", "len") +
    geom_bracket(xmin = "0.5", xmax = "1", y.position = 35, label = "p < 0.05")
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p1)))
  p2 <- ggboxplot(ToothGrowth, "dose", "len") + geom_pwc(method = "t_test")
  expect_no_error(suppressWarnings(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p2))))
})

# ::::::::::::::::::::::::::::::::::::::::::::::::::::
# General helper functions
# ::::::::::::::::::::::::::::::::::::::::::::::::::::

# Get random string
.random_string <- function(.length = 7) {
  index <- sample(1:26, .length)
  paste(letters[index], collapse = "")
}

# get file extension
.file_ext <- function(x) {
  pos <- regexpr("\\.([[:alnum:]]+)$", x)
  ifelse(pos > -1L, substring(x, pos + 1L), "")
}


# extract device name from file name
.device <- function(filename) {
  device <- .file_ext(filename)
  devices <- list(
    eps = grDevices::postscript, ps = grDevices::postscript,
    pdf = grDevices::pdf,
    svg = grDevices::svg,
    png = grDevices::png,
    jpg = grDevices::jpeg,
    jpeg = grDevices::jpeg,
    bmp = grDevices::bmp,
    tiff = grDevices::tiff
  )

  dev <- devices[[device]]
  if (is.null(dev)) {
    stop("Unknown graphics device '", device, "'", call. = FALSE)
  }

  dev
}


# Grouping data by variables
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.group_by <- function(data, grouping.vars) {
  . <- NULL # used in pipes

  # Grouping the data ==> list of data sets
  grouped.d <- data %>%
    df_nest_by(vars = grouping.vars)

  # Defining names for the list of data sets.
  # names = combination of the levels of the grouping variables
  .names.df <- grouped.d[, grouping.vars, drop = FALSE]
  .names <- .paste_colnames(.names.df, sep = ":") %>%
    apply(1, paste, collapse = ", ")
  names(grouped.d$data) <- .names
  return(grouped.d)
}


# Pasting the column name to each value of a dataframe
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.paste_colnames <- function(data, sep = ".") {
  data <- as.data.frame(data)

  if (ncol(data) == 1) {
    res <- paste0(colnames(data), ".", data[[1]])
    res <- data.frame(x = res, stringsAsFactors = FALSE)
    colnames(res) <- colnames(data)
    return(res)
  }

  res <- apply(
    data, 1,
    function(row, cname) {
      paste(cname, row, sep = sep)
    },
    colnames(data)
  ) %>%
    t() %>%
    as.data.frame(stringsAsFactors = FALSE)
  colnames(res) <- colnames(data)
  res
}


# Setting seed with possibility to restore initial random state
# :::::::::::::::::::::::::::::::::::::::::::::::
# Ref: https://github.com/florianhartig/DHARMa/blob/master/DHARMa/R/random.R
# seed numeric value to be used for setting seed
# return a function named restore_random_state()
# for restoring intitial random state
#
# Examples:
# seed <- set_seed(123)
# Restore back initial state
# seed$restore_random_state()
set_seed <- function(seed) {
  # Record current random state
  current <- mget(".Random.seed", envir = .GlobalEnv, ifnotfound = list(NULL))[[1]]
  restore_random_state <- function() {
    if (is.null(current)) {
      rm(".Random.seed", envir = .GlobalEnv)
    } else {
      assign(".Random.seed", current, envir = .GlobalEnv)
    }
  }

  # Setting seed
  if (is.numeric(seed)) set.seed(seed)
  # ensuring that RNG has been initialized
  if (is.null(current)) stats::runif(1)

  # Returning function for restoring state
  invisible(list(restore_random_state = restore_random_state))
}

#' @include utilities.R
NULL
#' Draw a Paragraph of Text
#' @description Draw a paragraph o text. Splits a long text into multiple lines
#'  (by inserting line breaks) so that the output will fit within the current
#'  viewport.
#' @param text the text to plot.
#' @param color font color, example: color = "black"
#' @param size font size, example: size = 12
#' @param face font face. Allowed values are one of "plain", "italic", "bold",
#'  "bold.italic".
#' @param family font family
#' @param lineheight Line height, example: lineheight = 2.
#' @author Laszlo Erdey <erdey.laszlo@@econ.unideb.hu>
#' @examples
#' # Density plot
#' density.p <- ggdensity(iris,
#'   x = "Sepal.Length",
#'   fill = "Species", palette = "jco"
#' )
#'
#' # Text plot
#' text <- paste("iris data set gives the measurements in cm",
#'   "of the variables sepal length and width",
#'   "and petal length and width, respectively,",
#'   "for 50 flowers from each of 3 species of iris.",
#'   "The species are Iris setosa, versicolor, and virginica.",
#'   sep = " "
#' )
#' text.p <- ggparagraph(text, face = "italic", size = 12)
#'
#' # Arrange the plots on the same page
#' ggarrange(density.p, text.p,
#'   ncol = 1, nrow = 2,
#'   heights = c(1, 0.3)
#' )
#'
#' @export
ggparagraph <- function(text, color = NULL, size = NULL, face = NULL,
                        family = NULL, lineheight = NULL) {
  style <- grid::gpar(
    col = color, fontsize = size,
    fontface = face, fontfamily = family,
    lineheight = lineheight
  )

  p <- grid::grob(text = text, cl = "splitText", gp = style)
  p <- as_ggplot(p)
  return(p)
}


# Helper functions
# Extracted from RGraphics package
# :::::::::::::::::::::::::::::::::::::::::::
splitString <- function(text) {
  strings <- strsplit(text, " ")[[1]]
  if (length(strings) < 2) {
    return(text)
  }
  newstring <- strings[1]
  linewidth <- grid::stringWidth(newstring)
  gapwidth <- grid::stringWidth(" ")
  availwidth <-
    grid::convertWidth(unit(1, "npc"),
      "in",
      valueOnly = TRUE
    )
  for (i in seq_along(strings)[-1]) {
    width <- grid::stringWidth(strings[i])
    if (grid::convertWidth(linewidth + gapwidth + width,
      "in",
      valueOnly = TRUE
    ) <
      availwidth) {
      sep <- " "
      linewidth <- linewidth + gapwidth + width
    } else {
      sep <- "\n"
      linewidth <- width
    }
    newstring <- paste(newstring, strings[i], sep = sep)
  }
  newstring
}

#' @method drawDetails splitText
#' @rdname ggparagraph
#' @param x a grid grob
#' @param recording a logical value indicating whether a grob is being added to
#'   the display list or redrawn from the display list.
#' @export
drawDetails.splitText <- function(x, recording) {
  grid::grid.text(splitString(x$text),
    x = 0, y = 1, just = c("left", "top")
  )
}

#' @include utilities.R
NULL
#' Scatter Plot with Marginal Histograms
#' @description Create a scatter plot with marginal histograms, density plots or
#'  box plots.
#' @inheritParams ggscatter
#' @param main.plot.size the width of the main plot. Default is 2.
#' @param margin.plot.size the width of the marginal plot. Default is 1.
#' @param group a grouping variable. Change points color and shape by groups if
#'  the options \code{color} and \code{shape} are missing. Should be also
#'  specified when you want to create a marginal box plot that is grouped.
#' @param margin.plot the type of the marginal plot. Default is "hist".
#' @param margin.params parameters to be applied to the marginal plots.
#' @param margin.ggtheme the theme of the marginal plot. Default is
#'  \code{\link[ggplot2:ggtheme]{theme_void}()}.
#' @param margin.space logical value. If TRUE, adds space between the main plot
#'  and the marginal plot.
#' @param bins Number of histogram bins. Defaults to 30. Pick a better value that
#'  fit to your data.
#' @param linetype line type ("solid", "dashed", ...)
#' @param legend specify the legend position. Allowed values include: "top",
#'  "bottom", "left", "right".
#' @param ggtheme the theme to be used for the scatter plot. Default is
#'  \code{\link{theme_pubr}()}.
#' @param print logical value. If \code{TRUE} (default), print the plot.
#' @param ... other arguments passed to the function \code{\link{ggscatter}()}.
#' @return an object of class \code{ggscatterhist}, which is a list of ggplots,
#'  including the following elements: \itemize{\item sp: main scatter plot;
#'  \item xplot: marginal x-axis plot; \item yplot: marginal y-axis plot. }.
#'
#'  User can modify each of plot before printing.
#'
#' @examples
#' # Basic scatter plot with marginal density plot
#' ggscatterhist(iris,
#'   x = "Sepal.Length", y = "Sepal.Width",
#'   color = "#00AFBB",
#'   margin.params = list(fill = "lightgray")
#' )
#'
#'
#' # Grouped data
#' ggscatterhist(
#'   iris,
#'   x = "Sepal.Length", y = "Sepal.Width",
#'   color = "Species", size = 3, alpha = 0.6,
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   margin.params = list(fill = "Species", color = "black", size = 0.2)
#' )
#'
#' # Use boxplot as marginal
#' ggscatterhist(
#'   iris,
#'   x = "Sepal.Length", y = "Sepal.Width",
#'   color = "Species", size = 3, alpha = 0.6,
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   margin.plot = "boxplot",
#'   ggtheme = theme_bw()
#' )
#'
#' # Add vertical and horizontal line to a ggscatterhist
#' plots <- ggscatterhist(iris, x = "Sepal.Length", y = "Sepal.Width", print = FALSE)
#' plots$sp <- plots$sp +
#'   geom_hline(yintercept = 3, linetype = "dashed", color = "blue") +
#'   geom_vline(xintercept = 6, linetype = "dashed", color = "red")
#' plots
#'
#' @export
ggscatterhist <- function(
  data, x, y, group = NULL,
  color = "black", fill = NA, palette = NULL,
  shape = 19, size = 2,
  linetype = "solid", bins = 30,
  margin.plot = c("density", "histogram", "boxplot"),
  margin.params = list(),
  margin.ggtheme = theme_void(),
  margin.space = FALSE,
  main.plot.size = 2,
  margin.plot.size = 1,
  title = NULL, xlab = NULL, ylab = NULL,
  legend = "top", ggtheme = theme_pubr(),
  print = TRUE, ...
) {
  if (!has_cowplot_v0.9()) {
    warning("Install the latest developmental version of cowplot on github ",
      "to fully use all the features of ggscatterhist",
      call. = FALSE
    )
    # margin.space = TRUE
  }

  margin.plot <- match.arg(margin.plot)
  margin.params <- .check_margin_params(
    margin.params, data, color, fill, linetype
  )
  if (margin.plot == "histogram") {
    if (is.null(margin.params$position)) {
      margin.params$position <- "identity"
    }
    margin.params <- margin.params %>%
      .add_item(bins = bins)
  }

  if (!is.null(group)) {
    if (missing(color)) color <- group
    if (missing(shape)) shape <- group
  }

  . <- NULL

  sp <- ggscatter(
    data, x, y,
    color = color, fill = fill,
    palette = palette,
    shape = shape, size = size,
    xlab = xlab, ylab = ylab,
    ggtheme = ggtheme, title = title,
    legend = legend,
    ...
  )

  # Type of graphics to be added in the margins
  geomfunc <- switch(margin.plot,
    histogram = geom_histogram,
    density = geom_density,
    boxplot = geom_boxplot,
    geom_histogram
  )

  # Define the x and the y variables depending on
  # the geometry used in margins
  if (margin.plot %in% c("density", "histogram")) {
    xplot.x <- x
    xplot.y <- NULL
    yplot.x <- y
    yplot.y <- NULL
  } else if (margin.plot %in% c("boxplot")) {
    if (is.null(group)) {
      data <- data %>%
        mutate(.xgroupx. = factor(1))
      group <- ".xgroupx."
    }
    xplot.x <- group
    xplot.y <- x
    yplot.x <- group
    yplot.y <- y
  }

  # Create the different marginal plot
  xplot <- ggplot() + margin.params %>%
    .add_item(
      geomfunc = geomfunc, data = data,
      x = xplot.x, y = xplot.y, alpha = 0.7
    ) %>%
    do.call(geom_exec, .)
  xplot <- set_palette(xplot, palette)

  yplot <- ggplot() + margin.params %>%
    .add_item(
      geomfunc = geomfunc, data = data,
      x = yplot.x, y = yplot.y, alpha = 0.7
    ) %>%
    do.call(geom_exec, .)
  yplot <- set_palette(yplot, palette)

  # Align the marginal plot axes with the main scatter plot's data ranges.
  # The margins are built from the raw data, so any transformation that changes
  # the scatter limits (e.g. ellipse = TRUE, position jitter, explicit
  # xlim/ylim) was not reflected in the margins, making them misaligned with the
  # scatter (#220, #420). Anchor each margin's DATA axis to the scatter's panel
  # range: set that scale's expansion to zero (the scatter range already
  # includes expansion) and zoom with coord_*(). The other (count/density) axis
  # keeps its default expansion. Only linear axes are aligned (see
  # .get_scatter_ranges): forcing a transformed scatter range onto a raw-data
  # margin would push it off-screen.
  sp.ranges <- .get_scatter_ranges(sp)
  # Flip the marginal plots
  if (margin.plot %in% c("density", "histogram")) {
    if (sp.ranges$x_identity && !is.null(sp.ranges$x)) {
      xplot <- xplot +
        scale_x_continuous(expand = expansion(0)) +
        coord_cartesian(xlim = sp.ranges$x)
    }
    if (sp.ranges$y_identity && !is.null(sp.ranges$y)) {
      yplot <- yplot +
        scale_x_continuous(expand = expansion(0)) +
        coord_flip(xlim = sp.ranges$y)
    } else {
      yplot <- yplot + coord_flip()
    }
  } else if (margin.plot %in% c("boxplot")) {
    xplot <- xplot + coord_flip()
  }

  # Cleaning the plot
  .legend <- get_legend(sp)
  sp <- sp + theme(plot.margin = grid::unit(c(0, 0, 0.25, 0.25), "cm"))
  xplot <- xplot + margin.ggtheme + clean_theme() + rremove("legend") +
    theme(plot.margin = grid::unit(c(0, 0, 0, 0), "cm"))
  yplot <- yplot + margin.ggtheme + clean_theme() + rremove("legend") +
    theme(plot.margin = grid::unit(c(0, 0, 0, 0), "cm"))

  plots <- list(sp = sp, xplot = xplot, yplot = yplot)
  class(plots) <- c("ggscatterhist", "list")
  if (print) {
    res <- print(
      plots,
      margin.space = margin.space,
      main.plot.size = main.plot.size, margin.plot.size = margin.plot.size,
      title = title, legend = legend
    )
  }
  invisible(plots)
}

#' @method print ggscatterhist
#' @param x an object of class \code{ggscatterhist}.
#' @rdname ggscatterhist
#' @export
print.ggscatterhist <- function(x, margin.space = FALSE, main.plot.size = 2,
                                margin.plot.size = 1, title = NULL, legend = "top", ...) {
  sp <- x$sp
  xplot <- x$xplot
  yplot <- x$yplot
  .legend <- get_legend(sp)

  if (margin.space) {
    common.legend <- FALSE
    if (!is.null(.legend)) common.legend <- TRUE
    sp <- sp + theme(
      plot.title = element_blank(),
      plot.subtitle = element_blank()
    )

    fig <- ggarrange(
      xplot, NULL, sp, yplot,
      ncol = 2, nrow = 2,
      align = "hv", widths = c(main.plot.size, margin.plot.size),
      heights = c(margin.plot.size, main.plot.size),
      common.legend = common.legend, legend = legend
    )
    if (!is.null(title)) {
      fig <- annotate_figure(
        fig,
        top = text_grob(title, color = "black", size = 13, face = "bold")
      )
    }
  } else {
    if (!is.null(title)) sp <- sp + ggtitle(title)
    fig <- .insert_xaxis_grob(sp, xplot, grid::unit(margin.plot.size / 5, "null"), position = "top")
    fig <- .insert_yaxis_grob(fig, yplot, grid::unit(margin.plot.size / 5, "null"), position = "right")
    fig <- cowplot::ggdraw(fig)
  }
  print(fig)
  invisible(fig)
}


has_cowplot_v0.9 <- function() {
  vv <- as.character(utils::packageVersion("cowplot"))
  cc <- utils::compareVersion(vv, "0.8.0.8") > 0
  cc
}

# Extract the final x and y coordinate ranges (including expansion) of a built
# scatter plot, plus whether each axis uses an identity (linear) transformation.
# Used to align the marginal plots in ggscatterhist() (#220, #420). Alignment is
# applied only to linear axes: a transformed scatter axis (log, sqrt, ...) lives
# in different units than the raw-data margins, so forcing its range onto the
# margin would push the margin off-screen. Falls back gracefully across ggplot2
# panel_params / scale layouts.
.get_scatter_ranges <- function(sp) {
  b <- ggplot2::ggplot_build(sp)
  pp <- b$layout$panel_params[[1]]
  axis_range <- function(axis) {
    r <- pp[[paste0(axis, ".range")]]
    if (is.null(r) && !is.null(pp[[axis]])) r <- pp[[axis]]$continuous_range
    r
  }
  axis_is_identity <- function(axis) {
    sc <- b$layout[[paste0("panel_scales_", axis)]][[1]]
    if (is.null(sc)) return(TRUE)
    tr <- tryCatch(sc$get_transformation(), error = function(e) NULL)
    if (is.null(tr)) tr <- tryCatch(sc$trans, error = function(e) NULL)
    # Treat "can't determine" as linear (the common case); a real transform
    # reports its name (e.g. "log-10", "sqrt").
    is.null(tr) || is.null(tr$name) || identical(tr$name, "identity")
  }
  list(
    x = axis_range("x"), y = axis_range("y"),
    x_identity = axis_is_identity("x"), y_identity = axis_is_identity("y")
  )
}

.check_margin_params <- function(params, data, color = "black", fill = NA, linetype = "solid") {
  if (is.null(params$color)) {
    if (color %in% colnames(data)) {
      col.val <- dplyr::pull(data, color)
      if (!is.factor(col.val)) {
        params$color <- "black"
      } else {
        params$color <- color
      }
    }
  }

  if (is.null(params$fill)) {
    if (fill %in% colnames(data)) {
      fill.val <- .select_vec(data, fill)
      if (!is.factor(fill.val)) {
        params$fill <- "black"
      } else {
        params$fill <- fill
      }
    }
  }
  if (is.null(params$linetype)) params$linetype <- linetype
  params
}

# Helper functions to insert marginal plots
# ::::::::::::::::::::::::::::::::::::::::::::::::::::
# Use cowplot::insert_xaxis_grob and cowplot::insert_yaxis_grob,
# when 0.9 stable version released

.insert_xaxis_grob <- function(
  plot, grob, height = grid::unit(0.2, "null"),
  position = c("top", "bottom")
) {
  if (inherits(grob, "ggplot")) {
    grob <- .get_panel(grob)
  }

  gt <- .plot_to_gtable(plot)
  pp <- gt$layout[gt$layout$name == "panel", ]

  if (position[1] == "top") {
    g <- gtable::gtable_add_rows(gt, height, pp$t - 1)
    g <- gtable::gtable_add_grob(
      g, grob, pp$t, pp$l, pp$t,
      pp$r,
      clip = "inherit", name = "xaxis-grob-t"
    )
  } else {
    g <- gtable::gtable_add_rows(gt, height, pp$b)
    g <- gtable::gtable_add_grob(
      g, grob, pp$b + 1, pp$l,
      pp$b + 1, pp$r,
      clip = "inherit", name = "xaxis-grob-b"
    )
  }
}

.insert_yaxis_grob <- function(
  plot, grob, width = grid::unit(0.2, "null"),
  position = c("right", "left")
) {
  if (inherits(grob, "ggplot")) {
    grob <- .get_panel(grob)
  }

  gt <- .plot_to_gtable(plot)
  pp <- gt$layout[gt$layout$name == "panel", ]

  if (position[1] == "right") {
    g <- gtable::gtable_add_cols(gt, width, pp$r)
    g <- gtable::gtable_add_grob(
      g, grob, pp$t, pp$r + 1,
      pp$b, pp$r + 1,
      clip = "inherit", name = "yaxis-grob-r"
    )
  } else {
    g <- gtable::gtable_add_cols(gt, width, pp$l - 1)
    g <- gtable::gtable_add_grob(
      g, grob, pp$t, pp$l, pp$b,
      pp$l,
      clip = "inherit", name = "yaxis-grob-l"
    )
  }
}


.plot_to_gtable <- function(plot) {
  if (inherits(plot, "gtable")) {
    return(plot)
  } else {
    ggplot2::ggplotGrob(plot)
  }
}

.get_panel <- function(plot) {
  gt <- .plot_to_gtable(plot)
  panelIndex <- which(gt$layout$name == "panel")
  panel <- gt$grobs[[panelIndex]]
}

context("test-ggscatterhist")

# Regression tests for #220 / #420: the marginal plots were built from the raw
# data, so transformations that change the scatter limits (ellipse, position
# jitter, explicit limits) were not reflected in the margins, leaving them
# misaligned with the scatter. The margins now inherit the scatter's ranges.

panel_x_range <- function(p) {
  pp <- ggplot2::ggplot_build(p)$layout$panel_params[[1]]
  r <- pp[["x.range"]]
  if (is.null(r)) r <- pp[["x"]]$continuous_range
  r
}
panel_y_range <- function(p) {
  pp <- ggplot2::ggplot_build(p)$layout$panel_params[[1]]
  r <- pp[["y.range"]]
  if (is.null(r)) r <- pp[["y"]]$continuous_range
  r
}

test_that("ggscatterhist marginal plots inherit the scatter's ranges with jitter (#420)", {
  set.seed(1)
  res <- ggscatterhist(
    mpg, x = "displ", y = "hwy",
    position = ggplot2::position_jitter(width = 5, height = 5, seed = 1),
    print = FALSE
  )
  sp.x <- panel_x_range(res$sp)
  sp.y <- panel_y_range(res$sp)
  # xplot (top margin) x-axis matches the scatter x-axis
  expect_equal(panel_x_range(res$xplot), sp.x, tolerance = 1e-6)
  # yplot is coord_flip()'d, so its y-axis (post-flip) matches the scatter y-axis
  expect_equal(panel_y_range(res$yplot), sp.y, tolerance = 1e-6)
  # the jittered scatter range is much wider than the raw data range: the fix
  # must have widened the margin beyond the raw displ range (1.6-7.0)
  expect_lt(sp.x[1], 1.6)
  expect_gt(sp.x[2], 7.0)
})

test_that("ggscatterhist marginal plots inherit the scatter's ranges with ellipse (#220)", {
  res <- ggscatterhist(
    iris, x = "Sepal.Length", y = "Sepal.Width",
    color = "Species", ellipse = TRUE, print = FALSE
  )
  expect_equal(panel_x_range(res$xplot), panel_x_range(res$sp), tolerance = 1e-6)
  expect_equal(panel_y_range(res$yplot), panel_y_range(res$sp), tolerance = 1e-6)
})

test_that("ggscatterhist default still renders and margins align (#220/#420)", {
  res <- ggscatterhist(iris, x = "Sepal.Length", y = "Sepal.Width", print = FALSE)
  expect_s3_class(res, "ggscatterhist")
  expect_equal(panel_x_range(res$xplot), panel_x_range(res$sp), tolerance = 1e-6)
  expect_equal(panel_y_range(res$yplot), panel_y_range(res$sp), tolerance = 1e-6)
  expect_no_error(print(res))
})

test_that("ggscatterhist old-cowplot warning does not append the call flag", {
  local_mocked_bindings(has_cowplot_v0.9 = function() FALSE)
  warning_message <- NULL
  withCallingHandlers(
    res <- ggscatterhist(iris, x = "Sepal.Length", y = "Sepal.Width", print = FALSE),
    warning = function(w) {
      warning_message <<- conditionMessage(w)
      invokeRestart("muffleWarning")
    }
  )
  expect_s3_class(res, "ggscatterhist")
  expect_match(warning_message, "features of ggscatterhist$")
})

test_that("ggscatterhist does NOT force a transformed (log) scatter range onto the raw-data margin (#220/#420)", {
  # A log-scaled scatter axis lives in different units than the raw-data margin.
  # The margin must stay in linear data units (visible), NOT be forced to the
  # scatter's log range (which would push it off-screen). Regression guard.
  res <- ggscatterhist(mpg, x = "displ", y = "hwy", xscale = "log10", print = FALSE)
  xplot.range <- panel_x_range(res$xplot)
  # margin still spans (roughly) the raw displ range 1.6..7.0, not the log range ~0.2..0.9
  expect_gt(xplot.range[2], 5)
  expect_gt(xplot.range[1], 1)
  expect_false(isTRUE(all.equal(xplot.range, panel_x_range(res$sp))))
  expect_no_error(print(res))
})

test_that("ggscatterhist preserves the marginal density-axis expansion (no default change, #220/#420)", {
  # Aligning the DATA axis must not remove the count/density-axis expansion:
  # the density peak should not sit flush against the panel edge.
  res <- ggscatterhist(iris, x = "Sepal.Length", y = "Sepal.Width", print = FALSE)
  dens.range <- panel_y_range(res$xplot)   # xplot's y = density magnitude
  built <- ggplot2::ggplot_build(res$xplot)$data[[1]]
  expect_lt(dens.range[1], 0)              # bottom expansion below 0 retained
  expect_gt(dens.range[2], max(built$y))   # top gap above the peak retained
})

#'@include utilities.R ggpar.R ggpie.R
NULL
#'Donut chart
#'@description Create a donut chart.
#'@inheritParams ggboxplot
#'@inheritParams ggplot2::geom_errorbar
#'@param x variable containing values for drawing.
#'@param label variable specifying the label of each slice.
#'@param lab.pos character specifying the position for labels. Allowed values
#'  are "out" (for outside) or "in" (for inside).
#'@param lab.adjust numeric value, used to adjust label position when lab.pos =
#'  "in". Increase or decrease this value to see the effect.
#'@param lab.font a vector of length 3 indicating respectively the size (e.g.:
#'  14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and the
#'  color (e.g.: "red") of label font. For example \emph{lab.font= c(4, "bold",
#'  "red")}.
#'@param font.family character vector specifying font family.
#'@param color,fill outline and fill colors.
#'@param ... other arguments to be passed to be passed to ggpar().
#'@details The plot can be easily customized using the function ggpar(). Read
#'  ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
#'  ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
#'  = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
#'  and position: legend = "right" \item plot orientation : orientation =
#'  c("vertical", "horizontal", "reverse") }
#'@seealso \code{\link{ggpar}}, \code{\link{ggpie}}
#' @examples
#'
#' # Data: Create some data
#' # +++++++++++++++++++++++++++++++
#'
#' df <- data.frame(
#'  group = c("Male", "Female", "Child"),
#'   value = c(25, 25, 50))
#'
#' head(df)
#'
#'
#' # Basic pie charts
#' # ++++++++++++++++++++++++++++++++
#'
#' ggdonutchart(df, "value", label = "group")
#'
#'
#' # Change color
#' # ++++++++++++++++++++++++++++++++
#'
#' # Change fill color by group
#' # set line color to white
#' # Use custom color palette
#'  ggdonutchart(df, "value", label = "group",
#'       fill = "group", color = "white",
#'        palette = c("#00AFBB", "#E7B800", "#FC4E07") )
#'
#'
#' # Change label
#' # ++++++++++++++++++++++++++++++++
#'
#' # Show group names and value as labels
#' labs <- paste0(df$group, " (", df$value, "%)")
#' ggdonutchart(df, "value", label = labs,
#'    fill = "group", color = "white",
#'    palette = c("#00AFBB", "#E7B800", "#FC4E07"))
#'
#' # Change the position and font color of labels
#' ggdonutchart(df, "value", label = labs,
#'    lab.pos = "in", lab.font = "white",
#'    fill = "group", color = "white",
#'    palette = c("#00AFBB", "#E7B800", "#FC4E07"))
#'
#'
#'
#'@export
ggdonutchart <- function(
  data, x, label = x, lab.pos = c("out", "in"), lab.adjust = 0,
  lab.font = c(4, "bold", "black"), font.family = "",
  color = "black", fill = "white", palette = NULL,
  size = NULL, ggtheme = theme_pubr(), ...
  )
  {

  lab.pos <- match.arg(lab.pos)
  lab.font <- .parse_font(lab.font) %>%
    .check_pie_labfont()

  # We should order the data in desc order. Because,
  # in stacked bar plot the order of factor levels are reversed
  # Very important to have the label in the right place
  group.vars <- intersect(c(fill, color), names(data))
  if(length(group.vars) != 0){
    group.val <- dplyr::pull(data, group.vars[1])
    data <- data %>% dplyr::arrange(dplyr::desc(group.val))
  }

  # Label y coordinates when placed inside slices
  .x <- dplyr::pull(data, x)
  data <- data %>%
    dplyr::mutate(
      .lab.ypos. = cumsum(.x) -0.5*.x -lab.adjust
    )

  if(length(label) > 1 & length(label) != nrow(data))
    stop("label should be of the same length as data")
  else if(length(label) > 1){
    # 1. Add label column
      data <- data %>%
        dplyr::mutate(.label. = label)
      label <- ".label."
  }

  p <- ggplot(data, create_aes(list(x = 2, y = x))) +
    geom_exec(
      geom_bar, data,  stat = "identity",
      fill = fill, color = color, size = size
      )

  p <- ggpar(
    p, palette = palette,
    ggtheme = ggtheme,
    font.family = font.family, ...
    ) +
    coord_polar(
      theta = "y", start = 0
      ) +
    ggtheme + .remove_axis()

  # Annotate pie slice
  #:::::::::::::::::::::::::::::::::::
  if(!is.null(label)){
    # Label each slice at the middle of the slice
    if(lab.pos == "out"){
      p <- p + scale_y_continuous(
        breaks = cumsum(.x) - .x/2,
        labels = dplyr::pull(data, label)
      )
    }
    # Compute the cumulative sum as label ypos
    if(lab.pos == "in"){
     p <- p + geom_text(
       create_aes(list(y = ".lab.ypos.", label = label)),
       size = lab.font$size, fontface = lab.font$face,
       colour = lab.font$color, family = font.family
      )+
       clean_theme()
    }

  }

  p + xlim(0.5, 2.5)
}

#' @include utilities.R utilities_label.R utils_stat_test_label.R p_format_utils.R
NULL
#' Add Mean Comparison P-values to a ggplot
#' @description Add mean comparison p-values to a ggplot, such as box plots, dot
#'  plots and stripcharts.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @inheritParams compare_means
#' @param method a character string indicating which method to be used for
#'  comparing means.
#' @param method.args a list of additional arguments used for the test method.
#'  For example one might use \code{method.args = list(alternative = "greater")}
#'  for wilcoxon test.
#' @param id optional character string naming a column that identifies matched
#'  subjects for a \strong{paired} comparison (\code{paired = TRUE}, with
#'  \code{method = "t.test"} or \code{"wilcox.test"}, and without
#'  \code{comparisons}). Without it the paired test pairs observations by row
#'  order, so the p-value is wrong when the data are not sorted so that the
#'  compared groups align by subject. Providing \code{id} pairs the observations
#'  by subject id instead (row-order independent); see \code{\link{compare_means}}.
#' @param comparisons A list of length-2 vectors. The entries in the vector are
#'  either the names of 2 values on the x-axis or the 2 integers that correspond
#'  to the index of the groups of interest, to be compared.
#'
#'  Note: when \code{comparisons} is specified, each pairwise test is computed
#'  independently and the displayed p-values are \strong{not} adjusted for
#'  multiple comparisons. For p-values corrected for multiple testing, use
#'  \code{\link{geom_pwc}()}, or \code{\link{stat_pvalue_manual}()} together with
#'  \code{\link{compare_means}(..., p.adjust.method = )}.
#' @param hide.ns logical value. If TRUE, hide ns symbol when displaying
#'  significance levels.
#' @param label character string specifying label type. Allowed values include
#'  \code{"p.signif"} (shows the significance levels), \code{"p.format"} (shows
#'  the formatted p-value), and \code{"p.format.signif"} (shows the formatted
#'  p-value followed by significance stars, e.g., "p = 0.01 **").
#' @param p.format.style character string specifying the p-value formatting style.
#'  One of: \code{"default"} (backward compatible, uses scientific notation),
#'  \code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
#'  \code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
#'  (GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
#'  See \code{\link{list_p_format_styles}} for details.
#' @param p.digits integer specifying the number of decimal places for p-values.
#'  If provided, overrides the style default.
#' @param p.leading.zero logical indicating whether to include leading zero before
#'  decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.
#' @param p.min.threshold numeric specifying the minimum p-value to display exactly.
#'  Values below this threshold are shown as "< threshold". If provided, overrides
#'  the style default.
#' @param p.decimal.mark character string to use as the decimal mark. If NULL,
#'  uses \code{getOption("OutDec")}.
#' @param signif.cutoffs numeric vector of p-value cutoffs in descending order
#'  for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
#'  means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
#'  If \code{use.four.stars = TRUE}, can include a fourth level.
#'  Default is NULL, which uses the package defaults.
#' @param signif.symbols character vector of symbols corresponding to
#'  \code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
#'  (and "****" if \code{use.four.stars = TRUE}).
#' @param ns.symbol character string for non-significant results. Default is "ns".
#'  Use "" (empty string) to show nothing.
#' @param use.four.stars logical. If TRUE, allows four stars (****) for the most
#'  significant level. Default is FALSE.
#' @param show.signif logical. If TRUE (default), shows significance symbols when
#'  using \code{label = "p.format.signif"}. If FALSE, falls back to showing only
#'  the p-value (equivalent to \code{label = "p.format"}) with a warning.
#' @param label.sep a character string to separate the terms. Default is ", ", to
#'  separate the test method name and the p-value.
#' @param label.x.npc,label.y.npc can be \code{numeric} or \code{character}
#'  vector of the same length as the number of groups and/or panels. If too
#'  short they will be recycled. \itemize{ \item If \code{numeric}, value should
#'  be between 0 and 1. Coordinates to be used for positioning the label,
#'  expressed in "normalized parent coordinates". \item If \code{character},
#'  allowed values include: i) one of c('right', 'left', 'center', 'centre',
#'  'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
#'  'middle') for y-axis.}
#' @param vjust move the text up or down relative to the bracket.
#' @param tip.length numeric vector with the fraction that the bracket tips go
#'  down to indicate the precise column. Default is 0.03. Can be of
#'  same length as the number of comparisons to adjust specifically the tip
#'  length of each comparison. For example tip.length = c(0.01, 0.03).
#'
#'  If too short they will be recycled.
#'
#'  Note: when \code{comparisons} is set, brackets are drawn via
#'  \code{ggsignif::geom_signif()} and \code{tip.length} is a fraction of the
#'  trained data range, so the absolute tip length scales with your data (plots
#'  with different scales get different tip lengths). To obtain visually constant
#'  tip lengths across plots, compute the tests with \code{\link{compare_means}()}
#'  and draw them with \code{\link{stat_pvalue_manual}(..., tip.length.ref = "axis")}
#'  (which makes \code{tip.length} a fraction of the y-axis range). Passing
#'  \code{tip.length.ref} directly to \code{stat_compare_means()} has no effect
#'  and emits an informative message.
#' @param label.x,label.y \code{numeric} Coordinates (in data units) to be used
#'  for absolute positioning of the label. If too short they will be recycled.
#' @param bracket.size Width of the lines of the bracket.
#' @param step.increase numeric vector with the increase in fraction of total
#'  height for every additional comparison to minimize overlap.
#' @param ... other arguments to pass to \code{\link[ggplot2]{geom_text}} or
#'  \code{\link[ggplot2:geom_text]{geom_label}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning. If
#'  TRUE silently removes missing values.
#' @seealso \code{\link{compare_means}}
#' @details For grouped plots, if one or more subsets do not contain enough levels
#'  to identify the requested comparison, those subsets are skipped and valid
#'  subsets are still tested. This avoids fatal layer failures in sparse grouped
#'  data settings (Issue #663).
#' @examples
#' # Load data
#' data("ToothGrowth")
#' head(ToothGrowth)
#'
#' # Two independent groups
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' p <- ggboxplot(ToothGrowth,
#'   x = "supp", y = "len",
#'   color = "supp", palette = "npg", add = "jitter"
#' )
#'
#' #  Add p-value
#' p + stat_compare_means()
#' # Change method
#' p + stat_compare_means(method = "t.test")
#'
#' # Paired samples
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' ggpaired(ToothGrowth,
#'   x = "supp", y = "len",
#'   color = "supp", line.color = "gray", line.size = 0.4,
#'   palette = "npg"
#' ) +
#'   stat_compare_means(paired = TRUE)
#'
#' # Paired samples matched by a subject id column (row-order independent):
#' # `id` gives the correct p-value even when the rows are not in subject order.
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' tg <- ToothGrowth
#' tg$id <- c(1:30, 30:1) # subjects, with the second group in reverse order
#' ggpaired(tg, x = "supp", y = "len", color = "supp",
#'   line.color = "gray", palette = "npg") +
#'   stat_compare_means(paired = TRUE, id = "id")
#'
#' # More than two groups
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' # Pairwise comparisons: Specify the comparisons you want
#' my_comparisons <- list(c("0.5", "1"), c("1", "2"), c("0.5", "2"))
#' ggboxplot(ToothGrowth,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "npg"
#' ) +
#'   # Add pairwise comparisons p-value
#'   stat_compare_means(comparisons = my_comparisons, label.y = c(29, 35, 40)) +
#'   stat_compare_means(label.y = 45) # Add global Anova p-value
#'
#' # Multiple pairwise test against a reference group
#' ggboxplot(ToothGrowth,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "npg"
#' ) +
#'   stat_compare_means(method = "anova", label.y = 40) + # Add global p-value
#'   stat_compare_means(aes(label = after_stat(p.signif)),
#'     method = "t.test", ref.group = "0.5"
#'   )
#'
#' # Multiple grouping variables
#' # :::::::::::::::::::::::::::::::::::::::::::::::::
#' # Box plot facetted by "dose"
#' p <- ggboxplot(ToothGrowth,
#'   x = "supp", y = "len",
#'   color = "supp", palette = "npg",
#'   add = "jitter",
#'   facet.by = "dose", short.panel.labs = FALSE
#' )
#' # Use only p.format as label. Remove method name.
#' p + stat_compare_means(
#'   aes(label = paste0("p = ", after_stat(p.format)))
#' )
#'
#' @export
stat_compare_means <- function(mapping = NULL, data = NULL,
                               method = NULL, paired = FALSE, id = NULL, method.args = list(), ref.group = NULL,
                               comparisons = NULL, hide.ns = FALSE, label.sep = ", ",
                               label = NULL, label.x.npc = "left", label.y.npc = "top",
                               label.x = NULL, label.y = NULL, vjust = 0, tip.length = 0.03,
                               bracket.size = 0.3, step.increase = 0,
                               symnum.args = list(),
                               p.format.style = "default", p.digits = NULL,
                               p.leading.zero = NULL, p.min.threshold = NULL,
                               p.decimal.mark = NULL,
                               signif.cutoffs = NULL, signif.symbols = NULL,
                               ns.symbol = "ns", use.four.stars = FALSE, show.signif = TRUE,
                               geom = "text", position = "identity", na.rm = FALSE, show.legend = NA,
                               inherit.aes = TRUE, ...) {
  # #560: `id` (subject matching for a paired test) is only meaningful for a
  # paired comparison and is not supported on the `comparisons` (ggsignif) path.
  # Error early rather than silently ignoring it and drawing a row-order p-value.
  if (!is.null(id)) {
    if (!isTRUE(paired)) {
      stop("`id` is only used for paired comparisons; set `paired = TRUE`.", call. = FALSE)
    }
    if (!is.null(comparisons)) {
      stop("`id` is not supported together with `comparisons`. Compute the test ",
           "with `compare_means(..., id = )` and draw it with `stat_pvalue_manual()`.",
           call. = FALSE)
    }
  }
  # Handle show.signif = FALSE with label = "p.format.signif"
  if (!show.signif && identical(label, "p.format.signif")) {
    warning("show.signif = FALSE with label = 'p.format.signif': ",
      "falling back to 'p.format' (p-value only, no significance symbols)",
      call. = FALSE
    )
    label <- "p.format"
  }

  # Build symnum.args from new parameters
  symnum.args <- build_symnum_args(
    signif.cutoffs = signif.cutoffs,
    signif.symbols = signif.symbols,
    ns.symbol = ns.symbol,
    use.four.stars = use.four.stars,
    symnum.args = symnum.args
  )

  if (!is.null(comparisons)) {
    # The `comparisons` path draws brackets via ggsignif::geom_signif(), whose
    # tip length is always a fraction of the data range - ggpubr cannot alter it.
    # `tip.length.ref` (supported by geom_bracket()/stat_pvalue_manual()) is not
    # honored here, so warn once instead of silently ignoring it (#362).
    if ("tip.length.ref" %in% names(list(...))) {
      rlang::inform(
        c(
          paste0(
            "`tip.length.ref` is not supported by `stat_compare_means(comparisons = )`; ",
            "it is ignored here."
          ),
          i = paste0(
            "Bracket tips are drawn by `ggsignif::geom_signif()` and are always a ",
            "fraction of the data range. For tip lengths that stay constant across ",
            "plots, compute the tests with `compare_means()` and draw them with ",
            "`stat_pvalue_manual(..., tip.length.ref = \"axis\")`."
          )
        ),
        .frequency = "once",
        .frequency_id = "ggpubr_stat_compare_means_tip_length_ref"
      )
    }
    # The `comparisons` path draws each pairwise test independently (via
    # ggsignif::geom_signif) and shows UNADJUSTED p-values. Warn once per session
    # when there is more than one comparison, so users aren't misled into thinking
    # the p-values are corrected for multiple testing (#293).
    if (length(comparisons) > 1) {
      rlang::inform(
        c(
          paste0(
            "`stat_compare_means()` with `comparisons` displays *unadjusted* ",
            "p-values (no correction for multiple comparisons)."
          ),
          i = paste0(
            "For p-values adjusted for multiple comparisons, use `geom_pwc()`, ",
            "or `stat_pvalue_manual()` together with ",
            "`compare_means(..., p.adjust.method = )`."
          )
        ),
        .frequency = "once",
        .frequency_id = "ggpubr_stat_compare_means_comparisons_unadjusted"
      )
    }
    method.info <- .method_info(method)
    method <- method.info$method

    method.args <- .add_item(method.args, paired = paired)

    pms <- list(...)
    size <- ifelse(is.null(pms$size), 3.88, pms$size)
    color <- ifelse(is.null(pms$color), "black", pms$color)
    # Forward the font family to the bracket labels; without this it was dropped
    # in the comparisons path while it worked in the default path (#592, #624).
    family <- ifelse(is.null(pms$family), "", pms$family)

    if (is.null(label)) {
      mapped_label <- .get_stat_compare_means_label_from_mapping(mapping)
      label <- if (is.null(mapped_label)) "p.format" else mapped_label
    }
    map_signif_level <- .make_stat_compare_means_map_signif_level(
      label = label,
      symnum.args = symnum.args,
      hide.ns = hide.ns,
      p.format.style = p.format.style,
      p.digits = p.digits,
      p.leading.zero = p.leading.zero,
      p.min.threshold = p.min.threshold,
      p.decimal.mark = p.decimal.mark
    )

    if (missing(step.increase)) {
      step.increase <- ifelse(is.null(label.y), 0.12, 0)
    }
    ggsignif::geom_signif(
      comparisons = comparisons, y_position = label.y,
      test = method, test.args = method.args,
      step_increase = step.increase, size = bracket.size, textsize = size, color = color,
      family = family,
      map_signif_level = map_signif_level, tip_length = tip.length,
      data = data, vjust = vjust
    )
  } else {
    mapping <- .update_mapping(mapping, label)
    # #560: for a paired test, carry the subject `id` column into the stat's
    # compute data as an aesthetic (a ggplot layer only sees mapped columns).
    # check.aes is disabled ONLY when id is supplied, so the geom does not warn
    # about the non-standard `id` aesthetic; default layers are unchanged.
    if (!is.null(id)) {
      if (is.null(mapping)) mapping <- ggplot2::aes()
      mapping$id <- as.name(id)
    }
    layer(
      stat = StatCompareMeans, data = data, mapping = mapping, geom = geom,
      position = position, show.legend = show.legend, inherit.aes = inherit.aes,
      check.aes = is.null(id),
      params = list(
        label.x.npc = label.x.npc, label.y.npc = label.y.npc,
        label.x = label.x, label.y = label.y, label.sep = label.sep,
        method = method, method.args = method.args,
        paired = paired, ref.group = ref.group,
        symnum.args = symnum.args,
        p.format.style = p.format.style, p.digits = p.digits,
        p.leading.zero = p.leading.zero, p.min.threshold = p.min.threshold,
        p.decimal.mark = p.decimal.mark,
        hide.ns = hide.ns, na.rm = na.rm, vjust = vjust, ...
      )
    )
  }
}


StatCompareMeans <- ggproto("StatCompareMeans", Stat,
  required_aes = c("x", "y"),
  default_aes = aes(hjust = after_stat(hjust), vjust = after_stat(vjust)),
  compute_panel = function(data, scales, method, method.args,
                           paired, ref.group, symnum.args,
                           p.format.style, p.digits,
                           p.leading.zero, p.min.threshold, p.decimal.mark,
                           hide.ns, label.x.npc, label.y.npc,
                           label.x, label.y, label.sep) {
    . <- x <- NULL
    .is.multiple.grouping.vars <- !all(data$x == data$group)

    if (!is.null(ref.group)) {
      if (ref.group != ".all.") ref.group <- scales$x$map(ref.group)
    }

    # Guess the number of group to be compared
    # ::::::::::::::::::::::::::::::::::::::::::::::::::
    if (.is.multiple.grouping.vars) {
      x.levels <- .levels(data$group)
    } else {
      x.levels <- .levels(data$x)
    }
    two.groups <- length(x.levels) == 2 | !is.null(ref.group)
    multi.groups <- length(x.levels) > 2

    # Guess the test to be performed
    # ::::::::::::::::::::::::::::::::::::::::::::::::::
    if (two.groups & is.null(method)) {
      method <- "wilcox.test"
    } else if (multi.groups & is.null(method)) {
      method <- "kruskal.test"
    }

    # Perform group comparisons
    # ::::::::::::::::::::::::::::::::::::::::::::::::::
    if (!is.null(ref.group)) {
      ref.group <- as.character(ref.group)
    }

    method.args <- method.args %>%
      .add_item(
        data = data, method = method,
        paired = paired, ref.group = ref.group,
        symnum.args = symnum.args,
        p.format.style = p.format.style, p.digits = p.digits,
        p.leading.zero = p.leading.zero, p.min.threshold = p.min.threshold,
        p.decimal.mark = p.decimal.mark
      )
    # #560: when a subject id column was mapped (via the `id` argument) AND a
    # paired test is requested, pass it to compare_means() so the pairing is
    # aligned by id. Gated on `paired` so a stray mapped `id` aesthetic on a
    # non-paired plot is ignored (unchanged vs. before).
    if (isTRUE(paired) && "id" %in% names(data)) {
      method.args <- method.args %>%
        .add_item(id = "id", .skip_insufficient_id_pairs = TRUE)
    }

    if (.is.multiple.grouping.vars) {
      method.args <- method.args %>%
        .add_item(formula = y ~ group, group.by = "x")
      .test <- do.call(compare_means, method.args)
    } else {
      method.args <- method.args %>%
        .add_item(formula = y ~ x)
      .test <- do.call(compare_means, method.args)
    }

    if (nrow(.test) == 0) {
      .test$label <- character(0)
      .test$x <- numeric(0)
      .test$y <- numeric(0)
      .test$hjust <- numeric(0)
      .test$vjust <- numeric(0)
      if (!("p.format.signif" %in% names(.test))) {
        .test$p.format.signif <- character(0)
      }
      return(.test)
    }

    # Format p-value for label using the specified style
    p_formatted <- format_p_value(.test$p,
      style = p.format.style,
      digits = p.digits,
      leading.zero = p.leading.zero,
      min.threshold = p.min.threshold,
      decimal.mark = p.decimal.mark
    )
    .test$p.format <- p_formatted
    .test <- add_p_format_signif(.test)
    pvaltxt <- create_p_label(p_formatted)
    .test$label <- paste(.test$method, pvaltxt, sep = label.sep)

    # Options for label positioning
    # ::::::::::::::::::::::::::::::::::::::::::::::::::
    label.opts <- list(
      data = data, scales = scales,
      label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y,
      symnum.args = symnum.args, .by = "panel"
    )

    if (.is.multiple.grouping.vars) {
      if (is.null(label.x) & length(label.x.npc) == 1) {
        label.opts$label.x <- .test$x
      }

      .label.pms <- label.opts %>%
        .add_item(group.ids = .test$x) %>%
        do.call(.label_params_by_group, .) # Returns a data frame with label: x, y, hjust, vjust
      # .test <- dplyr::select(.test, -x)
      .label.pms <- dplyr::select(.label.pms, -x)
    } else {
      .label.pms <- label.opts %>%
        do.call(.label_params, .) %>% # Returns a data frame with label: x, y, hjust, vjust
        dplyr::mutate(hjust = 0.2)
    }
    if (!is.null(ref.group)) {
      group.ids <- as.numeric(.test$group2)
      if (!is.null(label.y) & ref.group != ".all.") {
        if (length(label.y) == length(group.ids)) {
          label.opts$label.y <- c(0, label.y)
        }
      }
      .label.pms <- label.opts %>%
        .add_item(group.ids = group.ids) %>%
        do.call(.label_params_by_group, .)
    }

    res <- cbind(.test, .label.pms)

    if (!is.null(ref.group)) {
      # Set label x value to group names
      other.group.index <- as.numeric(res$group2)
      res$x <- scales$x$range$range[other.group.index]
      res <- res %>% dplyr::mutate(hjust = 0.5)
    }

    if (hide.ns) {
      p.signif <- res$p.signif
      p.format <- res$p.format
      p.signif[p.signif == "ns"] <- " "
      res$p.signif <- p.signif
    }
    if (all(c("p.format", "p.signif") %in% names(res))) {
      res[["p.format.signif"]] <- add_stat_label(res, label = "p.format.signif")$label
    }
    res
  }
)


# Check if p.signif is in mapping
.is_p.signif_in_mapping <- function(mapping) {
  res <- FALSE
  if (!is.null(mapping)) {
    if (!is.null(mapping$label)) {
      .label <- rlang::as_label(mapping$label)
      res <- grepl(pattern = "p\\.signif", .label)
    }
  }
  return(res)
}

.get_stat_compare_means_label_from_mapping <- function(mapping) {
  if (is.null(mapping) || is.null(mapping$label)) {
    return(NULL)
  }
  .label <- rlang::as_label(mapping$label)
  if (grepl("p\\.format\\.signif", .label)) {
    return("p.format.signif")
  }
  if (grepl("p\\.signif", .label)) {
    return("p.signif")
  }
  if (grepl("p\\.format", .label)) {
    return("p.format")
  }
  if (grepl("after_stat\\(p\\)|\\.\\.p\\.\\.", .label)) {
    return("p")
  }
  NULL
}

.make_stat_compare_means_map_signif_level <- function(label, symnum.args, hide.ns,
                                                      p.format.style, p.digits,
                                                      p.leading.zero, p.min.threshold,
                                                      p.decimal.mark) {
  p_to_signif <- function(p) {
    signif_label <- as.character(stats::symnum(
      p,
      cutpoints = symnum.args$cutpoints,
      symbols = symnum.args$symbols
    ))
    if (hide.ns) signif_label[signif_label == "ns"] <- ""
    signif_label
  }
  p_to_format <- function(p) {
    format_p_value(
      p,
      style = p.format.style,
      digits = p.digits,
      leading.zero = p.leading.zero,
      min.threshold = p.min.threshold,
      decimal.mark = p.decimal.mark
    )
  }

  if (label %in% c("p.signif", "..p.signif..")) {
    return(p_to_signif)
  }
  if (label %in% c("p.format", "..p.format..")) {
    return(p_to_format)
  }
  if (label %in% c("p", "..p..")) {
    return(function(p) create_p_label(p_to_format(p)))
  }
  if (label %in% c("p.format.signif", "..p.format.signif..")) {
    return(function(p) create_p_label(p_to_format(p), p_to_signif(p)))
  }
  FALSE
}

# Update mapping with label
.update_mapping <- function(mapping, label) {
  allowed.label <- list(
    "p.signif" = quote(ggplot2::after_stat(p.signif)),
    "..p.signif.." = quote(ggplot2::after_stat(p.signif)),
    "p.format" = quote(ggplot2::after_stat(create_p_label(p.format))),
    "..p.format.." = quote(ggplot2::after_stat(create_p_label(p.format))),
    "p" = quote(ggplot2::after_stat(create_p_label(p.format))),
    "..p.." = quote(ggplot2::after_stat(create_p_label(p.format))),
    "p.format.signif" = quote(ggplot2::after_stat(p.format.signif))
  )

  if (!is.null(label)) {
    if (!label %in% names(allowed.label)) {
      stop("Allowed values for label are: ", .collapse(names(allowed.label), sep = ", "))
    }
  }

  if (!is.null(mapping) & is.character(label)) {
    mapping$label <- allowed.label[[label]]
  } else if (is.character(label)) {
    mapping <- aes()
    mapping$label <- allowed.label[[label]]
  }
  convert_label_dotdot_notation_to_after_stat(mapping)
}

# Hide NS in map_signif_level
.hide_ns <- function(x) {
  n <- names(x)
  ns.pos <- which(n == "ns" | n == "NS")
  if (!.is_empty(ns.pos)) n[ns.pos] <- " "
  names(x) <- n
  x
}

context("test-stat-compare-means-id")

# #560: stat_compare_means() gains an `id` argument so a paired test in the plot
# is aligned by subject id (row-order independent), like compare_means(id=). The
# id column is carried into the stat's compute as an aesthetic without a
# "unknown aesthetics" warning; plots without `id` are unchanged.

suppressMessages(library(dplyr))

.stat_label <- function(p) {
  b <- ggplot2::ggplot_build(p)
  lab <- NULL
  for (d in b$data) if ("label" %in% names(d)) lab <- as.character(d$label)[1]
  lab
}

.warnings_of <- function(expr) {
  ws <- character(0)
  withCallingHandlers(force(expr),
    warning = function(w) { ws <<- c(ws, conditionMessage(w)); invokeRestart("muffleWarning") })
  ws
}

set.seed(123)
.d <- data.frame(
  sampleType = c(rep("Normal", 10), rep("Tumor", 10)),
  value = c(runif(10, 0, 0.2), runif(10, 0, 0.4)),
  ID = c(1:10, 10:1)  # unsorted: row-order pairing is wrong
)

test_that("id shows the id-aligned (correct) p regardless of row order (#560)", {
  correct <- compare_means(value ~ sampleType, .d, paired = TRUE, id = "ID")$p
  lab <- .stat_label(
    ggpaired(.d, x = "sampleType", y = "value") +
      stat_compare_means(paired = TRUE, id = "ID")
  )
  # the displayed p reflects the id-aligned value (formatted), not the row-order one
  expect_true(grepl(format(round(correct, 2), nsmall = 2), lab, fixed = TRUE) ||
              grepl(signif(correct, 2), lab, fixed = TRUE))
  # and it differs from the buggy row-order default
  lab_default <- .stat_label(
    ggpaired(.d, x = "sampleType", y = "value") + stat_compare_means(paired = TRUE)
  )
  expect_false(identical(lab, lab_default))
})

test_that("mapping the id emits no 'unknown aesthetics' warning", {
  ws <- .warnings_of(ggplot2::ggplot_build(
    ggpaired(.d, x = "sampleType", y = "value") +
      stat_compare_means(paired = TRUE, id = "ID")
  ))
  expect_false(any(grepl("unknown aesthetic", ws, ignore.case = TRUE)))
})

test_that("id also works when supplied via aes(id = ...)", {
  correct <- compare_means(value ~ sampleType, .d, paired = TRUE, id = "ID")$p
  lab <- .stat_label(
    ggpaired(.d, x = "sampleType", y = "value") +
      stat_compare_means(ggplot2::aes(id = ID), paired = TRUE, id = "ID")
  )
  expect_true(grepl(signif(correct, 2), lab, fixed = TRUE))
})

test_that("id works with ref.group in the plot (matches compare_means)", {
  set.seed(2)
  d3 <- data.frame(dose = factor(rep(c("d1", "d2", "d3"), each = 10)),
                   len = rnorm(30), ID = c(1:10, sample(1:10), sample(1:10)))
  p <- ggboxplot(d3, "dose", "len") +
    stat_compare_means(paired = TRUE, id = "ID", ref.group = "d1",
                       method = "t.test", label = "p.format")
  b <- ggplot2::ggplot_build(p)
  labs <- NULL
  for (d in b$data) if ("label" %in% names(d)) labs <- unique(as.character(d$label))
  ref <- compare_means(len ~ dose, d3, method = "t.test", paired = TRUE,
                       id = "ID", ref.group = "d1")
  # two labels (d1 vs d2, d1 vs d3), consistent with compare_means' p-values
  expect_equal(length(labs), 2L)
})

test_that("id forwards one-sided alternatives supplied through method.args", {
  one_sided <- .stat_label(
    ggpaired(.d, x = "sampleType", y = "value") +
      stat_compare_means(
        paired = TRUE, id = "ID", method = "t.test",
        method.args = list(alternative = "greater"),
        label = "p.format"
      )
  )
  two_sided <- .stat_label(
    ggpaired(.d, x = "sampleType", y = "value") +
      stat_compare_means(
        paired = TRUE, id = "ID", method = "t.test",
        label = "p.format"
      )
  )
  expected <- compare_means(
    value ~ sampleType, .d,
    paired = TRUE, id = "ID", method = "t.test",
    alternative = "greater"
  )$p.format
  expect_true(grepl(expected, one_sided, fixed = TRUE))
  expect_false(identical(one_sided, two_sided))
})

test_that("id skips sparse faceted subsets in the stat layer", {
  d <- data.frame(
    block = c(rep("x", 6), rep("y", 4)),
    g = c("A", "A", "A", "B", "B", "B", "A", "A", "B", "B"),
    value = c(1, 2.5, 4, 2.1, 3, 6.2, 5, 6.5, 8, 9),
    ID = c(1, 2, 3, 1, 2, 3, 1, 2, 3, 4)
  )
  warnings <- .warnings_of(
    built <- ggplot2::ggplot_build(
      ggboxplot(d, x = "g", y = "value", facet.by = "block") +
        stat_compare_means(method = "t.test", paired = TRUE, id = "ID")
    )
  )
  expect_false(any(grepl("Computation failed|not enough", warnings)))
  labels <- unlist(lapply(built$data, function(x) {
    if ("label" %in% names(x)) as.character(x$label) else NULL
  }))
  expect_true(length(labels) >= 1L)
})

test_that("id duplicate errors are surfaced in faceted stat layers", {
  d <- data.frame(
    block = c(rep("x", 4), rep("y", 4)),
    g = c("A", "A", "B", "B", "A", "A", "B", "B"),
    value = c(1, 2, 3, 5, 10, 11, 20, 21),
    ID = c(1, 2, 1, 2, 1, 1, 1, 2)
  )
  warnings <- .warnings_of(
    ggplot2::ggplot_build(
      ggboxplot(d, x = "g", y = "value", facet.by = "block") +
        stat_compare_means(method = "t.test", paired = TRUE, id = "ID")
    )
  )
  expect_true(any(grepl("duplicated ids", warnings)))
})

test_that("id errors clearly on unsupported combinations (not a silent wrong p)", {
  # with `comparisons`, id can't be honored (ggsignif path) -> error, do not
  # silently draw a row-order p-value
  expect_error(
    stat_compare_means(comparisons = list(c("Normal", "Tumor")), paired = TRUE, id = "ID"),
    "comparisons"
  )
  # id without paired
  expect_error(stat_compare_means(id = "ID"), "paired = TRUE")
})

test_that("a stray mapped `id` aesthetic on a non-paired plot does not error or change output", {
  # a data column named/mapped `id` unrelated to pairing must not be treated as
  # the pairing id when paired is not requested (no loud regression)
  df <- ToothGrowth
  df$dose <- as.factor(df$dose)
  df$id <- seq_len(nrow(df))
  expect_error(
    suppressWarnings(ggplot2::ggplot_build(
      ggplot2::ggplot(df, ggplot2::aes(dose, len, id = id)) +
        ggplot2::geom_boxplot() + stat_compare_means()
    )),
    NA
  )
})

test_that("default (no id) stat_compare_means output is unchanged", {
  df <- ToothGrowth
  df$dose <- as.factor(df$dose)
  # a plot without id builds the same label as before (spot check the value)
  lab <- .stat_label(ggboxplot(df, "dose", "len") + stat_compare_means())
  expect_match(lab, "Kruskal-Wallis")
  # no unknown-aes warning in the default path either
  ws <- .warnings_of(ggplot2::ggplot_build(
    ggboxplot(df, "dose", "len") + stat_compare_means()))
  expect_false(any(grepl("unknown aesthetic", ws, ignore.case = TRUE)))
})

# Regression tests for #293: stat_compare_means(comparisons = ) shows UNADJUSTED
# p-values. Users are now informed once per session (message + docs) and pointed
# to the adjusted-pairwise alternatives.

.cmp3 <- list(c("0.5", "1"), c("1", "2"), c("0.5", "2"))

test_that("multi-comparison stat_compare_means informs about unadjusted p-values (#293)", {
  old <- options(rlib_message_verbosity = "verbose")  # force the once-message
  on.exit(options(old), add = TRUE)
  expect_message(
    ggboxplot(ToothGrowth, "dose", "len") +
      stat_compare_means(comparisons = .cmp3),
    "unadjusted"
  )
  # the note points to the adjusted-pairwise alternatives
  expect_message(
    ggboxplot(ToothGrowth, "dose", "len") +
      stat_compare_means(comparisons = .cmp3),
    "geom_pwc|stat_pvalue_manual"
  )
})

test_that("single comparison does NOT emit the note (#293)", {
  old <- options(rlib_message_verbosity = "verbose")
  on.exit(options(old), add = TRUE)
  expect_no_message(
    ggboxplot(ToothGrowth, "dose", "len") +
      stat_compare_means(comparisons = list(c("0.5", "1")))
  )
})

test_that("stat_compare_means(comparisons=) still renders (no regression, #293)", {
  p <- ggboxplot(ToothGrowth, "dose", "len") +
    stat_compare_means(comparisons = .cmp3)
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

# Regression tests for #362: tip.length.ref is not supported on the
# stat_compare_means(comparisons=) path (delegates to ggsignif). Instead of
# silently ignoring it, users are informed once and pointed to stat_pvalue_manual.

test_that("stat_compare_means(comparisons=, tip.length.ref=) informs it is ignored (#362)", {
  old <- options(rlib_message_verbosity = "verbose")
  on.exit(options(old), add = TRUE)
  expect_message(
    ggboxplot(ToothGrowth, "dose", "len") +
      stat_compare_means(comparisons = list(c("0.5", "1")), tip.length.ref = "axis"),
    "tip.length.ref.*not supported|ignored"
  )
  expect_message(
    ggboxplot(ToothGrowth, "dose", "len") +
      stat_compare_means(comparisons = list(c("0.5", "1")), tip.length.ref = "axis"),
    "stat_pvalue_manual"
  )
})

test_that("stat_compare_means without tip.length.ref emits no such note (#362)", {
  old <- options(rlib_message_verbosity = "verbose")
  on.exit(options(old), add = TRUE)
  expect_no_message(
    ggboxplot(ToothGrowth, "dose", "len") +
      stat_compare_means(comparisons = list(c("0.5", "1"))),
    message = "tip.length.ref"
  )
})

test_that("passing tip.length.ref to stat_compare_means does not change output (#362)", {
  suppressMessages({
    with_arg <- ggplot2::ggplot_build(
      ggboxplot(ToothGrowth, "dose", "len") +
        stat_compare_means(comparisons = list(c("0.5", "1")), tip.length.ref = "axis")
    )
    without <- ggplot2::ggplot_build(
      ggboxplot(ToothGrowth, "dose", "len") +
        stat_compare_means(comparisons = list(c("0.5", "1")))
    )
  })
  expect_equal(with_arg$data, without$data)
})

# Regression tests for #594: stat_compare_means() failed when the grouped x
# value was 0 (or <= 0), because that value was used as a 1-based group index
# in .group_coord(), yielding coord.values[0] -> NA and a downstream
# ".npc coord ... should be ..." error.

test_that(".group_coord falls back to the first value for a non-positive id (#594)", {
  # group.id = 0 must not produce NA; it recycles to the first coord value.
  expect_equal(ggpubr:::.group_coord("left", 0), "left")
  expect_equal(ggpubr:::.group_coord(c("left", "right"), 0), "left")
  expect_equal(ggpubr:::.group_coord(c(0, 3), 0), 0)
})

test_that(".group_coord is unchanged for valid 1-based ids (no regression, #594)", {
  # Every previously-working id (>= 1) behaves exactly as before.
  expect_equal(ggpubr:::.group_coord(c("a", "b", "c"), 1), "a")
  expect_equal(ggpubr:::.group_coord(c("a", "b", "c"), 2), "b")
  expect_equal(ggpubr:::.group_coord(c("a", "b", "c"), 3), "c")
  # id longer than the vector recycles to the first value (unchanged behavior).
  expect_equal(ggpubr:::.group_coord(c("a", "b"), 3), "a")
  # empty coord values pass through untouched.
  expect_equal(ggpubr:::.group_coord(NULL, 0), NULL)
})

test_that("stat_compare_means() renders when a grouped x value is 0 (#594)", {
  data <- data.frame(
    x = c(0, 0, 0, 0, 3, 3, 3, 3, 0, 0, 0, 0, 3, 3, 3, 3),
    y = c(1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8,
          1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8),
    group = rep(c("A", "B"), each = 8)
  )
  p <- ggplot2::ggplot(data, ggplot2::aes(x = x, y = y, color = group)) +
    ggplot2::geom_point() +
    stat_compare_means(label = "p.signif")
  # Full render (not just build) so any draw-time error surfaces; must not warn
  # with the "Computation failed" message the bug produced.
  expect_no_warning(
    grob <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p))
  )
  expect_s3_class(grob, "gtable")
})

# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)
df$group <- factor(rep(c("grp1", "grp2"), 30))


# Comparing two groups
.get_stat_test <- function(df, ...) {
  bxp <- ggboxplot(df, x = "supp", y = "len") +
    stat_compare_means(...)
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  stat.test
}


# Pairwise comparison test
.get_pwc_test <- function(df, ...) {
  my_comparisons <- list(c("0.5", "1"), c("1", "2"), c("0.5", "2"))
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    stat_compare_means(comparisons = my_comparisons, method = "t.test", ...)

  bxp_build <- ggplot2::ggplot_build(bxp)
  pwc_test <- bxp_build$data[[2]]

  pwc_test <- pwc_test %>%
    dplyr::mutate(
      x = as.numeric(x), xend = as.numeric(xend),
      annotation = as.character(annotation)
    )
  pwc_test
}

# two independent tests -----------------------------------
test_that("stat_compare_means works for two independent tests: Wilcoxon test", {
  stat.test <- .get_stat_test(df)
  label_coords_expected <- data.frame(
    stringsAsFactors = FALSE,
    x = 1,
    y = c(33.9),
    label = c("Wilcoxon, p = 0.064")
  )
  label_coords_observed <- stat.test[, c("x", "y", "label")]
  label_coords_observed$x <- as.numeric(label_coords_observed$x)
  expect_equal(label_coords_expected, label_coords_observed)
})

test_that("stat_compare_means works for two independent tests when method changed to t.test", {
  stat.test <- .get_stat_test(df, method = "t.test")
  label_coords_expected <- data.frame(
    stringsAsFactors = FALSE,
    x = 1,
    y = c(33.9),
    label = c("T-test, p = 0.061")
  )
  label_coords_observed <- stat.test[, c("x", "y", "label")]
  label_coords_observed$x <- as.numeric(label_coords_observed$x)
  expect_equal(label_coords_expected, label_coords_observed)
})


test_that("stat_compare_means works when label specified as label='p.signif'", {
  stat.test <- .get_stat_test(df, label = "p.signif")
  label_coords_expected <- data.frame(
    stringsAsFactors = FALSE,
    x = 1,
    y = c(33.9),
    label = c("ns")
  )
  label_coords_observed <- stat.test[, c("x", "y", "label")]
  label_coords_observed$x <- as.numeric(label_coords_observed$x)
  expect_equal(label_coords_expected, label_coords_observed)
})


test_that("stat_compare_means works when label specified as aes(label=..p.signif..)", {
  stat.test <- .get_stat_test(df, aes(label = ..p.signif..))
  label_coords_expected <- data.frame(
    stringsAsFactors = FALSE,
    x = 1,
    y = c(33.9),
    label = c("ns")
  )
  label_coords_observed <- stat.test[, c("x", "y", "label")]
  label_coords_observed$x <- as.numeric(label_coords_observed$x)
  expect_equal(label_coords_expected, label_coords_observed)
})


# Paired samples-----------------------------------
test_that("stat_compare_means works for paired samples comparison", {
  stat.test <- .get_stat_test(df, paired = TRUE)
  label_coords_observed <- stat.test[, c("x", "y", "label")]
  label_coords_observed$x <- as.numeric(label_coords_observed$x)

  # Accept either 0.0043 (exact=FALSE) or 0.0038 (R-devel exact conditional inference)
  expect_true(label_coords_observed$label %in% c("Wilcoxon, p = 0.0043", "Wilcoxon, p = 0.0038"),
    info = paste("Observed label =", label_coords_observed$label)
  )
  expect_equal(label_coords_observed$x, 1)
  expect_equal(label_coords_observed$y, 33.9)
})


# More than two groups-------------------------------
test_that("stat_compare_means works for pairwise comparisons with multiple groups", {
  pwc_test <- .get_pwc_test(df)
  pwc_observed <- pwc_test[, c("x", "xend", "y", "yend", "annotation")]
  pwc_observed$annotation <- as.character(pwc_observed$annotation)
  pwc_expected <- data.frame(
    x = c(1, 1, 2, 1, 1, 3, 2, 2, 3),
    xend = c(1, 2, 2, 1, 3, 3, 2, 3, 3),
    y = c(
      34.494, 35.385, 35.385, 41.622,
      42.513, 42.513, 38.058, 38.949, 38.949
    ),
    yend = c(
      35.385, 35.385, 34.494, 42.513,
      42.513, 41.622, 38.949, 38.949, 38.058
    ),
    annotation = c(
      "1.3e-07",
      "1.3e-07", "1.3e-07", "4.4e-14", "4.4e-14", "4.4e-14",
      "1.9e-05", "1.9e-05", "1.9e-05"
    ),
    stringsAsFactors = FALSE
  )
  expect_equal(pwc_expected, pwc_observed)
})


# More than two groups-------------------------------
test_that("stat_compare_means works for pairwise comparisons with multiple groups when specifying label as label='p.signif'", {
  pwc_test <- .get_pwc_test(df, label = "p.signif")
  pwc_observed <- pwc_test[, c("x", "xend", "y", "yend", "annotation")]
  pwc_observed$annotation <- as.character(pwc_observed$annotation)
  pwc_expected <- data.frame(
    x = c(1, 1, 2, 1, 1, 3, 2, 2, 3),
    xend = c(1, 2, 2, 1, 3, 3, 2, 3, 3),
    y = c(
      34.494, 35.385, 35.385, 41.622,
      42.513, 42.513, 38.058, 38.949, 38.949
    ),
    yend = c(
      35.385, 35.385, 34.494, 42.513,
      42.513, 41.622, 38.949, 38.949, 38.058
    ),
    annotation = c(
      "****", "****", "****", "****", "****", "****",
      "****", "****", "****"
    ),
    stringsAsFactors = FALSE
  )
  expect_equal(pwc_expected, pwc_observed)
})


test_that("stat_compare_means works for pairwise comparisons with multiple groups when specifying label as aes(label=after_stat(p.signif))", {
  pwc_test <- .get_pwc_test(df, aes(label = after_stat(p.signif)))
  pwc_observed <- pwc_test[, c("x", "xend", "y", "yend", "annotation")]
  pwc_observed$annotation <- as.character(pwc_observed$annotation)
  pwc_expected <- data.frame(
    x = c(1, 1, 2, 1, 1, 3, 2, 2, 3),
    xend = c(1, 2, 2, 1, 3, 3, 2, 3, 3),
    y = c(
      34.494, 35.385, 35.385, 41.622,
      42.513, 42.513, 38.058, 38.949, 38.949
    ),
    yend = c(
      35.385, 35.385, 34.494, 42.513,
      42.513, 41.622, 38.949, 38.949, 38.058
    ),
    annotation = c(
      "****", "****", "****", "****", "****", "****",
      "****", "****", "****"
    ),
    stringsAsFactors = FALSE
  )
  expect_equal(pwc_expected, pwc_observed)
})


# Issue #4: Test for p.format.signif label type
test_that("stat_compare_means works with label='p.format.signif' to show p-value with significance stars", {
  stat.test <- .get_stat_test(df, label = "p.format.signif")
  label_coords_observed <- stat.test[, c("x", "y", "label")]
  label_coords_observed$x <- as.numeric(label_coords_observed$x)

  # The p-value is ~0.064 which is "ns" (not significant)
  expect_equal(label_coords_observed$label, "p = 0.064 ns")
  expect_equal(label_coords_observed$x, 1)
  expect_equal(label_coords_observed$y, 33.9)
})

test_that("stat_compare_means with label='p.format.signif' shows stars for significant results", {
  # Use paired test which has significant p-value
  stat.test <- .get_stat_test(df, label = "p.format.signif", paired = TRUE)
  label_value <- stat.test$label

  # Should have format "p = X.XXXX **" (p ~ 0.004 is **)
  expect_true(grepl("^p = .* \\*\\*$", label_value),
    info = paste("Observed label =", label_value)
  )
})


# Issue #5: Test that p-values with threshold show "p <" not "p = <"
test_that("stat_compare_means with p.format and threshold shows 'p <' not 'p = <'", {
  # Create data with very significant difference to get p < 0.001
  set.seed(123)
  test_df <- data.frame(
    group = rep(c("A", "B"), each = 30),
    value = c(rnorm(30, mean = 0, sd = 1), rnorm(30, mean = 5, sd = 1))
  )

  bxp <- ggboxplot(test_df, x = "group", y = "value") +
    stat_compare_means(label = "p.format", p.format.style = "nejm")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]

  # Should show "p < 0.001" not "p = < 0.001"
  expect_true(grepl("^p < ", stat.test$label) || grepl("^p = 0\\.", stat.test$label),
    info = paste("Observed label =", stat.test$label)
  )
  expect_false(grepl("p = <", stat.test$label),
    info = paste("Label should not contain 'p = <', but got:", stat.test$label)
  )
})

test_that("stat_compare_means with p.format.signif and threshold shows 'p <' not 'p = <'", {
  # Create data with very significant difference
  set.seed(123)
  test_df <- data.frame(
    group = rep(c("A", "B"), each = 30),
    value = c(rnorm(30, mean = 0, sd = 1), rnorm(30, mean = 5, sd = 1))
  )

  bxp <- ggboxplot(test_df, x = "group", y = "value") +
    stat_compare_means(label = "p.format.signif", p.format.style = "nejm")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]

  # Should show "p < 0.001 ****" not "p = < 0.001 ****"
  expect_true(grepl("^p < ", stat.test$label),
    info = paste("Observed label =", stat.test$label)
  )
  expect_false(grepl("p = <", stat.test$label),
    info = paste("Label should not contain 'p = <', but got:", stat.test$label)
  )
  # Should have stars
  expect_true(grepl("\\*+$", stat.test$label),
    info = paste("Label should end with stars, but got:", stat.test$label)
  )
})


# Tests for custom significance cutoffs and symbols
test_that("stat_compare_means works with custom signif.cutoffs (3 levels)", {
  stat.test <- .get_stat_test(df,
    label = "p.signif",
    signif.cutoffs = c(0.10, 0.05, 0.01)
  )
  # p = 0.064 is between 0.05 and 0.10, so should get "*"
  expect_equal(stat.test$label, "*")
})

test_that("stat_compare_means works with custom signif.cutoffs (4 levels with use.four.stars)", {
  # Use paired test which has significant p-value (~0.004)
  stat.test <- .get_stat_test(df,
    label = "p.signif", paired = TRUE,
    signif.cutoffs = c(0.10, 0.05, 0.01, 0.001),
    use.four.stars = TRUE
  )
  # p ~ 0.004 is between 0.001 and 0.01, so should get "***"
  expect_equal(stat.test$label, "***")
})

test_that("stat_compare_means works with custom signif.symbols", {
  stat.test <- .get_stat_test(df,
    label = "p.signif",
    signif.cutoffs = c(0.10, 0.05, 0.01),
    signif.symbols = c("+", "++", "+++")
  )
  # p = 0.064 is between 0.05 and 0.10, so should get "+"
  expect_equal(stat.test$label, "+")
})

test_that("stat_compare_means works with custom ns.symbol", {
  # Create data with non-significant difference
  set.seed(456)
  test_df <- data.frame(
    group = rep(c("A", "B"), each = 10),
    value = rnorm(20, mean = 5, sd = 1)
  )
  bxp <- ggboxplot(test_df, x = "group", y = "value") +
    stat_compare_means(label = "p.signif", ns.symbol = "N.S.")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]

  expect_equal(stat.test$label, "N.S.")
})

test_that("stat_compare_means works with empty ns.symbol", {
  # Create data with non-significant difference
  set.seed(456)
  test_df <- data.frame(
    group = rep(c("A", "B"), each = 10),
    value = rnorm(20, mean = 5, sd = 1)
  )
  bxp <- ggboxplot(test_df, x = "group", y = "value") +
    stat_compare_means(label = "p.signif", ns.symbol = "")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]

  expect_equal(stat.test$label, "")
})

test_that("show.signif = FALSE falls back to p.format with warning", {
  expect_warning(
    bxp <- ggboxplot(df, x = "supp", y = "len") +
      stat_compare_means(label = "p.format.signif", show.signif = FALSE),
    "falling back"
  )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  # Should show p-value only, no stars
  expect_equal(stat.test$label, "p = 0.064")
})

test_that("stat_compare_means with p.format.signif uses custom signif.cutoffs", {
  stat.test <- .get_stat_test(df,
    label = "p.format.signif",
    signif.cutoffs = c(0.10, 0.05, 0.01)
  )
  # p = 0.064 is between 0.05 and 0.10, so should get "*"
  expect_equal(stat.test$label, "p = 0.064 *")
})

test_that("stat_compare_means comparisons branch applies p.format.style", {
  pwc_default <- .get_pwc_test(df, label = "p.format")
  pwc_apa <- .get_pwc_test(df, label = "p.format", p.format.style = "apa")

  expect_false(identical(pwc_default$annotation, pwc_apa$annotation))
  expect_true(all(pwc_apa$annotation == "< .001"))
})

test_that("stat_compare_means comparisons branch supports p.format.signif labels", {
  pwc <- .get_pwc_test(df,
    label = "p.format.signif",
    p.format.style = "apa",
    signif.cutoffs = c(0.10, 0.05, 0.01, 0.001),
    use.four.stars = TRUE
  )

  expect_true(all(grepl("^p\\s*[=<]", pwc$annotation)))
  expect_true(any(grepl("\\*+$", pwc$annotation)))
})

test_that("stat_compare_means handles grouped subsets with <2 levels", {
  set.seed(42)
  dat <- data.frame(
    x = factor(c(rep("A", 8), rep("B", 8), rep("C", 8))),
    y = rnorm(24),
    grp = factor(
      c(rep("M", 8), rep(c("M", "F"), each = 4), rep("F", 8)),
      levels = c("M", "F")
    )
  )

  p <- ggboxplot(dat, x = "x", y = "y", color = "grp") +
    stat_compare_means(aes(group = grp))

  expect_no_error(
    b <- ggplot2::ggplot_build(p)
  )
  stat.test <- b$data[[2]]
  expect_true(all(c("group1", "group2", "p") %in% colnames(stat.test)))
  expect_gte(nrow(stat.test), 1)
})

test_that("stat_compare_means returns empty layer when no subset is comparable", {
  set.seed(2)
  dat <- data.frame(
    x = factor(rep(c("A", "B", "C"), each = 8)),
    y = rnorm(24),
    grp = factor(
      c(rep("M", 8), rep("F", 8), rep("M", 8)),
      levels = c("M", "F")
    )
  )

  p <- ggboxplot(dat, x = "x", y = "y", color = "grp") +
    stat_compare_means(aes(group = grp))

  expect_no_warning(
    b <- ggplot2::ggplot_build(p)
  )
  expect_equal(nrow(b$data[[2]]), 0)
})

test_that("stat_compare_means handles one-level x without error", {
  set.seed(3)
  dat <- data.frame(
    x = factor(rep("A", 20)),
    y = rnorm(20)
  )

  p <- ggboxplot(dat, x = "x", y = "y") + stat_compare_means()

  expect_no_error(
    b <- ggplot2::ggplot_build(p)
  )
  expect_equal(nrow(b$data[[2]]), 0)
})

# Regression tests for #592 / #624: stat_compare_means(family = ...) was ignored
# when `comparisons` was set (the comparisons path delegates to
# ggsignif::geom_signif and did not forward `family`).

.text_family <- function(p) {
  b <- ggplot2::ggplot_build(p)
  fam <- character(0)
  for (i in seq_along(b$data)) {
    d <- b$data[[i]]
    if ("family" %in% names(d)) fam <- c(fam, unique(as.character(d$family)))
  }
  fam
}

cmp <- list(c("0.5", "1"), c("1", "2"))

test_that("family is forwarded to comparison brackets (#592, #624)", {
  p <- ggboxplot(ToothGrowth, x = "dose", y = "len") +
    stat_compare_means(comparisons = cmp, method = "t.test", family = "mono")
  expect_true("mono" %in% .text_family(p))
  # renders at draw time
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

test_that("default family is unchanged when comparisons set (no regression, #592)", {
  p <- ggboxplot(ToothGrowth, x = "dose", y = "len") +
    stat_compare_means(comparisons = cmp, method = "t.test")
  fam <- .text_family(p)
  # No non-empty family is forced onto the labels (ggsignif default is "").
  expect_true(length(fam) == 0 || all(fam == ""))
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

test_that("family still works in the non-comparisons path (no regression, #592)", {
  p <- ggboxplot(ToothGrowth, x = "dose", y = "len") +
    stat_compare_means(method = "t.test", family = "mono")
  expect_true("mono" %in% .text_family(p))
})

#' @include utilities.R ggpar.R
NULL
#' Pie chart
#' @description Create a pie chart.
#' @inheritParams ggboxplot
#' @inheritParams ggplot2::geom_errorbar
#' @param x variable containing values for drawing.
#' @param label variable specifying the label of each slice.
#' @param lab.pos character specifying the position for labels. Allowed values
#'  are "out" (for outside) or "in" (for inside).
#' @param lab.adjust numeric value, used to adjust label position when lab.pos =
#'  "in". Increase or decrease this value to see the effect.
#' @param lab.font a vector of length 3 indicating respectively the size (e.g.:
#'  14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and the
#'  color (e.g.: "red") of label font. For example \emph{lab.font= c(4, "bold",
#'  "red")}.
#' @param font.family character vector specifying font family.
#' @param color,fill outline and fill colors.
#' @param ... other arguments to be passed to be passed to ggpar().
#' @details The plot can be easily customized using the function ggpar(). Read
#'  ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
#'  ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
#'  = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
#'  and position: legend = "right" \item plot orientation : orientation =
#'  c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}, \code{\link{ggline}}
#' @examples
#'
#' # Data: Create some data
#' # +++++++++++++++++++++++++++++++
#'
#' df <- data.frame(
#'   group = c("Male", "Female", "Child"),
#'   value = c(25, 25, 50)
#' )
#'
#' head(df)
#'
#'
#' # Basic pie charts
#' # ++++++++++++++++++++++++++++++++
#'
#' ggpie(df, "value", label = "group")
#'
#' # Reducing margins around the pie chart
#' ggpie(df, "value", label = "group") +
#'   theme(plot.margin = unit(c(-.75, -.75, -.75, -.75), "cm"))
#'
#'
#' # Change color
#' # ++++++++++++++++++++++++++++++++
#'
#' # Change fill color by group
#' # set line color to white
#' # Use custom color palette
#' ggpie(df, "value",
#'   label = "group",
#'   fill = "group", color = "white",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#'
#' # Change label
#' # ++++++++++++++++++++++++++++++++
#'
#' # Show group names and value as labels
#' labs <- paste0(df$group, " (", df$value, "%)")
#' ggpie(df, "value",
#'   label = labs,
#'   fill = "group", color = "white",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#' # Change the position and font color of labels
#' ggpie(df, "value",
#'   label = labs,
#'   lab.pos = "in", lab.font = "white",
#'   fill = "group", color = "white",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#' @export
ggpie <- function(
  data, x, label = x, lab.pos = c("out", "in"), lab.adjust = 0,
  lab.font = c(4, "plain", "black"), font.family = "",
  color = "black", fill = "white", palette = NULL,
  size = NULL, ggtheme = theme_pubr(), ...
) {
  lab.pos <- match.arg(lab.pos)
  lab.font <- .parse_font(lab.font) %>%
    .check_pie_labfont()


  if (length(label) > 1 & length(label) != nrow(data)) {
    stop("label should be of the same length as data")
  } else if (length(label) > 1) {
    # 1. Add label column
    data <- data %>%
      dplyr::mutate(.label. = label)
    label <- ".label."
  }

  # We should order the data in desc order. Because,
  # in stacked bar plot the order of factor levels are reversed
  # Very important to have the label in the right place
  group.vars <- intersect(c(fill, color), names(data))
  if (length(group.vars) != 0) {
    group.val <- dplyr::pull(data, group.vars[1])
    data <- data %>% dplyr::arrange(dplyr::desc(group.val))
  }

  # Label y coordinates when placed inside slices
  .x <- dplyr::pull(data, x)
  data <- data %>%
    dplyr::mutate(
      .lab.ypos. = cumsum(.x) - 0.5 * .x - lab.adjust
    )

  p <- ggplot(data, create_aes(list(x = "1", y = x))) +
    geom_exec(
      geom_bar, data,
      stat = "identity",
      fill = fill, color = color, size = size
    )

  p <- ggpar(
    p,
    palette = palette,
    ggtheme = ggtheme,
    font.family = font.family, ...
  ) +
    coord_polar(
      theta = "y", start = 0, clip = "off"
    ) +
    .remove_axis()

  # Annotate pie slice
  # :::::::::::::::::::::::::::::::::::
  if (!is.null(label)) {
    # Label each slice at the middle of the slice
    if (lab.pos == "out") {
      p <- p + scale_y_continuous(
        breaks = cumsum(.x) - .x / 2,
        labels = dplyr::pull(data, label)
      ) +
        ggplot2::theme(axis.text.x = element_text(
          face = lab.font$face, color = lab.font$color, size = 2.5 * lab.font$size,
          family = font.family
        ))
    }
    # Compute the cumulative sum as label ypos
    if (lab.pos == "in") {
      p <- p + geom_text(
        create_aes(list(y = ".lab.ypos.", label = label)),
        size = lab.font$size, fontface = lab.font$face,
        colour = lab.font$color, family = font.family
      ) +
        clean_theme()
    }
  }

  p
}


# Old version
ggpie_1 <- function(data, x, label = NULL, lab.pos = c("out", "in"), lab.adjust = 0,
                    lab.font = c(4, "bold", "black"), font.family = "",
                    color = "black", fill = "white", palette = NULL,
                    size = NULL, ggtheme = theme_classic(),
                    ...) {
  lab.pos <- match.arg(lab.pos)
  lab.font <- .parse_font(lab.font)

  # data <- data[order(data[, x]), , drop = FALSE]
  if (fill %in% colnames(data)) {
    fill_d <- dplyr::pull(data, fill)
    data[[fill]] <- factor(fill_d, levels = rev(fill_d))
  }

  if (is.null(lab.font)) lab.font <- list(size = 5, face = "bold", color = "black")


  p <- ggplot(data, create_aes(list(x = 1, y = x))) +
    .geom_exec(geom_bar, data, stat = "identity", fill = fill, color = color, size = size)
  p <- ggpar(p, palette = palette, ggtheme = ggtheme, font.family = font.family, ...)

  p <- p + coord_polar(theta = "y", start = 0) +
    theme(
      axis.title = element_blank(),
      axis.text.y = element_blank(),
      axis.line = element_blank(),
      # panel.border = element_blank(),
      # panel.grid=element_blank(),
      axis.ticks = element_blank()
    )

  # label each slice at the midle of the slice
  if (!is.null(label)) {
    if (label[1] %in% names(data)) label <- data[[label]]

    if (lab.pos == "out") {
      p <- p + scale_y_continuous(
        breaks = cumsum(data[[x]]) - data[[x]] / 2,
        labels = label
      )
    }
    if (lab.pos == "in") {
      df <- data
      y <- df[, x]
      df[, x] <- (y / (length(y) - lab.adjust)) + c(0, cumsum(y)[-length(y)])
      lab.font$size <- ifelse(is.null(lab.font$size), 5, lab.font$size)
      lab.font$color <- ifelse(is.null(lab.font$color), "black", lab.font$color)
      lab.font$face <- ifelse(is.null(lab.font$ace), "bold", lab.font$face)
      p <- p + .geom_exec(geom_text,
        data = df, x = 1, y = x, label = label,
        size = lab.font$size, fontface = lab.font$face, colour = lab.font$color,
        family = font.family
      ) +
        theme(
          axis.text.x = element_blank(),
          axis.text.y = element_blank(),
          axis.ticks = element_blank()
        )
    }
  }


  p
}


# Remove axis elements
.remove_axis <- function() {
  theme(
    axis.title = element_blank(),
    axis.text.y = element_blank(),
    axis.line = element_blank(),
    # panel.border = element_blank(),
    # panel.grid=element_blank(),
    axis.ticks = element_blank()
  )
}

# Check labels font
.check_pie_labfont <- function(lab.font) {
  lab.font$size <- ifelse(is.null(lab.font$size), 3.9, lab.font$size)
  lab.font$color <- ifelse(is.null(lab.font$color), "black", lab.font$color)
  lab.font$face <- ifelse(is.null(lab.font$face), "plain", lab.font$face)
  lab.font
}

# #655: opt-in ggrepel labels for donut charts (ggdonutchart). With many small
# slices the default labels collide; this spreads them around the ring with leader
# lines, instead of dropping any. `x.pos` is the radius at which the labels are
# anchored (just outside the ring) and `.lab.ypos.` (already on `data`) is the
# slice mid-point along the angular axis. Returns a ggrepel::geom_text_repel layer.
.ggpie_repel_labels <- function(data, label, x.pos, lab.font, font.family) {
  data[[".lab.xpos."]] <- x.pos
  max.overlaps <- getOption("ggrepel.max.overlaps", default = Inf)
  ggrepel::geom_text_repel(
    data = data,
    mapping = create_aes(list(x = ".lab.xpos.", y = ".lab.ypos.", label = label)),
    size = lab.font$size, fontface = lab.font$face, colour = lab.font$color,
    family = font.family, show.legend = FALSE, max.overlaps = max.overlaps,
    segment.size = 0.3, segment.colour = "grey50", box.padding = 0.4
  )
}

# Regression tests for #642: a NAMED palette vector triggered a spurious
# "No shared levels found between `names(values)` ..." warning at draw time,
# because ggpar() applied BOTH scale_color_manual and scale_fill_manual even
# when only one of color/fill was mapped to a variable.

.build_warnings <- function(p) {
  ws <- character(0)
  withCallingHandlers(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)),
    warning = function(w) {
      ws[[length(ws) + 1]] <<- conditionMessage(w)
      invokeRestart("muffleWarning")
    }
  )
  ws
}
.no_shared <- "No shared levels"

test_that("named palette on a fill-only plot does not warn (#642)", {
  df <- data.frame(prop = c(0.4, 0.6),
                   my_categories = c("X", "Y"), label = c("40%", "60%"))
  pal <- c("X" = "red", "Y" = "blue")
  p <- ggpie(df, "prop", fill = "my_categories", label = "label", palette = pal)
  expect_false(any(grepl(.no_shared, .build_warnings(p))))
  # fill colors are still applied by name
  fills <- unique(ggplot2::ggplot_build(p)$data[[1]]$fill)
  expect_setequal(fills, c("red", "blue"))
})

test_that("named palette on a color-only plot does not warn (#642)", {
  p <- ggline(ToothGrowth, "dose", "len", color = "supp",
              palette = c("OJ" = "red", "VC" = "blue"))
  expect_false(any(grepl(.no_shared, .build_warnings(p))))
})

test_that("unnamed palette keeps applying both color and fill (no regression, #642)", {
  # boxplot with color mapped: unnamed palette must still color it
  p <- ggboxplot(ToothGrowth, "dose", "len", color = "supp",
                 palette = c("red", "blue"))
  expect_false(any(grepl(.no_shared, .build_warnings(p))))
  cols <- unique(ggplot2::ggplot_build(p)$data[[1]]$colour)
  expect_setequal(cols, c("red", "blue"))
})

test_that("named palette with non-matching names still warns (#642)", {
  # user-supplied names genuinely don't match the data -> ggplot2's warning is
  # legitimate and must be preserved.
  p <- ggboxplot(ToothGrowth, "dose", "len", fill = "supp",
                 palette = c("A" = "red", "B" = "blue"))
  expect_true(any(grepl(.no_shared, .build_warnings(p))))
})

#' @include utilities.R p_format_utils.R
NULL
#' Comparison of Means
#' @description Performs one or multiple mean comparisons.
#' @param formula a formula of the form \code{x ~ group} where \code{x} is a
#'  numeric variable giving the data values and \code{group} is a factor with
#'  one or multiple levels giving the corresponding groups. For example,
#'  \code{formula = TP53 ~ cancer_group}.
#'
#'  It's also possible to perform the test for multiple response variables at
#'  the same time. For example, \code{formula = c(TP53, PTEN) ~ cancer_group}.
#' @param data a data.frame containing the variables in the formula.
#' @param method the type of test. Default is \link[stats]{wilcox.test}. Allowed
#'  values include: \itemize{ \item \code{\link[stats]{t.test}} (parametric) and
#'  \code{\link[stats]{wilcox.test}} (non-parametric). Perform comparison
#'  between two groups of samples. If the grouping variable contains more than
#'  two levels, then a pairwise comparison is performed. \item
#'  \code{\link[stats]{anova}} (parametric) and
#'  \code{\link[stats]{kruskal.test}} (non-parametric). Perform one-way ANOVA
#'  test comparing multiple groups. }
#' @param paired a logical indicating whether you want a paired test. Used only
#'  in \code{\link[stats]{t.test}} and in \link[stats]{wilcox.test}.
#' @param id optional character string naming a column that identifies matched
#'  subjects for a \strong{paired} comparison (\code{method = "t.test"} or
#'  \code{"wilcox.test"}). By default (\code{id = NULL}) a paired test pairs
#'  observations by \emph{row order}, so a p-value can be wrong if the data are
#'  not sorted so that the compared groups align. Providing \code{id} pairs the
#'  observations by subject id instead (row-order independent), using only the
#'  complete pairs (per-comparison pairwise deletion, via \pkg{rstatix}). It
#'  works for a two-group, a pairwise (more than two groups) and a
#'  \code{ref.group} comparison; it is an error to combine \code{id} with
#'  \code{anova}/\code{kruskal.test} or with \code{ref.group = ".all."}.
#' @param group.by  a character vector containing the name of grouping variables.
#' @param ref.group a character string specifying the reference group. If
#'  specified, for a given grouping variable, each of the group levels will be
#'  compared to the reference group (i.e. control group).
#'
#'  \code{ref.group} can be also \code{".all."}. In this case, each of the
#'  grouping variable levels is compared to all (i.e. basemean).
#' @param symnum.args a list of arguments to pass to the function
#'  \code{\link[stats]{symnum}} for symbolic number coding of p-values. For
#'  example, \code{symnum.args <- list(cutpoints = c(0, 0.0001, 0.001,
#'  0.01, 0.05, Inf), symbols = c("****", "***", "**", "*",  "ns"))}.
#'
#'  In other words, we use the following convention for symbols indicating
#'  statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
#'  \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001 \item \code{****}:  p <= 0.0001 }
#'
#'  Note: If \code{signif.cutoffs} is provided, it takes precedence over
#'  \code{symnum.args}.
#' @param signif.cutoffs numeric vector of p-value cutoffs in descending order
#'  for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
#'  means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
#'  If \code{use.four.stars = TRUE}, can include a fourth level (e.g.,
#'  \code{c(0.10, 0.05, 0.01, 0.001)} for "****" at p < 0.001).
#'  Default is NULL, which uses the package defaults (backward compatible).
#' @param signif.symbols character vector of symbols corresponding to
#'  \code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
#'  (and "****" if \code{use.four.stars = TRUE}). Must have the same length
#'  as \code{signif.cutoffs}.
#' @param ns.symbol character string for non-significant results. Default is "ns".
#'  Use "" (empty string) to show nothing for non-significant results.
#' @param use.four.stars logical. If TRUE and \code{signif.symbols} is NULL,
#'  allows four stars (****) for the most significant level when
#'  \code{signif.cutoffs} has 4 levels. Default is FALSE.
#' @param p.adjust.method method for adjusting p values (see
#'  \code{\link[stats]{p.adjust}}). Has impact only in a situation, where
#'  multiple pairwise tests are performed; or when there are multiple grouping
#'  variables. Allowed values include "holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none". If you don't want to adjust the p
#'  value (not recommended), use p.adjust.method = "none".
#'
#'  Note that, when the \code{formula} contains multiple variables, the p-value
#'  adjustment is done independently for each variable.
#' @param p.format.style character string specifying the p-value formatting style.
#'  One of: \code{"default"} (backward compatible, uses scientific notation),
#'  \code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
#'  \code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
#'  (GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
#'  See \code{\link{list_p_format_styles}} for details.
#' @param p.digits integer specifying the number of decimal places for p-values.
#'  If provided, overrides the style default.
#' @param p.leading.zero logical indicating whether to include leading zero before
#'  decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.
#' @param p.min.threshold numeric specifying the minimum p-value to display exactly.
#'  Values below this threshold are shown as "< threshold". If NULL, the selected
#'  style's default threshold is used; styles without a threshold show exact
#'  values. If provided, overrides the style default.
#' @param p.decimal.mark character string to use as the decimal mark. If NULL,
#'  uses \code{getOption("OutDec")}.
#' @return a data frame with the following columns:
#' \itemize{
#' \item \code{.y.}: the y variable used in the test.
#' \item \code{group1,group2}: the compared groups in the pairwise tests.
#' Available only when \code{method = "t.test"} or \code{method = "wilcox.test"}.
#' \item \code{p}: the p-value.
#' \item \code{p.adj}: the adjusted p-value. Default for \code{p.adjust.method = "holm"}.
#' \item \code{p.format}: the formatted p-value.
#' \item \code{p.format.signif}: the formatted p-value with significance symbols.
#' \item \code{p.signif}: the significance level.
#' \item \code{method}: the statistical test used to compare groups.
#'
#'
#' }
#' @param ... Other arguments to be passed to the test function.
#' @section Significance letters (compact letter display):
#' To label groups with letters instead of p-values or stars - groups that share a
#' letter are not significantly different - compute the pairwise comparisons and
#' derive the letters with \code{rstatix::add_cld()} (available without any extra
#' package, as \code{rstatix} is already a dependency), then place them with
#' \code{geom_text()}:
#'
#' \preformatted{
#' library(ggpubr)
#' library(rstatix)
#' library(dplyr)
#' df <- ToothGrowth
#' df$dose <- factor(df$dose)
#'
#' # all-pairwise comparisons, then compact letters (columns: group, cld)
#' cld <- df \%>\% tukey_hsd(len ~ dose) \%>\% add_cld()
#'
#' # one letter per group, placed above each box
#' ypos <- df \%>\% group_by(dose) \%>\%
#'   summarise(y.position = max(len) + 2, .groups = "drop")
#' cld$y.position <- ypos$y.position[match(cld$group, as.character(ypos$dose))]
#'
#' ggboxplot(df, "dose", "len") +
#'   geom_text(data = cld, aes(x = group, y = y.position, label = cld))
#' }
#'
#' \code{add_cld()} expects an all-pairwise result (\code{tukey_hsd()},
#' \code{dunn_test()}, \code{games_howell_test()}, pairwise
#' \code{wilcox_test()}/\code{t_test()}, or \code{compare_means()}); it is not
#' defined for a single \code{ref.group} comparison.
#'
#' To instead flag each treatment by which of several controls it differs from
#' (e.g. \code{"a"} for a negative control and \code{"b"} for a positive control),
#' run one comparison per control and assemble the letters:
#'
#' \preformatted{
#' trts <- c("trtA", "trtB", "trtC")
#' cn <- compare_means(value ~ group, df, ref.group = "neg.ctrl")
#' cp <- compare_means(value ~ group, df, ref.group = "pos.ctrl")
#' pv <- function(cc, g) dplyr::filter(cc, group1 == g | group2 == g)$p[1]
#' lab <- sapply(trts, function(g)
#'   paste0(if (pv(cn, g) < .05) "a" else "",
#'          if (pv(cp, g) < .05) "b" else ""))
#' # then place `lab` above each treatment with geom_text()
#' }
#' @examples
#' # Load data
#' # :::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # One-sample test
#' # :::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ 1, df, mu = 0)
#'
#' # Two-samples unpaired test
#' # :::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ supp, df)
#'
#' # Two-samples paired test
#' # :::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ supp, df, paired = TRUE)
#'
#' # Paired test pairing by a subject id column (row-order independent)
#' # :::::::::::::::::::::::::::::::::::::::::
#' df$id <- rep(1:30, 2) # pairs the two supp levels by subject
#' compare_means(len ~ supp, df, paired = TRUE, id = "id")
#'
#' # Compare supp levels after grouping the data by "dose"
#' # ::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ supp, df, group.by = "dose")
#'
#' # pairwise comparisons
#' # ::::::::::::::::::::::::::::::::::::::::
#' # As dose contains more thant two levels ==>
#' # pairwise test is automatically performed.
#' compare_means(len ~ dose, df)
#'
#' # Comparison against reference group
#' # ::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ dose, df, ref.group = "0.5")
#'
#' # Comparison against all
#' # ::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ dose, df, ref.group = ".all.")
#'
#' # Anova and kruskal.test
#' # ::::::::::::::::::::::::::::::::::::::::
#' compare_means(len ~ dose, df, method = "anova")
#' compare_means(len ~ dose, df, method = "kruskal.test")

#' @rdname compare_means
#' @export
compare_means <- function(formula, data, method = "wilcox.test",
                          paired = FALSE, id = NULL,
                          group.by = NULL, ref.group = NULL,
                          symnum.args = list(), p.adjust.method = "holm",
                          p.format.style = "default", p.digits = NULL,
                          p.leading.zero = NULL, p.min.threshold = NULL,
                          p.decimal.mark = NULL,
                          signif.cutoffs = NULL, signif.symbols = NULL,
                          ns.symbol = "ns", use.four.stars = FALSE, ...) {
  . <- NULL
  extra.args <- list(...)
  .skip_insufficient_id_pairs <- isTRUE(extra.args$.skip_insufficient_id_pairs)
  extra.args$.skip_insufficient_id_pairs <- NULL

  method.info <- .method_info(method)
  method <- method.info$method
  method.name <- method.info$name

  # #560: validate id-based pairing. When `id` is supplied the paired test is
  # aligned by subject id (row-order independent) instead of by row position,
  # via rstatix, which handles two-group, pairwise and ref.group comparisons
  # (per-comparison complete pairs). Only `ref.group = ".all."` (basemean, which
  # duplicates each observation) is unsupported.
  if (!is.null(id)) {
    if (!isTRUE(paired)) {
      stop("`id` is only used for paired comparisons; set `paired = TRUE`.", call. = FALSE)
    }
    if (!method %in% c("t.test", "wilcox.test")) {
      stop("`id`-based pairing is only supported for `method = \"t.test\"` or ",
           "`method = \"wilcox.test\"`.", call. = FALSE)
    }
    if (!is.null(ref.group) && ref.group == ".all.") {
      stop("`id`-based pairing is not supported with `ref.group = \".all.\"`.", call. = FALSE)
    }
    if (length(id) != 1L || !is.character(id)) {
      stop("`id` must be a single column name (a character string).", call. = FALSE)
    }
    if (!(id %in% colnames(data))) {
      stop("can't find the `id` column '", id, "' in the data.", call. = FALSE)
    }
  }

  # Build symnum.args from new parameters or use defaults
  symnum.args <- build_symnum_args(
    signif.cutoffs = signif.cutoffs,
    signif.symbols = signif.symbols,
    ns.symbol = ns.symbol,
    use.four.stars = use.four.stars,
    symnum.args = symnum.args
  )

  if (!inherits(data, "data.frame")) {
    stop("data must be a data.frame.")
  }

  variables <- response.var <- .formula_left_variables(formula)
  group <- .formula_right_variables(formula)
  if (group == "1") group <- NULL # NULL model

  if (!.is_empty(group)) {
    group.vals <- .select_vec(data, group)
    if (!is.factor(group.vals)) data[[group]] <- factor(group.vals, levels = unique(group.vals))
  }

  # Keep only variables of interest (retain the pairing `id` column when given, #560)
  data <- data %>%
    df_select(vars = c(group.by, group, variables, id))

  # Case of formula with multiple variables
  #   1. Gather the data
  #   2. group by variable
  #   3. Perform pairwise test between levels of each grouing variable
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # ex: formula = c(GATA3, XBP1, DEPDC1) ~ group
  if (.is_multi_formula(formula)) {
    data <- df_gather(data, cols = variables, names_to = ".y.", values_to = ".value.") %>%
      dplyr::mutate(.y. = factor(.data$.y., levels = unique(.data$.y.)))
    response.var <- ".value."
    group.by <- c(group.by, ".y.")
    formula <- .collapse(glue::backtick(response.var), glue::backtick(group), sep = " ~ ") %>%
      stats::as.formula()
  }

  # Check if comparisons should be done against a reference group
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (!is.null(ref.group)) {
    group.vals <- .select_vec(data, group)
    if (is.factor(group.vals)) {
      group.levs <- levels(group.vals)
    } else {
      group.levs <- unique(group.vals)
    }

    if (ref.group %in% group.levs) {
      data[[group]] <- stats::relevel(group.vals, ref.group)
    }

    if (ref.group == ".all.") {
      data <- data %>%
        mutate(
          .group. = as.character(group.vals),
          .all. = ".all."
        ) # Add 'all' column
      # Create a new grouping column gathering group and the .all. columns
      .group.name. <- NULL
      data <- data %>%
        df_gather(cols = c(".group.", ".all."), names_to = ".group.name.", values_to = ".group.") %>%
        dplyr::select(-.group.name.)
      data[[".group."]] <- factor(data[[".group."]], levels = c(".all.", group.levs))
      group <- ".group."
      formula <- .collapse(glue::backtick(response.var), glue::backtick(group), sep = " ~ ") %>%
        stats::as.formula()
    } else if (!(ref.group %in% group.levs)) {
      stop("Can't find specified reference group: ", ref.group, ". ",
        "Allowed values include one of: ", .collapse(group.levs, sep = ", "),
        call. = FALSE
      )
    }
  }

  # Peform the test
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  test.func <- .test_pairwise
  if (method %in% c("anova", "kruskal.test")) {
    test.func <- .test_multigroups
  }

  if (is.null(group.by)) {
    res <- do.call(
      test.func,
      c(
        list(
          formula = formula, data = data, method = method,
          paired = paired, id = id,
          .skip_insufficient_id_pairs = .skip_insufficient_id_pairs,
          p.adjust.method = "none"
        ),
        extra.args
      )
    )
  } else {
    grouped.d <- .group_by(data, group.by)
    test.args <- c(
      list(
        formula = formula,
        method = method, paired = paired, id = id,
        .skip_insufficient_id_pairs = TRUE,
        p.adjust.method = "none"
      ),
      extra.args
    )
    res <- grouped.d %>%
      mutate(p = purrr::map(data, ~do.call(test.func, c(list(data = .x), test.args)))) %>%
      df_select(vars = c(group.by, "p")) %>%
      unnest(cols = "p")
  }

  # Add response variables to the result
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (!c(".y." %in% colnames(res))) {
    res <- res %>%
      dplyr::mutate(.y. = variables) %>%
      dplyr::select(!!!syms(c(group.by, ".y.")), dplyr::everything())
  }

  # Select only reference groups if any
  # Skip for anova/kruskal.test which don't have group1/group2 columns (Issue #572)
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (!is.null(ref.group) && !method %in% c("anova", "kruskal.test")) {
    group.levs <- .select_vec(data, group) %>% .levels()
    group1 <- group2 <- NULL
    res <- res %>% dplyr::filter(group1 == ref.group | group2 == ref.group)
    # ref.group should be always in group1 column
    # swap group1 and group2 if group2 contains ref.group
    group2 <- res$group2
    res <- transform(res,
      group1 = ifelse(group2 == ref.group, group2, group1),
      group2 = ifelse(group2 == ref.group, group1, group2)
    )
  }
  # Formatting and adjusting pvalues, and adding significance symbols
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  symnum.args$x <- res$p
  pvalue.signif <- do.call(stats::symnum, symnum.args) %>%
    as.character()

  pvalue.format <- format_p_value(res$p,
    style = p.format.style,
    digits = p.digits,
    leading.zero = p.leading.zero,
    min.threshold = p.min.threshold,
    decimal.mark = p.decimal.mark
  )

  .y. <- p.adj <- p <- NULL
  # Adjust p-values WITHIN each grouping level (group.by) x response (.y.), not
  # pooled across all groups, so a grouped adjustment matches filtering to one
  # group and adjusting there (#200). Without group.by this reduces to grouping
  # by .y. only (the previous behavior). Use group_by + mutate (not reframe) so
  # the row order is preserved and p.adj stays aligned with p.format/p.signif.
  adjust.by <- intersect(unique(c(group.by, ".y.")), colnames(res))
  res <- res %>%
    dplyr::group_by(dplyr::across(dplyr::all_of(adjust.by))) %>%
    dplyr::mutate(p.adj = stats::p.adjust(p, method = p.adjust.method)) %>%
    dplyr::ungroup() %>%
    mutate(
      p.format = pvalue.format, p.signif = pvalue.signif,
      method = method.name
    )

  # Resolve p.digits for adjusted p-value rounding
  p_params <- resolve_p_format_params(
    p.format.style, p.digits, p.leading.zero, p.min.threshold, p.decimal.mark
  )
  res %>%
    mutate(p.adj = signif(p.adj, digits = p_params$digits)) %>%
    add_p_format_signif() %>%
    tibble::as_tibble()
}


# Check and get test method info
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# return a list(method, name)
.method_info <- function(method) {
  if (is.null(method)) {
    method <- "wilcox.test"
  }

  allowed.methods <- list(
    t = "t.test", t.test = "t.test", student = "t.test",
    wiloxon = "wilcox.test", wilcox = "wilcox.test", wilcox.test = "wilcox.test",
    anova = "anova", aov = "anova",
    kruskal = "kruskal.test", kruskal.test = "kruskal.test"
  )

  method.names <- list(
    t.test = "T-test", wilcox.test = "Wilcoxon",
    anova = "Anova", kruskal.test = "Kruskal-Wallis"
  )

  if (!(method %in% names(allowed.methods))) {
    stop(
      "Non-supported method specified. Allowed methods are one of: ",
      .collapse(allowed.methods, sep = ", ")
    )
  }
  method <- allowed.methods[[method]]
  method.name <- method.names[[method]]

  list(method = method, name = method.name)
}

# Comparing two groups
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.test <- function(data, formula, method = "t.test", ...) {
  test <- match.fun(method)

  x <- .strip_backticks(deparse(formula[[2]]))
  group <- .strip_backticks(attr(stats::terms(formula), "term.labels"))

  if (.is_empty(group)) { # Case of null model
    test.opts <- list(x = .select_vec(data, x), ...)
  } else {
    test.opts <- list(formula = formula, data = data, ...)
  }

  res <- data.frame(p = suppressWarnings(do.call(test, test.opts)$p.value))
  group1 <- group2 <- NULL

  if (!.is_empty(group)) {
    group.lev <- .select_vec(data, group) %>% levels()
    res <- res %>%
      dplyr::mutate(
        group1 = group.lev[1],
        group2 = group.lev[2]
      ) %>%
      dplyr::select(group1, group2, dplyr::everything())
  } else {
    res <- res %>%
      dplyr::mutate(group1 = 1, group2 = "null model") %>%
      dplyr::select(group1, group2, everything())
  }
  res
}


# pairwise test
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.test_pairwise <- function(data, formula, method = "wilcox.test",
                           paired = FALSE, pool.sd = !paired,
                           id = NULL, .skip_insufficient_id_pairs = FALSE, ...) {
  x <- .strip_backticks(deparse(formula[[2]]))
  group <- .strip_backticks(attr(stats::terms(formula), "term.labels"))

  # #560: id-based paired test for a two-group comparison. Aligns the pairs by
  # subject id (row-order independent) via rstatix, which inner-joins on id, so
  # unbalanced/reordered data give the statistically correct paired p-value.
  # `id` is captured as an explicit formal so it never leaks into the base
  # pairwise test's `...`.
  if (!is.null(id) && isTRUE(paired) && !.is_empty(group)) {
    return(.paired_test_by_id(
      data, x = x, group = group, method = method, id = id,
      .skip_insufficient_pairs = .skip_insufficient_id_pairs,
      ...
    ))
  }

  # One sample test
  if (.is_empty(group)) {
    res <- .test(data, formula, method = method, ...)
    return(res)
  }

  # Pairwise test
  method <- switch(method,
    t.test = "pairwise.t.test",
    wilcox.test = "pairwise.wilcox.test"
  )
  test <- match.fun(method)

  test.opts <- list(
    x = .select_vec(data, x),
    g = .select_vec(data, group),
    paired = paired,
    ...
  )
  if (method == "pairwise.t.test") {
    if (missing(pool.sd)) {
      if (!paired) pool.sd <- FALSE
    }
    test.opts$pool.sd <- pool.sd
  }

  pvalues <- suppressWarnings(do.call(test, test.opts)$p.value) %>%
    as.data.frame()

  # No pairwise comparisons are possible when a subset has < 2 group levels.
  # This occurs in grouped workflows (e.g., some x-groups missing one legend level).
  # Return an empty result instead of erroring in tidyr::pivot_longer(cols = -"..group2..").
  if (ncol(pvalues) == 0) {
    return(tibble::tibble(
      group1 = character(),
      group2 = character(),
      p = numeric()
    ))
  }

  p <- NULL
  pvalues$..group2.. <- rownames(pvalues)
  pvalues <- pvalues %>%
    tidyr::pivot_longer(
      cols = -"..group2..",
      names_to = "..group1..",
      values_to = "p"
    ) %>%
    dplyr::select(group1 = "..group1..", group2 = "..group2..", p) %>%
    dplyr::filter(!is.na(p))
  pvalues
}

# Paired test(s) aligned by subject id (#560). Delegates to rstatix's id-aware
# paired test, which joins groups on the id column (row-order independent) and
# uses only the complete pairs per comparison, so mis-sorted or unbalanced data
# yield the statistically correct paired p-value(s). Handles two-group and
# pairwise (>2 groups) comparisons; ref.group is applied by the caller's
# post-filter. Returns the same group1 | group2 | p shape as .test_pairwise().
.paired_test_by_id <- function(data, x, group, method, id,
                               .skip_insufficient_pairs = FALSE, ...) {
  group.values <- .select_vec(data, group)
  groups <- .paired_id_group_names(group.values)
  if (length(groups) < 2L) {
    return(tibble::tibble(
      group1 = character(),
      group2 = character(),
      p = numeric()
    ))
  }
  test.fun <- switch(method,
    t.test = rstatix::t_test,
    wilcox.test = rstatix::wilcox_test
  )
  # Run on a small frame with syntactic names so the formula interface handles
  # any original column name (spaces, hyphens, ...). rstatix returns the actual
  # group values in group1/group2, so the labels are preserved.
  d2 <- data.frame(
    outcome = .select_vec(data, x),
    grp = group.values,
    subject = .select_vec(data, id),
    stringsAsFactors = FALSE
  )
  test.args <- list(...)
  test.args$p.adjust.method <- "none"
  test.args$detailed <- FALSE
  if (isTRUE(.skip_insufficient_pairs)) {
    test.args$error.as.na <- TRUE
  }
  res <- suppressWarnings(do.call(
    test.fun,
    c(
      list(data = d2, formula = outcome ~ grp, paired = TRUE, id = "subject"),
      test.args
    )
  ))
  if (isTRUE(.skip_insufficient_pairs)) {
    res <- res[!is.na(res$p), , drop = FALSE]
  }
  tibble::tibble(
    group1 = as.character(res$group1),
    group2 = as.character(res$group2),
    p = res$p
  )
}

.paired_id_group_names <- function(group.values) {
  group.values <- group.values[!is.na(group.values)]
  if (is.factor(group.values)) {
    as.character(levels(droplevels(group.values)))
  } else {
    unique(as.character(group.values))
  }
}

# Compare multiple groups
# ::::::::::::::::::::::::::::::::::::::::::::::::::

.test_multigroups <- function(data, formula, method = c("anova", "kruskal.test"),
                              .skip_insufficient_id_pairs = FALSE, ...) {
  method <- match.arg(method)
  . <- NULL

  if (method == "anova") {
    pvalue <- stats::lm(formula, data = data) %>%
      stats::anova(.) %>%
      .$`Pr(>F)` %>%
      .[1]
  } else if (method == "kruskal.test") {
    pvalue <- stats::kruskal.test(formula, data = data)$p.value
  }

  data.frame(p = pvalue)
}


# Formula with multiple response variables
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# ex formula = c(GATA3, XBP1, DEPDC1) ~ group
.is_multi_formula <- function(formula) {
  x <- grep(",", formula)
  !.is_empty(x)
}

# Get formula variables
.formula_left_variables <- function(formula) {
  . <- NULL
  x <- deparse(formula[[2]]) %>%
    gsub("c\\(|\\)|\\s", "", .) %>%
    strsplit(",") %>%
    unlist()
  x
}
.formula_right_variables <- function(formula) {
  group <- attr(stats::terms(formula), "term.labels")
  if (.is_empty(group)) group <- "1"
  # term.labels wraps names with special characters (e.g. spaces) in backticks,
  # which don't match a data column in dplyr/`[[`. Return the bare column name;
  # the formula object itself keeps its backticks for base-R/rstatix tests (#385).
  .strip_backticks(group)
}

# Remove the surrounding backticks R adds to non-syntactic names in formula terms
# so the string matches an actual data-frame column name.
.strip_backticks <- function(x) {
  gsub("`", "", x)
}

.update_test_arguments <- function(formula, data, group.by) {
  variables <- .formula_left_variables(formula)
  group <- .formula_right_variables(formula)

  data <- data %>%
    df_gather(cols = variables, names_to = ".y.", values_to = ".value.") %>%
    dplyr::mutate(.y. = factor(.data$.y., levels = unique(.data$.y.)))
  formula <- .collapse(".value.", group, sep = " ~ ") %>% stats::as.formula()
  group.by <- c(group.by, ".y.")
}

data("gene_expression", package = "ggpubr")
data("ToothGrowth")


test_that("compare_means works for One-sample test", {
  results <- compare_means(len ~ 1, ToothGrowth, mu = 0)
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  expected <- tibble::tribble(
    ~.y., ~group1, ~group2, ~p.format, ~p.signif, ~method,
    "len", 1, "null model", "1.7e-11", "****", "Wilcoxon"
  )
  expect_equal(results, expected)
})


test_that("compare_means works for Two-samples unpaired test", {
  results <- compare_means(len ~ supp, ToothGrowth)
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  # Accept either 0.064 (exact=FALSE) or 0.063 (R-devel exact conditional inference)
  expect_true(results$p.format %in% c("0.064", "0.063"),
    info = paste("Observed p.format =", results$p.format)
  )
  expect_equal(results$.y., "len")
  expect_equal(results$group1, "OJ")
  expect_equal(results$group2, "VC")
  expect_equal(results$p.signif, "ns")
  expect_equal(results$method, "Wilcoxon")
})


test_that("compare_means works for Two-samples paired test", {
  results <- compare_means(len ~ supp, ToothGrowth, paired = TRUE)
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  # Accept either 0.0043 (exact=FALSE) or 0.0038 (R-devel exact conditional inference)
  expect_true(results$p.format %in% c("0.0043", "0.0038"),
    info = paste("Observed p.format =", results$p.format)
  )
  expect_equal(results$.y., "len")
  expect_equal(results$group1, "OJ")
  expect_equal(results$group2, "VC")
  expect_equal(results$p.signif, "**")
  expect_equal(results$method, "Wilcoxon")
})


test_that("compare_means works for pairwise comparisons", {
  results <- compare_means(len ~ dose, ToothGrowth)
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  # Accept both exact=FALSE and R-devel exact conditional inference p-values
  legacy_p <- c("7.0e-06", "8.4e-08", "0.00018")
  rdevel_p <- c("7.7e-07", "4.4e-11", "7.6e-05")
  observed_p <- results$p.format
  expect_true(all(observed_p %in% c(legacy_p, rdevel_p)),
    info = paste("Observed p.format =", paste(observed_p, collapse = ", "))
  )
  expect_equal(results$.y., rep("len", 3))
  expect_equal(results$group1, c("0.5", "0.5", "1"))
  expect_equal(results$group2, c("1", "2", "2"))
  expect_true(all(results$p.signif %in% c("***", "****")),
    info = paste("Observed p.signif =", paste(results$p.signif, collapse = ", "))
  )
  expect_equal(results$method, rep("Wilcoxon", 3))
})

test_that("compare_means works for comparison against reference groups", {
  results <- compare_means(len ~ dose, ToothGrowth, ref.group = "0.5")
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  # Accept both exact=FALSE and R-devel exact conditional inference p-values
  legacy_p <- c("7.0e-06", "8.4e-08")
  rdevel_p <- c("7.7e-07", "4.4e-11")
  observed_p <- results$p.format
  expect_true(all(observed_p %in% c(legacy_p, rdevel_p)),
    info = paste("Observed p.format =", paste(observed_p, collapse = ", "))
  )
  expect_equal(results$.y., rep("len", 2))
  expect_equal(results$group1, c("0.5", "0.5"))
  expect_equal(results$group2, c("1", "2"))
  expect_equal(results$p.signif, rep("****", 2))
  expect_equal(results$method, rep("Wilcoxon", 2))
})


test_that("compare_means works when using formula with multiple variables", {
  results <- compare_means(c(GATA3, XBP1, PTEN) ~ dataset, data = gene_expression)
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  expected <- tibble::tribble(
    ~.y., ~group1, ~group2, ~p.format, ~p.signif, ~method,
    "GATA3", "BRCA", "OV", "< 2e-16", "****", "Wilcoxon",
    "GATA3", "BRCA", "LUSC", "< 2e-16", "****", "Wilcoxon",
    "GATA3", "OV", "LUSC", "3.0e-08", "****", "Wilcoxon",
    "XBP1", "BRCA", "OV", "< 2e-16", "****", "Wilcoxon",
    "XBP1", "BRCA", "LUSC", "< 2e-16", "****", "Wilcoxon",
    "XBP1", "OV", "LUSC", "4.2e-11", "****", "Wilcoxon",
    "PTEN", "BRCA", "OV", "6.8e-05", "****", "Wilcoxon",
    "PTEN", "BRCA", "LUSC", "< 2e-16", "****", "Wilcoxon",
    "PTEN", "OV", "LUSC", "1.3e-07", "****", "Wilcoxon"
  ) %>%
    dplyr::mutate(.y. = factor(.y., levels = c("GATA3", "XBP1", "PTEN")))
  expect_equal(results, expected)
})


test_that("compare_means works when using ref.group = '.all.'", {
  results <- compare_means(len ~ dose, ToothGrowth, ref.group = ".all.")
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  expected <- tibble::tribble(
    ~.y., ~group1, ~group2, ~p.format, ~p.signif, ~method,
    "len", ".all.", "0.5", "5.1e-05", "****", "Wilcoxon",
    "len", ".all.", "1", "0.76404", "ns", "Wilcoxon",
    "len", ".all.", "2", "0.00018", "***", "Wilcoxon"
  )
  expect_equal(results, expected)
})


test_that("compare_means works for Anova", {
  results <- compare_means(len ~ dose, ToothGrowth, method = "anova")
  results <- results %>%
    dplyr::select(.y., p.format, p.signif, method)
  expected <- tibble::tribble(
    ~.y., ~p.format, ~p.signif, ~method,
    "len", "9.5e-16", "****", "Anova"
  )
  expect_equal(results, expected)
})


test_that("compare_means works for kruskal.test", {
  results <- compare_means(len ~ dose, ToothGrowth, method = "kruskal.test")
  results <- results %>%
    dplyr::select(.y., p.format, p.signif, method)
  expected <- tibble::tribble(
    ~.y., ~p.format, ~p.signif, ~method,
    "len", "1.5e-09", "****", "Kruskal-Wallis"
  )
  expect_equal(results, expected)
})


test_that("compare_means works when grouping variable levels contain group2", {
  tibble::tribble(
    ~val, ~gp, ~by,
    923, "group4", "V1",
    1252, "group4", "V1",
    1442, "group4", "V1",
    1398, "group2", "V1",
    1858, "group2", "V1",
    1330, "group2", "V1",
    2593, "group2", "V1",
    23, "group4", "V2",
    252, "group4", "V2",
    442, "group4", "V2",
    398, "group2", "V2",
    858, "group2", "V2",
    330, "group2", "V2",
    593, "group2", "V2"
  ) %>% dplyr::mutate(gp = factor(gp, levels = c("group2", "group4"))) -> dat

  results <- compare_means(val ~ gp, data = dat, group.by = "by", paired = F)
  results <- results %>%
    dplyr::select(.y., group1, group2, p.format, p.signif, method)
  # Accept either 0.23 (exact=FALSE) or 0.22 (R-devel exact conditional inference)
  expect_true(all(results$p.format %in% c("0.23", "0.22")),
    info = paste("Observed p.format =", paste(results$p.format, collapse = ", "))
  )
  expect_equal(results$.y., rep("val", 2))
  expect_equal(results$group1, c("group2", "group2"))
  expect_equal(results$group2, c("group4", "group4"))
  expect_equal(results$p.signif, rep("ns", 2))
  expect_equal(results$method, rep("Wilcoxon", 2))
})

test_that("compare_means skips grouped subsets with <2 levels without error", {
  set.seed(1)
  dat <- data.frame(
    x = factor(c(rep("A", 8), rep("B", 8), rep("C", 8))),
    y = rnorm(24),
    gp = factor(
      c(rep("M", 8), rep(c("M", "F"), each = 4), rep("F", 8)),
      levels = c("M", "F")
    )
  )

  expect_no_error(
    results <- compare_means(y ~ gp, data = dat, group.by = "x")
  )

  expect_equal(unique(as.character(results$x)), "B")
  expect_equal(nrow(results), 1)
  expect_equal(as.character(results$group1), "M")
  expect_equal(as.character(results$group2), "F")
})

context("test-compare-means-paired-id")

# compare_means() gains an opt-in `id` argument for paired tests (#560). Without
# it, paired tests pair by ROW ORDER, so mis-sorted data give a wrong p-value.
# With `id`, the pairs are aligned by subject id (row-order independent) via
# rstatix. Default (id = NULL) is byte-identical.

suppressMessages(library(dplyr))

.make_paired <- function(seed = 123) {
  set.seed(seed)
  data.frame(
    sampleType = c(rep("Normal", 10), rep("Tumor", 10)),
    value = c(runif(10, 0, 0.2), runif(10, 0, 0.4)),
    ID = c(1:10, 10:1) # deliberately unsorted so row-order pairing is wrong
  )
}

.hand_paired <- function(d, method = "wilcox.test", ...) {
  x <- d %>% filter(sampleType == "Normal") %>% arrange(ID) %>% pull(value)
  y <- d %>% filter(sampleType == "Tumor") %>% arrange(ID) %>% pull(value)
  fun <- match.fun(method)
  suppressWarnings(fun(x, y, paired = TRUE, ...))$p.value
}

test_that("id yields the correct paired p-value regardless of row order (#560)", {
  d <- .make_paired()
  sorted <- d %>% arrange(sampleType, ID)
  hand <- .hand_paired(d, "wilcox.test")

  # id path matches the hand id-aligned test...
  expect_equal(compare_means(value ~ sampleType, d, paired = TRUE, id = "ID")$p, hand)
  # ...and is order-independent (unsorted == sorted)
  expect_equal(
    compare_means(value ~ sampleType, d, paired = TRUE, id = "ID")$p,
    compare_means(value ~ sampleType, sorted, paired = TRUE, id = "ID")$p
  )
  # the buggy row-order default differs from the correct value (this is the bug)
  expect_false(isTRUE(all.equal(
    compare_means(value ~ sampleType, d, paired = TRUE)$p, hand
  )))
})

test_that("id works for both wilcox.test and t.test", {
  d <- .make_paired()
  expect_equal(compare_means(value ~ sampleType, d, method = "wilcox.test", paired = TRUE, id = "ID")$p,
               .hand_paired(d, "wilcox.test"))
  expect_equal(compare_means(value ~ sampleType, d, method = "t.test", paired = TRUE, id = "ID")$p,
               .hand_paired(d, "t.test"))
})

test_that("id forwards one-sided paired test alternatives", {
  d <- .make_paired()
  for (method in c("wilcox.test", "t.test")) {
    for (alternative in c("greater", "less")) {
      got <- compare_means(
        value ~ sampleType, d,
        method = method, paired = TRUE, id = "ID",
        alternative = alternative
      )$p
      expect_equal(got, .hand_paired(d, method, alternative = alternative))
      expect_false(isTRUE(all.equal(
        got,
        compare_means(value ~ sampleType, d, method = method, paired = TRUE, id = "ID")$p
      )))
    }
  }
})

test_that("group1/group2 labels match the default convention", {
  d <- .make_paired()
  got <- compare_means(value ~ sampleType, d, paired = TRUE, id = "ID")
  def <- compare_means(value ~ sampleType, d, paired = TRUE)
  expect_equal(c(got$group1, got$group2), c(def$group1, def$group2))
})

test_that("id inner-joins on mismatched membership (uses only complete pairs)", {
  # group A ids {1,2,3}, group B ids {1,2,4}: only {1,2} are shared
  mm <- data.frame(
    g = c("A", "A", "A", "B", "B", "B"),
    value = c(1.2, 2.4, 3.1, 1.9, 2.8, 9.2),
    ID = c(1, 2, 3, 1, 2, 4)
  )
  got <- compare_means(value ~ g, mm, method = "t.test", paired = TRUE, id = "ID")$p
  hand <- t.test(c(1.2, 2.4), c(1.9, 2.8), paired = TRUE)$p.value
  expect_equal(got, hand)
})

test_that("id validation rejects unsupported combinations", {
  d <- .make_paired()
  expect_error(compare_means(value ~ sampleType, d, id = "ID"), "paired = TRUE")
  expect_error(compare_means(value ~ sampleType, d, method = "anova", paired = TRUE, id = "ID"),
               "wilcox.test")
  expect_error(compare_means(value ~ sampleType, d, paired = TRUE, id = "nope"),
               "can't find")
  # ref.group = ".all." (basemean) is unsupported with id
  expect_error(compare_means(value ~ sampleType, d, paired = TRUE, id = "ID", ref.group = ".all."),
               "\\.all\\.")
  # duplicate id within a group is rejected (by the underlying paired test)
  dup <- data.frame(g = c("A", "A", "B", "B"), value = c(1, 2, 3, 4.5), ID = c(1, 1, 1, 2))
  expect_error(compare_means(value ~ g, dup, method = "t.test", paired = TRUE, id = "ID"))
})

test_that("id duplicate errors are preserved in grouped comparisons", {
  dup_grouped <- data.frame(
    block = c(rep("x", 4), rep("y", 4)),
    g = c("A", "A", "B", "B", "A", "A", "B", "B"),
    value = c(1, 2, 3, 5, 10, 11, 20, 21),
    ID = c(1, 2, 1, 2, 1, 1, 1, 2)
  )
  expect_error(
    compare_means(
      value ~ g, dup_grouped,
      method = "t.test", paired = TRUE, id = "ID",
      group.by = "block"
    ),
    "duplicated ids"
  )
})

test_that("id works for pairwise (>2 groups) comparisons, matching rstatix (#560)", {
  set.seed(3)
  d <- data.frame(g = rep(c("A", "B", "C"), each = 8),
                  value = rnorm(24),
                  ID = c(1:8, sample(1:8), sample(1:8)))  # unsorted
  got <- compare_means(value ~ g, d, method = "wilcox.test", paired = TRUE, id = "ID")
  ref <- rstatix::wilcox_test(d, value ~ g, paired = TRUE, id = "ID", p.adjust.method = "none")
  # same set of comparisons and p-values as a direct rstatix call
  key <- function(x) paste(x$group1, x$group2, signif(x$p, 6))
  expect_setequal(key(got), key(ref))
  expect_equal(nrow(got), 3L)  # A-B, A-C, B-C
})

test_that("id skips grouped subsets with fewer than two levels", {
  d <- data.frame(
    block = c(rep("x", 6), rep("y", 3)),
    g = c("A", "A", "A", "B", "B", "B", "A", "A", "A"),
    value = c(1, 2.5, 4, 2.1, 3, 6.2, 5, 6.5, 8),
    ID = c(1, 2, 3, 1, 2, 3, 1, 2, 3)
  )
  expect_no_error(
    got <- compare_means(
      value ~ g, d,
      method = "t.test", paired = TRUE, id = "ID",
      group.by = "block"
    )
  )
  expect_equal(unique(as.character(got$block)), "x")
  expect_equal(nrow(got), 1L)
  expect_equal(c(got$group1, got$group2), c("A", "B"))
})

test_that("id skips grouped t-test subsets with too few complete pairs", {
  d <- data.frame(
    block = c(rep("x", 6), rep("y", 4)),
    g = c("A", "A", "A", "B", "B", "B", "A", "A", "B", "B"),
    value = c(1, 2.5, 4, 2.1, 3, 6.2, 5, 6.5, 8, 9),
    ID = c(1, 2, 3, 1, 2, 3, 1, 2, 3, 4)
  )
  expect_no_error(
    got <- compare_means(
      value ~ g, d,
      method = "t.test", paired = TRUE, id = "ID",
      group.by = "block"
    )
  )
  expect_equal(unique(as.character(got$block)), "x")
  expect_equal(nrow(got), 1L)
  expect_equal(c(got$group1, got$group2), c("A", "B"))
})

test_that("id skips only invalid grouped pairwise t-test comparisons", {
  d <- data.frame(
    block = c(rep("x", 9), rep("y", 6)),
    g = c(rep(c("A", "B", "C"), each = 3), rep(c("A", "B", "C"), each = 2)),
    value = c(1, 2, 4, 2.2, 4.1, 7.4, 1.4, 2.9, 5.5, 10, 11, 20, 21, 30, 32),
    ID = c(1, 2, 3, 1, 2, 3, 1, 2, 3, 1, 2, 3, 4, 1, 2)
  )
  expect_no_error(
    got <- compare_means(
      value ~ g, d,
      method = "t.test", paired = TRUE, id = "ID",
      group.by = "block"
    )
  )
  got$key <- paste(as.character(got$block), got$group1, got$group2)
  expect_setequal(got$key, c("x A B", "x A C", "x B C", "y A C"))
})

test_that("id keeps grouped Wilcoxon comparisons with one complete pair", {
  d <- data.frame(
    block = c(rep("x", 4), rep("y", 2)),
    g = c("A", "A", "B", "B", "A", "B"),
    value = c(1, 3, 2, 5, 10, 12),
    ID = c(1, 2, 1, 2, 1, 1)
  )
  expect_no_error(
    got <- compare_means(
      value ~ g, d,
      method = "wilcox.test", paired = TRUE, id = "ID",
      group.by = "block"
    )
  )
  expect_setequal(as.character(got$block), c("x", "y"))
  expect_equal(nrow(got), 2L)
})

test_that("id works together with ref.group (only ref comparisons, matching rstatix)", {
  set.seed(4)
  d <- data.frame(g = rep(c("ctrl", "t1", "t2"), each = 8),
                  value = rnorm(24), ID = c(1:8, sample(1:8), sample(1:8)))
  got <- compare_means(value ~ g, d, method = "t.test", paired = TRUE, id = "ID", ref.group = "ctrl")
  expect_equal(nrow(got), 2L)                 # ctrl vs t1, ctrl vs t2
  expect_true(all(got$group1 == "ctrl"))      # ref is always group1
  ref <- rstatix::t_test(d, value ~ g, paired = TRUE, id = "ID", ref.group = "ctrl",
                         p.adjust.method = "none")
  expect_equal(sort(got$p), sort(ref$p))
})

test_that("id path works with a non-syntactic (hyphenated) group column (#385 parity)", {
  d <- data.frame(
    `grp-x` = c("A", "A", "B", "B"),
    value = c(1.2, 2.4, 1.9, 2.8),
    ID = c(1, 2, 1, 2),
    check.names = FALSE
  )
  got <- compare_means(value ~ `grp-x`, d, method = "t.test", paired = TRUE, id = "ID")$p
  hand <- t.test(c(1.2, 2.4), c(1.9, 2.8), paired = TRUE)$p.value
  expect_equal(got, hand)
  # and matches the default path's handling of the same column
  expect_equal(got, compare_means(value ~ `grp-x`, d %>% dplyr::arrange(`grp-x`, ID),
                                   method = "t.test", paired = TRUE)$p)
})

test_that("id type errors give a clear message", {
  d <- .make_paired()
  expect_error(compare_means(value ~ sampleType, d, paired = TRUE, id = 1),
               "single column name")
  expect_error(compare_means(value ~ sampleType, d, paired = TRUE, id = c("ID", "x")),
               "single column name")
})

test_that("default (id = NULL) is byte-identical to omitting it", {
  set.seed(1)
  dd <- data.frame(g = rep(c("A", "B"), each = 8), v = rnorm(16), ID = rep(1:8, 2))
  # paired and unpaired, with and without group.by
  expect_identical(compare_means(v ~ g, dd, paired = TRUE),
                   compare_means(v ~ g, dd, paired = TRUE, id = NULL))
  expect_identical(compare_means(v ~ g, dd),
                   compare_means(v ~ g, dd, id = NULL))
  dd$blk <- rep(c("x", "y"), 8)
  expect_identical(compare_means(v ~ g, dd, group.by = "blk", paired = TRUE),
                   compare_means(v ~ g, dd, group.by = "blk", paired = TRUE, id = NULL))
})

test_that("id column is retained (not dropped by the internal column selection)", {
  # regression guard for the df_select() fix that keeps the id column
  d <- .make_paired()
  expect_error(compare_means(value ~ sampleType, d, paired = TRUE, id = "ID"), NA)
})

# Regression tests for #385: compare_means() errored ("Can't extract columns that
# don't exist") when a formula variable name contains a space, because
# term.labels wraps such names in backticks, which don't match a data column in
# dplyr / `[[`.

.df <- ToothGrowth
.df[["spa ced"]] <- .df$supp
.df$dose <- factor(.df$dose)

test_that("compare_means() works with a spaced group name (#385)", {
  expect_no_error(res <- compare_means(len ~ `spa ced`, data = .df))
  # same result as the equivalent unspaced column (spaced is a copy of supp)
  ref <- compare_means(len ~ supp, data = ToothGrowth)
  expect_equal(res$p, ref$p)
  expect_equal(as.character(res$group1), as.character(ref$group1))
  expect_equal(as.character(res$group2), as.character(ref$group2))
})

test_that("spaced group name works for anova and with group.by (#385)", {
  expect_no_error(compare_means(len ~ `spa ced`, data = .df, method = "anova"))
  expect_no_error(compare_means(len ~ `spa ced`, data = .df, group.by = "dose"))
})

test_that("normal (unspaced) formulas are unchanged (no regression, #385)", {
  a <- compare_means(len ~ supp, data = ToothGrowth)
  expect_equal(a$method, "Wilcoxon")
  expect_equal(nrow(a), 1L)
  # multi-group + group.by path still works
  expect_no_error(compare_means(len ~ supp, data = ToothGrowth, group.by = "dose"))
})

test_that(".strip_backticks removes backticks only (#385)", {
  expect_equal(ggpubr:::.strip_backticks("`spa ced`"), "spa ced")
  expect_equal(ggpubr:::.strip_backticks("supp"), "supp")
})

# Regression tests for #200: compare_means() adjusted p-values across ALL groups
# pooled together instead of within each group.by level, so a grouped adjustment
# did not match filtering to one group and adjusting there.

suppressPackageStartupMessages(library(dplyr))

test_that("grouped p.adjust matches filter-then-adjust within a group (#200)", {
  grouped <- compare_means(len ~ dose, ToothGrowth, method = "t.test",
                           p.adjust.method = "bonferroni", group.by = "supp") %>%
    as.data.frame()
  filtered <- ToothGrowth %>% filter(supp == "OJ") %>%
    compare_means(len ~ dose, ., method = "t.test",
                  p.adjust.method = "bonferroni", group.by = "supp") %>%
    as.data.frame()
  oj <- grouped[grouped$supp == "OJ", ]
  oj <- oj[order(oj$group1, oj$group2), ]
  filtered <- filtered[order(filtered$group1, filtered$group2), ]
  expect_equal(oj$p.adj, filtered$p.adj)
})

test_that("grouped bonferroni adjusts by within-group comparison count (#200)", {
  grouped <- compare_means(len ~ dose, ToothGrowth, method = "t.test",
                           p.adjust.method = "bonferroni", group.by = "supp") %>%
    as.data.frame()
  # 3 comparisons per supp group -> p.adj ~= min(1, p * 3), NOT p * 6
  # (tolerance covers the 2-sig-fig rounding but still rules out the old x6)
  expect_equal(grouped$p.adj, pmin(1, grouped$p * 3), tolerance = 0.05)
})

test_that("ungrouped p.adjust is unchanged: adjusts across all comparisons (no regression, #200)", {
  ungrouped <- compare_means(len ~ dose, ToothGrowth, method = "t.test",
                             p.adjust.method = "bonferroni") %>%
    as.data.frame()
  # 3 comparisons total -> p * 3 (same as before this change)
  expect_equal(ungrouped$p.adj, pmin(1, ungrouped$p * 3), tolerance = 0.05)
})

test_that("p.adj stays aligned with p / group1 / group2 rows (#200)", {
  r <- compare_means(len ~ dose, ToothGrowth, method = "t.test",
                     p.adjust.method = "bonferroni", group.by = "supp") %>%
    as.data.frame()
  # p.adj must be >= p for every row (adjustment never decreases a p-value here)
  expect_true(all(r$p.adj >= r$p - 1e-12))
  expect_equal(nrow(r), 6L)
})

#' @include utilities.R ggpar.R
NULL
#' Empirical cumulative density function
#' @description Empirical Cumulative Density Function (ECDF).
#' @inheritParams ggboxplot
#' @param x variable to be drawn.
#' @param color line and point color.
#' @param linetype line type. See \code{\link{show_line_types}}.
#' @param size line and point size.
#' @param ... other arguments to be passed to
#'   \code{\link[ggplot2]{stat_ecdf}} and \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}
#'
#' @examples
#' # Create some data format
#' set.seed(1234)
#' wdata <- data.frame(
#'   sex = factor(rep(c("F", "M"), each = 200)),
#'   weight = c(rnorm(200, 55), rnorm(200, 58))
#' )
#'
#' head(wdata, 4)
#'
#' # Basic ECDF plot
#' ggecdf(wdata, x = "weight")
#'
#' # Change colors and linetype by groups ("sex")
#' # Use custom palette
#' ggecdf(wdata,
#'   x = "weight",
#'   color = "sex", linetype = "sex",
#'   palette = c("#00AFBB", "#E7B800")
#' )
#'
#' @export
ggecdf <- function(data, x, combine = FALSE, merge = FALSE,
                   color = "black", palette = NULL,
                   size = NULL, linetype = "solid",
                   title = NULL, xlab = NULL, ylab = NULL,
                   facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                   ggtheme = theme_pubr(),
                   ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, palette = palette,
    linetype = linetype, size = size,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }
  .opts$fun <- ggecdf_core
  .opts$y <- "..ecdf.."
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}

ggecdf_core <- function(data, x, y = "..ecdf..",
                        color = "black", palette = NULL,
                        size = NULL, linetype = "solid",
                        title = NULL, xlab = NULL, ylab = NULL,
                        ggtheme = theme_classic(),
                        ...) {
  p <- ggplot(data, create_aes(list(x = x)))

  p <- p +
    geom_exec(stat_ecdf,
      data = data,
      color = color, linewidth = size,
      linetype = linetype
    ) +
    labs(y = paste0("F(", x, ")"))
  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )
  p
}

#' @include utilities.R ggpar.R
NULL
#' Bar plot
#' @description Create a bar plot.
#' @inheritParams ggboxplot
#' @inheritParams ggplot2::geom_bar
#' @param x,y x and y variables for drawing.
#' @param color,fill outline and fill colors.
#' @param sort.val a string specifying whether the value should be sorted.
#' Allowed values are "none" (no sorting), "asc" (for ascending) or "desc" (for descending).
#' @param sort.by.groups logical value. If TRUE the data are sorted by groups.
#' Used only when sort.val != "none".
#' @param top a numeric value specifying the number of top elements to be shown.
#' @param numeric.x.axis logical. If TRUE, x axis will be treated as numeric.
#'   Default is FALSE. Useful, for example, to plot bars at their numeric x
#'   positions (e.g. a time axis) instead of at equally-spaced discrete
#'   categories. Ignored when \code{order} is set or \code{sort.val != "none"},
#'   which require a discrete x axis.
#' @param label specify whether to add labels on the bar plot. Allowed values
#'   are: \itemize{ \item \strong{logical value}: If TRUE, y values are added as
#'   labels on the bar plot \item \strong{character vector}: Used as text
#'   labels; must be the same length as y. }
#' @param lab.col,lab.size text color and size for labels.
#' @param lab.pos character specifying the position for labels. Allowed values
#'   are "out" (for outside) or "in" (for inside). Ignored when lab.vjust !=
#'   NULL.
#' @param lab.vjust numeric, vertical justification of labels. Provide negative
#'   value (e.g.: -0.4) to put labels outside the bars or positive value to put
#'   labels inside (e.g.: 2).
#' @param lab.hjust numeric, horizontal justification of labels.
#' @param lab.nb.digits integer indicating the number of decimal places (round) to be used.
#' @param ... other arguments to be passed to be passed to ggpar().
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}, \code{\link{ggline}}
#' @examples
#' # Data
#' df <- data.frame(
#'   dose = c("D0.5", "D1", "D2"),
#'   len = c(4.2, 10, 29.5)
#' )
#' print(df)
#'
#' # Basic plot with label outsite
#' # +++++++++++++++++++++++++++
#' ggbarplot(df,
#'   x = "dose", y = "len",
#'   label = TRUE, label.pos = "out"
#' )
#'
#' # Change width
#' ggbarplot(df, x = "dose", y = "len", width = 0.5)
#'
#' # Change the plot orientation: horizontal
#' ggbarplot(df, "dose", "len", orientation = "horiz")
#'
#' # Change the default order of items
#' ggbarplot(df, "dose", "len",
#'   order = c("D2", "D1", "D0.5")
#' )
#'
#'
#' # Change colors
#' # +++++++++++++++++++++++++++
#'
#' # Change fill and outline color
#' # add labels inside bars
#' ggbarplot(df, "dose", "len",
#'   fill = "steelblue", color = "steelblue",
#'   label = TRUE, lab.pos = "in", lab.col = "white"
#' )
#'
#' # Change colors by groups: dose
#' # Use custom color palette
#' ggbarplot(df, "dose", "len",
#'   color = "dose",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#' # Change fill and outline colors by groups
#' ggbarplot(df, "dose", "len",
#'   fill = "dose", color = "dose",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#'
#' # Plot with multiple groups
#' # +++++++++++++++++++++
#'
#' # Create some data
#' df2 <- data.frame(
#'   supp = rep(c("VC", "OJ"), each = 3),
#'   dose = rep(c("D0.5", "D1", "D2"), 2),
#'   len = c(6.8, 15, 33, 4.2, 10, 29.5)
#' )
#' print(df2)
#'
#' # Plot "len" by "dose" and change color by a second group: "supp"
#' # Add labels inside bars
#' ggbarplot(df2, "dose", "len",
#'   fill = "supp", color = "supp", palette = "Paired",
#'   label = TRUE, lab.col = "white", lab.pos = "in"
#' )
#'
#' # Change position: Interleaved (dodged) bar plot
#' ggbarplot(df2, "dose", "len",
#'   fill = "supp", color = "supp", palette = "Paired",
#'   label = TRUE,
#'   position = position_dodge(0.9)
#' )
#'
#' # Add points and errors
#' # ++++++++++++++++++++++++++
#'
#' # Data: ToothGrowth data set we'll be used.
#' df3 <- ToothGrowth
#' head(df3, 10)
#'
#' # It can be seen that for each group we have
#' # different values
#' ggbarplot(df3, x = "dose", y = "len")
#'
#' # Visualize the mean of each group
#' ggbarplot(df3,
#'   x = "dose", y = "len",
#'   add = "mean"
#' )
#'
#' # Add error bars: mean_se
#' # (other values include: mean_sd, mean_ci, median_iqr, ....)
#' # Add labels
#' ggbarplot(df3,
#'   x = "dose", y = "len",
#'   add = "mean_se", label = TRUE, lab.vjust = -1.6
#' )
#'
#' # Use only "upper_errorbar"
#' ggbarplot(df3,
#'   x = "dose", y = "len",
#'   add = "mean_se", error.plot = "upper_errorbar"
#' )
#'
#' # Change error.plot to "pointrange"
#' ggbarplot(df3,
#'   x = "dose", y = "len",
#'   add = "mean_se", error.plot = "pointrange"
#' )
#'
#' # Add jitter points and errors (mean_se)
#' ggbarplot(df3,
#'   x = "dose", y = "len",
#'   add = c("mean_se", "jitter")
#' )
#'
#' # Add dot and errors (mean_se)
#' ggbarplot(df3,
#'   x = "dose", y = "len",
#'   add = c("mean_se", "dotplot")
#' )
#'
#' # Multiple groups with error bars and jitter point
#' ggbarplot(df3,
#'   x = "dose", y = "len", color = "supp",
#'   add = "mean_se", palette = c("#00AFBB", "#E7B800"),
#'   position = position_dodge()
#' )
#' #
#'
#' @section Faceting a summarized bar plot:
#' When the bars show a computed summary (e.g. \code{add = "mean_se"}), facet the
#' plot with the \code{facet.by} argument - \strong{not} by appending
#' \code{+ facet_wrap()} / \code{+ facet_grid()}. The summaries are pre-computed,
#' grouping by \code{x}, \code{color}/\code{fill} and \code{facet.by}; a facet added
#' afterwards is not part of that grouping, so the bars (and, for stacked bars, the
#' error bars) are pooled over the whole data set and repeated identically in every
#' panel. Use \code{ggbarplot(..., facet.by = "group")} for correct per-panel
#' summaries.
#'
#' @export
ggbarplot <- function(data, x, y, combine = FALSE, merge = FALSE,
                      color = "black", fill = "white", palette = NULL,
                      size = NULL, width = NULL,
                      title = NULL, xlab = NULL, ylab = NULL,
                      facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                      select = NULL, remove = NULL, order = NULL,
                      add = "none", add.params = list(), error.plot = "errorbar",
                      label = FALSE, lab.col = "black", lab.size = 4,
                      lab.pos = c("out", "in"), lab.vjust = NULL, lab.hjust = NULL,
                      lab.nb.digits = NULL,
                      sort.val = c("none", "desc", "asc"), sort.by.groups = TRUE,
                      top = Inf,
                      position = position_stack(),
                      numeric.x.axis = FALSE,
                      ggtheme = theme_pubr(),
                      ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    size = size, width = width,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot,
    label = label, lab.col = lab.col, lab.size = lab.size,
    lab.pos = lab.pos, lab.vjust = lab.vjust, lab.hjust = lab.hjust,
    lab.nb.digits = lab.nb.digits,
    sort.val = sort.val, sort.by.groups = sort.by.groups, top = top,
    position = position, numeric.x.axis = numeric.x.axis, ggtheme = ggtheme, ...
  )

  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  if (is.logical(merge)) {
    if (merge & missing(position)) {
      .opts$position <- position_dodge(0.8)
    }
    if (merge & missing(lab.col)) {
      .opts$lab.col <- ".y."
    }
  } else if (is.character(merge)) {
    .opts$position <- position_dodge(0.8)
  }

  .opts$fun <- ggbarplot_core
  .opts$fun_name <- "barplot"
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}

ggbarplot_core <- function(data, x, y,
                           color = "black", fill = "white", palette = NULL,
                           size = NULL, width = 0.7,
                           title = NULL, xlab = NULL, ylab = NULL,
                           label = FALSE, lab.col = "black", lab.size = 4,
                           lab.pos = c("out", "in"), lab.vjust = NULL, lab.hjust = NULL,
                           lab.nb.digits = NULL,
                           select = NULL, order = NULL, facet.by = NULL,
                           sort.val = c("none", "desc", "asc"), sort.by.groups = TRUE,
                           merge = FALSE,
                           top = Inf,
                           add = "none",
                           add.params = list(),
                           error.plot = "errorbar",
                           position = position_stack(),
                           numeric.x.axis = FALSE,
                           ggtheme = theme_pubr(),
                           ...) {
  sort.val <- match.arg(sort.val)
  xx <- .select_vec(data, x)
  if (!is.null(order)) {
    data[[x]] <- factor(data[[x]], levels = order)
  } else if (inherits(xx, c("character", "numeric")) & !numeric.x.axis) {
    data[[x]] <- as.factor(data[[x]])
  }
  error.plot <- error.plot[1]
  lab.pos <- match.arg(lab.pos)
  label <- as.vector(label)
  if ("none" %in% add) add <- "none"

  . <- NULL

  # #404: an `alpha` aesthetic mapped to a discrete data column defines an extra
  # dodge subgroup (e.g. fill = cut, alpha = clarity -> 2 bars per cut). Detect it
  # so the summary keeps that column and the error layer dodges by it too. Scoped
  # to plain position_dodge(): the alpha subgroup adds a second dodge dimension,
  # which position_stack()/position_dodge2() do not resolve here (they are left
  # exactly as before, unchanged by this fix).
  alpha.var <- list(...)[["alpha"]]   # [[ ]] avoids $ partial-matching a `...` arg
  has.alpha.group <- !is.null(alpha.var) && length(alpha.var) == 1 &&
    is.character(alpha.var) && alpha.var %in% names(data) &&
    !is.numeric(.select_vec(data, alpha.var)) &&
    inherits(position, "PositionDodge") && !inherits(position, "PositionDodge2")

  grouping.vars <- intersect(c(x, color, fill, facet.by), names(data))
  # Include the alpha subgroup in the summary grouping. Otherwise the summarized
  # data drops the alpha column, which (a) makes geom_exec pass alpha as a static
  # value -> the "alpha * 255" draw error (#404), and (b) collapses the mean/CI
  # across the subgroups. Left unchanged when no discrete alpha var is mapped.
  if (has.alpha.group) grouping.vars <- unique(c(grouping.vars, alpha.var))

  # static summaries for computing mean/median and adding errors
  if (is.null(add.params$fill)) add.params$fill <- "white"
  if (is.null(add.params$group)) {
    if (fill %in% names(data)) {
      add.params$group <- fill
    } else if (color %in% names(data)) add.params$group <- color
  }
  # #404: the bars dodge by the interaction of fill/x and the alpha subgroup, so
  # the error layer must dodge by that same interaction to stay aligned; otherwise
  # the error bars are centered on each x while the bars are split (misaligned). We
  # materialise the interaction as a real column with a safe name (rather than an
  # "interaction(a, b)" mapping string), so it is robust to special characters in
  # the variable names and to options(ggpubr.parse_aes = FALSE).
  if (has.alpha.group) {
    base.group <- add.params$group %||% x
    data[[".ggpubr.alpha.group."]] <- interaction(
      .select_vec(data, base.group), .select_vec(data, alpha.var), drop = TRUE
    )
    add.params$group <- ".ggpubr.alpha.group."
  }
  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill, ...)

  if (any(.summary_functions() %in% add)) {
    data_sum <- desc_statby(data, measure.var = y, grps = grouping.vars)
    summary.funcs <- intersect(.summary_functions(), add)
    if (length(summary.funcs) > 1) {
      stop(
        "Only one summary function is allowed. ",
        "Choose one of ", .collapse(.summary_functions(), sep = ", ")
      )
    }
    .center <- .get_errorbar_center_func(summary.funcs)

    add <- setdiff(add, .center)
    names(data_sum)[which(names(data_sum) == .center)] <- y
    if (inherits(xx, c("character", "numeric")) & !numeric.x.axis) {
      data_sum[, x] <- .select_vec(data_sum, x) %>% as.factor()
    }
  } else {
    data_sum <- data
  }

  # Sorting
  # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if (top != Inf & sort.val == "none") sort.val <- "desc"
  if (top != Inf) {
    data_sum <- data_sum[order(-data_sum[, y]), ]
    data_sum <- utils::head(data_sum, n = top)
  }
  grps <- unique(intersect(c(color, fill), names(data)))
  if (length(grps) > 0) grps <- .get_not_numeric_vars(data[, grps, drop = FALSE])
  ngrps <- length(grps)
  if (!sort.by.groups) ngrps <- 0
  # Variables for ordering
  if (ngrps > 0) {
    dd <- data_sum[, c(grps, y)]
  } else {
    dd <- data_sum[, y, drop = FALSE]
  }
  if (sort.val == "desc") dd[, y] <- -dd[, y]
  # Sorting
  if (sort.val != "none") {
    if (ngrps == 0) {
      data_sum <- data_sum[order(dd[, y]), ]
    } else if (ngrps == 1) {
      data_sum <- data_sum[order(dd[, 1], dd[, y]), ]
    } else if (ngrps == 2) data_sum <- data_sum[order(dd[, 1], dd[, 2], dd[, y]), ]
    data_sum[, x] <- factor(data_sum[, x], levels = unique(data_sum[, x]))
  }

  # Main plot
  # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if (inherits(position, "PositionDodge") & is.null(position$width)) position$width <- 0.95
  p <- ggplot(data, create_aes(list(x = x, y = y)))
  p <- p +
    geom_exec(geom_bar,
      data = data_sum,
      stat = "identity",
      color = color, fill = fill,
      position = position,
      size = size, width = width, ...
    )

  # Add errors
  add.params <- add.params %>% .add_item(p = p, error.plot = error.plot)
  is.stacked.position <- inherits(position, "PositionStack")
  stack.groups <- unique(c(x, facet.by))
  nb.bars.by.xposition <- data_sum %>%
    group_by(!!!syms(stack.groups)) %>%
    dplyr::count() %>%
    dplyr::pull(.data$n) %>%
    max()
  if (is.stacked.position) {
    add.position <- "identity"
  } else {
    add.position <- position
  }
  if (is.stacked.position & nb.bars.by.xposition >= 2) {
    p <- add.params %>%
      .add_item(add = .remove_errorbar_func(add), position = add.position) %>%
      do.call(ggadd, .)
    if (any(.errorbar_functions() %in% add)) {
      p <- p + .geom_stacked_errorbar(
        data_sum, x, y,
        color = add.params$color, fill = add.params$fill,
        group = add.params$group, facet.by = facet.by,
        func = .get_summary_func(add), error.plot = error.plot
      )
    }
  } else if (inherits(position, "PositionDodge2") &&
             nb.bars.by.xposition >= 2 &&
             any(.errorbar_functions() %in% add) &&
             error.plot %in% .narrow_error_plots()) {
    # position_dodge2() misplaces thin error bars relative to the dodged bars
    # (#363). Draw any non-error add layers normally, then re-center the error
    # bars on the actual bar positions.
    p <- add.params %>%
      .add_item(add = .remove_errorbar_func(add), position = add.position) %>%
      do.call(ggadd, .)
    cap.width <- (add.params$width %||% 0.1) / nb.bars.by.xposition
    eb <- .geom_dodge2_errorbar(
      p, data_sum, x, y,
      color = add.params$color, fill = add.params$fill,
      group = add.params$group, facet.by = facet.by,
      func = .get_summary_func(add), error.plot = error.plot,
      width = cap.width
    )
    if (!is.null(eb)) {
      p <- p + eb
    } else {
      # Could not match centres -> keep the standard (unaligned) error layer
      p <- add.params %>%
        .add_item(add = .get_summary_func(add), position = add.position) %>%
        do.call(ggadd, .)
    }
  } else {
    p <- add.params %>%
      .add_item(add = add, position = add.position) %>%
      do.call(ggadd, .)
  }


  # Add labels
  add.label <- FALSE
  if (is.logical(label)) {
    .lab <- y
    add.label <- label
  } else {
    # Add user specified labels as data column
    data_sum$.ulabel. <- label
    .lab <- ".ulabel."
    add.label <- TRUE
  }

  if (add.label) {
    if (is.null(lab.vjust)) lab.vjust <- ifelse(lab.pos == "out", -0.4, 2)
    if (is.null(lab.hjust)) lab.hjust <- 0.5
    if (!is.null(lab.nb.digits)) {
      if (is.numeric(.lab)) {
        .lab <- round(.lab, digits = lab.nb.digits)
      } else if (.lab[1] %in% colnames(data_sum)) {
        data_sum[, .lab] <- dplyr::pull(data_sum, .lab) %>%
          round(digits = lab.nb.digits)
      }
    }

    # pos <- "identity"
    # if color or fill by groups
    .cols <- unique(c(color, fill))
    if (any(.cols %in% names(data))) {
      .in <- which(.cols %in% names(data))
      lab.fill <- color.var <- .cols[.in]
      data_sum <- data_sum %>%
        dplyr::arrange(!!!syms(x), desc(!!!syms(color.var)))

      group <- intersect(.cols, names(data))[1] # You should specify group for dodging text

      p <- p + geom_exec(geom_text,
        data = data_sum, label = .lab, # fill = lab.fill
        vjust = lab.vjust, hjust = lab.hjust, size = lab.size, color = lab.col,
        fontface = "plain", position = position, group = group
      )
    } else {
      p <- p + geom_exec(geom_text,
        data = data_sum, label = .lab,
        vjust = lab.vjust, hjust = lab.hjust, size = lab.size, color = lab.col,
        fontface = "plain", position = position
      )
    }
  }
  # To do
  # top10, visualizing error
  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )

  p
}

# Stacked error bar ----------------------------
.geom_stacked_errorbar <- function(data_sum, x, y, color = NULL, fill = NULL, facet.by = NULL, group = NULL,
                                   func = "mean_se", error.plot = "errorbar") {
  stack.groups <- unique(c(x, facet.by))
  legend.var <- intersect(unique(c(color, fill, group)), colnames(data_sum))
  error <- .get_errorbar_error_func(func)
  error.value <- data_sum %>% dplyr::pull(!!error)
  desc <- dplyr::desc
  errorbar.position <- data_sum %>%
    group_by(!!!syms(stack.groups)) %>%
    dplyr::arrange(!!sym(x), desc(!!sym(legend.var))) %>%
    dplyr::mutate(
      # position_stack() accumulates positive and negative segments SEPARATELY
      # (positives stack up from 0, negatives stack down from 0), so a mixed-sign
      # stack must cumulate each sign on its own. A single cumsum() over both signs
      # places the error bars of one sign on the wrong side (#426). For single-sign
      # data one of the two cumulative sums is identically zero, so this reduces to
      # the original cumsum() and the output is byte-identical.
      .ggpubr_cum_pos = cumsum(pmax(!!sym(y), 0)),
      .ggpubr_cum_neg = cumsum(pmin(!!sym(y), 0)),
      y = ifelse(!!sym(y) >= 0, .data$.ggpubr_cum_pos, .data$.ggpubr_cum_neg),
      ymin = .data$y - !!sym(error),
      ymax = .data$y + !!sym(error)
    ) %>%
    dplyr::select(-".ggpubr_cum_pos", -".ggpubr_cum_neg") %>%
    dplyr::ungroup()
  geom_error <- .get_geom_error_function(error.plot)

  args <- geom_exec(
    data = errorbar.position, color = color,
    group = group,
    x = x, ymin = "ymin", ymax = "ymax"
  )
  mapping <- args$mapping
  option <- args$option
  if (error.plot == "errorbar") option$width <- 0.15
  option[["mapping"]] <- create_aes(mapping)
  do.call(geom_error, option)
}


.get_geom_error_function <- function(error.plot = "errorbar") {
  error.plot <- error.plot[1]
  geom_func <- ggplot2::geom_errorbar
  if (error.plot %in% c("pointrange", "lower_pointrange", "upper_pointrange")) {
    geom_func <- ggplot2::geom_pointrange
  } else if (error.plot %in% c("linerange", "lower_linerange", "upper_linerange")) {
    geom_func <- ggplot2::geom_linerange
  } else if (error.plot %in% c("errorbar", "lower_errorbar", "upper_errorbar")) {
    geom_func <- ggplot2::geom_errorbar
  }
  geom_func
}

.is_stacked <- function(p) {
  inherits(p$layers[[1]]$position, "PositionStack")
}

# Error plots that draw as a thin element (vertical line/cap or a point) and so
# do NOT self-align with the wide bars under position_dodge2(). These are the
# ones that need the manual re-centering done by .geom_dodge2_errorbar() (#363).
.narrow_error_plots <- function() {
  c(
    "errorbar", "lower_errorbar", "upper_errorbar",
    "pointrange", "lower_pointrange", "upper_pointrange",
    "linerange", "lower_linerange", "upper_linerange"
  )
}

# Aligned error bars for position_dodge2() bars (#363).
# position_dodge2() packs elements according to their own width, so a thin error
# bar is not placed on the centre of the wide bar it belongs to: bars and error
# bars end up at different x (the reported bug). Here we read the actual dodged
# bar centres from the built plot and redraw the error layer at those x with
# position = "identity", so each error bar sits exactly on its bar while keeping
# a normal thin cap. Only used for position_dodge2(); every other position keeps
# the standard ggadd() path unchanged. Returns NULL (caller falls back) if the
# centres cannot be matched.
.geom_dodge2_errorbar <- function(p, data_sum, x, y, color = NULL, fill = NULL,
                                  group = NULL, facet.by = NULL, func = "mean_se",
                                  error.plot = "errorbar", width = 0.1) {
  legend.var <- intersect(unique(c(color, fill, group)), colnames(data_sum))
  if (length(legend.var) == 0) return(NULL)
  legend.var <- legend.var[1]

  # Error limits (centre +/- error), honouring upper_/lower_/both, from data_sum.
  # Only symmetric summaries whose error half-width is a data_sum column are
  # handled here (mean_se/sd/ci/range, median_iqr/mad/range). Asymmetric quantile
  # summaries (median_hilow(_)/median_q1q3) have no such column and are NOT a
  # centre +/- error, so we bail out (return NULL) and let the caller keep the
  # standard path, which draws their correct (if unaligned) interval.
  error <- .get_errorbar_error_func(func)
  if (is.null(error) || !(error %in% colnames(data_sum))) return(NULL)
  err.val <- data_sum[[error]]
  limit <- unlist(strsplit(error.plot, "_", fixed = TRUE))[1]
  if (!(limit %in% c("upper", "lower"))) limit <- "both"
  yc <- data_sum[[y]]
  ds <- data_sum
  ds$.yc. <- yc
  ds$.ymin. <- if (limit %in% c("both", "lower")) yc - err.val else yc
  ds$.ymax. <- if (limit %in% c("both", "upper")) yc + err.val else yc

  # Actual dodged bar centres from the built plot. Faceting is applied later (by
  # .plotter), so build against a temporarily-faceted copy: dodge2 positions
  # depend on which groups are present *within each panel*.
  build.p <- p
  if (!is.null(facet.by)) build.p <- p + ggplot2::facet_wrap(facet.by)
  built <- ggplot2::ggplot_build(build.p)
  bar.layer <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBar"), logical(1)))
  if (length(bar.layer) == 0) return(NULL)
  bd <- built$data[[bar.layer[1]]]
  bd <- bd[order(bd$PANEL, bd$x), , drop = FALSE]

  # Map each summary row to its panel, then to a bar centre. Bars are laid out
  # per panel left-to-right by x tick then group; sorting data_sum the same way
  # aligns it 1:1 with the ordered bar centres (robust to input row order).
  layout <- built$layout$layout
  if (!is.null(facet.by) && all(facet.by %in% colnames(layout))) {
    ds <- dplyr::left_join(ds, layout[, c("PANEL", facet.by), drop = FALSE], by = facet.by)
  } else {
    ds$PANEL <- factor(1L)
  }
  as.int <- function(v) if (is.factor(v)) as.integer(v) else as.integer(factor(v))
  ds <- ds[order(as.int(ds$PANEL), as.int(ds[[x]]), as.int(ds[[legend.var]])), , drop = FALSE]
  if (nrow(ds) != nrow(bd)) return(NULL)
  ds$.ebx. <- bd$x

  geom.error <- .get_geom_error_function(error.plot)
  color.is.var <- !is.null(color) && length(color) == 1 && color %in% colnames(ds)
  if (color.is.var) {
    mapping <- ggplot2::aes(
      x = .data$.ebx., ymin = .data$.ymin., ymax = .data$.ymax.,
      colour = .data[[color]]
    )
  } else {
    mapping <- ggplot2::aes(x = .data$.ebx., ymin = .data$.ymin., ymax = .data$.ymax.)
  }
  if (identical(geom.error, ggplot2::geom_pointrange)) mapping$y <- ggplot2::aes(y = .data$.yc.)$y
  opts <- list(data = ds, mapping = mapping, inherit.aes = FALSE, position = "identity")
  if (!color.is.var && !is.null(color) && length(color) == 1) opts$colour <- color
  if (identical(geom.error, ggplot2::geom_errorbar)) opts$width <- width
  do.call(geom.error, opts)
}

# remove "mean_se", "mean_sd", etc
.remove_errorbar_func <- function(add) {
  setdiff(add, .errorbar_functions())
}
# return "mean_se"
.get_summary_func <- function(add) {
  intersect(.errorbar_functions(), add)
}


# Returns: mean or median
.get_errorbar_center_func <- function(func = "mean_se") {
  . <- NULL
  func %>%
    strsplit("_", fixed = TRUE) %>%
    unlist() %>%
    .[1]
}

# Returns se, sd, iqr
.get_errorbar_error_func <- function(func = "mean_se") {
  res <- func %>%
    strsplit("_", fixed = TRUE) %>%
    unlist()
  if (length(res) >= 2) {
    res <- res[2]
  } else {
    res <- NULL
  }
  res
}

# Regression tests for #313: yticks.by / xticks.by produced breaks anchored at
# the expanded (slightly negative) axis minimum -- e.g. -19, 81, 181 or
# -20, 80, 180 -- instead of round multiples 0, 100, 200. get_breaks() now
# anchors the first break to a multiple of `by`.

.ybreaks <- function(p) {
  stats::na.omit(ggplot2::ggplot_build(p)$layout$panel_params[[1]]$y$breaks)
}

.df <- data.frame(trt = factor(c("a", "b", "c")), value = c(120, 250, 380))

test_that("yticks.by gives round breaks anchored at 0, not the negative expansion (#313)", {
  p <- ggbarplot(.df, "trt", "value", fill = "grey", yticks.by = 100)
  yb <- .ybreaks(p)
  expect_true(all(yb %% 100 == 0))          # round multiples of 100
  expect_false(any(yb < 0))                 # no negative break label
  expect_true(0 %in% yb)
})

test_that("ylim + yticks.by shows 0..400 round breaks (the #313 report)", {
  p <- ggbarplot(.df, "trt", "value", fill = "grey", ylim = c(0, 400), yticks.by = 100)
  yb <- .ybreaks(p)
  expect_setequal(yb, c(0, 100, 200, 300, 400))
})

test_that("get_breaks() anchors the first break to a multiple of `by` (#313)", {
  f <- get_breaks(by = 100)
  # expanded range dipping below 0 -> breaks still land on 0, 100, ...
  expect_true(all(f(c(-19, 399)) %% 100 == 0))
  expect_true(0 %in% f(c(-19, 399)))
  # genuinely negative data -> still round multiples
  expect_true(all(f(c(-50, 380)) %% 100 == 0))
})

test_that("strictly-positive-data breaks are unchanged (no regression, #313/#333)", {
  # xticks.by on positive scatter data: visible breaks identical to before
  p <- ggscatter(iris, x = "Sepal.Length", y = "Sepal.Width", xticks.by = 0.5)
  xb <- stats::na.omit(ggplot2::ggplot_build(p)$layout$panel_params[[1]]$x$breaks)
  expect_true(all(abs(diff(xb) - 0.5) < 1e-8))
  expect_true(min(xb) >= 4 && max(xb) <= 8)
})

context("test-ggbarplot-stacked-posneg")

# #426: for STACKED bars (default position = "stack"), ggbarplot() placed every
# error bar with a single cumulative sum, which assumes all stacked segments share
# one sign. position_stack() accumulates positive and negative segments SEPARATELY,
# so with mixed signs the error bars of one sign were drawn on the wrong side (and
# which one looked correct flipped with the factor-level order). Error bars are now
# cumulated per sign, matching position_stack(). Single-sign data is unchanged.

suppressPackageStartupMessages(library(ggplot2))

# self-contained data with KNOWN per-cell means (two identical rows per cell so
# the mean is exact); group g1 is negative, g2/g3 positive.
.mk_posneg <- function(sign1 = -1) {
  vals <- c(
    # x = A
    A_g1 = 10 * sign1, A_g2 = 5,  A_g3 = 8,
    # x = B
    B_g1 = 6  * sign1, B_g2 = 4,  B_g3 = 7
  )
  d <- do.call(rbind, lapply(names(vals), function(k) {
    xg <- strsplit(k, "_")[[1]]
    data.frame(x = xg[1], g = xg[2], y = rep(vals[[k]], 2))
  }))
  d$x <- factor(d$x); d$g <- factor(d$g)
  d
}

# error-bar centers per x position (sorted), from the built plot
.ebar_centers_by_x <- function(p) {
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  ei <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomErrorbar"), logical(1)))[1]
  d <- b$data[[ei]]
  d$center <- (d$ymin + d$ymax) / 2
  lapply(split(d$center, d$x), function(v) sort(round(v, 6)))
}

test_that("stacked mixed-sign error bars sit at the correct per-sign cumulative position (#426)", {
  d <- .mk_posneg(sign1 = -1)          # g1 negative, g2/g3 positive
  p <- ggbarplot(d, "x", "y", add = "mean_se", color = "g")
  centers <- .ebar_centers_by_x(p)
  # stacking order is desc(g): g3, g2, g1. Positives cumulate (g3 then g2), the
  # negative g1 cumulates on its own side.
  #   x = A: g3 -> 8, g2 -> 8 + 5 = 13, g1 -> -10
  #   x = B: g3 -> 7, g2 -> 7 + 4 = 11, g1 -> -6
  expect_equal(centers[["1"]], c(-10, 8, 13))
  expect_equal(centers[["2"]], c(-6, 7, 11))
})

test_that("the mixed-sign fix does not move a wholly-positive stack (#426)", {
  d <- .mk_posneg(sign1 = 1)           # all positive
  p <- ggbarplot(d, "x", "y", add = "mean_se", color = "g")
  centers <- .ebar_centers_by_x(p)
  # pure cumsum: x = A -> 8, 13, 23 ; x = B -> 7, 11, 17
  expect_equal(centers[["1"]], c(8, 13, 23))
  expect_equal(centers[["2"]], c(7, 11, 17))
})

test_that("error bars line up with the actual stacked bar mean-ends (#426)", {
  d <- .mk_posneg(sign1 = -1)
  p <- ggbarplot(d, "x", "y", add = "mean_se", color = "g")
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  bi <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomCol") ||
                       inherits(l$geom, "GeomBar"), logical(1)))[1]
  ei <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomErrorbar"), logical(1)))[1]
  bar <- b$data[[bi]]
  # a stacked segment's mean-end is its boundary farther from zero (robust to
  # multiple same-sign segments, not just the one that touches zero).
  bar$meanend <- ifelse(abs(bar$ymax) >= abs(bar$ymin), bar$ymax, bar$ymin)
  eb <- b$data[[ei]]; eb$center <- (eb$ymin + eb$ymax) / 2
  m <- merge(eb[, c("x", "colour", "center")], bar[, c("x", "colour", "meanend")],
             by = c("x", "colour"))
  expect_equal(nrow(m), 6)
  expect_true(all(abs(m$center - m$meanend) < 1e-6))
})

test_that("the corrected side is independent of the factor-level order (#426)", {
  d <- .mk_posneg(sign1 = -1)
  d$g <- factor(d$g, levels = c("g3", "g2", "g1"))   # reversed order (OP's note)
  p <- ggbarplot(d, "x", "y", add = "mean_se", color = "g")
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  bi <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomCol") ||
                       inherits(l$geom, "GeomBar"), logical(1)))[1]
  ei <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomErrorbar"), logical(1)))[1]
  bar <- b$data[[bi]]
  # a stacked segment's mean-end is its boundary farther from zero (robust to
  # multiple same-sign segments, not just the one that touches zero).
  bar$meanend <- ifelse(abs(bar$ymax) >= abs(bar$ymin), bar$ymax, bar$ymin)
  eb <- b$data[[ei]]; eb$center <- (eb$ymin + eb$ymax) / 2
  m <- merge(eb[, c("x", "colour", "center")], bar[, c("x", "colour", "meanend")],
             by = c("x", "colour"))
  expect_true(all(abs(m$center - m$meanend) < 1e-6))  # aligned regardless of order
})

context("test-ggbarplot-numeric-x")

# #463: numeric.x.axis (already available in ggline/ggerrorplot) is now
# supported by ggbarplot. Default (FALSE) keeps the discrete-factor x axis.

x_is_discrete <- function(p) {
  ggplot2::ggplot_build(p)$layout$panel_scales_x[[1]]$is_discrete()
}

test_that("ggbarplot default keeps a discrete x axis (no regression, #463)", {
  df <- data.frame(x = c(1, 2, 5, 10), y = c(3, 5, 4, 8))
  expect_true(x_is_discrete(ggbarplot(df, "x", "y")))
})

test_that("ggbarplot numeric.x.axis = TRUE gives a numeric x axis (#463)", {
  df <- data.frame(x = c(1, 2, 5, 10), y = c(3, 5, 4, 8))
  p <- ggbarplot(df, "x", "y", numeric.x.axis = TRUE, width = 1)
  expect_false(x_is_discrete(p))
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  # bars sit at their numeric x positions
  bx <- ggplot2::ggplot_build(p)$data[[1]]$x
  expect_equal(sort(bx), c(1, 2, 5, 10))
})

test_that("ggbarplot numeric.x.axis works with summary (mean_se) (#463)", {
  df <- data.frame(x = rep(c(1, 2, 5), each = 4), y = rnorm(12, 5))
  p <- ggbarplot(df, "x", "y", add = "mean_se", numeric.x.axis = TRUE, width = 0.8)
  expect_false(x_is_discrete(p))
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

context("test-ggbarplot-dodge2-errorbar")

# #363: with position_dodge2() the thin error bars did not line up with the
# dodged bars (dodge2 packs elements by their own width, so a thin error bar
# lands off the centre of its wide bar). ggbarplot() now re-centres the error
# bars on the actual bar positions when position_dodge2() is used. Every other
# position (identity / dodge / stack) keeps the standard path unchanged.

.bar_x <- function(p) {
  b <- ggplot2::ggplot_build(p)
  i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBar"), logical(1)))[1]
  sort(as.numeric(round(b$data[[i]]$x, 4)))
}
.err_x <- function(p) {
  b <- ggplot2::ggplot_build(p)
  i <- which(vapply(p$layers, function(l)
    inherits(l$geom, c("GeomErrorbar", "GeomPointrange", "GeomLinerange")), logical(1)))[1]
  sort(as.numeric(round(b$data[[i]]$x, 4)))
}

.reporter <- data.frame(
  Treatment = c("+", "+", "+", "+", "+", "+", "-", "-", "-"),
  Group     = c("a", "a", "a", "b", "b", "b", "a", "a", "a"),
  Count     = c(12, 13, 11, 14, 15, 14, 23, 24, 25)
)

test_that("dodge2 error bars align with the bars (#363)", {
  p <- ggbarplot(.reporter, x = "Treatment", y = "Count", add = "mean_se",
                 error.plot = "upper_errorbar", fill = "Group",
                 position = position_dodge2(preserve = "single"))
  expect_equal(.err_x(p), .bar_x(p))
  # renders at draw time (positioning bugs often only surface on draw)
  expect_silent(ggplot2::ggplotGrob(p))
})

test_that("alignment holds for both-sided errorbar, pointrange and linerange", {
  for (ep in c("errorbar", "pointrange", "linerange")) {
    p <- ggbarplot(.reporter, x = "Treatment", y = "Count", add = "mean_se",
                   error.plot = ep, fill = "Group",
                   position = position_dodge2(preserve = "single"))
    expect_equal(.err_x(p), .bar_x(p), info = ep)
  }
})

test_that("alignment is independent of input row order", {
  shuffled <- .reporter[c(7, 1, 4, 8, 2, 5, 9, 3, 6), ]
  p <- ggbarplot(shuffled, x = "Treatment", y = "Count", add = "mean_se",
                 error.plot = "upper_errorbar", fill = "Group",
                 position = position_dodge2(preserve = "single"))
  expect_equal(.err_x(p), .bar_x(p))
})

test_that("alignment holds for >2 groups with missing subgroups", {
  d <- data.frame(
    Treatment = factor(rep(c("+", "-"), c(9, 6)), levels = c("+", "-")),
    Group = c(rep(c("a", "b", "c"), each = 3), rep(c("a", "c"), each = 3)),
    Count = c(12, 13, 11, 14, 15, 14, 9, 10, 11, 23, 24, 25, 7, 8, 9)
  )
  p <- ggbarplot(d, x = "Treatment", y = "Count", add = "mean_se",
                 error.plot = "upper_errorbar", fill = "Group",
                 position = position_dodge2(preserve = "single"))
  expect_equal(.err_x(p), .bar_x(p))
})

test_that("alignment holds across facets", {
  d <- data.frame(
    Sex = rep(c("M", "F"), each = 9),
    Treatment = factor(rep(rep(c("+", "-"), c(6, 3)), 2), levels = c("+", "-")),
    Group = rep(c("a", "a", "a", "b", "b", "b", "a", "a", "a"), 2),
    Count = c(12, 13, 11, 14, 15, 14, 23, 24, 25, 8, 9, 7, 11, 12, 10, 20, 21, 19)
  )
  p <- ggbarplot(d, x = "Treatment", y = "Count", add = "mean_se",
                 error.plot = "upper_errorbar", fill = "Group",
                 facet.by = "Sex", position = position_dodge2(preserve = "single"))
  expect_equal(.err_x(p), .bar_x(p))
})

# ---- No-regression: non-dodge2 positions are unchanged ----

test_that("position_dodge() error bars are unchanged (still aligned)", {
  p <- ggbarplot(.reporter, x = "Treatment", y = "Count", add = "mean_se",
                 error.plot = "upper_errorbar", fill = "Group",
                 position = position_dodge(0.8))
  # dodge (v1) has always aligned thin error bars; pin the exact positions
  expect_equal(.err_x(p), c(0.8, 1.2, 2.0))
  expect_equal(.bar_x(p), c(0.8, 1.2, 2.0))
})

test_that("stacked bars keep the dedicated stacked-errorbar path", {
  d <- data.frame(
    Treatment = rep(c("+", "-"), each = 4),
    Group = rep(c("a", "b"), 4),
    Count = c(12, 14, 13, 15, 23, 10, 24, 11)
  )
  p <- ggbarplot(d, x = "Treatment", y = "Count", add = "mean_se",
                 error.plot = "upper_errorbar", fill = "Group",
                 position = position_stack())
  # stacked error bars sit at the x-tick centres (1 and 2), not dodged
  expect_equal(.err_x(p), c(1, 1, 2, 2))
  expect_silent(ggplot2::ggplotGrob(p))
})

test_that("asymmetric summaries (median_hilow/q1q3) keep correct interval under dodge2", {
  # These are NOT centre +/- error, so the aligned path must bail out and let the
  # standard path draw the real asymmetric interval (regression guard).
  set.seed(1)
  d <- data.frame(
    T = rep(c("+", "-"), c(12, 6)),
    G = c(rep(c("a", "b"), each = 6), rep("a", 6)),
    Count = c(rnorm(12, 15, 3), rnorm(6, 24, 3))
  )
  .h <- function(p) {
    b <- ggplot2::ggplot_build(p)
    i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomErrorbar"), logical(1)))[1]
    sort(round(b$data[[i]]$ymax - b$data[[i]]$ymin, 3))
  }
  for (f in c("median_hilow", "median_q1q3")) {
    p_d2 <- ggbarplot(d, "T", "Count", add = f, error.plot = "errorbar", fill = "G",
                      position = position_dodge2(preserve = "single"))
    p_d1 <- ggbarplot(d, "T", "Count", add = f, error.plot = "errorbar", fill = "G",
                      position = position_dodge(0.8))
    expect_equal(.h(p_d2), .h(p_d1), info = f)          # same interval as before
    expect_true(all(.h(p_d2) > 0), info = f)            # not collapsed to zero
  }
})

test_that("error bar colour follows the color argument (default black; grouped when mapped)", {
  pd <- ggbarplot(.reporter, x = "Treatment", y = "Count", add = "mean_se",
                  error.plot = "upper_errorbar", fill = "Group",
                  position = position_dodge2(preserve = "single"))
  b <- ggplot2::ggplot_build(pd)
  ei <- which(vapply(pd$layers, function(l) inherits(l$geom, "GeomErrorbar"), logical(1)))[1]
  expect_equal(unique(b$data[[ei]]$colour), "black")

  pc <- ggbarplot(.reporter, x = "Treatment", y = "Count", add = "mean_se",
                  error.plot = "upper_errorbar", fill = "Group", color = "Group",
                  position = position_dodge2(preserve = "single"))
  bc <- ggplot2::ggplot_build(pc)
  eic <- which(vapply(pc$layers, function(l) inherits(l$geom, "GeomErrorbar"), logical(1)))[1]
  expect_gt(length(unique(bc$data[[eic]]$colour)), 1)
})

context("test-ggbarplot-facet-summary")

# #739: a summarized ggbarplot (add = "mean_se") must be faceted with the
# `facet.by=` argument, which pre-computes the summaries per panel. This test
# locks that correct behavior: with facet.by=, panels with different data get
# DIFFERENT (per-panel) bar heights - they are not pooled across the whole data.

suppressPackageStartupMessages(library(ggplot2))

.bar_by_panel <- function(p) {
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  bi <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomBar") ||
                       inherits(l$geom, "GeomCol"), logical(1)))[1]
  d <- b$data[[bi]]
  split(round(d$y, 6), d$PANEL)
}

test_that("facet.by= gives correct per-panel summaries (not pooled) (#739)", {
  tg <- ToothGrowth
  tg$dose <- factor(tg$dose)
  # two blocks with deliberately different level: 'high' block shifted up by 20
  tg$blk <- ifelse(tg$len < stats::median(tg$len), "low", "high")

  p <- ggbarplot(tg, "dose", "len", add = "mean_se", fill = "supp", facet.by = "blk")
  panels <- .bar_by_panel(p)

  expect_length(panels, 2)                       # two panels
  # the two panels summarize different data -> their bar heights must differ
  expect_false(isTRUE(all.equal(panels[[1]], panels[[2]])))
})

test_that("facet.by= panels reflect their own subset means (#739)", {
  # construct data where panel means are known and different
  d <- rbind(
    data.frame(g = "p1", x = c("a", "a", "b", "b"), y = c(2, 2, 4, 4)),
    data.frame(g = "p2", x = c("a", "a", "b", "b"), y = c(10, 10, 20, 20))
  )
  d$x <- factor(d$x)
  p <- ggbarplot(d, "x", "y", add = "mean_se", facet.by = "g")
  panels <- .bar_by_panel(p)
  # p1 bars = 2,4 ; p2 bars = 10,20 (each panel its own means, not pooled 6,12)
  expect_equal(sort(unname(panels[["1"]])), c(2, 4))
  expect_equal(sort(unname(panels[["2"]])), c(10, 20))
})

# Regression tests for #646: with orientation = "horizontal", xlim/ylim were
# dropped because coord_flip() replaced the coord_cartesian() that carried the
# limits (also emitting "Coordinate system already present"). Limits are now
# passed into coord_flip().

.df <- data.frame(x = c("a", "b", "c"), y = c(10, 20, 30))

.range_of <- function(p, ax) {
  ggplot2::ggplot_build(p)$layout$panel_params[[1]][[ax]]
}
# Capture both messages and warnings emitted while CONSTRUCTING and building the
# plot. The "Coordinate system already present" note is emitted as a message at
# construction time (when coord_flip replaces coord_cartesian), so we must build
# the plot inside the handlers, not pass in a finished plot.
.construct_notes <- function(expr) {
  notes <- character(0)
  withCallingHandlers(
    {
      p <- force(expr)
      ggplot2::ggplot_gtable(ggplot2::ggplot_build(p))
    },
    message = function(m) { notes[[length(notes) + 1]] <<- conditionMessage(m); invokeRestart("muffleMessage") },
    warning = function(w) { notes[[length(notes) + 1]] <<- conditionMessage(w); invokeRestart("muffleWarning") }
  )
  notes
}

test_that("ylim is honored with orientation='horizontal' (#646)", {
  p <- ggbarplot(.df, x = "x", y = "y", fill = "x",
                 orientation = "horizontal", ylim = c(0, 50))
  # After coord_flip the value (y) axis is the horizontal one: x.range
  r <- .range_of(p, "x.range")
  expect_true(min(r) <= 0 + 1e-8 && max(r) >= 50 - 1e-8)
  # no "Coordinate system already present" note (message or warning)
  expect_false(any(grepl(
    "already present",
    .construct_notes(ggbarplot(.df, x = "x", y = "y", fill = "x",
                               orientation = "horizontal", ylim = c(0, 50)))
  )))
})

test_that("vertical orientation ylim is unchanged (no regression, #646)", {
  p <- ggbarplot(.df, x = "x", y = "y", fill = "x", ylim = c(0, 50))
  r <- .range_of(p, "y.range")
  expect_true(min(r) <= 0 + 1e-8 && max(r) >= 50 - 1e-8)
  expect_false(any(grepl(
    "already present",
    .construct_notes(ggbarplot(.df, x = "x", y = "y", fill = "x", ylim = c(0, 50)))
  )))
})

test_that("horizontal without limits, and reverse, still render (no regression, #646)", {
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(
    ggbarplot(.df, x = "x", y = "y", fill = "x", orientation = "horizontal"))))
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(
    ggbarplot(.df, x = "x", y = "y", fill = "x", orientation = "reverse"))))
})

#' Set ggplot Panel Border Line
#'
#' @description Change or set ggplot panel border.
#' @param color border line color.
#' @param size numeric value specifying border line size.
#' @param linetype line type. An integer (0:8), a name (blank, solid, dashed,
#'  dotted, dotdash, longdash, twodash). Sess \code{\link{show_line_types}}.
#'
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' p <- ggboxplot(df, x = "dose", y = "len")
#' p
#'
#' # Add border
#' p + border()
#' @export
border <- function(color = "black", size = 0.8, linetype = NULL) {
  theme(
    panel.background = element_rect(color = color, linewidth = size, linetype = linetype),
    axis.line = element_blank()
  )
}

# Regression test for #668: theme_pubr() left axis.ticks inheriting a lighter
# grey/thinner style while axis.line was black/0.5, visibly inconsistent when
# zoomed. Ticks now match the axis lines.

test_that("theme_pubr() axis.ticks match axis.line (black, 0.5) (#668)", {
  th <- theme_pubr()
  ticks <- ggplot2::calc_element("axis.ticks", th)
  line <- ggplot2::calc_element("axis.line", th)
  expect_equal(ticks$colour, "black")
  expect_equal(ticks$linewidth, 0.5)
  # consistent with the axis line
  expect_equal(ticks$colour, line$colour)
  expect_equal(ticks$linewidth, line$linewidth)
})

test_that("theme_pubr(border = TRUE) also has black/0.5 ticks (#668)", {
  ticks <- ggplot2::calc_element("axis.ticks", theme_pubr(border = TRUE))
  expect_equal(ticks$colour, "black")
  expect_equal(ticks$linewidth, 0.5)
})

test_that("theme_pubr() still renders and other elements are intact (#668)", {
  p <- ggplot2::ggplot(data.frame(x = 1:3, y = c("A", "B", "C")),
                       ggplot2::aes(x, y)) +
    ggplot2::geom_point() + theme_pubr()
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  th <- theme_pubr()
  # unrelated elements unchanged
  expect_true(inherits(th$panel.grid.major, "element_blank"))
  expect_equal(ggplot2::calc_element("axis.text", th)$colour, "black")
})

# Follow-up to #668: with the ggplot2 (>= 3.5.0) default (strip.clip = "on"),
# the facet strip background border is clipped to the strip area, so the outer
# half of the border stroke is cut and the border renders thinner than the
# requested linewidth -- misaligned with the panel/axis when zoomed. theme_pubr()
# now sets strip.clip = "off" so the strip border renders at its full width.
test_that("theme_pubr() sets strip.clip = 'off' so the strip border isn't clipped (#668)", {
  expect_equal(theme_pubr()$strip.clip, "off")
  expect_equal(theme_pubr(border = TRUE)$strip.clip, "off")
})

test_that("theme_pubr() faceted plot renders with a full-width strip border (#668)", {
  df <- data.frame(x = c(1, 2, 3, 1, 2, 3), y = c(2, 3, 4, 1, 5, 3),
                   g = rep(c("G1", "G2"), each = 3))
  p <- ggplot2::ggplot(df, ggplot2::aes(x, y)) +
    ggplot2::geom_point() + ggplot2::facet_wrap(~g) + theme_pubr()
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  # strip.background linewidth preserved (border consistency, not thinned)
  strip <- ggplot2::calc_element("strip.background", theme_pubr())
  expect_equal(strip$linewidth, 0.7)
  expect_equal(strip$colour, "black")
})

#' Execute ggplot2 functions
#' @description A helper function used by ggpubr functions to execute any geom_*
#'   functions in ggplot2. Useful only when you want to call a geom_* function
#'   without worrying about the arguments to put in aes(). Basic users of ggpubr
#'   don't need this function.
#' @param geomfunc a ggplot2 function (e.g.: geom_point)
#' @param data a data frame to be used for mapping
#' @param position Position adjustment, either as a string, or the result of a
#'   call to a position adjustment function.
#' @param ... arguments accepted by the function
#' @return a plot if geomfunc != NULL or a list(option, mapping) if
#'   geomfunc is NULL.
#' @examples
#' \dontrun{
#' ggplot() +
#'   geom_exec(geom_point,
#'     data = mtcars,
#'     x = "mpg", y = "wt", size = "cyl", color = "cyl"
#'   )
#' }
#' @export
geom_exec <- function(geomfunc = NULL, data = NULL,
                      position = NULL, ...) {
  params <- list(...)

  mapping <-
    list() # option to pass to mapping aes()
  option <- list() # option to the geom_*()

  allowed_options <- c(
    # general
    "x", "y", "color", "colour", "linetype", "fill", "size", "linewidth", "shape", "width",
    "alpha", "na.rm", "lwd", "pch", "cex", "position", "stat", "geom",
    "show.legend", "inherit.aes", "fun.args", "fontface",
    # point
    "stroke",
    # boxplot
    "outlier.colour", "outlier.shape", "outlier.size", "outliers",
    "outlier.stroke", "notch", "notchwidth", "varwidth", "coef",
    # dot plot
    "binwidth", "binaxis", "method", "binpositions",
    "stackdir", "stackratio", "dotsize",
    # Violin and density
    "trim", "drop", "draw_quantiles", "quantiles", "quantile.linetype", "quantile.type",
    "quantile.alpha", "quantile.colour", "quantile.color", "quantile.linewidth",
    "quantile.size", "scale", "adjust", "bw",
    # error
    "ymin", "ymax", "xmin", "xmax",
    # text
    "label", "hjust", "vjust", "fontface", "angle", "family", "parse",
    # text.repel
    "segment.size", "force", "max.overlaps", "seed",
    # smooth
    "se", "level", "fullrange",
    "conf.int.level",
    # straightline
    "xintercept", "yintercept",
    # histograms
    "bins", "weight",
    # rug
    "sides",
    # segment
    "arrow", "xend", "yend",
    # stat_summary,
    "fun.data", "fun", "fun.min", "fun.max",
    # bracket
    "y.position", "tip.length", "tip.length.ref", "label.size", "step.increase",
    "bracket.nudge.y", "bracket.shorten", "coord.flip"
  )

  columns <- colnames(data)

  # Helper to check for geoms/stats that use linewidth for strokes
  is_line_geom <- function(f) {
    if (is.null(f)) {
      return(FALSE)
    }

    # List of geoms/stats that use linewidth in modern ggplot2
    line_geoms <- list(
      ggplot2::geom_line, ggplot2::geom_path, ggplot2::geom_segment,
      ggplot2::geom_step, ggplot2::geom_density, ggplot2::geom_freqpoly,
      ggplot2::geom_histogram, ggplot2::geom_bar, ggplot2::geom_col,
      ggplot2::geom_smooth, ggplot2::geom_errorbar,
      ggplot2::geom_linerange, ggplot2::geom_vline, ggplot2::geom_hline,
      ggplot2::geom_abline, ggplot2::geom_rug, ggplot2::geom_rect,
      ggplot2::geom_tile, ggplot2::geom_polygon, ggplot2::geom_ribbon,
      ggplot2::geom_area, ggplot2::geom_crossbar, ggplot2::geom_boxplot,
      ggplot2::geom_violin,
      ggplot2::stat_ecdf, ggplot2::stat_qq_line
    )

    for (geom in line_geoms) {
      if (identical(f, geom)) {
        return(TRUE)
      }
    }
    return(FALSE)
  }

  # Auto-convert size to linewidth for line-based geoms. A `linewidth` that was
  # passed but is NULL must not block the conversion: callers forward
  # `linewidth = <maybe-NULL>` alongside `size`, and a named NULL is dropped
  # later anyway (see the NULL check in the loop below).
  if (is_line_geom(geomfunc) && "size" %in% names(params) &&
      (!"linewidth" %in% names(params) || is.null(params[["linewidth"]]))) {
    params[["linewidth"]] <- NULL
    names(params)[names(params) == "size"] <- "linewidth"
  }

  for (key in names(params)) {
    value <- params[[key]]
    if (is.null(value)) {} else if (unlist(value)[1] %in% columns & key %in% allowed_options) {
      mapping[[key]] <- value
    } else if (key %in% allowed_options) {
      option[[key]] <- value
    } else if (key == "group") {
      mapping[[key]] <- value # for line plot
    } else if (key == "step.group.by") {
      # for geom_bracket, value are variable name.
      # but this parameter is an option not an aes
      option[[key]] <- value
    }
    # else warnings("Don't know '", key, "'")
  }
  if (!is.null(position)) {
    option[["position"]] <- position
  }
  option[["data"]] <- data
  if (is.null(geomfunc)) {
    res <- list(option = option, mapping = mapping)
  } else {
    option[["mapping"]] <- create_aes(mapping)
    res <- do.call(geomfunc, option)
  }
  res
}

test_that("geom_exec converts size to linewidth for bar/col geoms", {
  df <- data.frame(x = 1, y = 1)
  layer <- geom_exec(ggplot2::geom_col, data = df, x = "x", y = "y", size = 0.6)
  expect_equal(layer[["aes_params"]][["linewidth"]], 0.6)
})

test_that("geom_exec keeps size for non-linewidth geoms", {
  df <- data.frame(x = 1, y = 1)
  layer <- geom_exec(ggplot2::geom_point, data = df, x = "x", y = "y", size = 2)
  expect_equal(layer[["aes_params"]][["size"]], 2)
})

test_that("geom_exec converts size to linewidth when linewidth is passed as NULL", {
  df <- data.frame(x = 1, y = 1)
  layer <- geom_exec(ggplot2::geom_col,
    data = df, x = "x", y = "y",
    size = 0.6, linewidth = NULL
  )
  expect_equal(layer[["aes_params"]][["linewidth"]], 0.6)
  expect_null(layer[["aes_params"]][["size"]])
})

#' Gene Expression Data
#'
#' @description Gene expression data extracted from TCGA using the `RTCGA` and
#'  `RTCGA.mRNA` R packages. It contains the mRNA expression for 3 genes -
#'  GATA3, PTEN and XBP1- from 3 different datasets: Breast invasive carcinoma
#'  (BRCA), Ovarian serous cystadenocarcinoma (OV) and Lung squamous cell
#'  carcinoma (LUSC)
#' @name gene_expression
#' @docType data
#' @usage data("gene_expression")
#' @format A data frame with 1305 rows and 5 columns. \describe{
#'  \item{\code{bcr_patient_barcode}}{sample ID} \item{\code{dataset}}{cancer type} \item{\code{GATA3}}{GATA3 gene expression}
#'  \item{\code{PTEN}}{PTEN gene expression}\item{\code{XBP1}}{XBP1 gene expression.}}
#'
#' @examples
#' data(gene_expression)
#'
#' ggboxplot(gene_expression,
#'   x = "dataset",
#'   y = c("GATA3", "PTEN", "XBP1"),
#'   combine = TRUE,
#'   ylab = "Expression",
#'   color = "dataset", palette = "jco"
#' )
#'
NULL

#' Convert NPC to Data Coordinates
#' @description Convert NPC (Normalized Parent Coordinates) into data coordinates.
#' @param npc a numeric vector. Each value should be in [0-1]
#' @param data.ranges a numeric vector of length 2 containing the data ranges (minimum and the maximum)
#' @return a numeric vector representing data coordinates.
#' @seealso \code{\link{as_npc}}, \code{\link{get_coord}}.
#' @examples
#' npc_to_data_coord(npc = c(0.2, 0.95), data.ranges = c(1, 20))
#' as_npc(c("top", "right")) %>%
#'   npc_to_data_coord(data.ranges = c(1, 20))
#'
#' @rdname npc_to_data_coord
#' @export
npc_to_data_coord <- function(npc, data.ranges) {
  data.ranges[1] + npc * (data.ranges[2] - data.ranges[1])
}

#' @include utilities.R utils_stat_test_label.R p_format_utils.R
NULL
#' Add Anova Test P-values to a GGPlot
#' @description Adds automatically one-way and two-way ANOVA test p-values to a
#'  ggplot, such as box plots, dot plots and stripcharts.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @inheritParams stat_pvalue_manual
#' @param method ANOVA test methods. Possible values are one of
#'  \code{c("one_way", "one_way_repeated", "two_way", "two_way_repeated",
#'  "two_way_mixed")}.
#' @param wid (factor) column name containing individuals/subjects identifier.
#'  Should be unique per individual. Required only for repeated measure tests
#'  (\code{"one_way_repeated", "two_way_repeated", "friedman_test", etc}).
#' @param group.by (optional) character vector specifying the grouping variable;
#'  it should be used only for grouped plots. Possible values are : \itemize{
#'  \item \code{"x.var"}: Group by the x-axis variable and perform the test
#'  between legend groups. In other words, the p-value is compute between legend
#'  groups at each x position \item \code{"legend.var"}: Group by the legend
#'  variable and perform the test between x-axis groups. In other words, the
#'  test is performed between the x-groups for each legend level. }
#' @param type the type of sums of squares for ANOVA. Allowed values are either
#'  1, 2 or 3. \code{type = 2} is the default because this will yield identical
#'  ANOVA results as type = 1 when data are balanced but type = 2 will
#'  additionally yield various assumption tests where appropriate. When the data
#'  are unbalanced the \code{type = 3} is used by popular commercial software
#'  including SPSS.
#' @param effect.size the effect size to compute and to show in the ANOVA
#'  results. Allowed values can be either "ges" (generalized eta squared) or
#'  "pes" (partial eta squared) or both. Default is "ges".
#' @param error (optional) for a linear model, an lm model object from which the
#'  overall error sum of squares and degrees of freedom are to be calculated.
#'  Read more in \code{\link[car]{Anova}()} documentation.
#' @param correction character. Used only in repeated measures ANOVA test to
#'  specify which correction of the degrees of freedom should be reported for
#'  the within-subject factors. Possible values are: \itemize{ \item{"GG"}:
#'  applies Greenhouse-Geisser correction to all within-subjects factors even if
#'  the assumption of sphericity is met (i.e., Mauchly's test is not
#'  significant, p > 0.05). \item{"HF"}: applies Huynh-Feldt correction to all
#'  within-subjects factors even if the assumption of sphericity is met,
#'  \item{"none"}: returns the ANOVA table without any correction and
#'  \item{"auto"}: apply automatically GG correction to only within-subjects
#'  factors violating the sphericity assumption (i.e., Mauchly's test p-value is
#'  significant, p <= 0.05). }
#' @param p.adjust.method method for adjusting p values (see
#'  \code{\link[stats]{p.adjust}}).  Has impact only in a situation, where
#'  multiple pairwise tests are performed; or when there are multiple grouping
#'  variables. Allowed values include "holm", "hochberg", "hommel",
#'  "bonferroni", "BH", "BY", "fdr", "none". If you don't want to adjust the p
#'  value (not recommended), use p.adjust.method = "none".
#' @param significance a list of arguments specifying the significance cutpoints
#'  and symbols. For example, \code{significance <- list(cutpoints = c(0,
#'  0.0001, 0.001, 0.01, 0.05, Inf), symbols = c("****", "***", "**", "*",
#'  "ns"))}.
#'
#'  In other words, we use the following convention for symbols indicating
#'  statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
#'  \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001
#'  \item \code{****}:  p <= 0.0001 }
#'
#'  Note: \code{significance} is kept for backward compatibility. For functions
#'  that also expose \code{signif.cutoffs} and related parameters,
#'  \code{significance} takes precedence when provided.
#' @param signif.cutoffs numeric vector of p-value cutoffs in descending order
#'  for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
#'  means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
#'  If \code{use.four.stars = TRUE}, can include a fourth level.
#'  Default is NULL, which uses the package defaults.
#' @param signif.symbols character vector of symbols corresponding to
#'  \code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
#'  (and "****" if \code{use.four.stars = TRUE}).
#' @param ns.symbol character string for non-significant results. Default is "ns".
#'  Use "" (empty string) to show nothing.
#' @param use.four.stars logical. If TRUE, allows four stars (****) for the most
#'  significant level. Default is FALSE.
#' @param p.format.style character string specifying the p-value formatting style.
#'  One of: \code{"default"} (backward compatible, uses scientific notation),
#'  \code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
#'  \code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
#'  (GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
#'  See \code{\link{list_p_format_styles}} for details.
#' @param p.digits integer specifying the number of decimal places for p-values.
#'  If provided, overrides the style default.
#' @param p.leading.zero logical indicating whether to include leading zero before
#'  decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.
#' @param p.min.threshold numeric specifying the minimum p-value to display exactly.
#'  Values below this threshold are shown as "< threshold". If provided, overrides
#'  the style default.
#' @param p.decimal.mark character string to use as the decimal mark. If NULL,
#'  uses \code{getOption("OutDec")}.
#' @param label character string specifying label. Can be: \itemize{ \item the
#'  column containing the label (e.g.: \code{label = "p"} or \code{label =
#'  "p.adj"}), where \code{p} is the p-value. Other possible values are
#'  \code{"p.signif", "p.adj.signif", "p.format", "p.format.signif", "p.adj.format"}.
#'  \item an
#'  expression that can be formatted by the \code{\link[glue]{glue}()} package.
#'  For example, when specifying \code{label = "Anova, p = \{p\}"}, the
#'  expression \{p\} will be replaced by its value. \item a combination of
#'  plotmath expressions and glue expressions. You may want some of the
#'  statistical parameter in italic; for example:\code{label = "Anova, italic(p)
#'  = {p}"}. \item a constant: \code{label = "as_italic"}: display statistical
#'  parameters in italic; \code{label = "as_detailed"}: detailed plain text;
#'  \code{label = "as_detailed_expression"} or \code{label =
#'  "as_detailed_italic"}: detailed plotmath expression. Statistical parameters
#'  will be displayed in italic.}.
#' @param label.x.npc,label.y.npc can be \code{numeric} or \code{character}
#'  vector of the same length as the number of groups and/or panels. If too
#'  short they will be recycled. \itemize{ \item If \code{numeric}, value should
#'  be between 0 and 1. Coordinates to be used for positioning the label,
#'  expressed in "normalized parent coordinates". \item If \code{character},
#'  allowed values include: i) one of c('right', 'left', 'center', 'centre',
#'  'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
#'  'middle') for y-axis.}
#' @param parse If TRUE, the labels will be parsed into expressions and displayed
#'  as described in \code{?plotmath}.
#' @param label.x,label.y \code{numeric} Coordinates (in data units) to be used
#'  for absolute positioning of the label. If too short they will be recycled.
#' @param step.increase numeric value in with the increase in fraction of total
#'  height for every additional comparison to minimize overlap. The step value
#'  can be negative to reverse the order of groups.
#' @param ... other arguments to pass to
#'  \code{\link[ggplot2:geom_text]{geom_text}}, such as:\itemize{ \item
#'  \code{hjust}: horizontal justification of the text. Move the text left or
#'  right and \item \code{vjust}: vertical justification of the text. Move the
#'  text up or down. }
#' @param na.rm If FALSE (the default), removes missing values with a warning. If
#'  TRUE silently removes missing values.
#'
#' @section Computed variables: \itemize{ \item{DFn}: Degrees of Freedom in the
#'  numerator (i.e. DF effect). \item{DFd}:	Degrees of Freedom in the
#'  denominator (i.e., DF error). \item{ges}:	Generalized Eta-Squared measure of
#'  effect size. Computed only when the option \code{effect.size = "ges"}.
#'  \item{pes}:	Partial Eta-Squared measure of effect size. Computed only when
#'  the option \code{effect.size = "pes"}. \item{F}:	F-value. \item{p}:	p-value.
#'  \item{p.adj}: Adjusted p-values. \item{p.signif}: P-value significance.
#'  \item{p.adj.signif}: Adjusted p-value significance. \item{p.format}:
#'  Formatted p-value. \item{p.format.signif}: Formatted p-value with significance symbols.
#'  \item{p.adj.format}: Formatted adjusted p-value. \item{n}:
#'  number of samples. }
#'
#' @examples
#' # Data preparation
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Transform `dose` into factor variable
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Add individuals id
#' df$id <- rep(1:10, 6)
#' # Add a random grouping variable
#' set.seed(123)
#' df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
#' df$len <- ifelse(df$group == "grp2", df$len + 2, df$len)
#' df$len <- ifelse(df$group == "grp3", df$len + 7, df$len)
#' head(df, 3)
#'
#'
#' # Basic boxplot
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Create a basic boxplot
#' # Add 5% and 10% space to the plot bottom and the top, respectively
#' bxp <- ggboxplot(df, x = "dose", y = "len") +
#'   scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))
#'
#' # Add the p-value to the boxplot
#' bxp + stat_anova_test()
#'
#' \dontrun{
#' # Change the label position
#' # Using coordinates in data units
#' bxp + stat_anova_test(label.x = "1", label.y = 10, hjust = 0)
#' }
#'
#' # Format the p-value differently
#' custom_p_format <- function(p) {
#'   rstatix::p_format(p, accuracy = 0.0001, digits = 3, leading.zero = FALSE)
#' }
#' bxp + stat_anova_test(
#'   label = "Anova, italic(p) = {custom_p_format(p)}{p.signif}"
#' )
#'
#' # Show a detailed label in italic
#' bxp + stat_anova_test(label = "as_detailed_italic")
#'
#'
#' # Faceted plots
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Create a ggplot facet
#' bxp <- ggboxplot(df, x = "dose", y = "len", facet.by = "supp") +
#'   scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))
#' # Add p-values
#' bxp + stat_anova_test()
#'
#'
#' # Grouped plots
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' bxp2 <- ggboxplot(df, x = "group", y = "len", color = "dose", palette = "npg")
#'
#' # For each x-position, computes tests between legend groups
#' bxp2 + stat_anova_test(aes(group = dose), label = "p = {p.format}{p.signif}")
#'
#' #  For each legend group, computes tests between x variable groups
#' bxp2 + stat_anova_test(aes(group = dose, color = dose), group.by = "legend.var")
#'
#' \dontrun{
#' # Two-way ANOVA: Independent measures
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Visualization: box plots with p-values
#' # Two-way interaction p-values between x and legend (group) variables
#' bxp3 <- ggboxplot(
#'   df,
#'   x = "supp", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' bxp3 + stat_anova_test(aes(group = dose), method = "two_way")
#'
#' # One-way repeatead measures ANOVA
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' df$id <- as.factor(c(rep(1:10, 3), rep(11:20, 3)))
#' ggboxplot(df, x = "dose", y = "len") +
#'   stat_anova_test(method = "one_way_repeated", wid = "id")
#'
#' # Two-way repeatead measures ANOVA
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' df$id <- as.factor(rep(1:10, 6))
#' ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
#'   stat_anova_test(aes(group = supp), method = "two_way_repeated", wid = "id")
#'
#' # Grouped one-way repeated measures ANOVA
#' ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
#'   stat_anova_test(aes(group = supp, color = supp),
#'     method = "one_way_repeated", wid = "id", group.by = "legend.var"
#'   )
#' }
#' @export
stat_anova_test <- function(mapping = NULL, data = NULL,
                            method = c("one_way", "one_way_repeated", "two_way", "two_way_repeated", "two_way_mixed"),
                            wid = NULL, group.by = NULL,
                            type = NULL, effect.size = "ges", error = NULL,
                            correction = c("auto", "GG", "HF", "none"),
                            label = "{method}, p = {p.format}",
                            label.x.npc = "left", label.y.npc = "top",
                            label.x = NULL, label.y = NULL, step.increase = 0.1,
                            p.adjust.method = "holm", significance = list(),
                            signif.cutoffs = NULL, signif.symbols = NULL,
                            ns.symbol = "ns", use.four.stars = FALSE,
                            p.format.style = "default", p.digits = NULL,
                            p.leading.zero = NULL, p.min.threshold = NULL,
                            p.decimal.mark = NULL,
                            geom = "text", position = "identity", na.rm = FALSE, show.legend = FALSE,
                            inherit.aes = TRUE, parse = FALSE, ...) {
  label <- get_anova_test_label_template(label)
  if (effect.size == "pes") {
    label <- gsub(pattern = "ges", replacement = "pes", x = label, fixed = TRUE)
    label <- gsub(pattern = "eta2[g]", replacement = "eta2[p]", x = label, fixed = TRUE)
  }
  if (missing(parse) & is_plotmath_expression(label)) {
    parse <- TRUE
  }
  correction <- match.arg(correction)
  method <- match.arg(method)

  if (!is.null(wid)) {
    if (!is.null(mapping)) {
      mapping$wid <- as.name(wid)
    } else {
      mapping <- create_aes(list(wid = wid))
    }
  }

  layer(
    stat = StatCompareMultipleMeans, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      method = method,
      method.args = list(type = type, effect.size = effect.size, error = error),
      group.by = group.by,
      correction = correction,
      na.rm = na.rm, stat.label = label,
      label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y, parse = parse,
      is.group.specified = is_group_aes_specified(mapping),
      step.increase = step.increase, p.adjust.method = p.adjust.method,
      significance = build_symnum_args(
        signif.cutoffs = signif.cutoffs,
        signif.symbols = signif.symbols,
        ns.symbol = ns.symbol,
        use.four.stars = use.four.stars,
        symnum.args = significance
      ),
      p.format.style = p.format.style, p.digits = p.digits,
      p.leading.zero = p.leading.zero, p.min.threshold = p.min.threshold,
      p.decimal.mark = p.decimal.mark, ...
    )
  )
}


StatCompareMultipleMeans <- ggproto("StatCompareMultipleMeans", Stat,
  required_aes = c("x", "y"),
  default_aes = aes(wid = NULL),
  setup_params = function(data, params) {
    # Initialize parameters
    methods_for_repeated_measures <- c(
      "one_way_repeated", "two_way_repeated", "two_way_mixed",
      "friedman_test"
    )
    if (params$method %in% methods_for_repeated_measures) {
      if (is.null(data$wid)) {
        stop(
          "The argument `wid` (sample id) is required for repeated measures tests.\n",
          "Specify it using wid = 'id_col_name', where id_col_name is the column containing ids.",
          call. = FALSE
        )
      }
    }
    return(params)
  },
  compute_panel = function(data, scales, method, method.args, group.by,
                           correction, p.adjust.method,
                           stat.label, label.x.npc, label.y.npc, label.x, label.y,
                           significance, is.group.specified, step.increase,
                           p.format.style, p.digits, p.leading.zero, p.min.threshold,
                           p.decimal.mark) {
    p <- p.adj <- x <- NULL
    if (method %in% c("one_way_repeated", "friedman_test")) {
      # One-way repeated measures ANOVA
      if (!is.group.specified) data$group <- data$x
    }

    df <- data
    is.grouped.plots <- contains_multiple_grouping_vars(df)


    # Statistical test parameters
    # test_args <- get_test_args(data, between, within)
    test_args <- get_test_args(data, method = method, group.by = group.by)
    wid <- test_args$wid
    between <- test_args$between
    within <- test_args$within
    group <- test_args$group

    nb_vars <- unique(c(between, within)) %>% length()
    is_two_way <- nb_vars >= 2
    # label group id. Used to shift coordinates to avoid overlaps.
    x.group <- y.group <- 1

    # Run statistical tests
    df <- df %>% rstatix::convert_as_factor(vars = c(between, within))
    if (!is.null(group)) {
      df <- df %>% rstatix::df_group_by(vars = group)
    }

    anova_methods <- c("one_way", "one_way_repeated", "two_way", "two_way_repeated", "two_way_mixed")
    if (method %in% anova_methods) {
      method.args <- method.args %>%
        .add_item(data = df, dv = "y", between = between, within = within, wid = wid)
      stat.test <- do.call(rstatix::anova_test, method.args) %>%
        rstatix::get_anova_table(correction = correction)
      method.name <- "Anova"
    } else if (method == "kruskal_test") {
      .formula <- paste0("y ~", between) %>% as.formula()
      method.name <- "Kruskal-Wallis"
      stat.test <- rstatix::kruskal_test(df, .formula)
      stat.test$statistic <- round(stat.test$statistic, 2)
    } else if (method == "welch_anova_test") {
      .formula <- paste0("y ~", between) %>% as.formula()
      method.name <- "Welch Anova"
      stat.test <- rstatix::welch_anova_test(df, .formula)
      stat.test$statistic <- round(stat.test$statistic, 2)
    } else if (method == "friedman_test") {
      if (is.null(within)) stop("The argument 'within' is required.")
      if (is.null(wid)) stop("The argument 'wid' is required.")
      .formula <- paste0("y ~", within, " | ", wid) %>% as.formula()
      method.name <- "Friedman test"
      stat.test <- rstatix::friedman_test(df, .formula)
      stat.test$statistic <- round(stat.test$statistic, 2)
    } else {
      stop("Don't support the method: ", method, call. = FALSE)
    }

    # Prepare the output for visualization
    if (is_two_way) {
      stat.test <- get_interaction_row(stat.test)
    }
    # Grouped tests
    else if (!is.null(group)) {
      # When data is grouped by x variable and
      # stat test computed between legend groups at each x position
      # labels group ids are x
      if (group == "x") {
        x.group <- stat.test$x
        if (is.null(label.x) & length(label.x.npc) == 1) {
          label.x <- stat.test$x
        }
      }
      # When data is grouped by legend variable and
      # stat test is computed between x variable groups
      # labels are stacked on y-axis. labels group ids are group (legend var)
      else if (group == "group") {
        if ("colour" %in% colnames(df)) stat.test$colour <- .levels(df$colour)
        y.group <- stat.test$group
      }
    }

    sy <- significance
    stat.test <- stat.test %>%
      rstatix::adjust_pvalue(method = p.adjust.method) %>%
      rstatix::add_significance(p.col = "p", cutpoints = sy$cutpoints, symbols = sy$symbols) %>%
      rstatix::add_significance(p.col = "p.adj", cutpoints = sy$cutpoints, symbols = sy$symbols)

    # Format p-values using the specified style
    if (p.format.style == "default") {
      stat.test <- stat.test %>%
        rstatix::p_format(p, p.adj, new.col = TRUE, accuracy = 1e-4)
    } else {
      stat.test <- stat.test %>%
        dplyr::mutate(
          p.format = format_p_value(p,
            style = p.format.style,
            digits = p.digits, leading.zero = p.leading.zero,
            min.threshold = p.min.threshold,
            decimal.mark = p.decimal.mark
          ),
          p.adj.format = format_p_value(p.adj,
            style = p.format.style,
            digits = p.digits, leading.zero = p.leading.zero,
            min.threshold = p.min.threshold,
            decimal.mark = p.decimal.mark
          )
        )
    }

    stat.test <- stat.test %>%
      add_stat_n() %>%
      keep_only_tbl_df_classes() %>%
      mutate(method = method.name) %>%
      add_stat_label(label = stat.label)


    stat.test$x <- get_coord(
      data.ranges = scales$x$dimension(),
      coord = label.x, npc = label.x.npc,
      margin.npc = 0, group = x.group
    )
    stat.test$y <- get_coord(
      data.ranges = scales$y$dimension(),
      coord = label.y, npc = label.y.npc,
      margin.npc = 0, group = y.group,
      step = -step.increase
    )

    # set label horizontal and vertical justification
    stat.test <- set_label_hvjust(stat.test, data = df, group)
    stat.test
  }
)


# Returns arguments for the tests
# data: layer data
get_test_args <- function(data, method = "one_way", group.by = NULL) {
  # Sample within id (considered for repeated measures)
  wid <- between <- within <- group <- NULL
  if ("wid" %in% colnames(data)) wid <- "wid"
  # Grouped or not grouped plots
  is_grouped_plots <- data %>% contains_multiple_grouping_vars()

  if (method %in% c("one_way", "kruskal_test", "welch_anova_test")) {
    between <- "x"
    if (is_grouped_plots) {
      if (is.null(group.by)) group.by <- "x.var"
      # Group by x axis variable and compute test between legend variable groups
      if (group.by == "x.var") {
        group <- "x"
        between <- "group"
      }
      # Group by legend variable and compute test between x axis groups
      else if (group.by == "legend.var") {
        group <- "group"
        between <- "x"
      }
    }
  } else if (method %in% c("one_way_repeated", "friedman_test")) {
    within <- "x"
    if (is_grouped_plots) {
      if (is.null(group.by)) group.by <- "x.var"
      # Group by x axis variable and compute test between legend variable groups
      if (group.by == "x.var") {
        group <- "x"
        within <- "group"
      }
      # Group by legend variable and compute test between x axis groups
      else if (group.by == "legend.var") {
        group <- "group"
        within <- "x"
      }
    }
  } else if (method == "two_way") {
    between <- c("x", "group")
  } else if (method == "two_way_repeated") {
    within <- c("x", "group")
  } else if (method == "two_way_mixed") {
    # Auto-detect between- and within-subjects variables
    # ID should be unique for one of the grouping variable
    # It's unique for the within-ss variable when data is in a wide format
    wid.freq.group <- data %>%
      dplyr::group_by(.data$group) %>%
      dplyr::count(.data$wid) %>%
      dplyr::pull(.data$n) %>%
      unique()
    wid.freq.x <- data %>%
      dplyr::group_by(.data$x) %>%
      dplyr::count(.data$wid) %>%
      dplyr::pull(.data$n) %>%
      unique()
    if (!(all(wid.freq.group == 1) | all(wid.freq.x == 1))) {
      stop("This is not a mixed design. Check your data.", call. = FALSE)
    }

    if (all(wid.freq.group == 1)) {
      within <- "group"
      between <- "x"
    } else if (all(wid.freq.x == 1)) {
      within <- "x"
      between <- "group"
    }
  } # end two-way-mixed

  list(dv = "y", between = between, within = within, wid = wid, group = group)
}


# Take the last row of the ANOVA table
# Corresponds to the interaction in the two-way model.
# In one-way: there is only one row
get_interaction_row <- function(stat.test) {
  utils::tail(stat.test, 1)
}


get_anova_test_label_template <- function(label) {
  switch(label,
    as_italic = "{method}, italic(p) = {p.format}",
    as_detailed = "{method}, F({DFn}, {DFd}) = {F}, eta2[g] = {ges}, p = {p.format}, n = {n}",
    as_detailed_italic = "{method}, italic(F)({DFn}, {DFd}) = {F}, eta2[g] = {ges}, italic(p) = {p.format}, italic(n) = {n}",
    as_detailed_expression = "{method}, italic(F)({DFn}, {DFd}) = {F}, eta2[g] = {ges}, italic(p) = {p.format}, italic(n) = {n}",
    label
  )
}


# set label horizontal and vertical justification
# group: grouping variable; either "x" or "group"
set_label_hvjust <- function(stat.test, data, group) {
  # Adapt hjust and x position for grouped plots
  hjust <- 0.2
  vjust <- 0
  if (!is.null(group)) {
    if (group == "x") {
      # Center label at each x position
      hjust <- 0.5
    } else if (group == "group") {
      n <- length(stat.test$group)
      dodge <- 0.8
      if (all(stat.test$x == 1)) {
        # Move labels to the left
        stat.test$x <- round(((dodge - dodge * n) / (2 * n)) + stat.test$x, 2)
      }
    }
  }
  # Two-way anova: move label to the left
  else if ("Effect" %in% colnames(stat.test)) {
    if (all(stat.test$Effect == "x:group") & all(stat.test$x == 1)) {
      n <- length(unique(data$group))
      dodge <- 0.8
      stat.test$x <- round(((dodge - dodge * n) / (2 * n)) + stat.test$x, 2)
    }
  }
  stat.test <- stat.test %>% tibble::add_column(hjust = hjust, vjust = vjust)
  stat.test
}

context("test-stat_anova_test")

# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)
df$id <- rep(1:10, 6) # Add individuals id
# Add a random grouping variable
set.seed(123)
df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
df <- df %>% mutate(
  len = ifelse(group == "grp2", len + 2, len),
  len = ifelse(group == "grp3", len + 7, len)
)


# Basic plots -----------------------------------
test_that("stat_anova_test works for basic ggplots", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    stat_anova_test()
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), 1)
  expect_equal(as.numeric(stat.test$y), 36.4)
  expect_equal(stat.test$label, "Anova, p < 0.0001")
})

# Grouped plots-----------------------------------
test_that("stat_anova_test works for grouped plots: grouped by x position", {
  bxp <- ggboxplot(df, x = "group", y = "len", color = "dose") +
    stat_anova_test(aes(group = dose), label = "{p.adj.format}")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(1, 2, 3))
  expect_equal(as.numeric(stat.test$y), c(36.4, 36.4, 36.4))
  expect_equal(stat.test$label, c("<0.0001", "0.00014", "0.00014"))
})

test_that("stat_anova_test works for grouped plots: grouped by legend variable", {
  bxp <- ggboxplot(df, x = "group", y = "len", color = "dose") +
    stat_anova_test(aes(group = dose), label = "{p.format}", group.by = "legend.var")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(0.73, 0.73, 0.73))
  expect_equal(as.numeric(stat.test$y), c(36.4, 39.62, 42.84))
  expect_equal(stat.test$label, c("0.009", "0.012", "0.019"))
})

test_that("stat_anova_test supports new significance parameters", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    stat_anova_test(
      label = "p.signif",
      signif.cutoffs = c(0.10, 0.05, 0.01),
      signif.symbols = c("+", "++", "+++")
    )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_true(stat.test$label %in% c("+", "++", "+++"))
})


# One-way repeated measure ANOVA ----------------------
test_that("stat_anova_test works for one-way repeated measure anova", {
  df$id <- as.factor(c(rep(1:10, 3), rep(11:20, 3)))
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    stat_anova_test(wid = "id", method = "one_way_repeated")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), 1)
  expect_equal(as.numeric(stat.test$y), 36.4)
  expect_equal(stat.test$label, "Anova, p < 0.0001")
})


test_that("stat_anova_test works for grouped one-way repeated measure anova: group by x var", {
  df$id <- as.factor(rep(1:10, 6))
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
    stat_anova_test(aes(group = supp, color = supp),
      label = "p",
      method = "one_way_repeated",
      wid = "id", group.by = "x.var"
    )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(1, 2, 3))
  expect_equal(as.numeric(stat.test$y), c(36.4, 36.4, 36.4))
  expect_equal(stat.test$label, c("0.016", "0.05", "0.848"))
})

test_that("stat_anova_test works for grouped one-way repeated measure anova: group by legend var", {
  df$id <- as.factor(rep(1:10, 6))
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
    stat_anova_test(aes(group = supp, color = supp),
      method = "one_way_repeated",
      wid = "id", group.by = "legend.var"
    )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(0.8, 0.8))
  expect_equal(as.numeric(stat.test$y), c(36.40, 39.62))
  expect_equal(stat.test$label, c("Anova, p = 0.004", "Anova, p < 0.0001"))
})

test_that("stat_anova_test works for two-way repeated measure anova", {
  df$id <- as.factor(rep(1:10, 6))
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
    stat_anova_test(aes(group = supp), method = "two_way_repeated", wid = "id")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), 0.8)
  expect_equal(as.numeric(stat.test$y), 36.4)
  expect_equal(stat.test$label, "Anova, p = 0.12")
})


# Two-way mixed anova -----------------------------
test_that("stat_anova_test works for two-way mixed anova", {
  anxiety <- data.frame(
    score = c(
      14.1, 14.5, 15.7, 16, 16.5, 16.9, 17, 17,
      17.3, 17.3, 17.8, 17.9, 19.1, 19.4, 19.8, 13.7, 14.7, 14.9, 15.1,
      15.8, 16.4, 16.6, 16.9, 16.9, 17.2, 17.8, 17.8, 18.2, 18.4,
      19.3, 14.6, 15, 15.5, 15.7, 16.4, 16.9, 17.1, 17.3, 17.5, 17.6,
      17.8, 17.9, 18.4, 18.5, 19, 14.4, 14.6, 15.2, 15.5, 15.8, 16.5,
      16.8, 17.1, 16.9, 17.1, 17.7, 17.7, 19.4, 19.2, 20, 13.4, 14.8,
      14.4, 14.8, 14.8, 15.7, 16.9, 16.9, 16.9, 17.3, 18.2, 17.7,
      17.8, 18.5, 18.9, 13, 13, 12.7, 14.1, 14.4, 14.5, 15.6, 15.4,
      15.6, 14.8, 16.1, 15.7, 16.7, 16.4, 17.2, 14.1, 14.3, 14.9, 15.3,
      15.7, 16.2, 16.5, 16.6, 16.5, 16.7, 17.3, 17.5, 19.3, 17.3,
      19.4, 12.7, 13.1, 13.6, 13.6, 14.2, 14.9, 16.1, 16.1, 16.3, 15.9,
      17.4, 16.9, 17.1, 17.3, 17.7, 11.7, 11.9, 11, 12.1, 12.3, 13.6,
      14.3, 14.2, 14.4, 13.8, 14.3, 13.8, 15.4, 15.1, 15.5
    ),
    id = as.factor(c(
      "1", "2", "3", "4", "5",
      "6", "7", "8", "9", "10", "11", "12", "13", "14",
      "15", "16", "17", "18", "19", "20", "21", "22", "23",
      "24", "25", "26", "27", "28", "29", "30", "31", "32",
      "33", "34", "35", "36", "37", "38", "39", "40", "41",
      "42", "43", "44", "45", "1", "2", "3", "4", "5",
      "6", "7", "8", "9", "10", "11", "12", "13", "14", "15",
      "16", "17", "18", "19", "20", "21", "22", "23", "24",
      "25", "26", "27", "28", "29", "30", "31", "32",
      "33", "34", "35", "36", "37", "38", "39", "40", "41",
      "42", "43", "44", "45", "1", "2", "3", "4", "5", "6",
      "7", "8", "9", "10", "11", "12", "13", "14", "15",
      "16", "17", "18", "19", "20", "21", "22", "23", "24",
      "25", "26", "27", "28", "29", "30", "31", "32", "33",
      "34", "35", "36", "37", "38", "39", "40", "41", "42",
      "43", "44", "45"
    )),
    group = as.factor(c(
      "grp1", "grp1", "grp1",
      "grp1", "grp1", "grp1", "grp1", "grp1", "grp1",
      "grp1", "grp1", "grp1", "grp1", "grp1", "grp1", "grp2",
      "grp2", "grp2", "grp2", "grp2", "grp2", "grp2", "grp2",
      "grp2", "grp2", "grp2", "grp2", "grp2", "grp2",
      "grp2", "grp3", "grp3", "grp3", "grp3", "grp3", "grp3",
      "grp3", "grp3", "grp3", "grp3", "grp3", "grp3",
      "grp3", "grp3", "grp3", "grp1", "grp1", "grp1", "grp1",
      "grp1", "grp1", "grp1", "grp1", "grp1", "grp1", "grp1",
      "grp1", "grp1", "grp1", "grp1", "grp2", "grp2",
      "grp2", "grp2", "grp2", "grp2", "grp2", "grp2", "grp2",
      "grp2", "grp2", "grp2", "grp2", "grp2", "grp2", "grp3",
      "grp3", "grp3", "grp3", "grp3", "grp3", "grp3",
      "grp3", "grp3", "grp3", "grp3", "grp3", "grp3", "grp3",
      "grp3", "grp1", "grp1", "grp1", "grp1", "grp1",
      "grp1", "grp1", "grp1", "grp1", "grp1", "grp1", "grp1",
      "grp1", "grp1", "grp1", "grp2", "grp2", "grp2", "grp2",
      "grp2", "grp2", "grp2", "grp2", "grp2", "grp2",
      "grp2", "grp2", "grp2", "grp2", "grp2", "grp3", "grp3",
      "grp3", "grp3", "grp3", "grp3", "grp3", "grp3", "grp3",
      "grp3", "grp3", "grp3", "grp3", "grp3", "grp3"
    )),
    time = as.factor(c(
      "t1", "t1", "t1", "t1",
      "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1",
      "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1",
      "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1",
      "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1", "t1",
      "t1", "t1", "t1", "t1", "t1", "t1", "t2", "t2", "t2",
      "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2",
      "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2",
      "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2",
      "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t2",
      "t2", "t2", "t2", "t2", "t2", "t2", "t2", "t3", "t3",
      "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3",
      "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3",
      "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3",
      "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3",
      "t3", "t3", "t3", "t3", "t3", "t3", "t3", "t3"
    ))
  )

  # two way
  bxp <- ggboxplot(anxiety, x = "group", y = "score", color = "time") +
    stat_anova_test(aes(group = time), method = "two_way_mixed", wid = "id")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  res_mixed_anova <- data.frame(
    x = as.numeric(stat.test$x),
    y = as.numeric(stat.test$y),
    label = as.character(stat.test$label),
    stringsAsFactors = FALSE
  )
  res_mixed_anova_expected <- data.frame(x = 0.73, y = 20, label = "Anova, p < 0.0001", stringsAsFactors = FALSE)


  # Effect of group: Group by time and compute anova between x groups
  bxp <- ggboxplot(anxiety, x = "group", y = "score", color = "time") +
    stat_anova_test(aes(group = time, color = time), method = "one_way", group.by = "legend.var")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  res_group_effect <- data.frame(
    x = as.numeric(stat.test$x),
    y = as.numeric(stat.test$y),
    label = as.character(stat.test$label),
    stringsAsFactors = FALSE
  )
  res_group_effect_expected <- data.frame(
    x = c(0.73, 0.73, 0.73),
    y = c(20.0, 20.9, 21.8),
    label = c("Anova, p = 0.696", "Anova, p = 0.006", "Anova, p < 0.0001"),
    stringsAsFactors = FALSE
  )


  # Effect of time: group by x  groups and perform the test within group (here time)
  bxp <- ggboxplot(anxiety, x = "group", y = "score", color = "time") +
    stat_anova_test(aes(group = time), method = "one_way_repeated", wid = "id", group.by = "x.var")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  res_time_effect <- data.frame(
    x = as.numeric(stat.test$x),
    y = as.numeric(stat.test$y),
    label = as.character(stat.test$label),
    stringsAsFactors = FALSE
  )
  res_time_effect_expected <- data.frame(
    x = 1:3,
    y = c(20, 20, 20),
    label = c("Anova, p < 0.0001", "Anova, p < 0.0001", "Anova, p < 0.0001"),
    stringsAsFactors = FALSE
  )

  expect_equal(res_mixed_anova, res_mixed_anova_expected)
  expect_equal(res_group_effect, res_group_effect_expected)
  expect_equal(res_time_effect, res_time_effect_expected)
})

#' @include utilities.R
NULL
#' Extract Legends from a ggplot object
#' @description Extract the legend labels from a ggplot object.
#' @param p an object of class ggplot or a list of ggplots. If p is a list, only the first legend is returned.
#' @param position character specifying legend position. Allowed values are one of
#'   c("top", "bottom", "left", "right", "none"). To remove the legend use
#'   legend = "none".
#' @return an object of class gtable.
#' @examples
#' # Create a scatter plot
#' p <- ggscatter(iris,
#'   x = "Sepal.Length", y = "Sepal.Width",
#'   color = "Species", palette = "jco",
#'   ggtheme = theme_minimal()
#' )
#' p
#'
#' # Extract the legend. Returns a gtable
#' leg <- get_legend(p)
#'
#' # Convert to a ggplot and print
#' as_ggplot(leg)
#'
#' @export
get_legend <- function(p, position = NULL) {
  if (.is_list(p)) {
    continue <- TRUE
    i <- 1
    while (i <= length(p) & continue) {
      leg <- .get_legend(p[[i]], position = position)
      if (!is.null(leg)) continue <- FALSE
      i <- i + 1
    }
  } else {
    leg <- .get_legend(p, position = position)
  }
  leg
}


# Return legend for one plot
.get_legend <- function(p, position = NULL) {
  if (is.null(p)) {
    return(NULL)
  }
  if (!is.null(position)) {
    p <- p + theme(legend.position = position)
  }
  tmp <- ggplot_gtable(ggplot_build(p))
  leg <- which(sapply(tmp$grobs, function(x) x$name) == "guide-box")
  if (length(leg) > 0) {
    leg <- tmp$grobs[[leg]]
  } else {
    leg <- NULL
  }
  leg
}

context("test-ggarrange-common-legend")

# #347: common.legend = TRUE keeps only the FIRST plot's legend (it is not merged
# or validated). A numeric common.legend now selects WHICH plot's legend to use as
# the shared one, e.g. common.legend = 2 uses the second plot's legend. Logical
# TRUE/FALSE are unchanged.

suppressPackageStartupMessages(library(ggplot2))

# p1 has 2 species, p2 has all 3 -> their legends genuinely differ.
p1 <- ggboxplot(subset(iris, Species != "virginica"), "Species", "Sepal.Length", fill = "Species")
p2 <- ggboxplot(iris, "Species", "Sepal.Width", fill = "Species")

# md5 of the rendered arrangement (same machine -> deterministic; fonts identical).
.arr_hash <- function(p) {
  f <- tempfile(fileext = ".png")
  on.exit(unlink(f))
  suppressMessages(ggplot2::ggsave(f, p, width = 6, height = 3, dpi = 72, device = "png"))
  unname(tools::md5sum(f))
}

test_that("numeric common.legend selects that plot's legend", {
  h_true <- .arr_hash(ggarrange(p1, p2, ncol = 2, common.legend = TRUE))
  h_i1   <- .arr_hash(ggarrange(p1, p2, ncol = 2, common.legend = 1))
  h_i2   <- .arr_hash(ggarrange(p1, p2, ncol = 2, common.legend = 2))
  # index 1 is exactly the historical behavior (first plot's legend)
  expect_identical(h_i1, h_true)
  # index 2 picks a DIFFERENT (representative) legend -> different output
  expect_false(identical(h_i2, h_i1))
  # equivalent to supplying that legend via legend.grob
  h_grob <- .arr_hash(ggarrange(p1, p2, ncol = 2, common.legend = TRUE,
                                legend.grob = get_legend(p2)))
  expect_identical(h_i2, h_grob)
})

test_that("common.legend = TRUE / FALSE are unchanged and render", {
  expect_error(cowplot::as_grob(ggarrange(p1, p2, ncol = 2, common.legend = TRUE)), NA)
  expect_error(cowplot::as_grob(ggarrange(p1, p2, ncol = 2, common.legend = FALSE)), NA)
})

test_that("invalid numeric common.legend is rejected with a clear error", {
  expect_error(ggarrange(p1, p2, common.legend = 5), "between")     # out of range
  expect_error(ggarrange(p1, p2, common.legend = 0), "between")     # < 1
  expect_error(ggarrange(p1, p2, common.legend = 1.5), "whole")     # non-integer
  expect_error(ggarrange(p1, p2, common.legend = NA_real_))         # NA
})

test_that("common.legend index pointing to a NULL plot errors clearly", {
  expect_error(ggarrange(p1, NULL, p2, common.legend = 2), "NULL")
})

# ---- D3: several indices combine the selected plots' legends ----

# a third plot with a DIFFERENT (continuous) legend
p3 <- ggscatter(mtcars, "wt", "mpg", color = "cyl")

test_that("a vector common.legend combines the selected plots' legends", {
  # combining two DIFFERENT legends must differ from either single legend alone
  h_1   <- .arr_hash(ggarrange(p2, p3, ncol = 2, common.legend = 1))
  h_3   <- .arr_hash(ggarrange(p2, p3, ncol = 2, common.legend = 2))
  h_13  <- .arr_hash(ggarrange(p2, p3, ncol = 2, common.legend = c(1, 2)))
  expect_false(identical(h_13, h_1))
  expect_false(identical(h_13, h_3))
})

test_that("vector common.legend renders for every legend position", {
  for (pos in c("top", "bottom", "left", "right")) {
    expect_error(
      cowplot::as_grob(ggarrange(p2, p3, ncol = 2, common.legend = c(1, 2), legend = pos)),
      NA
    )
  }
})

test_that("vector common.legend validation: duplicates and out-of-range are rejected", {
  expect_error(ggarrange(p1, p2, common.legend = c(1, 1)), "unique")       # duplicate index
  expect_error(ggarrange(p1, p2, common.legend = c(1, 9)), "between")      # out of range
  expect_error(ggarrange(p1, p2, common.legend = c(1, 2.5)))               # non-integer element
  expect_error(ggarrange(p1, p2, common.legend = c(1, NA)))                # NA element
})

test_that("single-element vector behaves like the scalar index", {
  h_scalar <- .arr_hash(ggarrange(p1, p2, ncol = 2, common.legend = 2))
  h_vec1   <- .arr_hash(ggarrange(p1, p2, ncol = 2, common.legend = c(2)))
  expect_identical(h_vec1, h_scalar)
})

#' geom_signif exported from ggsignif package
#'
#' See \code{ggsignif::\link[ggsignif:stat_signif]{geom_signif}} for details.
#'
#' @name geom_signif
#' @rdname geom_signif
#' @keywords internal
#' @export
#' @importFrom ggsignif geom_signif
NULL

# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Helper function for adding annotation to a ggplot
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::


# Get label parameters for each group
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Returns a data frame with x, y, hjust, vjust
# group.id is the index position of the group in a boxplot for example
# Pick the axis position for a label from its npc fraction. With
# label.anchor = "panel" the fraction is returned as an AsIs (`I()`) value so
# ggplot2 (>= 3.5.0) places it at the true panel-relative npc AT DRAW TIME -
# this keeps labels aligned across panels/facets whose axis ranges differ (e.g.
# scales = "free_y" or geom_smooth extending each panel), fixing #248. With
# label.anchor = "data" (default) the caller-supplied `data.value` - the exact
# historical data-coordinate expression for that anchor - is returned verbatim,
# so default output stays bit-identical.
.npc_to_position <- function(npc, data.value, label.anchor = "data") {
  if (identical(label.anchor, "panel")) return(I(npc))
  data.value
}

.label_params <- function(data, scales, label.x.npc = "left", label.y.npc = "right",
                          label.x = NULL, label.y = NULL, label.y.step = 1.4,
                          label.anchor = c("data", "panel"),
                          .by = c("group", "panel"),
                          group.id = NULL, ...) {
  .by <- match.arg(.by)
  label.anchor <- match.arg(label.anchor)
  # Check label coordinates for each group
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (is.null(group.id)) {
    group.id <- group.id <- abs(data$group[1])
  }
  label.x.npc <- .group_coord(label.x.npc, group.id)
  label.y.npc <- .group_coord(label.y.npc, group.id)
  label.x <- .group_coord(label.x, group.id)
  label.y <- .group_coord(label.y, group.id)

  .check_npc_coord(label.x.npc, axis = "x")
  .check_npc_coord(label.y.npc, axis = "y")


  if (length(label.x) > 0) {
    x <- label.x
    hjust <- 0.5
  } else if (length(label.x.npc) > 0) {
    # For each anchor, resolve the npc fraction (for panel mode) alongside the
    # EXACT historical data expression (for the default data mode), so
    # label.anchor = "data" is bit-identical to previous versions.
    if (is.numeric(label.x.npc)) {
      x <- .npc_to_position(
        label.x.npc,
        scales$x$dimension()[1] + label.x.npc * diff(scales$x$dimension()),
        label.anchor
      )
      hjust <- 0.5
    } else if (is.character(label.x.npc)) {
      if (label.x.npc == "right") {
        x <- .npc_to_position(1, scales$x$dimension()[2], label.anchor)
        hjust <- 1
      } else if (label.x.npc %in% c("center", "centre", "middle")) {
        x <- .npc_to_position(0.5, mean(scales$x$dimension()), label.anchor)
        hjust <- 0.5
      } else if (label.x.npc == "left") {
        x <- .npc_to_position(0, scales$x$dimension()[1], label.anchor)
        hjust <- 0
      }
    }
  }

  if (length(label.y) > 0) {
    y <- label.y
    vjust <- 0.5
  } else if (length(label.y.npc) > 0) {
    if (is.numeric(label.y.npc)) {
      y <- .npc_to_position(
        label.y.npc,
        scales$y$dimension()[1] + label.y.npc * diff(scales$y$dimension()),
        label.anchor
      )
      vjust <- label.y.step * group.id - (label.y.step / 2 * length(group.id))
    } else if (is.character(label.y.npc)) {
      if (label.y.npc == "bottom") {
        y <- .npc_to_position(0, scales$y$dimension()[1], label.anchor)
        vjust <- -label.y.step * group.id
      } else if (label.y.npc %in% c("center", "centre", "middle")) {
        y <- .npc_to_position(0.5, mean(scales$y$dimension()), label.anchor)
        vjust <- label.y.step * group.id - (label.y.step / 2 * length(group.id))
      } else if (label.y.npc == "top") {
        y <- .npc_to_position(1, scales$y$dimension()[2], label.anchor)
        vjust <- label.y.step * group.id
      }
    }
  }
  if (.by == "panel") {
    hjust <- 0.5
    vjust <- 0.5
  }

  data.frame(x = x, y = y, hjust = hjust, vjust = vjust)
}


# Get label parameters by group
# Useful in boxplot, where group.ids is the index of the group: 1, 2, 3, etc
# Useful only when computation is done by panel
.label_params_by_group <- function(..., group.ids) {
  purrr::map(
    group.ids,
    function(group.id, ...) {
      .label_params(..., group.id = group.id)
    },
    ...
  ) %>%
    dplyr::bind_rows() # %>%
  # dplyr::mutate(x = group.ids)
}


# Check label coordinates for each group
# :::::::::::::::::::::::::::::::::::::::::
# coord.values: label coordinate for each group. If too short, they are recycled.
# group.id the id of groups as returned by ggplot_build()
.group_coord <- function(coord.values, group.id) {
  if (!.is_empty(coord.values)) {
    # group.id must be a valid 1-based index; a non-positive id (e.g. when the
    # group value is derived from an x position of 0) would make
    # coord.values[group.id] empty and produce NA. Fall back to the first value
    # (recycling) in that case (#594).
    coord.values <- ifelse(length(coord.values) >= group.id & group.id > 0,
      coord.values[group.id], coord.values[1]
    )
  }
  coord.values
}


# Check NPC coord
# :::::::::::::::::::::::::::::::::::::::::
# npc: Normalised Parent Coordinates.
#   The origin of the viewport is (0, 0) and the viewport has a width and height of 1 unit.
#   For example, (0.5, 0.5) is the centre of the viewport.
# coord: should be between 0 and 1
# axis: should be "x" or "y"
.check_npc_coord <- function(.coord, axis = c("x", "y")) {
  axis <- match.arg(axis)
  if (axis == "x") {
    allowed.values <- c("right", "left", "center", "centre", "middle")
  } else if (axis == "y") {
    allowed.values <- c("bottom", "top", "center", "centre", "middle")
  }

  .message <- paste0(
    "'*.npc coord for ", axis, " axis should be either a numeric value in [0-1] ",
    "or a character strings including one of ",
    .collapse(allowed.values, sep = ", ")
  )

  if (!is.null(.coord)) {
    if (is.numeric(.coord)) {
      if (any(.coord < 0 | .coord > 1)) {
        stop(.message)
      }
    } else if (is.character(.coord)) {
      if (!(.coord %in% allowed.values)) {
        stop(.message)
      }
    } else {
      stop(.message)
    }
  }
}

#' @include utilities.R
NULL
#' Facet a ggplot into Multiple Panels
#' @description Create multi-panel plots of a data set grouped by one or two
#'  grouping variables. Wrapper around \code{\link[ggplot2]{facet_wrap}}
#' @param p a ggplot
#' @param facet.by character vector, of length 1 or 2, specifying grouping
#'  variables for faceting the plot into multiple panels. Should be in the data.
#' @param nrow,ncol Number of rows and columns in the panel. Used only when the
#'  data is faceted by one grouping variable.
#' @param scales should axis scales of panels be fixed ("fixed", the default),
#'  free ("free"), or free in one dimension ("free_x", "free_y").
#' @param short.panel.labs logical value. Default is TRUE. If TRUE, create short
#'  labels for panels by omitting variable names; in other words panels will be
#'  labelled only by variable grouping levels.
#' @param labeller Character vector. An alternative to the argument
#'  \code{short.panel.labs}. Possible values are one of "label_both" (panel
#'  labelled by both grouping variable names and levels) and "label_value"
#'  (panel labelled with only grouping levels).
#' @param panel.labs a list of one or two character vectors to modify facet panel
#'  labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
#'  the labels for the "sex" variable. For two grouping variables, you can use
#'  for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
#'  "Lev", "Lev2") ).
#' @param panel.labs.background a list to customize the background of panel
#'  labels. Should contain the combination of the following elements: \itemize{
#'  \item \code{color, linetype, linewidth}: background line color, type and width
#'  \item \code{fill}: background fill color. } For example,
#'  panel.labs.background = list(color = "blue", fill = "pink", linetype =
#'  "dashed", linewidth = 0.5). Note: \code{size} is deprecated, use
#'  \code{linewidth} instead.
#' @param panel.labs.font a list of aesthetics indicating the size (e.g.: 14), the
#'  face/style (e.g.: "plain", "bold", "italic", "bold.italic") and the color
#'  (e.g.: "red") and the orientation angle (e.g.: 45) of panel labels.
#' @param panel.labs.font.x,panel.labs.font.y same as panel.labs.font but for
#'  only x and y direction, respectively.
#' @param strip.position (used only in \code{facet_wrap()}). By default, the
#'  labels are displayed on the top of the plot. Using \code{strip.position} it
#'  is possible to place the labels on either of the four sides by setting
#'  \code{strip.position = c("top", "bottom", "left", "right")}
#' @param ... not used
#' @examples
#' p <- ggboxplot(ToothGrowth,
#'   x = "dose", y = "len",
#'   color = "supp"
#' )
#' print(p)
#'
#' facet(p, facet.by = "supp")
#'
#' # Customize
#' facet(p + theme_bw(),
#'   facet.by = "supp",
#'   short.panel.labs = FALSE, # Allow long labels in panels
#'   panel.labs.background = list(fill = "steelblue", color = "steelblue")
#' )
#' @name facet
#' @rdname facet
#' @export
facet <- function(p, facet.by, nrow = NULL, ncol = NULL,
                  scales = "fixed", short.panel.labs = TRUE, labeller = "label_value",
                  panel.labs = NULL,
                  panel.labs.background = list(color = NULL, fill = NULL),
                  panel.labs.font = list(face = NULL, color = NULL, size = NULL, angle = NULL),
                  panel.labs.font.x = panel.labs.font,
                  panel.labs.font.y = panel.labs.font,
                  strip.position = "top", ...) {
  if (length(facet.by) > 2) {
    stop("facet.by should be of length 1 or 2.")
  }
  if (!missing(labeller)) {
    if (labeller == "label_value") {
      short.panel.labs <- TRUE
    } else if (labeller == "label_both") {
      short.panel.labs <- FALSE
    } else {
      stop("Don't support the following labeller: ", labeller, call. = FALSE)
    }
  }

  panel.labs.background <- .compact(panel.labs.background)
  # Convert size to linewidth for element_rect (ggplot2 3.4.0+ compatibility)
  if ("size" %in% names(panel.labs.background)) {
    panel.labs.background$linewidth <- panel.labs.background$size
    panel.labs.background$size <- NULL
  }
  panel.labs.font.x <- .compact(panel.labs.font.x)
  panel.labs.font.y <- .compact(panel.labs.font.y)

  .labeller <- "label_value"

  if (!is.null(panel.labs)) {
    .labeller <- .create_labeller(p$data, panel.labs)
  } else if (!short.panel.labs) {
    .labeller <- label_both
  }


  if (length(facet.by) == 1) {
    facet.formula <- paste0("~", glue::backtick(facet.by)) %>% stats::as.formula()
    p <- p + facet_wrap(facet.formula,
      nrow = nrow, ncol = ncol, scales = scales, labeller = .labeller,
      strip.position = strip.position
    )
  } else if (length(facet.by) == 2) {
    facet.formula <- paste(glue::backtick(facet.by), collapse = " ~ ") %>% stats::as.formula()
    p <- p + facet_grid(facet.formula, scales = scales, labeller = .labeller)
  }

  if (!.is_empty(panel.labs.background)) {
    p <- p + theme(strip.background = do.call(element_rect, panel.labs.background))
  }
  if (!.is_empty(panel.labs.font.x)) {
    p <- p + theme(strip.text.x = do.call(element_text, panel.labs.font.x))
  }
  if (!.is_empty(panel.labs.font.y)) {
    p <- p + theme(strip.text.y = do.call(element_text, panel.labs.font.y))
  }

  p
}


# Create labeller to rename panel labels
.create_labeller <- function(data, panel.labs = NULL) {
  if (is.null(panel.labs)) {
    return(NULL)
  }

  if (!is.null(panel.labs) & !.is_list(panel.labs)) {
    stop("Argument panel.labs should be a list. Read the documentation.")
  }

  if (is.null(names(panel.labs))) {
    stop(
      "panel.labs should be a named list. ",
      "Ex: panel.labs = list(sex = c('Male', 'Female') )"
    )
  }

  variables <- names(panel.labs)

  . <- NULL
  .labels <- list()

  for (variable in variables) {
    current.levels <- .levels(data[[variable]])
    provided.levels <- panel.labs[[variable]]

    if (length(current.levels) != length(provided.levels)) {
      stop(
        "The number of ", variable, " levels in panel.labs ",
        "and in the data are different."
      )
    }

    if (!is.null(names(provided.levels)) &&
        all(current.levels %in% names(provided.levels))) {
      # Named labels covering every data level: match them to the levels by
      # name, so they aren't mis-assigned when the order differs (#643).
      provided.levels <- provided.levels[current.levels]
    } else {
      # Unnamed labels (or names that don't cover the levels): map positionally
      # to the data levels, preserving the previous default behavior.
      names(provided.levels) <- current.levels
    }
    .labels[[variable]] <- provided.levels
  }

  if (!.is_empty(.labels)) {
    do.call(ggplot2::labeller, .labels)
  } else {
    return(NULL)
  }
}

test_that(".create_labeller honors named panel.labs, else maps positionally (#643)", {
  df <- ToothGrowth
  df$dose <- factor(df$dose)  # levels "0.5","1","2"

  # Named labels covering all levels (given in a different order) are matched by name
  lab <- .create_labeller(df, list(dose = c("2" = "High", "0.5" = "Low", "1" = "Mid")))
  res <- lab(data.frame(dose = factor(c("0.5", "1", "2"))))$dose
  expect_equal(res, c("Low", "Mid", "High"))

  # No-regression: UNnamed labels still map positionally
  lab2 <- .create_labeller(df, list(dose = c("Low", "Mid", "High")))
  res2 <- lab2(data.frame(dose = factor(c("0.5", "1", "2"))))$dose
  expect_equal(res2, c("Low", "Mid", "High"))

  # No-regression: names that do NOT cover the data levels fall back to positional
  # (same as the previous behavior, which discarded the names)
  lab3 <- .create_labeller(df, list(dose = c(a = "Low", b = "Mid", c = "High")))
  res3 <- lab3(data.frame(dose = factor(c("0.5", "1", "2"))))$dose
  expect_equal(res3, c("Low", "Mid", "High"))
})

#' @include utilities.R ggpar.R
NULL
#' Violin plot
#' @description Create a violin plot with error bars. Violin plots are similar to
#' box plots, except that they also show the kernel probability density of the
#' data at different values.
#' @inheritParams ggboxplot
#' @param width violin width.
#' @param drop logical, passed to \code{\link[ggplot2]{geom_violin}()}, controlling
#'  whether grouped sub-samples with fewer than two data points (for which no
#'  density can be computed) are dropped from the plot and the dodge position. By
#'  default \code{ggviolin()} keeps grouped violins aligned with their other layers
#'  (e.g. added boxplots or dot plots) automatically: when you do not set \code{drop}
#'  or \code{position} and a grouped sub-group has a single data point, the empty
#'  dodge lane is reserved (equivalent to \code{drop = FALSE} together with
#'  \code{position = position_dodge(0.8, preserve = "single")}) so the violins stay
#'  aligned. Balanced, ungrouped, and faceted plots are unaffected. Setting
#'  \code{drop} or \code{position} explicitly turns off this automatic handling and
#'  uses exactly what you supply (#381).
#' @param alpha color transparency. Values should be between 0 and 1.
#' @param linewidth constant value specifying the line width.
#' @param quantiles numeric vector of quantiles to draw on the violin.
#' @param quantile.linetype linetype for quantile lines; set to draw quantiles
#'  with ggplot2 >= 4.0.0.
#' @param quantile.type quantile algorithm passed to ggplot2.
#' @param quantile.alpha,quantile.colour,quantile.color,quantile.linewidth,quantile.size
#'  aesthetics for quantile lines.
#' @inheritParams ggplot2::geom_violin
#' @param ... other arguments to be passed to
#'   \code{\link[ggplot2]{geom_violin}}, \code{\link{ggpar}} and
#'   \code{\link{facet}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'  ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
#'  ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
#'  = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
#'  and position: legend = "right" \item plot orientation : orientation =
#'  c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' # +++++++++++++++++++++++++++
#' ggviolin(df, x = "dose", y = "len")
#' # Change the plot orientation: horizontal
#' ggviolin(df, "dose", "len", orientation = "horiz")
#'
#' # Add summary statistics
#' # ++++++++++++++++++++++++++
#' # Draw quantiles
#' ggviolin(df, "dose", "len",
#'   add = "none",
#'   quantiles = 0.5, quantile.linetype = "dashed"
#' )
#'
#' # Add box plot
#' ggviolin(df,
#'   x = "dose", y = "len",
#'   add = "boxplot"
#' )
#'
#' ggviolin(df,
#'   x = "dose", y = "len",
#'   add = "dotplot"
#' )
#'
#' # Add jitter points and
#' # change point shape by groups ("dose")
#' ggviolin(df,
#'   x = "dose", y = "len",
#'   add = "jitter", shape = "dose"
#' )
#'
#'
#' # Add mean_sd + jittered points
#' ggviolin(df,
#'   x = "dose", y = "len",
#'   add = c("jitter", "mean_sd")
#' )
#'
#' # Change error.plot to "crossbar"
#' ggviolin(df,
#'   x = "dose", y = "len",
#'   add = "mean_sd", error.plot = "crossbar"
#' )
#'
#'
#' # Change colors
#' # +++++++++++++++++++++++++++
#' # Change outline and fill colors
#' ggviolin(df, "dose", "len",
#'   color = "black", fill = "gray"
#' )
#'
#' # Change outline colors by groups: dose
#' # Use custom color palette and add boxplot
#' ggviolin(df, "dose", "len",
#'   color = "dose",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   add = "boxplot"
#' )
#'
#' # Change fill color by groups: dose
#' # add boxplot with white fill color
#' ggviolin(df, "dose", "len",
#'   fill = "dose",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   add = "boxplot", add.params = list(fill = "white")
#' )
#'
#'
#' # Plot with multiple groups
#' # +++++++++++++++++++++
#' # fill or color box plot by a second group : "supp"
#' ggviolin(df, "dose", "len",
#'   color = "supp",
#'   palette = c("#00AFBB", "#E7B800"), add = "boxplot"
#' )
#'
#' @export
ggviolin <- function(data, x, y, combine = FALSE, merge = FALSE,
                     color = "black", fill = "white", palette = NULL, alpha = 1,
                     title = NULL, xlab = NULL, ylab = NULL,
                     facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                     linetype = "solid", trim = FALSE, drop = TRUE, size = NULL, linewidth = NULL, width = 1,
                     quantiles = NULL, quantile.linetype = NULL, quantile.type = NULL,
                     quantile.alpha = NULL, quantile.colour = NULL, quantile.color = NULL,
                     quantile.linewidth = NULL, quantile.size = NULL,
                     draw_quantiles = NULL,
                     select = NULL, remove = NULL, order = NULL,
                     add = "mean_se", add.params = list(),
                     error.plot = "pointrange",
                     label = NULL, font.label = list(size = 11, color = "black"),
                     label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                     position = position_dodge(0.8), ggtheme = theme_pubr(),
                     show.n = FALSE,
                     ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette, alpha = alpha,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    linetype = linetype, trim = trim, drop = drop, size = size, linewidth = linewidth, width = width,
    quantiles = quantiles, quantile.linetype = quantile.linetype, quantile.type = quantile.type,
    quantile.alpha = quantile.alpha, quantile.colour = quantile.colour, quantile.color = quantile.color,
    quantile.linewidth = quantile.linewidth, quantile.size = quantile.size, draw_quantiles = draw_quantiles,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle, position = position, ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  .opts$fun <- ggviolin_core
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }

  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)

  p <- do.call(.plotter, .opts)

  if (isTRUE(show.n) && !missing(x)) {
    if (.is_list(p)) {
      p <- purrr::map(p, .add_show_n, x = x, color = color, fill = fill,
                      facet.by = facet.by, position = position)
    } else {
      p <- .add_show_n(p, x = x, color = color, fill = fill,
                       facet.by = facet.by, position = position)
    }
  }

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


# #381: TRUE when a grouped (dodged) violin has a color/fill x x cell that
# contains exactly ONE non-missing y value - geom_violin's density-drop trigger
# (ggplot2 >= 4.0 drops groups with fewer than two points from the plot AND from
# position adjustment, so the remaining violins re-center and misalign). Used to
# decide whether to auto-reserve the empty dodge lane. Returns FALSE when there
# is no grouping aesthetic (no dodge in play), so ungrouped violins never change.
# A genuinely-absent cell (0 points, e.g. a legitimately-unbalanced design) has
# Freq 0 and is deliberately NOT treated as a trigger, keeping that case
# byte-identical to ggplot2's default centered position.
.violin_has_singleton_group <- function(data, x, color = NULL, fill = NULL, y = NULL) {
  group.vars <- unique(c(color, fill))
  group.vars <- group.vars[!is.na(group.vars)]
  group.vars <- setdiff(intersect(group.vars, colnames(data)), x)
  if (length(group.vars) == 0) return(FALSE)          # no dodge -> never trigger
  key.cols <- c(x, group.vars)
  keep <- if (!is.null(y) && y %in% colnames(data)) !is.na(data[[y]]) else rep(TRUE, nrow(data))
  df <- data[keep, key.cols, drop = FALSE]
  if (nrow(df) == 0) return(FALSE)
  df[] <- lapply(df, function(col) factor(as.character(col)))
  counts <- table(df)
  any(counts == 1)                                     # an OBSERVED 1-point cell
}

ggviolin_core <- function(data, x, y,
                          color = "black", fill = "white", palette = NULL, alpha = 1,
                          title = NULL, xlab = NULL, ylab = NULL,
                          linetype = "solid", trim = FALSE, drop = NULL, size = NULL, linewidth = NULL, width = 1,
                          quantiles = NULL, quantile.linetype = NULL, quantile.type = NULL,
                          quantile.alpha = NULL, quantile.colour = NULL, quantile.color = NULL,
                          quantile.linewidth = NULL, quantile.size = NULL,
                          draw_quantiles = NULL,
                          add = "mean_se", add.params = list(),
                          error.plot = "pointrange",
                          ggtheme = theme_pubr(),
                          position = NULL,
                          ...) {
  line_width <- .resolve_linewidth_args(
    size = size, linewidth = linewidth, default_linewidth = NULL
  )
  size <- line_width$size
  linewidth <- line_width$linewidth

  if (!is.factor(data[[x]])) data[[x]] <- as.factor(data[[x]])

  # #381: keep grouped violins aligned with their box/dot layers by default when a
  # sub-group is too sparse for geom_violin to compute a density. `drop`/`position`
  # arrive as NULL when the user did not set them; only then (and only when the data
  # actually has a grouped one-point cell) do we reserve the empty dodge lane via
  # `drop = FALSE` + a "preserve single" dodge - both are needed together. Any other
  # input keeps the historical default (drop = TRUE, position = position_dodge(0.8)),
  # so balanced / ungrouped / faceted / legitimately-unbalanced plots are unchanged.
  auto.align <- is.null(drop) && is.null(position)
  if (is.null(drop)) drop <- TRUE
  if (is.null(position)) position <- position_dodge(0.8)
  if (auto.align && .violin_has_singleton_group(data, x, color = color, fill = fill, y = y)) {
    drop <- FALSE
    position <- position_dodge(0.8, preserve = "single")
  }

  if (!is.null(quantile.color) && is.null(quantile.colour)) {
    quantile.colour <- quantile.color
  }

  if (!is.null(draw_quantiles)) {
    if (is.null(quantiles)) {
      quantiles <- draw_quantiles
      if (is.null(quantile.linetype)) {
        quantile.linetype <- "solid"
        warning(
          "`draw_quantiles` is deprecated. Mapped to `quantiles` and set ",
          "`quantile.linetype = 'solid'` to preserve legacy behavior. ",
          "With ggplot2 >= 4.0.0, quantile lines are only drawn when ",
          "`quantile.linetype` is set; set it explicitly (or set it to NULL ",
          "to keep the new default)."
        )
      } else {
        warning("`draw_quantiles` is deprecated. Use `quantiles` instead.")
      }
    } else {
      warning("`draw_quantiles` is deprecated and ignored because `quantiles` was supplied.")
    }
  }

  if (!is.null(quantiles) && is.null(quantile.linetype) && is.null(draw_quantiles)) {
    warning(
      "`quantiles` provided but `quantile.linetype` is NULL. ",
      "With ggplot2 >= 4.0.0, quantile lines are only drawn when ",
      "`quantile.linetype` is set."
    )
  }

  pms <- .violin_params(...)

  p <- ggplot(data, create_aes(list(x = x, y = y))) +
    geom_exec(geom_violin,
      data = data,
      color = color, fill = fill, linetype = linetype,
      trim = trim, drop = drop, size = linewidth, width = width, alpha = alpha,
      position = position, draw_quantiles = NULL, quantiles = quantiles,
      quantile.linetype = quantile.linetype, quantile.type = quantile.type,
      quantile.alpha = quantile.alpha, quantile.colour = quantile.colour,
      quantile.linewidth = quantile.linewidth, quantile.size = quantile.size,
      stat = pms$stat, scale = pms$scale, adjust = pms$adjust
    )

  # Add
  #+++++++++++++++++++
  if (is.null(add.params$group)) {
    if (fill %in% names(data)) add.params$group <- fill
  }
  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill, ...) %>%
    .add_item(p = p, add = add, error.plot = error.plot, position = position)
  p <- do.call(ggadd, add.params) %>%
    ggpar(
      palette = palette, ggtheme = ggtheme,
      title = title, xlab = xlab, ylab = ylab, ...
    )

  p
}

# Regression tests for #398: ggviolin(scales = "free") crashed at draw time
# ("object 'violinwidth' not found") because .violin_params() used `$`, which
# partial-matched the facet argument `scales` to the violin `scale` parameter,
# setting an invalid scale = "free".

test_that("ggviolin() with scales='free' renders (no crash) (#398)", {
  expect_no_error(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      ggviolin(ToothGrowth, x = "dose", y = "len", scales = "free")
    ))
  )
  # violinwidth must be computed (its absence caused the crash)
  d <- ggplot2::ggplot_build(
    ggviolin(ToothGrowth, x = "dose", y = "len", scales = "free"))$data[[1]]
  expect_true("violinwidth" %in% names(d))
})

test_that("ggviolin() scales='free' reaches faceting, not the violin scale (#398)", {
  p <- ggviolin(iris, x = "Species", y = "Sepal.Length",
                facet.by = "Species", scales = "free")
  b <- ggplot2::ggplot_build(p)
  expect_true(isTRUE(b$layout$facet$params$free$x))
  expect_true(isTRUE(b$layout$facet$params$free$y))
  # the violin scale itself stays at its default, NOT "free"
  expect_equal(p$layers[[1]]$stat_params$scale, "area")
})

test_that("ggviolin() explicit scale= is still honored (no regression, #398)", {
  expect_equal(
    ggviolin(ToothGrowth, x = "dose", y = "len", scale = "count")$layers[[1]]$stat_params$scale,
    "count"
  )
  expect_equal(
    ggviolin(ToothGrowth, x = "dose", y = "len")$layers[[1]]$stat_params$scale,
    "area"
  )
})

test_that("ggviolin maps draw_quantiles to quantiles with default linetype", {
  df <- ToothGrowth

  expect_warning(
    p <- ggviolin(df, "dose", "len", add = "none", draw_quantiles = 0.5),
    "Mapped to `quantiles`"
  )

  layer <- p$layers[[1]]
  expect_equal(layer$stat_params$quantiles, 0.5)
  expect_null(layer$geom_params$draw_quantiles)
})

test_that("ggviolin warns when draw_quantiles is provided with quantiles", {
  df <- ToothGrowth

  expect_warning(
    p <- ggviolin(
      df,
      "dose",
      "len",
      add = "none",
      draw_quantiles = 0.5,
      quantiles = 0.25
    ),
    "ignored because `quantiles` was supplied"
  )

  layer <- p$layers[[1]]
  expect_equal(layer$stat_params$quantiles, 0.25)
})

test_that("ggviolin warns when draw_quantiles is used with explicit linetype", {
  df <- ToothGrowth

  expect_warning(
    p <- ggviolin(
      df,
      "dose",
      "len",
      add = "none",
      draw_quantiles = 0.5,
      quantile.linetype = "dashed"
    ),
    "Use `quantiles` instead"
  )

  layer <- p$layers[[1]]
  expect_equal(layer$stat_params$quantiles, 0.5)
})

test_that("ggviolin warns when quantiles are provided without linetype", {
  df <- ToothGrowth

  expect_warning(
    ggviolin(df, "dose", "len", add = "none", quantiles = 0.5),
    "quantile.linetype"
  )
})

# drop argument: keep sparse groups aligned with other geoms (#381) -----------

# Data with a single-point sub-group at dose = 1 (OJ), which geom_violin drops.
.viol_sparse <- rbind(
  subset(ToothGrowth, dose == 0.5),
  ToothGrowth[ToothGrowth$dose == 1 & ToothGrowth$supp == "VC", ][1:6, ],
  ToothGrowth[ToothGrowth$dose == 1 & ToothGrowth$supp == "OJ", ][1, ],
  subset(ToothGrowth, dose == 2)
)
.viol_sparse$dose <- factor(.viol_sparse$dose)

.violin_lanes <- function(p, layer = 1) {
  # geom_violin emits expected "fewer than two datapoints" warnings for the
  # sparse group; they are incidental to what these tests assert.
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  d <- b$data[[layer]]
  s <- d[abs(d$x - 2) < 0.45, ]           # dose = 1 is the 2nd x position
  sort(unique(round(s$x, 3)))
}

test_that("ggviolin forwards an explicit drop to geom_violin", {
  p <- ggviolin(.viol_sparse, "dose", "len", color = "supp", add = "boxplot", drop = FALSE)
  expect_false(p$layers[[1]]$stat_params$drop)
  # An explicit drop = TRUE is respected (it turns the automatic alignment off).
  p2 <- ggviolin(.viol_sparse, "dose", "len", color = "supp", add = "boxplot", drop = TRUE)
  expect_true(p2$layers[[1]]$stat_params$drop)
})

test_that("ggviolin auto-aligns a sparse grouped violin by default (#381)", {
  # Omitting drop/position now reserves the empty dodge lane for the single-point
  # sub-group, so the sparse dose = 1 keeps TWO aligned lanes and the auto-applied
  # drop is FALSE.
  omitted <- ggviolin(.viol_sparse, "dose", "len", color = "supp", add = "boxplot")
  expect_length(.violin_lanes(omitted, 1), 2)
  expect_false(omitted$layers[[1]]$stat_params$drop)
  # Explicit drop = TRUE opts out -> historical behavior: the sparse lane is dropped
  # and the remaining violin re-centers (a single lane at dose = 1).
  explicit <- ggviolin(.viol_sparse, "dose", "len", color = "supp", add = "boxplot", drop = TRUE)
  expect_length(.violin_lanes(explicit, 1), 1)
})

test_that("omitting drop is byte-identical to drop = TRUE for balanced data (#381)", {
  # The automatic alignment only fires on a sparse one-point cell; for balanced
  # grouped data the default is unchanged, so omitting drop equals drop = TRUE.
  bal <- ToothGrowth
  bal$dose <- factor(bal$dose)
  omitted  <- ggviolin(bal, "dose", "len", color = "supp", add = "boxplot")
  explicit <- ggviolin(bal, "dose", "len", color = "supp", add = "boxplot", drop = TRUE)
  expect_equal(
    suppressWarnings(ggplot2::ggplot_build(omitted))$data[[1]],
    suppressWarnings(ggplot2::ggplot_build(explicit))$data[[1]]
  )
})

test_that("drop = FALSE + preserve = 'single' aligns violins with added boxplots (#381)", {
  p <- ggviolin(
    .viol_sparse, "dose", "len", color = "supp", add = "boxplot",
    drop = FALSE, position = position_dodge(0.8, preserve = "single")
  )
  violin_x <- .violin_lanes(p, 1)
  box_x    <- .violin_lanes(p, 2)
  expect_length(violin_x, 2)          # empty lane reserved -> two lanes
  expect_equal(violin_x, box_x, tolerance = 1e-6)  # violins line up with boxes
})

context("test-ggviolin-align")

# #381: grouped ggviolin() violins failed to line up with their box/dot layers
# when a sub-group was too sparse for geom_violin() to compute a density
# (ggplot2 >= 4.0 drops groups with fewer than two points from BOTH the plot and
# the dodge, so the remaining violin re-centers). ggviolin() now reserves the
# empty dodge lane automatically - but ONLY when the user set neither `drop` nor
# `position` AND the data actually has a grouped one-point cell. Every other input
# keeps the historical default (drop = TRUE, position = position_dodge(0.8)), so
# balanced / ungrouped / faceted / legitimately-unbalanced plots are byte-identical.

suppressPackageStartupMessages(library(ggplot2))

# sorted violin lane centers (npc-free, in data x units)
.violin_lane_centers <- function(p) {
  b <- suppressWarnings(suppressMessages(ggplot2::ggplot_build(p)))
  i <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomViolin"), logical(1)))[1]
  d <- b$data[[i]]
  sort(unname(vapply(split(d, d$group), function(g) mean(range(g$x)), numeric(1))))
}

tg <- ToothGrowth; tg$dose <- factor(tg$dose)

# data from the issue: dose = 1 / OJ has a single point (no density)
sparse <- rbind(
  subset(ToothGrowth, dose == 0.5),
  ToothGrowth[ToothGrowth$dose == 1 & ToothGrowth$supp == "VC", ][1:6, ],
  ToothGrowth[ToothGrowth$dose == 1 & ToothGrowth$supp == "OJ", ][1, ],
  subset(ToothGrowth, dose == 2)
)
sparse$dose <- factor(sparse$dose)

# ---- The helper: precise trigger ----

test_that(".violin_has_singleton_group triggers only on an observed 1-point cell", {
  # sparse: dose=1/OJ has exactly one point -> TRUE
  expect_true(.violin_has_singleton_group(sparse, "dose", color = "supp", y = "len"))
  # balanced ToothGrowth -> FALSE
  expect_false(.violin_has_singleton_group(tg, "dose", color = "supp", y = "len"))
  # legitimately-unbalanced (dose=2 has ZERO OJ, all present cells >= 2) -> FALSE
  d2 <- tg[!(tg$dose == "2" & tg$supp == "OJ"), ]
  expect_false(.violin_has_singleton_group(d2, "dose", color = "supp", y = "len"))
  # no grouping aesthetic (color is a constant, not a column) -> FALSE
  expect_false(.violin_has_singleton_group(tg, "dose", color = "black", y = "len"))
})

# ---- The fix: sparse grouped violin aligns by default ----

test_that("grouped violin with a sparse sub-group aligns by default (no manual args)", {
  centers <- .violin_lane_centers(ggviolin(sparse, "dose", "len", color = "supp", add = "boxplot"))
  # all six dodge lanes present and aligned like a fully balanced design
  expect_equal(centers, c(0.8, 1.2, 1.8, 2.2, 2.8, 3.2))
})

test_that("the auto-fix renders without error", {
  p <- ggviolin(sparse, "dose", "len", color = "supp", add = "boxplot")
  # geom_violin still emits its usual "cannot compute density for < 2 points"
  # message for the single-point sub-group; we only assert it does not ERROR.
  expect_error(
    suppressWarnings(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p))), NA
  )
})

# ---- No trigger: unchanged behavior ----

test_that("balanced and ungrouped violins keep their default positions", {
  expect_equal(.violin_lane_centers(ggviolin(tg, "dose", "len", color = "supp")),
               c(0.8, 1.2, 1.8, 2.2, 2.8, 3.2))
  expect_equal(.violin_lane_centers(ggviolin(tg, "dose", "len")), c(1, 2, 3))
})

test_that("a legitimately-unbalanced design keeps the lone violin centered (ggplot2 default)", {
  d2 <- tg[!(tg$dose == "2" & tg$supp == "OJ"), ]
  centers <- .violin_lane_centers(ggviolin(d2, "dose", "len", color = "supp"))
  # dose=2 has only VC -> its violin stays at the slot center (3.0), NOT the lane (2.8)
  expect_true(3.0 %in% centers)
  expect_false(2.8 %in% centers)
})

# ---- Explicit user arguments turn the auto-handling off ----

test_that("an explicit position= is respected (auto-handling disabled)", {
  centers <- .violin_lane_centers(
    ggviolin(sparse, "dose", "len", color = "supp", add = "boxplot",
             position = position_dodge(0.8))
  )
  # historical behavior: the sparse group is dropped from the dodge -> NOT all six aligned
  expect_false(isTRUE(all.equal(centers, c(0.8, 1.2, 1.8, 2.2, 2.8, 3.2))))
})

test_that("an explicit drop= is respected (auto-handling disabled)", {
  centers <- .violin_lane_centers(
    ggviolin(sparse, "dose", "len", color = "supp", add = "boxplot", drop = TRUE)
  )
  expect_false(isTRUE(all.equal(centers, c(0.8, 1.2, 1.8, 2.2, 2.8, 3.2))))
})

#' @include utilities.R
NULL
#' Graphical parameters
#'
#' @param p an object of class ggplot or a list of ggplots
#'
#' @param palette the color palette to be used for coloring or filling by
#'   groups. Allowed values include "grey" for grey color palettes; brewer
#'   palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
#'   "red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
#'   "aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
#'   "rickandmorty". Can be also a numeric vector of length(groups); in this
#'   case a basic color palette is created using the function
#'   \link[grDevices]{palette}.
#' @param gradient.cols vector of colors to use for n-colour gradient. Allowed
#'   values include brewer and ggsci color palettes.
#' @param main plot main title.
#' @param title plot main title.
#' @param submain,subtitle plot subtitle.
#' @param caption plot caption.
#' @param xlab character vector specifying x axis labels. Use xlab = FALSE to
#'   hide xlab.
#' @param ylab character vector specifying y axis labels. Use ylab = FALSE to
#'   hide ylab.
#' @param font.main,font.submain,font.caption,font.x,font.y a vector of length 3
#'   indicating respectively the size (e.g.: 14), the style (e.g.: "plain",
#'   "bold", "italic", "bold.italic") and the color (e.g.: "red") of main title,
#'   subtitle, caption, xlab and ylab, respectively. For example \emph{font.x =
#'   c(14, "bold", "red")}. Use font.x = 14, to change only font size; or use
#'   font.x = "bold", to change only font face.
#' @param font.title,font.subtitle alias of font.submain and font.submain,
#'   respectively.
#' @param font.family character vector specifying font family.
#' @param xlim,ylim a numeric vector of length 2, specifying  x and y axis
#'   limits (minimum and maximum), respectively. e.g.: ylim = c(0, 50).
#' @param xscale,yscale x and y axis scale, respectively. Allowed values are one
#'   of c("none", "log2", "log10", "sqrt"); e.g.: yscale="log2".
#' @param format.scale logical value. If TRUE, axis tick mark labels will be
#'   formatted when xscale or yscale = "log2" or "log10".
#' @param legend character specifying legend position. Allowed values are one of
#'   c("top", "bottom", "left", "right", "none"). To remove the legend use
#'   legend = "none". Legend position can be also specified using a numeric
#'   vector c(x, y); see details section.
#' @param legend.direction character specifying the layout of legend items.
#'   Allowed values are "horizontal" or "vertical". Useful to override the
#'   default direction, e.g. to stack items vertically with
#'   \code{legend = "bottom"}.
#' @param legend.title legend title, e.g.: \code{legend.title = "Species"}. Can
#'   be also a list, \code{legend.title = list(color = "Species", linetype = "Species",
#'   shape = "Species")}.
#' @param font.legend legend text font style; e.g.: font.legend = c(10, "plain",
#'   "black").
#' @param ticks logical value. Default is TRUE. If FALSE, hide axis tick marks.
#' @param tickslab logical value. Default is TRUE. If FALSE, hide axis tick
#'   labels.
#' @param font.tickslab,font.xtickslab,font.ytickslab Font style (size, face,
#'   color) for tick labels, e.g.: c(14, "bold", "red").
#' @param x.text.angle,y.text.angle Numeric value specifying the rotation angle
#'   of x and y axis tick labels, respectively. Default value is NULL. For
#'   vertical x axis texts use x.text.angle = 90.
#' @param xtickslab.rt,ytickslab.rt Same as x.text.angle and y.text.angle,
#'   respectively. Will be deprecated in the near future.
#' @param xticks.by,yticks.by numeric value controlling x and y axis breaks,
#'   respectively. For example, if yticks.by = 5, a tick mark is shown on every
#'   5. Default value is NULL.
#' @param rotate logical value. If TRUE, rotate the graph by setting the plot
#'   orientation to horizontal.
#' @param orientation change the orientation of the plot. Allowed values are one
#'   of c( "vertical", "horizontal", "reverse"). Partial match is allowed.
#' @param ggtheme function, ggplot2 theme name. The default is set by each
#'   function's \code{ggtheme} argument; see the function usage for the actual
#'   default. Set \code{ggtheme = NULL} to skip applying a ggpubr theme, so the
#'   plot keeps ggplot2 default theme or the theme set globally via
#'   \code{theme_set()}.
#'   Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
#'   theme_minimal(), theme_classic(), theme_void(), ....
#' @param ... not used
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic box plot
#' # +++++++++++++++++++++++++++
#'
#' p <- ggboxplot(df, x = "dose", y = "len")
#'
#' # Change the plot orientation: horizontal
#' ggpar(p, orientation = "horiz")
#'
#'
#' # Change main title and axis labels
#' # ++++++++++++++++++++++++++++
#'
#' ggpar(p,
#'   main = "Plot of length \n by dose",
#'   xlab = "Dose (mg)", ylab = "Length"
#' )
#'
#' # Title font styles: 'plain', 'italic', 'bold', 'bold.italic'
#' ggpar(p,
#'   main = "Length by dose",
#'   font.main = c(14, "bold.italic", "red"),
#'   font.x = c(14, "bold", "#2E9FDF"),
#'   font.y = c(14, "bold", "#E7B800")
#' )
#'
#' # Hide axis labels
#' ggpar(p, xlab = FALSE, ylab = FALSE)
#'
#'
#' # Change colors
#' # ++++++++++++++++++++++
#'
#' # Change outline colors by groups: dose
#' p2 <- ggboxplot(df, "dose", "len", color = "dose")
#' p2
#'
#' # Use custom color palette
#' ggpar(p2, palette = c("#00AFBB", "#E7B800", "#FC4E07"))
#'
#' # Use brewer palette
#' ggpar(p2, palette = "Dark2")
#'
#' # Use grey palette
#' ggpar(p2, palette = "grey")
#'
#' # Use scientific journal palette from ggsci package
#' ggpar(p2, palette = "npg") # nature
#'
#' # Axis ticks, limits, scales
#' # +++++++++++++++++++++++++
#'
#' # Axis ticks labels and rotation
#' ggpar(p,
#'   font.tickslab = c(14, "bold", "#993333"),
#'   xtickslab.rt = 45, ytickslab.rt = 45
#' )
#' # Hide axis ticks and tick labels
#' ggpar(p, ticks = FALSE, tickslab = FALSE)
#'
#' # Axis limits
#' ggpar(p, ylim = c(0, 50))
#'
#' # Axis scale
#' ggpar(p, yscale = "log2")
#'
#' # Format axis scale
#' ggpar(p, yscale = "log2", format.scale = TRUE)
#'
#' # Legends
#' # ++++++++++++++++++
#' # Change legend position and title
#' ggpar(p2,
#'   legend = "right", legend.title = "Dose (mg)",
#'   font.legend = c(10, "bold", "red")
#' )
#'
#' @export
ggpar <- function(p, palette = NULL, gradient.cols = NULL,
                  main = NULL, submain = NULL, caption = NULL, xlab = NULL, ylab = NULL,
                  title = NULL, subtitle = NULL,
                  font.main = NULL, font.submain = NULL, font.x = NULL, font.y = NULL, font.caption = NULL,
                  font.title = NULL, font.subtitle = NULL, font.family = "",
                  xlim = NULL, ylim = NULL,
                  xscale = c("none", "log2", "log10", "sqrt"),
                  yscale = c("none", "log2", "log10", "sqrt"),
                  format.scale = FALSE,
                  legend = NULL, legend.direction = NULL,
                  legend.title = NULL, font.legend = NULL,
                  ticks = TRUE, tickslab = TRUE, font.tickslab = NULL,
                  font.xtickslab = font.tickslab, font.ytickslab = font.tickslab,
                  x.text.angle = NULL, y.text.angle = NULL,
                  xtickslab.rt = x.text.angle, ytickslab.rt = y.text.angle,
                  xticks.by = NULL, yticks.by = NULL,
                  rotate = FALSE,
                  orientation = c("vertical", "horizontal", "reverse"),
                  ggtheme = NULL,
                  ...) {
  original.p <- p
  if (rotate) orientation <- "horizontal"
  orientation <- match.arg(orientation, c("vertical", "horizontal", "reverse"))

  if (is_ggplot(original.p)) {
    list.plots <- list(original.p)
  } else if (is.list(original.p)) {
    list.plots <- original.p
  } else {
    stop("Can't handle an object of class ", class(original.p))
  }
  if (!is.null(title)) main <- title
  if (!is.null(subtitle)) submain <- subtitle
  if (!is.null(font.title)) font.main <- font.title
  if (!is.null(font.subtitle)) font.submain <- font.subtitle
  if (is.numeric(palette)) palette <- grDevices::palette()[palette]


  for (i in seq_along(list.plots)) {
    p <- list.plots[[i]]
    if (is_ggplot(p)) {
      p <- .apply_palette(p, palette)
      if (!is.null(ggtheme)) p <- p + ggtheme # labs_pubr() +
      if (!is.null(gradient.cols)) p <- p + .gradient_col(gradient.cols)

      p <- .set_ticks(p, ticks, tickslab, font.tickslab,
        xtickslab.rt, ytickslab.rt,
        font.xtickslab = font.xtickslab, font.ytickslab = font.ytickslab
      )
      p <- .set_ticksby(p, xticks.by, yticks.by)
      # For horizontal orientation the limits are passed to coord_flip() below;
      # adding coord_cartesian() here would be replaced by coord_flip(), dropping
      # the limits and warning about two coordinate systems (#646).
      if (orientation != "horizontal") p <- p + .set_axis_limits(xlim, ylim)
      p <- .set_legend(p, legend, legend.title, font.legend, legend.direction)
      p <- .set_scale(p, xscale = xscale, yscale = yscale, format.scale = format.scale)
      p <- .labs(p, main, xlab, ylab,
        font.main, font.x, font.y,
        submain = submain, caption = caption, font.submain = font.submain, font.caption = font.caption
      )
      p <- .set_orientation(p, orientation, xlim = xlim, ylim = ylim)
      if (font.family != "") {
        p <- p + theme(text = element_text(family = font.family))
      }
      list.plots[[i]] <- p
    }
  }

  if (is_ggplot(original.p)) {
    list.plots[[1]]
  } else {
    list.plots
  }
}

context("test-ggpar-element-markdown")

# #382 / survminer #520: ggpar() re-themes a plot by adding element_text() to theme
# slots. When a slot already holds a ggtext::element_markdown() (survminer sets one
# on the risk-table strata labels), ggplot2 errored "Only elements of the same class
# can be merged" / "Can't merge the `<slot>` theme element". ggpar() now leaves such
# markdown slots untouched instead of crashing; normal element_text slots are handled
# exactly as before.
#
# The tests build a markdown-classed element WITHOUT depending on ggtext, so they run
# everywhere: an element carrying the same "element_markdown" class signature triggers
# the identical merge path (dispatch/assertion in ggplot2 is class-based).

suppressPackageStartupMessages(library(ggplot2))

# Synthetic ggtext::element_markdown()-classed element (no ggtext dependency).
.fake_markdown_element <- function() {
  el <- ggplot2::element_text()
  class(el) <- c("element_markdown", class(el))
  el
}

test_that("ggpar() does not crash and renders when a slot is element_markdown (#382)", {
  p <- ggboxplot(ToothGrowth, "dose", "len") +
    theme(axis.text.y = .fake_markdown_element())
  expect_error(
    q <- ggpar(p, font.tickslab = c(14, "bold"), font.x = c(16), font.main = c(18)),
    NA
  )
  # full render must also succeed (draw-time, not just build)
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(q)), NA)
})

test_that("ggpar() leaves the markdown slot intact but restyles other slots (#382)", {
  p <- ggboxplot(ToothGrowth, "dose", "len") +
    theme(axis.text.y = .fake_markdown_element())
  q <- ggpar(p, font.tickslab = c(14, "bold"), font.x = c(16))
  # markdown slot preserved (not overwritten with element_text)
  expect_true(inherits(q$theme[["axis.text.y"]], "element_markdown"))
  # a normal slot is still restyled as requested
  expect_true(inherits(q$theme[["axis.text.x"]], "element_text"))
  expect_false(inherits(q$theme[["axis.text.x"]], "element_markdown"))
  expect_equal(q$theme[["axis.text.x"]]$size, 14)
  expect_equal(q$theme[["axis.title.x"]]$size, 16)
})

test_that("ggpar() handles a list of ggplots, guarding each markdown slot (#382, survminer #520)", {
  # Mimic a ggsurvplot: a list holding two ggplots, one with a markdown element on
  # the risk-table-like component (as survminer sets on its strata labels). ggpar()
  # must iterate the list, not crash, and preserve the markdown element.
  p_plot <- ggboxplot(ToothGrowth, "dose", "len")
  p_tab <- ggboxplot(ToothGrowth, "dose", "len") +
    theme(axis.text.y = .fake_markdown_element())
  lst <- list(plot = p_plot, table = p_tab)
  expect_error(res <- ggpar(lst, font.title = c(16, "bold"), font.x = c(14),
                            font.tickslab = c(12)), NA)
  expect_true(is.list(res))
  # markdown slot on the "table" is preserved, curve component restyled
  expect_true(inherits(res$table$theme[["axis.text.y"]], "element_markdown"))
  expect_equal(res$plot$theme[["axis.title.x"]]$size, 14)
  # both components render
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(res$plot)), NA)
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(res$table)), NA)
})

test_that("ggpar() output is unchanged for a normal plot (no-regression, #382)", {
  # For a plot with no markdown element, ggpar() must behave exactly as before:
  # every requested font slot is applied as an element_text with the given values.
  p <- ggboxplot(ToothGrowth, "dose", "len", color = "supp")
  q <- ggpar(p, main = "M", xlab = "X", ylab = "Y",
             font.main = c(20, "bold", "red"), font.x = c(14, "bold", "blue"),
             font.y = c(14), font.tickslab = c(11, "plain", "darkgreen"),
             font.legend = c(9), legend.title = "Supp",
             xtickslab.rt = 45, ytickslab.rt = 90)
  expect_equal(q$theme[["plot.title"]]$size, 20)
  expect_equal(q$theme[["plot.title"]]$colour, "red")
  expect_equal(q$theme[["axis.title.x"]]$size, 14)
  expect_equal(q$theme[["axis.title.x"]]$colour, "blue")
  expect_equal(q$theme[["axis.text.x"]]$size, 11)
  expect_equal(q$theme[["axis.text.x"]]$colour, "darkgreen")
  expect_equal(q$theme[["axis.text.x"]]$angle, 45)
  expect_equal(q$theme[["axis.text.y"]]$angle, 90)
  expect_equal(q$theme[["legend.text"]]$size, 9)
  # ticks still applied
  expect_true(inherits(q$theme[["axis.ticks"]], "element_line"))
})

test_that("ggpar(tickslab = FALSE) still blanks the tick labels (#382)", {
  p <- ggboxplot(ToothGrowth, "dose", "len")
  q <- ggpar(p, tickslab = FALSE)
  expect_true(inherits(q$theme[["axis.text.x"]], "element_blank"))
  expect_true(inherits(q$theme[["axis.text.y"]], "element_blank"))
})

test_that("ggpar legend.title applies to size and alpha legends (#412)", {
  lab <- function(p, a) ggplot2::ggplot_build(p)$plot$labels[[a]]

  # Regression: legend.title now titles a size legend and an alpha legend
  ps <- ggpar(ggscatter(mtcars, "wt", "mpg", size = "qsec"),
              legend.title = "Size title")
  expect_equal(lab(ps, "size"), "Size title")
  pa <- ggpar(ggscatter(mtcars, "wt", "mpg", alpha = "qsec"),
              legend.title = "Alpha title")
  expect_equal(lab(pa, "alpha"), "Alpha title")

  # No-regression: color aesthetic path unchanged
  pc <- ggpar(ggscatter(mtcars, "wt", "mpg", color = "cyl"),
              legend.title = "Color title")
  expect_equal(lab(pc, "colour"), "Color title")

  # No-regression: without legend.title the size label is untouched
  pd <- ggscatter(mtcars, "wt", "mpg", size = "qsec")
  expect_equal(lab(pd, "size"), "qsec")
})

#' @include utilities.R utils_stat_test_label.R utils-aes.R p_format_utils.R
NULL

#' Add Pairwise Comparisons P-values to a GGPlot
#' @description add pairwise comparison p-values to a ggplot such as box plots,
#'  dot plots and stripcharts.
#' @param method a character string indicating which method to be used for
#'  pairwise comparisons. Default is \code{"wilcox_test"}. Allowed methods
#'  include pairwise comparisons methods implemented in the \code{rstatix} R
#'  package. These methods are: \code{"wilcox_test", "t_test", "sign_test",
#'  "dunn_test", "emmeans_test", "tukey_hsd", "games_howell_test"}.
#' @param method.args a list of additional arguments used for the test method.
#'  For example one might use \code{method.args = list(alternative = "greater")}
#'  for wilcoxon test.
#' @param ref.group a character string or a numeric value specifying the
#'  reference group. If specified, for a given grouping variable, each of the
#'  group levels will be compared to the reference group (i.e. control group).
#'
#'  \code{ref.group} can be also \code{"all"}. In this case, each of the
#'  grouping variable levels is compared to all (i.e. basemean).
#'
#'  Allowed values can be: \itemize{ \item \strong{numeric value}: specifying
#'  the rank of the reference group. For example, use \code{ref.group = 1} when
#'  the first group is the reference; use \code{ref.group = 2} when the second
#'  group is the reference, and so on. This works for all situations, including
#'  i) when comparisons are performed between x-axis groups and ii) when
#'  comparisons are performed between legend groups. \item \strong{character
#'  value}: For example, you can use \code{ref.group = "ctrl"} instead of using
#'  the numeric rank value of the "ctrl" group. \item \strong{"all"}: In this
#'  case, each of the grouping variable levels is compared to all (i.e.
#'  basemean). }
#' @param label character string specifying label. Can be: \itemize{ \item the
#'  column containing the label (e.g.: \code{label = "p"} or \code{label =
#'  "p.adj"}), where \code{p} is the p-value. Other possible values are
#'  \code{"p.signif", "p.adj.signif", "p.format", "p.format.signif", "p.adj.format"}.
#'  \item an
#'  expression that can be formatted by the \code{\link[glue]{glue}()} package.
#'  For example, when specifying \code{label = "Wilcoxon, p = \{p\}"}, the
#'  expression \{p\} will be replaced by its value. \item a combination of
#'  plotmath expressions and glue expressions. You may want some of the
#'  statistical parameter in italic; for example:\code{label = "Wilcoxon,
#'  italic(p)= {p}"}}.
#' @param y.position numeric vector with the y positions of the brackets
#' @param group.by (optional) character vector specifying the grouping variable;
#'  it should be used only for grouped plots. Possible values are : \itemize{
#'  \item \code{"x.var"}: Group by the x-axis variable and perform the test
#'  between legend groups. In other words, the p-value is compute between legend
#'  groups at each x position \item \code{"legend.var"}: Group by the legend
#'  variable and perform the test between x-axis groups. In other words, the
#'  test is performed between the x-groups for each legend level. }
#' @param dodge dodge width for grouped ggplot/test. Default is 0.8. It's used to
#'  dodge the brackets position when \code{group.by = "legend.var"}.
#' @param stack logical value. Default is FALSE; should be set to TRUE for
#'  stacked bar plots or line plots. If TRUE, then the brackets are
#'  automatically removed and the \code{dodge} value is set to zero.
#' @param bracket.nudge.y Vertical adjustment to nudge brackets by (in fraction
#'  of the total height). Useful to move up or move down the bracket. If
#'  positive value, brackets will be moved up; if negative value, brackets are
#'  moved down.
#' @param bracket.shorten a small numeric value in [0-1] for shortening the width
#'  of bracket.
#' @param bracket.group.by (optional); a variable name for grouping brackets
#'  before adding step.increase. Useful for grouped plots. Possible values
#'  include \code{"x.var"} and \code{"legend.var"}.
#' @param step.increase numeric vector with the increase in fraction of total
#'  height for every additional comparison to minimize overlap.
#' @param tip.length numeric vector with the fraction of total height that the
#'  bar goes down to indicate the precise column/
#' @param size change the width of the lines of the bracket
#' @param label.size change the size of the label text
#' @param family change the font used for the text
#' @param vjust move the text up or down relative to the bracket.
#' @param hjust move the text left or right relative to the bracket.
#' @param p.adjust.method method for adjusting p values (see
#'  \code{\link[stats]{p.adjust}}).  Has impact only in a situation, where
#'  multiple pairwise tests are performed; or when there are multiple grouping
#'  variables. Ignored when the specified method is \code{"tukey_hsd"} or
#'  \code{"games_howell_test"} because they come with internal p adjustment
#'  method. Allowed values include "holm", "hochberg", "hommel", "bonferroni",
#'  "BH", "BY", "fdr", "none". If you don't want to adjust the p value (not
#'  recommended), use p.adjust.method = "none".
#' @param p.adjust.by possible value is one of \code{c("group", "panel")}.
#'  Default is \code{"group"}: for a grouped data, if pairwise test is
#'  performed, then the p-values are adjusted for each group level
#'  independently. P-values are adjusted by panel when \code{p.adjust.by =
#'  "panel"}.
#' @param p.adjust.n optional single number giving the number of comparisons to
#'  use for the p-value adjustment, passed as \code{n} to
#'  \code{\link[stats]{p.adjust}}. Default is \code{NULL}, which uses the number
#'  of p-values actually computed (the standard behavior). Set it when the
#'  displayed comparisons are a subset of a larger family and the adjustment
#'  should reflect that larger family size. As required by
#'  \code{\link[stats]{p.adjust}}, \code{p.adjust.n} must be greater than or
#'  equal to the number of p-values being adjusted. It affects the adjustment
#'  that \code{geom_pwc()} performs itself: the panel-level adjustment
#'  (\code{p.adjust.by = "panel"}) and cases where the test returns a single set
#'  of p-values (a single comparison, or grouped comparisons with two levels per
#'  group). It does not alter the adjustment that pairwise tests already perform
#'  internally per group.
#' @param symnum.args a list of arguments to pass to the function
#'  \code{\link[stats]{symnum}} for symbolic number coding of p-values. For
#'  example, \code{symnum.args = list(cutpoints = c(0, 0.0001, 0.001, 0.01,
#'  0.05, Inf), symbols = c("****", "***", "**", "*",  "ns"))}.
#'
#'  In other words, we use the following convention for symbols indicating
#'  statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
#'  \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001
#'  \item \code{****}:  p <= 0.0001 }
#'
#'  Note: If \code{symnum.args} is provided, it takes precedence over
#'  \code{signif.cutoffs}.
#' @param signif.cutoffs numeric vector of p-value cutoffs in descending order
#'  for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
#'  means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
#'  Default is NULL, which uses the package defaults.
#' @param signif.symbols character vector of symbols corresponding to
#'  \code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
#'  (and "****" if \code{use.four.stars = TRUE}).
#' @param ns.symbol character string for non-significant results. Default is "ns".
#'  Use "" (empty string) to show nothing.
#' @param use.four.stars logical. If TRUE, allows four stars (****) for the most
#'  significant level. Default is FALSE.
#' @param hide.ns can be logical value (\code{TRUE} or \code{FALSE}) or a character vector (\code{"p.adj"} or \code{"p"}).
#' @param p.format.style character string specifying the p-value formatting style.
#'  One of: \code{"default"} (backward compatible, uses scientific notation),
#'  \code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
#'  \code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
#'  (GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
#'  See \code{\link{list_p_format_styles}} for details.
#' @param p.digits integer specifying the number of decimal places for p-values.
#'  If provided, overrides the style default.
#' @param p.leading.zero logical indicating whether to include leading zero before
#'  decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.
#' @param p.min.threshold numeric specifying the minimum p-value to display exactly.
#'  Values below this threshold are shown as "< threshold". If provided, overrides
#'  the style default.
#' @param p.decimal.mark character string to use as the decimal mark. If NULL,
#'  uses \code{getOption("OutDec")}.
#' @param remove.bracket logical, if \code{TRUE}, brackets are removed from the
#'  plot. \itemize{ \item Case when logical value. If TRUE, hide ns symbol when
#'  displaying significance levels. Filter is done by checking the column
#'  \code{p.adj.signif}, \code{p.signif}, \code{p.adj} and \code{p}. \item Case
#'  when character value. Possible values are "p" or "p.adj", for filtering out
#'  non significant. }
#' @param na.rm If \code{FALSE} (the default), removes missing values with a
#'  warning.  If \code{TRUE} silently removes missing values.
#' @param parse logical for parsing plotmath expression.
#' @param ... other arguments passed on to \code{\link[ggplot2]{layer}()}). These are often
#'  aesthetics, used to set an aesthetic to a fixed value, like \code{color =
#'  "red"} or \code{size = 3}. They may also be parameters to the paired
#'  geom/stat.
#' @details
#' \bold{Notes on adjusted p-values and facet}. When using the ggplot facet functions, the p-values are computed and adjusted by panel, without taking into account the other panels. This is by design in ggplot2.
#'
#' In this case, when there is only one computed p-value by panel, then using `label = "p"` or `label = "p.adj"` will give the same results using `geom_pwc()`. Again, p-value computation and adjustment in a given facet panel is done independently to the other panels.
#'
#' For grouped plots with sparse data, if some subsets do not contain enough
#' levels or observations for the requested test, those subsets are skipped while
#' preserving valid inferential comparisons from comparable subsets.
#' Informative messages list skipped grouped subsets and their skip reasons
#' (for example, missing \code{ref.group} or fewer than two comparison levels).
#'
#' One might want to adjust the p-values of all the facet panels together. There are two solutions for that:
#'
#' \itemize{
#' \item Using \code{\link{ggadjust_pvalue}(p)} after creating the plot \code{p}
#' \item or adding the adjusted p-value manually using \code{\link{stat_pvalue_manual}()}. Read more at:
#' \itemize{
#'   \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-to-ggplot-facets/}{How to Add P-values to GGPlot Facets}
#'   \item \href{https://www.datanovia.com/en/blog/add-p-values-to-ggplot-facets-with-different-scales/}{Add P-values to GGPlot Facets with Different Scales}
#' }
#' }
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @examples
#' df <- ToothGrowth
#' df$dose <- factor(df$dose)
#'
#' @rdname geom_pwc
#' @seealso \code{\link{ggadjust_pvalue}}
#' @examples
#' # Data preparation
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Transform `dose` into factor variable
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Add a random grouping variable
#' df$group <- factor(rep(c("grp1", "grp2"), 30))
#' head(df, 3)
#'
#'
#' # Two groups by x position
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#'
#' # Create a box plot
#' # Add 10% spaces between the p-value labels and the plot border
#' bxp <- ggboxplot(
#'   df,
#'   x = "dose", y = "len",
#'   color = "supp", palette = c("#00AFBB", "#E7B800")
#' ) +
#'   scale_y_continuous(expand = expansion(mult = c(0.05, 0.10)))
#'
#'
#' # Add p-values onto the box plots
#' # label can be "p.format"  or "p.adj.format"
#' bxp + geom_pwc(
#'   aes(group = supp),
#'   tip.length = 0,
#'   method = "t_test", label = "p.format"
#' )
#'
#' # Show adjusted p-values and significance levels
#' # Hide ns (non-significant)
#' bxp + geom_pwc(
#'   aes(group = supp),
#'   tip.length = 0,
#'   method = "t_test", label = "{p.adj.format}{p.adj.signif}",
#'   p.adjust.method = "bonferroni", p.adjust.by = "panel",
#'   hide.ns = TRUE
#' )
#'
#' # Complex cases
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # 1. Add p-values of OJ vs VC at each dose group
#' bxp.complex <- bxp +
#'   geom_pwc(
#'     aes(group = supp),
#'     tip.length = 0,
#'     method = "t_test", label = "p.adj.format",
#'     p.adjust.method = "bonferroni", p.adjust.by = "panel"
#'   )
#' # 2. Add pairwise comparisons between dose levels
#' # Nudge up the brackets by 20% of the total height
#' bxp.complex <- bxp.complex +
#'   geom_pwc(
#'     method = "t_test", label = "p.adj.format",
#'     p.adjust.method = "bonferroni",
#'     bracket.nudge.y = 0.2
#'   )
#' # 3. Display the plot
#' bxp.complex
#'
#'
#' # Three groups by x position
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#'
#' # Simple plots
#' # _____________________________________
#'
#' # Box plots with p-values
#' bxp <- ggboxplot(
#'   df,
#'   x = "supp", y = "len", fill = "dose",
#'   palette = "npg"
#' )
#' bxp +
#'   geom_pwc(
#'     aes(group = dose),
#'     tip.length = 0,
#'     method = "t_test", label = "p.adj.format",
#'     bracket.nudge.y = -0.08
#'   ) +
#'   scale_y_continuous(expand = expansion(mult = c(0, 0.1)))
#'
#' # Bar plots with p-values
#' bp <- ggbarplot(
#'   df,
#'   x = "supp", y = "len", fill = "dose",
#'   palette = "npg", add = "mean_sd",
#'   position = position_dodge(0.8)
#' )
#' bp +
#'   geom_pwc(
#'     aes(group = dose),
#'     tip.length = 0,
#'     method = "t_test", label = "p.adj.format",
#'     bracket.nudge.y = -0.08
#'   ) +
#'   scale_y_continuous(expand = expansion(mult = c(0, 0.1)))
#'
#' @export
stat_pwc <- function(mapping = NULL, data = NULL,
                     method = "wilcox_test", method.args = list(), ref.group = NULL,
                     label = "p.format",
                     y.position = NULL,
                     group.by = NULL, dodge = 0.8,
                     bracket.nudge.y = 0.05, bracket.shorten = 0,
                     bracket.group.by = c("x.var", "legend.var"),
                     step.increase = 0.12, tip.length = 0.03,
                     size = 0.3, label.size = 3.88, family = "", vjust = 0, hjust = 0.5,
                     p.adjust.method = "holm", p.adjust.by = c("group", "panel"),
                     p.adjust.n = NULL,
                     symnum.args = list(), hide.ns = FALSE, remove.bracket = FALSE,
                     p.format.style = "default", p.digits = NULL,
                     p.leading.zero = NULL, p.min.threshold = NULL,
                     p.decimal.mark = NULL,
                     signif.cutoffs = NULL, signif.symbols = NULL,
                     ns.symbol = "ns", use.four.stars = FALSE,
                     position = "identity", na.rm = FALSE, show.legend = NA,
                     inherit.aes = TRUE, parse = FALSE, ...) {
  # Build symnum.args from new parameters
  symnum.args <- build_symnum_args(
    signif.cutoffs = signif.cutoffs,
    signif.symbols = signif.symbols,
    ns.symbol = ns.symbol,
    use.four.stars = use.four.stars,
    symnum.args = symnum.args
  )

  p.adjust.by <- match.arg(p.adjust.by)
  if (missing(parse) & is_plotmath_expression(label)) {
    parse <- TRUE
  }
  if (is.null(group.by)) group.by <- "x.var"
  bracket.group.by <- match.arg(bracket.group.by)

  if (!is.null(ref.group)) {
    if (ref.group %in% c("all", ".all.")) {
      if (missing(step.increase)) step.increase <- 0
      remove.bracket <- TRUE
    }
  }

  # Keep legend variable in memory, we'll be useful to guess ref.group id
  # related question: https://stackoverflow.com/questions/63640543/how-to-access-to-a-legend-group-id-inside-a-ggplot2-extension
  legend.var <- aes_get_group(mapping)
  if (!is.null(legend.var)) mapping$legend.var <- as.name(legend.var)

  ggplot2::layer(
    stat = StatPwc, data = data, mapping = mapping, geom = "pwc",
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      method = method, method.args = method.args, ref.group = ref.group,
      stat.label = label,
      y.position = y.position, group.by = group.by, dodge = dodge,
      bracket.nudge.y = bracket.nudge.y, bracket.shorten = bracket.shorten,
      bracket.group.by = bracket.group.by, step.increase = step.increase,
      tip.length = tip.length, size = size, label.size = label.size,
      remove.bracket = remove.bracket,
      family = family, vjust = vjust, hjust = hjust, na.rm = na.rm,
      p.adjust.method = p.adjust.method, p.adjust.by = p.adjust.by,
      p.adjust.n = p.adjust.n,
      symnum.args = symnum.args,
      hide.ns = hide.ns, parse = parse,
      p.format.style = p.format.style, p.digits = p.digits,
      p.leading.zero = p.leading.zero, p.min.threshold = p.min.threshold,
      p.decimal.mark = p.decimal.mark, ...
    )
  )
}

StatPwc <- ggplot2::ggproto("StatPwc", ggplot2::Stat,
  required_aes = c("x", "y", "group"),
  setup_params = function(data, params) {
    if (length(params$tip.length) == 1) params$tip.length <- rep(params$tip.length, max(length(params$xmin), 1) * 2)
    if (length(params$tip.length) == length(params$xmin)) params$tip.length <- rep(params$tip.length, each = 2)

    # Statistical test methods functions and arguments
    method <- params$method
    method.args <- params$method.args
    if (is.null(method.args$p.adjust.method)) {
      method.args$p.adjust.method <- params$p.adjust.method
    }
    pwc_func <- get_pwc_stat_function(method, method.args)
    params$method <- pwc_func$method
    params$method.args <- pwc_func$method.args
    return(params)
  },
  compute_panel = function(self, data, scales, method, method.args, ref.group,
                           tip.length, stat.label, y.position, step.increase,
                           bracket.nudge.y, bracket.shorten, bracket.group.by,
                           p.adjust.method, p.adjust.by, p.adjust.n = NULL,
                           symnum.args, hide.ns, group.by, dodge, remove.bracket,
                           p.format.style, p.digits, p.leading.zero,
                           p.min.threshold, p.decimal.mark) {
    # Compute the statistical tests
    df <- data %>% mutate(x = as.factor(.data$x))
    is.grouped.plots <- contains_multiple_grouping_vars(df)
    formula <- y ~ x
    grouping.var <- NULL
    comparison.var <- "x"

    if (is.grouped.plots) {
      if (group.by == "legend.var") {
        grouping.var <- "group"
        formula <- y ~ x
        comparison.var <- "x"
      } else {
        grouping.var <- "x"
        formula <- y ~ group
        comparison.var <- "group"
      }
      df <- df %>% rstatix::df_group_by(vars = grouping.var)
    }
    is.comparisons.between.legend.grps <- is.grouped.plots & group.by == "x.var"

    # Comparison against reference group
    ref.group.id <- get_ref_group_id(
      scales = scales, data = df, ref.group = ref.group,
      is.comparisons.between.legend.grps = is.comparisons.between.legend.grps
    )
    if (!is.null(ref.group.id)) {
      method.args <- method.args %>% .add_item(ref.group = ref.group.id)
    }

    method.args <- method.args %>% .add_item(data = df, formula = formula)
    stat.test <- tryCatch(
      do.call(method, method.args),
      error = function(e) {
        # rstatix (>= 0.7.3.9000) tags an absent reference group with the S3
        # condition class "rstatix_missing_ref_group"; detect it by class
        # (locale-independent, robust to message wording), walking the parent
        # chain because the grouped path wraps the condition in dplyr/purrr
        # errors. Fall back to matching the message text for older rstatix and
        # for the other "sparse data" failures (too few levels / observations).
        sparse_error <- rlang::cnd_inherits(e, "rstatix_missing_ref_group") ||
          grepl(
            paste0(
              "not enough|exactly 2 levels|at least 2|same group|fewer than two levels",
              "|must be an existing level|not present in the data",
              # French locale equivalents
              "|insuffisant|niveau existant|pas assez|au moins 2"
            ),
            conditionMessage(e),
            ignore.case = TRUE
          )
        if (!sparse_error) {
          stop(e)
        }
        if (!is.grouped.plots) {
          # Non-grouped panel: when the user supplies an explicit list of
          # comparisons, a single untestable pair (e.g. a group that is entirely
          # NA, or has < 2 observations) makes the batched rstatix call abort the
          # WHOLE panel, dropping the valid comparisons too (#542). Retry each
          # requested comparison on its own and keep the ones that succeed, so
          # only the untestable pair is dropped. This runs ONLY on the error
          # path, so panels whose batched test succeeds are unaffected.
          comparisons <- method.args$comparisons
          if (is.null(comparisons)) {
            return(data.frame())
          }
          run_one_comparison <- function(pair) {
            args.one <- method.args
            args.one$comparisons <- list(pair)
            tryCatch(do.call(method, args.one), error = function(e) NULL)
          }
          per.comparison <- lapply(comparisons, run_one_comparison)
          kept <- !vapply(per.comparison, is.null, logical(1))
          if (!any(kept)) {
            return(data.frame())
          }
          if (any(!kept)) {
            skipped.desc <- vapply(
              comparisons[!kept],
              function(p) paste(as.character(p), collapse = " vs "),
              character(1)
            )
            message(
              "geom_pwc(): skipped ", sum(!kept),
              " untestable comparison(s): ",
              paste(skipped.desc, collapse = ", ")
            )
          }
          survivors <- per.comparison[kept]
          combined <- dplyr::bind_rows(survivors)
          # bind_rows() drops the rstatix attributes/classes that downstream
          # steps (get_description(), add_x_position(), ...) rely on; restore
          # them from the first surviving single-comparison result.
          first.survivor <- survivors[[1]]
          attr(combined, "args") <- attr(first.survivor, "args")
          class(combined) <- class(first.survivor)
          # Each survivor was tested in its OWN rstatix call, so its p.adj is
          # just its raw p (multiplicity correction over a single value = none).
          # Drop it so the downstream adjustment re-computes p.adj across the
          # surviving set, matching what a single test of only those comparisons
          # would report (otherwise survivors would show under-adjusted p.adj /
          # p.adj.signif under the default p.adjust.by = "group").
          combined$p.adj <- NULL
          return(combined)
        }

        # Retry using only grouped subsets with at least two comparison levels.
        df_fallback <- data %>% mutate(x = as.factor(.data$x))
        requires_ref_level <- !is.null(ref.group.id) && !(ref.group.id %in% c("all", ".all."))
        ref_level <- if (requires_ref_level) as.character(ref.group.id) else NA_character_
        ref_level_label <- if (requires_ref_level && !is.null(ref.group)) {
          as.character(ref.group)
        } else {
          ref_level
        }
        grouped_summary <- df_fallback %>%
          dplyr::group_by(.data[[grouping.var]]) %>%
          dplyr::summarise(
            .n_levels = dplyr::n_distinct(.data[[comparison.var]]),
            .has_ref = if (requires_ref_level) {
              ref_level %in% as.character(unique(.data[[comparison.var]]))
            } else {
              TRUE
            },
            .groups = "drop"
          )
        grouped_summary$.is_comparable <- grouped_summary$.n_levels >= 2 & grouped_summary$.has_ref
        if (requires_ref_level) {
          grouped_summary$.skip_reason <- ifelse(
            grouped_summary$.is_comparable,
            NA_character_,
            ifelse(
              !grouped_summary$.has_ref & grouped_summary$.n_levels < 2,
              paste0("missing ref.group='", ref_level_label, "' and <2 comparison levels"),
              ifelse(
                !grouped_summary$.has_ref,
                paste0("missing ref.group='", ref_level_label, "'"),
                "<2 comparison levels"
              )
            )
          )
        } else {
          grouped_summary$.skip_reason <- ifelse(
            grouped_summary$.is_comparable,
            NA_character_,
            "<2 comparison levels"
          )
        }

        comparable <- grouped_summary %>%
          dplyr::filter(.data$.is_comparable)

        skipped_summary <- grouped_summary %>%
          dplyr::filter(!.data$.is_comparable)
        if (nrow(skipped_summary) > 0) {
          skipped_desc <- paste0(
            as.character(skipped_summary[[grouping.var]]),
            " (",
            skipped_summary$.skip_reason,
            ")"
          )
          message(
            "geom_pwc(): skipped ",
            nrow(skipped_summary),
            " grouped subset(s): ",
            paste(skipped_desc, collapse = ", ")
          )
        }

        if (nrow(comparable) == 0) {
          return(data.frame())
        }

        df_fallback <- df_fallback %>%
          dplyr::semi_join(comparable, by = grouping.var) %>%
          rstatix::df_group_by(vars = grouping.var)
        method.args.fallback <- method.args
        method.args.fallback$data <- df_fallback
        do.call(method, method.args.fallback)
      }
    )

    if (nrow(stat.test) == 0) {
      return(data.frame())
    }

    # Add method name
    method.name <- rstatix::get_description(stat.test)
    if (method.name == "") {
      stat.label <- gsub(pattern = "\\{method\\},\\s?", replacement = "", stat.label)
    }
    stat.test$method <- method.name

    # P-value adjustment, formatting and significance
    if (!("p.adj" %in% colnames(stat.test))) {
      # Case of one comparison of two groups
      stat.test <- .pwc_adjust_pvalue(stat.test, p.adjust.method, p.adjust.n)
    }

    # Adjust all the p-values in a given panel
    # no matter the grouping
    if (p.adjust.by == "panel") {
      if ("p" %in% colnames(stat.test)) {
        stat.test <- .pwc_adjust_pvalue(stat.test, p.adjust.method, p.adjust.n)
      } else {
        warning(
          "p-values can't be adjusted by panel for the specified stat method.\n",
          "The result of the method doesn't contain the p column.\n",
          "Note that, tests such as tukey_hsd or games_howell_test handle p-value adjustement ",
          "internally; they only return the p.adj.",
          call. = FALSE
        )
      }
    }

    if (!("p" %in% colnames(stat.test))) {
      # Case when method is tukey or games-howel, etc
      # doesn't return p but p.adj
      stat.test <- stat.test %>%
        tibble::add_column(p = NA, .before = "p.adj")
      stat.label <- gsub(pattern = "p\\.format(?!\\.signif)", replacement = "p.adj.format", stat.label, perl = TRUE)
      stat.label <- gsub(pattern = "p\\.signif", replacement = "p.adj.signif", stat.label)
    }

    sy <- symnum.args
    stat.test <- stat.test %>%
      rstatix::add_x_position(x = "x", group = "group", dodge = dodge) %>%
      rstatix::add_significance(p.col = "p", cutpoints = sy$cutpoints, symbols = sy$symbols) %>%
      rstatix::add_significance(p.col = "p.adj", cutpoints = sy$cutpoints, symbols = sy$symbols)

    # For simple (non-grouped) comparisons, anchor the bracket x positions to
    # the actual x-axis positions. Observations whose y is NA are dropped
    # before the test runs, so a group that is entirely NA disappears from the
    # data; rstatix::add_x_position() then renumbers the surviving groups from
    # 1, shifting the brackets left (#575). Here data$x (and therefore group1/
    # group2) are the mapped discrete x positions, so we can restore the true
    # positions directly. Rows with non-numeric group ids (e.g. ".all."/"all"
    # basemean comparisons) are left as computed by add_x_position().
    if (!is.grouped.plots && !is.null(scales$x) && scales$x$is_discrete()) {
      g1 <- suppressWarnings(as.numeric(as.character(stat.test$group1)))
      g2 <- suppressWarnings(as.numeric(as.character(stat.test$group2)))
      ok <- !is.na(g1) & !is.na(g2)
      stat.test$xmin[ok] <- g1[ok]
      stat.test$xmax[ok] <- g2[ok]
    }

    # Format p-values using unified p-format implementation for all styles
    stat.test <- stat.test %>%
      dplyr::mutate(
        p.format = format_p_value(p,
          style = p.format.style,
          digits = p.digits, leading.zero = p.leading.zero,
          min.threshold = p.min.threshold,
          decimal.mark = p.decimal.mark
        ),
        p.adj.format = format_p_value(p.adj,
          style = p.format.style,
          digits = p.digits, leading.zero = p.leading.zero,
          min.threshold = p.min.threshold,
          decimal.mark = p.decimal.mark
        )
      )

    stat.test <- stat.test %>%
      add_stat_n() %>%
      keep_only_tbl_df_classes() %>%
      add_stat_label(label = stat.label)

    if (!is.null(ref.group)) {
      if (!(ref.group %in% c(".all.", "all"))) {
        # when comparisons is done against reference group
        if (remove.bracket) stat.test$xmin <- stat.test$xmax
      }
    }

    # Hide NS if hide.ns not null
    stat.test <- rstatix::remove_ns(stat.test, col = hide.ns)


    # Grouped bracket colors
    # When data is grouped by legend variable and
    # stat test is computed between x variable groups
    # labels are stacked on y-axis. labels group ids are group (legend var)
    if (is.grouped.plots) {
      if (grouping.var == "group" & "colour" %in% colnames(data)) {
        color.data <- data %>%
          select("group", "colour") %>%
          distinct(.data$group, .keep_all = TRUE)
        stat.test <- stat.test %>% dplyr::left_join(color.data, by = "group")
      }
    }

    # Bracket groups, used for spacing vertically brackets
    bracket.group <- 1
    if (nrow(stat.test) > 1) bracket.group <- seq_len(nrow(stat.test))
    if (is.grouped.plots) {
      if (grouping.var == "x") {
        nb.comparisons.by.group <- stat.test %>%
          rstatix::df_group_by(vars = grouping.var) %>%
          dplyr::summarise(n = dplyr::n())
        bracket.group <- sequence(nb.comparisons.by.group$n)
      } else if (grouping.var == "group" & bracket.group.by == "legend.var") {
        stat.test <- stat.test %>% dplyr::arrange(.data$group1, .data$group2)
      }
    }

    # Parameters for customizing brackets
    group <- 1
    if (nrow(stat.test) > 1) group <- seq_len(nrow(stat.test))
    stat.test <- stat.test %>%
      mutate(
        group = group,
        bracket.group = bracket.group,
        step.increase = step.increase,
        bracket.nudge.y = bracket.nudge.y,
        bracket.shorten = bracket.shorten
      )

    # Bracket x positions
    bracket.shorten <- stat.test$bracket.shorten / 2
    xmin <- as.numeric(stat.test$xmin) + bracket.shorten
    xmax <- as.numeric(stat.test$xmax) - bracket.shorten

    # Bracket y positions
    yrange <- scales$y$range$range
    y.scale.range <- yrange[2] - yrange[1]
    bracket.nudge.y <- stat.test$bracket.nudge.y
    step.increase <- stat.test$step.increase
    if (is.null(y.position)) {
      y.position <- yrange[2] + y.scale.range * bracket.nudge.y + y.scale.range * step.increase * (bracket.group - 1)
    } else if (length(y.position) == 1) {
      y.position <- y.position + y.scale.range * bracket.nudge.y + y.scale.range * step.increase * (bracket.group - 1)
    } else if (length(y.position) >= length(stat.test$group)) {
      y.position <- y.position[stat.test$group]
    }

    if ("tip.length" %in% colnames(stat.test)) {
      tip.length <- rep(stat.test$tip.length, each = 2)
    }
    # Preparing bracket data
    stat.test <- dplyr::bind_rows(stat.test, stat.test, stat.test)
    stat.test$x <- c(xmin, xmin, xmax)
    stat.test$xend <- c(xmin, xmax, xmax)
    stat.test$y <- c(y.position - y.scale.range * tip.length[seq_along(tip.length) %% 2 == 1], y.position, y.position)
    stat.test$yend <- c(y.position, y.position, y.position - y.scale.range * tip.length[seq_along(tip.length) %% 2 == 0])
    stat.test %>% select(-step.increase, -bracket.nudge.y, -bracket.shorten)
  }
)


# Adjust the p-values of a pwc stat.test, optionally against a user-supplied
# number of comparisons `n` (#612). With n = NULL (default) this is exactly
# rstatix::adjust_pvalue(method = method) -- byte-identical to the previous
# behavior. With n set, stats::p.adjust(p, method, n = n) is used so the
# adjustment reflects a custom family size (e.g. when the displayed comparisons
# are a subset of a larger set); `n` must be >= the number of p-values adjusted,
# matching stats::p.adjust()'s own requirement.
.pwc_adjust_pvalue <- function(stat.test, method, n = NULL) {
  if (is.null(n)) {
    return(rstatix::adjust_pvalue(stat.test, method = method))
  }
  if (!is.numeric(n) || length(n) != 1L || is.na(n) || n < 1) {
    stop("`p.adjust.n` must be a single positive number or NULL.", call. = FALSE)
  }
  n.pvalues <- sum(!is.na(stat.test$p))
  if (n < n.pvalues) {
    stop(
      "`p.adjust.n` (", n, ") must be >= the number of p-values being adjusted (",
      n.pvalues, ").", call. = FALSE
    )
  }
  stat.test$p.adj <- stats::p.adjust(stat.test$p, method = method, n = n)
  stat.test
}


#' @rdname geom_pwc
#' @export
geom_pwc <- function(mapping = NULL, data = NULL, stat = "pwc",
                     method = "wilcox_test", method.args = list(), ref.group = NULL,
                     label = "p.format",
                     y.position = NULL, group.by = NULL, dodge = 0.8, stack = FALSE,
                     step.increase = 0.12, tip.length = 0.03,
                     bracket.nudge.y = 0.05, bracket.shorten = 0, bracket.group.by = c("x.var", "legend.var"),
                     size = 0.3, label.size = 3.88, family = "", vjust = 0, hjust = 0.5,
                     p.adjust.method = "holm", p.adjust.by = c("group", "panel"),
                     p.adjust.n = NULL,
                     symnum.args = list(), hide.ns = FALSE, remove.bracket = FALSE,
                     p.format.style = "default", p.digits = NULL,
                     p.leading.zero = NULL, p.min.threshold = NULL,
                     p.decimal.mark = NULL,
                     signif.cutoffs = NULL, signif.symbols = NULL,
                     ns.symbol = "ns", use.four.stars = FALSE,
                     position = "identity", na.rm = FALSE,
                     show.legend = NA, inherit.aes = TRUE, parse = FALSE, ...) {
  # Build symnum.args from new parameters
  symnum.args <- build_symnum_args(
    signif.cutoffs = signif.cutoffs,
    signif.symbols = signif.symbols,
    ns.symbol = ns.symbol,
    use.four.stars = use.four.stars,
    symnum.args = symnum.args
  )

  p.adjust.by <- match.arg(p.adjust.by)
  if (missing(parse) & is_plotmath_expression(label)) {
    parse <- TRUE
  }
  bracket.group.by <- match.arg(bracket.group.by)
  if (is.null(group.by)) group.by <- "x.var"
  if (stack) {
    # for stacked bar plots/line plots
    dodge <- 0
    if (group.by == "x.var") remove.bracket <- TRUE
  }

  if (!is.null(ref.group)) {
    if (ref.group %in% c("all", ".all.")) {
      if (missing(step.increase)) step.increase <- 0
      remove.bracket <- TRUE
    }
  }

  # Keep legend variable in memory, we'll be useful to guess ref.group id
  # related question: https://stackoverflow.com/questions/63640543/how-to-access-to-a-legend-group-id-inside-a-ggplot2-extension
  legend.var <- aes_get_group(mapping)
  if (!is.null(legend.var)) mapping$legend.var <- as.name(legend.var)

  ggplot2::layer(
    stat = stat, geom = GeomPwc, mapping = mapping, data = data,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      method = method, method.args = method.args, ref.group = ref.group,
      stat.label = label,
      y.position = y.position, group.by = group.by, dodge = dodge,
      bracket.nudge.y = bracket.nudge.y,
      bracket.shorten = bracket.shorten, bracket.group.by = bracket.group.by,
      step.increase = step.increase, tip.length = tip.length,
      size = size, label.size = label.size,
      family = family, na.rm = na.rm, hjust = hjust, vjust = vjust,
      p.adjust.method = p.adjust.method, p.adjust.by = p.adjust.by,
      p.adjust.n = p.adjust.n,
      symnum.args = symnum.args,
      hide.ns = hide.ns, remove.bracket = remove.bracket,
      p.format.style = p.format.style, p.digits = p.digits,
      p.leading.zero = p.leading.zero, p.min.threshold = p.min.threshold,
      p.decimal.mark = p.decimal.mark,
      parse = parse,
      ...
    )
  )
}


GeomPwc <- ggplot2::ggproto("GeomPwc", ggplot2::Geom,
  required_aes = c("x", "xend", "y", "yend", "label"),
  default_aes = ggplot2::aes(
    shape = 19, colour = "black", label.size = 3.88, angle = NA, hjust = 0.5,
    vjust = 0, alpha = NA, family = "", fontface = 1, lineheight = 1.2, linetype = 1, size = 0.3, linewidth = NULL,
    legend.var = NA
  ),
  # draw_key = function(...){grid::nullGrob()},
  # for legend:
  draw_key = draw_key_path,
  draw_panel = function(data, panel_params, coord,
                        parse = FALSE,
                        remove.bracket = FALSE) {
    coords <- coord$transform(data, panel_params)
    coord.flip <- inherits(coord, "CoordFlip")
    if (remove.bracket) {
      text_grob <- get_text_grob(data, coords, coord.flip = coord.flip, parse = parse)
      return(text_grob)
    }
    grid::gList(
      get_text_grob(data, coords, coord.flip = coord.flip, parse = parse),
      grid::segmentsGrob(
        coords$x, coords$y,
        default.units = "native",
        coords$xend, coords$yend,
        gp = grid::gpar(
          col = scales::alpha(coords$colour, coords$alpha),
          lty = coords$linetype,
          lwd = (coords$linewidth %||% coords$size) * ggplot2::.pt
        )
      )
    )
  }
)

# Text grob ------------------------------
get_text_grob <- function(data, coords, coord.flip = FALSE, parse = FALSE) {
  lab <- as.character(data$label)
  if (parse) lab <- parse_as_expression(lab)
  label_coords <- get_pwc_label_coords(coords, coord.flip = coord.flip)
  grid::textGrob(
    lab,
    x = label_coords$x,
    y = label_coords$y,
    default.units = "native",
    hjust = coords$hjust, vjust = coords$vjust,
    rot = label_coords$angle,
    gp = grid::gpar(
      col = scales::alpha(coords$colour, coords$alpha),
      fontsize = coords$label.size * ggplot2::.pt,
      fontfamily = coords$family,
      fontface = coords$fontface,
      lineheight = coords$lineheight
    )
  )
}

# Source: https://github.com/tidyverse/ggplot2/issues/2864
parse_as_expression <- function(text) {
  stopifnot(is.character(text))
  out <- vector("expression", length(text))
  for (i in seq_along(text)) {
    expr <- parse(text = text[[i]])
    out[[i]] <- if (length(expr) == 0) NA else expr[[1]]
  }
  out
}

# Compute the coordinates  of pairwise comparison labels
get_pwc_label_coords <- function(coords, coord.flip = FALSE) {
  get_label_x <- function(coords) {
    mean(c(coords$x[1], utils::tail(coords$xend, n = 1)))
  }
  get_label_y <- function(coords) {
    max(c(coords$y, coords$yend)) + 0.01
  }
  get_label_angle <- function(coords, coord.flip = FALSE) {
    label.angle <- coords$angle[1]
    if (coord.flip & is.na(label.angle)) label.angle <- -90
    if (is.na(label.angle)) label.angle <- 0
    label.angle[1]
  }

  if (coord.flip) {
    get_label_x <- function(coords) {
      max(c(coords$x, coords$xend)) + 0.01
    }
    get_label_y <- function(coords) {
      mean(c(coords$y[1], utils::tail(coords$yend, n = 1)))
    }
  }

  if (!("group" %in% colnames(coords))) {
    coords$group <- 1
  }
  data <- NULL
  coords %>%
    rstatix::df_nest_by(vars = c("group", "bracket.group")) %>%
    dplyr::transmute(
      x = unlist(map(data, get_label_x)),
      y = unlist(map(data, get_label_y)),
      angle = unlist(map(data, get_label_angle, coord.flip = coord.flip))
    )
}

# Statistical tests --------------------------------------------------
# Get the pairwise comparison stat function
# returns a list(method, method.args)
get_pwc_stat_function <- function(method, method.args) {
  if (is.character(method)) {
    # replace t.test by t_test, etc
    method <- gsub(pattern = ".", replacement = "_", method, fixed = TRUE)
    if (method == "wilcoxon") {
      method <- "wilcox_test"
    } else if (method == "emmeans") {
      method <- "emmeans_test"
    } else if (method == "games_howell") method <- "games_howell_test"
    allowed.methods <- c(
      "wilcox_test", "wilcoxon", "t_test", "sign_test",
      "dunn_test", "emmeans_test", "emmeans", "tukey_hsd",
      "games_howell_test"
    )
    if (!(method %in% allowed.methods)) {
      stop(
        "Unknown stat method: '", method, "'. ",
        "Allowed method names are: ", paste(allowed.methods, collapse = ","),
        call. = FALSE
      )
    }
    # Ignoring p.adjust.method for tukey_hsd and games_howell
    # because they come with internal adjustment method
    if (method %in% c("tukey_hsd", "games_howell_test")) {
      method.args$p.adjust.method <- NULL
    }
    stat_func <- switch(method,
      wilcox_test = rstatix::wilcox_test,
      t_test = rstatix::t_test,
      sign_test = rstatix::sign_test,
      dunn_test = rstatix::dunn_test,
      emmeans_test = rstatix::emmeans_test,
      tukey_hsd = function(data, formula, ...) {
        rstatix::tukey_hsd(data, formula, ...)
      },
      games_howell_test = rstatix::games_howell_test,
      games_howell = rstatix::games_howell_test
    )
  } else if (is.function(method)) {
    stat_func <- method
  } else {
    stop("The argument method is required.")
  }
  list(
    method = stat_func,
    method.args = method.args
  )
}

# Get ref group rank id for comparison against reference -------------------------------------
# param scales: ggproto scales
# param data: ggproto data
# param ref.group: group name
# Returns 1 (for the first group), 2 for the second group
get_ref_group_id <- function(scales, data, ref.group = NULL, is.comparisons.between.legend.grps = FALSE) {
  if (is.null(ref.group)) {
    return(NULL)
  } else if (ref.group %in% c(".all.", "all")) {
    return(ref.group)
  }
  # Grouped plots
  if (is.comparisons.between.legend.grps) {
    if (is.character(ref.group)) {
      if ("legend.var" %in% colnames(data)) {
        ref.group <- get_group_id(data$legend.var, ref.group)
        if (is.na(ref.group)) stop("The ref.group ('", ref.group, "') doesn't exist.", call. = FALSE)
      } else {
        stop(
          "ref.group should be a numeric value indicating the rank of the ",
          "legend group to be used as the reference. \n",
          "For example, use ref.group = 1 when the first group is the reference; \n",
          "use ref.group = 2 when the second group is the reference and so on.",
          call. = FALSE
        )
      }
    }
  }
  # Basic plot: comparison between x-axis groups
  else if (is.character(ref.group)) {
    ref.group <- scales$x$map(ref.group) %>% as.character()
  }
  as.character(ref.group)
}


# x a factor
# group a level
get_group_id <- function(x, group) {
  ids <- x %>%
    as.factor() %>%
    as.numeric()
  names(ids) <- x
  unname(ids[group]) %>% unique()
}

context("test-geom_pwc")

# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)
df$group <- factor(rep(c("grp1", "grp2"), 30))


# Basic plots -----------------------------------
test_that("geom_pwc works for two independent tests", {
  bxp <- ggboxplot(df, x = "supp", y = "len") +
    geom_pwc(method = "wilcox_test")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = 1.5,
    y = 35.395,
    angle = 0
  )
  expect_equal(stat.test$group, c(1, 1, 1))
  expect_equal(stat.test$bracket.group, c(1, 1, 1))
  expect_equal(as.numeric(stat.test$x), c(1, 1, 2))
  expect_equal(as.numeric(stat.test$xend), c(1, 2, 2))
  expect_equal(stat.test$y, c(34.494, 35.385, 35.385))
  expect_equal(stat.test$yend, c(35.385, 35.385, 34.494))
  expect_equal(label_coords, label_coords_expected)
})

test_that("geom_pwc works for pairwise comparisons", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    geom_pwc(method = "wilcox_test")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(1.5, 2.0, 2.5),
    y = c(35.395, 38.959, 42.523),
    angle = c(0, 0, 0)
  )
  expect_equal(stat.test$group, c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L))
  expect_equal(stat.test$bracket.group, c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L))
  expect_equal(as.numeric(stat.test$x), c(1, 1, 2, 1, 1, 2, 2, 3, 3))
  expect_equal(as.numeric(stat.test$xend), c(1, 1, 2, 2, 3, 3, 2, 3, 3))
  expect_equal(stat.test$y, c(34.494, 38.058, 41.622, 35.385, 38.949, 42.513, 35.385, 38.949, 42.513))
  expect_equal(stat.test$yend, c(35.385, 38.949, 42.513, 35.385, 38.949, 42.513, 34.494, 38.058, 41.622))
  expect_equal(label_coords, label_coords_expected)
})


test_that("geom_pwc works for pairwise comparisons against a reference group", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    geom_pwc(method = "wilcox_test", ref.group = "0.5")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(1.5, 2.0),
    y = c(35.395, 38.959),
    angle = c(0, 0)
  )
  expect_equal(stat.test$group, c(1L, 2L, 1L, 2L, 1L, 2L))
  expect_equal(stat.test$bracket.group, c(1L, 2L, 1L, 2L, 1L, 2L))
  expect_equal(as.numeric(stat.test$x), c(1, 1, 1, 1, 2, 3))
  expect_equal(as.numeric(stat.test$xend), c(1, 1, 2, 3, 2, 3))
  expect_equal(stat.test$y, c(34.494, 38.058, 35.385, 38.949, 35.385, 38.949))
  expect_equal(stat.test$yend, c(35.385, 38.949, 35.385, 38.949, 34.494, 38.058))
  expect_equal(label_coords, label_coords_expected)
})


test_that("geom_pwc works for pairwise comparisons against a reference group when remove.bracket=TRUE", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    geom_pwc(method = "wilcox_test", ref.group = "0.5", remove.bracket = TRUE)
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(2, 3),
    y = c(35.395, 38.959),
    angle = c(0, 0)
  )
  expect_equal(label_coords, label_coords_expected)
})

test_that("geom_pwc works for pairwise comparisons against all. Same y.position for all labels", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    geom_pwc(method = "wilcox_test", ref.group = "all")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(1, 2, 3),
    y = c(35.395, 35.395, 35.395),
    angle = c(0, 0, 0)
  )
  expect_equal(label_coords, label_coords_expected)
})


# Grouped plots -----------------------------------

test_that("Grouped plots: Two groups by x position", {
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp") +
    geom_pwc(aes(group = supp), method = "wilcox_test")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(1, 2, 3),
    y = c(35.395, 35.395, 35.395),
    angle = c(0, 0, 0)
  )
  expect_equal(stat.test$group, c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L))
  expect_equal(stat.test$bracket.group, c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L))
  expect_equal(as.numeric(stat.test$x), c(0.8, 1.8, 2.8, 0.8, 1.8, 2.8, 1.2, 2.2, 3.2))
  expect_equal(as.numeric(stat.test$xend), c(0.8, 1.8, 2.8, 1.2, 2.2, 3.2, 1.2, 2.2, 3.2))
  expect_equal(stat.test$y, c(34.494, 34.494, 34.494, 35.385, 35.385, 35.385, 35.385, 35.385, 35.385))
  expect_equal(stat.test$yend, c(35.385, 35.385, 35.385, 35.385, 35.385, 35.385, 34.494, 34.494, 34.494))
  expect_equal(label_coords, label_coords_expected)
})


test_that("Grouped plots: grouping by legend var and comparing x-axis groups", {
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp") +
    geom_pwc(aes(group = supp, color = supp), group.by = "legend.var")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(1.3, 1.8, 2.3, 1.7, 2.2, 2.7),
    y = c(35.395, 38.959, 42.523, 46.087, 49.651, 53.215),
    angle = c(0, 0, 0, 0, 0, 0)
  )
  expect_equal(stat.test$group, c(1L, 1L, 1L, 2L, 2L, 2L, 1L, 1L, 1L, 2L, 2L, 2L, 1L, 1L, 1L, 2L, 2L, 2L))
  expect_equal(stat.test$bracket.group, c(1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L))
  expect_equal(as.numeric(stat.test$x), c(0.8, 0.8, 1.8, 1.2, 1.2, 2.2, 0.8, 0.8, 1.8, 1.2, 1.2, 2.2, 1.8, 2.8, 2.8, 2.2, 3.2, 3.2))
  expect_equal(as.numeric(stat.test$xend), c(0.8, 0.8, 1.8, 1.2, 1.2, 2.2, 1.8, 2.8, 2.8, 2.2, 3.2, 3.2, 1.8, 2.8, 2.8, 2.2, 3.2, 3.2))
  expect_equal(stat.test$y, c(34.494, 38.058, 41.622, 45.186, 48.75, 52.314, 35.385, 38.949, 42.513, 46.077, 49.641, 53.205, 35.385, 38.949, 42.513, 46.077, 49.641, 53.205))
  expect_equal(stat.test$yend, c(35.385, 38.949, 42.513, 46.077, 49.641, 53.205, 35.385, 38.949, 42.513, 46.077, 49.641, 53.205, 34.494, 38.058, 41.622, 45.186, 48.75, 52.314))
  expect_equal(label_coords, label_coords_expected)
})


test_that("Grouped plots: grouping by legend var and comparing x-axis groups, using dodge=0 and bracket.group.by = 'legend.var'", {
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp") +
    geom_pwc(
      aes(group = supp, color = supp),
      group.by = "legend.var",
      dodge = 0, bracket.group.by = "legend.var"
    )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords_expected <- data.frame(
    x = c(1.5, 1.5, 2, 2, 2.5, 2.5),
    y = c(35.395, 38.959, 42.523, 46.087, 49.651, 53.215),
    angle = c(0, 0, 0, 0, 0, 0)
  )
  expect_equal(stat.test$group, c(1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L))
  expect_equal(stat.test$bracket.group, c(1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L))
  expect_equal(as.numeric(stat.test$x), c(1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3))
  expect_equal(as.numeric(stat.test$xend), c(1, 1, 1, 1, 2, 2, 2, 2, 3, 3, 3, 3, 2, 2, 3, 3, 3, 3))
  expect_equal(stat.test$y, c(34.494, 38.058, 41.622, 45.186, 48.75, 52.314, 35.385, 38.949, 42.513, 46.077, 49.641, 53.205, 35.385, 38.949, 42.513, 46.077, 49.641, 53.205))
  expect_equal(stat.test$yend, c(35.385, 38.949, 42.513, 46.077, 49.641, 53.205, 35.385, 38.949, 42.513, 46.077, 49.641, 53.205, 34.494, 38.058, 41.622, 45.186, 48.75, 52.314))
  expect_equal(label_coords, label_coords_expected)
})


test_that("Grouped plots: 3 groups at x position. All Pairwise comparisons.", {
  bxp <- ggboxplot(df, x = "supp", y = "len", color = "dose") +
    geom_pwc(aes(group = dose))
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords$x <- round(label_coords$x, 2)
  label_coords$y <- round(label_coords$y, 2)
  label_coords_expected <- data.frame(
    x = c(0.87, 1, 1.13, 1.87, 2, 2.13),
    y = c(35.39, 38.96, 42.52, 35.39, 38.96, 42.52),
    angle = c(0, 0, 0, 0, 0, 0)
  )

  stat.test$x <- round(stat.test$x, 2)
  stat.test$xend <- round(stat.test$xend, 2)
  stat.test$y <- round(stat.test$y, 2)
  stat.test$yend <- round(stat.test$yend, 2)

  expect_equal(stat.test$group, c(1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L))
  expect_equal(stat.test$bracket.group, c(1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L, 1L, 2L, 3L))
  expect_equal(as.numeric(stat.test$x), c(0.73, 0.73, 1, 1.73, 1.73, 2, 0.73, 0.73, 1, 1.73, 1.73, 2, 1, 1.27, 1.27, 2, 2.27, 2.27))
  expect_equal(as.numeric(stat.test$xend), c(0.73, 0.73, 1, 1.73, 1.73, 2, 1, 1.27, 1.27, 2, 2.27, 2.27, 1, 1.27, 1.27, 2, 2.27, 2.27))
  expect_equal(stat.test$y, c(34.49, 38.06, 41.62, 34.49, 38.06, 41.62, 35.38, 38.95, 42.51, 35.38, 38.95, 42.51, 35.38, 38.95, 42.51, 35.38, 38.95, 42.51))
  expect_equal(stat.test$yend, c(35.38, 38.95, 42.51, 35.38, 38.95, 42.51, 35.38, 38.95, 42.51, 35.38, 38.95, 42.51, 34.49, 38.06, 41.62, 34.49, 38.06, 41.62))
  expect_equal(label_coords, label_coords_expected)
})


test_that("Grouped plots: 3 groups at x position. Pairwise comparisons against a ref group.", {
  bxp <- ggboxplot(df, x = "supp", y = "len", color = "dose") +
    geom_pwc(aes(group = dose), ref.group = 1)
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords$x <- round(label_coords$x, 2)
  label_coords$y <- round(label_coords$y, 2)
  label_coords_expected <- data.frame(
    x = c(0.87, 1, 1.87, 2),
    y = c(35.39, 38.96, 35.39, 38.96),
    angle = c(0, 0, 0, 0)
  )

  stat.test$x <- round(stat.test$x, 2)
  stat.test$xend <- round(stat.test$xend, 2)
  stat.test$y <- round(stat.test$y, 2)
  stat.test$yend <- round(stat.test$yend, 2)

  expect_equal(stat.test$group, c(1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L, 1L, 2L, 3L, 4L))
  expect_equal(stat.test$bracket.group, c(1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L))
  expect_equal(as.numeric(stat.test$x), c(0.73, 0.73, 1.73, 1.73, 0.73, 0.73, 1.73, 1.73, 1, 1.27, 2, 2.27))
  expect_equal(as.numeric(stat.test$xend), c(0.73, 0.73, 1.73, 1.73, 1, 1.27, 2, 2.27, 1, 1.27, 2, 2.27))
  expect_equal(stat.test$y, c(34.49, 38.06, 34.49, 38.06, 35.38, 38.95, 35.38, 38.95, 35.38, 38.95, 35.38, 38.95))
  expect_equal(stat.test$yend, c(35.38, 38.95, 35.38, 38.95, 35.38, 38.95, 35.38, 38.95, 34.49, 38.06, 34.49, 38.06))
  expect_equal(label_coords, label_coords_expected)
})

test_that("Grouped plots: 3 groups at x position. Pairwise comparisons against a ref group. Check label coords when remove.bracket=TRUE.", {
  bxp <- ggboxplot(df, x = "supp", y = "len", color = "dose") +
    geom_pwc(aes(group = dose), ref.group = 1, remove.bracket = TRUE)
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords$x <- round(label_coords$x, 2)
  label_coords$y <- round(label_coords$y, 2)
  label_coords_expected <- data.frame(
    x = c(1, 1.27, 2, 2.27),
    y = c(35.39, 38.96, 35.39, 38.96),
    angle = c(0, 0, 0, 0)
  )
  expect_equal(label_coords, label_coords_expected)
})


test_that("Grouped plots: test that geom_pwc() works with different number of groups at each x pos.", {
  # https://github.com/kassambara/ggpubr/issues/326
  demo_data <- data.frame(
    stringsAsFactors = FALSE,
    Study = c(
      "A", "A", "A", "A", "A", "A",
      "A", "A", "A", "A", "B", "B", "B", "B", "B", "B", "B", "B",
      "B", "B", "C", "C", "C", "C", "C", "C", "C", "C", "C",
      "C", "C", "C", "C", "C", "C", "D", "D", "D", "D", "D",
      "D", "D", "D", "D", "D", "D", "D", "D", "D", "D"
    ),
    Studytype = c(
      "X", "X", "X", "X", "X", "X",
      "X", "X", "X", "X", "X", "X", "X", "X", "X", "X", "X", "X",
      "X", "X", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y",
      "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y",
      "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y"
    ),
    Values = c(
      4469L, 4797L, 5101L, 5397L,
      4542L, 2780L, 4326L, 3396L, 3657L, 3199L, 9221L, 10176L,
      9277L, 10500L, 9707L, 7406L, 7756L, 7601L, 7586L, 7353L,
      1811L, 1485L, 3003L, 1629L, 2495L, 4207L, 4265L, 3629L,
      4157L, 3495L, 2075L, 2112L, 2973L, 3086L, 2943L, 5664L, 6690L,
      3538L, 5741L, 7880L, 5848L, 6390L, 6569L, 6114L, 6520L,
      7389L, 6843L, 7611L, 6621L, 7340L
    ),
    Group = as.factor(c(
      "CTR",
      "CTR", "CTR", "CTR", "CTR", "Dis1", "Dis1", "Dis1",
      "Dis1", "Dis1", "CTR", "CTR", "CTR", "CTR",
      "CTR", "Dis1", "Dis1", "Dis1", "Dis1", "Dis1",
      "CTR", "CTR", "CTR", "CTR", "CTR", "Dis2", "Dis2",
      "Dis2", "Dis2", "Dis2", "Dis3", "Dis3",
      "Dis3", "Dis3", "Dis3", "CTR", "CTR", "CTR", "CTR",
      "CTR", "Dis2", "Dis2", "Dis2", "Dis2", "Dis2",
      "Dis3", "Dis3", "Dis3", "Dis3", "Dis3"
    ))
  )
  bxp <- ggboxplot(demo_data, x = "Study", y = "Values", fill = "Group") +
    geom_pwc(aes(group = Group), dodge = 0.8, label = "p", ref.group = 1)
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  label_coords <- as.data.frame(get_pwc_label_coords(stat.test))
  label_coords$x <- round(label_coords$x, 2)
  label_coords$y <- round(label_coords$y, 2)
  label_coords_expected <- data.frame(
    x = c(1, 2, 2.87, 3, 3.87, 4),
    y = c(10950.76, 10950.76, 10950.76, 12032.56, 10950.76, 12032.56),
    angle = c(0, 0, 0, 0, 0, 0)
  )

  stat.test$x <- round(stat.test$x, 2)
  stat.test$xend <- round(stat.test$xend, 2)
  stat.test$y <- round(stat.test$y, 2)
  stat.test$yend <- round(stat.test$yend, 2)

  expect_equal(stat.test$group, c(1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L))
  expect_equal(stat.test$bracket.group, c(1L, 1L, 1L, 2L, 1L, 2L, 1L, 1L, 1L, 2L, 1L, 2L, 1L, 1L, 1L, 2L, 1L, 2L))
  expect_equal(as.numeric(stat.test$x), c(0.8, 1.8, 2.73, 2.73, 3.73, 3.73, 0.8, 1.8, 2.73, 2.73, 3.73, 3.73, 1.2, 2.2, 3, 3.27, 4, 4.27))
  expect_equal(as.numeric(stat.test$xend), c(0.8, 1.8, 2.73, 2.73, 3.73, 3.73, 1.2, 2.2, 3, 3.27, 4, 4.27, 1.2, 2.2, 3, 3.27, 4, 4.27))
  expect_equal(stat.test$y, c(10680.3, 10680.3, 10680.3, 11762.1, 10680.3, 11762.1, 10950.75, 10950.75, 10950.75, 12032.55, 10950.75, 12032.55, 10950.75, 10950.75, 10950.75, 12032.55, 10950.75, 12032.55))
  expect_equal(stat.test$yend, c(10950.75, 10950.75, 10950.75, 12032.55, 10950.75, 12032.55, 10950.75, 10950.75, 10950.75, 12032.55, 10950.75, 12032.55, 10680.3, 10680.3, 10680.3, 11762.1, 10680.3, 11762.1))
  expect_equal(label_coords, label_coords_expected)
})

test_that("geom_pwc skips non-comparable grouped subsets and keeps valid ones", {
  set.seed(100)
  dat <- data.frame(
    x = factor(c(rep("A", 8), rep("B", 8), rep("C", 8))),
    y = rnorm(24),
    grp = factor(
      c(rep("M", 8), rep(c("M", "F"), each = 4), rep("F", 8)),
      levels = c("M", "F")
    )
  )

  p <- ggboxplot(dat, x = "x", y = "y", color = "grp") +
    geom_pwc(aes(group = grp), method = "t_test")

  expect_no_warning(
    expect_message(
      b <- ggplot2::ggplot_build(p),
      "geom_pwc\\(\\): skipped .*<2 comparison levels"
    )
  )
  expect_gte(nrow(b$data[[2]]), 1)
})

test_that("geom_pwc returns empty layer when no grouped subset is comparable", {
  set.seed(101)
  dat <- data.frame(
    x = factor(rep(c("A", "B", "C"), each = 8)),
    y = rnorm(24),
    grp = factor(
      c(rep("M", 8), rep("F", 8), rep("M", 8)),
      levels = c("M", "F")
    )
  )

  p <- ggboxplot(dat, x = "x", y = "y", color = "grp") +
    geom_pwc(aes(group = grp), method = "t_test")

  expect_no_warning(
    expect_message(
      b <- ggplot2::ggplot_build(p),
      "geom_pwc\\(\\): skipped 3 grouped subset\\(s\\)"
    )
  )
  expect_equal(nrow(b$data[[2]]), 0)
})

test_that("geom_pwc skips grouped subsets missing ref.group and keeps valid ones", {
  set.seed(102)
  dat <- data.frame(
    x = factor(
      c(rep("A", 8), rep("B", 8), rep("B", 8), rep("C", 8)),
      levels = c("A", "B", "C")
    ),
    y = rnorm(32),
    grp = factor(
      c(rep("M", 8), rep("M", 8), rep("F", 8), rep("F", 8)),
      levels = c("M", "F")
    )
  )

  # group.by = "legend.var" compares x levels within each legend group.
  # grp == "F" has no "A", so it should be skipped when ref.group = "A".
  p <- ggboxplot(dat, x = "x", y = "y", color = "grp") +
    geom_pwc(
      aes(group = grp),
      group.by = "legend.var",
      method = "t_test",
      ref.group = "A"
    )

  expect_no_warning(
    expect_message(
      b <- ggplot2::ggplot_build(p),
      "geom_pwc\\(\\): skipped .*missing ref\\.group='A'"
    )
  )
  expect_gte(nrow(b$data[[2]]), 1)
})

# Per-comparison survival with explicit comparisons (#542) ----------------
test_that("geom_pwc keeps valid explicit comparisons when one pair is untestable (#542)", {
  set.seed(1)
  # Group D is entirely NA and B has a single finite value, so the B-D
  # comparison cannot be tested; A-C and C-E remain valid and must survive.
  dat <- data.frame(
    Cond = factor(rep(c("A", "B", "C", "D", "E"), each = 3)),
    Value = c(runif(3, 10, 12), c(NA, 11, NA), runif(3, 8, 13),
              c(NA, NA, NA), runif(3, 10, 15))
  )
  p <- ggboxplot(dat, x = "Cond", y = "Value") +
    geom_pwc(
      method = "t_test",
      method.args = list(comparisons = list(c(1, 3), c(3, 5), c(2, 4)))
    )
  expect_message(
    b <- ggplot2::ggplot_build(p),
    "geom_pwc\\(\\): skipped 1 untestable comparison\\(s\\): 2 vs 4"
  )
  stat.test <- b$data[[2]]
  # only the two valid comparisons survive (A-C at x 1-3, C-E at x 3-5)
  surviving <- unique(stat.test[, c("xmin", "xmax", "p", "p.adj")])
  expect_equal(nrow(surviving), 2)
  expect_true(all(paste(surviving$xmin, surviving$xmax) %in% c("1 3", "3 5")))
  expect_false(any(paste(surviving$xmin, surviving$xmax) == "2 4"))

  # The surviving comparisons must be multiplicity-corrected TOGETHER, exactly as
  # if only those two comparisons had been requested (not left with the raw,
  # unadjusted p as p.adj). Otherwise p.adj / p.adj.signif would be wrong (#542).
  clean <- ggboxplot(dat, x = "Cond", y = "Value") +
    geom_pwc(
      method = "t_test",
      method.args = list(comparisons = list(c(1, 3), c(3, 5)))
    )
  clean.test <- suppressMessages(ggplot2::ggplot_build(clean))$data[[2]]
  key <- function(d) paste(d$xmin, d$xmax)
  ord.s <- order(key(surviving))
  ord.c <- order(key(unique(clean.test[, c("xmin", "xmax", "p", "p.adj")])))
  clean.u <- unique(clean.test[, c("xmin", "xmax", "p", "p.adj")])
  expect_equal(surviving$p[ord.s], clean.u$p[ord.c])
  expect_equal(surviving$p.adj[ord.s], clean.u$p.adj[ord.c])
})

test_that("geom_pwc drops only the untestable pair per facet, keeping valid ones (#542 faceted)", {
  set.seed(1)
  # Reporter's scenario: two facets; in Sample2 group D is all-NA and B has a
  # single value, so B-D is untestable there but A-C / C-E are valid. Sample1 is
  # fully testable. Each facet must keep its own valid comparisons.
  dataset <- data.frame(
    Sample = rep(c("Sample1", "Sample2"), each = 15),
    Cond = factor(rep(rep(c("A", "B", "C", "D", "E"), each = 3), 2)),
    Value = c(runif(3, 10, 12), runif(3, 11, 14), runif(3, 8, 13),
              runif(3, 7, 10), runif(3, 10, 15),
              runif(3, 10, 12), c(NA, 11, NA), runif(3, 8, 13),
              c(NA, NA, NA), runif(3, 10, 15))
  )
  p <- ggplot2::ggplot(dataset, ggplot2::aes(Cond, Value)) +
    ggplot2::geom_jitter(width = 0.2) +
    ggplot2::facet_wrap(~Sample) +
    geom_pwc(
      method = "t_test",
      method.args = list(comparisons = list(c(1, 3), c(3, 5), c(2, 4)))
    )
  expect_message(
    b <- ggplot2::ggplot_build(p),
    "geom_pwc\\(\\): skipped 1 untestable comparison\\(s\\): 2 vs 4"
  )
  stat.test <- b$data[[2]]
  by.panel <- split(stat.test, stat.test$PANEL)
  pair.set <- function(d) sort(unique(paste(d$xmin, d$xmax)))
  # Sample1 (PANEL 1): all three comparisons; Sample2 (PANEL 2): 2 vs 4 dropped.
  expect_equal(pair.set(by.panel[["1"]]), c("1 3", "2 4", "3 5"))
  expect_equal(pair.set(by.panel[["2"]]), c("1 3", "3 5"))
})

test_that("geom_pwc explicit comparisons are unchanged when all pairs are testable (#542 no-regression)", {
  # Same call but on data with no untestable pair: no skip message, and all
  # three requested comparisons are drawn (byte-identical to previous behavior).
  df2 <- ToothGrowth
  df2$dose <- as.factor(df2$dose)
  p <- ggboxplot(df2, x = "dose", y = "len") +
    geom_pwc(
      method = "t_test",
      method.args = list(comparisons = list(c(1, 2), c(2, 3), c(1, 3)))
    )
  expect_no_message(
    b <- ggplot2::ggplot_build(p),
    message = "geom_pwc\\(\\): skipped"
  )
  stat.test <- b$data[[2]]
  expect_equal(nrow(unique(stat.test[, c("xmin", "xmax")])), 3)
})

# All-NA group placement (#575) -----------------------------------
test_that("geom_pwc places brackets over the correct groups when a group is all-NA (#575)", {
  # Group "a" is entirely NA; its rows are dropped before the stat runs, so the
  # only comparison is b vs c. The bracket must sit over b (x=2) and c (x=3),
  # not shift left to a-b (x=1,2). See #575.
  test <- data.frame(
    x = rep(c("a", "b", "c"), each = 4),
    y = c(NA, NA, NA, NA, 5:12)
  )
  p <- ggplot(test, aes(x, y)) +
    ggplot2::geom_point() +
    geom_pwc()
  b <- ggplot2::ggplot_build(p)
  st <- b$data[[which(vapply(
    b$data, function(d) all(c("xmin", "xmax") %in% colnames(d)), logical(1)
  ))[1]]]
  st <- unique(st[, c("group1", "group2", "xmin", "xmax")])
  expect_equal(nrow(st), 1L)
  expect_equal(as.character(st$group1), "2")
  expect_equal(as.character(st$group2), "3")
  expect_equal(as.numeric(st$xmin), 2)
  expect_equal(as.numeric(st$xmax), 3)
})

test_that("geom_pwc all-NA fix leaves the fully-populated case unchanged (#575)", {
  # No NA groups: every pairwise bracket keeps its natural position.
  tn <- data.frame(x = rep(c("a", "b", "c"), each = 4), y = 1:12)
  p <- ggplot(tn, aes(x, y)) + ggplot2::geom_point() + geom_pwc()
  b <- ggplot2::ggplot_build(p)
  st <- b$data[[which(vapply(
    b$data, function(d) all(c("xmin", "xmax") %in% colnames(d)), logical(1)
  ))[1]]]
  st <- unique(st[, c("xmin", "xmax")])
  st$xmin <- as.numeric(st$xmin)
  st$xmax <- as.numeric(st$xmax)
  st <- st[order(st$xmin, st$xmax), ]
  expect_equal(st$xmin, c(1, 1, 2))
  expect_equal(st$xmax, c(2, 3, 3))
})

context("test-geom-pwc-p-adjust-n")

# geom_pwc() / stat_pwc() gain p.adjust.n: an optional number of comparisons for
# the p-value adjustment, passed as `n` to stats::p.adjust (#612). Default NULL
# keeps the standard behavior (adjust by the number of computed p-values), so
# existing output is byte-identical.

df <- ToothGrowth
df$dose <- as.factor(df$dose)

.panel_padj <- function(layer) {
  p <- ggboxplot(df, "dose", "len") + layer
  b <- ggplot2::ggplot_build(p)
  for (d in b$data) {
    if ("p.adj" %in% names(d)) return(sort(unique(d$p.adj)))
  }
  stop("no p.adj found")
}

.raw_p <- sort(rstatix::wilcox_test(df, len ~ dose)$p)

test_that("default p.adjust.n = NULL is unchanged (byte-identical to holm on the computed p-values)", {
  got <- .panel_padj(geom_pwc(method = "wilcox_test", p.adjust.by = "panel"))
  expect_equal(got, sort(stats::p.adjust(.raw_p, method = "holm")))
})

test_that(".pwc_adjust_pvalue with n = NULL equals rstatix::adjust_pvalue", {
  st <- data.frame(group1 = c("a", "a", "b"), group2 = c("b", "c", "c"),
                   p = c(0.01, 0.04, 0.2))
  expect_equal(
    ggpubr:::.pwc_adjust_pvalue(st, "holm", NULL),
    rstatix::adjust_pvalue(st, method = "holm")
  )
})

test_that("p.adjust.n adjusts against the supplied family size", {
  got <- .panel_padj(geom_pwc(method = "wilcox_test", p.adjust.by = "panel", p.adjust.n = 10))
  expect_equal(got, sort(stats::p.adjust(.raw_p, method = "holm", n = 10)))
  # bonferroni is easy to reason about: p.adj = min(1, p * n)
  got_bonf <- .panel_padj(
    geom_pwc(method = "wilcox_test", p.adjust.by = "panel",
             p.adjust.method = "bonferroni", p.adjust.n = 100)
  )
  expect_equal(got_bonf, sort(pmin(1, .raw_p * 100)))
})

test_that("p.adjust.n below the number of p-values is rejected", {
  # 3 dose comparisons; n = 2 is invalid. The stat compute fails.
  expect_error(
    ggpubr:::.pwc_adjust_pvalue(
      data.frame(p = c(0.01, 0.02, 0.03)), "holm", 2
    ),
    "must be >= the number of p-values"
  )
})

test_that("p.adjust.n validates its input type", {
  st <- data.frame(p = c(0.01, 0.02))
  expect_error(ggpubr:::.pwc_adjust_pvalue(st, "holm", c(3, 4)),
               "single positive number")
  expect_error(ggpubr:::.pwc_adjust_pvalue(st, "holm", -1),
               "single positive number")
})

# Data preparation
# :::::::::::::::::::::::::::::::::::::::
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Add a random grouping variable
df$group <- factor(rep(c("grp1", "grp2"), 30))


test_that("ggadjust_pvalue works for geom_pwc() pairwise comparison using facet", {
  p <- ggboxplot(df, x = "supp", y = "len", facet.by = "dose") +
    geom_pwc(method = "t_test")
  # Adjust all p-values together after
  stat_test <- ggadjust_pvalue(
    p,
    p.adjust.method = "bonferroni",
    label = "{p.adj.format}{p.adj.signif}",
    output = "stat_test"
  )
  stat_test <- stat_test %>%
    dplyr::select(PANEL, x, y, group, group1, group2, label) %>%
    mutate(x = as.numeric(x), label = as.character(label))
  expected <- tibble::tribble(
    ~PANEL, ~x, ~y, ~group, ~group1, ~group2, ~label,
    "1", 1, 34.494, 1, "1", "2", "0.0191*",
    "1", 1, 35.385, 1, "1", "2", "0.0191*",
    "1", 2, 35.385, 1, "1", "2", "0.0191*",
    "2", 1, 34.494, 1, "1", "2", "0.0031**",
    "2", 1, 35.385, 1, "1", "2", "0.0031**",
    "2", 2, 35.385, 1, "1", "2", "0.0031**",
    "3", 1, 34.494, 1, "1", "2", "1ns",
    "3", 1, 35.385, 1, "1", "2", "1ns",
    "3", 2, 35.385, 1, "1", "2", "1ns"
  ) %>%
    dplyr::mutate(PANEL = as.factor(PANEL)) %>%
    as.data.frame(stringAsFactor = FALSE)
  expect_equal(stat_test, expected)
})

#' @include utilities.R ggpar.R stat_chull.R stat_conf_ellipse.R stat_stars.R stat_cor.R
NULL
#' Scatter plot
#' @description Create a scatter plot.
#' @inheritParams ggboxplot
#' @inheritParams facet
#' @inheritParams ggpar
#' @param x x variables for drawing.
#' @param y y variables for drawing.
#' @param color,fill point colors.
#' @param shape point shape. See \code{\link{show_point_shapes}}.
#' @param point logical value. If TRUE, show points.
#' @param rug logical value. If TRUE, add marginal rug.
#' @param add allowed values are one of "none", "reg.line" (for adding linear
#'   regression line) or "loess" (for adding local regression fitting).
#' @param add.params parameters (color, size, linetype) for the argument 'add';
#'   e.g.: add.params = list(color = "red").
#' @param conf.int logical value. If TRUE, adds confidence interval.
#' @param conf.int.level Level controlling confidence region. Default is 95\%.
#'   Used only when add != "none" and conf.int = TRUE.
#' @param fullrange should the fit span the full range of the plot, or just the
#'   data. Used only when add != "none".
#' @param ellipse logical value. If TRUE, draws ellipses around points.
#' @param ellipse.level the size of the concentration ellipse in normal
#'   probability.
#' @param ellipse.type Character specifying frame type. Possible values are
#'  \code{"convex"}, \code{"confidence"} or types supported by
#'  \code{\link[ggplot2]{stat_ellipse}()} including one of \code{c("t", "norm",
#'  "euclid")} for plotting concentration ellipses.
#'
#'  \itemize{ \item \code{"convex"}: plot convex hull of a set o points. \item
#'  \code{"confidence"}: plot confidence ellipses around group mean points as
#'  \code{FactoMineR::coord.ellipse()}. \item \code{"t"}:
#'  assumes a multivariate t-distribution. \item \code{"norm"}: assumes a
#'  multivariate normal distribution. \item \code{"euclid"}: draws a circle with
#'  the radius equal to level, representing the euclidean distance from the
#'  center. This ellipse probably won't appear circular unless
#'  \code{\link[ggplot2]{coord_fixed}()} is applied.}
#' @param ellipse.alpha Alpha for ellipse specifying the transparency level of
#'   fill color. Use alpha = 0 for no fill color.
#' @param ellipse.border.remove logical value. If TRUE, remove ellipse border lines.
#' @param mean.point logical value. If TRUE, group mean points are added to the
#'   plot.
#' @param mean.point.size numeric value specifying the size of mean points.
#' @param star.plot logical value. If TRUE, a star plot is generated.
#' @param star.plot.lty,star.plot.lwd line type and line width (size) for star
#'   plot, respectively.
#' @param label the name of the column containing point labels. Can be also a
#'   character vector with length = nrow(data).
#' @param font.label a vector of length 3 indicating respectively the size
#'   (e.g.: 14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and
#'   the color (e.g.: "red") of point labels. For example \emph{font.label =
#'   c(14, "bold", "red")}. To specify only the size and the style, use
#'   font.label = c(14, "plain").
#' @param font.family character vector specifying font family.
#' @param label.select character vector specifying some labels to show.
#' @param repel a logical value, whether to use ggrepel to avoid overplotting
#'   text labels or not.
#' @param label.rectangle logical value. If TRUE, add rectangle underneath the
#'   text, making it easier to read.
#' @param parse If \code{TRUE}, the labels will be parsed into expressions and
#'   displayed as described in \code{?plotmath}.
#' @param cor.coef logical value. If TRUE, correlation coefficient with the
#'   p-value will be added to the plot.
#' @param cor.coeff.args a list of arguments to pass to the function
#'   \code{\link{stat_cor}} for customizing the displayed correlation
#'   coefficients. For example: \code{cor.coeff.args = list(method = "pearson",
#'   label.x.npc = "right", label.y.npc = "top")}.
#' @param cor.method method for computing correlation coefficient. Allowed
#'   values are one of "pearson", "kendall", or "spearman".
#' @param cor.coef.coord numeric vector, of length 2, specifying the x and y
#'   coordinates of the correlation coefficient. Default values are NULL.
#' @param cor.coef.size correlation coefficient text font size.
#' @param ggp a ggplot. If not NULL, points are added to an existing plot.
#' @param show.legend.text logical. Should text be included in the legends? NA,
#'   the default, includes if any aesthetics are mapped. FALSE never includes,
#'   and TRUE always includes.
#' @param ... other arguments to be passed to \code{\link[ggplot2]{geom_point}}
#'   and \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{stat_cor}}, \code{\link{stat_stars}}, \code{\link{stat_conf_ellipse}} and \code{\link{ggpar}}.
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#' head(df[, c("wt", "mpg", "cyl")], 3)
#'
#' # Basic plot
#' # +++++++++++++++++++++++++++
#' ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "black", shape = 21, size = 3, # Points color, shape and size
#'   add = "reg.line", # Add regressin line
#'   add.params = list(color = "blue", fill = "lightgray"), # Customize reg. line
#'   conf.int = TRUE, # Add confidence interval
#'   cor.coef = TRUE, # Add correlation coefficient. see ?stat_cor
#'   cor.coeff.args = list(method = "pearson", label.x = 3, label.sep = "\n")
#' )
#'
#' # loess method: local regression fitting
#' ggscatter(df,
#'   x = "wt", y = "mpg",
#'   add = "loess", conf.int = TRUE
#' )
#'
#'
#' # Control point size by continuous variable values ("qsec")
#' ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "#00AFBB", size = "qsec"
#' )
#'
#'
#' # Change colors
#' # +++++++++++++++++++++++++++
#' # Use custom color palette
#' # Add marginal rug
#' ggscatter(df,
#'   x = "wt", y = "mpg", color = "cyl",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#'
#' # Add group ellipses and mean points
#' # Add stars
#' # +++++++++++++++++++
#' ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "cyl", shape = "cyl",
#'   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   ellipse = TRUE, mean.point = TRUE,
#'   star.plot = TRUE
#' )
#'
#'
#' # Textual annotation
#' # +++++++++++++++++
#' df$name <- rownames(df)
#' ggscatter(df,
#'   x = "wt", y = "mpg",
#'   color = "cyl", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   label = "name", repel = TRUE
#' )
#'
#' @export
ggscatter <- function(data, x, y, combine = FALSE, merge = FALSE,
                      color = "black", fill = "lightgray", palette = NULL,
                      shape = 19, size = 2, point = TRUE, rug = FALSE,
                      title = NULL, xlab = NULL, ylab = NULL,
                      facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                      add = c("none", "reg.line", "loess"), add.params = list(),
                      conf.int = FALSE, conf.int.level = 0.95, fullrange = FALSE,
                      ellipse = FALSE, ellipse.level = 0.95,
                      ellipse.type = "norm", ellipse.alpha = 0.1,
                      ellipse.border.remove = FALSE,
                      mean.point = FALSE, mean.point.size = ifelse(is.numeric(size), 2 * size, size),
                      star.plot = FALSE, star.plot.lty = 1, star.plot.lwd = NULL,
                      label = NULL, font.label = c(12, "plain"), font.family = "",
                      label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                      parse = FALSE,
                      cor.coef = FALSE, cor.coeff.args = list(), cor.method = "pearson", cor.coef.coord = c(NULL, NULL), cor.coef.size = 4,
                      ggp = NULL, show.legend.text = NA,
                      ggtheme = theme_pubr(),
                      ...) {
  add <- match.arg(add)
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    shape = shape, size = size, point = point, rug = rug,
    add = add, add.params = add.params,
    conf.int = conf.int, conf.int.level = conf.int.level, fullrange = fullrange,
    ellipse = ellipse, ellipse.level = ellipse.level,
    ellipse.type = ellipse.type, ellipse.alpha = ellipse.alpha,
    ellipse.border.remove = ellipse.border.remove,
    mean.point = mean.point, mean.point.size = mean.point.size,
    star.plot = star.plot, star.plot.lty = star.plot.lty, star.plot.lwd = star.plot.lwd,
    label = label, font.label = font.label, font.family = font.family,
    label.select = label.select, repel = repel, label.rectangle = label.rectangle,
    parse = parse,
    cor.coef = cor.coef, cor.coeff.args = cor.coeff.args, cor.method = cor.method,
    cor.coef.coord = cor.coef.coord, cor.coef.size = cor.coef.size,
    ggp = ggp, show.legend.text = show.legend.text, ggtheme = ggtheme, ...
  )

  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  font.label <- .parse_font(font.label) %>% .compact()
  font.label$color <- ifelse(is.null(font.label$color), color, font.label$color)
  .opts$font.label <- font.label

  .opts$fun <- ggscatter_core
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)
  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


ggscatter_core <- function(data, x, y,
                           color = "black", fill = "lightgray", palette = NULL,
                           shape = 19, size = 2, point = TRUE, rug = FALSE,
                           title = NULL, xlab = NULL, ylab = NULL,
                           add = c("none", "reg.line", "loess"), add.params = list(),
                           conf.int = FALSE, conf.int.level = 0.95, fullrange = FALSE,
                           ellipse = FALSE, ellipse.level = 0.95,
                           ellipse.type = "norm", ellipse.alpha = 0.1,
                           ellipse.border.remove = FALSE,
                           mean.point = FALSE, mean.point.size = ifelse(is.numeric(size), 2 * size, size),
                           star.plot = FALSE, star.plot.lty = 1, star.plot.lwd = NULL,
                           label = NULL, font.label = c(12, "plain"), font.family = "",
                           label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                           parse = FALSE,
                           cor.coef = FALSE, cor.coeff.args = list(), cor.method = "pearson", cor.coef.coord = c(NULL, NULL), cor.coef.size = 4,
                           ggp = NULL, show.legend.text = NA,
                           ggtheme = theme_classic(),
                           ...) {
  add <- match.arg(add)
  add.params <- .check_add.params(add, add.params, error.plot = "", data, color, fill, ...)

  if (length(label) > 1) {
    if (length(label) != nrow(data)) {
      stop(
        "The argument label should be a column name or a vector of length = nrow(data). ",
        "It seems that length(label) != nrow(data)"
      )
    } else {
      data$label.xx <- label
    }
    label <- "label.xx"
  }
  # label font
  font.label <- .parse_font(font.label)
  font.label$size <- ifelse(is.null(font.label$size), 12, font.label$size)
  font.label$color <- ifelse(is.null(font.label$color), color, font.label$color)
  font.label$face <- ifelse(is.null(font.label$face), "plain", font.label$face)

  if (is.null(ggp)) {
    p <- ggplot(data, create_aes(list(x = x, y = y)))
  } else {
    p <- ggp
  }

  if (point) {
    p <- p +
      .geom_exec(geom_point,
        data = data, x = x, y = y,
        color = color, fill = fill, size = size,
        shape = shape, ...
      )
  }

  # Adjust shape when ngroups > 6, to avoid ggplot warnings
  if (shape %in% colnames(data)) {
    ngroups <- length(levels(data[[shape]]))
    if (ngroups > 6) p <- p + scale_shape_manual(values = seq_len(ngroups), labels = levels(data[[shape]]))
  }

  # Add marginal rug
  # +++++++++++
  if (rug) {
    p <- p + .geom_exec(geom_rug,
      data = data,
      color = color, linewidth = size / 2
    )
  }

  # Add reg line or loess
  # ++++++++++++
  if (add %in% c("reg.line", "loess")) {
    add <- ifelse(add == "reg.line", stats::lm, stats::loess)
    if (is.null(add.params$linetype)) add.params$linetype <- "solid"
    if (conf.int == FALSE) add.params$fill <- "lightgray"

    # Use linewidth instead of size for geom_smooth (ggplot2 3.4.0+ compatibility)
    .args <- .geom_exec(NULL,
      data = data,
      se = conf.int, level = conf.int.level,
      color = add.params$color, fill = add.params$fill,
      linetype = add.params$linetype, linewidth = add.params$size,
      fullrange = fullrange
    )

    mapping <- .args$mapping
    option <- .args$option
    option[["method"]] <- add
    option[["formula"]] <- y ~ x
    option[["mapping"]] <- create_aes(mapping)
    p <- p + do.call(geom_smooth, option)
  }


  # Add ellipses
  # +++++++++++
  if (ellipse) {
    grp <- intersect(unique(c(color, fill, shape)), colnames(data))[1]
    # NO grouping variable
    if (is.na(grp)) {
      grp <- factor(rep(1, nrow(data)))
      grp_name <- "group"
      data$group <- grp
    }
    # Case of grouping variable
    else {
      grp_name <- grp
      data[[grp_name]] <- as.factor(data[[grp_name]])
    }

    if (ellipse.type == "convex") {
      p <- p + .convex_ellipse(data, x, y, grp_name, color, fill, ellipse.alpha,
        ellipse.border.remove = ellipse.border.remove
      )
    } else if (ellipse.type == "confidence") {
      p <- p + .confidence_ellipse(data, x, y, grp_name, color, fill,
        alpha = ellipse.alpha, level = ellipse.level,
        ellipse.border.remove = ellipse.border.remove
      )
    } else if (ellipse.type %in% c("t", "norm", "euclid")) {
      p <- p + .stat_ellipse(data, x, y, grp_name,
        color = color, fill = fill,
        alpha = ellipse.alpha, type = ellipse.type, level = ellipse.level,
        ellipse.border.remove = ellipse.border.remove
      )
    }
  }
  # /ellipse

  # Add mean points
  # +++++++++
  if (mean.point) {
    p <- p + .geom_exec(stat_mean,
      data = data,
      color = color, shape = shape, fill = fill,
      size = mean.point.size
    )
  }

  # Star plots
  # ++++++++++++
  if (star.plot) {
    p <- p + .geom_exec(stat_stars,
      data = data,
      color = color, linetype = star.plot.lty, linewidth = star.plot.lwd
    )
  }

  # / star plots

  # Add textual annotation
  # ++++++
  alpha <- 1
  if (!is.null(list(...)$alpha)) alpha <- list(...)$alpha
  if (!is.null(label)) {
    lab_data <- data
    # Select some labels to show
    if (!is.null(label.select)) {
      lab_data <- subset(lab_data, lab_data[[label]] %in% label.select,
        drop = FALSE
      )
    }

    if (repel) {
      max.overlaps <- getOption("ggrepel.max.overlaps", default = Inf)
      ggfunc <- ggrepel::geom_text_repel
      if (label.rectangle) ggfunc <- ggrepel::geom_label_repel
      p <- p + .geom_exec(ggfunc,
        data = lab_data, x = x, y = y,
        label = label, fontface = font.label$face,
        parse = parse,
        size = font.label$size / 3, color = font.label$color,
        alpha = alpha, family = font.family,
        box.padding = unit(0.35, "lines"),
        point.padding = unit(0.3, "lines"),
        force = 1, show.legend = show.legend.text, seed = 123,
        max.overlaps = max.overlaps
      )
    } else {
      ggfunc <- geom_text
      vjust <- -0.7
      if (label.rectangle) {
        ggfunc <- geom_label
        vjust <- -0.4
      }
      p <- p + .geom_exec(ggfunc,
        data = lab_data, x = x, y = y,
        label = label, fontface = font.label$face, family = font.family,
        parse = parse,
        size = font.label$size / 3, color = font.label$color,
        vjust = vjust, alpha = alpha, show.legend = show.legend.text
      )
    }
  }

  # Add correlation coefficient
  if (cor.coef) {
    if (!missing(cor.method)) {
      cor.coeff.args$method <- cor.method
    }
    if (!missing(cor.coef.size)) {
      cor.coeff.args$size <- cor.coef.size
    }
    if (!missing(cor.coef.coord)) {
      cor.coeff.args$label.x <- cor.coef.coord[1]
      cor.coeff.args$label.y <- cor.coef.coord[2]
    }
    p <- p + do.call(stat_cor, cor.coeff.args)
  }

  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )
  if (font.family != "") {
    p <- p + theme(text = element_text(family = font.family))
  }
  p
}


# Add convex ellipse
# data a data frame
# x,y: x and y variables
# grp_name: grp variable
.convex_ellipse <- function(data, x, y, grp_name, color = "black", fill = "lightgray", alpha = 0.1,
                            ellipse.border.remove = FALSE) {
  grp_levels <- levels(data[[grp_name]])
  if (length(grp_levels) == 1) {
    .geom_exec(
      geomfunc = stat_chull, data = data,
      color = color, fill = fill, alpha = alpha,
      geom = "polygon"
    )
  } else {
    if (ellipse.border.remove) {
      color <- NULL
    } else {
      color <- grp_name
    }
    .geom_exec(
      geomfunc = stat_chull, data = data,
      color = color, fill = grp_name, alpha = alpha,
      geom = "polygon"
    )
  }
}

# Confidence ellipse
.confidence_ellipse <- function(data, x, y, grp_name, color = "black", fill = "lightgray",
                                alpha = 0.1, level = 0.95, ellipse.border.remove = FALSE) {
  grp_levels <- levels(data[[grp_name]])
  if (length(grp_levels) == 1) {
    mapping <- create_aes(list(x = x, y = y))
    stat_conf_ellipse(
      mapping = mapping, data = data,
      color = color, fill = fill, alpha = alpha,
      level = level, geom = "polygon"
    )
  } else {
    mapping <- create_aes(list(x = x, y = y, colour = grp_name, fill = grp_name))
    if (ellipse.border.remove) mapping <- create_aes(list(x = x, y = y, colour = NULL, fill = grp_name))
    stat_conf_ellipse(
      mapping = mapping, data = data,
      level = level, alpha = alpha,
      geom = "polygon"
    )
  }
}


# Add ggplot2 stat ellipse
.stat_ellipse <- function(data, x, y, grp_name, color = "black", fill = "lightgray",
                          alpha = 0.1, type = "norm", level = 0.95, ellipse.border.remove = FALSE) {
  grp_levels <- levels(data[[grp_name]])
  if (length(grp_levels) == 1) {
    mapping <- create_aes(list(x = x, y = y))
    ggplot2::stat_ellipse(
      mapping = mapping, data = data,
      level = level, type = type,
      colour = color, fill = fill, alpha = alpha,
      geom = "polygon"
    )
  } else {
    mapping <- create_aes(list(x = x, y = y, colour = grp_name, group = grp_name, fill = grp_name))
    if (ellipse.border.remove) mapping <- create_aes(list(x = x, y = y, group = grp_name, fill = grp_name))
    ggplot2::stat_ellipse(
      mapping = mapping, data = data,
      level = level, type = type, alpha = alpha,
      geom = "polygon"
    )
  }
}

# Regression test for #652: legend.direction was ignored for legend = "bottom"/"top"
# (it only took effect via ggplot2 defaults for "left"/"right").

.df <- transform(mtcars, cyl = as.factor(cyl))

test_that("legend.direction is applied with legend='bottom' (#652)", {
  p <- ggscatter(.df, x = "wt", y = "mpg", color = "cyl",
                 legend = "bottom", legend.direction = "vertical")
  expect_equal(p$theme$legend.direction, "vertical")
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

test_that("legend.direction still works with legend='right' (#652)", {
  p <- ggscatter(.df, x = "wt", y = "mpg", color = "cyl",
                 legend = "right", legend.direction = "horizontal")
  expect_equal(p$theme$legend.direction, "horizontal")
})

test_that("omitting legend.direction leaves it unset (no regression, #652)", {
  p <- ggscatter(.df, x = "wt", y = "mpg", color = "cyl", legend = "bottom")
  expect_null(p$theme$legend.direction)
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

context("ggscatter")

test_that("ggscatter works when there is spaces in variable names", {
  data("mtcars")
  df <- mtcars[, c("wt", "mpg")]
  colnames(df) <- c("Dimension 1", "Dimension 2")
  p <- ggscatter(df, x = "Dimension 1", y = "Dimension 2")
  p_build <- ggplot2::ggplot_build(p)
  df_build <- p_build$data[[1]]
  expect_equal(df$`Dimension 1`, df_build$x)
})


test_that("ggscatter can handle non-standard column names when ggpubr.parse_aes global option is set to FALSE (#229)", {
  data("mtcars")
  df <- mtcars[, c("wt", "mpg")]
  colnames(df) <- c("A-A", "B-B")

  old_options <- options(ggpubr.parse_aes = FALSE)
  p <- ggscatter(df, x = "A-A", y = "B-B")
  options(old_options)
  p_build <- ggplot2::ggplot_build(p)
  df_build <- p_build$data[[1]]
  expect_equal(df$`A-A`, df_build$x)
})

#' @include utilities.R
NULL
#' Annotate Arranged Figure
#'
#' @description Annotate figures including: i) ggplots, ii) arranged ggplots from
#'  \code{\link{ggarrange}()}, \code{\link[gridExtra:arrangeGrob]{grid.arrange}()} and
#'  \code{\link[cowplot]{plot_grid}()}.
#' @param p (arranged) ggplots.
#' @param top,bottom,left,right optional string, or grob.
#' @param fig.lab figure label (e.g.: "Figure 1").
#' @param fig.lab.pos position of the figure label, can be one of "top.left",
#'  "top", "top.right", "bottom.left", "bottom", "bottom.right". Default is
#'  "top.left".
#' @param fig.lab.size optional size of the figure label.
#' @param fig.lab.face optional font face of the figure label. Allowed values
#'  include: "plain", "bold", "italic", "bold.italic".
#' @param column.titles,row.titles optional character vector giving one title per
#'  column / per row of the arranged figure, adding titles above each column and
#'  to the left of each row (useful for publication grids, see
#'  \code{\link{ggarrange}()}). Can also be a list of grobs (e.g. built with
#'  \code{\link{text_grob}()}) for full control over styling; a character vector
#'  is rendered in bold using the current theme text settings. The titles are
#'  evenly spaced and therefore assume equal-sized columns / rows (the
#'  \code{ggarrange()} default), with one entry per column / row. \code{row.titles}
#'  are rotated 90 degrees.
#' @author Laszlo Erdey \email{erdey.laszlo@@econ.unideb.hu}
#' @seealso \code{\link{ggarrange}()}
#' @examples
#' data("ToothGrowth")
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#'
#' # Create some plots
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::
#' # Box plot
#' bxp <- ggboxplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' # Dot plot
#' dp <- ggdotplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' # Density plot
#' dens <- ggdensity(df, x = "len", fill = "dose", palette = "jco")
#'
#' # Arrange and annotate
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::
#' figure <- ggarrange(bxp, dp, dens, ncol = 2, nrow = 2)
#' annotate_figure(figure,
#'   top = text_grob("Visualizing Tooth Growth", color = "red", face = "bold", size = 14),
#'   bottom = text_grob("Data source: \n ToothGrowth data set",
#'     color = "blue",
#'     hjust = 1, x = 1, face = "italic", size = 10
#'   ),
#'   left = text_grob("Figure arranged using ggpubr", color = "green", rot = 90),
#'   right = text_grob(bquote("Superscript: (" * kg ~ NH[3] ~ ha^-1 ~ yr^-1 * ")"), rot = 90),
#'   fig.lab = "Figure 1", fig.lab.face = "bold"
#' )
#'
#' # Add a title to each column and each row of an arranged grid
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::
#' grid <- ggarrange(bxp, dp, dens, bxp, ncol = 2, nrow = 2)
#' annotate_figure(grid,
#'   column.titles = c("Column 1", "Column 2"),
#'   row.titles = c("Row 1", "Row 2")
#' )
#'
#' @export
annotate_figure <- function(p,
                            top = NULL, bottom = NULL, left = NULL, right = NULL,
                            fig.lab = NULL,
                            fig.lab.pos = c(
                              "top.left", "top", "top.right",
                              "bottom.left", "bottom", "bottom.right"
                            ),
                            fig.lab.size, fig.lab.face,
                            column.titles = NULL, row.titles = NULL) {
  fig.lab.pos <- match.arg(fig.lab.pos)
  annot.args <- list(top = top, bottom = bottom, left = left, right = right) %>%
    .compact()

  # Anchor the figure label with a length-independent horizontal justification.
  # cowplot::draw_figure_label() uses hjust = -0.1 / 1.1 for the left/right
  # positions, a width-proportional offset, so a longer label is pushed further
  # from the corner and captions no longer align across figures (#185). Use a
  # fixed hjust (0 for left, 1 for right; the centre positions already use 0, as
  # cowplot did) with a fixed x anchor; vertical placement matches cowplot.
  fig.lab.args <- switch(fig.lab.pos,
    "top.left"     = list(x = 0.01, y = 1, hjust = 0, vjust = 1.1),
    "top"          = list(x = 0.50, y = 1, hjust = 0, vjust = 1.1),
    "top.right"    = list(x = 0.99, y = 1, hjust = 1, vjust = 1.1),
    "bottom.left"  = list(x = 0.01, y = 0, hjust = 0, vjust = -0.1),
    "bottom"       = list(x = 0.50, y = 0, hjust = 0, vjust = -0.1),
    "bottom.right" = list(x = 0.99, y = 0, hjust = 1, vjust = -0.1)
  )
  lab.args <- c(list(label = fig.lab), fig.lab.args)
  # Preserve the previous default label size/face, which cowplot::draw_figure_label()
  # took from the current theme text (not draw_plot_label's bold/16 default) (#185).
  lab.args$size <- if (!missing(fig.lab.size)) fig.lab.size else ggplot2::theme_get()$text$size
  lab.args$fontface <- if (!missing(fig.lab.face)) fig.lab.face else ggplot2::theme_get()$text$face


  # Per-column / per-row titles (#573). Wrapped closest to the plots (inside any
  # figure-wide top/left labels) using space-reserving strips so they do not
  # overlap the plot content. Evenly spaced -> assumes equal column/row sizes.
  has.col.titles <- !is.null(column.titles) && length(column.titles) > 0
  has.row.titles <- !is.null(row.titles) && length(row.titles) > 0
  if (has.col.titles || has.row.titles) {
    col.grob <- if (has.col.titles) .titles_grob(column.titles, rot = 0) else NULL
    row.grob <- if (has.row.titles) .titles_grob(row.titles, rot = 90) else NULL
    p <- gridExtra::arrangeGrob(p, top = col.grob, left = row.grob) %>%
      as_ggplot()
  }

  if (!.is_empty(annot.args)) {
    p <- gridExtra::arrangeGrob(p, top = top, bottom = bottom, left = left, right = right) %>%
      as_ggplot()
  }

  if (!is.null(fig.lab)) {
    p <- cowplot::ggdraw(p) + do.call(cowplot::draw_plot_label, lab.args)
  }

  p
}

# Build an evenly-spaced strip of per-column (rot = 0) or per-row (rot = 90)
# titles for annotate_figure() (#573). `titles` is a character vector (rendered
# in bold via text_grob()), a single grob, or a list of grobs.
.titles_grob <- function(titles, rot = 0) {
  if (is.character(titles)) {
    grobs <- lapply(titles, function(x) text_grob(x, face = "bold", rot = rot))
  } else if (grid::is.grob(titles)) {
    grobs <- list(titles)
  } else if (is.list(titles)) {
    grobs <- titles
  } else {
    stop(
      "`column.titles`/`row.titles` must be a character vector or a list of grobs.",
      call. = FALSE
    )
  }
  n <- length(grobs)
  if (rot == 0) {
    # per-column titles: a single row of n evenly spaced cells
    gridExtra::arrangeGrob(grobs = grobs, nrow = 1, ncol = n)
  } else {
    # per-row titles: a single column of n evenly spaced cells
    gridExtra::arrangeGrob(grobs = grobs, ncol = 1, nrow = n)
  }
}

# Regression tests for #185: annotate_figure(fig.lab=) positioned the label with
# cowplot's width-proportional hjust (-0.1 / 1.1), so a longer label was pushed
# further from the corner and figure captions no longer aligned across figures.
# The label now uses a length-independent hjust (0 / 1) and a fixed x anchor.

.fig <- ggarrange(ggboxplot(ToothGrowth, "dose", "len"), ncol = 1)

.lab_layer <- function(p) {
  # the figure label is the last (GeomText) layer added by draw_plot_label
  i <- utils::tail(which(vapply(p$layers,
    function(l) inherits(l$geom, "GeomText"), logical(1))), 1)
  p$layers[[i]]
}

test_that("fig.lab hjust is length-independent, not cowplot's -0.1 (#185)", {
  short <- .lab_layer(annotate_figure(.fig, fig.lab = "A", fig.lab.pos = "top.left"))
  long  <- .lab_layer(annotate_figure(.fig, fig.lab = "A very long figure label", fig.lab.pos = "top.left"))
  # top.left -> hjust 0 (was -0.1)
  expect_equal(short$aes_params$hjust, 0)
  # hjust and x anchor identical regardless of label length -> captions align
  expect_equal(short$aes_params$hjust, long$aes_params$hjust)
  expect_equal(short$data$x, long$data$x)
})

test_that("fig.lab positions map to fixed hjust 0/0.5?/1 (#185)", {
  tl <- .lab_layer(annotate_figure(.fig, fig.lab = "L", fig.lab.pos = "top.left"))
  tr <- .lab_layer(annotate_figure(.fig, fig.lab = "R", fig.lab.pos = "top.right"))
  expect_equal(tl$aes_params$hjust, 0)
  expect_equal(tr$aes_params$hjust, 1)
})

test_that("annotate_figure(fig.lab=) still renders (#185)", {
  expect_no_error(print(annotate_figure(.fig, fig.lab = "Fig 1", fig.lab.face = "bold")))
  expect_no_error(print(annotate_figure(.fig, fig.lab = "Fig 1", fig.lab.pos = "bottom.right")))
})

test_that("default fig.lab size/face are unchanged (no regression, #185)", {
  d <- .lab_layer(annotate_figure(.fig, fig.lab = "Fig 1"))
  # cowplot::draw_figure_label defaulted to the theme text size/face (11pt/plain),
  # NOT draw_plot_label's 16/bold -> preserve that default.
  expect_equal(d$aes_params$size, ggplot2::theme_get()$text$size / ggplot2::.pt)
  expect_equal(d$aes_params$fontface, ggplot2::theme_get()$text$face)
  # explicit size/face still honored
  e <- .lab_layer(annotate_figure(.fig, fig.lab = "X", fig.lab.size = 20, fig.lab.face = "italic"))
  expect_equal(e$aes_params$fontface, "italic")
})

context("test-annotate-figure-titles")

# annotate_figure() gains column.titles / row.titles to add a title above each
# column and to the left of each row of an arranged grid (#573). Opt-in: default
# NULL leaves the figure unchanged.

.fig <- ggarrange(
  ggboxplot(ToothGrowth, "dose", "len"),
  ggboxplot(ToothGrowth, "dose", "len"),
  ggboxplot(ToothGrowth, "dose", "len"),
  ggboxplot(ToothGrowth, "dose", "len"),
  ncol = 2, nrow = 2
)

# --- .titles_grob helper ----------------------------------------------------

test_that(".titles_grob builds one cell per title with the right labels", {
  g <- ggpubr:::.titles_grob(c("A", "B", "C"), rot = 0)
  expect_true(grid::is.grob(g))
  labels <- vapply(g$grobs, function(x) x$label, character(1))
  expect_equal(labels, c("A", "B", "C"))
})

test_that(".titles_grob rotates row titles by 90 degrees", {
  g <- ggpubr:::.titles_grob(c("R1", "R2"), rot = 90)
  rots <- vapply(g$grobs, function(x) x$rot, numeric(1))
  expect_equal(rots, c(90, 90))
})

test_that(".titles_grob accepts a list of grobs (custom styling passthrough)", {
  custom <- list(
    text_grob("X", color = "red", size = 14),
    text_grob("Y", color = "blue", size = 14)
  )
  g <- ggpubr:::.titles_grob(custom, rot = 0)
  labels <- vapply(g$grobs, function(x) x$label, character(1))
  expect_equal(labels, c("X", "Y"))
  expect_equal(g$grobs[[1]]$gp$col, "red")
})

test_that(".titles_grob rejects invalid input", {
  expect_error(ggpubr:::.titles_grob(1:3), "character vector or a list of grobs")
})

# --- annotate_figure() integration ------------------------------------------

test_that("column.titles / row.titles render without error", {
  expect_no_error(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      annotate_figure(.fig,
        column.titles = c("Col 1", "Col 2"),
        row.titles = c("Row 1", "Row 2")
      )
    ))
  )
  # combined with a figure-wide top label
  expect_no_error(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      annotate_figure(.fig,
        top = text_grob("Whole figure", face = "bold"),
        column.titles = c("Col 1", "Col 2")
      )
    ))
  )
})

test_that("column.titles accepts a list of grobs", {
  expect_no_error(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      annotate_figure(.fig,
        column.titles = list(
          text_grob("A", color = "red"),
          text_grob("B", color = "blue")
        )
      )
    ))
  )
})

test_that("default (no titles) is unchanged: passing NULL equals omitting", {
  omitted <- annotate_figure(.fig, top = text_grob("T"))
  explicit_null <- annotate_figure(.fig, top = text_grob("T"),
    column.titles = NULL, row.titles = NULL)
  # same object structure (the titles block is skipped when both are NULL)
  expect_equal(
    ggplot2::ggplot_build(omitted)$data,
    ggplot2::ggplot_build(explicit_null)$data
  )
})

test_that("positional fig.lab still binds correctly (no signature-order break)", {
  # column.titles / row.titles are appended at the END of the signature, so the
  # 6th positional argument is still fig.lab (not column.titles). A positional
  # fig.lab call must still draw the figure label.
  lab_layer <- function(p) {
    i <- utils::tail(which(vapply(p$layers,
      function(l) inherits(l$geom, "GeomText"), logical(1))), 1)
    p$layers[[i]]
  }
  p <- annotate_figure(.fig, NULL, NULL, NULL, NULL, "Figure 1")
  expect_equal(as.character(lab_layer(p)$data$text), "Figure 1")
})

test_that("empty title vectors are handled gracefully (no crash)", {
  expect_no_error(
    ggplot2::ggplot_gtable(ggplot2::ggplot_build(
      annotate_figure(.fig, column.titles = character(0), row.titles = character(0))
    ))
  )
  # an empty vector behaves like NULL (no titles added)
  base <- ggplot2::ggplot_build(annotate_figure(.fig))$data
  empty <- ggplot2::ggplot_build(
    annotate_figure(.fig, column.titles = character(0)))$data
  expect_equal(base, empty)
})

#' @include utilities.R ggpar.R
NULL
#' MA-plot from means and log fold changes
#' @description Make MA-plot which is a scatter plot of log2 fold changes (M, on
#'  the y-axis) versus the average expression signal (A, on the x-axis). \code{M
#'  = log2(x/y)} and \code{A = (log2(x) + log2(y))/2 = log2(xy)*1/2}, where x
#'  and y are respectively the mean of the two groups being compared.
#' @inheritParams ggboxplot
#' @inheritParams ggpar
#' @param data an object of class DESeqResults, get_diff, DE_Results, matrix or
#'  data frame containing the columns baseMean (or baseMeanLog2),
#'  log2FoldChange, and padj. Rows are genes.
#'
#'  Two possible formats are accepted for the input data: \itemize{ \item 1/
#'  \code{baseMean | log2FoldChange | padj}. This is a typical output from
#'  DESeq2 pipeline. Here, we'll use log2(baseMean) as the x-axis variable.
#'  \item 2/ \code{baseMeanLog2 | log2FoldChange | padj}. Here, baseMeanLog2 is
#'  assumed to be the mean of logged values; so we'll use it as the x-axis
#'  variable without any transformation. This is the real A in MA plot. In other
#'  words, it is the average of two log-scales values: \code{A = (log2(x) +
#'  log2(y))/2 = log2(xy)*1/2} }
#'
#'  Terminology:
#'
#'  \itemize{ \item baseMean: the mean expression of genes in the two groups.
#'  \item log2FoldChange: the log2 fold changes of group 2 compared to group 1
#'  \item padj: the adjusted p-value of the used statistical test. }
#' @param fdr Accepted false discovery rate for considering genes as
#'  differentially expressed.
#' @param fc the fold change threshold. Only genes with a fold change >= fc and
#'  padj <= fdr are considered as significantly differentially expressed.
#' @param genenames a character vector of length nrow(data) specifying gene names
#'  corresponding to each row. Used for point labels.
#' @param detection_call a numeric vector with length = nrow(data), specifying if
#'  the genes is expressed (value = 1) or not (value = 0). For example
#'  detection_call = c(1, 1, 0, 1, 0, 1). Default is NULL. If detection_call
#'  column is available in data, it will be used.
#' @param size points size.
#' @param alpha numeric value between 0 and 1 specifying point alpha for
#'  controlling transparency. For example, use alpha = 0.5.
#' @param seed Random seed passed to \code{set.seed}. if
#'  \code{NA}, set.seed will not be called. Default is 42 for reproducibility.
#' @param font.label a vector of length 3 indicating respectively the size (e.g.:
#'  14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and the
#'  color (e.g.: "red") of point labels. For example \emph{font.label = c(14,
#'  "bold", "red")}.
#' @param label.rectangle logical value. If TRUE, add rectangle underneath the
#'  text, making it easier to read.
#' @param top the number of top genes to be shown on the plot. Use top = 0 to
#'  hide to gene labels.
#' @param select.top.method methods to be used for selecting top genes. Allowed
#'  values include "padj" and "fc" for selecting by adjusted p values or fold
#'  changes, respectively.
#' @param label.select character vector specifying some labels to show.
#' @param facet.by character vector, of length 1 or 2, specifying grouping
#'   variables for faceting the plot into multiple panels (one MA plot per group).
#'   The variable(s) must be columns of \code{data} (supplied as a data frame). The
#'   top genes are selected \emph{per panel}; point colors use the same
#'   significance thresholds in every panel. Default is NULL (no faceting), in
#'   which case the output is unchanged.
#' @param line.color color of the horizontal threshold lines (the central line
#'   at 0 and the two fold-change cutoff lines). Default is "black".
#' @param ggtheme function, ggplot2 theme name. Default value is
#'   \code{theme_classic()}. Set \code{ggtheme = NULL} to skip applying a ggpubr
#'   theme, so the plot keeps ggplot2 default theme or the theme set globally via
#'   \code{theme_set()}.
#' @param ... other arguments to be passed to \code{\link{ggpar}}.
#' @return a ggplot.
#' @examples
#' data(diff_express)
#'
#' # Default plot
#' ggmaplot(diff_express,
#'   main = expression("Group 1" %->% "Group 2"),
#'   fdr = 0.05, fc = 2, size = 0.4,
#'   palette = c("#B31B21", "#1465AC", "darkgray"),
#'   genenames = as.vector(diff_express$name),
#'   legend = "top", top = 20,
#'   font.label = c("bold", 11),
#'   font.legend = "bold",
#'   font.main = "bold",
#'   ggtheme = ggplot2::theme_minimal()
#' )
#'
#' # Add rectangle around labels
#' ggmaplot(diff_express,
#'   main = expression("Group 1" %->% "Group 2"),
#'   fdr = 0.05, fc = 2, size = 0.4,
#'   palette = c("#B31B21", "#1465AC", "darkgray"),
#'   genenames = as.vector(diff_express$name),
#'   legend = "top", top = 20,
#'   font.label = c("bold", 11), label.rectangle = TRUE,
#'   font.legend = "bold",
#'   font.main = "bold",
#'   ggtheme = ggplot2::theme_minimal()
#' )
#'
#' # Select specific genes to show
#' # set top = 0, then specify genes using label.select argument
#' ggmaplot(diff_express,
#'   main = expression("Group 1" %->% "Group 2"),
#'   fdr = 0.05, fc = 2, size = 0.4,
#'   genenames = as.vector(diff_express$name),
#'   ggtheme = ggplot2::theme_minimal(),
#'   top = 0, label.select = c("BUB1", "CD83")
#' )
#'
#' @export
ggmaplot <- function(data, fdr = 0.05, fc = 1.5, genenames = NULL,
                     detection_call = NULL, size = NULL, alpha = 1,
                     seed = 42,
                     font.label = c(12, "plain", "black"), label.rectangle = FALSE,
                     palette = c("#B31B21", "#1465AC", "darkgray"),
                     top = 15, select.top.method = c("padj", "fc"),
                     label.select = NULL, facet.by = NULL,
                     main = NULL, xlab = "Log2 mean expression", ylab = "Log2 fold change",
                     line.color = "black",
                     ggtheme = theme_classic(), ...) {
  if (!base::inherits(data, c("matrix", "data.frame", "DataFrame", "DE_Results", "DESeqResults"))) {
    stop("data must be an object of class matrix, data.frame, DataFrame, DE_Results or DESeqResults")
  }
  if (!is.null(detection_call)) {
    if (nrow(data) != length(detection_call)) {
      stop("detection_call must be a numeric vector of length = nrow(data)")
    }
  } else if ("detection_call" %in% colnames(data)) {
    detection_call <- as.vector(data$detection_call)
  } else {
    detection_call <- rep(1, nrow(data))
  }

  # If basemean logged, we'll leave it as is, otherwise log2 transform
  is.basemean.logged <- "baseMeanLog2" %in% colnames(data)
  if (is.basemean.logged) {
    data$baseMean <- data$baseMeanLog2
  } else if ("baseMean" %in% colnames(data)) {
    data$baseMean <- log2(data$baseMean + 1)
  }

  # Check data format
  ss <- base::setdiff(c("baseMean", "log2FoldChange", "padj"), colnames(data))
  if (length(ss) > 0) {
    stop(
      "The colnames of data must contain: ",
      paste(ss, collapse = ", ")
    )
  }

  if (is.null(genenames)) {
    genenames <- rownames(data)
  } else if (length(genenames) != nrow(data)) {
    stop("genenames should be of length nrow(data).")
  }

  # #498: an optional facet.by builds a faceted MA plot. Capture the faceting
  # column(s) now - before the data frame is rebuilt below - so they can be
  # carried into the plotted layers; the top genes are then selected per facet.
  if (!is.null(facet.by)) {
    facet.by <- as.character(facet.by)
    miss <- base::setdiff(facet.by, colnames(data))
    if (length(miss) > 0) {
      stop("facet.by column(s) not found in data: ", paste(miss, collapse = ", "),
           ". Provide 'data' as a data frame containing the faceting variable(s).")
    }
    facet_data <- as.data.frame(data)[, facet.by, drop = FALSE]
  } else {
    facet_data <- NULL
  }

  sig <- rep(3, nrow(data))
  sig[which(data$padj <= fdr & data$log2FoldChange < 0 & abs(data$log2FoldChange) >= log2(fc) & detection_call == 1)] <- 2
  sig[which(data$padj <= fdr & data$log2FoldChange > 0 & abs(data$log2FoldChange) >= log2(fc) & detection_call == 1)] <- 1
  data <- data.frame(
    name = genenames, mean = data$baseMean, lfc = data$log2FoldChange,
    padj = data$padj, sig = sig
  )
  # #498: carry the faceting column(s) into the plotted data (only when faceting,
  # so an external facet() without facet.by still errors honestly instead of
  # silently using globally-selected top genes). Row order is preserved above.
  if (!is.null(facet_data)) data <- cbind(data, facet_data)

  # Change level labels
  . <- NULL
  data$sig <- as.factor(data$sig)
  .lev <- .levels(data$sig) %>% as.numeric()
  palette <- palette[.lev]
  if (is.null(facet.by)) {
    new.levels <- c(
      paste0("Up: ", sum(sig == 1)),
      paste0("Down: ", sum(sig == 2)),
      "NS"
    ) %>% .[.lev]
  } else {
    # #498: the Up/Down counts are global and cannot be shown per-panel in one
    # shared legend, so use plain labels when faceting to avoid misleading counts.
    new.levels <- c("Up", "Down", "NS") %>% .[.lev]
  }

  data$sig <- factor(data$sig, labels = new.levels)


  # Ordering for selecting top gene
  select.top.method <- match.arg(select.top.method)
  if (select.top.method == "padj") {
    data <- data[order(data$padj), ]
  } else if (select.top.method == "fc") data <- data[order(abs(data$lfc), decreasing = TRUE), ]
  # select data for top genes
  complete_data <- stats::na.omit(data)
  labs_data <- subset(complete_data, padj <= fdr & name != "" & abs(lfc) >= log2(fc))
  if (is.null(facet.by)) {
    labs_data <- utils::head(labs_data, top)
  } else {
    # #498: select the top genes PER facet. `data` is already ordered by the
    # select.top.method key, so taking the first `top` rows within each facet
    # group keeps that ordering.
    labs_data <- labs_data %>%
      dplyr::group_by(dplyr::across(dplyr::all_of(facet.by))) %>%
      dplyr::slice_head(n = top) %>%
      dplyr::ungroup() %>%
      as.data.frame()
  }
  # Select some specific labels to show
  if (!is.null(label.select)) {
    selected_labels <- complete_data %>%
      subset(complete_data$name %in% label.select, drop = FALSE)
    labs_data <- dplyr::bind_rows(labs_data, selected_labels) %>%
      dplyr::distinct(.data$name, .keep_all = TRUE)
  }


  font.label <- .parse_font(font.label)
  font.label$size <- ifelse(is.null(font.label$size), 12, font.label$size)
  font.label$color <- ifelse(is.null(font.label$color), "black", font.label$color)
  font.label$face <- ifelse(is.null(font.label$face), "plain", font.label$face)

  # Draw the non-significant ("NS") points first (behind) so the significant
  # Up/Down points are plotted on top and remain visible; the data was ordered
  # by padj, which otherwise put the NS points last and hid the significant hits
  # (#365). Stable ordering keeps the factor levels (and thus the legend) intact.
  is.ns <- as.character(data$sig) == "NS"
  data <- data[order(!is.ns), ]

  # Plot
  mean <- lfc <- sig <- name <- padj <- NULL
  # `size` defaults to NULL; passing it as an empty aesthetic warns, so only set
  # it when the user supplied a value (NULL keeps ggplot2's default point size).
  p <- ggplot(data, aes(x = mean, y = lfc))
  if (is.null(size)) {
    p <- p + geom_point(aes(color = sig), alpha = alpha)
  } else {
    p <- p + geom_point(aes(color = sig), size = size, alpha = alpha)
  }

  max.overlaps <- getOption("ggrepel.max.overlaps", default = Inf)

  if (label.rectangle) {
    p <- p + ggrepel::geom_label_repel(
      data = labs_data, mapping = aes(label = name),
      box.padding = unit(0.35, "lines"),
      point.padding = unit(0.3, "lines"),
      force = 1, seed = seed, fontface = font.label$face,
      size = font.label$size / 3, color = font.label$color,
      max.overlaps = max.overlaps
    )
  } else {
    p <- p + ggrepel::geom_text_repel(
      data = labs_data, mapping = aes(label = name),
      box.padding = unit(0.35, "lines"),
      point.padding = unit(0.3, "lines"),
      force = 1, seed = seed, fontface = font.label$face,
      size = font.label$size / 3, color = font.label$color,
      max.overlaps = max.overlaps
    )
  }

  p <- p + scale_x_continuous(breaks = seq(0, max(data$mean), 2)) +
    labs(x = xlab, y = ylab, title = main, color = "") # to remove legend title use color = ""
  if (is.null(facet.by)) {
    p <- p + geom_hline(
      yintercept = c(0, -log2(fc), log2(fc)), linetype = c(1, 2, 2),
      color = line.color
    )
  } else {
    # #498: a single geom_hline with a vectorised linetype breaks under faceting
    # (the yintercept recycles per panel but the linetype does not), so draw the
    # solid zero line and the two dashed threshold lines as separate facet-safe
    # layers. The rendered result is the same three lines.
    p <- p +
      geom_hline(yintercept = 0, linetype = 1, color = line.color) +
      geom_hline(yintercept = c(-log2(fc), log2(fc)), linetype = 2, color = line.color)
  }

  p <- ggpar(p, palette = palette, ggtheme = ggtheme, ...)
  # #498: split into one panel per facet.by group
  if (!is.null(facet.by)) p <- facet(p, facet.by = facet.by)
  p
}

test_that("ggmaplot line.color controls the threshold line color (#322)", {
  data("diff_express", package = "ggpubr")
  hline_color <- function(p) {
    b <- ggplot2::ggplot_build(p)
    i <- which(sapply(b$data, function(d) "yintercept" %in% names(d)))[1]
    unique(b$data[[i]]$colour)
  }
  # No-regression: default threshold lines are black
  expect_equal(hline_color(ggmaplot(diff_express)), "black")
  # Regression: line.color changes them
  expect_equal(hline_color(ggmaplot(diff_express, line.color = "red")), "red")
})

# Regression test for #365: ggmaplot() drew the non-significant ("NS") points
# last, so they covered the significant Up/Down points. NS points are now drawn
# first (behind) and the significant points remain visible on top.

test_that("ggmaplot() draws NS points before the significant ones (#365)", {
  data(diff_express, package = "ggpubr", envir = environment())
  p <- ggmaplot(diff_express, fdr = 0.05, fc = 2, size = 0.5, top = 0)
  d <- p$data   # the (reordered) data passed to ggplot / geom_point
  is_ns <- as.character(d$sig) == "NS"
  # every NS row comes before every significant row (drawn behind)
  if (any(is_ns) && any(!is_ns)) {
    expect_lt(max(which(is_ns)), min(which(!is_ns)))
  }
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

test_that("ggmaplot() legend keeps the Up / Down / NS order (no regression, #365)", {
  data(diff_express, package = "ggpubr", envir = environment())
  p <- ggmaplot(diff_express, fdr = 0.05, fc = 2, top = 0)
  levs <- levels(p$data$sig)
  # NS remains the last legend entry; Up/Down first
  expect_match(levs[1], "^Up")
  expect_match(levs[2], "^Down")
  expect_equal(levs[length(levs)], "NS")
})

context("test-ggmaplot-facet")

# #498: ggmaplot() gains an opt-in `facet.by`. ggmaplot() rebuilds its own plotting
# data (dropping the user's columns) and selects the top genes / Up-Down counts
# globally, so faceting used to fail ("At least one layer must contain all faceting
# variables"). With facet.by set, the grouping column is carried into the plotted
# layers and the top genes are selected PER panel. Without facet.by, output is
# unchanged.

suppressPackageStartupMessages({
  library(ggplot2)
})

# deterministic 2-group DE-like data
set.seed(42)
mk <- function(sp, shift) data.frame(
  baseMean = 10^runif(600, 0, 5),
  log2FoldChange = rnorm(600, shift, 1.2),
  padj = runif(600)^3,
  gene = paste0(sp, "_g", seq_len(600)),
  species = sp
)
df <- rbind(mk("A", 0), mk("B", 0.4))

.label_layer_index <- function(p) {
  which(vapply(p$layers, function(l) inherits(l$geom, "GeomTextRepel") ||
                inherits(l$geom, "GeomLabelRepel"), logical(1)))[1]
}

test_that("faceted ggmaplot renders at draw time (#498)", {
  p <- ggmaplot(df, fdr = 0.1, fc = 2, top = 8, genenames = df$gene,
                facet.by = "species")
  expect_true(inherits(p$facet, "FacetWrap"))
  # the crash was at draw time, not build time -> render the full gtable
  expect_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)), NA)
})

test_that("top genes are selected PER facet, not globally (#498)", {
  top <- 8
  p <- ggmaplot(df, fdr = 0.1, fc = 2, top = top, genenames = df$gene,
                facet.by = "species")
  li <- .label_layer_index(p)
  ld <- p$layers[[li]]$data
  expect_true("species" %in% names(ld))
  # each panel gets up to `top` labels, and its labels belong to that species
  per <- table(ld$species)
  expect_true(all(per <= top))
  expect_true(all(per > 0))
  expect_true(all(grepl("^A_", ld$gene[ld$species == "A"])))
  expect_true(all(grepl("^B_", ld$gene[ld$species == "B"])))
})

test_that("facet.by naming a non-existent column errors informatively (#498)", {
  expect_error(
    ggmaplot(df, genenames = df$gene, facet.by = "nope"),
    "facet.by column"
  )
})

test_that("facet.by = NULL leaves ggmaplot output unchanged (no-regression, #498)", {
  p <- ggmaplot(df, fdr = 0.1, fc = 2, top = 10, genenames = df$gene)
  # unchanged structure: single vectorised threshold-line layer, counted legend
  b <- suppressWarnings(ggplot2::ggplot_build(p))
  # the hline layer keeps its 3 rows (0 and +/- log2(fc)) as a single layer
  hl <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomHline"), logical(1)))
  expect_length(hl, 1)
  # legend keeps the "Up: n" / "Down: n" counted labels
  labs <- levels(b$plot$data$sig)
  expect_true(any(grepl("^Up: [0-9]", labs)))
  expect_true(any(grepl("^Down: [0-9]", labs)))
  expect_false(inherits(p$facet, "FacetWrap"))
})

test_that("faceted legend uses plain Up/Down/NS labels (no global counts) (#498)", {
  p <- ggmaplot(df, fdr = 0.1, fc = 2, top = 8, genenames = df$gene,
                facet.by = "species")
  labs <- levels(p$data$sig)
  expect_true(all(labs %in% c("Up", "Down", "NS")))
  expect_false(any(grepl(":", labs)))   # no "Up: 123" counts under faceting
})

#' @include utilities.R ggpar.R
#' @importFrom rstatix df_split_by get_summary_stats
NULL
#' GGPlot with Summary Stats Table Under the Plot
#'
#' @description Create a ggplot with summary stats (n, median, mean, iqr) table
#'   under the plot. Read more: \href{https://www.datanovia.com/en/blog/how-to-create-a-beautiful-plots-in-r-with-summary-statistics-labels/}{How to Create a Beautiful Plots in R with Summary Statistics Labels}.
#' @inheritParams ggboxplot
#' @param digits integer indicating the number of decimal places (round) to be
#'   used.
#' @param table.font.size the summary table font size.
#' @param position Position adjustment, either as a string, or the result of a
#'   call to a position adjustment function.
#' @param angle numeric value specifying the rotation angle (in degrees) of the
#'   summary table text. Default is 0.
#' @param summaries summary stats to display in the table. Possible values are
#'   those returned by the function \code{\link[rstatix]{get_summary_stats}()},
#'   including: \code{"n", "min", "max",  "median",  "q1", "q2", "q3", "mad",
#'   "mean", "sd", "se", "ci"}.
#' @param ggfunc a ggpubr function, including: ggboxplot, ggviolin, ggdotplot,
#'   ggbarplot, ggline, etc. Can be any other ggplot function that accepts the
#'   following arguments \code{data, x, color, fill, palette, ggtheme,
#'   facet.by}.
#' @param free.panels logical. If TRUE, create free plot panels when the
#'   argument \code{facet.by} is specified.
#' @param labeller Character vector. An alternative to the argument
#'  \code{short.panel.labs}. Possible values are one of "label_both" (panel
#'  labelled by both grouping variable names and levels) and "label_value"
#'  (panel labelled with only grouping levels).
#' @param ... other arguments passed to the function \code{\link{ggpar}()},
#'   \code{\link{facet}()} or \code{\link{ggarrange}()} when printing the plot.
#' @examples
#' # Data preparation
#' # ::::::::::::::::::::::::::::::::::::::::::::::::
#' data("ToothGrowth")
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Add random QC column
#' set.seed(123)
#' qc <- rep(c("pass", "fail"), 30)
#' df$qc <- as.factor(sample(qc, 60))
#' # Inspect the data
#' head(df)
#'
#'
#' # Basic summary stats
#' # ::::::::::::::::::::::::::::::::::::::::::::::::
#' # Compute summary statistics
#' summary.stats <- df %>%
#'   group_by(dose) %>%
#'   get_summary_stats(type = "common")
#' summary.stats
#'
#' # Visualize summary table
#' ggsummarytable(
#'   summary.stats,
#'   x = "dose", y = c("n", "median", "iqr"),
#'   ggtheme = theme_bw()
#' )
#'
#'
#' # Create plots with summary table under the plot
#' # ::::::::::::::::::::::::::::::::::::::::::::::::
#' # Basic plot
#' ggsummarystats(
#'   df,
#'   x = "dose", y = "len",
#'   ggfunc = ggboxplot, add = "jitter"
#' )
#'
#' # Color by groups
#' ggsummarystats(
#'   df,
#'   x = "dose", y = "len",
#'   ggfunc = ggboxplot, add = "jitter",
#'   color = "dose", palette = "npg"
#' )
#'
#' # Create a barplot
#' ggsummarystats(
#'   df,
#'   x = "dose", y = "len",
#'   ggfunc = ggbarplot, add = c("jitter", "median_iqr"),
#'   color = "dose", palette = "npg"
#' )
#'
#' # Facet
#' # ::::::::::::::::::::::::::::::::::::::::::::::::
#' # Specify free.panels = TRUE for free panels
#' ggsummarystats(
#'   df,
#'   x = "dose", y = "len",
#'   ggfunc = ggboxplot, add = "jitter",
#'   color = "dose", palette = "npg",
#'   facet.by = c("supp", "qc"),
#'   labeller = "label_both"
#' )
#'
#' @describeIn ggsummarystats Create a table of summary stats
#' @export
ggsummarytable <- function(data, x, y, digits = 0, size = 3, color = "black", palette = NULL,
                           facet.by = NULL, labeller = "label_value", position = "identity",
                           angle = 0, ggtheme = theme_pubr(), ...) {
  if (missing(ggtheme) & !is.null(facet.by)) {
    ggtheme <- theme_pubr(border = TRUE)
  }
  if (is.null(names(y))) names(y) <- y
  y_values <- as.vector(y)

  df <- as.data.frame(data)
  df$x <- df[[x]]
  if (color %in% colnames(df)) {
    if (missing(position)) position <- position_dodge(0.8)
    group <- color
  } else {
    group <- 1
  }

  df <- df %>%
    mutate(across(where(is.double), ~ round(.x, digits))) %>%
    unite(col = "label", !!!syms(y_values), sep = "\n") %>%
    mutate(y = paste(names(y), collapse = "\n"))
  p <- ggplot(data, aes(x, y)) +
    geom_exec(
      geom_text,
      data = df,
      label = "label", size = size, color = color, group = group,
      position = position, angle = angle
    )
  p <- ggpar(p, ggtheme = ggtheme, palette = palette, xlab = x, ...)
  if (!is.null(facet.by)) p <- facet(p, facet.by = facet.by, labeller = labeller, ...)
  p + rremove("ylab")
}


#' @describeIn ggsummarystats Create a ggplot with a summary stat table under the plot.
#' @export
ggsummarystats <- function(data, x, y, summaries = c("n", "median", "iqr"),
                           ggfunc = ggboxplot,
                           color = "black", fill = "white", palette = NULL,
                           facet.by = NULL, free.panels = FALSE, labeller = "label_value",
                           heights = c(0.80, 0.20), digits = 0, table.font.size = 3,
                           ggtheme = theme_pubr(), ...) {
  if (missing(ggtheme) & !is.null(facet.by)) {
    ggtheme <- theme_pubr(border = TRUE)
  }

  env <- c(as.list(environment()), list(...))
  if (is.null(facet.by)) {
    results <- do.call(ggsummarystats_core, env)
  } else {
    if (free.panels) {
      results <- do.call(ggsummarystats_free_facet, env)
    } else {
      results <- do.call(ggsummarystats_core, env)
    }
  }
  results
}


#' @method print ggsummarystats
#' @param x an object of class \code{ggsummarystats}.
#' @param heights a numeric vector of length 2, specifying the heights of the
#'   main and the summary table, respectively.
#' @rdname ggsummarystats
#' @export
print.ggsummarystats <- function(x, heights = c(0.80, 0.20), ...) {
  res <- ggarrange(plotlist = x, heights = heights, align = "v", ncol = 1)
  print(res)
  invisible(res)
}

#' @method print ggsummarystats_list
#' @param legend character specifying legend position. Allowed values are one of
#'   c("top", "bottom", "left", "right", "none"). To remove the legend use
#'   legend = "none".
#' @param x a list of \code{ggsummarystats}.
#' @rdname ggsummarystats
#' @export
print.ggsummarystats_list <- function(x, heights = c(0.80, 0.20), legend = NULL, ...) {
  # Create a common legend, if legend exists
  legend.grob <- get_legend(x[[1]]$main.plot, position = legend)
  has.legend <- !is.null(legend.grob)
  remove_legend <- function(ggsummarystats) {
    ggsummarystats[[1]] <- ggsummarystats[[1]] + theme(legend.position = "none")
    ggsummarystats
  }
  if (has.legend) {
    x <- map(x, remove_legend)
  }
  # Combining each ggsummarystats
  x <- x %>%
    map(
      function(x, ...) {
        ggarrange(plotlist = x, ...)
      },
      heights = heights, align = "v", ncol = 1
    )
  # Combine the list of ggsumarystats
  res <- ggarrange(plotlist = x, legend = legend, legend.grob = legend.grob)
  # Add legend if exist
  print(res)
  invisible(res)
}


ggsummarystats_core <- function(data, x, y, summaries = c("n", "median", "iqr"),
                                ggfunc = ggboxplot,
                                color = "black", fill = "white", palette = NULL,
                                ggtheme = theme_pubr(), heights = c(0.80, 0.20),
                                facet.by = NULL, free.panels = FALSE, labeller = "label_value",
                                digits = 0, table.font.size = 3, ...) {
  groups <- c(x, color, fill, facet.by) %>%
    unique() %>%
    intersect(colnames(data))
  summary.stats <- suppressWarnings(
    data %>%
      group_by(!!!syms(groups)) %>%
      get_summary_stats(!!y)
  )
  # No need to repeat the panel label on the table
  table.facet.by <- facet.by
  if (free.panels) table.facet.by <- NULL

  main.plot <- ggfunc(
    data,
    x = x, y = y, color = color, fill = fill,
    palette = palette,
    ggtheme = ggtheme,
    facet.by = facet.by, labeller = labeller, ...
  )
  summary.plot <- ggsummarytable(
    summary.stats,
    x = x, y = summaries,
    color = color, palette = palette, legend = "none",
    ggtheme = ggtheme,
    facet.by = table.facet.by, labeller = labeller,
    digits = digits, size = table.font.size
  ) +
    clean_table_theme()

  plots <- list(
    main.plot = main.plot,
    summary.plot = summary.plot
  )
  class(plots) <- c("ggsummarystats", "list")
  plots
}


ggsummarystats_free_facet <- function(data, x, y, facet.by, labeller = "label_value", ...) {
  labeller_func <- switch(labeller,
    label_both = rstatix::df_label_both,
    label_value = rstatix::df_label_value
  )
  groups <- facet.by
  data.grouped <- data %>%
    df_split_by(vars = groups, label_col = "panel", labeller = labeller_func) %>%
    mutate(
      plots = map(data, ggsummarystats_core, x = x, y = y, facet.by = "panel", ...)
    )
  plots <- data.grouped$plots
  names(plots) <- data.grouped$panel
  class(plots) <- c("ggsummarystats_list", "list")
  plots
}

test_that("ggsummarytable forwards angle to the text layer (#595)", {
  # Regression: angle rotates the summary-table text
  p90 <- ggsummarytable(ToothGrowth, x = "dose", y = "len",
                        summaries = "mean", angle = 90)
  expect_true(all(ggplot2::layer_data(p90, 1)$angle == 90))

  # No-regression: default (no angle) is unchanged (angle 0)
  p0 <- ggsummarytable(ToothGrowth, x = "dose", y = "len", summaries = "mean")
  expect_true(all(ggplot2::layer_data(p0, 1)$angle == 0))
})

#' @include utilities.R
#' @import ggplot2
NULL
#' Create a ggplot with Transparent Background
#'
#' @description Create a ggplot with transparent background.
#' @param base_size base font size
#' @param base_family base font family
#' @seealso \code{\link{theme_pubr}}
#' @examples
#' # Create a scatter plot
#' sp <- ggscatter(iris,
#'   x = "Sepal.Length", y = "Sepal.Width",
#'   color = "Species", palette = "jco",
#'   size = 3, alpha = 0.6
#' )
#' sp
#'
#' # Transparent theme
#' sp + theme_transparent()
#'
#' @export
theme_transparent <- function(base_size = 12, base_family = "") {
  theme_classic(base_size = base_size, base_family = base_family) +
    theme(
      panel.background = element_rect(fill = "transparent", colour = NA),
      plot.background = element_rect(fill = "transparent", colour = NA),
      legend.background = element_rect(fill = "transparent", colour = NA),
      legend.box.background = element_rect(fill = "transparent", colour = NA),
      axis.line = element_blank(),
      axis.title = element_blank(),
      plot.title = element_blank(),
      axis.text = element_blank(),
      axis.ticks = element_blank()
    )
}

#' @include utilities.R
NULL
#' Arrange Multiple ggplots
#'
#' @description Arrange multiple ggplots on the same page. Wrapper around
#'   \code{\link[cowplot]{plot_grid}()}. Can arrange multiple ggplots over
#'   multiple pages, compared to the standard
#'   \code{\link[cowplot]{plot_grid}()}. Can also create a common unique legend
#'   for multiple plots.
#' @inheritParams cowplot::plot_grid
#' @param ... list of plots to be arranged into the grid. The plots can be
#'   either ggplot2 plot objects or arbitrary gtables.
#' @param plotlist (optional) list of plots to display.
#' @param ncol (optional) number of columns in the plot grid.
#' @param nrow (optional) number of rows in the plot grid.
#' @param byrow logical. If \code{TRUE} (default), the plots are filled into the
#'   grid by row; set to \code{FALSE} to fill by column. Passed to
#'   \code{\link[cowplot]{plot_grid}()}.
#' @param labels (optional) list of labels to be added to the plots. You can
#'   also set labels="AUTO" to auto-generate upper-case labels or labels="auto"
#'   to auto-generate lower-case labels.
#' @param font.label a list of arguments for customizing labels. Allowed values
#'   are the combination of the following elements: size (e.g.: 14), face (e.g.:
#'   "plain", "bold", "italic", "bold.italic"), color (e.g.: "red") and family.
#'   For example font.label = list(size = 14, face = "bold", color ="red").
#' @param label.x (optional) Single value or vector of x positions for plot
#'   labels, relative to each subplot. Defaults to 0 for all labels. (Each label
#'   is placed all the way to the left of each plot.)
#' @param label.y (optional) Single value or vector of y positions for plot
#'   labels, relative to each subplot. Defaults to 1 for all labels. (Each label
#'   is placed all the way to the top of each plot.)
#' @param widths (optional) numerical vector of relative columns widths. For
#'   example, in a two-column grid, widths = c(2, 1) would make the first column
#'   twice as wide as the second column.
#' @param heights same as \code{widths} but for column heights.
#' @param legend character specifying legend position. Allowed values are one of
#'   c("top", "bottom", "left", "right", "none"). To remove the legend use
#'   legend = "none".
#' @param common.legend logical value, or one or several plot indices. Default is FALSE. If
#'   \code{TRUE}, a single shared legend is used for all the arranged plots.
#'   Note that this legend is \strong{not} merged or validated across plots: it
#'   is simply the legend of the \emph{first} plot, and the other legends are
#'   dropped. It is therefore only correct when every plot shares the same scale
#'   (same groups/levels, order and color range). If the first plot's legend is
#'   not representative - for example a group is missing in the first plot, or a
#'   continuous color scale spans a different range - the shared legend will
#'   misrepresent the other plots. In that case you can: (i) give the plots a
#'   consistent scale yourself (e.g. \code{scale_fill_manual(limits = ...)} or
#'   \code{scale_color_continuous(limits = ...)}) so a single legend is valid,
#'   and/or (ii) choose which plot's legend is shown by passing that plot's index,
#'   e.g. \code{common.legend = 2} to use the second plot's legend (equivalent to
#'   \code{legend.grob = get_legend(plots[[2]])}). You can also pass several indices,
#'   e.g. \code{common.legend = c(1, 2)}, to keep and combine the legends of those
#'   plots into a single shared block (side by side for \code{legend = "top"}/
#'   \code{"bottom"}, stacked for \code{"left"}/\code{"right"}) - useful when the
#'   plots genuinely need different legends. Note that (ii) only changes
#'   \emph{which} legend is displayed; it does not re-map the other plots' color
#'   scales, so for the legend keys to match every panel you still need a
#'   consistent scale as in (i). When the plots genuinely cannot be described by a
#'   single legend (e.g. a discrete fill in one plot and a continuous color bar in
#'   another), use \code{common.legend = FALSE} to keep a separate legend per plot.
#' @param legend.grob a legend grob as returned by the function
#'   \code{\link{get_legend}()}. If provided, it will be used as the common
#'   legend.
#' @param spacing numeric value giving the margin, in text-line units, set
#'   uniformly around each plot to increase the gap between the arranged plots.
#'   Default is 0, which leaves each plot's own margins untouched (existing
#'   arrangements are unchanged). A positive value sets a uniform margin of that
#'   many lines around every plot, replacing the plots' default margin; e.g.
#'   \code{spacing = 1} puts a one-line margin around each plot.
#' @return an object of class \code{ggarrange}, which is a ggplot or a
#'   list of ggplots.
#' @author Laszlo Erdey \email{erdey.laszlo@@econ.unideb.hu}
#' @seealso \code{\link{annotate_figure}()}
#' @examples
#' data("ToothGrowth")
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#'
#' # Create some plots
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::
#' # Box plot
#' bxp <- ggboxplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' # Dot plot
#' dp <- ggdotplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' # Density plot
#' dens <- ggdensity(df, x = "len", fill = "dose", palette = "jco")
#'
#' # Arrange
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::
#' ggarrange(bxp, dp, dens, ncol = 2, nrow = 2)
#' # Use a common legend for multiple plots
#' ggarrange(bxp, dp, common.legend = TRUE)
#'
#' @export
ggarrange <- function(..., plotlist = NULL, ncol = NULL, nrow = NULL,
                      labels = NULL, label.x = 0, label.y = 1, hjust = -0.5, vjust = 1.5,
                      font.label = list(size = 14, color = "black", face = "bold", family = NULL),
                      align = c("none", "h", "v", "hv"),
                      widths = 1, heights = 1, byrow = TRUE,
                      legend = NULL, common.legend = FALSE, legend.grob = NULL,
                      spacing = 0) {
  # Open null device to avoid blank page before plot------
  # see cowplot:::as_grob.ggplot
  null_device <- base::getOption("ggpubr.null_device", default = cowplot::pdf_null_device)
  cur_dev <- grDevices::dev.cur()
  # Open null device to avoid blank page before plot
  null_device(width = 6, height = 6)
  null_dev <- grDevices::dev.cur()
  on.exit({
    grDevices::dev.off(null_dev)
    if (cur_dev > 1) grDevices::dev.set(cur_dev)
  })


  plots <- c(list(...), plotlist)
  plots <- .add_plot_spacing(plots, spacing)
  align <- match.arg(align)
  nb.plots <- length(plots)
  page.layout <- .get_layout(ncol, nrow, nb.plots)
  ncol <- page.layout$ncol
  nrow <- page.layout$nrow
  nb.plots.per.page <- .nbplots_per_page(ncol, nrow)

  # #347: `common.legend` may also be one or several plot indices, selecting which
  # plot(s) supply the shared legend (useful when the first plot's legend is not
  # representative, e.g. a group is missing in the first plot, or when the plots
  # genuinely need several different legends). A single index uses that plot's
  # legend; several indices combine those plots' legends into one block. Logical
  # TRUE/FALSE behave exactly as before: is.numeric(TRUE) is FALSE, so this branch
  # is skipped for them and the default output is unchanged.
  legend.plot.index <- NULL
  if (is.numeric(common.legend)) {
    idx <- common.legend
    if (length(idx) < 1L || anyNA(idx) || !all(is.finite(idx)) ||
        !all(idx == as.integer(idx)) || any(idx < 1) || any(idx > nb.plots)) {
      stop("When numeric, `common.legend` must be whole-number plot indices between ",
           "1 and the number of plots (", nb.plots, ").", call. = FALSE)
    }
    idx <- as.integer(idx)
    if (anyDuplicated(idx)) {
      stop("`common.legend` plot indices must be unique.", call. = FALSE)
    }
    if (any(vapply(plots[idx], is.null, logical(1)))) {
      stop("`common.legend` points to an empty (NULL) plot.", call. = FALSE)
    }
    legend.plot.index <- idx
    common.legend <- TRUE
  }

  if (!is.null(legend.grob)) {
    common.legend <- TRUE
  }
  if (is.null(legend) & common.legend) {
    legend <- "top"
  }
  legend <- .check_legend(legend)
  if (!is.null(legend)) {
    plots <- purrr::map(
      plots,
      function(x) {
        if (!is.null(x)) x + theme(legend.position = legend) else x
      }
    )
  }

  if (common.legend) {
    if (is.null(legend.grob)) {
      legend.grob <- if (!is.null(legend.plot.index)) {
        .combine_legends(plots, legend.plot.index, legend)
      } else {
        get_legend(plots)
      }
    }
    plots <- purrr::map(
      plots,
      function(x) {
        if (!is.null(x)) x + theme(legend.position = "none") else x
      }
    )
  }

  # Split plots over multiple pages
  if (nb.plots > nb.plots.per.page) {
    plots <- split(plots, ceiling(seq_along(plots) / nb.plots.per.page))
  }
  # One unique page
  else {
    plots <- list(plots)
  }

  # label arguments
  .lab <- .update_label_pms(font.label,
    label.x = label.x, label.y = label.y,
    hjust = hjust, vjust = vjust
  )

  res <- purrr::map(plots, .plot_grid,
    ncol = ncol, nrow = nrow, labels = labels,
    label_size = .lab$size, label_fontfamily = .lab$family,
    label_fontface = .lab$face, label_colour = .lab$color,
    label_x = .lab$label.x, label_y = .lab$label.y,
    hjust = .lab$hjust, vjust = .lab$vjust, align = align,
    rel_widths = widths, rel_heights = heights, byrow = byrow,
    legend = legend, common.legend.grob = legend.grob
  )


  if (length(res) == 1) res <- res[[1]]

  class(res) <- c(class(res), "ggarrange")
  res
}


.get_layout <- function(ncol, nrow, nb.plots) {
  if (!is.null(ncol) & !is.null(nrow)) {} else if (!is.null(ncol)) {
    if (ncol == 1) nrow <- nb.plots
  } else if (!is.null(nrow)) {
    if (nrow == 1) ncol <- nb.plots
  }
  list(ncol = ncol, nrow = nrow)
}

# Compute number of plots per page
.nbplots_per_page <- function(ncol = NULL, nrow = NULL) {
  if (!is.null(ncol) & !is.null(nrow)) {
    ncol * nrow
  } else if (!is.null(ncol)) {
    ncol
  } else if (!is.null(nrow)) {
    nrow
  } else {
    Inf
  }
}


# Increase the space between arranged plots by adding a uniform margin (in
# text-line units) around each ggplot. spacing = 0 (default) leaves the plots
# untouched, so the arrangement is unchanged (#151). Non-ggplot entries (NULL
# spacers, grobs) are returned as-is.
.add_plot_spacing <- function(plots, spacing = 0) {
  if (is.null(spacing)) {
    return(plots)
  }
  if (!is.numeric(spacing) || anyNA(spacing)) {
    stop("`spacing` must be a single non-missing numeric value.", call. = FALSE)
  }
  if (all(spacing == 0)) {
    return(plots)
  }
  margin <- grid::unit(rep(spacing[1], 4), "lines")
  purrr::map(plots, function(x) {
    if (inherits(x, "ggplot")) x + ggplot2::theme(plot.margin = margin) else x
  })
}

# #347: build the shared legend for a numeric `common.legend`. For a single index
# this is simply that plot's legend (identical to the scalar path). For several
# indices the selected plots' legends are combined into one grob: side by side for a
# horizontal legend strip (legend = "top"/"bottom") and stacked for a vertical one
# ("left"/"right"). gtable's cbind/rbind (dispatched via the base generics) keep the
# result a gtable with the same $height/$width interface a single legend has, so
# `.plot_grid` needs no change and the single-legend / TRUE / FALSE paths stay
# byte-identical. Selected plots whose legend is empty are skipped; if none remain
# the result is NULL (no shared legend, no error). The plots passed here already have
# `legend.position` set to `legend`, so the extracted legends have the right
# orientation for the strip.
.combine_legends <- function(plots, index, legend = "top") {
  legs <- lapply(index, function(i) get_legend(plots[[i]]))
  legs <- Filter(function(g) !is.null(g) && !inherits(g, "zeroGrob"), legs)
  if (length(legs) == 0) {
    return(NULL)
  }
  if (length(legs) == 1) {
    return(legs[[1]])
  }
  horizontal <- isTRUE(legend %in% c("top", "bottom"))
  combine2 <- if (horizontal) {
    function(a, b) cbind(a, b, size = "max")
  } else {
    function(a, b) rbind(a, b, size = "max")
  }
  Reduce(combine2, legs)
}

.plot_grid <- function(plotlist, legend = "top", common.legend.grob = NULL, ...) {
  res <- cowplot::plot_grid(plotlist = plotlist, ...)
  if (is.null(common.legend.grob)) {
    return(res)
  } else {
    leg <- common.legend.grob
    lheight <- sum(leg$height)
    lwidth <- sum(leg$width)
  }

  arrangeGrob <- gridExtra::arrangeGrob
  unit.c <- grid::unit.c
  .unit <- grid::unit(1, "npc")

  res <- switch(legend,
    top = arrangeGrob(leg, res,
      ncol = 1,
      heights = unit.c(lheight, .unit - lheight)
    ),
    bottom = arrangeGrob(res, leg,
      ncol = 1,
      heights = unit.c(unit(1, "npc") - lheight, lheight)
    ),
    left = arrangeGrob(leg, res,
      ncol = 2,
      widths = unit.c(lwidth, .unit - lwidth)
    ),
    right = arrangeGrob(res, leg,
      ncol = 2,
      widths = unit.c(.unit - lwidth, lwidth)
    )
  )

  p <- cowplot::ggdraw() + cowplot::draw_grob(grid::grobTree(res))
  p
}

# update label parameters for cowplot::plot_grid()
.update_label_pms <- function(font.label,
                              label.x = 0, label.y = 1, hjust = -0.5, vjust = 1.5) {
  .font <- list(size = 14, color = "black", face = "bold", family = NULL)
  new.font.names <- names(font.label)
  for (i in new.font.names) .font[[i]] <- font.label[[i]]

  pms <- .font
  list(
    size = pms$size,
    family = pms$family,
    face = pms$face,
    color = pms$color,
    label.x = label.x, label.y = label.y,
    hjust = hjust, vjust = vjust
  )
}

context("test-ggarrange-spacing")

.p1 <- ggscatter(mtcars, "wt", "mpg")
.p2 <- ggboxplot(ToothGrowth, "dose", "len")

test_that("spacing = 0 (default) leaves the plots untouched (#151 no-regression)", {
  plots <- list(.p1, .p2)
  expect_identical(ggpubr:::.add_plot_spacing(plots, 0), plots)
  expect_identical(ggpubr:::.add_plot_spacing(plots, spacing = 0), plots)
  # NULL spacing also a no-op
  expect_identical(ggpubr:::.add_plot_spacing(plots, NULL), plots)
})

test_that("spacing > 0 adds a uniform plot.margin (in lines) to each ggplot (#151)", {
  out <- ggpubr:::.add_plot_spacing(list(.p1, .p2), spacing = 1)
  m1 <- out[[1]]$theme$plot.margin
  expect_true(inherits(m1, "unit"))
  expect_equal(as.numeric(m1), rep(1, 4))
  expect_true(all(grid::unitType(m1) == "lines"))
  # both plots get it
  expect_equal(as.numeric(out[[2]]$theme$plot.margin), rep(1, 4))
})

test_that("spacing passes non-ggplot entries (NULL spacers) through unchanged (#151)", {
  plots <- list(.p1, NULL, .p2)
  out <- ggpubr:::.add_plot_spacing(plots, spacing = 2)
  expect_null(out[[2]])
  expect_equal(as.numeric(out[[1]]$theme$plot.margin), rep(2, 4))
})

test_that("ggarrange gains a spacing argument that renders (#151)", {
  expect_error(ggarrange(.p1, .p2, ncol = 2, spacing = 2), NA)
  # default call still works and returns a ggarrange object
  g <- ggarrange(.p1, .p2, ncol = 2)
  expect_s3_class(g, "ggarrange")
  g2 <- ggarrange(.p1, .p2, ncol = 2, spacing = 1.5)
  expect_s3_class(g2, "ggarrange")
})

test_that("spacing rejects non-numeric / NA input cleanly (#151)", {
  expect_error(ggpubr:::.add_plot_spacing(list(.p1), spacing = "a"), "numeric")
  expect_error(ggpubr:::.add_plot_spacing(list(.p1), spacing = NA), "numeric|non-missing")
  expect_error(ggarrange(.p1, .p2, spacing = "big"), "numeric")
})

# Regression test for #225: ggarrange() gains a `byrow` argument to fill the grid
# by column (byrow = FALSE) instead of by row (the default).

.plots <- list(
  ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) + ggplot2::geom_point(),
  ggplot2::ggplot(mtcars, ggplot2::aes(wt, disp)) + ggplot2::geom_point(),
  ggplot2::ggplot(mtcars, ggplot2::aes(wt, hp)) + ggplot2::geom_point(),
  ggplot2::ggplot(mtcars, ggplot2::aes(mpg, hp)) + ggplot2::geom_point()
)

.render_md5 <- function(byrow) {
  f <- tempfile(fileext = ".png")
  on.exit(unlink(f))
  g <- do.call(ggarrange, c(.plots, list(ncol = 2, nrow = 2, byrow = byrow)))
  ggplot2::ggsave(f, g, width = 5, height = 4, dpi = 72)
  unname(tools::md5sum(f))
}

test_that("ggarrange() has a byrow argument that changes the layout (#225)", {
  expect_true("byrow" %in% names(formals(ggarrange)))
  # the comparison below renders to PNG; skip where no PNG device is available
  skip_if_not(isTRUE(capabilities("png")), "no PNG device")
  # byrow = FALSE (fill by column) must render a DIFFERENT arrangement than the
  # by-row default; if byrow were not forwarded these would be identical.
  expect_false(identical(.render_md5(TRUE), .render_md5(FALSE)))
})

test_that("ggarrange() default is unchanged (byrow = TRUE, no regression, #225)", {
  g <- ggarrange(plotlist = .plots, ncol = 2, nrow = 2)
  expect_s3_class(g, "ggarrange")
  skip_if_not(isTRUE(capabilities("png")), "no PNG device")
  # default and explicit byrow = TRUE produce the same rendering
  expect_identical(.render_md5(TRUE), .render_md5(TRUE))
})

#' @include desc_statby.R utilities_base.R utilities_color.R
NULL
#' @import ggplot2
#' @importFrom magrittr %>%
#' @importFrom rstatix df_group_by df_nest_by df_select df_arrange
#' @importFrom tibble as_tibble
#' @importFrom dplyr group_by mutate across where group_nest arrange desc reframe slice_head slice_tail
#' @importFrom purrr map2 map
#' @importFrom tidyr unite
#' @importFrom dplyr select distinct
#' @importFrom dplyr summarise
#' @importFrom dplyr everything
#' @importFrom grid drawDetails
#' @importFrom rlang !!
#' @importFrom rlang !!!
#' @importFrom rlang sym syms .data
#' @importFrom rlang %||%


required_package <- function(pkg) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    stop(
      pkg, " package needed to be installed before using this function. ",
      "Type this in R: install.packages('", pkg, "')"
    )
  }
}

# Resolve line width compatibility between `size` and `linewidth`.
.resolve_linewidth_args <- function(size = NULL, linewidth = NULL, default_linewidth = NULL,
                                    warn_if_size = TRUE, null_size_after = FALSE) {
  if (!is.null(size) && !is.null(linewidth)) {
    stop("Please specify either 'size' or 'linewidth', not both. Use 'linewidth' for ggplot2 3.4.0+ compatibility.")
  }

  if (!is.null(size)) {
    if (isTRUE(warn_if_size)) {
      warning("The 'size' parameter for lines is deprecated in ggplot2 3.4.0+. Please use 'linewidth' instead to avoid this warning in future versions.")
    }
    linewidth <- size
    if (isTRUE(null_size_after)) size <- NULL
  } else if (is.null(linewidth)) {
    linewidth <- default_linewidth
  }

  list(size = size, linewidth = linewidth)
}

# Unnesting, adapt to tidyr 1.0.0
unnest <- function(data, cols = "data", ...) {
  if (is_pkg_version_sup("tidyr", "0.8.3")) {
    results <- tidyr::unnest(data, cols = dplyr::all_of(cols), ...)
  } else {
    results <- tidyr::unnest(data, ...)
  }
  results
}

# Check if an installed package version is superior to a specified version
# Version, pkg: character vector
is_pkg_version_sup <- function(pkg, version) {
  vv <- as.character(utils::packageVersion(pkg))
  cc <- utils::compareVersion(vv, version) > 0
  cc
}

keep_only_tbl_df_classes <- function(x) {
  toremove <- setdiff(class(x), c("tbl_df", "tbl", "data.frame"))
  if (length(toremove) > 0) {
    class(x) <- setdiff(class(x), toremove)
  }
  x
}

# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Execute a geom_* function from ggplot2
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# geomfunc : gem_*() functions
# data data for mapping
# ... argument accepeted by the function
# return a plot if geomfunc != NULL or a list(option, mapping) if geomfunc is NULL
.geom_exec <- function(geomfunc = NULL, data = NULL,
                       position = NULL, ...) {
  geom_exec(geomfunc = geomfunc, data = data, position = position, ...)
}

# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Official argument from ggplot2
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# bar plot arguments
.barplot_params <- function(...) {
  x <- list(...)
  res <- list()
  res$width <- x$width
  res$binwidth <- x$binwidth
  res$na.rm <- ifelse(!is.null(x$na.rm), x$na.rm, FALSE)
  res$show.legend <- ifelse(!is.null(x$show.legend), x$show.legend, NA)
  res$inherit.aes <- ifelse(!is.null(x$inherit.aes), x$inherit.aes, TRUE)
  return(res)
}

# box plot arguments
.boxplot_params <- function(...) {
  x <- list(...)
  res <- list()
  res$outlier.colour <- x$outlier.colour
  res$outlier.shape <- ifelse(!is.null(x$outlier.shape), x$outlier.shape, 19)
  res$outlier.size <- ifelse(!is.null(x$outlier.size), x$outlier.size, 1.5)
  res$outlier.stroke <- ifelse(!is.null(x$outlier.stroke), x$outlier.stroke, 0.5)
  res$notch <- ifelse(!is.null(x$notch), x$notch, FALSE)
  res$notchwidth <- ifelse(!is.null(x$notchwidth), x$notchwidth, 0.5)
  res$varwidth <- ifelse(!is.null(x$varwidth), x$varwidth, FALSE)
  res$na.rm <- ifelse(!is.null(x$na.rm), x$na.rm, FALSE)
  res$show.legend <- ifelse(!is.null(x$show.legend), x$show.legend, NA)
  res$inherit.aes <- ifelse(!is.null(x$inherit.aes), x$inherit.aes, TRUE)
  return(res)
}

.dotplot_params <- function(...) {
  x <- list(...)
  res <- list()
  res$stackratio <- ifelse(!is.null(x$stackratio), x$stackratio, 1)
  res$width <- ifelse(!is.null(x$width), x$width, 0.9)
  return(res)
}

.violin_params <- function(...) {
  x <- list(...)
  res <- list()
  # Use exact ([[) matching, not $, so a facet argument like scales = "free"
  # is not partial-matched to the violin `scale` parameter (which would set an
  # invalid scale and crash geom_violin at draw time, #398).
  res$stat <- ifelse(!is.null(x[["stat"]]), x[["stat"]], "ydensity")
  res$draw_quantiles <- x[["draw_quantiles"]]
  res$scale <- ifelse(!is.null(x[["scale"]]), x[["scale"]], "area")
  res$trim <- ifelse(!is.null(x[["trim"]]), x[["trim"]], TRUE)
  res$adjust <- x[["adjust"]] # Bandwidth adjustment for kernel density (Issue #552)
  return(res)
}

.hist_params <- function(...) {
  x <- list(...)
  res <- list()
  res$binwidth <- x$binwidth
  res$bins <- x$bins
  return(res)
}

.standard_params <- function(...) {
  x <- list(...)
  res <- list()
  res$color <- ifelse(!is.null(x$color), x$color, "black")
  res$color <- ifelse(!is.null(x$colour), x$colour, res$color)
  res$linetype <- ifelse(!is.null(x$linetype), x$linetype, "solid")
  res$size <- ifelse(!is.null(x$size), x$size, 1)
  res$fill <- ifelse(!is.null(x$fill), x$fill, "black")
  res$shape <- ifelse(!is.null(x$shape), x$shape, 19)
  res
}


# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Graphical parameters
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# Set plot orientation
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.set_orientation <-
  function(p, orientation = c("vertical", "horizontal", "reverse"),
           xlim = NULL, ylim = NULL) {
    ori <- match.arg(orientation)
    if (ori == "horizontal") {
      # Pass the axis limits into coord_flip() (in the original, pre-flip
      # coordinates, so ylim still refers to the value axis). A separate
      # coord_cartesian() would be dropped when coord_flip() replaces it (#646).
      p + coord_flip(xlim = xlim, ylim = ylim)
    } else if (ori == "reverse") {
      p + scale_y_reverse()
    } else {
      p
    }
  }


# Change title and labels
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.labs <- function(p, main = NULL, xlab = NULL, ylab = NULL,
                  font.main = NULL, font.x = NULL, font.y = NULL,
                  submain = NULL, caption = NULL,
                  font.submain = NULL, font.caption = NULL) {
  font.main <- .parse_font(font.main)
  font.x <- .parse_font(font.x)
  font.y <- .parse_font(font.y)
  font.submain <- .parse_font(font.submain)
  font.caption <- .parse_font(font.caption)

  if (is.logical(main)) {
    if (!main) main <- NULL
  }

  if (is.logical(submain)) {
    if (!submain) submain <- NULL
  }

  if (is.logical(caption)) {
    if (!caption) caption <- NULL
  }


  if (!is.null(main)) {
    p <- p + labs(title = main)
  }

  if (!is.null(submain)) {
    p <- p + labs(subtitle = submain)
  }

  if (!is.null(caption)) {
    p <- p + labs(caption = caption)
  }

  if (!is.null(xlab)) {
    if (xlab == FALSE) {
      p <- p + theme(axis.title.x = element_blank())
    } else {
      p <- p + labs(x = xlab)
    }
  }

  if (!is.null(ylab)) {
    if (ylab == FALSE) {
      p <- p + theme(axis.title.y = element_blank())
    } else {
      p <- p + labs(y = ylab)
    }
  }

  if (!is.null(font.main)) {
    p <- .add_text_theme(p, "plot.title", element_text(
      size = font.main$size,
      lineheight = 1.0, face = font.main$face, colour = font.main$color
    ))
  }
  if (!is.null(font.submain)) {
    p <- .add_text_theme(p, "plot.subtitle", element_text(
      size = font.submain$size,
      lineheight = 1.0, face = font.submain$face, colour = font.submain$color
    ))
  }
  if (!is.null(font.caption)) {
    p <- .add_text_theme(p, "plot.caption", element_text(
      size = font.caption$size,
      lineheight = 1.0, face = font.caption$face, colour = font.caption$color
    ))
  }
  if (!is.null(font.x)) {
    p <- .add_text_theme(p, "axis.title.x", element_text(
      size = font.x$size,
      face = font.x$face, colour = font.x$color
    ))
  }
  if (!is.null(font.y)) {
    p <- .add_text_theme(p, "axis.title.y", element_text(
      size = font.y$size,
      face = font.y$face, colour = font.y$color
    ))
  }


  p
}


# Merge a text theme element into a single theme slot, unless the plot already
# carries a markdown element there. ggtext's element_markdown()/element_textbox()
# (used e.g. by survminer on risk-table strata labels) cannot be merged with a
# plain element_text(): ggplot2 errors "Only elements of the same class can be
# merged". For such slots we leave the existing element untouched, so ggpar() no
# longer crashes on ggsurvplot objects. Normal element_text/element_blank/NULL
# slots are handled exactly as before (p + theme(...)), so output is unchanged
# (#382).
.add_text_theme <- function(p, slot, element) {
  existing <- p$theme[[slot]]
  if (inherits(existing, c("element_markdown", "element_textbox"))) {
    return(p)
  }
  p + do.call(ggplot2::theme, stats::setNames(list(element), slot))
}

# ticks
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.set_ticks <-
  function(p, ticks = TRUE, tickslab = TRUE, font.tickslab = NULL,
           xtickslab.rt = NULL, ytickslab.rt = NULL,
           font.xtickslab = font.tickslab, font.ytickslab = font.tickslab) {
    . <- xhjust <- xvjust <- NULL
    if (!is.null(xtickslab.rt)) {
      if (xtickslab.rt > 5) xhjust <- 1
      if (xtickslab.rt == 90) xvjust <- 0.5
    } else {
      xhjust <- NULL
    }

    if (ticks) {
      ticks <-
        element_line(colour = "black")
    } else {
      ticks <- element_blank()
    }

    if (is.null(font.xtickslab)) {
      font.x <- list()
    } else {
      font.x <- .parse_font(font.xtickslab)
    }
    if (is.null(font.ytickslab)) {
      font.y <- list()
    } else {
      font.y <- .parse_font(font.ytickslab)
    }

    p <- p + theme(axis.ticks = ticks)
    if (tickslab) {
      xtickslab <- font.x %>%
        .add_item(hjust = xhjust, vjust = xvjust, angle = xtickslab.rt) %>%
        do.call(element_text, .)
      ytickslab <- font.y %>%
        .add_item(angle = ytickslab.rt) %>%
        do.call(element_text, .)
      # Skip a slot whose existing element is markdown (see .add_text_theme, #382)
      p <- .add_text_theme(p, "axis.text.x", xtickslab)
      p <- .add_text_theme(p, "axis.text.y", ytickslab)
    } else {
      p <- p + theme(axis.text.x = element_blank(), axis.text.y = element_blank())
    }
    p
  }


# Change Axis limits
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.set_axis_limits <- function(xlim = NULL, ylim = NULL) {
  if (!is.null(xlim) | !is.null(ylim)) coord_cartesian(xlim, ylim)
}


# Axis scales
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.set_scale <- function(p, xscale = c("none", "log2", "log10", "sqrt"),
                       yscale = c("none", "log2", "log10", "sqrt"),
                       format.scale = FALSE) {
  xscale <- match.arg(xscale)
  yscale <- match.arg(yscale)
  .x <- ".x"

  if (format.scale) {
    if (!requireNamespace("scales")) stop("The R package 'scales' is required.")

    if (yscale == "log2") {
      p <- p + scale_y_continuous(
        trans = scales::log2_trans(),
        breaks = scales::trans_breaks("log2", function(x) 2^x),
        labels = scales::trans_format("log2", scales::math_format(2^.x))
      )
    } else if (yscale == "log10") {
      p <- p + scale_y_continuous(
        trans = scales::log10_trans(),
        breaks = scales::trans_breaks("log10", function(x) 10^x),
        labels = scales::trans_format("log10", scales::math_format(10^.x))
      )
    }

    if (xscale == "log2") {
      p <- p + scale_x_continuous(
        trans = scales::log2_trans(),
        breaks = scales::trans_breaks("log2", function(x) 2^x),
        labels = scales::trans_format("log2", scales::math_format(2^.x))
      )
    } else if (xscale == "log10") {
      p <- p + scale_x_continuous(
        trans = scales::log10_trans(),
        breaks = scales::trans_breaks("log10", function(x) 10^x),
        labels = scales::trans_format("log10", scales::math_format(10^.x))
      )
    }
  } else {
    if (xscale != "none") p <- p + scale_x_continuous(trans = xscale)
    if (yscale != "none") p <- p + scale_y_continuous(trans = yscale)
  }
  p
}

# Legends
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.set_legend <- function(p, legend = NULL,
                        legend.title = NULL, font.legend = NULL,
                        legend.direction = NULL) {
  if (is.null(legend.title)) legend.title <- waiver()
  font <- .parse_font(font.legend)

  if (!is.null(legend)) p <- p + theme(legend.position = legend)
  # legend.direction was previously ignored for legend = "top"/"bottom" (it only
  # took effect via ggplot2 defaults for "left"/"right"); apply it explicitly (#652).
  if (!is.null(legend.direction)) {
    p <- p + theme(legend.direction = legend.direction)
  }

  # Handle legend.title - skip if empty string (used to hide legend title)
  # Empty string causes "Ignoring unknown labels" warnings in ggplot2
  if (!.is_empty(legend.title) && !(is.character(legend.title) && legend.title == "")) {
    if (.is_list(legend.title)) {
      p <- p + do.call(ggplot2::labs, legend.title)
    } else {
      # Only set legend title for aesthetics actually used in the plot
      # to avoid "Ignoring unknown labels" warnings from ggplot2
      # Check both plot-level and layer-level mappings
      used_aes <- names(p$mapping)
      for (layer in p$layers) {
        used_aes <- c(used_aes, names(layer$mapping))
      }
      used_aes <- unique(used_aes)
      labs_args <- list()
      if ("colour" %in% used_aes || "color" %in% used_aes) labs_args$colour <- legend.title
      if ("fill" %in% used_aes) labs_args$fill <- legend.title
      if ("linetype" %in% used_aes) labs_args$linetype <- legend.title
      if ("shape" %in% used_aes) labs_args$shape <- legend.title
      if ("size" %in% used_aes) labs_args$size <- legend.title
      if ("alpha" %in% used_aes) labs_args$alpha <- legend.title
      if (length(labs_args) > 0) {
        p <- p + do.call(ggplot2::labs, labs_args)
      }
    }
  }

  if (!is.null(font)) {
    p <- .add_text_theme(p, "legend.text", element_text(
      size = font$size,
      face = font$face, colour = font$color
    ))
    p <- .add_text_theme(p, "legend.title", element_text(
      size = font$size,
      face = font$face, colour = font$color
    ))
  }

  p
}


# Set ticks by
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.set_ticksby <- function(p, xticks.by = NULL, yticks.by = NULL) {
  # x and y must be handled independently: an `else if` here dropped xticks.by
  # whenever yticks.by was also supplied (#333).
  if (!is.null(yticks.by)) {
    # Forcing ymin to start at 0 when distribution plot
    gg_mapping <- .get_gg_xy_variables(p)
    is_density_plot <- gg_mapping["y"] %in% c("..count..", "..density..", "..ecdf..")
    ymin <- NULL
    if (is_density_plot) ymin <- 0
    p <- p + scale_y_continuous(breaks = get_breaks(by = yticks.by, from = ymin))
  }
  # Skip xticks.by only when x is discrete: adding scale_x_continuous() to a
  # discrete x (e.g. ggboxplot) errors at draw time, so a discrete x keeps its
  # default axis (as it silently did when xticks.by was combined with yticks.by
  # before this fix). Every non-discrete x (numeric, integer, Date, POSIXct)
  # still receives the continuous scale exactly as before (#333).
  if (!is.null(xticks.by) && !.is_discrete_x(p)) {
    p <- p + scale_x_continuous(breaks = get_breaks(by = xticks.by))
  }
  p
}

# TRUE when the plot's x aesthetic resolves to discrete data (factor, character
# or logical), for which a continuous x scale cannot be applied. Falls back to
# FALSE (treat as continuous, i.e. apply the scale as before) if x cannot be
# evaluated, so behavior for non-discrete x is unchanged from previous versions.
.is_discrete_x <- function(p) {
  x_data <- tryCatch(
    rlang::eval_tidy(p$mapping$x, p$data),
    error = function(e) NULL
  )
  is.factor(x_data) || is.character(x_data) || is.logical(x_data)
}


# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Add stat
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

.check_add.params <- function(add, add.params, error.plot, data, color, fill, ...) {
  if (color %in% names(data) & is.null(add.params$color)) add.params$color <- color
  if (fill %in% names(data) & is.null(add.params$fill)) add.params$fill <- fill
  if (is.null(add.params$color)) add.params$color <- color
  if (is.null(add.params$fill) & ("crossbar" %in% error.plot | "boxplot" %in% add | "violin" %in% add)) add.params$fill <- fill
  if (is.null(add.params$fill)) add.params$fill <- add.params$color
  # else add.params$fill <- add.params$color
  if (!is.null(list(...)$shape) & is.null(add.params$shape)) add.params$shape <- list(...)$shape
  add.params
}


# Calculate the mean and the SD in each group
#+++++++++++++++++++++++++
# data : a data frame
# varname : the name of the variable to be summariezed
# grps : column names to be used as grouping variables
# .mean_sd <- function(data, varname, grps){
#   summary_func <- function(x, col){
#     c(mean = base::mean(x[[col]], na.rm=TRUE),
#       sd = stats::sd(x[[col]], na.rm=TRUE))
#   }
#   data_sum <- plyr::ddply(data, grps, .fun=summary_func, varname)
#   data_sum$ymin <- data_sum$mean-data_sum$sd
#   data_sum$ymax <- data_sum$mean+data_sum$sd
#   names(data_sum)[ncol(data_sum)-3] <- varname
#   # data_sum <- plyr::rename(data_sum, c("mean" = varname))
#   return(data_sum)
# }


# Summary functions
.summary_functions <- function() {
  c(
    "mean", "mean_se", "mean_se_", "mean_sd", "mean_ci",
    "mean_range", "median", "median_iqr", "median_hilow", "median_hilow_",
    "median_q1q3", "median_mad", "median_range"
  )
}
.errorbar_functions <- function() {
  setdiff(.summary_functions(), c("mean", "median"))
}


# parse font
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.parse_font <- function(font) {
  if (is.null(font)) {
    res <- NULL
  } else if (inherits(font, "list")) {
    res <- font
  } else {
    # matching size and face (supports integer, dot, or comma decimals)
    size_idx <- grep("^[0-9]+([\\.,][0-9]+)?$", font, perl = TRUE)
    face_idx <- grep("plain|bold|italic|bold.italic", font, perl = TRUE)
    if (length(size_idx) == 0) {
      size <- NULL
    } else {
      size_vals <- font[size_idx]
      size_vals <- gsub(",", ".", size_vals, fixed = TRUE)
      size <- as.numeric(size_vals)
    }
    if (length(face_idx) == 0) face <- NULL else face <- font[face_idx]
    color_idx <- setdiff(seq_along(font), c(size_idx, face_idx))
    if (length(color_idx) == 0) color <- NULL else color <- font[color_idx]
    res <- list(size = size, face = face, color = color)
  }
  res
}


# Add annotation to a plot
# label: text to be added to a plot
# size: text size
# coord: x and coordinates
.ggannotate <- function(label, size = 12, coord = c(NULL, NULL)) {
  if (is.null(unique(coord))) {
    grob <- grid::grobTree(grid::textGrob(label,
      x = 0.3, y = 0.80, hjust = 0,
      gp = grid::gpar(col = "black", fontsize = size, fontface = "plain")
    ))
    ggplot2::annotation_custom(grob)
  } else {
    ggplot2::annotate("text",
      x = coord[1], y = coord[2],
      label = label, size = size / 3
    )
  }
}

# :::::::::::::::::::::::::::::::::::::::::
# Check the data provided by user
# :::::::::::::::::::::::::::::::::::::::::
# combine: if TRUE, gather y variables
# return a list(data, x, y)
.check_data <- function(data, x, y, combine = FALSE) {
  if (missing(x) & missing(y)) {
    if (!is.numeric(data)) {
      stop("x and y are missing. In this case data should be a numeric vector.")
    } else {
      data <- data.frame(y = data, x = rep(1, length(data)))
      x <- "x"
      y <- "y"
    }
  } else if (missing(x)) {
    x <- "x"
    if (is.numeric(data)) {
      data <- data.frame(x = data)
    } else {
      data$x <- rep("1", nrow(data))
    }
  }
  # A list of y elements to plot
  else if (length(y) > 1) {
    if (!all(y %in% colnames(data))) {
      not_found <- setdiff(y, colnames(data))
      y <- intersect(y, colnames(data))

      if (.is_empty(y)) {
        stop("Can't find the y elements in the data.")
      } else if (!.is_empty(not_found)) {
        warning(
          "Can't find the following element in the data: ",
          .collapse(not_found)
        )
      }
    }
  }

  if (inherits(data, c("tbl_df", "tbl"))) {
    data <- as.data.frame(data)
  }

  # Combining y variables
  # ......................................................
  if (is.null(y)) y <- ""
  if (combine & length(y) > 1) {
    data <- data %>%
      df_gather(cols = y, names_to = ".y.", values_to = ".value.") %>%
      dplyr::mutate(.y. = factor(.data$.y., levels = unique(.data$.y.)))
    y <- ".value."
  }
  # Combining x variables: Case of density plot or histograms
  # ......................................................
  else if (combine & length(x) > 1 & y[1] %in% c("..density..", "..count..", "..ecdf..", "..qq..")) {
    data <- data %>%
      df_gather(cols = x, names_to = ".y.", values_to = ".value.") %>%
      dplyr::mutate(.y. = factor(.data$.y., levels = unique(.data$.y.)))
    x <- ".value."
  }

  # If not factor, x elements on the plot should
  # appear in the same order as in the data
  if (is.character(data[[x]])) {
    data[[x]] <- factor(data[[x]], levels = unique(data[[x]]))
  }

  y <- unique(y)
  names(y) <- y
  x <- unique(x)
  names(x) <- x

  if (y[1] %in% c("..density..", "..count..", "..ecdf..", "..qq..")) {
    list(x = x, data = data, y = y)
  } # The name of plots are x variables
  else {
    list(y = y, data = data, x = x)
  } # The name of plots will be y variables
}


# Adjust shape when ngroups > 6, to avoid ggplot warnings
.scale_point_shape <- function(p, data, shape) {
  if (shape %in% colnames(data)) {
    grp <- data[[shape]]
    if (!inherits(grp, "factor")) grp <- as.factor(grp)
    ngroups <- length(levels(data[[shape]]))
    if (ngroups > 6) p <- p + scale_shape_manual(values = seq_len(ngroups), labels = levels(data[[shape]]))
  }
  p
}

# Get not numeric columns in a data.frame
.get_not_numeric_vars <- function(data_frame) {
  is_numeric <- sapply(data_frame, is.numeric)
  if (sum(!is_numeric) == 0) {
    res <- NULL
  } else {
    res <- colnames(data_frame[, !is_numeric, drop = FALSE])
  }
  res
}


# Get the current color used in ggplot
.get_ggplot_ncolors <- function(p) {
  g <- ggplot_build(p)
  gdata <- g$data[[1]]
  cols <- fills <- 1
  if ("colour" %in% names(gdata)) cols <- unique(unlist(gdata["colour"]))
  if ("fills" %in% names(gdata)) fills <- unique(unlist(gdata["fill"]))
  max(length(cols), length(fills))
}

# Check if character string is a valid color representation
.is_color <- function(x) {
  sapply(x, function(X) {
    tryCatch(is.matrix(grDevices::col2rgb(X)),
      error = function(e) FALSE
    )
  })
}


# Collapse one or two vectors
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.collapse <- function(x, y = NULL, sep = ".") {
  if (missing(y)) {
    paste(x, collapse = sep)
  } else if (is.null(x) & is.null(y)) {
    return(NULL)
  } else if (is.null(x)) {
    return(as.character(y))
  } else if (is.null(y)) {
    return(as.character(x))
  } else {
    paste0(x, sep, y)
  }
}

# Check if en object is empty
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.is_empty <- function(x) {
  length(x) == 0
}

# Remove NULL items in a vector or list
#
# x a vector or list
.compact <- function(x) {
  Filter(Negate(is.null), x)
}

# Check if is a list
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.is_list <- function(x) {
  inherits(x, "list")
}

# Returns the levels of a factor variable
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.levels <- function(x) {
  if (!is.factor(x)) x <- as.factor(x)
  levels(x)
}

# Remove items from a list
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.remove_item <- function(.list, items) {
  for (item in items) {
    .list[[item]] <- NULL
  }
  .list
}

# Additems in a list
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.add_item <- function(.list, ...) {
  pms <- list(...)
  for (pms.names in names(pms)) {
    .list[[pms.names]] <- pms[[pms.names]]
  }
  .list
}


# Select a colun as vector from tiblle data frame
.select_vec <- function(df, column) {
  dplyr::pull(df, column)
}

# Select the top up or down rows of a data frame sorted by variables
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# - df: data frame
# - x: x axis variables (grouping variables)
# - y: y axis variables (sorting variables)
# - n the number of rows
# - grps: other grouping variables
.top_up <- function(df, x, y, n, grouping.vars = NULL) {
  grouping.vars <- c(x, grouping.vars) %>%
    unique()
  df %>%
    df_arrange(vars = c(grouping.vars, y)) %>%
    df_group_by(vars = grouping.vars) %>%
    slice_tail(n = n)
}


.top_down <- function(df, x, y, n, grouping.vars = NULL) {
  grouping.vars <- c(x, grouping.vars) %>%
    unique()
  df %>%
    df_arrange(vars = c(grouping.vars, y)) %>%
    df_group_by(vars = grouping.vars) %>%
    slice_head(n = n)
}


# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Apply ggpubr functions on a data
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# fun: function, can be ggboxplot, ggdotplot, ggstripchart, ...
.plotter <- function(fun, data, x, y, combine = FALSE, merge = FALSE,
                     color = "black", fill = "white",
                     title = NULL, xlab = NULL, ylab = NULL,
                     legend = NULL, legend.title = NULL,
                     facet.by = NULL,
                     select = NULL, remove = NULL, order = NULL,
                     add = "none", add.params = list(),
                     label = NULL, font.label = list(size = 11, color = "black"),
                     label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                     font.family = "", parse = FALSE,
                     ggtheme = theme_pubr(),
                     fun_name = "", group = 1, # used only by ggline
                     show.legend.text = NA,
                     ...) {
  # Accept the British spelling `colour` as an alias for `color` (#317).
  # Map it to `color` and drop it from the extra args so it is not also passed
  # to the geom (which would raise a "Duplicated aesthetics" warning). `.extra`
  # replaces `...` in the downstream calls below.
  .extra <- list(...)
  if (!is.null(.extra$colour)) {
    color <- .extra$colour
    .extra$colour <- NULL
  }
  if (is.logical(merge)) {
    if (merge) {
      merge <- "asis"
    } else {
      merge <- "none"
    }
  }
  if (combine & merge != "none") {
    stop("You should use either combine = TRUE or merge = TRUE, but not both together.")
  }

  font.label <- .parse_font(font.label)
  if (is.null(label) & fun_name == "barplot") label <- FALSE
  .lab <- label
  if (fun_name != "barplot") .lab <- NULL

  if (!missing(x) & !missing(y)) {
    if (length(y) == 1 & length(x) == 1) {
      combine <- FALSE
      merge <- "none"
    }
  }

  # Check data
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # - returns a list of updated main options:
  #       list(y, data,  x)
  opts <- .check_data(data, x, y, combine = combine | merge != "none")
  data <- opts$data
  x <- opts$x
  y <- opts$y


  is_density_plot <- y[1] %in% c("..count..", "..density..", "..ecdf..", "..qq..")

  if (combine) facet.by <- ".y." # Faceting by y variables
  if (merge != "none") {
    if (!is_density_plot) facet.by <- NULL
    if (is.null(legend.title)) legend.title <- "" # remove .y. in the legend
  }


  # Updating parameters after merging
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::

  # Special case for density and histograms:
  # x are variables and y is ..count.. or ..density..
  # after merging ggpubr add a new column .y. which hold x variables
  # User might want to color by x variables as follows: color = ".x." and
  # he aren't aware that the column is ".y." --> so we should translate this (see from line 1055)

  user.add.color <- add.params$color
  geom.text.position <- "identity"

  if (merge == "asis") {
    .grouping.var <- ".y." # y variables become grouping variable
  } else if (merge == "flip") {
    .grouping.var <- opts$x # x variable becomes grouping variable
    opts$x <- ".y." # y variables become x tick labels
    if (is.null(xlab)) xlab <- FALSE
  }

  if (merge == "asis" | merge == "flip") {
    if (is_density_plot) {
      color <- ifelse(color == ".x.", ".y.", color)
      fill <- ifelse(fill == ".x.", ".y.", fill)
    }

    if (any(c(color, fill) %in% names(data))) {
      add.params$color <- font.label$color <- ifelse(color %in% names(data), color, fill)
    } else if (!all(c(color, fill) %in% names(data))) {
      color <- add.params$color <- font.label$color <- .grouping.var
      # fill <- "white"
    }
    group <- .grouping.var
    geom.text.position <- position_dodge(0.8)
  }

  if (!combine & merge == "none" & length(opts$y) > 1 & is.null(title)) {
    title <- opts$y
  }

  if (!combine & merge == "none" & is.null(title)) {
    if (length(opts$y) > 1) {
      title <- opts$y
    } else if (length(opts$x) > 1 & is_density_plot) { # case of density plot
      title <- opts$x
    }
  }

  # Item to display
  x <- opts$data[[opts$x]] %>% as.vector()
  if (!is.null(select)) {
    opts$data <- subset(opts$data, x %in% select)
  }
  if (!is.null(remove)) {
    opts$data <- subset(opts$data, !(x %in% remove))
  }
  if (!is.null(order)) opts$data[[opts$x]] <- factor(opts$data[[opts$x]], levels = order)

  # Add additional options, which can be potentially vectorized
  # when multiple plots
  opts <- opts %>%
    c(list(title = title, xlab = xlab, ylab = ylab)) %>%
    .compact()
  data <- opts$data
  opts$data <- list(opts$data)
  if (fun_name %in% c("ggline", "ggdotchart")) opts$group <- group
  # Plotting
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Apply function to each y variables
  p <- do.call(purrr::pmap, c(
    list(
      .l = opts, .f = fun,
      color = color, fill = fill, legend = legend,
      legend.title = legend.title, ggtheme = ggtheme, facet.by = facet.by,
      add = add, add.params = add.params,
      # group = group, # for line plot
      user.add.color = user.add.color,
      label = .lab, # used only in ggbarplot
      font.label = font.label, repel = repel, label.rectangle = label.rectangle
    ),
    .extra
  ))
  # Faceting
  if (!is.null(facet.by)) {
    p <- do.call(purrr::map, c(list(.x = p, .f = facet, facet.by = facet.by), .extra))
  }


  # Add labels
  if (!is.null(label) & fun_name != "barplot") {
    if (is.logical(label)) {
      if (label) label <- opts$y
    }

    grouping.vars <- intersect(c(facet.by, color, fill), colnames(data))

    label.opts <- font.label %>%
      .add_item(
        data = data, x = opts$x, y = opts$y,
        label = label, label.select = label.select,
        repel = repel, label.rectangle = label.rectangle,
        family = font.family, parse = parse,
        ggtheme = NULL,
        grouping.vars = grouping.vars, facet.by = facet.by, position = geom.text.position,
        show.legend = show.legend.text
      )
    p <- purrr::map(
      p,
      function(p, label.opts) {
        . <- NULL
        label.opts %>%
          .add_item(ggp = p) %>%
          do.call(ggtext, .)
      },
      label.opts
    )
  }

  # Take into account the legend argument, when the main plot has no legend and ggtext has legend
  p <- purrr::map(p, ggpar, legend = legend, legend.title = legend.title)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  p
}


# get the geometry of the first layer
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.geom <- function(p, .layer = 1) {
  . <- NULL
  if (is.null(p) | .is_empty(p$layers)) {
    return("")
  }
  class(p$layers[[.layer]]$geom)[1] %>%
    tolower() %>%
    gsub("geom", "", .)
}


# Get the mapping variables of the first layer
# :::::::::::::::::::::::::::::::::::::::::::::::::
.mapping <- function(p) {
  if (is.null(p)) {
    return(list())
  }

  . <- NULL

  layer0.mapping <- as.character(p$mapping) %>% gsub("~", "", .)
  layer0.mapping.labels <- p$mapping %>% names()
  names(layer0.mapping) <- layer0.mapping.labels

  layer1.mapping <- NULL

  if (!.is_empty(p$layers)) {
    layer1.mapping <- p$layers[[1]]$mapping %>%
      as.character() %>%
      gsub("~", "", .)
    layer1.mapping.labels <- p$layers[[1]]$mapping %>%
      names()
    names(layer1.mapping) <- layer1.mapping.labels
  }

  c(layer0.mapping, layer1.mapping) %>%
    as.list()
}

# Call geom_exec function to update a plot
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.update_plot <- function(opts, p) {
  p + do.call(geom_exec, opts)
}


# Get ggplot2 x and y variable
# ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.get_gg_xy_variables <- function(p) {
  . <- NULL
  x <- p$mapping["x"] %>%
    as.character() %>%
    gsub("~", "", .)
  y <- p$mapping["y"] %>%
    as.character() %>%
    gsub("~", "", .)
  xy <- c(x, y)
  names(xy) <- c("x", "y")
  return(xy)
}

# Add mean or median line
# used by ggdensity and gghistogram
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# p: main plot
# data: data frame
# x: measure variables
# add: center to add
# grouping.vars: grouping variables
.add_center_line <- function(p, add = c("none", "mean", "median"), grouping.vars = NULL,
                             color = "black", linetype = "dashed", size = NULL, linewidth = NULL) {
  # Handle size vs linewidth parameter compatibility
  # size deprecated in ggplot2 v >= 3.4.0
  if (!is.null(linewidth)) {
    size <- linewidth
  }
  add <- match.arg(add)
  data <- p$data
  # x <- .mapping(p)$x
  .mapping <- .get_gg_xy_variables(p)
  x <- .mapping["x"]

  if (!(add %in% c("mean", "median"))) {
    return(p)
  }
  compute_center <- switch(add,
    mean = mean,
    median = stats::median
  )

  # NO grouping variable
  if (.is_empty(grouping.vars)) {
    m <- ifelse(add == "mean",
      mean(data[[x]], na.rm = TRUE),
      stats::median(data[[x]], na.rm = TRUE)
    )
    # geom_vline takes `linewidth`, not the deprecated `size` (ggplot2 >= 3.4.0).
    # Only pass it when set, so the default width is used otherwise (avoids an
    # "empty aesthetic" warning when no width is supplied).
    width_opt <- if (!is.null(size)) list(linewidth = size) else list()
    p <- p + do.call(
      ggplot2::geom_vline,
      c(
        list(
          xintercept = m, color = color,
          linetype = linetype, inherit.aes = FALSE
        ),
        width_opt
      )
    )
  }
  # Case of grouping variable
  else {
    data_sum <- data %>%
      group_by(!!!syms(grouping.vars)) %>%
      summarise(.center = compute_center(!!sym(x), na.rm = TRUE))
    p <- p + geom_exec(geom_vline,
      data = data_sum,
      xintercept = ".center", color = color,
      linetype = linetype, linewidth = size,
      inherit.aes = FALSE
    )
  }

  p
}


# Check legend argument
.check_legend <- function(legend) {
  allowed.values <- c("top", "bottom", "left", "right", "none")

  if (is.null(legend) | is.numeric(legend)) {
    return(legend)
  } else if (is.logical(legend)) {
    if (legend) {
      legend <- "top"
    } else {
      legend <- "none"
    }
  } else if (is.character(legend)) {
    legend <- legend[1]
    if (!legend %in% allowed.values) {
      stop("Argument legend should be one of ", .collapse(allowed.values, sep = ", "))
    }
  }
  return(legend)
}


# show.n: per-group observation counts on box/violin/strip charts (#598, #627)
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Dodge width implied by a position (NULL = the layer is not dodged).
.dodge_width_of <- function(position) {
  if (is.null(position)) return(NULL)
  if (is.character(position)) {
    if (position %in% c("dodge", "dodge2")) return(0.8)
    if (position == "jitterdodge") return(0.75)
    return(NULL)
  }
  if (inherits(position, "PositionJitterdodge")) return(position$dodge.width %||% 0.75)
  if (inherits(position, c("PositionDodge2", "PositionDodge"))) return(position$width %||% 0.8)
  NULL
}

# Facet variables actually used by the plot. Reads them from the facet object so
# the internal `.y.` panel of combine = TRUE is handled like any other facet.
.plot_facet_vars <- function(p) {
  f <- p$facet
  if (is.null(f) || inherits(f, "FacetNull")) return(character(0))
  pr <- f$params
  unique(c(names(pr$facets), names(pr$rows), names(pr$cols)))
}

# Add "n = <count>" labels at the top of each panel, one per group. Counts are
# taken from the plot's own data (so select/remove/order and combine's reshaped
# data are respected). When the groups are dodged (a dodging position + a
# colour/fill grouping), the labels dodge to sit above each box; otherwise a
# single count per x tick is shown. Opt-in via show.n = TRUE; default output is
# unchanged.
.add_show_n <- function(p, x, color = NULL, fill = NULL, facet.by = NULL,
                        position = NULL, size = 3.2, vjust = 1.4) {
  d <- p$data
  if (is.null(d) || !(x %in% colnames(d))) return(p)
  group <- intersect(unique(c(color, fill)), colnames(d))
  facet.vars <- intersect(unique(c(.plot_facet_vars(p), facet.by)), colnames(d))
  dodge.w <- .dodge_width_of(position)
  dodged <- length(group) > 0 && !is.null(dodge.w)
  keys <- if (dodged) unique(c(facet.vars, x, group)) else unique(c(facet.vars, x))
  keys <- intersect(keys, colnames(d))
  cnt <- d %>%
    dplyr::group_by(dplyr::across(dplyr::all_of(keys))) %>%
    dplyr::summarise(.n = dplyr::n(), .groups = "drop") %>%
    as.data.frame()
  if (nrow(cnt) == 0) return(p)
  cnt$.n. <- paste0("n = ", cnt$.n)
  # y = Inf pins the label to the top of each panel (facet-robust); the .data
  # pronoun keeps non-syntactic column names (spaces/hyphens) working.
  if (dodged) {
    # dodge by the interaction of all grouping columns, matching the boxes.
    # Use a collision-proof temp column name (never a real data column).
    grp.col <- ".ggpubr_show_n_grp."
    cnt[[grp.col]] <- interaction(cnt[group], drop = TRUE, lex.order = TRUE)
    mapping <- ggplot2::aes(
      x = .data[[x]], y = Inf, label = .data$.n., group = .data[[grp.col]]
    )
    pos <- ggplot2::position_dodge(width = dodge.w)
  } else {
    mapping <- ggplot2::aes(x = .data[[x]], y = Inf, label = .data$.n.)
    pos <- "identity"
  }
  p + ggplot2::geom_text(
    data = cnt, mapping = mapping, position = pos, vjust = vjust,
    size = size, colour = "black", inherit.aes = FALSE, show.legend = FALSE
  )
}

# Regression tests for #333: when BOTH xticks.by and yticks.by were supplied,
# .set_ticksby() used an `else if`, so xticks.by was silently ignored.

.x_breaks <- function(p) {
  b <- ggplot2::ggplot_build(p)
  stats::na.omit(b$layout$panel_params[[1]]$x$breaks)
}
.y_breaks <- function(p) {
  b <- ggplot2::ggplot_build(p)
  stats::na.omit(b$layout$panel_params[[1]]$y$breaks)
}

test_that("xticks.by and yticks.by both apply when supplied together (#333)", {
  p <- ggscatter(iris, x = "Sepal.Length", y = "Sepal.Width",
                 xticks.by = 0.5, yticks.by = 0.5)
  xb <- .x_breaks(p)
  yb <- .y_breaks(p)
  # 0.5 spacing on both axes
  expect_true(all(abs(diff(xb) - 0.5) < 1e-8))
  expect_true(all(abs(diff(yb) - 0.5) < 1e-8))
})

test_that("xticks.by alone is unchanged (no regression, #333)", {
  p_both  <- ggscatter(iris, x = "Sepal.Length", y = "Sepal.Width", xticks.by = 0.5)
  xb <- .x_breaks(p_both)
  expect_true(all(abs(diff(xb) - 0.5) < 1e-8))
})

test_that("yticks.by alone is unchanged (no regression, #333)", {
  p <- ggscatter(iris, x = "Sepal.Length", y = "Sepal.Width", yticks.by = 0.5)
  yb <- .y_breaks(p)
  expect_true(all(abs(diff(yb) - 0.5) < 1e-8))
  # x axis keeps ggplot2 default breaks (integers here), i.e. NOT 0.5-spaced
  xb <- .x_breaks(p)
  expect_false(isTRUE(all(abs(diff(xb) - 0.5) < 1e-8)))
})

test_that("xticks.by on a discrete x axis does not error, yticks.by still applies (#333)", {
  # ggboxplot has a discrete x: applying a continuous x scale would error, so
  # xticks.by is skipped (as it was silently before), while yticks.by still
  # works. Must RENDER (draw-time), not just build.
  p <- ggboxplot(ToothGrowth, x = "supp", y = "len",
                 xticks.by = 1, yticks.by = 5)
  expect_no_error(grob <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  expect_s3_class(grob, "gtable")
  yb <- .y_breaks(p)
  expect_true(all(abs(diff(yb) - 5) < 1e-8))
})

test_that(".is_discrete_x detects discrete vs continuous x (#333)", {
  # numeric x -> continuous -> not discrete
  expect_false(ggpubr:::.is_discrete_x(
    ggscatter(iris, x = "Sepal.Length", y = "Sepal.Width")))
  # factor x -> discrete
  expect_true(ggpubr:::.is_discrete_x(
    ggboxplot(ToothGrowth, x = "supp", y = "len")))
  # Date x is continuous (numeric storage) -> not discrete, so xticks.by still
  # applies exactly as in previous versions.
  df <- data.frame(d = as.Date("2020-01-01") + 0:9, y = seq_len(10))
  pd <- ggplot2::ggplot(df, ggplot2::aes(x = d, y = y)) + ggplot2::geom_point()
  expect_false(ggpubr:::.is_discrete_x(pd))
})

test_that(".parse_font parses decimal sizes with dot or comma", {
  res <- ggpubr:::.parse_font(c("2.4", "italic", "black"))
  expect_equal(res$size, 2.4)
  expect_equal(res$face, "italic")
  expect_equal(res$color, "black")

  old <- options(OutDec = ",")
  on.exit(options(old), add = TRUE)

  res_locale <- ggpubr:::.parse_font(c(2.4, "bold", "red"))
  expect_equal(res_locale$size, 2.4)
  expect_equal(res_locale$face, "bold")
  expect_equal(res_locale$color, "red")

  res_comma <- ggpubr:::.parse_font(c("2,4", "plain", "blue"))
  expect_equal(res_comma$size, 2.4)
  expect_equal(res_comma$face, "plain")
  expect_equal(res_comma$color, "blue")
})

# Gene expression data
df_expr <- tibble::tribble(
  ~sample, ~dataset, ~GATA3, ~PTEN, ~XBP1,
  "ID.1", "BRCA.mRNA", 2.23, 0.77, 2.91,
  "ID.2", "BRCA.mRNA", 1.24, 1.32, 1.68,
  "ID.3", "OV.mRNA", -3.44, 1.65, -1.29,
  "ID.4", "OV.mRNA", -5.21, 1.11, -0.62,
  "ID.5", "LUSC.mRNA", -4.01, -0.05, -0.57,
  "ID.6", "LUSC.mRNA", -3.29, 0.3, -0.02
)


test_that("check_data works when combine=TRUE and length(y) > 1", {
  observed <- .check_data(df_expr, x = "dataset", y = c("GATA3", "PTEN", "XBP1"), combine = TRUE)
  expected_y <- c(".value." = ".value.")
  expected_x <- c("dataset" = "dataset")
  expected_data <- tibble::tribble(
    ~sample, ~dataset, ~.y., ~.value.,
    "ID.1", "BRCA.mRNA", "GATA3", 2.23,
    "ID.2", "BRCA.mRNA", "GATA3", 1.24,
    "ID.3", "OV.mRNA", "GATA3", -3.44,
    "ID.4", "OV.mRNA", "GATA3", -5.21,
    "ID.5", "LUSC.mRNA", "GATA3", -4.01,
    "ID.6", "LUSC.mRNA", "GATA3", -3.29,
    "ID.1", "BRCA.mRNA", "PTEN", 0.77,
    "ID.2", "BRCA.mRNA", "PTEN", 1.32,
    "ID.3", "OV.mRNA", "PTEN", 1.65,
    "ID.4", "OV.mRNA", "PTEN", 1.11,
    "ID.5", "LUSC.mRNA", "PTEN", -0.05,
    "ID.6", "LUSC.mRNA", "PTEN", 0.3,
    "ID.1", "BRCA.mRNA", "XBP1", 2.91,
    "ID.2", "BRCA.mRNA", "XBP1", 1.68,
    "ID.3", "OV.mRNA", "XBP1", -1.29,
    "ID.4", "OV.mRNA", "XBP1", -0.62,
    "ID.5", "LUSC.mRNA", "XBP1", -0.57,
    "ID.6", "LUSC.mRNA", "XBP1", -0.02
  ) %>%
    dplyr::mutate(
      .y. = factor(.y., levels = c("GATA3", "PTEN", "XBP1")),
      dataset = factor(dataset, levels = c("BRCA.mRNA", "OV.mRNA", "LUSC.mRNA"))
    )

  expect_equal(observed$y, expected_y)
  expect_equal(observed$x, expected_x)
  expect_equal(observed$data, expected_data)
})


test_that("check_data works when combine=TRUE and length(x) > 1", {
  observed <- .check_data(df_expr, x = c("GATA3", "PTEN", "XBP1"), y = "..density..", combine = TRUE)
  expected_y <- c("..density.." = "..density..")
  expected_x <- c(".value." = ".value.")
  expected_data <- tibble::tribble(
    ~sample, ~dataset, ~.y., ~.value.,
    "ID.1", "BRCA.mRNA", "GATA3", 2.23,
    "ID.2", "BRCA.mRNA", "GATA3", 1.24,
    "ID.3", "OV.mRNA", "GATA3", -3.44,
    "ID.4", "OV.mRNA", "GATA3", -5.21,
    "ID.5", "LUSC.mRNA", "GATA3", -4.01,
    "ID.6", "LUSC.mRNA", "GATA3", -3.29,
    "ID.1", "BRCA.mRNA", "PTEN", 0.77,
    "ID.2", "BRCA.mRNA", "PTEN", 1.32,
    "ID.3", "OV.mRNA", "PTEN", 1.65,
    "ID.4", "OV.mRNA", "PTEN", 1.11,
    "ID.5", "LUSC.mRNA", "PTEN", -0.05,
    "ID.6", "LUSC.mRNA", "PTEN", 0.3,
    "ID.1", "BRCA.mRNA", "XBP1", 2.91,
    "ID.2", "BRCA.mRNA", "XBP1", 1.68,
    "ID.3", "OV.mRNA", "XBP1", -1.29,
    "ID.4", "OV.mRNA", "XBP1", -0.62,
    "ID.5", "LUSC.mRNA", "XBP1", -0.57,
    "ID.6", "LUSC.mRNA", "XBP1", -0.02
  ) %>%
    dplyr::mutate(.y. = factor(.y., levels = c("GATA3", "PTEN", "XBP1")))

  expect_equal(observed$y, expected_y)
  expect_equal(observed$x, expected_x)
  expect_equal(observed$data, expected_data)
})

#' @include utilities.R utilities_label.R utils_stat_test_label.R
#' @importFrom dplyr everything
#' @importFrom dplyr select
NULL

# stat_regline_equation() was adapted from the source code of ggpmisc::stat_poly_eq()
# by Pedro J. Aphalo. See ggpmisc::stat_poly_eq() as an alternative with additional
# features.

#' Add Regression Line Equation and R-Square to a GGPlot.
#' @description Add regression line equation and R^2 to a ggplot. Regression
#'  model is fitted using the function \code{\link[stats]{lm}}.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @param formula a formula object
#' @param label.x.npc,label.y.npc can be \code{numeric} or \code{character}
#'  vector of the same length as the number of groups and/or panels. If too
#'  short they will be recycled. \itemize{ \item If \code{numeric}, value should
#'  be between 0 and 1. Coordinates to be used for positioning the label,
#'  expressed in "normalized parent coordinates". \item If \code{character},
#'  allowed values include: i) one of c('right', 'left', 'center', 'centre',
#'  'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
#'  'middle') for y-axis.}
#'
#'  If too short they will be recycled.
#' @param label.x,label.y \code{numeric} Coordinates (in data units) to be used
#'  for absolute positioning of the label. If too short they will be recycled.
#' @param label.y.step numeric value giving the vertical spacing (in text-line
#'  units) between the labels of successive groups. Default is 1.4 (unchanged
#'  behavior). Set \code{label.y.step = 0} to stop the per-group vertical shift so
#'  labels align across facet panels.
#' @param label.anchor character. How \code{label.x.npc}/\code{label.y.npc} are
#'  interpreted. \code{"data"} (default) converts them to data coordinates (the
#'  previous behavior). \code{"panel"} places the label at the true panel-relative
#'  position (npc), so labels stay aligned across panels/facets with different
#'  axis ranges (e.g. \code{scales = "free_y"} or \code{geom_smooth()} extending
#'  each panel). An explicit \code{label.x}/\code{label.y} always stays in data
#'  units. Requires ggplot2 >= 3.5.0 (already a dependency).
#' @param output.type character One of "expression", "latex" or "text".
#' @param decreasing logical. If \code{TRUE} (the default), the equation is
#'   formatted in standard mathematical convention with terms in decreasing
#'   order of powers (e.g., "y = 2*x + 1"). If \code{FALSE}, terms are in
#'   increasing order (e.g., "y = 1 + 2*x").
#' @param coef.digits integer indicating the number of significant digits to use
#'   for the regression equation coefficients. Default is 2.
#' @param rr.digits integer indicating the number of significant digits to use
#'   for R2 and adjusted R2. Default is 2.
#' @param ... other arguments to pass to \code{\link[ggplot2]{geom_text}} or
#'  \code{\link[ggplot2:geom_text]{geom_label}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning. If
#'  TRUE silently removes missing values.
#' @seealso \code{\link{ggscatter}}
#' @references \code{stat_regline_equation()} was adapted from the source code of
#'  \code{stat_poly_eq()} in the 'ggpmisc' package by Pedro J. Aphalo. See
#'  \code{ggpmisc::stat_poly_eq()} as an alternative with additional features.
#'
#' @section Computed variables:
#'   \describe{ \item{x}{x position for left edge}
#'   \item{y}{y position near upper edge}
#'   \item{eq.label}{equation for the
#'   fitted polynomial as a character string to be parsed}
#'   \item{rr.label}{\eqn{R^2} of the fitted model as a character string to be parsed}
#'   \item{adj.rr.label}{Adjusted \eqn{R^2} of the fitted model as a character string
#'   to be parsed}
#'   \item{AIC.label}{AIC for the fitted model.}
#'   \item{BIC.label}{BIC for the fitted model.}
#'   \item{hjust}{Set to zero to override the default of the "text" geom.}}
#' @examples
#'
#' # Simple scatter plot with correlation coefficient and
#' # regression line
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::::
#' ggscatter(mtcars, x = "wt", y = "mpg", add = "reg.line") +
#'   stat_cor(label.x = 3, label.y = 34) +
#'   stat_regline_equation(label.x = 3, label.y = 32)
#'
#'
#' # Groupped scatter plot
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::::
#' ggscatter(
#'   iris,
#'   x = "Sepal.Length", y = "Sepal.Width",
#'   color = "Species", palette = "jco",
#'   add = "reg.line"
#' ) +
#'   facet_wrap(~Species) +
#'   stat_cor(label.y = 4.4) +
#'   stat_regline_equation(label.y = 4.2)
#'
#' # Polynomial equation
#' # ::::::::::::::::::::::::::::::::::::::::::::::::::::
#'
#' # Demo data
#' set.seed(4321)
#' x <- 1:100
#' y <- (x + x^2 + x^3) + rnorm(length(x), mean = 0, sd = mean(x^3) / 4)
#' my.data <- data.frame(x, y,
#'   group = c("A", "B"),
#'   y2 = y * c(0.5, 2), block = c("a", "a", "b", "b")
#' )
#'
#' # Fit polynomial regression line and add labels
#' formula <- y ~ poly(x, 3, raw = TRUE)
#' p <- ggplot(my.data, aes(x, y2, color = group)) +
#'   geom_point() +
#'   stat_smooth(aes(fill = group, color = group), method = "lm", formula = formula) +
#'   stat_regline_equation(
#'     aes(label = paste(..eq.label.., ..adj.rr.label.., sep = "~~~~")),
#'     formula = formula
#'   ) +
#'   theme_bw()
#' ggpar(p, palette = "jco")
#'
#' @export
stat_regline_equation <- function(
  mapping = NULL, data = NULL, formula = y ~ x,
  label.x.npc = "left", label.y.npc = "top",
  label.x = NULL, label.y = NULL, label.y.step = 1.4,
  label.anchor = c("data", "panel"),
  output.type = "expression", decreasing = TRUE,
  coef.digits = 2, rr.digits = 2,
  geom = "text", position = "identity", na.rm = FALSE, show.legend = NA,
  inherit.aes = TRUE, ...
) {
  parse <- ifelse(output.type == "expression", TRUE, FALSE)
  label.anchor <- match.arg(label.anchor)
  # Convert any dot-dot notation in user-provided mapping
  mapping <- convert_label_dotdot_notation_to_after_stat(mapping)

  layer(
    stat = StatReglineEquation, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      formula = formula, label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y, label.y.step = label.y.step,
      label.anchor = label.anchor,
      output.type = output.type, decreasing = decreasing,
      coef.digits = coef.digits, rr.digits = rr.digits,
      parse = parse, na.rm = na.rm, ...
    )
  )
}


StatReglineEquation <- ggproto("StatReglineEquation", Stat,
  required_aes = c("x", "y"),
  default_aes = aes(label = after_stat(eq.label), hjust = after_stat(hjust), vjust = after_stat(vjust)),
  compute_group = function(data, scales, formula, label.x.npc, label.y.npc,
                           label.x, label.y, label.y.step = 1.4, label.anchor = "data",
                           output.type, decreasing,
                           coef.digits = 2, rr.digits = 2) {
    force(data)

    if (length(unique(data$x)) < 2) {
      return(data.frame()) # Not enough data to perform test
    }

    .test <- .stat_lm(formula, data, output.type = output.type, decreasing = decreasing,
                      coef.digits = coef.digits, rr.digits = rr.digits)
    # Returns a data frame with label: x, y, hjust, vjust
    .label.pms <- .label_params(
      data = data, scales = scales,
      label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y, label.y.step = label.y.step,
      label.anchor = label.anchor
    ) %>%
      mutate(hjust = 0)
    cbind(.test, .label.pms)
  }
)


# Compute regression line equation
.stat_lm <- function(formula, data, output.type = "expression", decreasing = TRUE,
                     coef.digits = 2, rr.digits = 2) {
  res.lm <- stats::lm(formula, data)
  coefs <- stats::coef(res.lm)

  # With an orthogonal poly() fit (the default, raw = FALSE), coef() returns
  # coefficients in the orthogonal basis, but the displayed equation treats them
  # as raw polynomial coefficients (c0 + c1 x + c2 x^2 + ...), giving a wrong
  # equation (#653). For a simple poly(x, k) term with an intercept, refit with
  # raw = TRUE - an identical fit - to obtain the correct raw coefficients for
  # display. R^2/AIC/BIC are unchanged (same fit) and stay from the original
  # model. .polynomial_raw_formula() returns NULL for cases that are not safe to
  # rewrite (no intercept, transformed poly argument, ...), leaving the original
  # coefficients untouched.
  raw.formula <- .polynomial_raw_formula(formula)
  if (!is.null(raw.formula)) {
    coefs <- tryCatch(
      stats::coef(stats::lm(raw.formula, data)),
      error = function(e) coefs
    )
  }

  formula.rhs.chr <- as.character(formula)[3]
  if (grepl("-1", formula.rhs.chr) || grepl("- 1", formula.rhs.chr)) {
    coefs <- c(0, coefs)
  }

  rr <- summary(res.lm)$r.squared %>% signif(rr.digits)
  adj.rr <- summary(res.lm)$adj.r.squared %>% signif(rr.digits)
  AIC <- stats::AIC(res.lm) %>% signif(2)
  BIC <- stats::BIC(res.lm) %>% signif(2)

  # Build model equation
  eq.char <- as.character(signif(polynom::as.polynomial(coefs), coef.digits), decreasing = decreasing)
  eq.char <- gsub("e([+-]?[0-9]*)", "%*%10^\\1", eq.char)
  if (output.type %in% c("latex", "tex", "tikz")) {
    eq.char <- gsub("*", " ", eq.char, fixed = TRUE)
  }
  # Add y
  if (output.type == "expression") {
    lhs <- "italic(y)~`=`~"
  } else if (output.type %in% c("latex", "tex", "tikz", "text")) {
    lhs <- "y = "
  }
  eq.char <- paste(lhs, eq.char, sep = "")

  # Build data frame with the output
  if (output.type == "expression") {
    eq.x.rhs <- "~italic(x)"
  } else {
    eq.x.rhs <- " x"
  }

  if (output.type == "expression") {
    z <- data.frame(
      eq.label = gsub("x", eq.x.rhs, eq.char, fixed = TRUE),
      rr.label = paste("italic(R)^2", rr, sep = "~`=`~"),
      adj.rr.label = paste("italic(R)[adj]^2",
        adj.rr,
        sep = "~`=`~"
      ),
      AIC.label = paste("AIC", AIC, sep = "~`=`~"),
      BIC.label = paste("BIC", BIC, sep = "~`=`~")
    )
  } else if (output.type %in% c("latex", "tex", "text")) {
    z <- data.frame(
      eq.label = gsub("x", eq.x.rhs, eq.char, fixed = TRUE),
      rr.label = paste("R^2", rr, sep = " = "),
      adj.rr.label = paste("R_{adj}^2", adj.rr, sep = " = "),
      AIC.label = paste("AIC", AIC, sep = " = "),
      BIC.label = paste("BIC", BIC, sep = " = ")
    )
  }

  z <- z %>%
    mutate(rr = rr, adj.rr = adj.rr, AIC = AIC, BIC = BIC) %>%
    dplyr::select(rr, adj.rr, AIC, BIC, everything())

  z
}

# If a formula fits an ORTHOGONAL polynomial - poly(x, k) or poly(x, k, raw =
# FALSE) - return an equivalent formula using raw = TRUE, whose coefficients are
# the raw polynomial coefficients suitable for display. Returns NULL when no
# rewrite is needed or cannot be done safely, in which case the original
# coefficients are used (unchanged behavior):
#   * no poly(), or poly() already raw = TRUE, or a raw form such as y ~ x /
#     y ~ x + I(x^2);
#   * a model without an intercept (- 1): an orthogonal poly() without an
#     intercept does NOT fit the same curve as its raw = TRUE counterpart, so
#     rewriting would not be an identical fit;
#   * a poly() whose first argument is itself a call, e.g. poly(log(x), 2): the
#     simple textual rewrite can't be applied reliably.
# Only a single, simple poly(<var>, ...) term is rewritten (#653).
.polynomial_raw_formula <- function(formula) {
  rhs <- paste(deparse(formula[[3]]), collapse = "")
  # Match one simple poly() call with no nested parentheses in its arguments.
  poly.call <- regmatches(rhs, regexpr("poly\\s*\\([^()]*\\)", rhs))
  if (length(poly.call) != 1) {
    return(NULL)
  }
  if (grepl("raw\\s*=\\s*TRUE", poly.call)) {
    return(NULL)
  }
  # No-intercept models: orthogonal vs raw poly are not the same fit.
  if (grepl("-\\s*1(\\b|$)", rhs)) {
    return(NULL)
  }
  if (grepl("raw\\s*=", poly.call)) {
    # explicit raw = FALSE (or other) -> force raw = TRUE
    new.call <- sub("raw\\s*=\\s*[^,)]+", "raw = TRUE", poly.call)
  } else {
    # no raw argument -> add it inside the poly() call
    new.call <- sub("\\)\\s*$", ", raw = TRUE)", poly.call)
  }
  rhs <- sub(poly.call, new.call, rhs, fixed = TRUE)
  tryCatch(
    stats::as.formula(
      paste(deparse(formula[[2]]), "~", rhs),
      env = environment(formula)
    ),
    error = function(e) NULL
  )
}

# Regression tests for #312: stat_regline_equation() hardcoded signif(., 2) for
# the equation coefficients and R^2. New coef.digits / rr.digits control the
# number of significant digits; defaults (2) reproduce the previous output.

.set <- function() {
  set.seed(1)
  data.frame(x = 1:20, y = -3.3462 + 1.02 * (1:20) + rnorm(20))
}
.eq_label <- function(d, aes.label = ggplot2::aes(label = ggplot2::after_stat(eq.label)), ...) {
  p <- ggplot2::ggplot(d, ggplot2::aes(x, y)) + ggplot2::geom_point() +
    stat_regline_equation(aes.label, ...)
  as.character(ggplot2::ggplot_build(p)$data[[2]]$label)
}
.rr_label <- function(d, ...) {
  p <- ggplot2::ggplot(d, ggplot2::aes(x, y)) + ggplot2::geom_point() +
    stat_regline_equation(ggplot2::aes(label = ggplot2::after_stat(rr.label)), ...)
  as.character(ggplot2::ggplot_build(p)$data[[2]]$label)
}

test_that("default digits are unchanged (no regression, #312)", {
  d <- .set()
  # default equals explicit coef.digits = 2 / rr.digits = 2
  expect_identical(.eq_label(d), .eq_label(d, coef.digits = 2))
  expect_identical(.rr_label(d), .rr_label(d, rr.digits = 2))
  # the default equation is the 2-sig-fig form (slope 1.0 hidden by polynom)
  expect_match(.eq_label(d), "3.4")
})

test_that("coef.digits increases equation coefficient precision (#312)", {
  d <- .set()
  lab3 <- .eq_label(d, coef.digits = 3)
  # slope becomes visible at 3 sig figs and intercept shows more digits
  expect_match(lab3, "1.04")
  expect_match(lab3, "3.38")
  expect_false(identical(.eq_label(d), lab3))
})

test_that("rr.digits controls R^2 precision (#312)", {
  d <- .set()
  expect_match(.rr_label(d, rr.digits = 4), "0.9789")
  expect_false(identical(.rr_label(d, rr.digits = 2), .rr_label(d, rr.digits = 4)))
})

# Regression tests for #653: stat_regline_equation() displayed the orthogonal
# poly() coefficients as if they were raw polynomial coefficients, giving a
# wrong equation. It now refits with raw = TRUE for display.

test_that("stat_regline_equation shows the correct raw polynomial for poly(x, 2) (#653)", {
  test_db <- data.frame(
    x = c(0, -0.1297, -0.2185, -0.2795),
    y = c(0.7569, 0.7396, 0.6561, 0.5476)
  )
  eq <- function(formula) {
    p <- ggplot2::ggplot(test_db, ggplot2::aes(x, y)) + ggplot2::geom_point() +
      stat_regline_equation(
        ggplot2::aes(label = ggplot2::after_stat(eq.label)),
        formula = formula, coef.digits = 3
      )
    as.character(ggplot2::ggplot_build(p)$data[[2]]$label)
  }
  # correct raw equation: y = -4.24 x^2 - 0.447 x + 0.756
  lab.orth <- eq(y ~ poly(x, 2))
  expect_match(lab.orth, "-4.24", fixed = TRUE)
  expect_match(lab.orth, "0.756", fixed = TRUE)
  # orthogonal poly() now matches raw = TRUE (same fitted curve)
  expect_identical(lab.orth, eq(y ~ poly(x, 2, raw = TRUE)))
  # the buggy orthogonal coefficient (~ -0.0716) must NOT appear
  expect_false(grepl("0.0716", lab.orth, fixed = TRUE))
})

test_that("stat_regline_equation linear/raw/I() equations are unchanged (#653)", {
  d <- .set()
  # y ~ x has no poly() -> untouched
  expect_match(.eq_label(d), "3.4")
  # helper: rewrite only orthogonal poly()
  expect_null(.polynomial_raw_formula(y ~ x))
  expect_null(.polynomial_raw_formula(y ~ poly(x, 2, raw = TRUE)))
  expect_null(.polynomial_raw_formula(y ~ x + I(x^2)))
  expect_equal(
    deparse(.polynomial_raw_formula(y ~ poly(x, 2))),
    "y ~ poly(x, 2, raw = TRUE)"
  )
  expect_equal(
    deparse(.polynomial_raw_formula(y ~ poly(x, 3, raw = FALSE))),
    "y ~ poly(x, 3, raw = TRUE)"
  )
})

test_that("stat_regline_equation skips unsafe poly rewrites (#653)", {
  # No-intercept: orthogonal poly() != raw poly() fit -> leave as-is
  expect_null(.polynomial_raw_formula(y ~ poly(x, 2) - 1))
  # Transformed poly argument: cannot rewrite reliably -> leave as-is
  expect_null(.polynomial_raw_formula(y ~ poly(log(x), 2)))
  # These still render without error (fall back to original coefficients)
  d <- data.frame(x = 1:20, y = (1:20)^2 + rnorm(20))
  p <- ggplot2::ggplot(d, ggplot2::aes(x, y)) + ggplot2::geom_point() +
    stat_regline_equation(formula = y ~ poly(x, 2) - 1)
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

#' @include utilities.R
NULL
#' Export ggplots
#' @description Export ggplots
#' @inheritParams ggarrange
#' @param ... list of plots to be arranged into the grid. The plots can be either
#'  ggplot2 plot objects, arbitrary gtables or an object of class
#'  \code{\link{ggarrange}}.
#' @param filename File name to create on disk.
#' @param width,height plot width and height, respectively (example, width = 800,
#'  height = 800). Applied only to raster plots: "png", "jpeg", "jpg", "bmp" and
#'  "tiff".
#' @param pointsize the default pointsize of plotted text (example, pointsize =
#'  8). Used only for raster plots.
#' @param res the resolution in ppi (example, res = 250). Used only for raster
#'  plots.
#' @param verbose logical. If TRUE, show message.
#' @author Laszlo Erdey <erdey.laszlo@@econ.unideb.hu>
#' @examples
#' \dontrun{
#' require("magrittr")
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#'
#' # Box plot
#' bxp <- ggboxplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' # Dot plot
#' dp <- ggdotplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' # Density plot
#' dens <- ggdensity(df, x = "len", fill = "dose", palette = "jco")
#'
#' # Export to pdf
#' ggarrange(bxp, dp, dens, ncol = 2) %>%
#'   ggexport(filename = "test.pdf")
#'
#' # Export to png
#' ggarrange(bxp, dp, dens, ncol = 2) %>%
#'   ggexport(filename = "test.png")
#' }
#'
#' @export
ggexport <- function(..., plotlist = NULL, filename = NULL, ncol = NULL, nrow = NULL,
                     width = 480, height = 480, pointsize = 12, res = NA, verbose = TRUE) {
  # File name and extension
  if (is.null(filename)) {
    filename <- .collapse(.random_string(), ".pdf", sep = "")
  }
  file.ext <- .file_ext(filename)


  # Device
  dev <- .device(filename)
  dev.opts <- list(file = filename)
  if (file.ext %in% c("ps", "eps")) {
    dev.opts <- dev.opts %>%
      .add_item(onefile = FALSE, horizontal = FALSE)
  } else if (file.ext %in% c("png", "jpeg", "jpg", "bmp", "tiff")) {
    dev.opts <- dev.opts %>%
      .add_item(width = width, height = height, pointsize = pointsize, res = res)
  }

  if (file.ext %in% c("pdf", "svg")) {
    if (!missing(width)) dev.opts <- dev.opts %>% .add_item(width = width)
    if (!missing(height)) dev.opts <- dev.opts %>% .add_item(height = height)
    if (!missing(pointsize)) dev.opts <- dev.opts %>% .add_item(pointsize = pointsize)
  }

  # width=800, height=800, pointsize=8, res=250

  # Plots
  plots <- c(list(...), plotlist)
  nb.plots <- length(plots)

  if (nb.plots == 1) {
    plots <- plots[[1]]
  } else if (!is.null(ncol) | !is.null(nrow)) {
    plots <- ggarrange(plotlist = plots, ncol = ncol, nrow = nrow)
  }
  if (inherits(plots, "ggarrange") & .is_list(plots)) {
    nb.plots <- length(plots)
  }

  if (nb.plots > 1 & file.ext %in% c("eps", "ps", "png", "jpeg", "jpg", "tiff", "bmp", "svg")) {
    filename <- gsub(paste0(".", file.ext), paste0("%03d.", file.ext), filename)
    dev.opts$file <- filename
    if (verbose) print(filename)
  }

  do.call(dev, dev.opts)
  utils::capture.output(print(plots))
  utils::capture.output(grDevices::dev.off())

  if (verbose) message("file saved to ", filename)
}

test_that("ggexport suppresses filename output when verbose is FALSE", {
  p <- ggplot2::ggplot(ToothGrowth, ggplot2::aes(dose, len)) +
    ggplot2::geom_point()
  filename <- tempfile(fileext = ".png")
  exported_pattern <- paste0(tools::file_path_sans_ext(filename), "*.png")
  on.exit(unlink(Sys.glob(exported_pattern)), add = TRUE)

  expect_silent(
    ggexport(
      p, p,
      filename = filename,
      width = 320, height = 320, res = 72,
      verbose = FALSE
    )
  )
  expect_gte(length(Sys.glob(exported_pattern)), 1)
})

test_that("ggexport reports filename when verbose is TRUE", {
  p <- ggplot2::ggplot(ToothGrowth, ggplot2::aes(dose, len)) +
    ggplot2::geom_point()
  filename <- tempfile(fileext = ".png")
  exported_pattern <- paste0(tools::file_path_sans_ext(filename), "*.png")
  on.exit(unlink(Sys.glob(exported_pattern)), add = TRUE)

  expect_output(
    expect_message(
      ggexport(
        p, p,
        filename = filename,
        width = 320, height = 320, res = 72,
        verbose = TRUE
      ),
      "file saved to"
    ),
    "%03d\\.png"
  )
})

#' @include utilities.R utils_stat_test_label.R stat_anova_test.R
NULL
#' Add Kruskal-Wallis Test P-values to a GGPlot
#' @description Add Kruskal-Wallis test p-values to a ggplot, such as
#'  box plots, dot plots and stripcharts.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @inheritParams stat_pvalue_manual
#' @inheritParams stat_anova_test
#'
#' @section Computed variables: \itemize{
#'  \item{statistic}:	the Kruskal-Wallis rank sum chi-squared statistic used to compute the p-value.
#'  \item{p}:	p-value.
#'  \item{p.adj}: Adjusted p-values.
#'  \item{p.signif}: P-value significance.
#'  \item{p.adj.signif}: Adjusted p-value significance.
#'  \item{p.format}: Formatted p-value.
#'  \item{p.format.signif}: Formatted p-value with significance symbols.
#'  \item{p.adj.format}: Formatted adjusted p-value.
#'  \item{n}: number of samples.
#'   }
#'
#' @examples
#' # Data preparation
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Transform `dose` into factor variable
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Add a random grouping variable
#' set.seed(123)
#' df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
#' df$len <- ifelse(df$group == "grp2", df$len + 2, df$len)
#' df$len <- ifelse(df$group == "grp3", df$len + 7, df$len)
#' head(df, 3)
#'
#'
#' # Basic boxplot
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Create a basic boxplot
#' # Add 5% and 10% space to the plot bottom and the top, respectively
#' bxp <- ggboxplot(df, x = "dose", y = "len") +
#'   scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))
#'
#' # Add the p-value to the boxplot
#' bxp + stat_kruskal_test()
#'
#' # Change the label position
#' # Using coordinates in data units
#' bxp + stat_kruskal_test(label.x = "1", label.y = 10, hjust = 0)
#'
#' # Format the p-value differently
#' custom_p_format <- function(p) {
#'   rstatix::p_format(p, accuracy = 0.0001, digits = 3, leading.zero = FALSE)
#' }
#' bxp + stat_kruskal_test(
#'   label = "Kruskal-Wallis, italic(p) = {custom_p_format(p)}{p.signif}"
#' )
#'
#' # Show a detailed label in italic
#' bxp + stat_kruskal_test(label = "as_detailed_italic")
#'
#'
#' # Faceted plots
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' # Create a ggplot facet
#' bxp <- ggboxplot(df, x = "dose", y = "len", facet.by = "supp") +
#'   scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))
#' # Add p-values
#' bxp + stat_kruskal_test()
#'
#'
#' # Grouped plots
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' bxp2 <- ggboxplot(df, x = "group", y = "len", color = "dose", palette = "npg")
#'
#' # For each x-position, computes tests between legend groups
#' bxp2 + stat_kruskal_test(aes(group = dose), label = "p = {p.format}{p.signif}")
#'
#' #  For each legend group, computes tests between x variable groups
#' bxp2 + stat_kruskal_test(aes(group = dose, color = dose), group.by = "legend.var")
#'
#' @export
stat_kruskal_test <- function(mapping = NULL, data = NULL, group.by = NULL,
                              label = "{method}, p = {p.format}",
                              label.x.npc = "left", label.y.npc = "top",
                              label.x = NULL, label.y = NULL, step.increase = 0.1,
                              p.adjust.method = "holm", significance = list(),
                              p.format.style = "default", p.digits = NULL,
                              p.leading.zero = NULL, p.min.threshold = NULL,
                              p.decimal.mark = NULL,
                              geom = "text", position = "identity", na.rm = FALSE, show.legend = FALSE,
                              inherit.aes = TRUE, parse = FALSE, ...) {
  label <- get_kruskal_test_label_template(label)
  if (missing(parse) & is_plotmath_expression(label)) {
    parse <- TRUE
  }

  layer(
    stat = StatCompareMultipleMeans, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      method = "kruskal_test",
      group.by = group.by,
      method.args = list(),
      correction = "none",
      na.rm = na.rm, stat.label = label,
      label.x.npc = label.x.npc, label.y.npc = label.y.npc,
      label.x = label.x, label.y = label.y, parse = parse,
      is.group.specified = is_group_aes_specified(mapping),
      step.increase = step.increase, p.adjust.method = p.adjust.method,
      significance = fortify_signif_symbols_encoding(significance),
      p.format.style = p.format.style, p.digits = p.digits,
      p.leading.zero = p.leading.zero, p.min.threshold = p.min.threshold,
      p.decimal.mark = p.decimal.mark, ...
    )
  )
}

get_kruskal_test_label_template <- function(label) {
  switch(label,
    as_italic = "{method}, italic(p) = {p.format}",
    as_detailed = "{method}, X2({df}) = {statistic}, p = {p.format}, n = {n}",
    as_detailed_italic = "{method}, italic(chi)^2 ({df}) = {statistic}, italic(p) = {p.format}, italic(n) = {n}",
    as_detailed_expression = "{method}, italic(chi)^2({df}) = {statistic}, italic(p) = {p.format}, italic(n) = {n}",
    label
  )
}

context("test-stat_kruskal_test")

# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)
df$id <- rep(1:10, 6) # Add individuals id
# Add a random grouping variable
set.seed(123)
df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
df <- df %>% mutate(
  len = ifelse(group == "grp2", len + 2, len),
  len = ifelse(group == "grp3", len + 7, len)
)


# Basic plots -----------------------------------
test_that("stat_kruskal_test works for basic ggplots", {
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    stat_kruskal_test()
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), 1)
  expect_equal(as.numeric(stat.test$y), 36.4)
  expect_equal(stat.test$label, "Kruskal-Wallis, p < 0.0001")
})


# Grouped plots-----------------------------------
test_that("stat_kruskal_test works for grouped plots: grouped by x position", {
  bxp <- ggboxplot(df, x = "group", y = "len", color = "dose") +
    stat_kruskal_test(aes(group = dose), label = "{p.adj.format}")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(1, 2, 3))
  expect_equal(as.numeric(stat.test$y), c(36.4, 36.4, 36.4))
  expect_equal(stat.test$label, c("0.0028", "0.0056", "0.0056"))
})

test_that("stat_kruskal_test works for grouped plots: grouped by legend variable", {
  bxp <- ggboxplot(df, x = "group", y = "len", color = "dose") +
    stat_kruskal_test(aes(group = dose), label = "{p.format}", group.by = "legend.var")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(0.73, 0.73, 0.73))
  expect_equal(as.numeric(stat.test$y), c(36.4, 39.62, 42.84))
  expect_equal(stat.test$label, c("0.012", "0.018", "0.038"))
})

#' @include utilities.R
NULL
#' Storing grid.arrange() arrangeGrob() and plots
#'
#' @description Transform the output of
#'  \code{\link[gridExtra:arrangeGrob]{arrangeGrob}()} and
#'  \code{\link[gridExtra:arrangeGrob]{grid.arrange}()} to an object of class
#'  ggplot.
#' @param x an object of class gtable or grob as returned by the functions
#'  \code{\link[gridExtra:arrangeGrob]{arrangeGrob}()} and
#'  \code{\link[gridExtra:arrangeGrob]{grid.arrange}()}.
#' @return an object of class ggplot.
#'
#' @examples
#' # Create some plots
#' bxp <- ggboxplot(iris, x = "Species", y = "Sepal.Length")
#' vp <- ggviolin(iris,
#'   x = "Species", y = "Sepal.Length",
#'   add = "mean_sd"
#' )
#'
#' # Arrange the plots in one page
#' # Returns a gtable (grob) object
#' library(gridExtra)
#' gt <- arrangeGrob(bxp, vp, ncol = 2)
#'
#' # Transform to a ggplot and print
#' as_ggplot(gt)
#'
#' @export
as_ggplot <- function(x) {
  # Open null device to avoid blank page before plot------
  # see cowplot:::as_grob.ggplot
  null_device <- base::getOption(
    "ggpubr.null_device",
    default = cowplot::pdf_null_device
  )
  cur_dev <- grDevices::dev.cur()
  # Open null device to avoid blank page before plot
  null_device(width = 6, height = 6)
  null_dev <- grDevices::dev.cur()
  on.exit({
    grDevices::dev.off(null_dev)
    if (cur_dev > 1) grDevices::dev.set(cur_dev)
  })

  # Convert to ggplot-------------
  cowplot::ggdraw() +
    cowplot::draw_grob(grid::grobTree(x))
}

#' @include utilities.R ggpar.R stat_chull.R stat_conf_ellipse.R
NULL
#' Cleveland's Dot Plots
#' @description Draw a Cleveland dot plot.
#' @inheritParams ggboxplot
#' @inheritParams ggpar
#' @param data a data frame
#' @param x,y x and y variables for drawing.
#' @param color,size points color and size.
#' @param dot.size numeric value specifying the dot size.
#' @param shape point shape. See \code{\link{show_point_shapes}}.
#' @param label the name of the column containing point labels.
#' @param group an optional column name indicating how the elements of x are
#'   grouped.
#' @param sorting a character vector for sorting into ascending or descending
#'   order. Allowed values are one of "descending", "ascending" and "none". Partial
#'   match are allowed (e.g. sorting = "desc" or "asc"). Default is
#'   "descending".
#' @param x.text.col logical. If TRUE (default), x axis texts are colored by
#'   groups.
#' @param position Position adjustment, either as a string, or the result of a
#'   call to a position adjustment function.
#' @param ... other arguments to be passed to \code{\link[ggplot2]{geom_point}}
#'   and \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#' df$name <- rownames(df)
#' head(df[, c("wt", "mpg", "cyl")], 3)
#'
#' # Basic plot
#' ggdotchart(df,
#'   x = "name", y = "mpg",
#'   ggtheme = theme_bw()
#' )
#'
#' # Change colors by  group cyl
#' ggdotchart(df,
#'   x = "name", y = "mpg",
#'   group = "cyl", color = "cyl",
#'   palette = c("#999999", "#E69F00", "#56B4E9"),
#'   rotate = TRUE,
#'   sorting = "descending",
#'   ggtheme = theme_bw(),
#'   y.text.col = TRUE
#' )
#'
#'
#' # Plot with multiple groups
#' # +++++++++++++++++++++
#' # Create some data
#' df2 <- data.frame(
#'   supp = rep(c("VC", "OJ"), each = 3),
#'   dose = rep(c("D0.5", "D1", "D2"), 2),
#'   len = c(6.8, 15, 33, 4.2, 10, 29.5)
#' )
#' print(df2)
#'
#' ggdotchart(df2,
#'   x = "dose", y = "len",
#'   color = "supp", size = 3,
#'   add = "segment",
#'   add.params = list(color = "lightgray", linewidth = 1.5),
#'   position = position_dodge(0.3),
#'   palette = "jco",
#'   ggtheme = theme_pubclean()
#' )
#'
#' @export
ggdotchart <- function(data, x, y, group = NULL,
                       combine = FALSE,
                       color = "black", palette = NULL,
                       shape = 19, size = NULL, dot.size = size,
                       sorting = c("ascending", "descending", "none"),
                       add = c("none", "segment"), add.params = list(),
                       x.text.col = TRUE,
                       rotate = FALSE,
                       title = NULL, xlab = NULL, ylab = NULL,
                       facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                       select = NULL, remove = NULL, order = NULL,
                       label = NULL, font.label = list(size = 11, color = "black"),
                       label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                       position = "identity",
                       ggtheme = theme_pubr(),
                       ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    data = data, x = x, y = y,
    group = group, combine = combine,
    color = color, palette = palette, shape = shape,
    size = size, dot.size = dot.size, sorting = sorting,
    x.text.col = x.text.col,
    rotate = rotate, title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle,
    position = position, ggtheme = ggtheme, ...
  )

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  .opts$fun <- ggdotchart_core
  .opts$fun_name <- "ggdotchart"

  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }

  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)

  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


ggdotchart_core <- function(data, x, y, group = NULL,
                            color = "black", palette = NULL,
                            shape = 19, size = NULL, dot.size = size,
                            sorting = c("ascending", "descending", "none"),
                            add = c("none", "segments"), add.params = list(),
                            x.text.col = FALSE,
                            rotate = FALSE,
                            title = NULL, xlab = NULL, ylab = NULL,
                            ggtheme = theme_bw(),
                            position = "identity",
                            ...) {
  add <- match.arg(add)
  if (!is.null(group)) {
    if (group == 1) {
      group <- NULL
    }
  }

  # if(is.null(group) & color[1] %in% names(data)){
  #   group <- color[1]
  # }


  .dots <- list(...)
  sorting <- match.arg(sorting)
  decreasing <- ifelse(sorting == "descending", FALSE, TRUE)
  x.text.angle <- ifelse(rotate, 0, 90)
  if (!is.null(.dots$y.text.col)) {
    x.text.col <- .dots$y.text.col
  }
  data <- as.data.frame(data)
  label <- .select_vec(data, x)
  . <- NULL

  if (rotate & sorting == "descending") {
    sorting <- "ascending"
  } else if (rotate & sorting == "ascending") sorting <- "descending"
  if (sorting != "none") {
    if (is.null(group)) {
      if (sorting == "descending") {
        data <- arrange(data, desc(!!sym(y)))
      } else if (sorting == "ascending") {
        data <- arrange(data, !!sym(y))
      }
    } else if (group != 1) {
      if (sorting == "descending") {
        data <- arrange(data, !!sym(group), desc(!!sym(y)))
      } else if (sorting == "ascending") {
        data <- arrange(data, !!!syms(c(group, y)))
      }
    }
    data[[x]] <- factor(data[[x]], levels = unique(as.vector(data[[x]])))
  }

  if (!is.factor(data[[x]])) data[[x]] <- as.factor(data[[x]])

  p <- ggplot(data, create_aes(list(x = x, y = y)))

  if (add == "segments") {
    seg.opts <- geom_exec(
      data = data, color = color,
      size = size, position = position
    )

    mapping <- seg.opts$mapping %>%
      .add_item(x = x, ymin = 0, ymax = y, group = group)
    option <- seg.opts$option

    # mapping <- seg.opts$mapping %>%
    #   .add_item(y = 0, x = x, yend = y, xend = x)
    # option <- seg.opts$option

    seg.col <- "lightgray"
    if (!is.null(add.params$color)) {
      seg.col <- add.params$color
    } else if (!is.null(add.params$colour)) {
      seg.col <- add.params$colour
    }
    if (seg.col %in% names(data)) {
      mapping$color <- seg.col
    } else {
      option$color <- seg.col
    }


    # Use linewidth instead of size for geom_linerange (ggplot2 3.4.0+ compatibility)
    if (!is.null(add.params$linewidth)) {
      option$linewidth <- add.params$linewidth
    } else if (!is.null(add.params$size)) {
      option$linewidth <- add.params$size
    }

    # geom_linerange takes `linewidth`, not the deprecated `size` (ggplot2 >= 3.4.0).
    # Migrate any size to linewidth, then drop size -- for both the constant
    # parameter (option$size) and a size mapped to a data column (mapping$size).
    if (is.null(option$linewidth) && !is.null(option$size)) option$linewidth <- option$size
    option$size <- NULL
    if (is.null(mapping$linewidth) && !is.null(mapping$size)) mapping$linewidth <- mapping$size
    mapping$size <- NULL

    option[["mapping"]] <- create_aes(mapping)
    p <- p + do.call(geom_linerange, option)
  }


  p <- p + geom_exec(geom_point,
    data = data, shape = shape,
    color = color, size = dot.size, position = position
  )


  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme, x.text.angle = x.text.angle,
    title = title, xlab = xlab, ylab = ylab, ...
  )


  # Change x axis text colors
  if (x.text.col) {
    if (!rotate) {
      p <- .set_x_text_col(p, label, x.text.angle)
    } else {
      p <- .set_y_text_col(p, label, x.text.angle)
    }
  }

  if (x.text.angle == 90 & !rotate) {
    p <- p + theme(axis.text.x = element_text(vjust = 0.5))
  }

  if (rotate) p <- p + coord_flip()

  p
}


# Set x text color
# Note: Vectorized input to element_text() is not officially supported by ggplot2,
# but this is a deliberate feature for Cleveland dot plots. We suppress the warning.
.set_x_text_col <- function(p, label, angle) {
  g <- ggplot2::ggplot_build(p)
  cols <- unlist(g$data[[1]]["colour"])
  names(cols) <- as.vector(label) # Give every color an appropriate name
  suppressWarnings(
    p + theme(axis.text.x = element_text(colour = cols, angle = angle, hjust = 1))
  )
}

# Set y text color
# Note: Vectorized input to element_text() is not officially supported by ggplot2,
# but this is a deliberate feature for Cleveland dot plots. We suppress the warning.
.set_y_text_col <- function(p, label, angle) {
  g <- ggplot2::ggplot_build(p)
  cols <- unlist(g$data[[1]]["colour"])
  names(cols) <- as.vector(label) # Give every color an appropriate name
  suppressWarnings(
    p + theme(axis.text.y = element_text(colour = cols))
  )
}


# Helper functions
# +++++++++++++++++++++++++
#' @export
#' @rdname ggdotchart
theme_cleveland <- function(rotate = TRUE) {
  if (rotate) {
    theme(
      panel.grid.major.x = element_blank(),
      panel.grid.minor.x = element_blank(),
      panel.grid.major.y = element_line(colour = "grey70", linetype = "dashed"),
      axis.title.y = element_blank(),
      axis.ticks.y = element_blank()
    )
  } else {
    theme(
      panel.grid.major.x = element_line(colour = "grey70", linetype = "dashed"),
      panel.grid.major.y = element_blank(),
      panel.grid.minor.y = element_blank(),
      axis.title.x = element_blank(),
      axis.ticks.x = element_blank()
    )
  }
}

#' @include utilities.R
#' @import ggplot2
NULL
#' Publication ready theme
#'
#' @description \itemize{ \item \strong{theme_pubr()}: Create a publication ready
#'  theme \item \strong{theme_pubclean()}: a clean theme without axis lines, to
#'  direct more attention to the data.  \item \strong{labs_pubr()}: Format only
#'  plot labels to a publication ready style \item \strong{theme_classic2()}:
#'  Create a classic theme with axis lines. \item \strong{clean_theme()}: Remove
#'  axis lines, ticks, texts and titles. \item \strong{clean_table_theme()}:
#'  Clean the theme of a table, such as those created by
#'  \code{\link{ggsummarytable}()}}.
#' @details \code{theme_pubr()} sets \code{strip.clip = "off"} so that the facet
#'  strip background border is drawn at its full width (with the ggplot2 default
#'  \code{strip.clip = "on"} the border is clipped to the strip area and renders
#'  thinner than requested). As a side effect, a facet label that is wider than
#'  its panel will overflow the strip instead of being truncated; if that is a
#'  problem for very long labels, restore clipping with
#'  \code{+ theme(strip.clip = "on")}.
#' @param base_size base font size
#' @param base_family base font family
#' @param border logical value. Default is FALSE. If TRUE, add panel border.
#' @param margin logical value. Default is TRUE. If FALSE, reduce plot margin.
#' @param legend character specifying legend position. Allowed values are one of
#'  c("top", "bottom", "left", "right", "none"). Default is "top" side position.
#'  to remove the legend use legend = "none". Legend position can be also
#'  specified using a numeric vector c(x, y).  In this case it is possible to
#'  position the legend inside the plotting area. x and y are the coordinates of
#'  the legend box. Their values should be between 0 and 1. c(0,0) corresponds
#'  to the "bottom left" and c(1,1) corresponds to the "top right" position. For
#'  instance use legend = c(0.8, 0.2).
#' @param x.text.angle Rotation angle of x axis tick labels. Default value is 0.
#'  Use 90 for vertical text.
#' @param flip logical. If TRUE, grid lines are added to y axis instead of x
#'  axis.
#' @examples
#' p <- ggplot(mtcars, aes(x = wt, y = mpg)) +
#'   geom_point(aes(color = gear))
#'
#' # Default plot
#' p
#'
#' # Use theme_pubr()
#' p + theme_pubr()
#'
#' # Format labels
#' p + labs_pubr()
#'
#' @name theme_pubr
#' @rdname theme_pubr
#' @export
theme_pubr <- function(base_size = 12, base_family = "",
                       border = FALSE, margin = TRUE,
                       legend = c("top", "bottom", "left", "right", "none"),
                       x.text.angle = 0) {
  half_line <- base_size / 2
  if (!is.numeric(legend)) legend <- match.arg(legend)
  if (x.text.angle > 5) xhjust <- 1 else xhjust <- NULL

  if (border) {
    panel.border <- element_rect(fill = NA, colour = "black", linewidth = 0.7)
    axis.line <- element_blank()
  } else {
    panel.border <- element_blank()
    axis.line <- element_line(colour = "black", linewidth = 0.5)
  }


  if (margin) {
    plot.margin <- margin(
      half_line, half_line, half_line,
      half_line
    )
  } else {
    plot.margin <- unit(c(0.5, 0.3, 0.3, 0.3), "mm")
  }

  .theme <- theme_bw(base_size = base_size, base_family = base_family) %+replace%
    theme(
      panel.border = panel.border,
      panel.grid.major = element_blank(),
      panel.grid.minor = element_blank(),
      axis.line = axis.line, axis.text = element_text(color = "black"),
      # Match the tick marks to the axis lines (black, 0.5); by default they
      # inherited a lighter grey/thinner style, visibly inconsistent when zoomed
      # (#668).
      axis.ticks = element_line(colour = "black", linewidth = 0.5),
      legend.key = element_blank(),
      strip.background = element_rect(fill = "#F2F2F2", colour = "black", linewidth = 0.7),
      # Don't clip the facet strip background to the strip area: by default
      # ggplot2 (>= 3.5.0) clips it, which cuts the outer half of the strip's
      # border stroke so it renders thinner than the requested linewidth and
      # looks misaligned with the panel/axis lines when zoomed (follow-up to
      # the tick-consistency fix in #668).
      strip.clip = "off",
      plot.margin = plot.margin,
      legend.position = legend,
      complete = TRUE
    )

  if (x.text.angle != 0) {
    .theme <- .theme + theme(axis.text.x = element_text(angle = x.text.angle, hjust = xhjust))
  }

  .theme
}

#' @rdname theme_pubr
#' @export
theme_pubclean <- function(base_size = 12, base_family = "", flip = FALSE) {
  res <- theme_grey(base_size = base_size, base_family = base_family) +
    theme(
      panel.background = element_rect(fill = "white"),
      legend.background = element_rect(fill = "white"),
      legend.position = "top"
    )
  if (flip) {
    res <- res + theme(
      panel.grid.major.x = element_line(linetype = "dotted", color = "grey"),
      axis.line.y = element_line(color = "black")
    )
  } else {
    res <- res + theme(
      panel.grid.major.y = element_line(linetype = "dotted", color = "grey")
    )
  }
  res
}


#' @rdname theme_pubr
#' @export
labs_pubr <- function(base_size = 14, base_family = "") {
  theme(
    text = element_text(
      family = base_family,
      face = "plain", colour = "black", size = base_size, lineheight = 0.9,
      hjust = 0.5, vjust = 0.5, angle = 0, margin = margin(),
      debug = FALSE
    ),
    # Tick labels
    axis.text.x = element_text(size = rel(0.86), colour = "black", face = "bold"),
    axis.text.y = element_text(size = rel(0.86), colour = "black", face = "bold"),

    # Axis labels
    axis.title = element_text(size = rel(1), colour = "black", face = "bold"),

    # Main title
    plot.title = element_text(
      size = rel(1), colour = "black",
      lineheight = 1.0, face = "bold"
    ),
    legend.title = element_text(size = rel(0.7), face = "bold", colour = "black"),
    legend.text = element_text(size = rel(0.7), face = "plain", colour = "black")
  )
}

#' @rdname theme_pubr
#' @export
theme_classic2 <-
  function(base_size = 12, base_family = "") {
    theme_classic(base_size = base_size, base_family = base_family) %+replace%
      theme(
        axis.line.x = element_line(),
        axis.line.y = element_line()
      )
  }


#' @export
#' @rdname theme_pubr
clean_theme <- function() {
  theme(
    axis.line.x = element_blank(),
    axis.line.y = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks.x = element_blank(),
    axis.ticks.y = element_blank(),
    axis.title.x = element_blank(),
    axis.title.y = element_blank()
  )
}

#' @export
#' @rdname theme_pubr
clean_table_theme <- function() {
  theme(
    axis.line.x = element_blank(), axis.line.y = element_blank(),
    axis.text.x = element_blank(), axis.ticks.x = element_blank(),
    axis.ticks.y = element_blank(), axis.title.x = element_blank(),
    axis.title.y = element_blank()
  )
}

#' @include utilities.R ggpar.R
NULL
#' QQ Plots
#' @description Quantile-Quantile plot.
#' @inheritParams ggboxplot
#' @param x variable to be drawn.
#' @param color point color.
#' @param size point size.
#' @param shape point shape.
#' @param add character vector. Allowed values are one of "none" and "qqline"
#'   (for adding qqline).
#' @param add.params parameters (color, size,  linetype) for the
#'   argument 'add'; e.g.: add.params = list(color = "red").
#' @param conf.int logical value. If TRUE, confidence interval is added.
#' @param conf.int.level the confidence level. Default value is 0.95.
#' @param ... other arguments to be passed to \code{\link{ggpar}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}
#'
#' @examples
#' # Create some data format
#' set.seed(1234)
#' wdata <- data.frame(
#'   sex = factor(rep(c("F", "M"), each = 200)),
#'   weight = c(rnorm(200, 55), rnorm(200, 58))
#' )
#'
#' head(wdata, 4)
#'
#' # Basic QQ plot
#' ggqqplot(wdata, x = "weight")
#'
#' # Change colors and shape by groups ("sex")
#' # Use custom palette
#' ggqqplot(wdata,
#'   x = "weight",
#'   color = "sex", palette = c("#00AFBB", "#E7B800")
#' )
#'
#' @export
ggqqplot <- function(data, x, combine = FALSE, merge = FALSE,
                     color = "black", palette = NULL,
                     size = NULL, shape = NULL,
                     add = c("qqline", "none"),
                     add.params = list(linetype = "solid"),
                     conf.int = TRUE, conf.int.level = 0.95,
                     title = NULL, xlab = NULL, ylab = NULL,
                     facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                     ggtheme = theme_pubr(),
                     ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, palette = palette,
    size = size, shape = shape,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    conf.int = conf.int, conf.int.level = conf.int.level,
    ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }
  .opts$fun <- ggqqplot_core
  .opts$y <- "..qq.."
  if (is.null(.opts$xlab)) .opts$xlab <- "Theoretical"
  if (is.null(.opts$ylab)) .opts$ylab <- "Sample"
  if (is.null(.opts$add)) .opts$add <- "qqline"
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}

ggqqplot_core <- function(data, x, y = "..qq..",
                          color = "black", palette = NULL,
                          size = NULL, shape = NULL, fill = "white",
                          add = c("qqline", "none"),
                          add.params = list(linetype = "solid"),
                          conf.int = TRUE, conf.int.level = 0.95,
                          title = NULL, xlab = NULL, ylab = NULL,
                          ggtheme = theme_pubr(),
                          ...) {
  # Check data
  .dd <- .check_data(data, x, y = NULL)
  data <- .dd$data
  x <- .dd$x
  y <- .dd$y

  group <- c(shape, color) %>%
    unique() %>%
    intersect(colnames(data))
  group <- ifelse(.is_empty(group), 1, group[1])

  add <- match.arg(add)
  add.params <- .check_add.params(add, add.params, error.plot = "", data, color, fill = fill, ...)
  if (is.null(add.params$size)) add.params$size <- size
  if (is.null(add.params$linetype)) add.params$linetype <- "solid"

  p <- ggplot(data, create_aes(list(sample = x)))

  p <- p +
    geom_exec(stat_qq,
      data = data,
      color = color, size = size, shape = shape
    )


  # Use linewidth instead of size for .stat_qqline (ggplot2 3.4.0+ compatibility)
  if ("qqline" %in% add) {
    p <- p +
      geom_exec(.stat_qqline,
        data = data,
        color = add.params$color, linewidth = add.params$size,
        linetype = add.params$linetype, group = group
      )
  }

  # Confidence interval
  if (conf.int) {
    p <- p + geom_exec(.stat_qq_confint,
      data = data,
      fill = color, alpha = 0.2, conf.int.level = conf.int.level,
      group = group
    )
  }

  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )
  p
}


# Helper functions
########################
# from: http://stackoverflow.com/questions/4357031/qqnorm-and-qqline-in-ggplot2
# qf : qfunction (e.g.: qnorm)
.qq_line <- function(data, qf, na.rm) {
  q.sample <- stats::quantile(data, c(0.25, 0.75), na.rm = na.rm)
  q.theory <- qf(c(0.25, 0.75))
  slope <- diff(q.sample) / diff(q.theory)
  intercept <- q.sample[1] - slope * q.theory[1]
  list(slope = slope, intercept = intercept)
}


StatQQLine <- ggproto("StatQQLine", Stat,
  # http://docs.ggplot2.org/current/vignettes/extending-ggplot2.html
  # https://github.com/hadley/ggplot2/blob/master/R/stat-qq.r
  required_aes = c("sample"),
  compute_group = function(data, scales,
                           distribution = stats::qnorm,
                           dparams = list(),
                           conf.int.level = 0.95,
                           na.rm = FALSE) {
    qf <- function(p) do.call(distribution, c(list(p = p), dparams))
    n <- length(data$sample)
    P <- stats::ppoints(n)
    theoretical <- qf(P)
    qq <- .qq_line(data$sample, qf = qf, na.rm = na.rm)
    line <- qq$intercept + theoretical * qq$slope
    # Confidence interval
    zz <- stats::qnorm(1 - (1 - conf.int.level) / 2)
    SE <- (qq$slope / stats::dnorm(theoretical)) * sqrt(P * (1 - P) / n)
    fit.value <- qq$intercept + qq$slope * theoretical
    ymax <- fit.value + zz * SE
    ymin <- fit.value - zz * SE
    data.frame(sample = line, x = theoretical, y = line, ymin = ymin, ymax = ymax)
  }
)

.stat_qqline <- function(mapping = NULL, data = NULL, geom = "line",
                         position = "identity", ...,
                         distribution = stats::qnorm,
                         dparams = list(),
                         na.rm = FALSE,
                         show.legend = NA,
                         inherit.aes = TRUE,
                         conf.int.level = 0.95) {
  layer(
    stat = StatQQLine, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      distribution = distribution,
      dparams = dparams,
      na.rm = na.rm, conf.int.level = conf.int.level, ...
    )
  )
}


.stat_qq_confint <- function(mapping = NULL, data = NULL, geom = "ribbon",
                             position = "identity", ...,
                             distribution = stats::qnorm,
                             dparams = list(),
                             na.rm = FALSE,
                             show.legend = NA,
                             inherit.aes = TRUE,
                             conf.int.level = 0.95) {
  layer(
    stat = StatQQLine, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(
      distribution = distribution,
      dparams = dparams,
      na.rm = na.rm, conf.int.level = conf.int.level, ...
    )
  )
}

#' Easy Break Creation for Numeric Axes
#'
#' @description Creates breaks for numeric axes to be used in the functions
#'   \code{\link[ggplot2:scale_continuous]{scale_x_continuous}()} and
#'   \code{\link[ggplot2:scale_continuous]{scale_y_continuous}()}. Can be used to increase the
#'   number of x and y ticks by specifying the option \code{n}. It's also
#'   possible to control axis breaks by specifying a step between ticks.  For
#'   example, if \code{by = 5}, a tick mark is shown on every 5.
#' @param n number of breaks.
#' @param by number: the step between breaks.
#' @param from the starting value of breaks. By default, 0 is used for positive
#'   variables
#' @param to the end values of breaks. This corresponds generally to the maximum
#'   limit of the axis.
#' @return a break function
#'
#' @examples
#'
#' # Generate 5 breaks for a variable x
#' get_breaks(n = 5)(x = 1:100)
#'
#' # Generate breaks using an increasing step
#' get_breaks(by = 10)(x = 1:100)
#'
#' # Combine with ggplot scale_xx functions
#' library(ggplot2)
#'
#' # Create a basic plot
#' p <- ggscatter(mtcars, x = "wt", y = "mpg")
#' p
#'
#' # Increase the number of ticks
#' p +
#'   scale_x_continuous(breaks = get_breaks(n = 10)) +
#'   scale_y_continuous(breaks = get_breaks(n = 10))
#'
#' # Set ticks according to a specific step, starting from 0
#' p + scale_x_continuous(
#'   breaks = get_breaks(by = 1.5, from = 0),
#'   limits = c(0, 6)
#' ) +
#'   scale_y_continuous(
#'     breaks = get_breaks(by = 10, from = 0),
#'     limits = c(0, 40)
#'   )
#'
#' @export
get_breaks <- function(n = NULL, by = NULL, from = NULL, to = NULL) {
  breaks <- ggplot2::waiver()
  if (!is.null(n)) {
    breaks <- get_breaks_number(n = n)
  } else if (is.numeric(by)) {
    breaks <- get_breaks_position(by = by, from = from, to = to)
  }
  breaks
}


# Set the number of breaks
get_breaks_number <- function(n) {
  scales::breaks_extended(n = n)
}

# Set breaks using increasing step
# Adapted from scales::breaks_extended
get_breaks_position <- function(by, from = NULL, to = NULL) {
  by_default <- by
  from_default <- from
  to_default <- to
  function(x, by = by_default, from = from_default, to = to_default) {
    x <- x[is.finite(x)]
    if (length(x) == 0) {
      return(numeric())
    }
    rng <- range(x)
    if (rng[1] > 0 & is.null(from)) from <- 0
    # Anchor the first break to a multiple of `by` so the ticks land on round
    # numbers (e.g. 0, 100, 200) even when the axis range dips slightly below 0
    # from the default scale expansion; otherwise the breaks started at the
    # expanded minimum (e.g. -19, 81, 181) (#313). Out-of-range breaks are
    # dropped by the scale.
    xmin <- ifelse(is.null(from), floor(rng[1] / by) * by, from)
    xmax <- ifelse(is.null(to), rng[2], to)
    seq(from = xmin, to = xmax, by = by)
  }
}

#' Checks and Returns Data Coordinates from Multiple Input Options
#'
#' @description Checks and returns selected coordinates from multiple input
#'   options, which can be either data (x-y) coordinates or npc (normalized
#'   parent coordinates).
#'
#'   Helper function internally used in \code{ggpubr} function to guess the type
#'   of coordinates specified by the user. For example, in the function
#'   \code{stat_cor()}, users can specify either the option \code{label.x} (data
#'   coordinates) or \code{label.x.npc} (npc coordinates); those coordinates are
#'   passed to \code{get_coord()}, which will make some checking and then return
#'   a unique coordinates for the label position.
#' @inheritParams as_npc
#' @param data.ranges a numeric vector of length 2 containing the data ranges
#'   (minimum and the maximum). Should be specified only when \code{coord =
#'   NULL} and \code{npc} is specified. Used to convert \code{npc} to data
#'   coordinates. Considered only when the argument \code{npc} is specified.
#' @param coord data coordinates (i.e., either x or y coordinates).
#' @param npc numeric (in [0-1]) or character vector of coordinates. If
#'   character, should be one of c('right', 'left', 'bottom', 'top', 'center',
#'   'centre', 'middle'). Note that, the \code{data.ranges}, \code{step} and
#'   \code{margin.npc}, arguments are considered only when \code{npc} is
#'   specified. The option \code{npc} is ignored when the argument \code{coord} is specified.
#' @return a numeric vector representing data coordinates.
#' @seealso \code{\link{as_npc}}, \code{\link{npc_to_data_coord}}.
#' @examples
#' # If npc is specified, it is converted into data coordinates
#' get_coord(data.ranges = c(2, 20), npc = "left")
#' get_coord(data.ranges = c(2, 20), npc = 0.1)
#'
#' # When coord is specified, no transformation is performed
#' # because this is assumed to be a data coordinate
#' get_coord(coord = 5)
#'
#' # For grouped plots
#' res_top <- get_coord(
#'   data.ranges = c(4.2, 36.4), group = c(1, 2, 3),
#'   npc = "top", step = -0.1, margin.npc = 0
#' )
#' res_top
#'
#' @rdname get_coord
#' @export
get_coord <- function(group = 1L, data.ranges = NULL, coord = NULL, npc = "left",
                      step = 0.1, margin.npc = 0.05) {
  if (!is.null(coord)) {
    if (!is.numeric(group)) {
      stop(
        "get_coord: 'group' should be numeric. ",
        "Current class is: ", class(group)
      )
    }
    # If coords are too short, they are recycled.
    coord <- ifelse(length(coord) >= group, coord[group], coord[1])
    return(coord)
  } else if (!is.null(npc)) {
    if (is.null(data.ranges)) {
      stop("Specify the option data.ranges", call. = FALSE)
    }
    npc <- ifelse(length(npc) >= group, npc[group], npc[1])
    coord <- as_npc(
      npc,
      group = group, step = step,
      margin.npc = margin.npc
    ) %>%
      npc_to_data_coord(data.ranges)
  }
  coord
}

context("test-get_coord")

# Basic plots: one group -----------------------------------
test_that("No transformation is performed when coord specified", {
  res_10 <- get_coord(coord = 10)
  res_text <- get_coord(coord = "text")
  expect_equal(res_10, 10)
  expect_equal(res_text, "text")
})

test_that("get_coord works for one group", {
  # x- axis
  res_left <- get_coord(
    data.ranges = c(1, 3), group = 1,
    npc = "left", step = 0.1, margin.npc = 0
  )
  # Y -axis
  res_top <- get_coord(
    data.ranges = c(4.2, 36.4), group = 1,
    npc = "top", step = -0.1, margin.npc = 0
  )
  expect_equal(res_top, 36.4)
})


# Grouped plots --------------------------
test_that("get_coord works for grouped plots", {
  # x- axis
  res_left <- get_coord(
    data.ranges = c(1, 3), group = 1:3,
    npc = "left", step = 0.1, margin.npc = 0
  )
  # no transformation when coord specified
  res_coord <- get_coord(
    data.ranges = c(1, 3), group = 1:3,
    coord = 1:3, step = 0.1, margin.npc = 0
  )
  # Y-axis
  res_top <- get_coord(
    data.ranges = c(4.2, 36.4), group = c(1, 2, 3),
    npc = "top", step = -0.1, margin.npc = 0
  )
  expect_equal(res_left, c(1.0, 1.2, 1.4))
  expect_equal(res_coord, 1:3)
  expect_equal(res_top, c(36.40, 39.62, 42.84))
})

#' @include utilities.R ggpar.R
NULL
#' Plot Paired Data
#' @description Plot paired data.
#' @inheritParams ggboxplot
#' @param cond1 variable name corresponding to the first condition.
#' @param cond2 variable name corresponding to the second condition.
#' @param x,y x and y variables, where x is a grouping variable and y contains
#'  values for each group. Considered only when \code{cond1} and \code{cond2}
#'  are missing.
#' @param id variable name corresponding to paired samples' id. Used to connect
#'  paired points with lines.
#' @param color points and box plot colors. To color by conditions, use color =
#'  "condition".
#' @param fill box plot fill color. To change fill color by conditions, use fill
#'  = "condition".
#' @param line.color line color.
#' @param linetype line type.
#' @param point.size,line.size point and line size, respectively.
#' @param width box plot width.
#' @param jitter numeric value (default 0, no jitter) giving the amount of
#'   horizontal jitter added to the paired points to reduce overlap. Points are
#'   nudged sideways within \code{[-jitter, jitter]}; each subject (\code{id})
#'   gets a single offset so its two points move together and the connecting
#'   line stays intact. Only the horizontal positions change (the values are
#'   never moved). Typical values are small relative to the box \code{width}
#'   (e.g. \code{jitter = 0.05} to \code{0.1}). \code{jitter = 0} leaves the plot
#'   unchanged.
#' @param ... other arguments to be passed to be passed to \link{ggpar}().
#' @examples
#'
#' # Example 1
#' # ::::::::::::::::::::::::::::::::::::::::::
#' before <- c(200.1, 190.9, 192.7, 213, 241.4, 196.9, 172.2, 185.5, 205.2, 193.7)
#' after <- c(392.9, 393.2, 345.1, 393, 434, 427.9, 422, 383.9, 392.3, 352.2)
#'
#' d <- data.frame(before = before, after = after)
#' ggpaired(d,
#'   cond1 = "before", cond2 = "after",
#'   fill = "condition", palette = "jco"
#' )
#'
#' # Example 2
#' # ::::::::::::::::::::::::::::::::::::::::::
#' ggpaired(ToothGrowth,
#'   x = "supp", y = "len",
#'   color = "supp", line.color = "gray", line.size = 0.4,
#'   palette = "npg"
#' )
#'
#' @export
ggpaired <- function(data, cond1, cond2, x = NULL, y = NULL, id = NULL,
                     color = "black", fill = "white", palette = NULL,
                     width = 0.5, point.size = 1.2, line.size = 0.5, line.color = "black",
                     linetype = "solid",
                     title = NULL, xlab = "Condition", ylab = "Value",
                     facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                     label = NULL, font.label = list(size = 11, color = "black"),
                     label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                     ggtheme = theme_pubr(),
                     jitter = 0,
                     ...) {
  grouping.vars <- c(x, color, fill) %>%
    unique() %>%
    intersect(colnames(data))

  if (!missing(cond1) & !missing(cond2)) {
    data <- data %>%
      df_gather(
        cols = c(cond1, cond2),
        names_to = "condition", values_to = "val"
      )
    data$condition <- factor(data$condition, levels = c(cond1, cond2))
    x <- "condition"
    y <- "val"
  } else if (!is.null(x) & !is.null(y)) {
    if (missing(xlab)) xlab <- x
    if (missing(ylab)) ylab <- y
  }


  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    id = id,
    color = color, fill = fill, palette = palette,
    width = width, point.size = point.size,
    line.size = line.size, line.color = line.color, linetype = linetype,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle, ggtheme = ggtheme,
    jitter = jitter, ...
  )

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }
  .opts$data <- data
  .opts$x <- x
  .opts$y <- y


  .opts$fun <- ggpaired_core
  if (missing(ggtheme) & (!is.null(facet.by))) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}

ggpaired_core <- function(data, x = NULL, y = NULL, id = NULL,
                          color = "black", fill = "white", palette = NULL,
                          width = 0.5, point.size = 1.2, line.size = 0.5, line.color = "black",
                          linetype = "solid", title = NULL, xlab = "Condition", ylab = "Value",
                          ggtheme = theme_pubr(),
                          jitter = 0,
                          ...) {
  if (!is.factor(data[[x]])) data[[x]] <- as.factor(data[[x]])

  grouping.vars <- c(x, color, fill) %>%
    unique() %>%
    intersect(colnames(data))


  # Add paired sample ids
  if (!is.null(id)) {
    id <- .select_vec(data, id)
  } else {
    id <- rep(seq_len(nrow(data) / 2), 2)
  }
  data$id <- id

  # Optional horizontal jitter of the paired points (#407). Spreads the points
  # to reduce overlap while keeping each pair's connecting line intact: one
  # offset per subject id (so the connecting lines stay parallel and readable),
  # applied to BOTH the points and the lines, and horizontal only (the values
  # `y` are never moved). A fixed seed keeps a given plot reproducible; the
  # caller's RNG stream is left untouched. Default jitter = 0 leaves the output
  # exactly unchanged (no jittered-x column, no RNG use).
  jitter.x <- NULL
  if (!is.null(jitter) && length(jitter) == 1 && is.numeric(jitter) &&
      !is.na(jitter) && jitter > 0) {
    ids <- unique(data$id)
    if (exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
      old.seed <- get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
      on.exit(assign(".Random.seed", old.seed, envir = .GlobalEnv), add = TRUE)
    } else {
      # no RNG used yet: remove the seed we are about to create, so ggpaired()
      # leaves the caller's random stream exactly as it found it
      on.exit(suppressWarnings(rm(".Random.seed", envir = .GlobalEnv)), add = TRUE)
    }
    set.seed(123)
    offsets <- stats::runif(length(ids), -jitter, jitter)
    names(offsets) <- as.character(ids)
    data$.ggpubr.x.jitter. <- as.integer(data[[x]]) + offsets[as.character(data$id)]
    jitter.x <- ".ggpubr.x.jitter."
  }

  position <- "identity"
  # if(length(grouping.vars) > 1)
  #   position <- position_dodge(0.8)

  condition <- val <- id <- NULL
  p <- ggplot(data, create_aes(list(x = x, y = y))) +
    geom_exec(geom_boxplot,
      data = data, color = color, fill = fill, width = width,
      position = position
    ) +
    geom_exec(geom_line,
      data = data, x = jitter.x, group = "id",
      color = line.color, linewidth = line.size, linetype = linetype,
      position = position
    ) +
    geom_exec(geom_point,
      data = data, x = jitter.x, color = color, size = point.size,
      position = position
    )

  p <- ggpar(p, palette = palette, ggtheme = ggtheme, xlab = xlab, ylab = ylab, title = title, ...)

  p
}

context("test-ggpaired-jitter")

# #407: ggpaired() gains an opt-in `jitter` argument that spreads the paired
# points horizontally to reduce overlap, while keeping each pair's connecting
# line intact (one offset per subject id) and never moving the values (y).
# Default jitter = 0 leaves the output byte-identical.

before <- c(200.1, 190.9, 192.7, 213, 241.4, 196.9, 172.2, 185.5, 205.2, 193.7)
after  <- c(392.9, 393.2, 345.1, 393, 434, 427.9, 422, 383.9, 392.3, 352.2)
d <- data.frame(before = before, after = after)

.layers <- function(p) {
  b <- ggplot2::ggplot_build(p)
  pt <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomPoint"), logical(1)))[1]
  ln <- which(vapply(p$layers, function(l) inherits(l$geom, "GeomLine"), logical(1)))[1]
  list(pt = b$data[[pt]], ln = b$data[[ln]])
}

test_that("default jitter = 0 is byte-identical to omitting it", {
  p_omit <- ggpaired(d, cond1 = "before", cond2 = "after", fill = "condition")
  p_zero <- ggpaired(d, cond1 = "before", cond2 = "after", fill = "condition", jitter = 0)
  expect_equal(ggplot2::ggplot_build(p_omit)$data, ggplot2::ggplot_build(p_zero)$data)
  # points sit exactly on x = 1 and x = 2 (no spread)
  expect_equal(sort(unique(round(.layers(p_zero)$pt$x, 6))), c(1, 2))
})

test_that("jitter > 0 spreads the points but does not move the values", {
  p <- ggpaired(d, cond1 = "before", cond2 = "after", jitter = 0.1)
  L <- .layers(p)
  expect_gt(diff(range(L$pt$x)), 0.15)                 # horizontal spread
  expect_equal(sort(L$pt$y), sort(c(before, after)))   # y untouched
  expect_silent(ggplot2::ggplotGrob(p))
})

test_that("the connecting lines follow the jittered points", {
  p <- ggpaired(d, cond1 = "before", cond2 = "after", jitter = 0.12)
  L <- .layers(p)
  lv <- as.matrix(L$ln[order(L$ln$x, L$ln$y), c("x", "y")])
  pv <- as.matrix(L$pt[order(L$pt$x, L$pt$y), c("x", "y")])
  expect_equal(lv, pv, tolerance = 1e-6)
})

test_that("jitter is paired: each subject's two points share the same offset", {
  p <- ggpaired(d, cond1 = "before", cond2 = "after", jitter = 0.1)
  pt <- .layers(p)$pt
  cond <- ifelse(pt$x < 1.5, "before", "after")
  ob <- (pt$x[cond == "before"] - 1)[order(match(round(pt$y[cond == "before"], 4), round(before, 4)))]
  oa <- (pt$x[cond == "after"]  - 2)[order(match(round(pt$y[cond == "after"], 4),  round(after, 4)))]
  expect_equal(ob, oa, tolerance = 1e-6)
})

test_that("jitter is reproducible across calls (fixed seed) and leaves the RNG untouched", {
  p1 <- ggpaired(d, cond1 = "before", cond2 = "after", jitter = 0.1)
  p2 <- ggpaired(d, cond1 = "before", cond2 = "after", jitter = 0.1)
  expect_equal(.layers(p1)$pt$x, .layers(p2)$pt$x)

  set.seed(999); r1 <- runif(3)
  set.seed(999); invisible(ggpaired(d, cond1 = "before", cond2 = "after", jitter = 0.1)); r2 <- runif(3)
  expect_equal(r1, r2)
})

test_that("jitter works in x/y mode with color and a supplied id", {
  tg <- ToothGrowth
  tg$id <- rep(1:30, 2)
  p <- ggpaired(tg, x = "supp", y = "len", color = "supp", id = "id", jitter = 0.08)
  L <- .layers(p)
  lv <- as.matrix(L$ln[order(L$ln$x, L$ln$y), c("x", "y")])
  pv <- as.matrix(L$pt[order(L$pt$x, L$pt$y), c("x", "y")])
  expect_equal(lv, pv, tolerance = 1e-6)
  expect_silent(ggplot2::ggplotGrob(p))
})

# Transform data into long format--------------------
# Replacement of deprecated tidyr::gather_()
# wrapper around tidyr::pivot_longer() but using `cols_vary = "slowest"` as default
# instead of `cols_vary = "fastest"`
df_gather <- function(data, cols, cols_vary = "slowest",
                      names_to = "name", values_to = "value", ...) {
  data %>%
    tidyr::pivot_longer(
      cols = dplyr::all_of(cols), names_to = names_to, values_to = values_to,
      cols_vary = cols_vary, ...
    )
}

# Wide data
get_wide_data <- function() {
  tibble::tibble(
    before = c(200.1, 190.9, 192.7),
    after = c(392.9, 393.2, 345.1)
  )
}


test_that("df_gather is sorting rows as done by gather_()", {
  df <- get_wide_data()
  expected_output <- tibble::tribble(
    ~condition, ~val,
    "before", 200.1,
    "before", 190.9,
    "before", 192.7,
    "after", 392.9,
    "after", 393.2,
    "after", 345.1
  )
  observed_output <- df_gather(
    df,
    cols = c("before", "after"),
    names_to = "condition", values_to = "val"
  )
  expect_equal(observed_output, expected_output)
})

#' @include utilities.R
NULL
#' Point shapes available in R
#' @description Show point shapes available in R.
#' @return a ggplot.
#'
#' @seealso \code{\link{ggpar}} and \code{\link{ggline}}.
#' @examples
#' show_point_shapes() +
#'   theme_minimal()
#' @export
show_point_shapes <- function() {
  d <- data.frame(p = c(0:25))
  p <- ggplot() +
    scale_y_continuous(name = "") +
    scale_x_continuous(name = "") +
    scale_shape_identity() +
    geom_point(data = d, mapping = aes(x = p %% 6, y = p %/% 6, shape = p), size = 5, fill = "blue") +
    geom_text(data = d, mapping = aes(x = p %% 6, y = p %/% 6 + 0.25, label = p), size = 3) +
    scale_y_reverse() +
    theme(
      axis.title = element_blank()
    )
  ggpar(p,
    ticks = FALSE, tickslab = FALSE,
    main = "Point shapes available in R"
  )
}

#' @include utilities.R ggpar.R
NULL
#' Box plot
#' @description Create a box plot with points. Box plots display a group of
#'  numerical data through their quartiles.
#' @param data a data frame
#' @param x character string containing the name of x variable.
#' @param y character vector containing one or more variables to plot
#' @param combine logical value. Default is FALSE. Used only when y is a vector
#'  containing multiple variables to plot. If TRUE, create a multi-panel plot by
#'  combining the plot of y variables.
#' @param merge logical or character value. Default is FALSE. Used only when y is
#'  a vector containing multiple variables to plot. If TRUE, merge multiple y
#'  variables in the same plotting area. Allowed values include also "asis"
#'  (TRUE) and "flip". If merge = "flip", then y variables are used as x tick
#'  labels and the x variable is used as grouping variable.
#' @param color outline color.
#' @param fill fill color.
#' @param palette the color palette to be used for coloring or filling by groups.
#'  Allowed values include "grey" for grey color palettes; brewer palettes e.g.
#'  "RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
#'  scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
#'  "lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".
#' @param bxp.errorbar logical value. If TRUE, shows error bars of box plots.
#' @param bxp.errorbar.width numeric value specifying the width of box plot error
#'  bars. Default is 0.4.
#' @param linetype line types.
#' @param size Numeric value (e.g.: size = 1). change the size of points and
#'  outlines.
#' @param linewidth constant value specifying the line width.
#' @param width numeric value between 0 and 1 specifying the width of the plot elements.
#' @inheritParams ggplot2::geom_boxplot
#' @param outlier.shape point shape of outlier. Default is 19. To hide outlier,
#'  specify \code{outlier.shape = NA}. When jitter is added, then outliers will
#'  be automatically hidden.
#' @inheritParams facet
#' @inheritParams ggpar
#' @inheritParams ggtext
#' @param select character vector specifying which items to display.
#' @param remove character vector specifying which items to remove from the plot.
#' @param order character vector specifying the order of items.
#' @param add character vector for adding another plot element (e.g.: dot plot or
#'  error bars). Allowed values are one or the combination of: "none",
#'  "dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
#'  "mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
#'  "median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
#'  details.
#' @param add.params parameters (color, shape, size, fill, linetype) for the
#'  argument 'add'; e.g.: add.params = list(color = "red").
#' @param error.plot plot type used to visualize error. Allowed values are one of
#'  c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
#'  "lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
#'  "lower_linerange"). Default value is "pointrange" or "errorbar". Used only
#'  when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
#'  se, ....
#' @param font.label a list which can contain the combination of the following
#'  elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
#'  "bold.italic") and the color (e.g.: "red") of labels. For example font.label
#'  = list(size = 14, face = "bold", color ="red"). To specify only the size and
#'  the style, use font.label = list(size = 14, face = "plain").
#' @param position position adjustment, either as a string, or the result of a
#'  call to a position adjustment function (e.g. \code{position_dodge(0.8)}).
#'  Used to control the spacing between grouped elements.
#' @param show.n logical. If TRUE, displays the number of observations
#'  (\code{"n = <count>"}) at the top of each group. Off by default. When the
#'  groups are dodged (a \code{color}/\code{fill} grouping with a dodging
#'  \code{position}), one count is shown per group; otherwise a single count is
#'  shown per x-axis tick. Counts respect \code{select}/\code{remove} and are
#'  computed per facet.
#' @param ggtheme function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
#'  Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
#'  theme_minimal(), theme_classic(), theme_void(), ....
#' @param ... other arguments to be passed to
#'  \code{\link[ggplot2]{geom_boxplot}}, \code{\link{ggpar}} and
#'  \code{\link{facet}}.
#' @details The plot can be easily customized using the function ggpar(). Read
#'  ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
#'  ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
#'  = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
#'  and position: legend = "right" \item plot orientation : orientation =
#'  c("vertical", "horizontal", "reverse") }
#'
#' @section Suggestions for the argument "add": Suggested values are one of
#'  c("dotplot", "jitter").
#'
#' @seealso \code{\link{ggpar}}, \code{\link{ggviolin}}, \code{\link{ggdotplot}}
#'  and \code{\link{ggstripchart}}.
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' # +++++++++++++++++++++++++++
#' # width: change box plots width
#' ggboxplot(df, x = "dose", y = "len", width = 0.8)
#'
#' # Change orientation: horizontal
#' ggboxplot(df, "dose", "len", orientation = "horizontal")
#'
#' # Notched box plot
#' ggboxplot(df,
#'   x = "dose", y = "len",
#'   notch = TRUE
#' )
#'
#' # Add dots
#' # ++++++++++++++++++++++++++
#' ggboxplot(df,
#'   x = "dose", y = "len",
#'   add = "dotplot"
#' )
#'
#' # Add jitter points and change the shape by groups
#' ggboxplot(df,
#'   x = "dose", y = "len",
#'   add = "jitter", shape = "dose"
#' )
#'
#'
#' # Select and order items
#' # ++++++++++++++++++++++++++++++
#'
#' # Select which items to display: "0.5" and "2"
#' ggboxplot(df, "dose", "len",
#'   select = c("0.5", "2")
#' )
#'
#' # Change the default order of items
#' ggboxplot(df, "dose", "len",
#'   order = c("2", "1", "0.5")
#' )
#'
#'
#' # Change colors
#' # +++++++++++++++++++++++++++
#' # Change outline and fill colors
#' ggboxplot(df, "dose", "len",
#'   color = "black", fill = "gray"
#' )
#'
#' # Change outline colors by groups: dose
#' # Use custom color palette
#' # Add jitter points and change the shape by groups
#' ggboxplot(df, "dose", "len",
#'   color = "dose", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
#'   add = "jitter", shape = "dose"
#' )
#'
#' # Change fill color by groups: dose
#' ggboxplot(df, "dose", "len",
#'   fill = "dose", palette = c("#00AFBB", "#E7B800", "#FC4E07")
#' )
#'
#'
#' # Box plot with multiple groups
#' # +++++++++++++++++++++
#' # fill or color box plot by a second group : "supp"
#' ggboxplot(df, "dose", "len",
#'   color = "supp",
#'   palette = c("#00AFBB", "#E7B800")
#' )
#'
#' @export
ggboxplot <- function(data, x, y, combine = FALSE, merge = FALSE,
                      color = "black", fill = "white", palette = NULL,
                      title = NULL, xlab = NULL, ylab = NULL,
                      bxp.errorbar = FALSE, bxp.errorbar.width = 0.4,
                      facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                      linetype = "solid", size = NULL, linewidth = NULL, width = 0.7, notch = FALSE,
                      outliers = TRUE, outlier.shape = 19,
                      select = NULL, remove = NULL, order = NULL,
                      add = "none", add.params = list(),
                      error.plot = "pointrange",
                      label = NULL, font.label = list(size = 11, color = "black"),
                      label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                      position = position_dodge(0.8),
                      ggtheme = theme_pubr(), show.n = FALSE, ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    title = title, xlab = xlab, ylab = ylab,
    bxp.errorbar = bxp.errorbar, bxp.errorbar.width = bxp.errorbar.width,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    linetype = linetype, size = size, linewidth = linewidth, width = width, notch = notch,
    outliers = outliers, outlier.shape = outlier.shape,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle, position = position,
    ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }

  .opts$fun <- ggboxplot_core
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (isTRUE(show.n) && !missing(x)) {
    if (.is_list(p)) {
      p <- purrr::map(p, .add_show_n, x = x, color = color, fill = fill,
                      facet.by = facet.by, position = position)
    } else {
      p <- .add_show_n(p, x = x, color = color, fill = fill,
                       facet.by = facet.by, position = position)
    }
  }

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


ggboxplot_core <- function(data, x, y,
                           color = "black", fill = "white", palette = NULL,
                           linetype = "solid", size = NULL, linewidth = NULL, width = 0.7, notch = FALSE,
                           outliers = TRUE, outlier.shape = 19,
                           title = NULL, xlab = NULL, ylab = NULL,
                           bxp.errorbar = FALSE, bxp.errorbar.width = 0.4,
                           add = "none", add.params = list(),
                           error.plot = "pointrange",
                           ggtheme = theme_pubr(),
                           position = position_dodge(0.8),
                           ...) {
  line_width <- .resolve_linewidth_args(
    size = size, linewidth = linewidth, default_linewidth = NULL
  )
  size <- line_width$size
  linewidth <- line_width$linewidth

  if (!is.factor(data[[x]])) data[[x]] <- as.factor(data[[x]])
  if ("jitter" %in% add) outlier.shape <- NA

  p <- ggplot(data, create_aes(list(x = x, y = y)))
  if (bxp.errorbar) {
    if (fill %in% colnames(data)) {
      # Important, so that the fill grouping is taken into account in the errorbar
      p <- p + geom_exec(
        geomfunc = stat_boxplot, data = data, geom = "errorbar", width = bxp.errorbar.width,
        color = color, fill = fill, linetype = linetype, position = position
      )
    } else {
      p <- p + geom_exec(
        geomfunc = stat_boxplot, data = data, geom = "errorbar", width = bxp.errorbar.width,
        color = color, linetype = linetype, position = position
      )
    }
  }

  p <- p + geom_exec(geom_boxplot,
    data = data,
    color = color, fill = fill, linetype = linetype,
    size = linewidth, width = width, notch = notch,
    outliers = outliers, outlier.shape = outlier.shape,
    position = position, ...
  )

  # Add
  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill, ...) %>%
    .add_item(p = p, add = add, error.plot = error.plot)
  p <- do.call(ggadd, add.params) %>%
    ggpar(
      palette = palette, ggtheme = ggtheme,
      title = title, xlab = xlab, ylab = ylab, ...
    )
  p
}

context("test-stat_friedman_test")

# Data preparation
df <- ToothGrowth
df$dose <- as.factor(df$dose)
df$id <- rep(1:10, 6) # Add individuals id
# Add a random grouping variable
set.seed(123)
df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
df <- df %>% mutate(
  len = ifelse(group == "grp2", len + 2, len),
  len = ifelse(group == "grp3", len + 7, len)
)


# Basic plots----------------------
test_that("stat_friedman_test works for basic plots", {
  df$id <- as.factor(c(rep(1:10, 3), rep(11:20, 3)))
  bxp <- ggboxplot(df, x = "dose", y = "len") +
    stat_friedman_test(wid = "id")
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), 1)
  expect_equal(as.numeric(stat.test$y), 36.4)
  expect_equal(stat.test$label, "Friedman test, p < 0.0001")
})


test_that("stat_friedman_test works for grouped plot: group by x var", {
  df$id <- as.factor(rep(1:10, 6))
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
    stat_friedman_test(aes(group = supp, color = supp),
      label = "p.format", wid = "id", group.by = "x.var"
    )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  # Replace possible "1." (with trailing dot) by "1"
  # this trailing dot appear only on linux
  stat.test$label <- gsub(pattern = "^1\\.$", replacement = "1", stat.test$label)
  expect_equal(as.numeric(stat.test$x), c(1, 2, 3))
  expect_equal(as.numeric(stat.test$y), c(36.4, 36.4, 36.4))
  expect_equal(stat.test$label, c("0.011", "0.058", "1"))
})

test_that("stat_friedman_test works for grouped one-way repeated measure anova: group by legend var", {
  df$id <- as.factor(rep(1:10, 6))
  bxp <- ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
    stat_friedman_test(aes(group = supp, color = supp),
      wid = "id", group.by = "legend.var"
    )
  bxp_build <- ggplot2::ggplot_build(bxp)
  stat.test <- bxp_build$data[[2]]
  expect_equal(as.numeric(stat.test$x), c(0.8, 0.8))
  expect_equal(as.numeric(stat.test$y), c(36.40, 39.62))
  expect_equal(stat.test$label, c("Friedman test, p = 0.02472", "Friedman test, p = 0.00012"))
})

# Regression test for #517: ggboxplot() ignored `coef` (it was not in geom_exec's
# allowed_options), so `coef = 0` could not be used to omit the whiskers.

.nowhisker <- function(p) {
  d <- ggplot2::ggplot_build(p)$data[[1]]
  all(abs(d$ymin - d$lower) < 1e-9 & abs(d$ymax - d$upper) < 1e-9)
}

test_that("ggboxplot(coef = 0) omits the whiskers (#517)", {
  p <- ggboxplot(ToothGrowth, "dose", "len", coef = 0)
  expect_true(.nowhisker(p))
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
})

test_that("ggboxplot() default keeps whiskers (no regression, #517)", {
  p <- ggboxplot(ToothGrowth, "dose", "len")
  expect_false(.nowhisker(p))
  # coef = 1.5 (the geom_boxplot default) is byte-identical to not passing coef
  d0 <- ggplot2::ggplot_build(p)$data[[1]]
  d1 <- ggplot2::ggplot_build(ggboxplot(ToothGrowth, "dose", "len", coef = 1.5))$data[[1]]
  expect_equal(d0$ymin, d1$ymin)
  expect_equal(d0$ymax, d1$ymax)
})

# Data preparation
data("ToothGrowth")
df <- ToothGrowth
# Adding column name with space
df[["spa ced"]] <- df[["supp"]]


test_that("facet works", {
  p <- ggpubr::ggboxplot(df, "dose", "len", facet.by = "supp")
  panels <- ggplot2::layer_data(p)[["PANEL"]] %>%
    unique() %>%
    as.numeric()
  expect_equal(panels, c(1, 2))
})


test_that("facet works when column names contain space", {
  p <- ggpubr::ggboxplot(df, "dose", "len", facet.by = "spa ced")
  panels <- ggplot2::layer_data(p)[["PANEL"]] %>%
    unique() %>%
    as.numeric()
  expect_equal(panels, c(1, 2))
})

#' Global Options for ggpubr
#' @description Displays allowed global options in ggpubr.
#' @examples
#'
#' ggpubr_options()
#'
#' @export
ggpubr_options <- function() {
  list(
    ggpubr.parse_aes = TRUE,
    ggpubr.null_device = cowplot::pdf_null_device
  )
}

# check symnum.args --------------------------------
fortify_signif_symbols_encoding <- function(symnum.args = list()) {
  if (.is_empty(symnum.args)) {
    symnum.args <- list(
      cutpoints = c(0, 1e-04, 0.001, 0.01, 0.05, Inf),
      symbols = c("****", "***", "**", "*", "ns")
    )
  } else {
    symnum.args.isok <- (length(symnum.args$symbols) == length(symnum.args$cutpoints) - 1)
    if (!symnum.args.isok) {
      stop(
        "Incorrect format detected in 'symnum.args'. ",
        "Check the documentation. ",
        "length(symbols) should be length(cutpoints)-1",
        call. = FALSE
      )
    }
  }
  symnum.args
}


#' Build symnum.args from Significance Parameters
#'
#' @description Internal helper function to build a symnum.args list from
#'   user-friendly significance parameters. This allows users to specify
#'   custom significance cutoffs and symbols without using the complex
#'   symnum.args format directly.
#'
#' @param signif.cutoffs Numeric vector of p-value cutoffs in descending order
#'   (e.g., \code{c(0.10, 0.05, 0.01)} or \code{c(0.10, 0.05, 0.01, 0.001)}).
#'   Values smaller than each cutoff receive the corresponding symbol.
#' @param signif.symbols Character vector of symbols matching signif.cutoffs.
#'   If NULL, auto-generated based on length: 3 cutoffs -> c("*", "**", "***"),
#'   4 cutoffs -> c("*", "**", "***", "****").
#' @param ns.symbol Character string for non-significant results. Default is "ns".
#'   Use "" (empty string) to show nothing for non-significant results.
#' @param use.four.stars Logical. If TRUE and signif.symbols is NULL, include
#'   four stars (****) for the most significant level. Default is FALSE.
#' @param symnum.args Existing symnum.args list. If provided and non-empty,
#'   it takes precedence over other parameters (for backward compatibility).
#'
#' @return A list suitable for use as symnum.args parameter.
#'
#' @details
#' Priority order:
#' 1. If symnum.args is provided (non-empty), use it directly
#' 2. If signif.cutoffs is provided, build symnum.args from it
#' 3. Otherwise, use package defaults
#'
#' @keywords internal
build_symnum_args <- function(signif.cutoffs = NULL,
                              signif.symbols = NULL,
                              ns.symbol = "ns",
                              use.four.stars = FALSE,
                              symnum.args = list()) {
  # Priority 1: If symnum.args is provided, use it directly (backward compatibility)
  if (!.is_empty(symnum.args)) {
    return(fortify_signif_symbols_encoding(symnum.args))
  }

  # Priority 2: If signif.cutoffs is provided, build from it
  if (!is.null(signif.cutoffs)) {
    # Validate cutoffs
    if (!is.numeric(signif.cutoffs) || any(signif.cutoffs <= 0) || any(signif.cutoffs >= 1)) {
      stop("signif.cutoffs must be numeric values between 0 and 1 (exclusive)", call. = FALSE)
    }

    # Sort cutoffs in descending order (largest to smallest)
    signif.cutoffs <- sort(signif.cutoffs, decreasing = TRUE)
    n_cutoffs <- length(signif.cutoffs)

    # Auto-generate symbols if not provided
    if (is.null(signif.symbols)) {
      # Generate symbols: *, **, ***, (****)
      if (use.four.stars && n_cutoffs >= 4) {
        signif.symbols <- vapply(
          seq_len(n_cutoffs),
          function(i) paste(rep("*", i), collapse = ""),
          FUN.VALUE = character(1)
        )
      } else if (n_cutoffs > 3 && !use.four.stars) {
        stop("signif.cutoffs has more than 3 levels but use.four.stars = FALSE. ",
          "Either set use.four.stars = TRUE or provide signif.symbols explicitly.",
          call. = FALSE
        )
      } else {
        signif.symbols <- vapply(
          seq_len(n_cutoffs),
          function(i) paste(rep("*", i), collapse = ""),
          FUN.VALUE = character(1)
        )
      }
    }

    # Validate symbols length
    if (length(signif.symbols) != n_cutoffs) {
      stop("signif.symbols must have the same length as signif.cutoffs (", n_cutoffs, ")",
        call. = FALSE
      )
    }

    # Build cutpoints: 0, cutoffs (ascending), Inf
    cutpoints <- c(0, rev(signif.cutoffs), Inf)

    # Build symbols: most significant first, then less significant, then ns
    symbols <- c(rev(signif.symbols), ns.symbol)

    return(list(cutpoints = cutpoints, symbols = symbols))
  }

  # Priority 3: Use package defaults, but respect ns.symbol if customized
  default_args <- fortify_signif_symbols_encoding(list())

  # If ns.symbol is different from default, update it
  if (ns.symbol != "ns") {
    # Replace the last symbol (ns) with the custom one
    default_args$symbols[length(default_args$symbols)] <- ns.symbol
  }

  default_args
}

# Check user specified label -----------------------------

# Check if is glue package expression
is_glue_expression <- function(label) {
  grepl("\\{|\\}", label, perl = TRUE)
}

# Check if label is a plotmath expression
contains_plotmath_symbols <- function(label) {
  grepl("==|italic\\s?\\(|bold\\s?\\(|bolditalic\\s?\\(", label)
}
starts_with_list <- function(label) {
  grepl("^list\\s?\\(", label)
}
is_plotmath_expression <- function(label) {
  starts_with_list(label) | contains_plotmath_symbols(label)
}

# Fortify label --------------------
# if label is plotmath expression, then
# fortify it in case users miss something
contains_p_signif <- function(label) {
  any(grepl("p(\\.adj|\\.format)?\\.signif", label))
}
contains_twoequal_signs <- function(label) {
  any(grepl("==", label))
}
replace_simple_by_double_equals <- function(label) {
  if (!contains_twoequal_signs(label)) {
    label <- gsub("=", "==", label)
  }
  label
}

escape_psignif_asteriks <- function(label) {
  # Escaping asteriks (special plotmath symbols) in p.signif or p.adj.signif by adding quote
  label <- gsub(pattern = "\\}\\{(p.*.signif)\\}", replacement = "}*`{\\1}`", x = label)
  # p signif preceded with space
  label <- gsub(pattern = "~\\{(p.*.signif)\\}", replacement = "~`{\\1}`", x = label)
  label <- gsub(pattern = "=+?\\s+?\\{(p.*.signif)}", replacement = "== `{\\1}`", x = label)
  label
}


# Get statistical test label to be displayed -------------------
# stat.test: statistical test output
# description: the description of the stat test, for example: "Anova"
# label: can be p, p.signif, p.adj or glue expression
add_stat_label <- function(stat.test, label = NULL) {
  is_plotmath <- FALSE
  stat.test <- add_p_format_signif(stat.test)
  if (is.null(label)) {
    stat.test$label <- add_p(stat.test$p.format)
  } else {
    is_plotmath <- is_plotmath_expression(label)
    if (is_plotmath) {
      label <- fortify_plotmath(label)
    }
    if (is_glue_expression(label)) {
      stat.test <- stat.test %>% mutate(label = glue(label))
    } else {
      if (!(label %in% colnames(stat.test))) {
        stop(
          "Can't find the value of the argument label ('", label, "') in the computed ",
          "statistical test table.",
          call. = FALSE
        )
      }
      stat.test$label <- as.character(stat.test[[label]])
    }
  }
  label <- gsub(pattern = "=+(\\s+)?<", replacement = "<\\1", stat.test$label)
  if (is_plotmath) {
    label <- replace_simple_by_double_equals(label)
    label <- gsub(pattern = "\\s", replacement = "~", label)
    label <- gsub(pattern = "~==~", replacement = "~`=`~", label)
    label <- gsub(pattern = "~<~", replacement = "~`<`~", label)
    # Make sure that decimal values will be displayed asis in character when parsed by ggplot
    # Add quote around numeric values
    label <- gsub("([0-9.-]+)", "'\\1'", label)
    # Escape minus in text
    label <- gsub("Kruskal'-'Wallis", "'Kruskal-Wallis'", label)
  }
  stat.test$label <- label
  stat.test
}

fortify_plotmath <- function(label) {
  label <- gsub(pattern = "~", replacement = " ", x = label, fixed = TRUE)
  if (!starts_with_list(label)) label <- paste0("list(", label, ")")
  if (contains_p_signif(label)) {
    # Escape p signif stars
    label <- gsub(pattern = "\\}\\{p.signif\\}", replacement = "}*`{p.signif}`", x = label)
    label <- gsub(pattern = "\\}\\{p.adj.signif\\}", replacement = "}*`{p.adj.signif}`", x = label)
    label <- gsub(pattern = "\\}\\{p.format.signif\\}", replacement = "}*`{p.format.signif}`", x = label)
    # Escape p signif stars preceded by space
    label <- gsub(pattern = "\\s\\{p.signif\\}", replacement = " `{p.signif}`", x = label)
    label <- gsub(pattern = "\\s\\{p.adj.signif\\}", replacement = " `{p.adj.signif}`", x = label)
    label <- gsub(pattern = "\\s\\{p.format.signif\\}", replacement = " `{p.format.signif}`", x = label)
    # Escape p signif stars preceded by equal signs
    label <- gsub(pattern = "=(\\s+)?\\{p.signif}", replacement = "=\\1`{p.signif}`", x = label)
    label <- gsub(pattern = "=(\\s+)?\\{p.adj.signif}", replacement = "=\\1`{p.adj.signif}`", x = label)
    label <- gsub(pattern = "=(\\s+)?\\{p.format.signif}", replacement = "=\\1`{p.format.signif}`", x = label)
  }
  label <- gsub(pattern = "eta2[g]", replacement = "eta[g]^2", x = label, fixed = TRUE)
  label <- gsub(pattern = "eta2[p]", replacement = "eta[p]^2", x = label, fixed = TRUE)
  label
}

# Add p prefix
# add_p(0.05) --> p = 0.05
# add_p("<0.05") --> p < 0.05
add_p <- function(label) {
  create_p_label(label)
}

# Add combined p.format + p.signif column when available
# Falls back to adjusted values when raw p-values are missing.
add_p_format_signif <- function(stat.test) {
  if ("p.format.signif" %in% colnames(stat.test)) {
    return(stat.test)
  }

  has_p <- all(c("p.format", "p.signif") %in% colnames(stat.test))
  has_adj <- all(c("p.adj.format", "p.adj.signif") %in% colnames(stat.test))
  if (!has_p && !has_adj) {
    return(stat.test)
  }

  if (has_p) {
    p_format <- stat.test$p.format
    p_signif <- stat.test$p.signif
  } else {
    p_format <- rep(NA_character_, nrow(stat.test))
    p_signif <- rep(NA_character_, nrow(stat.test))
  }

  if (length(p_format) == 0) {
    stat.test[["p.format.signif"]] <- character(0)
    return(stat.test)
  }

  if (has_adj) {
    use_adj <- is.na(p_format) | p_format == ""
    p_format[use_adj] <- stat.test$p.adj.format[use_adj]
    p_signif[use_adj] <- stat.test$p.adj.signif[use_adj]
  }

  if (all(is.na(p_format))) {
    return(stat.test)
  }

  p_signif[is.na(p_signif)] <- ""
  combined <- rep(NA_character_, length(p_format))
  ok <- !is.na(p_format)
  combined[ok] <- create_p_label(p_format[ok], p_signif[ok])
  stat.test[["p.format.signif"]] <- combined
  stat.test
}

# Add statistical test number of samples
add_stat_n <- function(stat.test) {
  stat.test$n <- rstatix::get_n(stat.test)
  stat.test
}


# gg layer data checking --------------------------
# Check whether there is multiple grouping variables
# This is the case for grouped plots
contains_multiple_grouping_vars <- function(data) {
  !all(data$x == data$group)
}

# Check if group variable is specified in aes
is_group_aes_specified <- function(mapping) {
  answer <- FALSE
  if (is.null(mapping)) {
    answer <- FALSE
  } else if (!is.null(mapping$group)) {
    answer <- TRUE
  }
  answer
}

# The dot-dot notation (`..p.signif..`) was deprecated in ggplot2 3.4.0.
# after_stat(p.signif) should be used. This function makes automatic
# conversion if user specified ..p.signif..
convert_label_dotdot_notation_to_after_stat <- function(mapping) {
  if (!is.null(mapping)) {
    label <- mapping$label
    if (!is.null(mapping$label)) {
      label <- rlang::as_label(mapping$label)
      dot_dot_labels <- c(
        "p.signif", "p.adj.signif", "p.format", "p.format.signif", "p", "p.adj",
        "eq.label", "adj.rr.label", "rr.label", "AIC.label", "BIC.label"
      )
      for (dot_dot_label in dot_dot_labels) {
        # Escape dots in the label name for regex matching
        escaped_label <- gsub("\\.", "\\\\.", dot_dot_label)
        # Use word boundaries that ensure the pattern is not part of a larger identifier
        # (?<![a-zA-Z0-9_.]) ensures no alphanumeric, underscore, or dot before
        # (?![a-zA-Z0-9_.]) ensures no alphanumeric, underscore, or dot after
        label <- gsub(
          pattern = paste0("(?<![a-zA-Z0-9_.])\\.\\.", escaped_label, "\\.\\.", "(?![a-zA-Z0-9_.])"),
          replacement = paste0("ggplot2::after_stat(", dot_dot_label, ")"),
          x = label,
          perl = TRUE
        )
      }
      # Only qualify unqualified after_stat() calls using negative lookbehind
      # This correctly handles mixed qualified and unqualified cases
      # replace after_stat() with ggplot2::after_stat()
      label <- gsub(
        pattern = "(?<!ggplot2::)after_stat\\s*\\(",
        replacement = "ggplot2::after_stat(",
        x = label,
        perl = TRUE
      )

      mapping$label <- parse(text = label)[[1]]
    }
  }
  mapping
}

context("test-add_stat_label")

stat.test <- data.frame(
  stringsAsFactors = FALSE,
  .y. = c("y", "y", "y"),
  group1 = c("1", "1", "2"),
  group2 = c("2", "3", "3"),
  n1 = c(20L, 20L, 20L),
  n2 = c(20L, 20L, 20L),
  statistic = c(33.5, 1.5, 61),
  p = c(7.02e-06, 8.41e-08, 0.000177),
  p.adj = c(1.4e-05, 2.52e-07, 0.000177),
  p.adj.signif = c("****", "****", "***"),
  method = c("Wilcoxon test", "Wilcoxon test", "Wilcoxon test"),
  xmin = c(1, 1, 2),
  xmax = c(2, 3, 3),
  p.signif = c("****", "****", "***"),
  p.format = c("<0.0001", "<0.0001", "0.00018"),
  p.adj.format = c("<0.0001", "<0.0001", "0.00018"),
  n = c(40L, 40L, 40L)
)


test_that("add_stat_label works with basic label", {
  formatted_p <- add_stat_label(stat.test, label = "p.format")
  adjusted_p <- add_stat_label(stat.test, label = "p.adj")
  expect_equal(formatted_p$label, c("<0.0001", "<0.0001", "0.00018"))
  expect_equal(adjusted_p$label, c("1.4e-05", "2.52e-07", "0.000177"))
})

test_that("add_stat_label works with glue expression", {
  glueed <- add_stat_label(stat.test, label = "p = {p.format} {p.signif}")
  expect_equal(glueed$label, c("p < 0.0001 ****", "p < 0.0001 ****", "p = 0.00018 ***"))
})

test_that("add_stat_label supports p.format.signif label", {
  combined <- add_stat_label(stat.test, label = "p.format.signif")
  expect_equal(combined$label, c("p < 0.0001 ****", "p < 0.0001 ****", "p = 0.00018 ***"))
})

test_that("add_stat_label falls back to adjusted p-values when raw p is missing", {
  stat.test.adj <- data.frame(
    stringsAsFactors = FALSE,
    p.format = c(NA_character_),
    p.signif = c(NA_character_),
    p.adj.format = c("<0.001"),
    p.adj.signif = c("***")
  )
  combined <- add_stat_label(stat.test.adj, label = "p.format.signif")
  expect_equal(combined$label, "p < 0.001 ***")
})

test_that("add_stat_label works with plotmath: Usage of simple equal '='", {
  simple_equals <- add_stat_label(stat.test, label = "italic(p)={p.format}{p.signif}")
  simple_equals_space <- add_stat_label(stat.test, label = "italic(p) = {p.format}{p.signif}")
  simple_equals_tilde <- add_stat_label(stat.test, label = "italic(p)~=~{p.format}{p.signif}")
  expect_equal(
    simple_equals$label,
    c("list(italic(p)<'0.0001'*`****`)", "list(italic(p)<'0.0001'*`****`)", "list(italic(p)=='0.00018'*`***`)")
  )
  expect_equal(
    simple_equals_space$label,
    c("list(italic(p)~`<`~'0.0001'*`****`)", "list(italic(p)~`<`~'0.0001'*`****`)", "list(italic(p)~`=`~'0.00018'*`***`)")
  )
  expect_equal(
    simple_equals_tilde$label,
    c("list(italic(p)~`<`~'0.0001'*`****`)", "list(italic(p)~`<`~'0.0001'*`****`)", "list(italic(p)~`=`~'0.00018'*`***`)")
  )
})


test_that("add_stat_label works with plotmath: Escaping stars in p.signif/p.adj.signif", {
  # datapasta::dpasta(as.character(res$label))
  stars <- add_stat_label(stat.test, label = "italic(p)={p.format}{p.signif}")
  stars_space <- add_stat_label(stat.test, label = "italic(p)={p.format} {p.signif}")
  stars_tilde <- add_stat_label(stat.test, label = "italic(p)={p.format}~{p.signif}")
  stars_equals <- add_stat_label(stat.test, label = "italic(p)={p.signif}")
  stars_equals_space <- add_stat_label(stat.test, label = "italic(p) = {p.signif}")
  expect_equal(
    stars$label,
    c("list(italic(p)<'0.0001'*`****`)", "list(italic(p)<'0.0001'*`****`)", "list(italic(p)=='0.00018'*`***`)")
  )
  expect_equal(
    stars_space$label,
    c("list(italic(p)<'0.0001'~`****`)", "list(italic(p)<'0.0001'~`****`)", "list(italic(p)=='0.00018'~`***`)")
  )
  expect_equal(
    stars_tilde$label,
    c("list(italic(p)<'0.0001'~`****`)", "list(italic(p)<'0.0001'~`****`)", "list(italic(p)=='0.00018'~`***`)")
  )
  expect_equal(
    stars_equals$label,
    c("list(italic(p)==`****`)", "list(italic(p)==`****`)", "list(italic(p)==`***`)")
  )
  expect_equal(
    stars_equals_space$label,
    c("list(italic(p)~`=`~`****`)", "list(italic(p)~`=`~`****`)", "list(italic(p)~`=`~`***`)")
  )
})


test_that("add_stat_label works with ANOVA stats label formats", {
  res.aov <- data.frame(
    stringsAsFactors = FALSE,
    check.names = FALSE,
    Effect = c("x"),
    DFn = c(2),
    DFd = c(57),
    F = c(29.543),
    p = c(1.57e-09),
    `p<.05` = c("*"),
    ges = c(0.509),
    p.adj = c(1.57e-09),
    p.signif = c("****"),
    p.adj.signif = c("****"),
    p.format = c("<0.0001"),
    p.adj.format = c("<0.0001"),
    n = c(60L),
    method = c("Anova")
  )
  res.italic <- add_stat_label(res.aov, label = get_anova_test_label_template("as_italic"))
  res.detailed <- add_stat_label(res.aov, label = get_anova_test_label_template("as_detailed"))
  res.detailed.italic <- add_stat_label(res.aov, label = get_anova_test_label_template("as_detailed_italic"))
  res.detailed.expression <- add_stat_label(res.aov, label = get_anova_test_label_template("as_detailed_expression"))
  res.plotmath <- add_stat_label(res.aov, label = "bold(Anova), italic(F)({DFn}, {DFd}) = {F}, eta2[g] = {ges}, italic(p) = {p.format}{p.signif}, italic(n) = {n}")
  res.rounding <- add_stat_label(res.aov, label = "Anova, italic(F)({DFn}, {DFd}) = {round(F, 1)}, eta2[g] = {round(ges, 1)}, italic(p) = {p.format}{p.signif}, italic(n) = {n}")
  # Custom p format
  res.customp <- add_stat_label(res.aov, label = "Anova, italic(p) = {rstatix::p_format(p,  accuracy = 0.0001, digits = 3, leading.zero = FALSE)}{p.signif}")

  expect_equal(res.italic$label, "list(Anova,~italic(p)~`<`~'0.0001')")
  expect_equal(res.detailed$label, "Anova, F(2, 57) = 29.543, eta2[g] = 0.509, p < 0.0001, n = 60")
  expect_equal(res.detailed.italic$label, "list(Anova,~italic(F)('2',~'57')~`=`~'29.543',~eta[g]^'2'~`=`~'0.509',~italic(p)~`<`~'0.0001',~italic(n)~`=`~'60')")
  expect_equal(res.detailed.expression$label, "list(Anova,~italic(F)('2',~'57')~`=`~'29.543',~eta[g]^'2'~`=`~'0.509',~italic(p)~`<`~'0.0001',~italic(n)~`=`~'60')")
  expect_equal(res.plotmath$label, "list(bold(Anova),~italic(F)('2',~'57')~`=`~'29.543',~eta[g]^'2'~`=`~'0.509',~italic(p)~`<`~'0.0001'*`****`,~italic(n)~`=`~'60')")
  expect_equal(res.rounding$label, "list(Anova,~italic(F)('2',~'57')~`=`~'29.5',~eta[g]^'2'~`=`~'0.5',~italic(p)~`<`~'0.0001'*`****`,~italic(n)~`=`~'60')")
  expect_equal(res.customp$label, "list(Anova,~italic(p)~`<`~'.0001'*`****`)")
})


test_that("add_stat_label works with Kruskal-Wallis stats label formats", {
  res <- data.frame(
    stringsAsFactors = FALSE,
    x = c("1"),
    y = c(36.4),
    n = c(60L),
    statistic = c(31.21),
    df = c(2L),
    p = c(1.67e-07),
    method = c("Kruskal-Wallis"),
    p.adj = c(1.67e-07),
    p.signif = c("****"),
    p.adj.signif = c("****"),
    p.format = c("<0.0001"),
    p.adj.format = c("<0.0001")
  )
  res.italic <- add_stat_label(res, label = get_kruskal_test_label_template("as_italic"))
  res.detailed <- add_stat_label(res, label = get_kruskal_test_label_template("as_detailed"))
  res.detailed.italic <- add_stat_label(res, label = get_kruskal_test_label_template("as_detailed_italic"))
  res.detailed.expression <- add_stat_label(res, label = get_kruskal_test_label_template("as_detailed_expression"))

  expect_equal(res.italic$label, "list('Kruskal-Wallis',~italic(p)~`<`~'0.0001')")
  expect_equal(res.detailed$label, "Kruskal-Wallis, X2(2) = 31.21, p < 0.0001, n = 60")
  expect_equal(res.detailed.italic$label, "list('Kruskal-Wallis',~italic(chi)^'2'~('2')~`=`~'31.21',~italic(p)~`<`~'0.0001',~italic(n)~`=`~'60')")
  expect_equal(res.detailed.expression$label, "list('Kruskal-Wallis',~italic(chi)^'2'('2')~`=`~'31.21',~italic(p)~`<`~'0.0001',~italic(n)~`=`~'60')")
})


test_that("add_stat_label works with Welch ANOVA detailed label formats", {
  res <- data.frame(
    stringsAsFactors = FALSE,
    x = c("1"),
    y = c(36.4),
    n = c(60L),
    statistic = c(45.57),
    DFn = c(2),
    DFd = c(37.7432475430268),
    p = c(1.23e-10),
    method = c("Welch Anova"),
    p.adj = c(1.23e-10),
    p.signif = c("****"),
    p.adj.signif = c("****"),
    p.format = c("<0.0001"),
    p.adj.format = c("<0.0001")
  )
  res.detailed <- add_stat_label(res, label = get_welch_anova_test_label_template("as_detailed"))
  res.detailed.italic <- add_stat_label(res, label = get_welch_anova_test_label_template("as_detailed_italic"))
  res.detailed.expression <- add_stat_label(res, label = get_welch_anova_test_label_template("as_detailed_expression"))

  expect_equal(res.detailed$label, "Welch Anova, F(2, 37.7432475430268) = 45.57, p < 0.0001, n = 60")
  expect_equal(res.detailed.italic$label, "list(Welch~Anova,~italic(F)('2',~'37.7432475430268')~`=`~'45.57',~italic(p)~`<`~'0.0001',~italic(n)~`=`~'60')")
  expect_equal(res.detailed.expression$label, "list(Welch~Anova,~italic(F)('2',~'37.7432475430268')~`=`~'45.57',~italic(p)~`<`~'0.0001',~italic(n)~`=`~'60')")
  expect_false(any(grepl("FALSE", c(res.detailed.italic$label, res.detailed.expression$label), fixed = TRUE)))
})


# Tests for build_symnum_args helper function
test_that("build_symnum_args uses symnum.args when provided (backward compatibility)", {
  result <- build_symnum_args(
    symnum.args = list(cutpoints = c(0, 0.01, 0.05, 1), symbols = c("**", "*", "ns"))
  )
  expect_equal(result$cutpoints, c(0, 0.01, 0.05, 1))
  expect_equal(result$symbols, c("**", "*", "ns"))
})

test_that("build_symnum_args builds from signif.cutoffs (3 levels)", {
  result <- build_symnum_args(
    signif.cutoffs = c(0.10, 0.05, 0.01)
  )
  # Cutoffs should be sorted ascending and wrapped with 0 and Inf
  expect_equal(result$cutpoints, c(0, 0.01, 0.05, 0.10, Inf))
  # Symbols should be: most significant first, then less, then ns
  expect_equal(result$symbols, c("***", "**", "*", "ns"))
})

test_that("build_symnum_args builds from signif.cutoffs (4 levels with use.four.stars)", {
  result <- build_symnum_args(
    signif.cutoffs = c(0.10, 0.05, 0.01, 0.001),
    use.four.stars = TRUE
  )
  expect_equal(result$cutpoints, c(0, 0.001, 0.01, 0.05, 0.10, Inf))
  expect_equal(result$symbols, c("****", "***", "**", "*", "ns"))
})

test_that("build_symnum_args uses custom signif.symbols", {
  result <- build_symnum_args(
    signif.cutoffs = c(0.10, 0.05, 0.01),
    signif.symbols = c("+", "++", "+++")
  )
  expect_equal(result$symbols, c("+++", "++", "+", "ns"))
})

test_that("build_symnum_args uses custom ns.symbol", {
  result <- build_symnum_args(
    signif.cutoffs = c(0.10, 0.05, 0.01),
    ns.symbol = "N.S."
  )
  expect_equal(result$symbols[4], "N.S.")
})

test_that("build_symnum_args uses empty ns.symbol", {
  result <- build_symnum_args(
    signif.cutoffs = c(0.10, 0.05, 0.01),
    ns.symbol = ""
  )
  expect_equal(result$symbols[4], "")
})

test_that("build_symnum_args errors when signif.cutoffs has 4+ levels without use.four.stars", {
  expect_error(
    build_symnum_args(signif.cutoffs = c(0.10, 0.05, 0.01, 0.001)),
    "use.four.stars"
  )
})

test_that("build_symnum_args errors when signif.symbols length doesn't match", {
  expect_error(
    build_symnum_args(
      signif.cutoffs = c(0.10, 0.05, 0.01),
      signif.symbols = c("+", "++") # Only 2, but need 3
    ),
    "same length"
  )
})

test_that("build_symnum_args uses defaults when nothing provided", {
  result <- build_symnum_args()
  # Should use package defaults
  expect_equal(result$cutpoints, c(0, 1e-04, 0.001, 0.01, 0.05, Inf))
  expect_equal(result$symbols, c("****", "***", "**", "*", "ns"))
})

test_that("convert_label_dotdot_notation_to_after_stat handles NULL inputs", {
  # NULL mapping
  expect_null(convert_label_dotdot_notation_to_after_stat(NULL))

  # mapping with NULL label (simulate aes without label)
  mapping <- aes(x = var1, y = var2)
  mapping$label <- NULL
  result <- convert_label_dotdot_notation_to_after_stat(mapping)
  expect_equal(result, mapping)

  # Empty mapping
  mapping <- aes()
  result <- convert_label_dotdot_notation_to_after_stat(mapping)
  expect_equal(result, mapping)
})


test_that("convert_label_dotdot_notation_to_after_stat converts dot-dot notation", {
  # Single dot-dot variable
  mapping <- aes(label = ..eq.label..)
  result <- convert_label_dotdot_notation_to_after_stat(mapping)
  observed_label <- rlang::as_label(result$label)
  expected_label <- "ggplot2::after_stat(eq.label)"
  expect_equal(observed_label, expected_label)

  # Multiple dot-dot variables in paste
  mapping <- aes(label = paste(..eq.label.., ..adj.rr.label.., sep = "~~~~"))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = paste(ggplot2::after_stat(eq.label),
    ggplot2::after_stat(adj.rr.label),
    sep = "~~~~"
  ))
  expect_equal(observed_mapping$label, expected_mapping$label)


  # Statistical test variables
  mapping <- aes(label = ..p.signif..)
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = ggplot2::after_stat(p.signif))
  expect_equal(observed_mapping$label, expected_mapping$label)
})


test_that("convert_label_dotdot_notation_to_after_stat qualifies unqualified after_stat", {
  # Basic after_stat
  mapping <- aes(label = after_stat(eq.label))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = ggplot2::after_stat(eq.label))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # after_stat with spaces (using tidy eval)
  mapping <- aes(label = !!rlang::parse_expr("after_stat (eq.label)"))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = ggplot2::after_stat(eq.label))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # after_stat with multiple spaces
  mapping <- aes(label = !!rlang::parse_expr("after_stat  (eq.label)"))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = ggplot2::after_stat(eq.label))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Multiple after_stat calls
  mapping <- aes(label = !!rlang::parse_expr("paste(after_stat(eq.label), after_stat(rr.label))"))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = paste(ggplot2::after_stat(eq.label), ggplot2::after_stat(rr.label)))
  expect_equal(observed_mapping$label, expected_mapping$label)
})


test_that("convert_label_dotdot_notation_to_after_stat preserves qualified after_stat", {
  # Already qualified - should not change
  mapping <- aes(label = ggplot2::after_stat(eq.label))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = ggplot2::after_stat(eq.label))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Already qualified with spaces - should not change
  # Note: This test case requires programmatic construction due to spaces
  # In practice, this would be created by the ggplot2 parser
  mapping <- list(label = quote(ggplot2::after_stat(eq.label)))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = quote(ggplot2::after_stat(eq.label)))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Mixed qualified and unqualified
  mapping <- aes(label = paste(ggplot2::after_stat(eq.label), after_stat(rr.label)))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = paste(ggplot2::after_stat(eq.label), ggplot2::after_stat(rr.label)))
  expect_equal(observed_mapping$label, expected_mapping$label)
})

test_that("convert_label_dotdot_notation_to_after_stat handles complex expressions", {
  # Nested function calls
  mapping <- list(label = parse(text = "format(..p.format.., digits = 3)")[[1]])
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = parse(text = "format(ggplot2::after_stat(p.format), digits = 3)")[[1]])
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Mathematical operations
  mapping <- list(label = parse(text = "..eq.label.. * 2")[[1]])
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = parse(text = "ggplot2::after_stat(eq.label) * 2")[[1]])
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Conditional expressions
  mapping <- list(label = parse(text = 'ifelse(..p.. < 0.05, ..p.signif.., "ns")')[[1]])
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = parse(text = 'ifelse(ggplot2::after_stat(p) < 0.05, ggplot2::after_stat(p.signif), "ns")')[[1]])
  expect_equal(observed_mapping$label, expected_mapping$label)
})

test_that("convert_label_dotdot_notation_to_after_stat preserves other aesthetics", {
  # Multiple aesthetics - only label should be modified
  mapping <- aes(x = var1, y = var2, color = group, label = ..eq.label..)
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(x = var1, y = var2, color = group, label = ggplot2::after_stat(eq.label))

  # Other aesthetics should be unchanged
  expect_equal(observed_mapping$x, expected_mapping$x)
  expect_equal(observed_mapping$y, expected_mapping$y)
  expect_equal(observed_mapping$colour, expected_mapping$colour) # Note: aes() converts 'color' to 'colour'

  # Only label should be converted
  expect_equal(observed_mapping$label, expected_mapping$label)
})

test_that("convert_label_dotdot_notation_to_after_stat handles edge cases", {
  # String containing after_stat but not as function call (should not change)
  mapping <- list(label = quote("This is after_stat text"))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = quote("This is after_stat text"))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Variable names containing dot-dot pattern but not exact match
  mapping <- list(label = quote(my..eq.label..var))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = quote(my..eq.label..var))
  # This should not be converted as it's not an exact ..eq.label.. match
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Empty label expression
  mapping <- list(label = NULL)
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- list(label = NULL)
  expect_equal(observed_mapping$label, expected_mapping$label)
})


test_that("convert_label_dotdot_notation_to_after_stat handles real-world examples", {
  # Example from bSi package
  mapping <- aes(label = after_stat(eq.label))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = ggplot2::after_stat(eq.label))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # ggpubr documentation example
  mapping <- aes(label = paste(..eq.label.., ..adj.rr.label.., sep = "~~~~"))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = paste(ggplot2::after_stat(eq.label), ggplot2::after_stat(adj.rr.label), sep = "~~~~"))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Statistical significance example
  mapping <- aes(label = paste("p =", ..p.format.., ..p.signif..))
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = paste("p =", ggplot2::after_stat(p.format), ggplot2::after_stat(p.signif)))
  expect_equal(observed_mapping$label, expected_mapping$label)
})


test_that("convert_label_dotdot_notation_to_after_stat correctly uses rlang::as_label", {
  # Test that the function correctly extracts labels as character strings
  # This tests the internal behavior that the function uses rlang::as_label()

  # Create a mapping with a complex expression
  mapping <- aes(label = paste(..eq.label.., "R² =", ..adj.rr.label..))

  # Extract what rlang::as_label would produce
  label_as_char <- rlang::as_label(mapping$label)
  expected_char <- 'paste(..eq.label.., "R² =", ..adj.rr.label..)'
  expect_equal(label_as_char, expected_char)

  # Test that the conversion works correctly
  observed_mapping <- convert_label_dotdot_notation_to_after_stat(mapping)
  expected_mapping <- aes(label = paste(ggplot2::after_stat(eq.label), "R² =", ggplot2::after_stat(adj.rr.label)))
  expect_equal(observed_mapping$label, expected_mapping$label)

  # Test with after_stat expression
  mapping2 <- aes(label = after_stat(eq.label))
  label_as_char2 <- rlang::as_label(mapping2$label)
  expected_char2 <- "after_stat(eq.label)"
  expect_equal(label_as_char2, expected_char2)

  # Test the conversion
  observed_mapping2 <- convert_label_dotdot_notation_to_after_stat(mapping2)
  expected_mapping2 <- aes(label = ggplot2::after_stat(eq.label))
  expect_equal(observed_mapping2$label, expected_mapping2$label)
})

#' @include utilities.R
NULL
#' Add Summary Statistics or a Geom onto a ggplot
#' @description Add summary statistics or a geometry onto a ggplot.
#' @inheritParams add_summary
#' @param p a ggplot
#' @param add character vector specifying other plot elements to be added.
#'  Allowed values are one or the combination of: "none", "dotplot", "jitter",
#'  "boxplot", "point", "mean", "mean_se", "mean_sd", "mean_ci", "mean_range",
#'  "median", "median_iqr", "median_hilow", "median_q1q3", "median_mad",
#'  "median_range".
#' @param color point or outline color.
#' @param fill fill color. Used only when \code{error.plot = "crossbar"}.
#' @param group grouping variable. Allowed values are 1 (for one group) or a
#'  character vector specifying the name of the grouping variable. Used only for
#'  adding statistical summary per group.
#' @param width numeric value between 0 and 1 specifying bar or box width.
#'  Example width = 0.8. Used only when \code{error.plot} is one of
#'  c("crossbar", "errorbar").
#' @param shape point shape. Allowed values can be displayed using the function
#'  \code{\link{show_point_shapes}()}.
#' @param size numeric value in [0-1] specifying point and line size.
#' @param linewidth numeric value in [0-1] specifying line width. Used only when
#'  \code{add} contains "line" or for adding error bars. Default is \code{size}.
#' @param linetype line type.
#' @param show.legend logical. Should this layer be included in the legends? NA,
#'  the default, includes if any aesthetics are mapped. \code{FALSE} never
#'  includes, and TRUE always includes. It can also be a named logical vector to
#'  finely select the aesthetics to display.
#' @param alpha numeric value specifying fill color transparency. Value should be
#'  in [0, 1], where 0 is full transparency and 1 is no transparency.
#' @param jitter a numeric value specifying the amount of jittering. Used only
#'  when \code{add} contains "jitter".
#' @param seed A random seed to make the jitter reproducible. Default is `123`. Useful if you need
#'  to apply the same jitter twice, e.g., for a point and a corresponding label.
#'  The random seed is reset after jittering. If `NA`, the
#'  seed is initialized with a random value; this makes sure that two subsequent
#'  calls start with a different seed. Use NULL to use the current random seed
#'  and also avoid resetting (the behaviour of ggplot 2.2.1 and earlier).
#' @param binwidth numeric value specifying bin width. use value between 0 and 1
#'  when you have a strong dense dotplot. For example binwidth = 0.2. Used only
#'  when \code{add} contains "dotplot".
#' @param dotsize as \code{size} but applied only to dotplot.
#' @param outliers logical. If TRUE (default), outliers are displayed in boxplots.
#'  If FALSE, outliers are not displayed.
#' @param outlier.shape numeric value specifying the shape of outliers in boxplots.
#'  Default is 19 (filled circle). if set to NA, outliers are not displayed.
#' @param p_geom the geometry of the main plot. Ex: p_geom = "geom_line". If
#'  NULL, the geometry is extracted from p. Used only by \link{ggline}().
#' @examples
#' # Basic violin plot
#' data("ToothGrowth")
#' p <- ggviolin(ToothGrowth, x = "dose", y = "len", add = "none")
#'
#' # Add mean +/- SD and jitter points
#' p %>% ggadd(c("mean_sd", "jitter"), color = "dose")
#'
#' # Add box plot
#' p %>% ggadd(c("boxplot", "jitter"), color = "dose")
#'
#' @export
ggadd <- function(p, add = NULL, color = "black", fill = "white", group = 1,
                  width = 1, shape = 19, size = NULL, alpha = 1, jitter = 0.2, seed = 123,
                  binwidth = NULL, dotsize = size, linetype = 1, linewidth = size, show.legend = NA,
                  error.plot = "pointrange", ci = 0.95,
                  outliers = TRUE, outlier.shape = 19,
                  data = NULL, position = position_dodge(0.8),
                  p_geom = "") {
  . <- NULL
  if (missing(group)) group <- NULL
  # Checkpoints
  # :::::::::::::::::::::::::::::::::::::::::::
  if ("none" %in% add) add <- "none"
  # Adding mean or median point
  center <- intersect(c("mean", "median"), add)
  if (length(center) == 2) {
    stop("Use mean or mdedian, but not both at the same time.")
  }
  # Adding error bars
  errors <- intersect(.errorbar_functions(), add)
  if (length(errors) > 1) {
    stop("Choose only one of these: ", .collapse(errors, sep = ", "))
  }

  if (is.null(dotsize)) dotsize <- 1


  # Data and mapping
  # :::::::::::::::::::::::::::::::::::::::::::
  if (is.null(data)) data <- p$data
  .map <- .mapping(p)
  x <- .map$x
  y <- .map$y
  if (missing(color) & !is.null(.map$colour)) {
    color <- .map$colour
  }
  if (missing(fill) & !is.null(.map$fill)) {
    fill <- .map$fill
  }
  ngrps <- intersect(names(data), c(.map$x, fill, color, group)) %>%
    length() # number of grouping variables

  # Amount of jittering when add = "jitter"
  # :::::::::::::::::::::::::::::::::::::::::::
  .jitter <- jitter
  if (is.numeric(jitter)) jitter <- position_jitter(jitter, seed = seed)
  if (p_geom == "geom_line" & ngrps > 1) {
    # #436: for a grouped line plot, jitter points are centered by default
    # (position = "identity"), matching the non-dodged line. When the user
    # requests a position_dodge, dodge the jitter too -- with the SAME dodge
    # width as the line / error bars -- so the points sit under their group.
    # Scoped to plain position_dodge(): position_dodge2() packs zero-width
    # summary geoms differently, so dodging the jitter by its width would
    # overshoot; leave those centered (unchanged).
    if (inherits(position, "PositionDodge") && !inherits(position, "PositionDodge2") &&
        !is.null(position$width)) {
      jitter <- position_jitterdodge(
        jitter.width = .jitter, dodge.width = position$width, seed = seed
      )
    }
  } else if (ngrps > 1) jitter <- position_jitterdodge(jitter.width = .jitter, dodge.width = 0.8, seed = seed)

  common.opts <- opts <- list(
    data = data, color = color, fill = fill,
    size = size, position = position, alpha = alpha,
    show.legend = show.legend
  )

  if ("boxplot" %in% add) {
    if (.geom(p) == "violin" & missing(width)) {
      width <- 0.2
    } else if (missing(width)) width <- 0.7
    p <- common.opts %>%
      .add_item(geomfunc = geom_boxplot, width = width, outliers = outliers, outlier.shape = outlier.shape) %>%
      .update_plot(p)
  }
  if ("violin" %in% add) {
    if (missing(width)) width <- 1
    p <- common.opts %>%
      .add_item(geomfunc = geom_violin, width = width, trim = FALSE) %>%
      .update_plot(p)
  }
  if ("dotplot" %in% add) {
    p <- common.opts %>%
      .add_item(
        geomfunc = geom_dotplot, binaxis = "y", stackdir = "center",
        dotsize = dotsize, stackratio = 1, binwidth = binwidth
      ) %>%
      .update_plot(p)
  }
  if ("jitter" %in% add) {
    jitter.opts <- common.opts
    if (!(shape %in% 21:25)) jitter.opts$fill <- NULL
    p <- jitter.opts %>%
      .add_item(geomfunc = geom_jitter, position = jitter, shape = shape) %>%
      .update_plot(p)
  }

  if ("point" %in% add) {
    p <- common.opts %>%
      .remove_item("fill") %>%
      .add_item(geomfunc = geom_point, group = group) %>%
      .update_plot(p)
  }
  if ("line" %in% add) {
    p <- common.opts %>%
      .remove_item("fill") %>%
      .remove_item("size") %>%
      .add_item(geomfunc = geom_line, group = 1, linewidth = linewidth) %>%
      .update_plot(p)
  }

  # Add mean or median
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (!.is_empty(center)) {
    p <- p %>% add_summary(
      fun = center, color = color, shape = shape,
      position = position, size = size, group = group
    )
  }
  # Add erors
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (!.is_empty(errors)) {
    if (missing(width)) {
      if (error.plot %in% c("errorbar", "lower_errorbar", "upper_errorbar")) {
        width <- 0.1
      } else if (error.plot == "crossbar" & .geom(p) == "violin") width <- 0.2
    }
    p <- p %>% add_summary(errors,
      error.plot = error.plot, color = color, shape = shape,
      position = position, size = size, linewidth = linewidth, width = width, ci = ci, group = group,
      linetype = linetype, show.legend = show.legend
    )
  }

  p
}

#' Rotate Axes Text
#'
#' @description Rotate the x-axis text (tick mark labels).
#' \itemize{
#' \item \code{rotate_x_text}(): Rotate x axis text.
#' \item \code{rotate_y_text}(): Rotate y axis text.
#' }
#'
#' @param angle numeric value specifying the rotation angle. Default is 90 for vertical x-axis text.
#' @param hjust horizontal justification (in [0, 1]).
#' @param vjust vertical justification (in [0, 1]).
#' @param ... other arguments to pass to the function
#'  \code{\link[ggplot2:element]{element_text}()}.
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' p <- ggboxplot(df, x = "dose", y = "len")
#' p
#' # Vertical x axis text
#' p + rotate_x_text()
#' # Set rotation angle to 45
#' p + rotate_x_text(45)
#' p + rotate_y_text(45)
#'
#' @name rotate_axis_text
#' @rdname rotate_axis_text
#' @export
rotate_x_text <- function(angle = 90, hjust = NULL, vjust = NULL, ...) {
  if (missing(hjust) & angle > 5) {
    hjust <- 1
  }
  if (missing(vjust) & angle == 90) {
    vjust <- 0.5
  }

  theme(
    axis.text.x = element_text(angle = angle, hjust = hjust, vjust = vjust, ...)
  )
}

#' @name rotate_axis_text
#' @rdname rotate_axis_text
#' @export
rotate_y_text <- function(angle = 90, hjust = NULL, vjust = NULL, ...) {
  if (missing(hjust) & angle == 90) {
    hjust <- 0.5
  } else if (missing(hjust) & angle > 5) {
    hjust <- 1
  }

  theme(
    axis.text.y = element_text(angle = angle, hjust = hjust, vjust = vjust, ...)
  )
}

#' @include utilities.R ggpar.R
NULL
#' Line plot
#' @description Create a line plot.
#' @inheritParams ggboxplot
#' @inheritParams ggplot2::geom_line
#' @param x,y x and y variables for drawing.
#' @param group grouping variable to connect points by line.
#' Allowed values are 1 (for one line, one group) or a character vector specifying
#' the name of the grouping variable (case of multiple lines).
#' @param numeric.x.axis logical. If TRUE, x axis will be treated as numeric. Default is FALSE.
#' @param color line colors.
#' @param linetype line type.
#' @param plot_type plot type. Allowed values are one of "b" for both line and point;
#' "l" for line only; and "p" for point only. Default is "b".
#' @param size line size. Deprecated in ggplot2 v >= 3.4.0, use \code{linewidth} instead.
#' @param linewidth line width. Default is 0.5. Recommended parameter for
#' ggplot2 version >= 3.4.0. If both \code{size} and \code{linewidth} are specified, an error is thrown.
#' @param shape point shapes.
#' @param stroke point stroke. Used only for shapes 21-24 to control the thickness of points border.
#' @param show.line.label logical value. If TRUE, shows line labels.
#' @param point.size point size.
#' @param point.color point color.
#' @param ... other arguments to be passed to geom_dotplot.
#'
#'
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}, \code{\link{ggbarplot}}
#' @examples
#' # Data
#' df <- data.frame(
#'   dose = c("D0.5", "D1", "D2"),
#'   len = c(4.2, 10, 29.5)
#' )
#' print(df)
#'
#' # Basic plot
#' # +++++++++++++++++++++++++++
#' ggline(df, x = "dose", y = "len")
#'
#'
#' # Plot with multiple groups
#' # +++++++++++++++++++++
#'
#' # Create some data
#' df2 <- data.frame(
#'   supp = rep(c("VC", "OJ"), each = 3),
#'   dose = rep(c("D0.5", "D1", "D2"), 2),
#'   len = c(6.8, 15, 33, 4.2, 10, 29.5)
#' )
#' print(df2)
#'
#' # Plot "len" by "dose" and
#' # Change line types and point shapes by a second groups: "supp"
#' ggline(df2, "dose", "len",
#'   linetype = "supp", shape = "supp"
#' )
#'
#'
#' # Change colors
#' # +++++++++++++++++++++
#'
#' # Change color by group: "supp"
#' # Use custom color palette
#' ggline(df2, "dose", "len",
#'   linetype = "supp", shape = "supp",
#'   color = "supp", palette = c("#00AFBB", "#E7B800")
#' )
#'
#'
#' # Add points and errors
#' # ++++++++++++++++++++++++++
#'
#' # Data: ToothGrowth data set we'll be used.
#' df3 <- ToothGrowth
#' head(df3, 10)
#'
#' # It can be seen that for each group we have
#' # different values
#' ggline(df3, x = "dose", y = "len")
#'
#' # Visualize the mean of each group
#' ggline(df3,
#'   x = "dose", y = "len",
#'   add = "mean"
#' )
#'
#' # Add error bars: mean_se
#' # (other values include: mean_sd, mean_ci, median_iqr, ....)
#' # Add labels
#' ggline(df3, x = "dose", y = "len", add = "mean_se")
#'
#' # Change error.plot to "pointrange"
#' ggline(df3,
#'   x = "dose", y = "len",
#'   add = "mean_se", error.plot = "pointrange"
#' )
#'
#' # Add jitter points and errors (mean_se)
#' ggline(df3,
#'   x = "dose", y = "len",
#'   add = c("mean_se", "jitter")
#' )
#'
#' # Add dot and errors (mean_se)
#' ggline(df3,
#'   x = "dose", y = "len",
#'   add = c("mean_se", "dotplot"), color = "steelblue"
#' )
#'
#' # Add violin and errors (mean_se)
#' ggline(df3,
#'   x = "dose", y = "len",
#'   add = c("mean_se", "violin"), color = "steelblue"
#' )
#'
#' # Multiple groups with error bars
#' # ++++++++++++++++++++++
#'
#' ggline(df3,
#'   x = "dose", y = "len", color = "supp",
#'   add = "mean_se", palette = c("#00AFBB", "#E7B800")
#' )
#'
#' # Add jitter
#' ggline(df3,
#'   x = "dose", y = "len", color = "supp",
#'   add = c("mean_se", "jitter"), palette = c("#00AFBB", "#E7B800")
#' )
#'
#' # Add dot plot
#' ggline(df3,
#'   x = "dose", y = "len", color = "supp",
#'   add = c("mean_se", "dotplot"), palette = c("#00AFBB", "#E7B800")
#' )
#' #
#'
#' @export
ggline <- function(data, x, y, group = 1,
                   numeric.x.axis = FALSE,
                   combine = FALSE, merge = FALSE,
                   color = "black", palette = NULL,
                   linetype = "solid",
                   plot_type = c("b", "l", "p"),
                   size = NULL, linewidth = NULL,
                   shape = 19, stroke = NULL,
                   point.size = NULL, point.color = color,
                   title = NULL, xlab = NULL, ylab = NULL,
                   facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                   select = NULL, remove = NULL, order = NULL,
                   add = "none",
                   add.params = list(),
                   error.plot = "errorbar",
                   label = NULL, font.label = list(size = 11, color = "black"),
                   label.select = NULL, repel = FALSE, label.rectangle = FALSE,
                   show.line.label = FALSE,
                   position = "identity",
                   ggtheme = theme_pubr(),
                   ...) {
  if (is.null(point.size)) {
    point.size <- linewidth
  }

  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    group = group, numeric.x.axis = numeric.x.axis,
    combine = combine, merge = merge,
    color = color, palette = palette,
    linetype = linetype, plot_type = plot_type,
    size = size, linewidth = linewidth, shape = shape, stroke = stroke,
    point.size = point.size, point.color = point.color,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot,
    label = label, font.label = font.label, label.select = label.select,
    repel = repel, label.rectangle = label.rectangle,
    show.line.label = show.line.label, position = position, ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }
  .opts$fun <- ggline_core
  .opts$fun_name <- "ggline"
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


ggline_core <- function(data, x, y, group = 1,
                        numeric.x.axis = FALSE,
                        color = "black", fill = "white", palette = NULL,
                        linetype = "solid",
                        plot_type = c("b", "l", "p"),
                        size = NULL, linewidth = NULL,
                        shape = 19, stroke = NULL,
                        point.size = NULL, point.color = color,
                        title = NULL, xlab = NULL, ylab = NULL,
                        select = NULL, order = NULL,
                        facet.by = NULL,
                        add = "none",
                        add.params = list(),
                        error.plot = "errorbar",
                        show.line.label = FALSE,
                        font.label = list(size = 11, color = "black"),
                        repel = FALSE, label.rectangle = FALSE,
                        position = "identity",
                        ggtheme = theme_pubr(),
                        ...) {
  line_width <- .resolve_linewidth_args(
    size = size, linewidth = linewidth, default_linewidth = 0.5
  )
  size <- line_width$size
  linewidth <- line_width$linewidth

  if (is.null(point.size)) {
    point.size <- if (!is.null(size)) size else if (!is.null(linewidth)) linewidth else 0.5
  }

  xx <- .select_vec(data, x)
  if (inherits(xx, c("character", "numeric")) & !numeric.x.axis) {
    data[[x]] <- .select_vec(data, x) %>% as.factor()
  }

  error.plot <- error.plot[1]
  plot_type <- match.arg(plot_type)
  if ("none" %in% add) add <- "none"
  grouping.vars <- intersect(c(x, color, linetype, group, facet.by), names(data))
  . <- NULL

  # static summaries for computing mean/median and adding errors
  if (is.null(add.params$fill)) add.params$fill <- "white"
  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill = "white", ...)

  if (any(.summary_functions() %in% add)) {
    data_sum <- desc_statby(data, measure.var = y, grps = grouping.vars)
    summary.funcs <- intersect(.summary_functions(), add)
    if (length(summary.funcs) > 1) {
      stop(
        "Only one summary function is allowed. ",
        "Choose one of ", .collapse(.summary_functions(), sep = ", ")
      )
    }

    .center <- summary.funcs %>%
      strsplit("_", fixed = TRUE) %>%
      unlist() %>%
      .[1]

    add <- setdiff(add, .center)
    names(data_sum)[which(names(data_sum) == .center)] <- y
    # data_sum[, x] <- as.factor(data_sum[, x])
    if (inherits(xx, c("character", "numeric")) & !numeric.x.axis) {
      data_sum[, x] <- .select_vec(data_sum, x) %>% as.factor()
    }
  } else {
    data_sum <- data
  }

  # Remember whether the user supplied an explicit grouping variable (a data
  # column, not the default group = 1). Following ggplot2 semantics, an explicit
  # group takes precedence over aesthetic-derived grouping (see below).
  explicit.group <- if (length(group) == 1 && is.character(group) &&
    group %in% names(data)) {
    group
  } else {
    NULL
  }

  .cols <- unique(c(color, linetype, group))
  if (any(.cols %in% names(data))) {
    .in <- which(.cols %in% names(data))
    group <- .cols[.in]
    if (is.null(add.params$group)) add.params$group <- group[1]
  }

  # When several variables define the grouping (e.g. color and linetype map to
  # different columns), a length > 1 group aesthetic otherwise errored (#616,
  # #375). Resolve to a single group column, matching ggplot2:
  #  - an explicit `group` variable takes precedence (group by it alone);
  #  - otherwise combine the aesthetic variables into one interaction group so
  #    the right points are connected.
  # The single-variable case is unchanged (group stays length 1).
  if (length(group) > 1) {
    if (!is.null(explicit.group)) {
      group <- explicit.group
    } else {
      data_sum[[".ggpubr.group"]] <- do.call(
        interaction,
        c(data_sum[group], list(drop = TRUE, lex.order = TRUE))
      )
      group <- ".ggpubr.group"
    }
  }

  p <- ggplot(data, create_aes(list(x = x, y = y)))

  # Add other geom or summary
  # :::::::::::::::::::::::::::::::::::::::
  add.params <- add.params %>%
    .add_item(
      error.plot = error.plot,
      position = position, p_geom = "geom_line"
    )

  # First add geom if any
  p <- add.params %>%
    .add_item(p = p, add = setdiff(add, .summary_functions())) %>%
    do.call(ggadd, .)
  # Then add summary statistics
  p <- add.params %>%
    .add_item(p = p, size = linewidth, linewidth = linewidth, add = intersect(add, .summary_functions())) %>%
    do.call(ggadd, .)


  # add.params <- add.params %>%
  #   .add_item(p = p, add = add, error.plot = error.plot,
  #             position = "identity", p_geom = "geom_line")
  # p <- do.call(ggadd, add.params)

  # Main plot
  if (plot_type %in% c("b", "l")) {
    line_args <- geom_exec(NULL,
      data = data_sum,
      stat = "identity",
      color = color, linetype = linetype,
      position = position,
      linewidth = linewidth
    )
    mapping <- line_args$mapping
    mapping[["group"]] <- group
    option <- line_args$option
    option[["mapping"]] <- create_aes(mapping)
    p <- p + do.call(geom_line, option)
  }

  if (plot_type %in% c("p", "b")) {
    p <- p +
      geom_exec(geom_point,
        data = data_sum,
        color = point.color, shape = shape,
        size = 1.2 + point.size, stroke = stroke,
        position = position
      )
    # Adjust shape when ngroups > 6, to avoid ggplot warnings
    p <- .scale_point_shape(p, data_sum, shape)
  }

  # Color palette
  user.add.color <- list(...)$user.add.color
  if (is.null(user.add.color)) user.add.color <- ""
  if (.is_color(user.add.color) & !is.numeric(group)) {
    ngroup <- nlevels(.select_vec(data_sum, group))
    palette <- rep(user.add.color, ngroup)
  }

  if (show.line.label & !is.numeric(group)) {
    xval <- .select_vec(data_sum, x)
    last.xval <- .levels(xval) %>% utils::tail(1)
    groupval <- .select_vec(data_sum, group)
    label.data <- subset(data_sum, xval == last.xval)

    font.label <- .parse_font(font.label)
    p <- font.label %>%
      .add_item(
        data = label.data, x = x, y = y, label = group,
        repel = repel, label.rectangle = label.rectangle,
        ggtheme = ggtheme, ggp = p
      ) %>%
      do.call(ggtext, .)
  }

  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )

  p
}

# Regression tests for #616 / #375: ggline() errored ("condition has length > 1")
# when two different variables defined the grouping (e.g. color + linetype, or an
# explicit group + color), because the internal `group` became a length-2 vector.

.n_line_groups <- function(p) {
  b <- ggplot2::ggplot_build(p)
  # the line layer carries both a group column and >2 rows per line
  idx <- which(vapply(b$data, function(d)
    all(c("group", "linetype") %in% names(d)) && nrow(d) > 2, logical(1)))
  length(unique(b$data[[idx[1]]]$group))
}

test_that("ggline() combines color + linetype into one grouping (#616)", {
  set.seed(1)
  data <- data.frame(
    x = rep(seq(0, 1, length.out = 10), 4),
    type  = rep(c("poly", "poly", "inv", "inv"), each = 10),
    order = rep(c("low", "high", "low", "high"), each = 10)
  )
  data$val <- data$x + as.integer(factor(paste(data$type, data$order)))
  p <- ggline(data, x = "x", y = "val", linetype = "type", color = "order",
              numeric.x.axis = TRUE)
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  # 2 types x 2 orders = 4 lines, matching geom_line(aes(linetype=, color=))
  expect_equal(.n_line_groups(p), 4L)
})

test_that("ggline() supports explicit group + color (#375)", {
  df2 <- data.frame(
    supp = rep(c("VC", "OJ", "VC"), each = 3),
    dose = rep(c("D0.5", "D1", "D2"), 3),
    len  = c(6.8, 15, 33, 4.2, 10, 29.5, 12, 6, 14.2),
    num  = as.character(rep(1:3, each = 3))
  )
  p <- ggline(df2, "dose", "len", group = "num", color = "supp")
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  # 3 num groups (matching geom_line(aes(group=num, color=supp)))
  expect_equal(.n_line_groups(p), 3L)
})

test_that("ggline() explicit group takes precedence over color (#375)", {
  # supp appears with BOTH num values (full factorial): an explicit group must
  # group by `num` ALONE (matching geom_line(aes(group=num, color=supp))), not
  # by interaction(supp, num).
  dfe <- data.frame(
    dose = rep(c("D0.5", "D1", "D2"), 4),
    num  = as.character(rep(c(1, 1, 2, 2), each = 3)),
    supp = rep(c("VC", "OJ"), times = 6),
    len  = 1:12
  )
  p <- ggline(dfe, "dose", "len", group = "num", color = "supp")
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(p)))
  # 2 num groups, NOT 4 interaction groups
  expect_equal(.n_line_groups(p), 2L)
  # matches geom_line's explicit-group grouping
  gl <- ggplot2::ggplot_build(
    ggplot2::ggplot(dfe, ggplot2::aes(dose, len, group = num, color = supp)) +
      ggplot2::geom_line()
  )
  expect_equal(.n_line_groups(p), length(unique(gl$data[[1]]$group)))
})

test_that("ggline() single grouping variable is unchanged (no regression, #616)", {
  p <- ggline(ToothGrowth, x = "dose", y = "len", color = "supp", add = "mean_se")
  b <- ggplot2::ggplot_build(p)
  # the new .ggpubr.group column must NOT be introduced in the single-var path
  expect_false(any(vapply(b$data,
    function(d) ".ggpubr.group" %in% names(d), logical(1))))
  expect_no_error(ggplot2::ggplot_gtable(b))
  expect_equal(.n_line_groups(p), 2L)
})

#' Rotate a ggplot Horizontally
#'
#' @description Rotate a ggplot to create horizontal plots. Wrapper around
#'  \code{\link[ggplot2]{coord_flip}}.
#' @param ... other arguments to pass to \code{\link[ggplot2]{coord_flip}}.
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#' df <- ToothGrowth
#'
#' # Basic plot
#' p <- ggboxplot(df,
#'   x = "dose", y = "len",
#'   color = "dose", palette = "jco"
#' )
#' p
#' # Create horizontal plots
#' p + rotate()
#' @export
rotate <- function(...) {
  coord_flip(...)
}

#' Gene Citation Index
#'
#' @description Contains the mean citation index of 66 genes obtained by
#'  assessing PubMed abstracts and annotations using two key words i) Gene name
#'  + b cell differentiation and ii) Gene name + plasma cell differentiation.
#' @name gene_citation
#' @docType data
#' @usage data("gene_citation")
#' @format A data frame with 66 rows and 2 columns. \describe{
#'  \item{\code{gene}}{gene names} \item{\code{citation_index}}{mean citation index} }
#'
#' @examples
#' data(gene_citation)
#'
#' # Some key genes of interest to be highlighted
#' key.gns <- c(
#'   "MYC", "PRDM1", "CD69", "IRF4", "CASP3", "BCL2L1", "MYB", "BACH2", "BIM1", "PTEN",
#'   "KRAS", "FOXP1", "IGF1R", "KLF4", "CDK6", "CCND2", "IGF1", "TNFAIP3", "SMAD3", "SMAD7",
#'   "BMPR2", "RB1", "IGF2R", "ARNT"
#' )
#' # Density distribution
#' ggdensity(gene_citation,
#'   x = "citation_index", y = "..count..",
#'   xlab = "Number of citation",
#'   ylab = "Number of genes",
#'   fill = "lightgray", color = "black",
#'   label = "gene", label.select = key.gns, repel = TRUE,
#'   font.label = list(color = "citation_index"),
#'   xticks.by = 20, # Break x ticks by 20
#'   gradient.cols = c("blue", "red"),
#'   legend = "bottom",
#'   legend.title = "" # Hide legend title
#' )
#'
NULL

#' @include utilities.R ggpar.R
NULL
#' Line types available in R
#' @description Show line types available in R.
#' @return a ggplot.
#'
#' @seealso \code{\link{ggpar}} and \code{\link{ggline}}.
#' @examples
#' show_line_types() +
#'   theme_minimal()
#' @export
show_line_types <- function() {
  lt <- c(
    "blank", "solid", "dashed", "dotted",
    "dotdash", "longdash", "twodash"
  )

  d <- data.frame(lt = factor(lt, levels = lt))

  ggplot() +
    scale_x_continuous(name = "", limits = c(0, 1), breaks = NULL) +
    scale_linetype_identity() +
    geom_segment(data = d, mapping = aes(
      x = 0, xend = 1, y = lt,
      yend = lt, linetype = lt
    )) +
    labs(title = "Line types available in R", y = "") +
    theme(axis.text.y = element_text(face = "bold", color = "black"))
}

#' @include utilities.R
NULL
#' Add Background Image to ggplot2
#' @description Add background image to ggplot2.
#' @param raster.img raster object to display, as returned by the function
#'  \code{readPNG()}[in \code{png} package] and \code{readJPEG()} [in \code{jpeg} package].
#' @author Laszlo Erdey <erdey.laszlo@@econ.unideb.hu>
#' @examples
#' \dontrun{
#' install.packages("png")
#'
#' # Import the image
#' img.file <- system.file(file.path("images", "background-image.png"),
#'   package = "ggpubr"
#' )
#' img <- png::readPNG(img.file)
#'
#' # Plot with background image
#' ggplot(iris, aes(Species, Sepal.Length)) +
#'   background_image(img) +
#'   geom_boxplot(aes(fill = Species), color = "white") +
#'   fill_palette("jco")
#' }
#'
#' @export
background_image <- function(raster.img) {
  annotation_raster(raster.img,
    xmin = -Inf, xmax = Inf,
    ymin = -Inf, ymax = Inf
  )
}

#' Balloon plot
#'
#' @description Plot a graphical matrix where each cell contains a dot whose
#'   size reflects the relative magnitude of the corresponding component. Useful
#'   to visualize contingency table formed by two categorical variables.
#'
#' @inheritParams ggpar
#' @param data a data frame. Can be: \itemize{ \item \bold{a standard
#'   contingency table} formed by two categorical variables: a data frame with
#'   row names and column names. The categories of the first variable are
#'   columns and the categories of the second variable are rows. \item \bold{a
#'   stretched contingency table}: a data frame containing at least three columns
#'   corresponding, respectively, to (1) the categories of the first variable,
#'   (2) the categories of the second variable, (3) the frequency value. In this
#'   case, you should specify the argument x and y in the function
#'   \code{ggballoonplot()}}.
#' @param x,y the column names specifying, respectively, the first and the
#'   second variable forming the contingency table. Required only when the data
#'   is a stretched contingency table.
#' @param color point border line color.
#' @param fill point fill color. Default is "lightgray". Considered only for
#'   points 21 to 25.
#' @param shape points shape. The default value is 21. Alternative values include
#'   22, 23, 24, 25.
#' @param size point size. By default, the points size reflects the relative
#'   magnitude of the value of the corresponding cell (\code{size = "value"}).
#'   Can be also numeric (\code{size = 4}).
#' @param size.range a numeric vector of length 2 that specifies the minimum and
#'   maximum size of the plotting symbol. Default values are \code{size.range =
#'   c(1, 10)}.
#' @param facet.by character vector, of length 1 or 2, specifying grouping variables for
#'   faceting the plot into multiple panels. Should be in the data.
#' @param show.label logical. If TRUE, show the data cell values as point
#'   labels.
#' @param font.label a vector of length 3 indicating respectively the size
#'   (e.g.: 14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and
#'   the color (e.g.: "red") of point labels. For example font.label = c(14,
#'   "bold", "red"). To specify only the size and the style, use font.label =
#'   c(14, "plain").
#' @param rotate.x.text logical. If TRUE (default), rotate the x axis text.
#' @param ggtheme function, ggplot2 theme name. Default value is
#'   \code{theme_minimal()}. Set \code{ggtheme = NULL} to skip applying a ggpubr
#'   theme, so the plot keeps ggplot2 default theme or the theme set globally via
#'   \code{theme_set()}.
#' @param ... other arguments passed to the function \code{\link{ggpar}}
#'
#' @examples
#' # Define color palette
#' my_cols <- c(
#'   "#0D0887FF", "#6A00A8FF", "#B12A90FF",
#'   "#E16462FF", "#FCA636FF", "#F0F921FF"
#' )
#'
#' # Standard contingency table
#' # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#' # Read a contingency table: housetasks
#' # Repartition of 13 housetasks in the couple
#' data <- read.delim(
#'   system.file("demo-data/housetasks.txt", package = "ggpubr"),
#'   row.names = 1
#' )
#' data
#'
#' # Basic ballon plot
#' ggballoonplot(data)
#'
#' # Change color and fill
#' ggballoonplot(data, color = "#0073C2FF", fill = "#0073C2FF")
#'
#'
#' # Change color according to the value of table cells
#' ggballoonplot(data, fill = "value") +
#'   scale_fill_gradientn(colors = my_cols)
#'
#' # Change the plotting symbol shape
#' ggballoonplot(data, fill = "value", shape = 23) +
#'   gradient_fill(c("blue", "white", "red"))
#'
#'
#' # Set points size to 8, but change fill color by values
#' # Sow labels
#' ggballoonplot(data,
#'   fill = "value", color = "lightgray",
#'   size = 10, show.label = TRUE
#' ) +
#'   gradient_fill(c("blue", "white", "red"))
#'
#' # Stretched contingency table
#' # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#'
#' # Create an Example Data Frame Containing Car x Color data
#' carnames <- c("bmw", "renault", "mercedes", "seat")
#' carcolors <- c("red", "white", "silver", "green")
#' datavals <- round(rnorm(16, mean = 100, sd = 60), 1)
#' car_data <- data.frame(
#'   Car = rep(carnames, 4),
#'   Color = rep(carcolors, c(4, 4, 4, 4)),
#'   Value = datavals
#' )
#'
#' car_data
#'
#' ggballoonplot(car_data,
#'   x = "Car", y = "Color",
#'   size = "Value", fill = "Value"
#' ) +
#'   scale_fill_gradientn(colors = my_cols) +
#'   guides(size = "none")
#'
#'
#' # Grouped frequency table
#' # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#' data("Titanic")
#' dframe <- as.data.frame(Titanic)
#' head(dframe)
#' ggballoonplot(
#'   dframe,
#'   x = "Class", y = "Sex",
#'   size = "Freq", fill = "Freq",
#'   facet.by = c("Survived", "Age"),
#'   ggtheme = theme_bw()
#' ) +
#'   scale_fill_gradientn(colors = my_cols)
#'
#' # Hair and Eye Color of Statistics Students
#' data(HairEyeColor)
#' ggballoonplot(as.data.frame(HairEyeColor),
#'   x = "Hair", y = "Eye", size = "Freq",
#'   ggtheme = theme_gray()
#' ) %>%
#'   facet("Sex")
#'
#' @export
ggballoonplot <- function(
  data, x = NULL, y = NULL, size = "value",
  facet.by = NULL, size.range = c(1, 10),
  shape = 21, color = "black", fill = "gray",
  show.label = FALSE, font.label = list(size = 12, color = "black"),
  rotate.x.text = TRUE,
  ggtheme = theme_minimal(), ...
) {
  if (inherits(data, "matrix")) {
    data <- as.data.frame(data)
  }

  # Check the data
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::


  # case 1: x and y specified. we assume that the data is stretched
  # if size not specified, then take the third column as size values
  # detect fill variable if misspecified
  if (!is.null(x) & !is.null(y)) {
    if (missing(size)) size <- colnames(data)[3]
    if (fill == "value" & !("value" %in% colnames(data))) {
      fill <- colnames(data)[3]
    }
    label <- colnames(data)[3]
  }


  # case 2: x and y are not specified
  # - check if the data is stretched:
  #     If yes, then consider the first 3 columns as x, y and size values
  #     If no, then stretch the data and continue
  else if (is.null(x) | is.null(y)) {
    if (.is_stretched(data)) {
      .cnames <- colnames(data)
      x <- .cnames[1]
      y <- .cnames[2]
      if (missing(size)) size <- .cnames[3]
      if (fill == "value" & !("value" %in% .cnames)) fill <- .cnames[3]
      label <- .cnames[3]
      # Reverse y levels so that it appears in the right order on the plot
      y.val <- dplyr::pull(data, 2)
      data[[2]] <- y.val %>% factor(levels = rev(.levels(y.val)))
    } else {
      data <- .df_stretch(data) # Stretch the data into 3 columns: .row|.col|.value
      x <- ".col"
      y <- ".row"
      if (missing(size)) size <- "value"
      label <- "value"
    }
  }

  p <- ggplot(data, create_aes(list(x = x, y = y))) +
    geom_exec(
      geom_point,
      data = data,
      size = size, fill = fill, shape = shape,
      color = color
    )

  p <- p +
    scale_size(range = size.range) +
    guides(size = guide_legend(reverse = TRUE)) +
    ggtheme
  # Blank the axis titles by default, but keep them when the user supplies
  # xlab/ylab (passed via ... to ggpar) so those labels are honored (#639).
  dots <- list(...)
  if (is.null(dots$xlab)) {
    p <- p + theme(axis.title.x = ggplot2::element_blank())
  }
  if (is.null(dots$ylab)) {
    p <- p + theme(axis.title.y = ggplot2::element_blank())
  }

  if (show.label) {
    font.label <- .check_lab_font(font.label)
    text.args <- font.label %>%
      .add_item(
        geomfunc = geom_text, data = data,
        x = x, y = y, label = label
      )
    p <- p + do.call(geom_exec, text.args)
  }

  if (rotate.x.text) {
    p <- p + rotate_x_text(45, hjust = 1, vjust = 1)
  }

  if (!is.null(facet.by)) {
    p <- facet(p, facet.by)
  }

  ggpar(p, ...)
}


# Font label
.check_lab_font <- function(font.label) {
  font.label <- .parse_font(font.label)
  font.label$size <- ifelse(is.null(font.label$size), 4, font.label$size / 3)
  font.label$color <- ifelse(is.null(font.label$color), "black", font.label$color)
  font.label$face <- ifelse(is.null(font.label$face), "plain", font.label$face)
  font.label
}


# Stretch a data frame with row names
# returns a data frame with 3 columns .row, .col, value
.df_stretch <- function(data) {
  .col.names <- colnames(data)
  .row.names <- rownames(data)
  data <- data %>%
    dplyr::mutate(.row = .row.names) %>%
    dplyr::select(.row, dplyr::everything())

  # Stretch the data into three columns
  .col <- .row <- NULL
  data <- data %>%
    tidyr::pivot_longer(
      cols = -".row",
      names_to = ".col",
      values_to = "value"
    ) %>%
    dplyr::mutate(
      .col = factor(.col, levels = .col.names),
      .row = factor(.row, levels = rev(.row.names))
    )
  data
}


# Check if the contingency table is in the stretched format
.is_stretched <- function(data, x = NULL, y = NULL) {
  stretched <- TRUE
  if (is.null(x) | is.null(y)) {
    if (.is_numeric_data(data)) {
      stretched <- FALSE
    } else {
      x <- dplyr::pull(data, 1)
      y <- dplyr::pull(data, 2)
      z <- dplyr::pull(data, 3)
      stretched <- (
        (is.character(x) | is.factor(x)) &
          (is.character(y) | is.factor(y)) &
          is.numeric(z)
      )
    }
  }

  stretched
}

# Check if a data matrix is numeric
.is_numeric_data <- function(x) {
  all(apply(x, 2, is.numeric))
}

# For tibble data. The first column should be row names
.is_correct_tbl <- function(x) {
  ok <- FALSE
  if (inherits(x, "tbl_df")) ok <- TRUE

  if (ok) {
    .rows <- dplyr::pull(x, 1)
    ok <- is.character(.rows) & .is_numeric_data(x[, -1])
  }

  ok
}

test_that("ggballoonplot honors xlab/ylab and blanks them by default (#639)", {
  dat <- as.data.frame(matrix(1:6, nrow = 2,
    dimnames = list(c("r1", "r2"), c("c1", "c2", "c3"))))

  # Default: axis titles are blanked (unchanged behavior)
  th_default <- ggplot2::ggplot_build(ggballoonplot(dat))$plot$theme
  expect_s3_class(th_default$axis.title.x, "element_blank")
  expect_s3_class(th_default$axis.title.y, "element_blank")

  # When supplied, xlab/ylab are honored (not blanked, and label text set)
  p <- ggballoonplot(dat, xlab = "My X", ylab = "My Y")
  built <- ggplot2::ggplot_build(p)
  expect_false(inherits(built$plot$theme$axis.title.x, "element_blank"))
  expect_false(inherits(built$plot$theme$axis.title.y, "element_blank"))
  expect_equal(built$plot$labels$x, "My X")
  expect_equal(built$plot$labels$y, "My Y")

  # Only one label supplied: it shows, the other stays blank
  p_x <- ggballoonplot(dat, xlab = "Only X")
  th_x <- ggplot2::ggplot_build(p_x)$plot$theme
  expect_false(inherits(th_x$axis.title.x, "element_blank"))
  expect_s3_class(th_x$axis.title.y, "element_blank")

  # xlab = FALSE keeps the ggpar hide behavior (title blank)
  th_false <- ggplot2::ggplot_build(ggballoonplot(dat, xlab = FALSE))$plot$theme
  expect_s3_class(th_false$axis.title.x, "element_blank")
})

#' @include utilities.R
NULL
#' Plot confidence ellipses.
#' @description Plot confidence ellipses around barycenters. The method for
#'   computing confidence ellipses has been modified from \code{FactoMineR::coord.ellipse()}.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @inheritParams ggplot2::stat_ellipse
#' @param level confidence level used to construct the ellipses. By
#'   default, 0.95.
#' @param npoint number of points used to draw the ellipses.
#' @param bary logical value. If TRUE, the coordinates of the ellipse around the
#'   barycentre of individuals are calculated.
#' @seealso \code{\link{stat_conf_ellipse}}
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#'
#' # scatter plot with confidence ellipses
#' ggscatter(df, x = "wt", y = "mpg", color = "cyl") +
#'   stat_conf_ellipse(aes(color = cyl))
#'
#' ggscatter(df, x = "wt", y = "mpg", color = "cyl") +
#'   stat_conf_ellipse(aes(color = cyl, fill = cyl), alpha = 0.1, geom = "polygon")
#'
#' @export
stat_conf_ellipse <- function(mapping = NULL, data = NULL, geom = "path",
                              position = "identity", na.rm = FALSE, show.legend = NA,
                              inherit.aes = TRUE, level = 0.95, npoint = 100, bary = TRUE,
                              ...) {
  layer(
    stat = StatConfEllipse, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, level = level, npoint = npoint, bary = bary, ...)
  )
}


StatConfEllipse <- ggproto("StatConfEllipse", Stat,
  required_aes = c("x", "y"),
  compute_group = function(data, scales, params, level = 0.95,
                           npoint = 100, bary = TRUE) {
    .coord_ellipse(data$x, data$y,
      level = level,
      npoint = npoint, bary = bary
    )
  }
)

# Compute confidence ellipses.
#  x,y x and y variables for drawing.
#  level confidence level used to construct the ellipses. By
#   default, 0.95.
#  npoint number of points used to draw the ellipses.
#  bary logical value. If TRUE, the coordinates of the ellipse around the
#   barycentre of individuals are calculated.
.coord_ellipse <- function(x, y, level = 0.95,
                           npoint = 100, bary = FALSE) {
  .ellipse <- function(x, scale = c(1, 1), centre = c(0, 0),
                       level = 0.95, t = sqrt(stats::qchisq(level, 2)), which = c(
                         1,
                         2
                       ), npoints = 100) {
    names <- c("x", "y")
    if (is.matrix(x)) {
      xind <- which[1]
      yind <- which[2]
      r <- x[xind, yind]
      if (missing(scale)) {
        scale <- sqrt(c(x[xind, xind], x[yind, yind]))
        if (scale[1] > 0) {
          r <- r / scale[1]
        }
        if (scale[2] > 0) {
          r <- r / scale[2]
        }
      }
      if (!is.null(dimnames(x)[[1]])) {
        names <- dimnames(x)[[1]][c(xind, yind)]
      }
    } else {
      r <- x
    }
    r <- min(max(r, -1), 1)
    d <- acos(r)
    a <- seq(0, 2 * pi, len = npoints)
    matrix(
      c(t * scale[1] * cos(a + d / 2) + centre[1], t *
        scale[2] * cos(a - d / 2) + centre[2]), npoints, 2,
      dimnames = list(NULL, names)
    )
  }


  center <- c(mean(x, na.rm = TRUE), mean(y, na.rm = TRUE))
  tab <- data.frame(x = x, y = y)
  mat.cov <- stats::cov(tab)
  if (bary) {
    mat.cov <- mat.cov / nrow(tab)
  }
  res <- .ellipse(mat.cov, centre = center, level = level, npoints = npoint)
  return(data.frame(res, stringsAsFactors = FALSE))
}

#' @include utilities.R geom_bracket.R
#' @importFrom dplyr pull
#' @importFrom glue glue
NULL
#' Add P-values Manually to a ggplot
#'
#' @description Add p-values manually to a ggplot, such as box plots, dot plots
#'  and stripcharts. Frequently asked questions are available on \href{https://www.datanovia.com/en/blog/tag/ggpubr/}{Datanovia ggpubr FAQ page}, for example:
#'  \itemize{
#'  \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-basic-ggplots/}{How to Add P-Values onto Basic GGPlots}
#'  \item \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-adjusted-p-values-to-a-multi-panel-ggplot/}{How to Add Adjusted P-values to a Multi-Panel GGPlot}
#'  \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-to-ggplot-facets/}{How to Add P-values to GGPlot Facets}
#'  \item \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-p-values-generated-elsewhere-to-a-ggplot/}{How to Add P-Values Generated Elsewhere to a GGPlot}
#'  \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-a-grouped-ggplot-using-the-ggpubr-r-package/}{How to Add P-Values onto a Grouped GGPlot using the ggpubr R Package}
#'  \item \href{https://www.datanovia.com/en/blog/how-to-create-stacked-bar-plots-with-error-bars-and-p-values/}{How to Create Stacked Bar Plots with Error Bars and P-values}
#'  \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-horizontal-ggplots/}{How to Add P-Values onto Horizontal GGPlots}
#'  }
#' @inheritParams geom_bracket
#' @param data a data frame containing statistical test results. The expected
#'  default format should contain the following columns: \code{group1 | group2 |
#'  p | y.position | etc}. \code{group1} and \code{group2} are the groups that
#'  have been compared. \code{p} is the resulting p-value. \code{y.position} is
#'  the y coordinates of the p-values in the plot.
#' @param label the column containing the label (e.g.: label = "p" or label =
#'  "p.adj"), where \code{p} is the p-value. Can be also an expression that can
#'  be formatted by the \code{\link[glue]{glue}()} package. For example, when
#'  specifying label = "t-test, p = \{p\}", the expression \{p\} will be
#'  replaced by its value. Values inserted through a glue expression are taken
#'  from the raw data column and are \strong{not} rounded by \code{p.digits}; to
#'  round within a glue expression, wrap the value, e.g.
#'  \code{label = "p = \{signif(p, 3)\}"} (or \code{\{format_p_value(p)\}} for the
#'  p-value house style).
#' @param y.position column containing the coordinates (in data units) to be used
#'  for absolute positioning of the label. Default value is "y.position". Can be
#'  also a numeric vector.
#' @param xmin column containing the position of the left sides of the brackets.
#'  Default value is "group1".
#' @param xmax  (optional) column containing the position of the right sides of
#'  the brackets. Default value is "group2". If NULL, the p-values are plotted
#'  as a simple text.
#' @param x x position of the p-value. Should be used only when you want plot the
#'  p-value as text (without brackets).
#' @param size,label.size size of label text.
#' @param p.digits integer indicating the number of digits used to
#'  format recognized \strong{numeric} p-value label columns (\code{label} is one
#'  of \code{"p"}, \code{"p.adj"}, \code{"p.value"}, \code{"p.val"},
#'  \code{"pval"}, \code{"padj"}). Default is 3. Set to \code{NULL} to print the
#'  raw value without rounding (this also disables \code{p.format.style} and the
#'  related formatting arguments). All other labels are left unchanged: other
#'  numeric columns (e.g. \code{"statistic"}, \code{"n"}, effect sizes),
#'  significance symbols (\code{"p.signif"}, \code{"p.adj.signif"}),
#'  already-formatted strings, and \code{\link[glue]{glue}} expressions. A
#'  p-named column whose values fall outside \code{[0, 1]} is also left as-is.
#'  Uses the same formatting engine (\code{\link{format_p_value}()}) as
#'  \code{\link{stat_anova_test}()} for consistency across layers. The style of
#'  this formatting is further controlled by \code{p.format.style},
#'  \code{p.leading.zero}, \code{p.min.threshold} and \code{p.decimal.mark}
#'  (same arguments as \code{\link{stat_compare_means}()}).
#' @param p.format.style character string specifying the p-value formatting style
#'  applied to recognized numeric p-value label columns (see \code{p.digits}).
#'  One of: \code{"default"} (backward compatible, uses scientific notation),
#'  \code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
#'  \code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
#'  (GraphPad style), or \code{"scientific"} (scientific notation for GWAS). See
#'  \code{\link{list_p_format_styles}} for details. Default is \code{"default"},
#'  which leaves the rendered p-value labels unchanged.
#' @param p.leading.zero logical indicating whether to include the leading zero
#'  before the decimal point (e.g., "0.05" vs ".05"). If provided, overrides the
#'  style default.
#' @param p.min.threshold numeric specifying the minimum p-value to display
#'  exactly. Values below this threshold are shown as "< threshold" (e.g.
#'  \code{p.min.threshold = 0.001} renders very small p-values as "< 0.001"). If
#'  provided, overrides the style default.
#' @param p.decimal.mark character string to use as the decimal mark. If
#'  \code{NULL}, uses \code{getOption("OutDec")}.
#' @param bracket.size Width of the lines of the bracket.
#' @param color text and line color. Can be variable name in the data for coloring by groups.
#' @param linetype linetype. Can be variable name in the data for changing linetype by groups.
#' @param tip.length numeric vector with the fraction that the bracket tips go
#'  down to indicate the precise column. Interpreted relative to the reference set
#'  by \code{tip.length.ref}. Default is 0.03.
#' @param tip.length.ref character string specifying what \code{tip.length} is a
#'  fraction of. Either \code{"data"} (default): fraction of the trained data
#'  range (tips scale with the data; existing plots unchanged); or \code{"axis"}:
#'  fraction of the y-axis range (\code{ylim}/\code{scale_y_*}), giving visually
#'  constant tip lengths across plots with different scales (#362).
#' @param remove.bracket logical, if \code{TRUE}, brackets are removed from the
#'  plot. Considered only in the situation, where comparisons are performed
#'  against reference group or against "all".
#' @param hide.ns can be logical value or a character vector.
#' \itemize{
#' \item Case when logical value. If TRUE, hide ns symbol when displaying
#'  significance levels. Filter is done by checking the column
#'  \code{p.adj.signif}, \code{p.signif}, \code{p.adj} and \code{p}.
#'  \item Case when character value. Possible values are "p" or "p.adj", for filtering out non significant.
#'  }
#' @param vjust move the text up or down relative to the bracket. Can be also a
#'  column name available in the data.
#' @param position position adjustment, either as a string, or the result of a
#'  call to a position adjustment function.
#' @param inherit.aes If \code{FALSE} (the default), overrides the default aesthetics,
#'  rather than combining with them. This is most useful for helper functions
#'  that define both data and aesthetics and shouldn't inherit behaviour from
#'  the default plot specification, e.g. \code{stat_pvalue_manual()}. Most layers should use
#'  \code{TRUE} (the \pkg{ggplot2} default), but \code{stat_pvalue_manual()} uses
#'  its own data with different column names than the parent plot data, so
#'  \code{FALSE} avoids conflicts between parent plot aesthetics (like \code{fill} or
#'  \code{color}) and the p-value annotation data.
#' @param ... other arguments passed to the function \code{geom_bracket()} or
#'  \code{geom_text()}
#' @rdname stat_pvalue_manual
#' @seealso \code{\link{stat_compare_means}}
#' @examples
#'
#' # T-test
#' stat.test <- compare_means(
#'   len ~ dose,
#'   data = ToothGrowth,
#'   method = "t.test"
#' )
#' stat.test
#'
#' # Create a simple box plot
#' p <- ggboxplot(ToothGrowth, x = "dose", y = "len")
#' p
#'
#' # Perform a t-test between groups
#' stat.test <- compare_means(
#'   len ~ dose,
#'   data = ToothGrowth,
#'   method = "t.test"
#' )
#' stat.test
#'
#' # Add p-values manually from stat.test data
#' # First specify the y.position of each comparison
#' stat.test <- stat.test %>%
#'   mutate(y.position = c(29, 35, 39))
#' p + stat_pvalue_manual(stat.test, label = "p.adj")
#'
#' # Customize the label with glue expression
#' # (https://github.com/tidyverse/glue)
#' p + stat_pvalue_manual(stat.test, label = "p = {p.adj}")
#'
#'
#' # Grouped bar plots
#' # %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#' ToothGrowth$dose <- as.factor(ToothGrowth$dose)
#' # Comparisons against reference
#' stat.test <- compare_means(
#'   len ~ dose,
#'   data = ToothGrowth, group.by = "supp",
#'   method = "t.test", ref.group = "0.5"
#' )
#' stat.test
#' # Plot
#' bp <- ggbarplot(ToothGrowth,
#'   x = "supp", y = "len",
#'   fill = "dose", palette = "jco",
#'   add = "mean_sd", add.params = list(group = "dose"),
#'   position = position_dodge(0.8)
#' )
#' bp + stat_pvalue_manual(
#'   stat.test,
#'   x = "supp", y.position = 33,
#'   label = "p.signif",
#'   position = position_dodge(0.8)
#' )
#'
#' @export
stat_pvalue_manual <- function(
  data, label = NULL, y.position = "y.position",
  xmin = "group1", xmax = "group2", x = NULL,
  size = 3.88, label.size = size, p.digits = 3,
  p.format.style = "default", p.leading.zero = NULL,
  p.min.threshold = NULL, p.decimal.mark = NULL, bracket.size = 0.3,
  bracket.nudge.y = 0, bracket.shorten = 0,
  color = "black", linetype = 1, tip.length = 0.03,
  tip.length.ref = c("data", "axis"),
  remove.bracket = FALSE, step.increase = 0, step.group.by = NULL,
  hide.ns = FALSE, vjust = 0, coord.flip = FALSE, position = "identity", inherit.aes = FALSE, ...
) {
  tip.length.ref <- match.arg(tip.length.ref)
  if (is.null(label)) {
    label <- guess_signif_label_column(data)
  }
  data <- add_p_format_signif(data)
  # Hide NS
  if (is.logical(hide.ns)) {
    if (hide.ns) data <- remove_ns(data)
  } else if (is.character(hide.ns)) {
    filter <- dplyr::filter
    if (hide.ns == "p") {
      data <- data %>% filter(.data$p <= 0.05)
    } else if (hide.ns == "p.adj") data <- data %>% filter(.data$p.adj <= 0.05)
  }
  data <- asserttat_group_columns_exists(data)
  comparison <- detect_comparison_type(data)
  all.x.is.missing <- is.null(x) & missing(xmin) & missing(xmax)
  if (all(data$group1 == "all") & all.x.is.missing) {
    is.grouped <- length(data$group2) > length(unique(data$group2))
    if (!is.grouped) x <- "group2" # labels will be plotted at x = "group2"
  }
  # Detect automatically if xmin and xmax exists in the data.
  if (all.x.is.missing) {
    if (all(c("xmin", "xmax") %in% colnames(data))) {
      xmin <- "xmin"
      xmax <- "xmax"
    }
  }
  # should stay before (!is.null(x))
  if (remove.bracket) {
    group1.length <- unique(data$group1) %>% length()
    if (group1.length == 1) {
      xmin <- xmax
      xmax <- NULL
    }
  }
  # P-value displayed as text (without brackets)
  if (!is.null(x)) {
    xmin <- x
    xmax <- NULL
  }

  # If label is a glue package expression
  if (.contains_curlybracket(label)) {
    data <- data %>% mutate(label = glue(label))
    label <- "label"
  }

  available.variables <- colnames(data)
  if (!(label %in% available.variables)) {
    stop("can't find the label variable '", label, "' in the data")
  }
  if (!(xmin %in% available.variables)) {
    stop("can't find the xmin variable '", xmin, "' in the data")
  }

  # Round recognized numeric p-value label columns for display (e.g.
  # label = "p"/"p.adj"). Restricted to known p-value column names so that other
  # numeric label columns (e.g. "statistic", "n", effect sizes, fold changes) are
  # left untouched. The extra range check leaves the column as-is unless the
  # values actually look like probabilities in [0, 1] -- format_p_value() rejects
  # out-of-range values (NA + warning), so a numeric column merely *named* like a
  # p-value but holding other quantities passes through unchanged. Character
  # labels (significance symbols, pre-formatted strings, glue output) also pass
  # through unchanged. Opt out with p.digits = NULL.
  .p_value_label_columns <- c("p", "p.adj", "p.value", "p.val", "pval", "padj")
  .label_values <- data[[label]]
  if (!is.null(p.digits) && label %in% .p_value_label_columns &&
      is.numeric(.label_values) &&
      all(.label_values >= 0 & .label_values <= 1, na.rm = TRUE)) {
    data[[label]] <- format_p_value(
      .label_values, style = p.format.style, digits = p.digits,
      leading.zero = p.leading.zero, min.threshold = p.min.threshold,
      decimal.mark = p.decimal.mark
    )
  }

  y.position <- .valide_y_position(y.position, data)
  if (is.numeric(y.position)) {
    data$y.position <- y.position
    y.position <- "y.position"
  }


  # If xmax is null, pvalue is drawn as text
  if (!is.null(xmax)) {
    xmax <- data %>% pull(!!xmax)
    pvalue.geom <- "bracket"
  } else {
    xmax <- NA
    pvalue.geom <- "text"
  }
  if (!is.null(xmin)) {
    xmin <- data %>% pull(!!xmin)
  } else {
    xmin <- NA
  }

  # Build the statistical table for plotting
  xxmax <- xmax # so that mutate will avoid re-using an existing xmax in the data
  xxmin <- xmin
  data <- data %>%
    dplyr::mutate(
      label = as.character(data %>% pull(!!label)),
      y.position = data %>% pull(!!y.position),
      xmin = xxmin,
      xmax = xxmax
    )
  # vjust
  if (is.character(vjust)) {
    vjust <- data %>% pull(!!vjust)
  } else if (missing(vjust)) {
    vjust <- guess_labels_default_vjust(data$label)
  }
  data <- data %>% mutate(vjust = !!vjust)


  if (pvalue.geom == "bracket") {
    if (identical(data$xmin, data$xmax) | remove.bracket) {
      # case when ref.group = "all"
      bracket.size <- 0
    }

    geom_exec(
      geom_bracket,
      data = data, xmin = "xmin", xmax = "xmax",
      label = "label", y.position = "y.position", vjust = "vjust",
      group = seq_len(nrow(data)), tip.length = tip.length,
      tip.length.ref = tip.length.ref,
      label.size = label.size, size = bracket.size,
      bracket.nudge.y = bracket.nudge.y, bracket.shorten = bracket.shorten,
      color = color, linetype = linetype, step.increase = step.increase,
      step.group.by = step.group.by, coord.flip = coord.flip, position = position, inherit.aes = inherit.aes, ...
    )
  } else {
    if (comparison == "each_vs_ref") {
      ref.group <- unique(data$group1)
      group2 <- NULL
      data <- add_ctr_rows(data, ref.group = ref.group)
      mapping <- aes(x = xmin, y = y.position, vjust = vjust, label = label, group = group2)
      if (missing(position) & !missing(x)) {
        if (is_grouping_variable(x)) {
          position <- position_dodge(0.8)
        }
      }
    } else {
      mapping <- aes(x = xmin, y = y.position, vjust = vjust, label = label)
    }
    option <- list(data = data, size = label.size, position = position, inherit.aes = inherit.aes, ...)
    if (color %in% colnames(data)) {
      mapping$colour <- rlang::ensym(color)
    } else {
      option$color <- color
    }
    option[["mapping"]] <- mapping
    do.call(geom_text, option)
  }
}

asserttat_group_columns_exists <- function(data) {
  groups.exist <- all(c("group1", "group2") %in% colnames(data))
  if (!groups.exist) {
    if (inherits(data, "rstatix_test") & "group" %in% colnames(data)) {
      data$group1 <- "all"
      data$group2 <- data$group
    } else {
      stop("data should contain group1 and group2 columns")
    }
  }
  invisible(data)
}

# get validate p-value y-position
.valide_y_position <- function(y.position, data) {
  if (is.numeric(y.position)) {
    number.of.test <- nrow(data)
    number.of.ycoord <- length(y.position)
    xtimes <- number.of.test / number.of.ycoord

    if (number.of.ycoord < number.of.test) {
      y.position <- rep(y.position, xtimes)
    }
  } else if (is.character(y.position)) {
    if (!(y.position %in% colnames(data))) {
      stop("can't find the y.position variable '", y.position, "' in the data")
    }
  }
  return(y.position)
}


# Check if a string contains curly bracket
.contains_curlybracket <- function(x) {
  grepl("\\{|\\}", x, perl = TRUE)
}

# For ctr rows: the comparaison of ctr against itself
# useful only when positionning the label of grouped bars
add_ctr_rows <- function(data, ref.group) {
  xmin <- NULL
  data <- keep_only_tbl_df_classes(data)
  ctr <- data %>%
    dplyr::distinct(xmin, .keep_all = TRUE) %>%
    mutate(group2 = ref.group) %>%
    mutate(label = " ")
  dplyr::bind_rows(ctr, data)
}

# Returns the type of comparisons: one_group, two_groups, each_vs_ref, pairwise
detect_comparison_type <- function(data) {
  ngroup1 <- unique(data$group1) %>% length()
  ngroup2 <- unique(data$group2) %>% length()
  if (is_null_model(data)) {
    type <- "one_group"
  } else if (ngroup1 == 1 & ngroup2 >= 2) {
    type <- "each_vs_ref"
  } else if (ngroup1 == 1 & ngroup2 == 1) {
    type <- "two_groups"
  } else if (ngroup1 >= 2 & ngroup2 >= 2) {
    type <- "pairwise"
  } else if (all(c("group1", "group2") %in% colnames(data))) {
    # filtered data
    type <- "pairwise"
  } else {
    stop("Make sure that group1 and group2 columns exist in the data.")
  }
  type
}


is_null_model <- function(data) {
  group2 <- unique(data$group2)
  .diff <- setdiff(group2, "null model")
  length(.diff) == 0
}

is_grouping_variable <- function(x) {
  !(x %in% c("group1", "group2"))
}


# Check if label column contains stars
contains_signif_stars <- function(data, label.col) {
  result <- FALSE
  if (label.col[1] %in% colnames(data)) {
    labels <- data %>% pull(!!label.col)
    stars <- c("****", "***", "**", "*")
    result <- any(labels %in% stars)
  }
  result
}

# guess label default vjust
guess_label_default_vjust <- function(label) {
  if (label %in% c("****", "***", "**", "*")) {
    vjust <- 0.5
  } else {
    vjust <- 0
  }
  vjust
}
guess_labels_default_vjust <- function(labels) {
  labels %>%
    purrr::map(guess_label_default_vjust) %>%
    unlist()
}

# remove non significant
remove_ns <- function(data) {
  filter <- dplyr::filter
  columns <- colnames(data)
  if ("p.adj.signif" %in% columns) {
    data <- data %>% filter(.data$p.adj.signif != "ns")
  } else if ("p.adj" %in% columns) {
    data <- data %>% filter(.data$p.adj <= 0.05)
  } else if ("p.signif" %in% columns) {
    data <- data %>% filter(.data$p.signif != "ns")
  } else if ("p" %in% columns) {
    data <- data %>% filter(.data$p <= 0.05)
  }
  data
}

#' @include utilities.R
NULL
#' Draw a Textual Table
#' @description Draw a textual table. \itemize{ \item \code{ggtexttable()}: draw
#'  a textual table. \item \code{ttheme()}: customize table theme. \item
#'  \code{rownames_style(), colnames_style(), tbody_style()}: helper functions
#'  to customize the table row names, column names and body.
#'  \item \code{table_cell_font()}: access to a table cell for changing the text font (size and face).
#'  \item \code{table_cell_bg()}: access to a table cell for changing the background (fill, color, linewidth).
#'  \item \code{tab_cell_crossout()}: cross out a table cell.
#'  \item \code{tab_ncol(), tab_nrow()}: returns, respectively, the number of columns and rows in a ggtexttable.
#'  \item \code{tab_add_hline()}: Creates horizontal lines or separators at the top or the bottom side of a given specified row.
#'  \item \code{tab_add_vline()}: Creates vertical lines or separators at the right or the left side of a given specified column.
#'  \item \code{tab_add_border(), tbody_add_border(), thead_add_border()}: Add borders to table; tbody is for table body and thead is for table head.
#'  \item \code{tab_add_title(),tab_add_footnote()}: Add title, subtitle and footnote to a table.
#'   }
#' @inheritParams gridExtra::tableGrob
#' @param x a \code{data.frame} or \code{matrix}.
#' @param theme a list, as returned by the function \code{ttheme()}, defining the
#'  parameters of the table theme. Allowed values include one of \code{ttheme()}
#'  and \code{ttheme_clean()}.
#' @param base_style character string the table style/theme. The available themes
#'  are illustrated in the
#'  \href{https://rpkgs.datanovia.com/ggpubr/files/ggtexttable-theme.pdf}{ggtexttable-theme.pdf}
#'   file. Allowed values include one of \code{c("default", "blank", "classic",
#'  "minimal", "light", "lBlack", "lBlue", "lRed", "lGreen", "lViolet", "lCyan",
#'  "lOrange", "lBlackWhite", "lBlueWhite", "lRedWhite", "lGreenWhite",
#'  "lVioletWhite", "lCyanWhite", "lOrangeWhite", "mBlack", "mBlue", "mRed",
#'  "mGreen", "mViolet", "mCyan", "mOrange", "mBlackWhite", "mBlueWhite",
#'  "mRedWhite", "mGreenWhite", "mVioletWhite", "mCyanWhite", "mOrangeWhite" )}.
#'  Note that, l = "light"; m = "medium".
#' @param colnames.style a list, as returned by the function
#'  \code{colnames_style()}, defining the style of the table column names.
#'  Considered only when \code{base_size = "default"}.
#' @param rownames.style a list, as returned by the function
#'  \code{rownames_style()}, defining the style of the table row names.
#'  Considered only when \code{base_size = "default"}.
#' @param tbody.style a list, as returned by the function \code{tbody_style()},
#'  defining the style of the table body. Considered only when \code{base_size =
#'  "default"}.
#' @param color,face,size text font color, face and size, respectively. Allowed
#'  values for face include c("plain", "bold", "italic", "bold.italic").
#' @param fill background color.
#' @param linewidth,linecolor line width and color, respectively.
#' @param alpha numeric value specifying fill color transparency.
#' Value should be in [0, 1], where 0 is full transparency and 1 is no transparency.
#' @param at.row a numeric vector of row indexes; for example \code{at.row = c(1, 2)}.
#' @param row.side row side to which the horizontal line should be added. Can be one of \code{c("bottom", "top")}.
#' @param from.column integer indicating the column from which to start drawing the horizontal line.
#' @param to.column integer indicating the column to which the horizontal line should end.
#' @param linetype line type
#' @param at.column a numeric vector of column indexes; for example \code{at.column = c(1, 2)}.
#' @param column.side column side to which the vertical line should be added. Can be one of \code{c("left", "right")}.
#' @param from.row integer indicating the row from which to start drawing the horizontal line.
#' @param to.row integer indicating the row to which the vertical line should end.
#' @param ... extra parameters for text justification, e.g.: hjust and x. Default
#'  is "centre" for the body and header, and "right" for the row names. Left
#'  justification: \code{hjust = 0, x = 0.1}. Right justification: \code{hjust = 1, x = 0.9}.
#' @return an object of class ggplot.
#'
#'
#' @examples
#' # data
#' df <- head(iris)
#'
#' # Default table
#' # Remove row names using rows = NULL
#' ggtexttable(df, rows = NULL)
#'
#' # Text justification for individual cells/rows/columns (#335)
#' # First column is left justified i.e., hjust = 0 , x = 0.1
#' # Remaining columns are right justified i.e., hjust = 1 , x = 0.9
#' table_theme <- ttheme(
#'   tbody.style = tbody_style(
#'     hjust = as.vector(matrix(c(0, 1, 1, 1, 1), ncol = 5, nrow = nrow(df), byrow = TRUE)),
#'     x = as.vector(matrix(c(.1, .9, .9, .9, .9), ncol = 5, nrow = nrow(df), byrow = TRUE))
#'   )
#' )
#' ggtexttable(df, rows = NULL, theme = table_theme)
#'
#' # Blank theme
#' ggtexttable(df, rows = NULL, theme = ttheme("blank"))
#'
#' # light theme
#' ggtexttable(df, rows = NULL, theme = ttheme("light"))
#'
#' # Column names border only
#' ggtexttable(df, rows = NULL, theme = ttheme("blank")) %>%
#'   tab_add_hline(at.row = 1:2, row.side = "top", linewidth = 2)
#'
#' # classic theme
#' ggtexttable(df, rows = NULL, theme = ttheme("classic"))
#'
#' # minimal theme
#' ggtexttable(df, rows = NULL, theme = ttheme("minimal"))
#'
#' # Medium blue (mBlue) theme
#' ggtexttable(df, rows = NULL, theme = ttheme("mBlue"))
#'
#'
#' # Customize the table as you want
#' ggtexttable(df,
#'   rows = NULL,
#'   theme = ttheme(
#'     colnames.style = colnames_style(color = "white", fill = "#8cc257"),
#'     tbody.style = tbody_style(color = "black", fill = c("#e8f3de", "#d3e8bb"))
#'   )
#' )
#'
#' # Use RColorBrewer palette
#' # Provide as many fill color as there are rows in the table body, here nrow = 6
#' ggtexttable(df,
#'   theme = ttheme(
#'     colnames.style = colnames_style(fill = "white"),
#'     tbody.style = tbody_style(fill = get_palette("RdBu", 6))
#'   )
#' )
#'
#' # Text justification
#' # ::::::::::::::::::::::::::::::::::::::::::::::
#' # Default is "centre" for the body and header, and "right" for the row names.
#' # Left justification: hjust=0, x=0.1
#' # Right justification: hjust=1, x=0.9
#' tbody.style <- tbody_style(
#'   color = "black",
#'   fill = c("#e8f3de", "#d3e8bb"), hjust = 1, x = 0.9
#' )
#' ggtexttable(head(iris),
#'   rows = NULL,
#'   theme = ttheme(
#'     colnames.style = colnames_style(color = "white", fill = "#8cc257"),
#'     tbody.style = tbody.style
#'   )
#' )
#'
#' # Access and modify the font and
#' # the background of table cells
#' # :::::::::::::::::::::::::::::::::::::::::::::
#' tab <- ggtexttable(head(iris),
#'   rows = NULL,
#'   theme = ttheme("classic")
#' )
#' tab <- table_cell_font(tab,
#'   row = 3, column = 2,
#'   face = "bold"
#' )
#' tab <- table_cell_bg(tab,
#'   row = 4, column = 3, linewidth = 5,
#'   fill = "darkolivegreen1", color = "darkolivegreen4"
#' )
#' tab
#'
#' # Change table cells background and font for column 3,
#' # Spaning from row 2 to the last row in the data
#' tab <- ggtexttable(df, rows = NULL, theme = ttheme("classic"))
#' tab %>%
#'   table_cell_bg(row = 2:tab_nrow(tab), column = 3, fill = "darkblue") %>%
#'   table_cell_font(row = 2:tab_nrow(tab), column = 3, face = "italic", color = "white")
#'
#' # Add separators and borders
#' # :::::::::::::::::::::::::::::::::::::::::::::::::::
#' # Table with blank theme
#' tab <- ggtexttable(df, theme = ttheme("blank"), rows = NULL)
#' # Add horizontal and vertical lines
#' tab %>%
#'   tab_add_hline(at.row = c(1, 2), row.side = "top", linewidth = 3, linetype = 1) %>%
#'   tab_add_hline(at.row = c(7), row.side = "bottom", linewidth = 3, linetype = 1) %>%
#'   tab_add_vline(at.column = 2:tab_ncol(tab), column.side = "left", from.row = 2, linetype = 2)
#'
#' # Add borders to table body and header
#' # Cross out some cells
#' tab %>%
#'   tbody_add_border() %>%
#'   thead_add_border() %>%
#'   tab_cell_crossout(
#'     row = c(2, 4), column = 3, linecolor = "red",
#'     reduce.size.by = 0.6
#'   )
#'
#' # Add titles andd footnote
#' # :::::::::::::::::::::::::::::::::::::::::::::::::::
#' # Add titles and footnote
#' # Wrap subtitle into multiple lines using strwrap()
#' main.title <- "Edgar Anderson's Iris Data"
#' subtitle <- paste0(
#'   "This famous (Fisher's or Anderson's) iris data set gives the measurements",
#'   " in centimeters of the variables sepal length and width and petal length and width,",
#'   " respectively, for 50 flowers from each of 3 species of iris.",
#'   " The species are Iris setosa, versicolor, and virginica."
#' ) %>%
#'   strwrap(width = 80) %>%
#'   paste(collapse = "\n")
#'
#' tab <- ggtexttable(head(iris), theme = ttheme("light"))
#' tab %>%
#'   tab_add_title(text = subtitle, face = "plain", size = 10) %>%
#'   tab_add_title(text = main.title, face = "bold", padding = unit(0.1, "line")) %>%
#'   tab_add_footnote(text = "*Table created using ggpubr", size = 10, face = "italic")
#'
#'
#' # Combine density plot and summary table
#' # :::::::::::::::::::::::::::::::::::::
#' # Density plot of "Sepal.Length"
#' density.p <- ggdensity(iris,
#'   x = "Sepal.Length",
#'   fill = "Species", palette = "jco"
#' )
#'
#' # Draw the summary table of Sepal.Length
#' # Descriptive statistics by groups
#' stable <- desc_statby(iris,
#'   measure.var = "Sepal.Length",
#'   grps = "Species"
#' )
#' stable <- stable[, c("Species", "length", "mean", "sd")]
#' stable.p <- ggtexttable(stable,
#'   rows = NULL,
#'   theme = ttheme("mOrange")
#' )
#'
#' # Arrange the plots on the same page
#' ggarrange(density.p, stable.p,
#'   ncol = 1, nrow = 2,
#'   heights = c(1, 0.5)
#' )
#'
#' @rdname ggtexttable
#' @export
ggtexttable <- function(x, rows = rownames(x), cols = colnames(x), vp = NULL,
                        theme = ttheme(), ...) {
  style <- attr(theme, "style")
  res <- gridExtra::tableGrob(x,
    rows = rows, cols = cols, vp = vp,
    theme = theme, ...
  )
  if (style == "minimal") {
    # minimal = blank theme + left vertical line
    res <- tab_add_vline(
      res,
      at.column = 2:tab_ncol(res), column.side = "left",
      from.row = 2, to.row = tab_nrow(res)
    )
  } else if (style == "light") {
    # light = blank theme + horizontal line
    res <- tab_add_hline(
      res,
      at.row = c(1, tab_nrow(res)), row.side = "bottom",
      linewidth = 1.5, linetype = 1, from.column = 1
    )
  }
  .grob <- res
  res <- as_ggplot(res)
  attr(res, "ggtexttableGrob") <- .grob
  return(res)
}

#' @export
#' @rdname ggtexttable
ttheme <- function(base_style = "default", base_size = 11, base_colour = "black", padding = unit(c(4, 4), "mm"),
                   colnames.style = colnames_style(size = base_size),
                   rownames.style = rownames_style(size = base_size),
                   tbody.style = tbody_style(size = base_size)) {
  style <- tstyle(base_style, size = base_size)
  if (!is.null(style)) {
    if (missing(colnames.style)) colnames.style <- style$colnames.style
    if (missing(rownames.style)) rownames.style <- style$rownames.style
    if (missing(tbody.style)) tbody.style <- style$tbody.style
  }

  .ttheme <- gridExtra::ttheme_default(
    base_size = base_size,
    base_colour = base_colour,
    padding = padding
  )

  .ttheme$colhead <- do.call(.add_item, c(list(.list = .ttheme$colhead), colnames.style))
  .ttheme$rowhead <- do.call(.add_item, c(list(.list = .ttheme$rowhead), rownames.style))
  .ttheme$core <- do.call(.add_item, c(list(.list = .ttheme$core), tbody.style))
  attr(.ttheme, "style") <- base_style

  .ttheme
}


#' @export
#' @rdname ggtexttable
colnames_style <- function(color = "black", face = "bold", size = 12,
                           fill = "grey80", linewidth = 1, linecolor = "white",
                           parse = FALSE, ...) {
  list(
    fg_params = list(
      parse = parse, col = color,
      fontface = face, fontsize = size
    ) %>%
      .add_item(...), # Accept extra parameters
    bg_params = list(fill = fill, lwd = linewidth, col = linecolor)
  )
}

#' @export
#' @rdname ggtexttable
rownames_style <- function(color = "black", face = "italic", size = 12,
                           fill = NA, linewidth = 1, linecolor = "white",
                           parse = FALSE, ...) {
  list(
    fg_params = list(
      parse = parse, col = color,
      fontface = face, fontsize = size, hjust = 1, x = 0.95
    ) %>%
      .add_item(...), # Accept extra parameters
    bg_params = list(fill = fill, lwd = linewidth, col = linecolor)
  )
}


#' @export
#' @rdname ggtexttable
tbody_style <- function(color = "black", face = "plain", size = 12,
                        fill = c("grey95", "grey90"),
                        linewidth = 1, linecolor = "white",
                        parse = FALSE, ...) {
  list(
    fg_params = list(
      parse = parse, col = color,
      fontface = face, fontsize = size
    ) %>%
      .add_item(...), # Accept extra parameters
    bg_params = list(fill = fill, lwd = linewidth, col = linecolor)
  )
}


#' @export
#' @rdname ggtexttable
#' @param tab an object from \code{ggtexttable} or from \code{gridExtra::tableGrob()}.
#' @param row,column an integer specifying the row and the column numbers for the cell of interest.
table_cell_font <- function(tab, row, column, face = NULL, size = NULL, color = NULL) {
  tabGrob <- get_tablegrob(tab)
  cells <- expand.grid(row = row, column = column)
  for (i in seq_len(nrow(cells))) {
    # Match the body cells ("core-fg") as well as the header row ("colhead-fg"),
    # so an individual header cell can be styled (row 1), not only body cells (#535).
    tc <- .find_cell(tabGrob, cells$row[i], cells$column[i], c("core-fg", "colhead-fg"))
    tabGrob$grobs[tc][[1]][["gp"]] <- grid::gpar(fontface = face, fontsize = size, col = color)
  }
  tab_return_same_class_as_input(tabGrob, input = tab)
}

#' @export
#' @rdname ggtexttable
table_cell_bg <- function(tab, row, column, fill = NULL, color = NULL, linewidth = NULL, alpha = NULL) {
  tabGrob <- get_tablegrob(tab)
  cells <- expand.grid(row = row, column = column)
  for (i in seq_len(nrow(cells))) {
    # Match body ("core-bg") and header ("colhead-bg") backgrounds, so a single
    # header cell background can be changed too (#535).
    tc <- .find_cell(tabGrob, cells$row[i], cells$column[i], c("core-bg", "colhead-bg"))
    tabGrob$grobs[tc][[1]][["gp"]] <- grid::gpar(
      fill = fill, col = color, lwd = linewidth, alpha = alpha
    )
  }
  tab_return_same_class_as_input(tabGrob, input = tab)
}

.find_cell <- function(tab, row, column, name = "core-fg") {
  l <- tab$layout
  which(l$t == row & l$l == column & l$name %in% name)
}

#' @export
#' @rdname ggtexttable
#' @param reduce.size.by Numeric value in [0, 1] to reduce the size by.
tab_cell_crossout <- function(tab, row, column, linetype = 1, linewidth = 1, linecolor = "black", reduce.size.by = 0) {
  required_package("gtable")
  tabgrob <- get_tablegrob(tab)
  crosses <- replicate(
    n = length(row),
    tab_cross(linetype = linetype, linewidth = linewidth, linecolor = linecolor, reduce.size.by = reduce.size.by),
    simplify = FALSE
  )
  tabgrob <- gtable::gtable_add_grob(
    tabgrob,
    grobs = crosses,
    t = row, b = row,
    l = column, r = column
  )
  tab_return_same_class_as_input(tabgrob, input = tab)
}

tab_cross <- function(linetype = 1, linewidth = 1, linecolor = "black", reduce.size.by = 0) {
  reduce <- reduce.size.by / 2
  grid::grobTree(
    grid::segmentsGrob( # diagonal line ul -> lr
      x0 = unit(0 + reduce, "npc"),
      y0 = unit(1 - reduce, "npc"),
      x1 = unit(1 - reduce, "npc"),
      y1 = unit(0 + reduce, "npc"),
      gp = grid::gpar(lty = linetype, lwd = linewidth, col = linecolor, reduce = reduce)
    ),
    grid::segmentsGrob( # diagonal line ll -> ur
      x0 = unit(0 + reduce, "npc"),
      y0 = unit(0 + reduce, "npc"),
      x1 = unit(1 - reduce, "npc"),
      y1 = unit(1 - reduce, "npc"),
      gp = grid::gpar(lty = linetype, lwd = linewidth, col = linecolor)
    )
  )
}


#' @export
#' @rdname  ggtexttable
tab_ncol <- function(tab) {
  ncol(get_tablegrob(tab))
}

#' @export
#' @rdname  ggtexttable
tab_nrow <- function(tab) {
  nrow(get_tablegrob(tab))
}

#' @export
#' @rdname ggtexttable
tab_add_hline <- function(tab, at.row = 2:tab_nrow(tab), row.side = c("bottom", "top"),
                          from.column = 1, to.column = tab_ncol(tab),
                          linetype = 1, linewidth = 1, linecolor = "black") {
  required_package("gtable")
  row.side <- match.arg(row.side)
  tabgrob <- get_tablegrob(tab)
  separators <- replicate(
    n = length(at.row),
    tab_hline(row.side = row.side, linetype = linetype, linewidth = linewidth, linecolor = linecolor),
    simplify = FALSE
  )
  tabgrob <- gtable::gtable_add_grob(
    tabgrob,
    grobs = separators,
    t = at.row, b = at.row,
    l = from.column, r = to.column
  )
  tab_return_same_class_as_input(tabgrob, input = tab)
}

# Create hline at the top or the bottom side of a given row
tab_hline <- function(row.side = c("bottom", "top"), linetype = 1, linewidth = 1, linecolor = "black") {
  row.side <- match.arg(row.side)
  y0 <- y1 <- unit(0, "npc")
  if (row.side == "top") y0 <- y1 <- unit(1, "npc")
  grid::segmentsGrob(
    x0 = unit(0, "npc"), x1 = unit(1, "npc"),
    y0 = y0, y1 = y1,
    gp = grid::gpar(lty = linetype, lwd = linewidth, col = linecolor)
  )
}


#' @export
#' @rdname ggtexttable
tab_add_vline <- function(tab, at.column = 2:tab_ncol(tab), column.side = c("left", "right"),
                          from.row = 1, to.row = tab_nrow(tab),
                          linetype = 1, linewidth = 1, linecolor = "black") {
  required_package("gtable")
  column.side <- match.arg(column.side)
  tabgrob <- get_tablegrob(tab)
  separators <- replicate(
    n = length(at.column),
    tab_vline(column.side = column.side, linetype = linetype, linewidth = linewidth, linecolor = linecolor),
    simplify = FALSE
  )
  tabgrob <- gtable::gtable_add_grob(
    tabgrob,
    grobs = separators,
    t = from.row, b = to.row,
    l = at.column, r = at.column
  )
  tab_return_same_class_as_input(tabgrob, input = tab)
}
# Create vline at the left or the right side of a given column
tab_vline <- function(column.side = c("left", "right"), linetype = 1, linewidth = 1, linecolor = "black") {
  column.side <- match.arg(column.side)
  x0 <- x1 <- unit(0, "npc")
  if (column.side == "right") x0 <- x1 <- unit(1, "npc")
  grid::segmentsGrob(
    x0 = x0, x1 = x1,
    y0 = unit(0, "npc"), y1 = unit(1, "npc"),
    gp = grid::gpar(lty = linetype, lwd = linewidth, col = linecolor)
  )
}

#' @export
#' @rdname ggtexttable
tab_add_border <- function(tab, from.row = 2, to.row = tab_nrow(tab),
                           from.column = 1, to.column = tab_ncol(tab),
                           linetype = 1, linewidth = 1, linecolor = "black") {
  required_package("gtable")
  tabgrob <- get_tablegrob(tab)
  border <- grid::rectGrob(
    gp = grid::gpar(fill = NA, lty = linetype, lwd = linewidth, col = linecolor)
  )
  tabgrob <- gtable::gtable_add_grob(
    tabgrob,
    grobs = border,
    t = from.row, b = to.row,
    l = from.column, r = to.column
  )
  tab_return_same_class_as_input(tabgrob, input = tab)
}

#' @export
#' @rdname ggtexttable
tbody_add_border <- function(tab, from.row = 2, to.row = tab_nrow(tab),
                             from.column = 1, to.column = tab_ncol(tab),
                             linetype = 1, linewidth = 1, linecolor = "black") {
  tab_add_border(
    tab,
    from.row = from.row, to.row = to.row,
    from.column = from.column, to.column = to.column,
    linetype = linetype, linewidth = linewidth, linecolor = linecolor
  )
}

#' @export
#' @rdname ggtexttable
thead_add_border <- function(tab, from.row = 1, to.row = 1,
                             from.column = 1, to.column = tab_ncol(tab),
                             linetype = 1, linewidth = 1, linecolor = "black") {
  tab_add_border(
    tab,
    from.row = from.row, to.row = to.row,
    from.column = from.column, to.column = to.column,
    linetype = linetype, linewidth = linewidth, linecolor = linecolor
  )
}

# Anchor x-position for a title/footnote text grob derived from `just`, so that
# `just` actually positions the text across the table width. Previously x was
# hardcoded, so "center"/"right" only aligned the text around a fixed left/right
# anchor and a centred title spilled off-canvas and clipped (#302). The `left`,
# `center` and `right` defaults keep each function's historical home position
# byte-identical (title left = 0.02, footnote right = 0.95).
.tab_text_x <- function(just, left = 0.02, center = 0.5, right = 0.98, home = left) {
  # A numeric or unrecognized `just` keeps the historical hardcoded anchor
  # (`home`), so those calls are unchanged; only the named string justifications
  # reposition the text.
  if (!is.character(just) || length(just) != 1) {
    return(home)
  }
  switch(just,
    "left" = left,
    "center" = center, "centre" = center, "middle" = center,
    "right" = right,
    home
  )
}

#' @export
#' @rdname ggtexttable
#' @param text text to be added as title or footnote.
tab_add_title <- function(tab, text, face = NULL, size = NULL, color = NULL,
                          family = NULL, padding = unit(1.5, "line"),
                          just = "left", hjust = NULL, vjust = NULL) {
  required_package("gtable")
  tabgrob <- get_tablegrob(tab)
  text <- grid::textGrob(
    text,
    x = .tab_text_x(just), just = just, hjust = hjust, vjust = vjust,
    gp = grid::gpar(fontsize = size, fontface = face, fontfamily = family, col = color)
  )
  # Add row at the top
  tabgrob <- gtable::gtable_add_rows(
    tabgrob,
    heights = grid::grobHeight(text) + padding,
    pos = 0
  )
  tabgrob <- gtable::gtable_add_grob(
    tabgrob, list(text),
    t = 1, b = 1, l = 1, r = ncol(tabgrob),
    # Don't clip the title to the (often narrow) table width, so a title wider
    # than the table is fully visible instead of being cut off (#302).
    clip = "off"
  )
  tab_return_same_class_as_input(tabgrob, input = tab)
}


#' @export
#' @rdname ggtexttable
#' @param text text to be added as title or footnote.
#' @param family font family
#' @param just The justification of the text relative to its (x, y) location. If
#'   there are two values, the first value specifies horizontal justification
#'   and the second value specifies vertical justification. Possible string
#'   values are: "left", "right", "centre", "center", "bottom", and "top". For
#'   numeric values, 0 means left (bottom) alignment and 1 means right (top)
#'   alignment.
#' @param hjust A numeric vector specifying horizontal justification. If
#'   specified, overrides the just setting.
#' @param vjust A numeric vector specifying vertical justification. If
#'   specified, overrides the just setting.
#'
tab_add_footnote <- function(tab, text, face = NULL, size = NULL, color = NULL,
                             family = NULL, padding = unit(1.5, "line"),
                             just = "right", hjust = NULL, vjust = NULL) {
  required_package("gtable")
  tabgrob <- get_tablegrob(tab)
  text <- grid::textGrob(
    text,
    x = .tab_text_x(just, right = 0.95, home = 0.95), just = just, hjust = hjust, vjust = vjust,
    gp = grid::gpar(fontsize = size, fontface = face, fontfamily = family, col = color)
  )
  # Add row at the bottom
  tabgrob <- gtable::gtable_add_rows(
    tabgrob,
    heights = grid::grobHeight(text) + padding,
    pos = -1
  )
  tabgrob <- gtable::gtable_add_grob(
    tabgrob, list(text),
    t = nrow(tabgrob), b = nrow(tabgrob), l = 1, r = ncol(tabgrob),
    # Don't clip a footnote wider than the table width (#302).
    clip = "off"
  )
  tab_return_same_class_as_input(tabgrob, input = tab)
}


# ::::::::::::::::::::::::::::::::::::::::
# Helper function
# ::::::::::::::::::::::::::::::::::::::::

is_ggtexttable <- function(tab) {
  !is.null(attr(tab, "ggtexttableGrob"))
}
is_tablegrob <- function(tab) {
  inherits(tab, "gtable") & inherits(tab, "grob")
}

# Transform a table grob in ggtexttable like object
as_ggtexttable <- function(tabgrob) {
  res <- as_ggplot(tabgrob)
  attr(res, "ggtexttableGrob") <- tabgrob
  res
}

# Extract tableGrob from ggtexttable()
get_tablegrob <- function(tab) {
  if (is_ggtexttable(tab)) {
    tabgrob <- attr(tab, "ggtexttableGrob")
  } else if (is_tablegrob(tab)) {
    tabgrob <- tab
  } else {
    stop("tab should be an object from either ggpubr::ggtexttable() or gridExtra::tableGrob().")
  }
  tabgrob
}

# Return the same class as the input data,
# which can be either ggtextable or a gridExtra::tableGrob
tab_return_same_class_as_input <- function(tabgrob, input) {
  if (is_ggtexttable(input)) {
    return(as_ggtexttable(tabgrob))
  } else if (is_tablegrob(input)) {
    return(tabgrob)
  }
  tabgrob
}


# Define table style
tstyle <- function(pal, size = 12) {
  allowed.palettes <- c(
    "default", "blank", "classic", "minimal", "light",
    "lBlack", "lBlue", "lRed", "lGreen", "lViolet", "lCyan", "lOrange",
    "lBlackWhite", "lBlueWhite", "lRedWhite", "lGreenWhite", "lVioletWhite", "lCyanWhite", "lOrangeWhite",
    "mBlack", "mBlue", "mRed", "mGreen", "mViolet", "mCyan", "mOrange",
    "mBlackWhite", "mBlueWhite", "mRedWhite", "mGreenWhite", "mVioletWhite", "mCyanWhite", "mOrangeWhite"
  )
  if (!(pal %in% allowed.palettes)) {
    stop(pal, " is not a supported palette")
  }

  style <- switch(pal,
    blank = list(
      colnames.style = colnames_style(fill = NA, linecolor = NA, size = size),
      rownames.style = rownames_style(fill = NA, linecolor = NA, size = size),
      tbody.style = tbody_style(fill = NA, linecolor = NA, size = size)
    ),
    classic = list(
      colnames.style = colnames_style(fill = NA, linecolor = "black", size = size),
      rownames.style = rownames_style(fill = NA, linecolor = NA, size = size),
      tbody.style = tbody_style(fill = NA, linecolor = "black", size = size)
    ),
    minimal = list(
      colnames.style = colnames_style(fill = NA, linecolor = NA, size = size),
      rownames.style = rownames_style(fill = NA, linecolor = NA, size = size),
      tbody.style = tbody_style(fill = NA, linecolor = NA, size = size)
    ),
    light = list(
      colnames.style = colnames_style(fill = NA, linecolor = NA, size = size),
      rownames.style = rownames_style(fill = NA, linecolor = NA, size = size),
      tbody.style = tbody_style(fill = NA, linecolor = NA, size = size)
    ),
    lBlack = list(
      colnames.style = colnames_style(
        color = "black", fill = "white", size = size,
        linecolor = NA, linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#D9D9D9", "white"),
        linecolor = NA, size = size
      )
    ),
    lBlue = list(
      colnames.style = colnames_style(
        color = "#2F5D94", fill = "white",
        linecolor = NA, linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "#2F5D94", fill = c("#DAE6F2", "white"),
        linecolor = NA, size = size
      )
    ),
    lRed = list(
      colnames.style = colnames_style(
        color = "#A3262A", fill = "white",
        linecolor = NA, linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "#A3262A", fill = c("#F7DBDA", "white"),
        linecolor = NA, size = size
      )
    ),
    lGreen = list(
      colnames.style = colnames_style(
        color = "#7F993A", fill = "white",
        linecolor = NA, linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "#7F993A", fill = c("#E8F3DE", "white"),
        linecolor = NA, size = size
      )
    ),
    lViolet = list(
      colnames.style = colnames_style(
        color = "#67417B", fill = "white",
        linecolor = NA, linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "#67417B", fill = c("#E6DEEC", "white"),
        linecolor = NA, size = size
      )
    ),
    lCyan = list(
      colnames.style = colnames_style(
        color = "#00889E", fill = "white",
        linecolor = NA, linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "#00889E", fill = c("#D5EFF4", "white"),
        linecolor = NA, size = size
      )
    ),
    lOrange = list(
      colnames.style = colnames_style(
        color = "#F15F00", fill = "white",
        linecolor = NA, linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "#F15F00", fill = c("#FBE5D4", "white"),
        linecolor = NA, size = size
      )
    ),
    lBlackWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "black", size = size,
        linecolor = "black", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "black", size = size
      )
    ),
    lBlueWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#477DC0", size = size,
        linecolor = "#477DC0", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "#477DC0", size = size
      )
    ),
    lRedWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#D04042", size = size,
        linecolor = "#D04042", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "#D04042", size = size
      )
    ),
    lGreenWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#8CC257", size = size,
        linecolor = "#8CC257", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "#8CC257", size = size
      )
    ),
    lVioletWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#895AA3", size = size,
        linecolor = "#895AA3", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "#895AA3", size = size
      )
    ),
    lCyanWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#00AEC9", size = size,
        linecolor = "#00AEC9", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "#00AEC9", size = size
      )
    ),
    lOrangeWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#FB8F2D", size = size,
        linecolor = "#FB8F2D", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = "white",
        linecolor = "#FB8F2D", size = size
      )
    ),
    mBlack = list(
      colnames.style = colnames_style(
        color = "white", fill = "black", size = size,
        linecolor = "white", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#A6A6A6", "#D9D9D9"),
        linecolor = "white", size = size
      )
    ),
    mBlue = list(
      colnames.style = colnames_style(
        color = "white", fill = "#477DC0",
        linecolor = "white", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#B5CBE5", "#DAE6F2"),
        linecolor = "white", size = size
      )
    ),
    mRed = list(
      colnames.style = colnames_style(
        color = "white", fill = "#D04042",
        linecolor = "white", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#EFB4B5", "#F7DBDA"),
        linecolor = "white", size = size
      )
    ),
    mGreen = list(
      colnames.style = colnames_style(
        color = "white", fill = "#8CC257",
        linecolor = "white", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#D3E8BB", "#E8F3DE"),
        linecolor = "white", size = size
      )
    ),
    mViolet = list(
      colnames.style = colnames_style(
        color = "white", fill = "#895AA3",
        linecolor = "white", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#D0BDDA", "#E6DEEC"),
        linecolor = "white", size = size
      )
    ),
    mCyan = list(
      colnames.style = colnames_style(
        color = "white", fill = "#00AEC9",
        linecolor = "white", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#ADDFEA", "#D5EFF4"),
        linecolor = "white", size = size
      )
    ),
    mOrange = list(
      colnames.style = colnames_style(
        color = "white", fill = "#EC8C2C",
        linecolor = "white", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#FDD2AF", "#FFE9D7"),
        linecolor = "white", size = size
      )
    ),
    mBlackWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "black", size = size,
        linecolor = "black", linewidth = 1
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#A6A6A6", "white"),
        linecolor = "black", size = size
      )
    ),
    mBlueWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#477DC0",
        linecolor = "#477DC0", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#B5CBE5", "white"),
        linecolor = "#477DC0", size = size
      )
    ),
    mRedWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#D04042",
        linecolor = "#D04042", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#EFB4B5", "white"),
        linecolor = "#D04042", size = size
      )
    ),
    mGreenWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#8CC257",
        linecolor = "#8CC257", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#D3E8BB", "white"),
        linecolor = "#8CC257", size = size
      )
    ),
    mVioletWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#895AA3",
        linecolor = "#895AA3", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#D0BDDA", "white"),
        linecolor = "#895AA3", size = size
      )
    ),
    mCyanWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#00AEC9",
        linecolor = "#00AEC9", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#ADDFEA", "white"),
        linecolor = "#00AEC9", size = size
      )
    ),
    mOrangeWhite = list(
      colnames.style = colnames_style(
        color = "white", fill = "#EC8C2C",
        linecolor = "#EC8C2C", linewidth = 1, size = size
      ),
      rownames.style = rownames_style(size = size),
      tbody.style = tbody_style(
        color = "black", fill = c("#FDD2AF", "white"),
        linecolor = "#EC8C2C", size = size
      )
    ),
    NULL
  )

  style
}

# Regression tests for #535: table_cell_font()/table_cell_bg() can now style an
# individual header cell (row 1, "colhead-*"), not only body cells ("core-*").

.df <- data.frame(Group = c("a", "b", "c"), Value = c(1, 2, 3))

.cell_gp <- function(tab, row, column, names) {
  tg <- ggpubr:::get_tablegrob(tab)
  l <- tg$layout
  i <- which(l$t == row & l$l == column & l$name %in% names)
  tg$grobs[[i]]$gp
}

test_that("table_cell_font() styles a single header cell (#535)", {
  tab <- ggtexttable(.df, rows = NULL)
  tab <- table_cell_font(tab, row = 1, column = 2, face = "italic", color = "red")
  gp <- .cell_gp(tab, 1, 2, c("core-fg", "colhead-fg"))
  # grid normalises fontface = "italic" into font = c(italic = 3L)
  expect_equal(names(gp$font), "italic")
  expect_equal(gp$col, "red")
  # the other header cell is untouched
  gp1 <- .cell_gp(tab, 1, 1, c("core-fg", "colhead-fg"))
  expect_false(identical(gp1$col, "red"))
})

test_that("table_cell_bg() styles a single header cell background (#535)", {
  tab <- ggtexttable(.df, rows = NULL)
  tab <- table_cell_bg(tab, row = 1, column = 2, fill = "yellow")
  gp <- .cell_gp(tab, 1, 2, c("core-bg", "colhead-bg"))
  expect_equal(gp$fill, "yellow")
})

test_that("body-cell styling is unchanged (no regression, #535)", {
  tab <- ggtexttable(.df, rows = NULL)
  tab <- table_cell_font(tab, row = 2, column = 1, face = "bold", color = "blue")
  gp <- .cell_gp(tab, 2, 1, "core-fg")
  expect_equal(names(gp$font), "bold")
  expect_equal(gp$col, "blue")
  expect_no_error(print(tab))
})

# Regression tests for #302: tab_add_title()/tab_add_footnote() ignored `just`
# (the anchor x was hardcoded, so "center"/"right" only aligned the text around a
# fixed point) and clipped a title/footnote wider than the table.

.df <- tibble::tibble(X1 = c(1, 2), Y1 = c("a", "b"))

.grob_x <- function(tab, pattern) {
  tg <- ggpubr:::get_tablegrob(tab)
  i <- which(vapply(tg$grobs, function(g)
    inherits(g, "text") && grepl(pattern, g$label %||% ""), logical(1)))[1]
  as.numeric(tg$grobs[[i]]$x)
}
.grob_clip <- function(tab, pattern) {
  tg <- ggpubr:::get_tablegrob(tab)
  i <- which(vapply(tg$grobs, function(g)
    inherits(g, "text") && grepl(pattern, g$label %||% ""), logical(1)))[1]
  # find the layout row for that grob
  tg$layout$clip[which(vapply(tg$grobs, function(g) identical(g, tg$grobs[[i]]), logical(1)))[1]]
}

test_that("tab_add_title(just=) positions the title (#302)", {
  center <- ggtexttable(.df, rows = NULL) %>% tab_add_title("T", just = "center")
  right  <- ggtexttable(.df, rows = NULL) %>% tab_add_title("T", just = "right")
  left   <- ggtexttable(.df, rows = NULL) %>% tab_add_title("T", just = "left")
  expect_equal(.grob_x(center, "^T$"), 0.5)
  expect_equal(.grob_x(right, "^T$"), 0.98)
  expect_equal(.grob_x(left, "^T$"), 0.02)  # default, byte-identical
})

test_that("tab_add_footnote(just=) keeps its right default and honors center (#302)", {
  def <- ggtexttable(.df, rows = NULL) %>% tab_add_footnote("F")
  ctr <- ggtexttable(.df, rows = NULL) %>% tab_add_footnote("F", just = "center")
  expect_equal(.grob_x(def, "^F$"), 0.95)   # default right, byte-identical
  expect_equal(.grob_x(ctr, "^F$"), 0.5)
})

test_that("title/footnote are not clipped to the table width (#302)", {
  tab <- ggtexttable(.df, rows = NULL) %>%
    tab_add_title("A very wide sample table title", just = "center") %>%
    tab_add_footnote("A very wide footnote text here", just = "center")
  expect_equal(.grob_clip(tab, "wide sample"), "off")
  expect_equal(.grob_clip(tab, "wide footnote"), "off")
  expect_no_error(ggplot2::ggplot_gtable(ggplot2::ggplot_build(tab)))
})

#' @include utilities.R
NULL
#' Add Central Tendency Measures to a GGPlot
#'
#' @description Add central tendency measures (mean, median, mode) to density
#'   and histogram plots created using ggplots.
#'
#'   Note that, normally, the mode is used for categorical data where we wish to
#'   know which is the most common category. Therefore, we can have two or
#'   more values that share the highest frequency. This might be problematic for
#'   a continuous variable.
#'
#'   For a continuous variable, we can consider using mean or median as the
#'   measures of the central tendency.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @param ... other arguments to pass to \code{\link[ggplot2:geom_path]{geom_line}}.
#' @param na.rm If FALSE (the default), removes missing values with a warning.
#'   If TRUE silently removes missing values.
#' @param type the type of central tendency measure to be used. Possible values
#'   include: \code{"mean", "median", "mode"}.
#' @seealso \code{\link{ggdensity}}
#' @examples
#' # Simple density plot
#' data("mtcars")
#' ggdensity(mtcars, x = "mpg", fill = "red") +
#'   scale_x_continuous(limits = c(-1, 50)) +
#'   stat_central_tendency(type = "mean", linetype = "dashed")
#'
#' # Color by groups
#' data(iris)
#' ggdensity(iris, "Sepal.Length", color = "Species") +
#'   stat_central_tendency(aes(color = Species), type = "median", linetype = 2)
#'
#' # Use geom = "point" for central tendency
#' data(iris)
#' ggdensity(iris, "Sepal.Length", color = "Species") +
#'   stat_central_tendency(
#'     aes(color = Species),
#'     type = "median",
#'     geom = "point", size = 4
#'   )
#'
#' # Facet
#' ggdensity(iris, "Sepal.Length", facet.by = "Species") +
#'   stat_central_tendency(type = "mean", color = "red", linetype = 2) +
#'   stat_central_tendency(type = "median", color = "blue", linetype = 2)
#'
#' @export
stat_central_tendency <- function(mapping = NULL, data = NULL, geom = c("line", "point"),
                                  position = "identity", na.rm = FALSE, show.legend = NA,
                                  inherit.aes = TRUE,
                                  type = c("mean", "median", "mode"),
                                  ...) {
  type <- match.arg(type)
  geom <- match.arg(geom)
  if (is.null(mapping)) {
    mapping <- ggplot2::aes(y = NULL)
  } else {
    mapping["y"] <- list(NULL)
  }
  layer(
    stat = StatCentralTendency, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, type = type, geom = geom, ...)
  )
}

StatCentralTendency <- ggproto("StatCentralTendency", Stat,
  required_aes = c("x"),
  compute_group = function(data, scales, type, geom) {
    center.func <- switch(type,
      mean = mean,
      median = stats::median,
      mode = .get_mode
    )
    .center <- center.func(data$x, na.rm = TRUE)
    n <- length(.center)
    if (geom == "line") {
      results <- data.frame(x = rep(.center, each = 2), y = c(-Inf, Inf))
      if (n >= 2) results$group <- rep(seq_len(n), each = 2)
    } else if (geom == "point") {
      results <- data.frame(x = .center, y = -Inf)
    }
    results
  }
)


# from rstatix
.get_mode <- function(x, na.rm = TRUE) {
  if (na.rm) x <- stats::na.omit(x)
  .x <- factor(x)
  .table <- table(.x)
  .max <- max(.table)
  if (all(.table == .max)) {
    .mode <- NA
  } else {
    .mode <- names(.table)[.table == .max]
  }
  if (is.numeric(x)) {
    .mode <- as.numeric(.mode)
  }
  .mode
}

#' Descriptive statistics by groups
#'
#' @description Computes descriptive statistics by groups for a measure variable.
#'
#' @param data a data frame.
#' @param measure.var the name of a column containing the variable to be summarized.
#' @param grps a character vector containing grouping variables; e.g.: grps = c("grp1", "grp2")
#' @param ci the percent range of the confidence interval (default is 0.95).
#'
#' @return
#' A data frame containing descriptive statistics, such as:
#' \itemize{
#' \item \strong{length}: the number of elements in each group
#' \item \strong{min}: minimum
#' \item \strong{max}: maximum
#' \item \strong{median}: median
#' \item \strong{mean}: mean
#' \item \strong{iqr}: interquartile range
#' \item \strong{mad}: median absolute deviation (see ?MAD)
#' \item \strong{sd}: standard deviation of the sample
#' \item \strong{se}: standard error of the mean. It's calculated as the sample standard deviation divided by the root of the sample size.
#' \item \strong{ci}: confidence interval of the mean
#' \item \strong{range}: the range = max - min
#' \item \strong{cv}: coefficient of variation, sd/mean
#' \item \strong{var}: variance, sd^2
#' }
#'
#' @examples
#' # Load data
#' data("ToothGrowth")
#'
#' # Descriptive statistics
#' res <- desc_statby(ToothGrowth,
#'   measure.var = "len",
#'   grps = c("dose", "supp")
#' )
#' head(res[, 1:10])
#'
#' @export
desc_statby <- function(data, measure.var, grps, ci = 0.95) {
  if (!inherits(data, "data.frame")) {
    stop("data must be a data.frame.")
  }

  data %>%
    as.data.frame() %>%
    df_group_by(vars = grps) %>%
    reframe(.summary(.data[[measure.var]], ci = ci)) %>%
    as.data.frame()
}

# Helper function to compute summary statistics
.summary <- function(x, ci = 0.95) {
  if (is.data.frame(x)) {
    if (ncol(x) == 1) x <- .select_vec(x, 1)
  }
  if (!is.numeric(x)) {
    stop("x should be a numeric vector or a data frame with one numeric column")
  }

  data_sum <- data.frame(
    length = base::sum(!is.na(x)),
    min = base::min(x, na.rm = TRUE),
    max = base::max(x, na.rm = TRUE),
    median = stats::median(x, na.rm = TRUE),
    mean = base::mean(x, na.rm = TRUE),
    iqr = stats::IQR(x, na.rm = TRUE),
    mad = stats::mad(x, na.rm = TRUE),
    sd = stats::sd(x, na.rm = TRUE)
  )

  data_sum$se <- data_sum$sd / sqrt(data_sum$length) # standard error
  # Confidence interval from t-distribution
  # e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
  df <- data_sum$length - 1
  data_sum$ci <- ifelse(
    is.finite(df) & df > 0 & is.finite(data_sum$se),
    stats::qt(ci / 2 + .5, df) * data_sum$se,
    NA_real_
  )
  data_sum$range <- data_sum$max - data_sum$min
  data_sum$cv <- data_sum$sd / data_sum$mean
  data_sum$var <- data_sum$sd^2
  data_sum[, 2:ncol(data_sum)] <- data_sum[, 2:ncol(data_sum)]
  return(data_sum)
}

#' @include utilities.R ggpar.R
NULL
#' Visualizing Error
#' @description Visualizing error.
#' @inheritParams ggboxplot
#' @inheritParams ggplot2::geom_errorbar
#' @param ci the percent range of the confidence interval (default is 0.95).
#' @param x,y x and y variables for drawing.
#' @param numeric.x.axis logical. If TRUE, x axis will be treated as numeric. Default is FALSE.
#' @param order character vector specifying the order of items. Considered only when x axis is a factor variable.
#' @param color,fill outline and fill colors.
#' @param desc_stat descriptive statistics to be used for visualizing errors. Default value is "mean_se".
#' Allowed values are one of , "mean", "mean_se", "mean_sd", "mean_ci", "mean_range",
#' "median", "median_iqr", "median_hilow", "median_q1q3", "median_mad", "median_range"; see \code{\link{desc_statby}} for more details.
#' @param ... other arguments to be passed to be passed to ggpar().
#' @details The plot can be easily customized using the function ggpar(). Read
#'   ?ggpar for changing: \itemize{ \item main title and axis labels: main,
#'   xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
#'   scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
#'   palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
#'   labels and position: legend = "right" \item plot orientation : orientation
#'   = c("vertical", "horizontal", "reverse") }
#' @seealso \code{\link{ggpar}}, \code{\link{ggline}}
#' @examples
#'
#' # Data: ToothGrowth data set we'll be used.
#' df <- ToothGrowth
#' head(df, 10)
#'
#' # Plot mean_se
#' ggerrorplot(df, x = "dose", y = "len")
#'
#'
#' # Change desc_stat to mean_sd
#' # (other values include: mean_sd, mean_ci, median_iqr, ....)
#' # Add labels
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   desc_stat = "mean_sd"
#' )
#'
#' # Change error.plot to "errorbar" and add mean point
#' # Visualize the mean of each group
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   add = "mean", error.plot = "errorbar"
#' )
#'
#' # Horizontal plot
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   add = "mean", error.plot = "errorbar",
#'   orientation = "horizontal"
#' )
#'
#'
#' # Change error.plot to "crossbar"
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   error.plot = "crossbar", width = 0.5
#' )
#'
#'
#' # Add jitter points and errors (mean_se)
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   add = "jitter"
#' )
#'
#' # Add dot and errors (mean_se)
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   add = "dotplot"
#' )
#'
#' # Multiple groups with error bars and jitter point
#' ggerrorplot(df,
#'   x = "dose", y = "len",
#'   color = "supp", palette = "Paired",
#'   error.plot = "pointrange",
#'   position = position_dodge(0.5)
#' )
#' #
#'
#' @export
ggerrorplot <- function(data, x, y, desc_stat = "mean_se",
                        numeric.x.axis = FALSE,
                        combine = FALSE, merge = FALSE,
                        color = "black", fill = "white", palette = NULL,
                        size = NULL, width = NULL,
                        title = NULL, xlab = NULL, ylab = NULL,
                        facet.by = NULL, panel.labs = NULL, short.panel.labs = TRUE,
                        select = NULL, remove = NULL, order = NULL,
                        add = "none",
                        add.params = list(),
                        error.plot = "pointrange", ci = 0.95,
                        position = position_dodge(),
                        ggtheme = theme_pubr(),
                        ...) {
  # Default options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .opts <- list(
    desc_stat = desc_stat,
    numeric.x.axis = numeric.x.axis,
    combine = combine, merge = merge,
    color = color, fill = fill, palette = palette,
    size = size, width = width,
    title = title, xlab = xlab, ylab = ylab,
    facet.by = facet.by, panel.labs = panel.labs, short.panel.labs = short.panel.labs,
    select = select, remove = remove, order = order,
    add = add, add.params = add.params, error.plot = error.plot, ci = ci,
    position = position, ggtheme = ggtheme, ...
  )
  if (!missing(data)) .opts$data <- data
  if (!missing(x)) .opts$x <- x
  if (!missing(y)) .opts$y <- y

  # User options
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  .user.opts <- as.list(match.call(expand.dots = TRUE))
  .user.opts[[1]] <- NULL # Remove the function name
  # keep only user arguments
  for (opt.name in names(.opts)) {
    if (is.null(.user.opts[[opt.name]])) {
      .opts[[opt.name]] <- NULL
    }
  }
  .opts$fun <- ggerrorplot_core
  if (missing(ggtheme) & (!is.null(facet.by) | combine)) {
    .opts$ggtheme <- theme_pubr(border = TRUE)
  }
  # Honor an explicit `ggtheme = NULL` (skip theming). The NULL-filter loop above
  # drops it like an unset argument, so restore any explicitly passed value here,
  # keeping an explicit NULL intact via single-bracket list assignment (#561).
  if (!missing(ggtheme)) .opts["ggtheme"] <- list(ggtheme)
  p <- do.call(.plotter, .opts)

  if (.is_list(p) & length(p) == 1) p <- p[[1]]
  return(p)
}


ggerrorplot_core <- function(data, x, y, desc_stat = "mean_se",
                             numeric.x.axis = FALSE,
                             color = "black", fill = "white", palette = NULL,
                             title = NULL, xlab = NULL, ylab = NULL,
                             size = NULL, width = NULL,
                             select = NULL, order = NULL,
                             add = "none",
                             add.params = list(),
                             error.plot = "pointrange", ci = 0.95,
                             position = position_dodge(0.8),
                             ggtheme = theme_pubr(),
                             ...) {
  if (numeric.x.axis) {} else {
    if (!is.null(order)) {
      data[[x]] <- factor(data[[x]], levels = order)
    } else if (!is.factor(data[[x]])) data[[x]] <- as.factor(data[[x]])
  }
  error.plot <- error.plot[1]
  if ("none" %in% add) add <- "none"

  # static summaries for computing mean/median and adding errors
  if (is.null(add.params$fill)) add.params$fill <- "white"
  add.params <- .check_add.params(add, add.params, error.plot, data, color, fill, ...)

  add <- setdiff(add, desc_stat)
  if (inherits(position, "PositionDodge") & is.null(position$width)) position$width <- 0.8
  p <- ggplot(data, create_aes(list(x = x, y = y)))
  add.params <- add.params %>%
    .add_item(add = add, data = data, error.plot = error.plot, ci = ci, position = position, p = p)
  p <- do.call(ggadd, add.params)

  # Main plot
  add.params <- add.params %>%
    .add_item(
      color = color, fill = fill, size = size, width = width,
      add = desc_stat, p = p
    )
  p <- do.call(ggadd, add.params)

  p <- ggpar(p,
    palette = palette, ggtheme = ggtheme,
    title = title, xlab = xlab, ylab = ylab, ...
  )

  p
}

.brewerpal <- function() {
  c(
    # sequential
    "Blues", "BuGn", "BuPu", "GnBu", "Greens", "Greys", "Oranges",
    "OrRd", "PuBu", "PuBuGn", "PuRd", "Purples", "RdPu", "Reds",
    "YlGn", "YlGnBu", "YlOrBr", "YlOrRd",
    # Divergent
    "BrBG", "PiYG", "PRGn", "PuOr", "RdBu", "RdGy", "RdYlBu", "RdYlGn", "Spectral",
    # Qualitative
    "Accent", "Dark2", "Paired", "Pastel1", "Pastel2", "Set1", "Set2", "Set3"
  )
}
.ggscipal <- function() {
  # Scientific Journal and Sci-Fi Themed Color Palettes for ggplot2
  # ggsci package: https://cran.r-project.org/web/packages/ggsci/vignettes/ggsci.html
  c(
    "npg", "aaas", "nejm", "lancet", "jama", "jco", "ucscgb", "d3", "locuszoom",
    "igv", "uchicago", "startrek", "tron", "futurama", "rickandmorty", "simpsons"
  )
}

# Check if color palette or default hue
.is_col_palette <- function(pal) {
  if (is.null(pal)) {
    return(FALSE)
  } else {
    return(length(pal) == 1 & pal[1] %in% c(
      .brewerpal(), .ggscipal(),
      "default", "hue", "grey_pal", "gray_pal"
    ))
  }
}
.is_color_palette <- .is_col_palette # alias

# Change color manually
# possible value for palette: brewer palette, "grey" or a vector of colors
.ggcolor <- function(palette = NULL, ...) {
  color_palette(palette = palette, ...)
}

# Change fill color manually
# possible value for palette: brewer palette, "grey" or a vector of colors
.ggfill <- function(palette = NULL, ...) {
  fill_palette(palette = palette, ...)
}

# Aesthetics mapped in a plot (plot-level + all layer-level mappings)
.mapped_aesthetics <- function(p) {
  used <- names(p$mapping)
  for (layer in p$layers) {
    used <- c(used, names(layer$mapping))
  }
  unique(used)
}

# Apply the color/fill palette to a plot.
# For an UNNAMED palette (or NULL) the previous behavior is preserved exactly:
# both the color and fill scales are added. For a NAMED palette, the manual
# scale is only added for an aesthetic actually mapped in the plot -- adding
# scale_color/fill_manual() with named values to an aesthetic whose data has
# none of those names raises a spurious "No shared levels found ..." warning at
# draw time (#642).
.apply_palette <- function(p, palette) {
  is_named_palette <- !is.null(palette) &&
    !is.null(names(palette)) && any(nzchar(names(palette)))
  if (!is_named_palette) {
    return(p + .ggcolor(palette) + .ggfill(palette))
  }
  used <- .mapped_aesthetics(p)
  if (any(c("colour", "color") %in% used)) p <- p + .ggcolor(palette)
  if ("fill" %in% used) p <- p + .ggfill(palette)
  p
}


# Helper function to use palette from ggsci package
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
.scale_color_ggsci <- function(palette = c(
                                 "npg", "aaas", "nejm", "lancet", "jama",
                                 "jco", "ucscgb", "d3", "locuszoom",
                                 "igv", "uchicago", "startrek", "tron",
                                 "futurama", "rickandmorty", "simpsons"
                               )) {
  pal <- match.arg(palette)

  functs <- list(
    npg = ggsci::scale_color_npg(),
    aaas = ggsci::scale_color_aaas(),
    nejm = ggsci::scale_color_nejm(),
    lancet = ggsci::scale_color_lancet(),
    jama = ggsci::scale_color_jama(),
    jco = ggsci::scale_color_jco(),
    ucscgb = ggsci::scale_color_ucscgb(),
    d3 = ggsci::scale_color_d3(),
    locuszoom = ggsci::scale_color_locuszoom(),
    igv = ggsci::scale_color_igv(),
    uchicago = ggsci::scale_color_uchicago(),
    startrek = ggsci::scale_color_startrek(),
    tron = ggsci::scale_color_tron(),
    futurama = ggsci::scale_color_futurama(),
    rickandmorty = ggsci::scale_color_rickandmorty(),
    simpsons = ggsci::scale_color_simpsons()
  )
  functs[[pal]]
}

.scale_fill_ggsci <- function(palette = c(
                                "npg", "aaas", "nejm", "lancet", "jama",
                                "jco", "ucscgb", "d3", "locuszoom",
                                "igv", "uchicago", "startrek", "tron",
                                "futurama", "rickandmorty", "simpsons"
                              )) {
  pal <- match.arg(palette)

  functs <- list(
    npg = ggsci::scale_fill_npg(),
    aaas = ggsci::scale_fill_aaas(),
    nejm = ggsci::scale_fill_nejm(),
    lancet = ggsci::scale_fill_lancet(),
    jama = ggsci::scale_fill_jama(),
    jco = ggsci::scale_fill_jco(),
    ucscgb = ggsci::scale_fill_ucscgb(),
    d3 = ggsci::scale_fill_d3(),
    locuszoom = ggsci::scale_fill_locuszoom(),
    igv = ggsci::scale_fill_igv(),
    uchicago = ggsci::scale_fill_uchicago(),
    startrek = ggsci::scale_fill_startrek(),
    tron = ggsci::scale_fill_tron(),
    futurama = ggsci::scale_fill_futurama(),
    rickandmorty = ggsci::scale_fill_rickandmorty(),
    simpsons = ggsci::scale_fill_simpsons()
  )
  functs[[pal]]
}

# Generate color palette from ggsci or Rcolorbrewer
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# pal could be a brewer or ggsci palette
.get_pal <- function(pal = "default", k) {
  if (pal %in% .brewerpal()) {
    .get_brewer_pal(pal, k)
  } else if (pal %in% .ggscipal()) {
    .get_ggsci_pal(pal, k)
  } else if (pal %in% c("default", "hue")) {
    hues <- seq(15, 375, length = k + 1)
    grDevices::hcl(h = hues, l = 65, c = 100, alpha = 1)[seq_len(k)]
  }
}
.get_palette <- .get_pal # alias

# Generate color palette from ggsci
# k the number of color
.get_ggsci_pal <- function(palette = c(
                             "npg", "aaas", "nejm", "lancet", "jama",
                             "jco", "ucscgb", "d3", "locuszoom",
                             "igv", "uchicago", "startrek", "tron",
                             "futurama", "rickandmorty", "simpsons"
                           ), k) {
  pal <- match.arg(palette)

  if (pal %in% c("npg", "aaas", "jco", "d3")) {
    max_k <- 10
  } else if (pal %in% c("nejm")) {
    max_k <- 8
  } else if (pal %in% c("jama", "locuszoom", "startrek", "tron")) {
    max_k <- 7
  } else if (pal %in% c("igv")) {
    max_k <- 51
  } else if (pal %in% c("lancet", "uchicago")) {
    max_k <- 9
  } else if (pal %in% c("ucscgb")) {
    max_k <- 26
  } else if (pal %in% c("futurama", "rickandmorty")) {
    max_k <- 12
  } else if (pal %in% c("simpsons")) {
    max_k <- 16
  } else {
    stop("Don't support palette name: ", pal)
  }

  functs <- list(
    npg = ggsci::pal_npg(),
    aaas = ggsci::pal_aaas(),
    nejm = ggsci::pal_nejm(),
    lancet = ggsci::pal_lancet(),
    jama = ggsci::pal_jama(),
    jco = ggsci::pal_jco(),
    ucscgb = ggsci::pal_ucscgb(),
    d3 = ggsci::pal_d3(),
    locuszoom = ggsci::pal_locuszoom(),
    igv = ggsci::pal_igv(),
    uchicago = ggsci::pal_uchicago(),
    startrek = ggsci::pal_startrek(),
    tron = ggsci::pal_tron(),
    futurama = ggsci::pal_futurama(),
    rickandmorty = ggsci::pal_rickandmorty(),
    simpsons = ggsci::pal_simpsons()
  )

  if (k <= max_k) {
    functs[[pal]](k)
  } else {
    grDevices::colorRampPalette(functs[[pal]](max_k))(k)
  }
}

# Generate a color palette from brewer
.get_brewer_pal <- function(palette, k) {
  if (!requireNamespace("RColorBrewer", quietly = TRUE)) {
    stop("RColorBrewer package needed. Please install it using install.packages('RColorBrewer').")
  }
  initial.k <- k
  k <- max(c(k, 3)) # Kshoud be at least 3
  pal <- palette[1]
  sequential <- c(
    "Blues", "BuGn", "BuPu", "GnBu", "Greens", "Greys", "Oranges",
    "OrRd", "PuBu", "PuBuGn", "PuRd", "Purples", "RdPu", "Reds", "YlGn", "YlGnBu", "YlOrBr", "YlOrRd"
  )
  divergent <- c("BrBG", "PiYG", "PRGn", "PuOr", "RdBu", "RdGy", "RdYlBu", "RdYlGn", "Spectral")

  if (pal %in% sequential) {
    max_k <- 9
  } else if (pal %in% divergent) {
    max_k <- 11
  } else if (pal %in% c("Accent", "Dark2", "Pastel2", "Set2")) {
    max_k <- 8
  } else if (pal %in% c("Pastel1", "Set1")) {
    max_k <- 9
  } else if (pal %in% c("Paired", "Set3")) {
    max_k <- 12
  } else {
    stop("Don't support palette name: ", pal)
  }

  if (k <= max_k) {
    cols <- RColorBrewer::brewer.pal(k, palette)
    if (initial.k == 2) {
      cols <- cols[c(1, 3)]
    } else if (initial.k == 1) cols <- cols[1]
    cols
  } else {
    grDevices::colorRampPalette(RColorBrewer::brewer.pal(max_k, palette))(k)
  }
}

# Set gradient colors
# cols a vector of colors
.gradient_col <- function(cols) {
  gradient_color(cols)
}

test_that(".brewerpal keeps distinct sequential palette names", {
  pals <- .brewerpal()

  expect_true("YlGnBu" %in% pals)
  expect_true("YlOrBr" %in% pals)
  expect_false("YlGnBu YlOrBr" %in% pals)
})

test_that(".get_brewer_pal supports YlOrBr", {
  skip_if_not_installed("RColorBrewer")
  cols <- .get_brewer_pal("YlOrBr", 5)
  expect_length(cols, 5)
  expect_type(cols, "character")
})

test_that(".get_pal routes YlOrBr to brewer palettes", {
  skip_if_not_installed("RColorBrewer")
  cols <- .get_pal("YlOrBr", 5)
  expect_length(cols, 5)
  expect_type(cols, "character")
})

#' @keywords internal
#'
#' @details
#'
#' General resources:
#'
#' \itemize{
#' \item \href{https://rpkgs.datanovia.com/ggpubr/}{ggpubr documentation}
#' \item \href{https://www.datanovia.com/en/blog/tag/ggpubr/}{ggpubr tutorials}
#' }
#'
#' @section P-Value Formatting:
#'
#' ggpubr provides customizable p-value formatting with predefined style presets
#' to match different journal and publication requirements. Use \code{\link{format_p_value}()}
#' for direct formatting, or set \code{p.format.style} in statistical functions.
#'
#' Available styles: \code{"default"}, \code{"apa"}, \code{"nejm"}, \code{"lancet"},
#' \code{"ama"}, \code{"graphpad"}, \code{"scientific"}.
#'
#' See \code{\link{list_p_format_styles}()} for details on each style.
#'
#' @section Package options:
#'
#' \describe{
#' \item{ggpubr.parse_aes}{logical indicating whether to parse aesthetic variable names.
#' Default is \code{TRUE}. For example, if you want \code{ggpubr} to handle non-standard column names, like \code{A-A},
#' without parsing, then set this option to \code{FALSE} using \code{options(ggpubr.parse_aes = FALSE)}.}
#' \item{ggpubr.null_device}{A function that creates an appropriate null device.
#' These include: \code{\link[cowplot:pdf_null_device]{cowplot::pdf_null_device}},
#' \code{\link[cowplot:png_null_device]{cowplot::png_null_device}},
#' \code{\link[cowplot:cairo_null_device]{cowplot::cairo_null_device}} and
#' \code{\link[cowplot:agg_null_device]{cowplot::agg_null_device}}. Default is
#' \code{\link[cowplot:pdf_null_device]{cowplot::pdf_null_device}}. This is used in
#' functions like \code{\link{as_ggplot}()}, which need to open a graphics
#' device to render ggplot objects into grid graphics objects. This function is
#' used to open a null device to avoid displaying an unnecessary blank page
#' when calling \code{\link{ggarrange}()} or \code{\link{as_ggplot}()}} }
#'
"_PACKAGE"

#' @include utilities.R ggpar.R
NULL
#' Plot convex hull of a set of points
#' @description Plot convex hull of a set of points.
#' @inheritParams ggpubr-common-params
#' @inheritParams ggplot2::layer
#' @inheritParams ggplot2::stat_ellipse
#' @seealso \code{\link{ggpar}}, \code{\link{ggscatter}}
#' @examples
#' # Load data
#' data("mtcars")
#' df <- mtcars
#' df$cyl <- as.factor(df$cyl)
#'
#' # scatter plot with convex hull
#' ggscatter(df, x = "wt", y = "mpg", color = "cyl") +
#'   stat_chull(aes(color = cyl))
#'
#' ggscatter(df, x = "wt", y = "mpg", color = "cyl") +
#'   stat_chull(aes(color = cyl, fill = cyl), alpha = 0.1, geom = "polygon")
#'
#' @export
stat_chull <- function(mapping = NULL, data = NULL, geom = "path",
                       position = "identity", na.rm = FALSE, show.legend = NA,
                       inherit.aes = TRUE, ...) {
  layer(
    stat = StatChull, data = data, mapping = mapping, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = inherit.aes,
    params = list(na.rm = na.rm, ...)
  )
}

# Convex ellipse
StatChull <- ggproto("StatChull", Stat,
  compute_group = function(data, scales) {
    data[chull(data$x, data$y), , drop = FALSE]
  },
  required_aes = c("x", "y")
)

#' @include utilities.R geom_pwc.R p_format_utils.R
NULL

#' Adjust p-values Displayed on a GGPlot
#' @description Adjust p-values produced by \code{\link{geom_pwc}()} on a ggplot.
#'  This is mainly useful when using facet, where p-values are generally
#'  computed and adjusted by panel without taking into account the other panels.
#'  In this case, one might want to adjust after the p-values of all panels together.
#' @inheritParams geom_pwc
#' @param p a ggplot
#' @param layer An integer indicating the statistical layer rank in the ggplot
#'  (in the order added to the plot).
#' @param output character. Possible values are one of \code{c("plot",
#'  "stat_test")}. Default is "plot".
#' @examples
#' # Data preparation
#' # :::::::::::::::::::::::::::::::::::::::
#' df <- ToothGrowth
#' df$dose <- as.factor(df$dose)
#' # Add a random grouping variable
#' df$group <- factor(rep(c("grp1", "grp2"), 30))
#' head(df, 3)
#'
#' # Boxplot: Two groups by panel
#' # :::::::::::::::::::::::::::::::::::::::
#' # Create a box plot
#' bxp <- ggboxplot(
#'   df,
#'   x = "supp", y = "len", fill = "#00AFBB",
#'   facet.by = "dose"
#' )
#' # Make facet and add p-values
#' bxp <- bxp + geom_pwc(method = "t_test")
#' bxp
#' # Adjust all p-values together after
#' ggadjust_pvalue(
#'   bxp,
#'   p.adjust.method = "bonferroni",
#'   label = "{p.adj.format}{p.adj.signif}", hide.ns = TRUE
#' )
#'
#'
#' # Boxplot: Three groups by panel
#' # :::::::::::::::::::::::::::::::::::::::
#' # Create a box plot
#' bxp <- ggboxplot(
#'   df,
#'   x = "dose", y = "len", fill = "#00AFBB",
#'   facet.by = "supp"
#' )
#' # Make facet and add p-values
#' bxp <- bxp + geom_pwc(method = "t_test")
#' bxp
#' # Adjust all p-values together after
#' ggadjust_pvalue(
#'   bxp,
#'   p.adjust.method = "bonferroni",
#'   label = "{p.adj.format}{p.adj.signif}"
#' )
#' @param signif.cutoffs numeric vector of p-value cutoffs in descending order
#'  for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
#'  means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
#'  Default is NULL, which uses the package defaults.
#' @param signif.symbols character vector of symbols corresponding to
#'  \code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
#'  (and "****" if \code{use.four.stars = TRUE}).
#' @param ns.symbol character string for non-significant results. Default is "ns".
#'  Use "" (empty string) to show nothing.
#' @param use.four.stars logical. If TRUE, allows four stars (****) for the most
#'  significant level. Default is FALSE.
#' @export
ggadjust_pvalue <- function(p, layer = NULL, p.adjust.method = "holm", label = "p.adj",
                            hide.ns = NULL, symnum.args = list(),
                            p.format.style = "default", p.digits = NULL,
                            p.leading.zero = NULL, p.min.threshold = NULL,
                            p.decimal.mark = NULL,
                            signif.cutoffs = NULL, signif.symbols = NULL,
                            ns.symbol = "ns", use.four.stars = FALSE,
                            output = c("plot", "stat_test")) {
  # Build symnum.args from new parameters
  symnum.args <- build_symnum_args(
    signif.cutoffs = signif.cutoffs,
    signif.symbols = signif.symbols,
    ns.symbol = ns.symbol,
    use.four.stars = use.four.stars,
    symnum.args = symnum.args
  )

  output <- match.arg(output)
  .build <- ggplot_build(p)
  .build_data <- .build$data

  # Pairwise comparison--------------------------
  # Find layer containing statistical test data
  key_columns <- c("group1", "group2", "p")
  if (is.null(layer)) {
    for (i in seq_along(.build_data)) {
      if (all(key_columns %in% colnames(.build_data[[i]]))) {
        layer <- i
        break
      }
    }
  }
  if (is.null(layer)) {
    stop("Can't find any layer containing statistical tests")
  }

  stat_test <- .build$data[[layer]]
  sy <- symnum.args
  if (all(is.na(stat_test$p))) {
    warning(
      "p-values can't be adjusted for the specified stat method.\n",
      "The result of the method doesn't contain the p column.\n",
      "Note that, tests such as tukey_hsd or games_howell_test handle p-value adjustement ",
      "internally; they only return the p.adj.",
      call. = FALSE
    )
    label <- gsub(pattern = "p\\.format(?!\\.signif)", replacement = "p.adj.format", label, perl = TRUE)
    label <- gsub(pattern = "p\\.signif", replacement = "p.adj.signif", label)
  } else {
    padjusted <- stat_test %>%
      dplyr::select(dplyr::all_of(c("PANEL", "group", "group1", "group2", "p"))) %>%
      dplyr::distinct(.keep_all = TRUE) %>%
      rstatix::adjust_pvalue(method = p.adjust.method)
    # Hide NS if hide.ns not null
    if (!is.null(hide.ns)) {
      padjusted <- rstatix::remove_ns(padjusted, col = hide.ns)
    }

    p <- p.adj <- NULL
    stat_test <- stat_test %>%
      dplyr::select(-dplyr::any_of(c("p", "p.adj", "p.format", "p.adj.format", "label"))) %>%
      dplyr::inner_join(padjusted, by = c("PANEL", "group", "group1", "group2"))

    # Format p-values using unified p-format implementation for all styles
    stat_test <- stat_test %>%
      dplyr::mutate(
        p.format = format_p_value(p,
          style = p.format.style,
          digits = p.digits, leading.zero = p.leading.zero,
          min.threshold = p.min.threshold,
          decimal.mark = p.decimal.mark
        ),
        p.adj.format = format_p_value(p.adj,
          style = p.format.style,
          digits = p.digits, leading.zero = p.leading.zero,
          min.threshold = p.min.threshold,
          decimal.mark = p.decimal.mark
        )
      )

    stat_test <- stat_test %>%
      rstatix::add_significance(p.col = "p", cutpoints = sy$cutpoints, symbols = sy$symbols) %>%
      rstatix::add_significance(p.col = "p.adj", cutpoints = sy$cutpoints, symbols = sy$symbols) %>%
      add_stat_label(label = label)
    .build$data[[layer]] <- stat_test
  }
  if (output == "stat_test") {
    return(stat_test)
  }
  as_ggplot(ggplot_gtable(.build))
}

#' Convert Character Coordinates into Normalized Parent Coordinates (NPC)
#'
#' Convert character coordinates to npc units and shift positions to avoid
#' overlaps when grouping is active. If numeric validate npc values.
#'
#' @param value numeric (in [0-1]) or character vector of coordinates. If
#'   character, should be one of \code{c('right', 'left', 'bottom', 'top',
#'   'center', 'centre', 'middle')}.
#' @param group integer ggplot's group id. Used to shift coordinates to avoid
#'   overlaps.
#' @param step numeric value in [0-1]. The step size for shifting coordinates
#'   in npc units. Considered as horizontal step for x-axis and vertical step
#'   for y-axis. For y-axis, the step value can be negative to reverse the order of groups.
#' @param margin.npc numeric [0-1] The margin added towards the nearest
#'   plotting area edge when converting character coordinates into npc.
#' @param axis the concerned axis . Should be one of \code{c("xy", "x", "y")}.
#'
#' @return A numeric vector with values in the range [0-1] representing npc
#'   coordinates.
#' @seealso \code{\link{npc_to_data_coord}}, \code{\link{get_coord}}.
#' @details the \code{as_npc()} function is an adaptation from
#'   \code{ggpmisc::compute_npc()}.
#'
#' @examples
#' as_npc(c("left", "right"))
#' as_npc(c("top", "right"))
#'
#' @describeIn as_npc converts x or y coordinate values into npc. Input values
#'   should be numeric or one of the following values \code{c('right', 'left',
#'   'bottom', 'top', 'center', 'centre', 'middle')}.
#' @export
as_npc <- function(value, group = 1L, step = 0.1, margin.npc = 0.05, axis = c("xy", "x", "y")) {
  group <- abs(group)
  axis <- match.arg(axis)
  if (is.factor(value)) {
    value <- as.character(value)
  }
  assertthat_coord_is_valid(value, axis = axis)
  if (is.character(value)) {
    map <- c(
      right = 1 - margin.npc,
      left = 0 + margin.npc,
      top = 1 - margin.npc,
      bottom = 0 + margin.npc,
      centre = 0.5,
      center = 0.5,
      middle = 0.5,
      NA_real_
    )
    value <- unname(map[value])
  }
  if (any(group > 1L) & step != 0) {
    value <- value + (group - 1L) * step * ifelse(value < 0.5, 1, -1)
  }
  # value <- ifelse(value > 1, 1, value)
  # value <- ifelse(value < 0, 0, value)
  value
}

#' @describeIn as_npc converts x coordinate values into npc. Input values should
#'   be numeric or one of the following values \code{c('right', 'left',
#'   'center', 'centre', 'middle')}. Wrapper around \code{as_npc(axis = "x")}.
#' @export
as_npcx <- function(value, group = 1L, step = 0.1, margin.npc = 0.05) {
  as_npc(value, group = group, step = step, margin.npc = margin.npc, axis = "x")
}

#' @describeIn as_npc converts y coordinate values into npc. Input values should
#'   be numeric or one of the following values \code{c( 'bottom', 'top',
#'   'center', 'centre', 'middle')}. Wrapper around \code{as_npc(axis = "y")}.
#' @export
as_npcy <- function(value, group = 1L, step = 0.1, margin.npc = 0.05) {
  as_npc(value, group = group, step = step, margin.npc = margin.npc, axis = "y")
}


# Check NPC coord-----------------------------------------
# coord: should be between 0 and 1 or a character
# axis: should be "x" or "y"
assertthat_coord_is_valid <- function(.coord, axis = c("x", "y", "xy")) {
  axis <- match.arg(axis)
  allowed.values <- switch(axis,
    x = c("right", "left", "center", "centre", "middle"),
    y = c("bottom", "top", "center", "centre", "middle"),
    xy = c("right", "left", "bottom", "top", "center", "centre", "middle")
  )
  .message <- paste0(
    "'*.npc coord for ", axis, " axis should be either a numeric value in [0-1] ",
    "or a character strings including one of: ",
    paste(allowed.values, collapse = ", ")
  )
  if (is.numeric(.coord)) {
    if (any(.coord < 0 | .coord > 1)) {
      stop(.message, call. = FALSE)
    }
  } else if (is.character(.coord)) {
    if (!all(.coord %in% allowed.values)) {
      stop(.message, call. = FALSE)
    }
  }
}

context("test-as_npc")

test_that("as_npc returns the same value as the input when the input is.numeric", {
  expect_equal(as_npc(0.5), 0.5)
})


test_that("as_npc works correctly for character coordinates", {
  res <- as_npc(c("right", "left", "bottom", "top", "center", "centre", "middle"))
  expect_equal(res, c(0.95, 0.05, 0.05, 0.95, 0.5, 0.5, 0.5))
})

#' Common parameters for ggpubr stat and geom functions
#'
#' @param inherit.aes If \code{FALSE} (the default for most ggpubr functions),
#'   overrides the default aesthetics, rather than combining with them. This is
#'   most useful for helper functions that define both data and aesthetics and
#'   shouldn't inherit behaviour from the default plot specification. Set to
#'   \code{TRUE} to inherit aesthetics from the parent ggplot layer.
#'
#' @name ggpubr-common-params
#' @keywords internal
NULL

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bgcolor.R
\name{bgcolor}
\alias{bgcolor}
\title{Change ggplot Panel Background Color}
\usage{
bgcolor(color)
}
\arguments{
\item{color}{background color.}
}
\description{
Change ggplot panel background color.
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
p <- ggboxplot(df, x = "dose", y = "len")
p

# Change panel background color
p +
  bgcolor("#BFD5E3") +
  border("#BFD5E3")
}
\seealso{
\link{border}().
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggexport.R
\name{ggexport}
\alias{ggexport}
\title{Export ggplots}
\usage{
ggexport(
  ...,
  plotlist = NULL,
  filename = NULL,
  ncol = NULL,
  nrow = NULL,
  width = 480,
  height = 480,
  pointsize = 12,
  res = NA,
  verbose = TRUE
)
}
\arguments{
\item{...}{list of plots to be arranged into the grid. The plots can be either
ggplot2 plot objects, arbitrary gtables or an object of class
\code{\link{ggarrange}}.}

\item{plotlist}{(optional) list of plots to display.}

\item{filename}{File name to create on disk.}

\item{ncol}{(optional) number of columns in the plot grid.}

\item{nrow}{(optional) number of rows in the plot grid.}

\item{width, height}{plot width and height, respectively (example, width = 800,
height = 800). Applied only to raster plots: "png", "jpeg", "jpg", "bmp" and
"tiff".}

\item{pointsize}{the default pointsize of plotted text (example, pointsize =
8). Used only for raster plots.}

\item{res}{the resolution in ppi (example, res = 250). Used only for raster
plots.}

\item{verbose}{logical. If TRUE, show message.}
}
\description{
Export ggplots
}
\examples{
\dontrun{
require("magrittr")
# Load data
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Box plot
bxp <- ggboxplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
# Dot plot
dp <- ggdotplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
# Density plot
dens <- ggdensity(df, x = "len", fill = "dose", palette = "jco")

# Export to pdf
ggarrange(bxp, dp, dens, ncol = 2) \%>\%
  ggexport(filename = "test.pdf")

# Export to png
ggarrange(bxp, dp, dens, ncol = 2) \%>\%
  ggexport(filename = "test.png")
}

}
\author{
Laszlo Erdey <erdey.laszlo@econ.unideb.hu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/background_image.R
\name{background_image}
\alias{background_image}
\title{Add Background Image to ggplot2}
\usage{
background_image(raster.img)
}
\arguments{
\item{raster.img}{raster object to display, as returned by the function
\code{readPNG()}[in \code{png} package] and \code{readJPEG()} [in \code{jpeg} package].}
}
\description{
Add background image to ggplot2.
}
\examples{
\dontrun{
install.packages("png")

# Import the image
img.file <- system.file(file.path("images", "background-image.png"),
  package = "ggpubr"
)
img <- png::readPNG(img.file)

# Plot with background image
ggplot(iris, aes(Species, Sepal.Length)) +
  background_image(img) +
  geom_boxplot(aes(fill = Species), color = "white") +
  fill_palette("jco")
}

}
\author{
Laszlo Erdey <erdey.laszlo@econ.unideb.hu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggparagraph.R
\name{ggparagraph}
\alias{ggparagraph}
\alias{drawDetails.splitText}
\title{Draw a Paragraph of Text}
\usage{
ggparagraph(
  text,
  color = NULL,
  size = NULL,
  face = NULL,
  family = NULL,
  lineheight = NULL
)

\method{drawDetails}{splitText}(x, recording)
}
\arguments{
\item{text}{the text to plot.}

\item{color}{font color, example: color = "black"}

\item{size}{font size, example: size = 12}

\item{face}{font face. Allowed values are one of "plain", "italic", "bold",
"bold.italic".}

\item{family}{font family}

\item{lineheight}{Line height, example: lineheight = 2.}

\item{x}{a grid grob}

\item{recording}{a logical value indicating whether a grob is being added to
the display list or redrawn from the display list.}
}
\description{
Draw a paragraph o text. Splits a long text into multiple lines
 (by inserting line breaks) so that the output will fit within the current
 viewport.
}
\examples{
# Density plot
density.p <- ggdensity(iris,
  x = "Sepal.Length",
  fill = "Species", palette = "jco"
)

# Text plot
text <- paste("iris data set gives the measurements in cm",
  "of the variables sepal length and width",
  "and petal length and width, respectively,",
  "for 50 flowers from each of 3 species of iris.",
  "The species are Iris setosa, versicolor, and virginica.",
  sep = " "
)
text.p <- ggparagraph(text, face = "italic", size = 12)

# Arrange the plots on the same page
ggarrange(density.p, text.p,
  ncol = 1, nrow = 2,
  heights = c(1, 0.3)
)

}
\author{
Laszlo Erdey <erdey.laszlo@econ.unideb.hu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_palette.R
\name{get_palette}
\alias{get_palette}
\title{Generate Color Palettes}
\usage{
get_palette(palette = "default", k)
}
\arguments{
\item{palette}{Color palette. Allowed values include: \itemize{ \item
\strong{Grey color palettes}: "grey" or "gray"; \item \strong{RColorBrewer
palettes}, see \code{\link[RColorBrewer:ColorBrewer]{brewer.pal}} and details section.
Examples of palette names include: "RdBu", "Blues", "Dark2", "Set2", ...;
\item \strong{Custom color palettes}. For example, palette = c("#00AFBB",
"#E7B800", "#FC4E07"); \item \strong{ggsci scientific journal palettes},
e.g.: "npg", "aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". }}

\item{k}{the number of colors to generate.}
}
\value{
Returns a vector of color palettes.
}
\description{
Generate a palette of k colors from ggsci palettes, RColorBrewer
 palettes and custom color palettes. Useful to extend RColorBrewer and ggsci to support more colors.
}
\details{
\strong{RColorBrewer palettes}: To display all available color
 palettes, type this in R:RColorBrewer::display.brewer.all(). Color palette
 names include:

 \itemize{ \item \strong{Sequential palettes}, suited to ordered data that
 progress from low to high. Palette names include: Blues BuGn BuPu GnBu
 Greens Greys Oranges OrRd PuBu PuBuGn PuRd Purples RdPu Reds YlGn YlGnBu
 YlOrBr YlOrRd. \item \strong{Diverging palettes}:Gradient colors. Names
 include: BrBG PiYG PRGn PuOr RdBu RdGy RdYlBu RdYlGn Spectral. \item
 \strong{Qualitative palettes}:  Best suited to representing nominal or
 categorical data. Names include: Accent, Dark2, Paired, Pastel1, Pastel2,
 Set1, Set2, Set3.
 }
}
\examples{
data("iris")
iris$Species2 <- factor(rep(c(1:10), each = 15))

# Generate a gradient of 10 colors
ggscatter(iris,
  x = "Sepal.Length", y = "Petal.Length",
  color = "Species2",
  palette = get_palette(c("#00AFBB", "#E7B800", "#FC4E07"), 10)
)

# Scatter plot with default color palette
ggscatter(iris,
  x = "Sepal.Length", y = "Petal.Length",
  color = "Species"
)

# RColorBrewer color palettes
ggscatter(iris,
  x = "Sepal.Length", y = "Petal.Length",
  color = "Species", palette = get_palette("Dark2", 3)
)

# ggsci color palettes
ggscatter(iris,
  x = "Sepal.Length", y = "Petal.Length",
  color = "Species", palette = get_palette("npg", 3)
)

# Custom color palette
ggscatter(iris,
  x = "Sepal.Length", y = "Petal.Length",
  color = "Species",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)

# Or use this
ggscatter(iris,
  x = "Sepal.Length", y = "Petal.Length",
  color = "Species",
  palette = get_palette(c("#00AFBB", "#FC4E07"), 3)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_legend.R
\name{get_legend}
\alias{get_legend}
\title{Extract Legends from a ggplot object}
\usage{
get_legend(p, position = NULL)
}
\arguments{
\item{p}{an object of class ggplot or a list of ggplots. If p is a list, only the first legend is returned.}

\item{position}{character specifying legend position. Allowed values are one of
c("top", "bottom", "left", "right", "none"). To remove the legend use
legend = "none".}
}
\value{
an object of class gtable.
}
\description{
Extract the legend labels from a ggplot object.
}
\examples{
# Create a scatter plot
p <- ggscatter(iris,
  x = "Sepal.Length", y = "Sepal.Width",
  color = "Species", palette = "jco",
  ggtheme = theme_minimal()
)
p

# Extract the legend. Returns a gtable
leg <- get_legend(p)

# Convert to a ggplot and print
as_ggplot(leg)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_chull.R
\name{stat_chull}
\alias{stat_chull}
\title{Plot convex hull of a set of points}
\usage{
stat_chull(
  mapping = NULL,
  data = NULL,
  geom = "path",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}} or
\code{\link[ggplot2:aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{geom}{The geometric object to use display the data}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{na.rm}{If \code{FALSE}, the default, missing values are removed with
a warning. If \code{TRUE}, missing values are silently removed.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:borders]{borders()}}.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
Plot convex hull of a set of points.
}
\examples{
# Load data
data("mtcars")
df <- mtcars
df$cyl <- as.factor(df$cyl)

# scatter plot with convex hull
ggscatter(df, x = "wt", y = "mpg", color = "cyl")+
 stat_chull(aes(color = cyl))

ggscatter(df, x = "wt", y = "mpg", color = "cyl")+
 stat_chull(aes(color = cyl, fill = cyl), alpha = 0.1, geom = "polygon")

}
\seealso{
\code{\link{ggpar}}, \code{\link{ggscatter}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpubr_options.R
\name{ggpubr_options}
\alias{ggpubr_options}
\title{Global Options for ggpubr}
\usage{
ggpubr_options()
}
\description{
Displays allowed global options in ggpubr.
}
\examples{

ggpubr_options()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpaired.R
\name{ggpaired}
\alias{ggpaired}
\title{Plot Paired Data}
\usage{
ggpaired(
  data,
  cond1,
  cond2,
  x = NULL,
  y = NULL,
  id = NULL,
  color = "black",
  fill = "white",
  palette = NULL,
  width = 0.5,
  point.size = 1.2,
  line.size = 0.5,
  line.color = "black",
  linetype = "solid",
  title = NULL,
  xlab = "Condition",
  ylab = "Value",
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  ggtheme = theme_pubr(),
  jitter = 0,
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{cond1}{variable name corresponding to the first condition.}

\item{cond2}{variable name corresponding to the second condition.}

\item{x, y}{x and y variables, where x is a grouping variable and y contains
values for each group. Considered only when \code{cond1} and \code{cond2}
are missing.}

\item{id}{variable name corresponding to paired samples' id. Used to connect
paired points with lines.}

\item{color}{points and box plot colors. To color by conditions, use color =
"condition".}

\item{fill}{box plot fill color. To change fill color by conditions, use fill
= "condition".}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{width}{box plot width.}

\item{point.size, line.size}{point and line size, respectively.}

\item{line.color}{line color.}

\item{linetype}{line type.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variables values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{jitter}{numeric value (default 0, no jitter) giving the amount of
horizontal jitter added to the paired points to reduce overlap. Points are
nudged sideways within \code{[-jitter, jitter]}; each subject (\code{id})
gets a single offset so its two points move together and the connecting
line stays intact. Only the horizontal positions change (the values are
never moved). Typical values are small relative to the box \code{width}
(e.g. \code{jitter = 0.05} to \code{0.1}). \code{jitter = 0} leaves the plot
unchanged.}

\item{...}{other arguments to be passed to be passed to \link{ggpar}().}
}
\description{
Plot paired data.
}
\examples{

# Example 1
# ::::::::::::::::::::::::::::::::::::::::::
before <- c(200.1, 190.9, 192.7, 213, 241.4, 196.9, 172.2, 185.5, 205.2, 193.7)
after <- c(392.9, 393.2, 345.1, 393, 434, 427.9, 422, 383.9, 392.3, 352.2)

d <- data.frame(before = before, after = after)
ggpaired(d,
  cond1 = "before", cond2 = "after",
  fill = "condition", palette = "jco"
)

# Example 2
# ::::::::::::::::::::::::::::::::::::::::::
ggpaired(ToothGrowth,
  x = "supp", y = "len",
  color = "supp", line.color = "gray", line.size = 0.4,
  palette = "npg"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\docType{data}
\name{p_format_styles}
\alias{p_format_styles}
\alias{.p_format_styles}
\title{P-Value Formatting Styles}
\format{
An object of class \code{list} of length 7.
}
\usage{
.p_format_styles
}
\description{
Predefined p-value formatting styles based on major scientific
  publication standards and citation styles.
}
\details{
The following styles are available:
\itemize{
  \item \code{default}: Current behavior with scientific notation (backward compatible)
  \item \code{apa}: APA Style - no leading zero, 3 decimals, threshold at 0.001
  \item \code{nejm}: New England Journal of Medicine - leading zero, 3 decimals, threshold at 0.001
  \item \code{lancet}: The Lancet - leading zero, 4 decimals, threshold at 0.0001
  \item \code{ama}: American Medical Association - no leading zero, 3 decimals, threshold at 0.001
  \item \code{graphpad}: GraphPad Prism - leading zero, 4 decimals, threshold at 0.0001
  \item \code{scientific}: Scientific notation for genomics/GWAS studies
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/axis_scale.R
\name{axis_scale}
\alias{axis_scale}
\alias{xscale}
\alias{yscale}
\title{Change Axis Scale: log2, log10 and more}
\usage{
xscale(.scale, .format = FALSE)

yscale(.scale, .format = FALSE)
}
\arguments{
\item{.scale}{axis scale. Allowed values are one of c("none", "log2", "log10",
"sqrt", "percent", "dollar", "scientific"); e.g.: .scale="log2".}

\item{.format}{logical value. If TRUE, axis tick mark labels will be formatted
when .scale  = "log2" or "log10".}
}
\description{
Change axis scale.
\itemize{
\item \code{xscale}: change x axis scale.
\item \code{yscale}: change y axis scale.
}
}
\examples{
# Basic scatter plots
data(cars)
p <- ggscatter(cars, x = "speed", y = "dist")
p

# Set log scale
p + yscale("log2", .format = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_central_tendency.R
\name{stat_central_tendency}
\alias{stat_central_tendency}
\title{Add Central Tendency Measures to a GGPLot}
\usage{
stat_central_tendency(
  mapping = NULL,
  data = NULL,
  geom = c("line", "point"),
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  type = c("mean", "median", "mode"),
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}} or
\code{\link[ggplot2:aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{geom}{The geometric object to use display the data}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{na.rm}{If FALSE (the default), removes missing values with a warning.
If TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:borders]{borders()}}.}

\item{type}{the type of central tendency measure to be used. Possible values
include: \code{"mean", "median", "mode"}.}

\item{...}{other arguments to pass to \code{\link[ggplot2:geom_path]{geom_line}}.}
}
\description{
Add central tendency measures (mean, median, mode) to density
  and histogram plots created using ggplots.

  Note that, normally, the mode is used for categorical data where we wish to
  know which is the most common category. Therefore, we can have have two or
  more values that share the highest frequency. This might be problematic for
  continuous variable.

  For continuous variable, we can consider using mean or median as the
  measures of the central tendency.
}
\examples{
# Simple density plot
data("mtcars")
ggdensity(mtcars, x = "mpg", fill = "red") +
  scale_x_continuous(limits = c(-1, 50)) +
  stat_central_tendency(type = "mean", linetype = "dashed")

# Color by groups
data(iris)
ggdensity(iris, "Sepal.Length", color = "Species") +
  stat_central_tendency(aes(color = Species), type = "median", linetype = 2)

# Use geom = "point" for central tendency
data(iris)
ggdensity(iris, "Sepal.Length", color = "Species") +
  stat_central_tendency(
     aes(color = Species), type = "median",
     geom = "point", size = 4
     )

# Facet
ggdensity(iris, "Sepal.Length", facet.by = "Species") +
  stat_central_tendency(type = "mean", color = "red", linetype = 2) +
  stat_central_tendency(type = "median", color = "blue", linetype = 2)

}
\seealso{
\code{\link{ggdensity}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diff_express.R
\docType{data}
\name{diff_express}
\alias{diff_express}
\title{Differential gene expression analysis results}
\format{
A data frame with 36028 rows and 5 columns. \describe{
  \item{\code{name}}{gene names} \item{\code{baseMean}}{mean expression
  signal across all samples} \item{\code{log2FoldChange}}{log2 fold change}
  \item{\code{padj}}{Adjusted p-value}\item{\code{detection_call}}{a numeric
  vector specifying whether the genes is expressed (value = 1) or not (value = 0).}}
}
\usage{
data("diff_express")
}
\description{
Differential gene expression analysis results obtained from
  comparing the RNAseq data of two different cell populations using DESeq2
}
\examples{
data(diff_express)

# Default plot
ggmaplot(diff_express,
  main = expression("Group 1" \%->\% "Group 2"),
  fdr = 0.05, fc = 2, size = 0.4,
  palette = c("#B31B21", "#1465AC", "darkgray"),
  genenames = as.vector(diff_express$name),
  legend = "top", top = 20,
  font.label = c("bold", 11),
  font.legend = "bold",
  font.main = "bold",
  ggtheme = ggplot2::theme_minimal()
)

# Add rectangle around labesl
ggmaplot(diff_express,
  main = expression("Group 1" \%->\% "Group 2"),
  fdr = 0.05, fc = 2, size = 0.4,
  palette = c("#B31B21", "#1465AC", "darkgray"),
  genenames = as.vector(diff_express$name),
  legend = "top", top = 20,
  font.label = c("bold", 11), label.rectangle = TRUE,
  font.legend = "bold",
  font.main = "bold",
  ggtheme = ggplot2::theme_minimal()
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_summary.R
\name{add_summary}
\alias{add_summary}
\alias{mean_se_}
\alias{mean_sd}
\alias{mean_ci}
\alias{mean_range}
\alias{median_iqr}
\alias{median_hilow_}
\alias{median_q1q3}
\alias{median_mad}
\alias{median_range}
\title{Add Summary Statistics onto a ggplot.}
\usage{
add_summary(
  p,
  fun = "mean_se",
  error.plot = "pointrange",
  color = "black",
  fill = "white",
  group = 1,
  width = NULL,
  shape = 19,
  size = 1,
  linewidth = size,
  linetype = 1,
  show.legend = NA,
  ci = 0.95,
  data = NULL,
  position = position_dodge(0.8)
)

mean_se_(x, error.limit = "both")

mean_sd(x, error.limit = "both")

mean_ci(x, ci = 0.95, error.limit = "both")

mean_range(x, error.limit = "both")

median_iqr(x, error.limit = "both")

median_hilow_(x, ci = 0.95, error.limit = "both")

median_q1q3(x, error.limit = "both")

median_mad(x, error.limit = "both")

median_range(x, error.limit = "both")
}
\arguments{
\item{p}{a ggplot on which you want to add summary statistics.}

\item{fun}{a function that is given the complete data and should return a data
frame with variables ymin, y, and ymax. Allowed values are one of: "mean",
"mean_se", "mean_sd", "mean_ci", "mean_range", "median", "median_iqr",
"median_hilow", "median_q1q3", "median_mad", "median_range".}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
\code{c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange")}. Default value is "pointrange".}

\item{color}{point or outline color.}

\item{fill}{fill color. Used only when \code{error.plot = "crossbar"}.}

\item{group}{grouping variable. Allowed values are 1 (for one group) or a
character vector specifying the name of the grouping variable. Used only for
adding statistical summary per group.}

\item{width}{numeric value between 0 and 1 specifying bar or box width.
Example width = 0.8. Used only when \code{error.plot} is one of
c("crossbar", "errorbar").}

\item{shape}{point shape. Allowed values can be displayed using the function
\code{\link{show_point_shapes}()}.}

\item{size}{numeric value in [0-1] specifying point and line size.}

\item{linewidth}{numeric value in [0-1] specifying line width.}

\item{linetype}{line type.}

\item{show.legend}{logical. Should this layer be included in the legends? NA,
the default, includes if any aesthetics are mapped. \code{FALSE} never includes,
and TRUE always includes. It can also be a named logical vector to finely
select the aesthetics to display.}

\item{ci}{the percent range of the confidence interval (default is 0.95).}

\item{data}{a \code{data.frame} to be displayed. If \code{NULL}, the default,
the data is inherited from the plot data as specified in the call to
\link[ggplot2]{ggplot}.}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function. Used to adjust position for multiple
groups.}

\item{x}{a numeric vector.}

\item{error.limit}{allowed values are one of ("both", "lower", "upper",
"none") specifying whether to plot the lower and/or the upper limits of
error interval.}
}
\description{
add summary statistics onto a ggplot.
}
\section{Functions}{
\itemize{
\item \code{add_summary()}: add summary statistics onto a ggplot.

\item \code{mean_se_()}: returns the \code{mean} and the error limits defined by the
\code{standard error}. We used the name \code{mean_se_}() to avoid masking \code{\link[ggplot2]{mean_se}}().

\item \code{mean_sd()}: returns the \code{mean} and the error limits defined by the
\code{standard deviation}.

\item \code{mean_ci()}: returns the \code{mean} and the error limits defined by the
\code{confidence interval}.

\item \code{mean_range()}: returns the \code{mean} and the error limits defined by the
\code{range = max - min}.

\item \code{median_iqr()}: returns the \code{median} and the error limits
defined by the \code{interquartile range}.

\item \code{median_hilow_()}: computes the sample median and a selected pair of
outer quantiles having equal tail areas. This function is a reformatted
version of \code{Hmisc::smedian.hilow()}. The confidence limits are computed
as follows: \code{lower.limits = (1-ci)/2} percentiles; \code{upper.limits =
(1+ci)/2} percentiles. By default (\code{ci = 0.95}), the 2.5th and the
97.5th percentiles are used as the lower and the upper confidence limits,
respectively. If you want to use the 25th and the 75th percentiles as the
confidence limits, then specify \code{ci = 0.5} or use the function
\code{median_q1q3()}.

\item \code{median_q1q3()}: computes the sample median and, the 25th and 75th
percentiles. Wrapper around the function \code{median_hilow_()} using
\code{ci = 0.5}.

\item \code{median_mad()}: returns the \code{median} and the error limits
defined by the \code{median absolute deviation}.

\item \code{median_range()}: returns the \code{median} and the error limits
defined by the \code{range = max - min}.

}}
\examples{

# Basic violin plot
p <- ggviolin(ToothGrowth, x = "dose", y = "len", add = "none")
p

# Add mean_sd
add_summary(p, "mean_sd")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggsummarystats.R
\name{ggsummarytable}
\alias{ggsummarytable}
\alias{ggsummarystats}
\alias{print.ggsummarystats}
\alias{print.ggsummarystats_list}
\title{GGPlot with Summary Stats Table Under the Plot}
\usage{
ggsummarytable(
  data,
  x,
  y,
  digits = 0,
  size = 3,
  color = "black",
  palette = NULL,
  facet.by = NULL,
  labeller = "label_value",
  position = "identity",
  angle = 0,
  ggtheme = theme_pubr(),
  ...
)

ggsummarystats(
  data,
  x,
  y,
  summaries = c("n", "median", "iqr"),
  ggfunc = ggboxplot,
  color = "black",
  fill = "white",
  palette = NULL,
  facet.by = NULL,
  free.panels = FALSE,
  labeller = "label_value",
  heights = c(0.8, 0.2),
  digits = 0,
  table.font.size = 3,
  ggtheme = theme_pubr(),
  ...
)

\method{print}{ggsummarystats}(x, heights = c(0.8, 0.2), ...)

\method{print}{ggsummarystats_list}(x, heights = c(0.8, 0.2), legend = NULL, ...)
}
\arguments{
\item{data}{a data frame}

\item{x}{a list of \code{ggsummarystats}.}

\item{y}{character vector containing one or more variables to plot}

\item{digits}{integer indicating the number of decimal places (round) to be
used.}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{color}{outline color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{labeller}{Character vector. An alternative to the argument
\code{short.panel.labs}. Possible values are one of "label_both" (panel
labelled by both grouping variable names and levels) and "label_value"
(panel labelled with only grouping levels).}

\item{position}{Position adjustment, either as a string, or the result of a
call to a position adjustment function.}

\item{angle}{numeric value specifying the rotation angle (in degrees) of the
summary table text. Default is 0.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments passed to the function \code{\link{ggpar}()},
\code{\link{facet}()} or \code{\link{ggarrange}()} when printing the plot.}

\item{summaries}{summary stats to display in the table. Possible values are
those returned by the function \code{\link[rstatix]{get_summary_stats}()},
including: \code{"n", "min", "max",  "median",  "q1", "q2", "q3", "mad",
"mean", "sd", "se", "ci"}.}

\item{ggfunc}{a ggpubr function, including: ggboxplot, ggviolin, ggdotplot,
ggbarplot, ggline, etc. Can be any other ggplot function that accepts the
following arguments \code{data, x, color, fill, palette, ggtheme,
facet.by}.}

\item{fill}{fill color.}

\item{free.panels}{logical. If TRUE, create free plot panels when the
argument \code{facet.by} is specified.}

\item{heights}{a numeric vector of length 2, specifying the heights of the
main and the summary table, respectively.}

\item{table.font.size}{the summary table font size.}

\item{legend}{character specifying legend position. Allowed values are one of
c("top", "bottom", "left", "right", "none"). To remove the legend use
legend = "none".}
}
\description{
Create a ggplot with summary stats (n, median, mean, iqr) table
  under the plot. Read more: \href{https://www.datanovia.com/en/blog/how-to-create-a-beautiful-plots-in-r-with-summary-statistics-labels/}{How to Create a Beautiful Plots in R with Summary Statistics Labels}.
}
\section{Functions}{
\itemize{
\item \code{ggsummarytable()}: Create a table of summary stats

\item \code{ggsummarystats()}: Create a ggplot with a summary stat table under the plot.

}}
\examples{
# Data preparation
# ::::::::::::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Add random QC column
set.seed(123)
qc <- rep(c("pass", "fail"), 30)
df$qc <- as.factor(sample(qc, 60))
# Inspect the data
head(df)


# Basic summary stats
# ::::::::::::::::::::::::::::::::::::::::::::::::
# Compute summary statistics
summary.stats <- df \%>\%
  group_by(dose) \%>\%
  get_summary_stats(type = "common")
summary.stats

# Visualize summary table
ggsummarytable(
  summary.stats,
  x = "dose", y = c("n", "median", "iqr"),
  ggtheme = theme_bw()
)


# Create plots with summary table under the plot
# ::::::::::::::::::::::::::::::::::::::::::::::::
# Basic plot
ggsummarystats(
  df,
  x = "dose", y = "len",
  ggfunc = ggboxplot, add = "jitter"
)

# Color by groups
ggsummarystats(
  df,
  x = "dose", y = "len",
  ggfunc = ggboxplot, add = "jitter",
  color = "dose", palette = "npg"
)

# Create a barplot
ggsummarystats(
  df,
  x = "dose", y = "len",
  ggfunc = ggbarplot, add = c("jitter", "median_iqr"),
  color = "dose", palette = "npg"
)

# Facet
# ::::::::::::::::::::::::::::::::::::::::::::::::
# Specify free.panels = TRUE for free panels
ggsummarystats(
  df,
  x = "dose", y = "len",
  ggfunc = ggboxplot, add = "jitter",
  color = "dose", palette = "npg",
  facet.by = c("supp", "qc"),
  labeller = "label_both"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggqqplot.R
\name{ggqqplot}
\alias{ggqqplot}
\title{QQ Plots}
\usage{
ggqqplot(
  data,
  x,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  palette = NULL,
  size = NULL,
  shape = NULL,
  add = c("qqline", "none"),
  add.params = list(linetype = "solid"),
  conf.int = TRUE,
  conf.int.level = 0.95,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{variable to be drawn.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{point color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{point size.}

\item{shape}{point shape.}

\item{add}{character vector. Allowed values are one of "none" and "qqline"
(for adding qqline).}

\item{add.params}{parameters (color, size,  linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{conf.int}{logical value. If TRUE, confidence interval is added.}

\item{conf.int.level}{the confidence level. Default value is 0.95.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to \code{\link{ggpar}}.}
}
\description{
Quantile-Quantile plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Create some data format
set.seed(1234)
wdata <- data.frame(
  sex = factor(rep(c("F", "M"), each = 200)),
  weight = c(rnorm(200, 55), rnorm(200, 58))
)

head(wdata, 4)

# Basic QQ plot
ggqqplot(wdata, x = "weight")

# Change colors and shape by groups ("sex")
# Use custom palette
ggqqplot(wdata,
  x = "weight",
  color = "sex", palette = c("#00AFBB", "#E7B800")
)

}
\seealso{
\code{\link{ggpar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_transparent.R
\name{theme_transparent}
\alias{theme_transparent}
\title{Create a ggplot with Transparent Background}
\usage{
theme_transparent(base_size = 12, base_family = "")
}
\arguments{
\item{base_size}{base font size}

\item{base_family}{base font family}
}
\description{
Create a ggplot with transparent background.
}
\examples{
# Create a scatter plot
sp <- ggscatter(iris,
  x = "Sepal.Length", y = "Sepal.Width",
  color = "Species", palette = "jco",
  size = 3, alpha = 0.6
)
sp

# Transparent theme
sp + theme_transparent()

}
\seealso{
\code{\link{theme_pubr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggmaplot.R
\name{ggmaplot}
\alias{ggmaplot}
\title{MA-plot from means and log fold changes}
\usage{
ggmaplot(
  data,
  fdr = 0.05,
  fc = 1.5,
  genenames = NULL,
  detection_call = NULL,
  size = NULL,
  alpha = 1,
  seed = 42,
  font.label = c(12, "plain", "black"),
  label.rectangle = FALSE,
  palette = c("#B31B21", "#1465AC", "darkgray"),
  top = 15,
  select.top.method = c("padj", "fc"),
  label.select = NULL,
  facet.by = NULL,
  main = NULL,
  xlab = "Log2 mean expression",
  ylab = "Log2 fold change",
  line.color = "black",
  ggtheme = theme_classic(),
  ...
)
}
\arguments{
\item{data}{an object of class DESeqResults, get_diff, DE_Results, matrix or
 data frame containing the columns baseMean (or baseMeanLog2),
 log2FoldChange, and padj. Rows are genes.

 Two possible formats are accepted for the input data: \itemize{ \item 1/
 \code{baseMean | log2FoldChange | padj}. This is a typical output from
 DESeq2 pipeline. Here, we'll use log2(baseMean) as the x-axis variable.
 \item 2/ \code{baseMeanLog2 | log2FoldChange | padj}. Here, baseMeanLog2 is
 assumed to be the mean of logged values; so we'll use it as the x-axis
 variable without any transformation. This is the real A in MA plot. In other
 words, it is the average of two log-scales values: \code{A = (log2(x) +
 log2(y))/2 = log2(xy)*1/2} }

 Terminology:

 \itemize{ \item baseMean: the mean expression of genes in the two groups.
 \item log2FoldChange: the log2 fold changes of group 2 compared to group 1
 \item padj: the adjusted p-value of the used statistical test. }}

\item{fdr}{Accepted false discovery rate for considering genes as
differentially expressed.}

\item{fc}{the fold change threshold. Only genes with a fold change >= fc and
padj <= fdr are considered as significantly differentially expressed.}

\item{genenames}{a character vector of length nrow(data) specifying gene names
corresponding to each row. Used for point labels.}

\item{detection_call}{a numeric vector with length = nrow(data), specifying if
the genes is expressed (value = 1) or not (value = 0). For example
detection_call = c(1, 1, 0, 1, 0, 1). Default is NULL. If detection_call
column is available in data, it will be used.}

\item{size}{points size.}

\item{alpha}{numeric value between 0 and 1 specifying point alpha for
controlling transparency. For example, use alpha = 0.5.}

\item{seed}{Random seed passed to \code{set.seed}. if
\code{NA}, set.seed will not be called. Default is 42 for reproducibility.}

\item{font.label}{a vector of length 3 indicating respectively the size (e.g.:
14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and the
color (e.g.: "red") of point labels. For example \emph{font.label = c(14,
"bold", "red")}.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{top}{the number of top genes to be shown on the plot. Use top = 0 to
hide to gene labels.}

\item{select.top.method}{methods to be used for selecting top genes. Allowed
values include "padj" and "fc" for selecting by adjusted p values or fold
changes, respectively.}

\item{label.select}{character vector specifying some labels to show.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels (one MA plot per group).
The variable(s) must be columns of \code{data} (supplied as a data frame). The
top genes are selected \emph{per panel}; point colors use the same
significance thresholds in every panel. Default is NULL (no faceting), in
which case the output is unchanged.}

\item{main}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{line.color}{color of the horizontal threshold lines (the central line
at 0 and the two fold-change cutoff lines). Default is "black".}

\item{ggtheme}{function, ggplot2 theme name. Default value is
\code{theme_classic()}. Set \code{ggtheme = NULL} to skip applying a ggpubr
theme, so the plot keeps ggplot2 default theme or the theme set globally via
\code{theme_set()}.}

\item{...}{other arguments to be passed to \code{\link{ggpar}}.}
}
\value{
a ggplot.
}
\description{
Make MA-plot which is a scatter plot of log2 fold changes (M, on
 the y-axis) versus the average expression signal (A, on the x-axis). \code{M
 = log2(x/y)} and \code{A = (log2(x) + log2(y))/2 = log2(xy)*1/2}, where x
 and y are respectively the mean of the two groups being compared.
}
\examples{
data(diff_express)

# Default plot
ggmaplot(diff_express,
  main = expression("Group 1" \%->\% "Group 2"),
  fdr = 0.05, fc = 2, size = 0.4,
  palette = c("#B31B21", "#1465AC", "darkgray"),
  genenames = as.vector(diff_express$name),
  legend = "top", top = 20,
  font.label = c("bold", 11),
  font.legend = "bold",
  font.main = "bold",
  ggtheme = ggplot2::theme_minimal()
)

# Add rectangle around labels
ggmaplot(diff_express,
  main = expression("Group 1" \%->\% "Group 2"),
  fdr = 0.05, fc = 2, size = 0.4,
  palette = c("#B31B21", "#1465AC", "darkgray"),
  genenames = as.vector(diff_express$name),
  legend = "top", top = 20,
  font.label = c("bold", 11), label.rectangle = TRUE,
  font.legend = "bold",
  font.main = "bold",
  ggtheme = ggplot2::theme_minimal()
)

# Select specific genes to show
# set top = 0, then specify genes using label.select argument
ggmaplot(diff_express,
  main = expression("Group 1" \%->\% "Group 2"),
  fdr = 0.05, fc = 2, size = 0.4,
  genenames = as.vector(diff_express$name),
  ggtheme = ggplot2::theme_minimal(),
  top = 0, label.select = c("BUB1", "CD83")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggline.R
\name{ggline}
\alias{ggline}
\title{Line plot}
\usage{
ggline(
  data,
  x,
  y,
  group = 1,
  numeric.x.axis = FALSE,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  palette = NULL,
  linetype = "solid",
  plot_type = c("b", "l", "p"),
  size = 0.5,
  shape = 19,
  stroke = NULL,
  point.size = size,
  point.color = color,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  select = NULL,
  remove = NULL,
  order = NULL,
  add = "none",
  add.params = list(),
  error.plot = "errorbar",
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  show.line.label = FALSE,
  position = "identity",
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x, y}{x and y variables for drawing.}

\item{group}{grouping variable to connect points by line.
Allowed values are 1 (for one line, one group) or a character vector specifying
the name of the grouping variable (case of multiple lines).}

\item{numeric.x.axis}{logical. If TRUE, x axis will be treated as numeric. Default is FALSE.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{line colors.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{linetype}{line type.}

\item{plot_type}{plot type. Allowed values are one of "b" for both line and point;
"l" for line only; and "p" for point only. Default is "b".}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{shape}{point shapes.}

\item{stroke}{point stroke. Used only for shapes 21-24 to control the thickness of points border.}

\item{point.size}{point size.}

\item{point.color}{point color.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variabes values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{show.line.label}{logical value. If TRUE, shows line labels.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to geom_dotplot.}
}
\description{
Create a line plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Data
df <- data.frame(dose=c("D0.5", "D1", "D2"),
   len=c(4.2, 10, 29.5))
print(df)

# Basic plot
# +++++++++++++++++++++++++++
ggline(df, x = "dose", y = "len")


# Plot with multiple groups
# +++++++++++++++++++++

# Create some data
df2 <- data.frame(supp=rep(c("VC", "OJ"), each=3),
   dose=rep(c("D0.5", "D1", "D2"),2),
   len=c(6.8, 15, 33, 4.2, 10, 29.5))
print(df2)

# Plot "len" by "dose" and
# Change line types and point shapes by a second groups: "supp"
ggline(df2, "dose", "len",
  linetype = "supp", shape = "supp")


# Change colors
# +++++++++++++++++++++

# Change color by group: "supp"
# Use custom color palette
ggline(df2, "dose", "len",
   linetype = "supp", shape = "supp",
   color = "supp", palette = c("#00AFBB", "#E7B800"))


# Add points and errors
# ++++++++++++++++++++++++++

# Data: ToothGrowth data set we'll be used.
df3 <- ToothGrowth
head(df3, 10)

# It can be seen that for each group we have
# different values
ggline(df3, x = "dose", y = "len")

# Visualize the mean of each group
ggline(df3, x = "dose", y = "len",
 add = "mean")

# Add error bars: mean_se
# (other values include: mean_sd, mean_ci, median_iqr, ....)
# Add labels
ggline(df3, x = "dose", y = "len", add = "mean_se")

# Change error.plot to "pointrange"
ggline(df3, x = "dose", y = "len",
 add = "mean_se", error.plot = "pointrange")

# Add jitter points and errors (mean_se)
ggline(df3, x = "dose", y = "len",
 add = c("mean_se", "jitter"))

# Add dot and errors (mean_se)
ggline(df3, x = "dose", y = "len",
 add = c("mean_se", "dotplot"), color = "steelblue")

# Add violin and errors (mean_se)
ggline(df3, x = "dose", y = "len",
 add = c("mean_se", "violin"), color = "steelblue")

# Multiple groups with error bars
# ++++++++++++++++++++++

ggline(df3, x = "dose", y = "len", color = "supp",
 add = "mean_se", palette = c("#00AFBB", "#E7B800"))

# Add jitter
ggline(df3, x = "dose", y = "len", color = "supp",
 add = c("mean_se", "jitter"), palette = c("#00AFBB", "#E7B800"))

# Add dot plot
ggline(df3, x = "dose", y = "len", color = "supp",
 add = c("mean_se", "dotplot"), palette = c("#00AFBB", "#E7B800"))


}
\seealso{
\code{\link{ggpar}}, \code{\link{ggbarplot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotate_figure.R
\name{annotate_figure}
\alias{annotate_figure}
\title{Annotate Arranged Figure}
\usage{
annotate_figure(
  p,
  top = NULL,
  bottom = NULL,
  left = NULL,
  right = NULL,
  fig.lab = NULL,
  fig.lab.pos = c("top.left", "top", "top.right", "bottom.left", "bottom",
    "bottom.right"),
  fig.lab.size,
  fig.lab.face,
  column.titles = NULL,
  row.titles = NULL
)
}
\arguments{
\item{p}{(arranged) ggplots.}

\item{top, bottom, left, right}{optional string, or grob.}

\item{fig.lab}{figure label (e.g.: "Figure 1").}

\item{fig.lab.pos}{position of the figure label, can be one of "top.left",
"top", "top.right", "bottom.left", "bottom", "bottom.right". Default is
"top.left".}

\item{fig.lab.size}{optional size of the figure label.}

\item{fig.lab.face}{optional font face of the figure label. Allowed values
include: "plain", "bold", "italic", "bold.italic".}

\item{column.titles, row.titles}{optional character vector giving one title per
column / per row of the arranged figure, adding titles above each column and
to the left of each row (useful for publication grids, see
\code{\link{ggarrange}()}). Can also be a list of grobs (e.g. built with
\code{\link{text_grob}()}) for full control over styling; a character vector
is rendered in bold using the current theme text settings. The titles are
evenly spaced and therefore assume equal-sized columns / rows (the
\code{ggarrange()} default), with one entry per column / row. \code{row.titles}
are rotated 90 degrees.}
}
\description{
Annotate figures including: i) ggplots, ii) arranged ggplots from
 \code{\link{ggarrange}()}, \code{\link[gridExtra:arrangeGrob]{grid.arrange}()} and
 \code{\link[cowplot]{plot_grid}()}.
}
\examples{
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Create some plots
# ::::::::::::::::::::::::::::::::::::::::::::::::::
# Box plot
bxp <- ggboxplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
# Dot plot
dp <- ggdotplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
# Density plot
dens <- ggdensity(df, x = "len", fill = "dose", palette = "jco")

# Arrange and annotate
# ::::::::::::::::::::::::::::::::::::::::::::::::::
figure <- ggarrange(bxp, dp, dens, ncol = 2, nrow = 2)
annotate_figure(figure,
  top = text_grob("Visualizing Tooth Growth", color = "red", face = "bold", size = 14),
  bottom = text_grob("Data source: \n ToothGrowth data set",
    color = "blue",
    hjust = 1, x = 1, face = "italic", size = 10
  ),
  left = text_grob("Figure arranged using ggpubr", color = "green", rot = 90),
  right = text_grob(bquote("Superscript: (" * kg ~ NH[3] ~ ha^-1 ~ yr^-1 * ")"), rot = 90),
  fig.lab = "Figure 1", fig.lab.face = "bold"
)

# Add a title to each column and each row of an arranged grid
# ::::::::::::::::::::::::::::::::::::::::::::::::::
grid <- ggarrange(bxp, dp, dens, bxp, ncol = 2, nrow = 2)
annotate_figure(grid,
  column.titles = c("Column 1", "Column 2"),
  row.titles = c("Row 1", "Row 2")
)

}
\seealso{
\code{\link{ggarrange}()}
}
\author{
Laszlo Erdey \email{erdey.laszlo@econ.unideb.hu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_pubr.R
\name{theme_pubr}
\alias{theme_pubr}
\alias{theme_pubclean}
\alias{labs_pubr}
\alias{theme_classic2}
\alias{clean_theme}
\alias{clean_table_theme}
\title{Publication ready theme}
\usage{
theme_pubr(
  base_size = 12,
  base_family = "",
  border = FALSE,
  margin = TRUE,
  legend = c("top", "bottom", "left", "right", "none"),
  x.text.angle = 0
)

theme_pubclean(base_size = 12, base_family = "", flip = FALSE)

labs_pubr(base_size = 14, base_family = "")

theme_classic2(base_size = 12, base_family = "")

clean_theme()

clean_table_theme()
}
\arguments{
\item{base_size}{base font size}

\item{base_family}{base font family}

\item{border}{logical value. Default is FALSE. If TRUE, add panel border.}

\item{margin}{logical value. Default is TRUE. If FALSE, reduce plot margin.}

\item{legend}{character specifying legend position. Allowed values are one of
c("top", "bottom", "left", "right", "none"). Default is "top" side position.
to remove the legend use legend = "none". Legend position can be also
specified using a numeric vector c(x, y).  In this case it is possible to
position the legend inside the plotting area. x and y are the coordinates of
the legend box. Their values should be between 0 and 1. c(0,0) corresponds
to the "bottom left" and c(1,1) corresponds to the "top right" position. For
instance use legend = c(0.8, 0.2).}

\item{x.text.angle}{Rotation angle of x axis tick labels. Default value is 0.
Use 90 for vertical text.}

\item{flip}{logical. If TRUE, grid lines are added to y axis instead of x
axis.}
}
\description{
\itemize{ \item \strong{theme_pubr()}: Create a publication ready
 theme \item \strong{theme_pubclean()}: a clean theme without axis lines, to
 direct more attention to the data.  \item \strong{labs_pubr()}: Format only
 plot labels to a publication ready style \item \strong{theme_classic2()}:
 Create a classic theme with axis lines. \item \strong{clean_theme()}: Remove
 axis lines, ticks, texts and titles. \item \strong{clean_table_theme()}:
 Clean the theme of a table, such as those created by
 \code{\link{ggsummarytable}()}}.
}
\details{
\code{theme_pubr()} sets \code{strip.clip = "off"} so that the facet
 strip background border is drawn at its full width (with the ggplot2 default
 \code{strip.clip = "on"} the border is clipped to the strip area and renders
 thinner than requested). As a side effect, a facet label that is wider than
 its panel will overflow the strip instead of being truncated; if that is a
 problem for very long labels, restore clipping with
 \code{+ theme(strip.clip = "on")}.
}
\examples{
p <- ggplot(mtcars, aes(x = wt, y = mpg)) +
  geom_point(aes(color = gear))

# Default plot
p

# Use theme_pubr()
p + theme_pubr()

# Format labels
p + labs_pubr()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/desc_statby.R
\name{desc_statby}
\alias{desc_statby}
\title{Descriptive statistics by groups}
\usage{
desc_statby(data, measure.var, grps, ci = 0.95)
}
\arguments{
\item{data}{a data frame.}

\item{measure.var}{the name of a column containing the variable to be summarized.}

\item{grps}{a character vector containing grouping variables; e.g.: grps = c("grp1", "grp2")}

\item{ci}{the percent range of the confidence interval (default is 0.95).}
}
\value{
A data frame containing descriptive statistics, such as:
\itemize{
\item \strong{length}: the number of elements in each group
\item \strong{min}: minimum
\item \strong{max}: maximum
\item \strong{median}: median
\item \strong{mean}: mean
\item \strong{iqr}: interquartile range
\item \strong{mad}: median absolute deviation (see ?MAD)
\item \strong{sd}: standard deviation of the sample
\item \strong{se}: standard error of the mean. It's calculated as the sample standard deviation divided by the root of the sample size.
\item \strong{ci}: confidence interval of the mean
\item \strong{range}: the range = max - min
\item \strong{cv}: coefficient of variation, sd/mean
\item \strong{var}: variance, sd^2
}
}
\description{
Computes descriptive statistics by groups for a measure variable.
}
\examples{
# Load data
data("ToothGrowth")

# Descriptive statistics
res <- desc_statby(ToothGrowth,
  measure.var = "len",
  grps = c("dose", "supp")
)
head(res[, 1:10])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grids.R
\name{grids}
\alias{grids}
\title{Add Grids to a ggplot}
\usage{
grids(axis = c("xy", "x", "y"), color = "grey92", size = NULL, linetype = NULL)
}
\arguments{
\item{axis}{axis for which grid should be added. Allowed values include \code{c("xy", "x", "y")}.}

\item{color}{grid line color.}

\item{size}{numeric value specifying grid line size.}

\item{linetype}{line type. An integer (0:8), a name (blank, solid, dashed,
dotted, dotdash, longdash, twodash). Sess \code{\link{show_line_types}}.}
}
\description{
Add grids to ggplot.
}
\examples{
# Load data
data("ToothGrowth")

# Basic plot
p <- ggboxplot(ToothGrowth, x = "dose", y = "len")
p

# Add border
p + grids(linetype = "dashed")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_cor.R
\name{stat_cor}
\alias{stat_cor}
\title{Add Correlation Coefficients with P-values to a Scatter Plot}
\usage{
stat_cor(
  mapping = NULL,
  data = NULL,
  method = "pearson",
  alternative = "two.sided",
  cor.coef.name = c("R", "rho", "tau"),
  label.sep = ", ",
  label.x.npc = "left",
  label.y.npc = "top",
  label.x = NULL,
  label.y = NULL,
  label.y.step = 1.4,
  label.anchor = c("data", "panel"),
  output.type = "expression",
  digits = 2,
  r.digits = digits,
  p.digits = digits,
  r.accuracy = NULL,
  p.accuracy = NULL,
  r.leading.zero = NULL,
  p.format.style = "default",
  p.leading.zero = NULL,
  p.decimal.mark = NULL,
  p.coef.name = "p",
  conf.level = 0.95,
  geom = "text",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{method}{a character string indicating which correlation coefficient (or
covariance) is to be computed. One of "pearson" (default), "kendall", or
"spearman".}

\item{alternative}{a character string specifying the alternative hypothesis,
must be one of "two.sided" (default), "greater" or "less". You can specify
just the initial letter.}

\item{cor.coef.name}{character. Can be one of \code{"R"} (pearson coef),
\code{"rho"} (spearman coef) and \code{"tau"} (kendall coef). Uppercase and
lowercase are allowed.}

\item{label.sep}{a character string to separate the terms. Default is ", ", to
separate the correlation coefficient and the p.value.}

\item{label.x.npc, label.y.npc}{can be \code{numeric} or \code{character}
 vector of the same length as the number of groups and/or panels. If too
 short they will be recycled. \itemize{ \item If \code{numeric}, value should
 be between 0 and 1. Coordinates to be used for positioning the label,
 expressed in "normalized parent coordinates". \item If \code{character},
 allowed values include: i) one of c('right', 'left', 'center', 'centre',
 'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
 'middle') for y-axis.}

 If too short they will be recycled.}

\item{label.x, label.y}{\code{numeric} Coordinates (in data units) to be used
for absolute positioning of the label. If too short they will be recycled.}

\item{label.y.step}{numeric value giving the vertical spacing (in text-line
units) between the labels of successive groups when several groups are present.
Default is 1.4 (unchanged behavior). Set \code{label.y.step = 0} to stop the
per-group vertical shift, so that labels align across facet panels when a
factor is mapped to an aesthetic that also defines the facets. This places the
labels flush with the top of each panel; it is similar in spirit to the
\code{aes(group = 1)} workaround, which instead leaves them one text line lower.}

\item{label.anchor}{character. How \code{label.x.npc}/\code{label.y.npc} are
interpreted. \code{"data"} (default) converts them to data coordinates using
the data range, so the label follows the data; the output is unchanged from
previous versions. \code{"panel"} places the label at the true panel-relative
position (npc), so the labels stay aligned across panels/facets whose axis
ranges differ - e.g. with \code{facet_wrap(scales = "free_y")}, with
\code{geom_smooth()} extending each panel by a different amount, or across
separate plots combined with \code{\link{ggarrange}()}. An explicit
\code{label.x}/\code{label.y} always stays in data units regardless of
\code{label.anchor}. Requires ggplot2 >= 3.5.0 (already a dependency).}

\item{output.type}{character One of "expression", "latex", "tex" or "text".}

\item{digits, r.digits, p.digits}{integer indicating the number of decimal
places (round) or significant digits (signif) to be used for the correlation
coefficient and the p-value, respectively. In the default p-value label style,
the displayed p-value keeps the legacy raw value unless \code{p.accuracy} is
set; \code{p.digits} is used by non-default \code{p.format.style} outputs and
the computed \code{p} variable.}

\item{r.accuracy}{a real value specifying the number of decimal places of
precision for the correlation coefficient. Default is NULL. Use (e.g.) 0.01
to show 2 decimal places of precision. If specified, then \code{r.digits} is
ignored.}

\item{p.accuracy}{a real value specifying the number of decimal places of
precision for the p-value. Default is NULL. Use (e.g.) 0.0001 to show 4
decimal places of precision. If specified, then \code{p.digits} is ignored.}

\item{r.leading.zero}{logical. Whether to include the leading zero before the
decimal point in the correlation coefficient (e.g., \code{"0.73"} vs
\code{".73"}). Default (NULL) keeps the leading zero; set to \code{FALSE}
for APA-style reporting.}

\item{p.format.style}{character specifying the p-value formatting style.
One of "default", "apa", "nejm", "lancet", "ama", "graphpad", "scientific".
Default is "default" for backward compatibility.}

\item{p.leading.zero}{logical. Whether to include leading zero before decimal
point (e.g., "0.05" vs ".05"). If NULL, uses the style's default setting.}

\item{p.decimal.mark}{character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{p.coef.name}{character. Symbol used for the p-value label. Default is
\code{"p"}; use \code{"P"} for an uppercase p-value label. For
\code{output.type = "expression"} this should be a single valid plotmath
symbol (e.g. \code{"P"}), since it is parsed as an expression.}

\item{conf.level}{confidence level for the confidence interval of the
correlation coefficient, used to compute the \code{conf.int.low},
\code{conf.int.high} and \code{conf.int.label} computed variables (see the
\strong{Computed variables} section). Default is 0.95. A confidence interval
is only available for \code{method = "pearson"} (with at least 4 complete
observations); for \code{"spearman"}/\code{"kendall"} the confidence-interval
variables are \code{NA}.}

\item{geom}{The geometric object to use to display the data for this layer.
When using a \verb{stat_*()} function to construct a layer, the \code{geom} argument
can be used to override the default coupling between stats and geoms. The
\code{geom} argument accepts the following:
\itemize{
\item A \code{Geom} ggproto subclass, for example \code{GeomPoint}.
\item A string naming the geom. To give the geom as a string, strip the
function name of the \code{geom_} prefix. For example, to use \code{geom_point()},
give the geom as \code{"point"}.
\item For more information and other ways to specify the geom, see the
\link[ggplot2:layer_geoms]{layer geom} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If FALSE (the default), removes missing values with a warning. If
TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{...}{other arguments to pass to \code{\link[ggplot2]{geom_text}} or
\code{\link[ggplot2:geom_text]{geom_label}}.}
}
\description{
Add correlation coefficients with p-values to a scatter plot. Can
 be also used to add `R2`.
}
\section{Computed variables}{
 \describe{ \item{r}{correlation coefficient}
 \item{rr}{correlation coefficient squared} \item{rmse}{root mean square
 deviation (RMSE/RMSD) between \code{x} and \code{y}, computed on the complete
 pairs; meaningful when \code{x} and \code{y} are on the same scale (e.g.
 predicted vs. reference values). Rounded and formatted with the coefficient
 settings (\code{r.digits} / \code{r.accuracy}).} \item{r.label}{formatted
 label for the correlation coefficient} \item{rr.label}{formatted label for
 the squared correlation coefficient} \item{p.label}{label for the p-value}
 \item{rmse.label}{formatted label for the RMSE/RMSD}
 \item{conf.int.low, conf.int.high}{lower and upper bounds of the confidence
 interval of the correlation coefficient (Pearson only; \code{NA} for
 Spearman/Kendall), at the level given by \code{conf.level}}
 \item{conf.int.label}{formatted label for the confidence interval, e.g.
 \code{"95\% CI [0.21, 0.75]"} (\code{NA} when the interval is unavailable)}
 \item{label}{default label displayed by \code{stat_cor()}} }
}

\examples{
# Load data
data("mtcars")
df <- mtcars
df$cyl <- as.factor(df$cyl)

# Scatter plot with correlation coefficient
# :::::::::::::::::::::::::::::::::::::::::::::::::
sp <- ggscatter(df,
  x = "wt", y = "mpg",
  add = "reg.line", # Add regressin line
  add.params = list(color = "blue", fill = "lightgray"), # Customize reg. line
  conf.int = TRUE # Add confidence interval
)
# Add correlation coefficient
sp + stat_cor(method = "pearson", label.x = 3, label.y = 30)

# Specify the number of decimal places of precision for p and r
# Using 3 decimal places for the p-value and
# 2 decimal places for the correlation coefficient (r)
sp + stat_cor(p.accuracy = 0.001, r.accuracy = 0.01)

# Show only the r.label but not the p.label
sp + stat_cor(aes(label = after_stat(r.label)), label.x = 3)

# Use R2 instead of R
ggscatter(df, x = "wt", y = "mpg", add = "reg.line") +
  stat_cor(
    aes(label = paste(after_stat(rr.label), after_stat(p.label), sep = "~`,`~")),
    label.x = 3
  )

# Show the RMSE/RMSD (root mean square deviation) between x and y
# (useful for agreement between paired measurements on the same scale)
sp + stat_cor(aes(label = after_stat(rmse.label)), label.x = 3)

# Combine the correlation coefficient and the RMSE (comma-separated)
sp + stat_cor(
  aes(label = paste(after_stat(r.label), after_stat(rmse.label), sep = "~`,`~")),
  label.x = 3
)

# Show the confidence interval of the correlation coefficient (Pearson)
sp + stat_cor(aes(label = after_stat(conf.int.label)), label.x = 3)

# Correlation coefficient with its confidence interval
sp + stat_cor(
  aes(label = paste(after_stat(r.label), after_stat(conf.int.label), sep = "~`,`~")),
  label.x = 1.5
)

# Color by groups and facet
# ::::::::::::::::::::::::::::::::::::::::::::::::::::
sp <- ggscatter(df,
  x = "wt", y = "mpg",
  color = "cyl", palette = "jco",
  add = "reg.line", conf.int = TRUE
)
sp + stat_cor(aes(color = cyl), label.x = 3)

}
\seealso{
\code{\link{ggscatter}}. For an alternative implementation with more
 control over label positioning (native NPC coordinates, per-group vertical and
 horizontal steps), see \code{ggpmisc::stat_correlation()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggecdf.R
\name{ggecdf}
\alias{ggecdf}
\title{Empirical cumulative density function}
\usage{
ggecdf(
  data,
  x,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  palette = NULL,
  size = NULL,
  linetype = "solid",
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{variable to be drawn.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{line and point color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{line and point size.}

\item{linetype}{line type. See \code{\link{show_line_types}}.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to
\code{\link[ggplot2]{stat_ecdf}} and \code{\link{ggpar}}.}
}
\description{
Empirical Cumulative Density Function (ECDF).
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Create some data format
set.seed(1234)
wdata = data.frame(
   sex = factor(rep(c("F", "M"), each=200)),
   weight = c(rnorm(200, 55), rnorm(200, 58)))

head(wdata, 4)

# Basic ECDF plot
ggecdf(wdata, x = "weight")

# Change colors and linetype by groups ("sex")
# Use custom palette
ggecdf(wdata, x = "weight",
   color = "sex", linetype = "sex",
   palette = c("#00AFBB", "#E7B800"))

}
\seealso{
\code{\link{ggpar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_ggplot.R
\name{as_ggplot}
\alias{as_ggplot}
\title{Storing grid.arrange() arrangeGrob() and plots}
\usage{
as_ggplot(x)
}
\arguments{
\item{x}{an object of class gtable or grob as returned by the functions
\code{\link[gridExtra:arrangeGrob]{arrangeGrob}()} and
\code{\link[gridExtra:arrangeGrob]{grid.arrange}()}.}
}
\value{
an object of class ggplot.
}
\description{
Transform the output of
 \code{\link[gridExtra:arrangeGrob]{arrangeGrob}()} and
 \code{\link[gridExtra:arrangeGrob]{grid.arrange}()} to an object of class
 ggplot.
}
\examples{
# Create some plots
bxp <- ggboxplot(iris, x = "Species", y = "Sepal.Length")
vp <- ggviolin(iris,
  x = "Species", y = "Sepal.Length",
  add = "mean_sd"
)

# Arrange the plots in one page
# Returns a gtable (grob) object
library(gridExtra)
gt <- arrangeGrob(bxp, vp, ncol = 2)

# Transform to a ggplot and print
as_ggplot(gt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggviolin.R
\name{ggviolin}
\alias{ggviolin}
\title{Violin plot}
\usage{
ggviolin(
  data,
  x,
  y,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = "white",
  palette = NULL,
  alpha = 1,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  linetype = "solid",
  trim = FALSE,
  size = NULL,
  width = 1,
  draw_quantiles = NULL,
  select = NULL,
  remove = NULL,
  order = NULL,
  add = "mean_se",
  add.params = list(),
  error.plot = "pointrange",
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  position = position_dodge(0.8),
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{character string containing the name of x variable.}

\item{y}{character vector containing one or more variables to plot}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{outline color.}

\item{fill}{fill color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{alpha}{color transparency. Values should be between 0 and 1.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{linetype}{line types.}

\item{trim}{If \code{TRUE} (default), trim the tails of the violins
to the range of the data. If \code{FALSE}, don't trim the tails.}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{width}{violin width.}

\item{draw_quantiles}{If \code{not(NULL)} (default), draw horizontal lines
at the given quantiles of the density estimate.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variabes values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to
\code{\link[ggplot2]{geom_violin}}, \code{\link{ggpar}} and
\code{\link{facet}}.}
}
\description{
Create a violin plot with error bars. Violin plots are similar to
box plots, except that they also show the kernel probability density of the
data at different values.
}
\details{
The plot can be easily customized using the function ggpar(). Read
 ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
 ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
 scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
 = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
 and position: legend = "right" \item plot orientation : orientation =
 c("vertical", "horizontal", "reverse") }
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
# +++++++++++++++++++++++++++
ggviolin(df, x = "dose", y = "len")
# Change the plot orientation: horizontal
ggviolin(df, "dose", "len", orientation = "horiz")

# Add summary statistics
# ++++++++++++++++++++++++++
# Draw quantiles
ggviolin(df, "dose", "len", add = "none",
   draw_quantiles = 0.5)

# Add box plot
ggviolin(df, x = "dose", y = "len",
 add = "boxplot")

ggviolin(df, x = "dose", y = "len",
 add = "dotplot")

# Add jitter points and
# change point shape by groups ("dose")
ggviolin(df, x = "dose", y = "len",
add = "jitter", shape = "dose")


# Add mean_sd + jittered points
ggviolin(df, x = "dose", y = "len",
 add = c("jitter", "mean_sd"))

# Change error.plot to "crossbar"
ggviolin(df, x = "dose", y = "len",
 add = "mean_sd", error.plot = "crossbar")


# Change colors
# +++++++++++++++++++++++++++
# Change outline and fill colors
ggviolin(df, "dose", "len",
   color = "black", fill = "gray")

# Change outline colors by groups: dose
# Use custom color palette and add boxplot
ggviolin(df, "dose", "len",  color = "dose",
   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
   add = "boxplot")

# Change fill color by groups: dose
# add boxplot with white fill color
ggviolin(df, "dose", "len", fill = "dose",
   palette = c("#00AFBB", "#E7B800", "#FC4E07"),
   add = "boxplot", add.params = list(fill = "white"))


# Plot with multiple groups
# +++++++++++++++++++++
# fill or color box plot by a second group : "supp"
ggviolin(df, "dose", "len", color = "supp",
 palette = c("#00AFBB", "#E7B800"), add = "boxplot")

}
\seealso{
\code{\link{ggpar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_coord.R
\name{get_coord}
\alias{get_coord}
\title{Checks and Returns Data Coordinates from Multiple Input Options}
\usage{
get_coord(
  group = 1L,
  data.ranges = NULL,
  coord = NULL,
  npc = "left",
  step = 0.1,
  margin.npc = 0.05
)
}
\arguments{
\item{group}{integer ggplot's group id. Used to shift coordinates to avoid
overlaps.}

\item{data.ranges}{a numeric vector of length 2 containing the data ranges
(minimum and the maximum). Should be specified only when \code{coord =
NULL} and \code{npc} is specified. Used to convert \code{npc} to data
coordinates. Considered only when the argument \code{npc} is specified.}

\item{coord}{data coordinates (i.e., either x or y coordinates).}

\item{npc}{numeric (in [0-1]) or character vector of coordinates. If
character, should be one of c('right', 'left', 'bottom', 'top', 'center',
'centre', 'middle'). Note that, the \code{data.ranges}, \code{step} and
\code{margin.npc}, arguments are considered only when \code{npc} is
specified. The option \code{npc} is ignored when the argument \code{coord} is specified.}

\item{step}{numeric value in [0-1]. The step size for shifting coordinates
in npc units. Considered as horizontal step for x-axis and vertical step
for y-axis. For y-axis, the step value can be negative to reverse the order of groups.}

\item{margin.npc}{numeric [0-1] The margin added towards the nearest
plotting area edge when converting character coordinates into npc.}
}
\value{
a numeric vector representing data coordinates.
}
\description{
Checks and returns selected coordinates from multiple input
  options, which can be either data (x-y) coordinates or npc (normalized
  parent coordinates).

  Helper function internally used in \code{ggpubr} function to guess the type
  of coordinates specified by the user. For example, in the function
  \code{stat_cor()}, users can specify either the option \code{label.x} (data
  coordinates) or \code{label.x.npc} (npc coordinates); those coordinates are
  passed to \code{get_coord()}, which will make some checking and then return
  a unique coordinates for the label position.
}
\examples{
# If npc is specified, it is converted into data coordinates
get_coord(data.ranges = c(2, 20), npc = "left")
get_coord(data.ranges = c(2, 20), npc = 0.1)

# When coord is specified, no transformation is performed
# because this is assumed to be a data coordinate
get_coord(coord = 5)

# For grouped plots
res_top <- get_coord(
  data.ranges = c(4.2, 36.4), group = c(1, 2, 3),
  npc = "top", step = -0.1, margin.npc = 0
)
res_top

}
\seealso{
\code{\link{as_npc}}, \code{\link{npc_to_data_coord}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/font.R
\name{font}
\alias{font}
\title{Change the Appearance of Titles and Axis Labels}
\usage{
font(object, size = NULL, color = NULL, face = NULL, family = NULL, ...)
}
\arguments{
\item{object}{character string specifying the plot components. Allowed values
include: \itemize{ \item \code{"title"} for the main title \item
\code{"subtitle"} for the plot subtitle \item \code{"caption"} for the plot
caption \item \code{"legend.title"} for the legend title \item
\code{"legend.text"} for the legend text \item \code{"x", "xlab", or "x.title"}
for x axis label \item \code{"y", "ylab", or "y.title"} for y axis label \item
\code{"xy", "xylab", "xy.title" or "axis.title"} for both x and y axis
labels \item \code{"x.text"} for x axis texts (x axis tick labels) \item
\code{"y.text"} for y axis texts (y axis tick labels) \item \code{"xy.text"}
or \code{"axis.text"} for both x and y axis texts }}

\item{size}{numeric value specifying the font size, (e.g.: \code{size = 12}).}

\item{color}{character string specifying the font color, (e.g.: \code{color =
"red"}).}

\item{face}{the font face or style. Allowed values include one of
\code{"plain", "bold", "italic", "bold.italic"}, (e.g.: \code{face =
"bold.italic"}).}

\item{family}{the font family.}

\item{...}{other arguments to pass to the function
\code{\link[ggplot2:element]{element_text}()}.}
}
\description{
Change the appearance of the main title, subtitle, caption, axis
 labels and text, as well as the legend title and texts. Wrapper around
 \code{\link[ggplot2:element]{element_text}()}.
}
\examples{
# Load data
data("ToothGrowth")

# Basic plot
p <- ggboxplot(ToothGrowth,
  x = "dose", y = "len", color = "dose",
  title = "Box Plot created with ggpubr",
  subtitle = "Length by dose",
  caption = "Source: ggpubr",
  xlab = "Dose (mg)", ylab = "Teeth length"
)
p

# Change the appearance of titles and labels
p +
  font("title", size = 14, color = "red", face = "bold.italic") +
  font("subtitle", size = 10, color = "orange") +
  font("caption", size = 10, color = "orange") +
  font("xlab", size = 12, color = "blue") +
  font("ylab", size = 12, color = "#993333") +
  font("xy.text", size = 12, color = "gray", face = "bold")

# Change the appearance of legend title and texts
p +
  font("legend.title", color = "blue", face = "bold") +
  font("legend.text", color = "red")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_kruskal_test.R
\name{stat_kruskal_test}
\alias{stat_kruskal_test}
\title{Add Kruskal-Wallis Test P-values to a GGPlot}
\usage{
stat_kruskal_test(
  mapping = NULL,
  data = NULL,
  group.by = NULL,
  label = "{method}, p = {p.format}",
  label.x.npc = "left",
  label.y.npc = "top",
  label.x = NULL,
  label.y = NULL,
  step.increase = 0.1,
  p.adjust.method = "holm",
  significance = list(),
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  geom = "text",
  position = "identity",
  na.rm = FALSE,
  show.legend = FALSE,
  inherit.aes = TRUE,
  parse = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{group.by}{(optional) character vector specifying the grouping variable;
it should be used only for grouped plots. Possible values are : \itemize{
\item \code{"x.var"}: Group by the x-axis variable and perform the test
between legend groups. In other words, the p-value is compute between legend
groups at each x position \item \code{"legend.var"}: Group by the legend
variable and perform the test between x-axis groups. In other words, the
test is performed between the x-groups for each legend level. }}

\item{label}{the column containing the label (e.g.: label = "p" or label =
"p.adj"), where \code{p} is the p-value. Can be also an expression that can
be formatted by the \code{\link[glue]{glue}()} package. For example, when
specifying label = "t-test, p = \{p\}", the expression \{p\} will be
replaced by its value. Values inserted through a glue expression are taken
from the raw data column and are \strong{not} rounded by \code{p.digits}; to
round within a glue expression, wrap the value, e.g.
\code{label = "p = \{signif(p, 3)\}"} (or \code{\{format_p_value(p)\}} for the
p-value house style).}

\item{label.x.npc, label.y.npc}{can be \code{numeric} or \code{character}
vector of the same length as the number of groups and/or panels. If too
short they will be recycled. \itemize{ \item If \code{numeric}, value should
be between 0 and 1. Coordinates to be used for positioning the label,
expressed in "normalized parent coordinates". \item If \code{character},
allowed values include: i) one of c('right', 'left', 'center', 'centre',
'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
'middle') for y-axis.}}

\item{label.x, label.y}{\code{numeric} Coordinates (in data units) to be used
for absolute positioning of the label. If too short they will be recycled.}

\item{step.increase}{numeric vector with the increase in fraction of total
height for every additional comparison to minimize overlap.}

\item{p.adjust.method}{method for adjusting p values (see
\code{\link[stats]{p.adjust}}).  Has impact only in a situation, where
multiple pairwise tests are performed; or when there are multiple grouping
variables. Allowed values include "holm", "hochberg", "hommel",
"bonferroni", "BH", "BY", "fdr", "none". If you don't want to adjust the p
value (not recommended), use p.adjust.method = "none".}

\item{significance}{a list of arguments specifying the significance cutpoints
 and symbols. For example, \code{significance <- list(cutpoints = c(0,
 0.0001, 0.001, 0.01, 0.05, Inf), symbols = c("****", "***", "**", "*",
 "ns"))}.

 In other words, we use the following convention for symbols indicating
 statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
 \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001
 \item \code{****}:  p <= 0.0001 }

 Note: \code{significance} is kept for backward compatibility. For functions
 that also expose \code{signif.cutoffs} and related parameters,
 \code{significance} takes precedence when provided.}

\item{p.format.style}{character string specifying the p-value formatting style
applied to recognized numeric p-value label columns (see \code{p.digits}).
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS). See
\code{\link{list_p_format_styles}} for details. Default is \code{"default"},
which leaves the rendered p-value labels unchanged.}

\item{p.digits}{integer indicating the number of digits used to
format recognized \strong{numeric} p-value label columns (\code{label} is one
of \code{"p"}, \code{"p.adj"}, \code{"p.value"}, \code{"p.val"},
\code{"pval"}, \code{"padj"}). Default is 3. Set to \code{NULL} to print the
raw value without rounding (this also disables \code{p.format.style} and the
related formatting arguments). All other labels are left unchanged: other
numeric columns (e.g. \code{"statistic"}, \code{"n"}, effect sizes),
significance symbols (\code{"p.signif"}, \code{"p.adj.signif"}),
already-formatted strings, and \code{\link[glue]{glue}} expressions. A
p-named column whose values fall outside \code{[0, 1]} is also left as-is.
Uses the same formatting engine (\code{\link{format_p_value}()}) as
\code{\link{stat_anova_test}()} for consistency across layers. The style of
this formatting is further controlled by \code{p.format.style},
\code{p.leading.zero}, \code{p.min.threshold} and \code{p.decimal.mark}
(same arguments as \code{\link{stat_compare_means}()}).}

\item{p.leading.zero}{logical indicating whether to include the leading zero
before the decimal point (e.g., "0.05" vs ".05"). If provided, overrides the
style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display
exactly. Values below this threshold are shown as "< threshold" (e.g.
\code{p.min.threshold = 0.001} renders very small p-values as "< 0.001"). If
provided, overrides the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If
\code{NULL}, uses \code{getOption("OutDec")}.}

\item{geom}{The geometric object to use to display the data for this layer.
When using a \verb{stat_*()} function to construct a layer, the \code{geom} argument
can be used to override the default coupling between stats and geoms. The
\code{geom} argument accepts the following:
\itemize{
\item A \code{Geom} ggproto subclass, for example \code{GeomPoint}.
\item A string naming the geom. To give the geom as a string, strip the
function name of the \code{geom_} prefix. For example, to use \code{geom_point()},
give the geom as \code{"point"}.
\item For more information and other ways to specify the geom, see the
\link[ggplot2:layer_geoms]{layer geom} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If FALSE (the default), removes missing values with a warning. If
TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{parse}{If TRUE, the labels will be parsed into expressions and displayed
as described in \code{?plotmath}.}

\item{...}{other arguments passed to the function \code{geom_bracket()} or
\code{geom_text()}}
}
\description{
Add Kruskal-Wallis test p-values to a ggplot, such as
 box plots, dot plots and stripcharts.
}
\section{Computed variables}{
 \itemize{
 \item{statistic}:	the Kruskal-Wallis rank sum chi-squared statistic used to compute the p-value.
 \item{p}:	p-value.
 \item{p.adj}: Adjusted p-values.
 \item{p.signif}: P-value significance.
 \item{p.adj.signif}: Adjusted p-value significance.
 \item{p.format}: Formatted p-value.
 \item{p.format.signif}: Formatted p-value with significance symbols.
 \item{p.adj.format}: Formatted adjusted p-value.
 \item{n}: number of samples.
  }
}

\examples{
# Data preparation
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Transform `dose` into factor variable
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Add a random grouping variable
set.seed(123)
df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
df$len <- ifelse(df$group == "grp2", df$len + 2, df$len)
df$len <- ifelse(df$group == "grp3", df$len + 7, df$len)
head(df, 3)


# Basic boxplot
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Create a basic boxplot
# Add 5\% and 10\% space to the plot bottom and the top, respectively
bxp <- ggboxplot(df, x = "dose", y = "len") +
  scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))

# Add the p-value to the boxplot
bxp + stat_kruskal_test()

# Change the label position
# Using coordinates in data units
bxp + stat_kruskal_test(label.x = "1", label.y = 10, hjust = 0)

# Format the p-value differently
custom_p_format <- function(p) {
  rstatix::p_format(p, accuracy = 0.0001, digits = 3, leading.zero = FALSE)
}
bxp + stat_kruskal_test(
  label = "Kruskal-Wallis, italic(p) = {custom_p_format(p)}{p.signif}"
)

# Show a detailed label in italic
bxp + stat_kruskal_test(label = "as_detailed_italic")


# Faceted plots
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Create a ggplot facet
bxp <- ggboxplot(df, x = "dose", y = "len", facet.by = "supp") +
  scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))
# Add p-values
bxp + stat_kruskal_test()


# Grouped plots
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
bxp2 <- ggboxplot(df, x = "group", y = "len", color = "dose", palette = "npg")

# For each x-position, computes tests between legend groups
bxp2 + stat_kruskal_test(aes(group = dose), label = "p = {p.format}{p.signif}")

#  For each legend group, computes tests between x variable groups
bxp2 + stat_kruskal_test(aes(group = dose, color = dose), group.by = "legend.var")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shared_docs.R
\name{ggpubr-common-params}
\alias{ggpubr-common-params}
\title{Common parameters for ggpubr stat and geom functions}
\arguments{
\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}
}
\description{
Common parameters for ggpubr stat and geom functions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\name{format_single_p}
\alias{format_single_p}
\title{Format a Single P-Value}
\usage{
format_single_p(p, digits, leading.zero, decimal.mark)
}
\arguments{
\item{p}{Single numeric p-value.}

\item{digits}{Number of decimal places.}

\item{leading.zero}{Whether to include leading zero.}

\item{decimal.mark}{Character string to use as the decimal mark.}
}
\value{
Formatted character string.
}
\description{
Internal helper function to format a single p-value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_line_types.R
\name{show_line_types}
\alias{show_line_types}
\title{Line types available in R}
\usage{
show_line_types()
}
\value{
a ggplot.
}
\description{
Show line types available in R.
}
\examples{
show_line_types() +
  theme_minimal()
}
\seealso{
\code{\link{ggpar}} and \code{\link{ggline}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils-geom-signif.R
\name{geom_signif}
\alias{geom_signif}
\title{geom_signif exported from ggsignif package}
\description{
See \code{ggsignif::\link[ggsignif:stat_signif]{geom_signif}} for details.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/border.R
\name{border}
\alias{border}
\title{Set ggplot Panel Border Line}
\usage{
border(color = "black", size = 0.8, linetype = NULL)
}
\arguments{
\item{color}{border line color.}

\item{size}{numeric value specifying border line size.}

\item{linetype}{line type. An integer (0:8), a name (blank, solid, dashed,
dotted, dotdash, longdash, twodash). Sess \code{\link{show_line_types}}.}
}
\description{
Change or set ggplot panel border.
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
p <- ggboxplot(df, x = "dose", y = "len")
p

# Add border
p + border()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_exec.R
\name{geom_exec}
\alias{geom_exec}
\title{Execute ggplot2 functions}
\usage{
geom_exec(geomfunc = NULL, data = NULL, position = NULL, ...)
}
\arguments{
\item{geomfunc}{a ggplot2 function (e.g.: geom_point)}

\item{data}{a data frame to be used for mapping}

\item{position}{Position adjustment, either as a string, or the result of a
call to a position adjustment function.}

\item{...}{arguments accepted by the function}
}
\value{
a plot if geomfunc != NULL or a list(option, mapping) if
  geomfunc is NULL.
}
\description{
A helper function used by ggpubr functions to execute any geom_*
  functions in ggplot2. Useful only when you want to call a geom_* function
  without worrying about the arguments to put in aes(). Basic users of ggpubr
  don't need this function.
}
\examples{
\dontrun{
ggplot() +
  geom_exec(geom_point,
    data = mtcars,
    x = "mpg", y = "wt", size = "cyl", color = "cyl"
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/facet.R
\name{facet}
\alias{facet}
\title{Facet a ggplot into Multiple Panels}
\usage{
facet(
  p,
  facet.by,
  nrow = NULL,
  ncol = NULL,
  scales = "fixed",
  short.panel.labs = TRUE,
  labeller = "label_value",
  panel.labs = NULL,
  panel.labs.background = list(color = NULL, fill = NULL),
  panel.labs.font = list(face = NULL, color = NULL, size = NULL, angle = NULL),
  panel.labs.font.x = panel.labs.font,
  panel.labs.font.y = panel.labs.font,
  strip.position = "top",
  ...
)
}
\arguments{
\item{p}{a ggplot}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{nrow, ncol}{Number of rows and columns in the panel. Used only when the
data is faceted by one grouping variable.}

\item{scales}{should axis scales of panels be fixed ("fixed", the default),
free ("free"), or free in one dimension ("free_x", "free_y").}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{labeller}{Character vector. An alternative to the argument
\code{short.panel.labs}. Possible values are one of "label_both" (panel
labelled by both grouping variable names and levels) and "label_value"
(panel labelled with only grouping levels).}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{panel.labs.background}{a list to customize the background of panel
labels. Should contain the combination of the following elements: \itemize{
\item \code{color, linetype, linewidth}: background line color, type and width
\item \code{fill}: background fill color. } For example,
panel.labs.background = list(color = "blue", fill = "pink", linetype =
"dashed", linewidth = 0.5). Note: \code{size} is deprecated, use
\code{linewidth} instead.}

\item{panel.labs.font}{a list of aesthetics indicating the size (e.g.: 14), the
face/style (e.g.: "plain", "bold", "italic", "bold.italic") and the color
(e.g.: "red") and the orientation angle (e.g.: 45) of panel labels.}

\item{panel.labs.font.x, panel.labs.font.y}{same as panel.labs.font but for
only x and y direction, respectively.}

\item{strip.position}{(used only in \code{facet_wrap()}). By default, the
labels are displayed on the top of the plot. Using \code{strip.position} it
is possible to place the labels on either of the four sides by setting
\code{strip.position = c("top", "bottom", "left", "right")}}

\item{...}{not used}
}
\description{
Create multi-panel plots of a data set grouped by one or two
 grouping variables. Wrapper around \code{\link[ggplot2]{facet_wrap}}
}
\examples{
p <- ggboxplot(ToothGrowth,
  x = "dose", y = "len",
  color = "supp"
)
print(p)

facet(p, facet.by = "supp")

# Customize
facet(p + theme_bw(),
  facet.by = "supp",
  short.panel.labs = FALSE, # Allow long labels in panels
  panel.labs.background = list(fill = "steelblue", color = "steelblue")
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggballoonplot.R
\name{ggballoonplot}
\alias{ggballoonplot}
\title{Balloon plot}
\usage{
ggballoonplot(
  data,
  x = NULL,
  y = NULL,
  size = "value",
  facet.by = NULL,
  size.range = c(1, 10),
  shape = 21,
  color = "black",
  fill = "gray",
  show.label = FALSE,
  font.label = list(size = 12, color = "black"),
  rotate.x.text = TRUE,
  ggtheme = theme_minimal(),
  ...
)
}
\arguments{
\item{data}{a data frame. Can be: \itemize{ \item \bold{a standard
contingency table} formed by two categorical variables: a data frame with
row names and column names. The categories of the first variable are
columns and the categories of the second variable are rows. \item \bold{a
stretched contingency table}: a data frame containing at least three columns
corresponding, respectively, to (1) the categories of the first variable,
(2) the categories of the second variable, (3) the frequency value. In this
case, you should specify the argument x and y in the function
\code{ggballoonplot()}}.}

\item{x, y}{the column names specifying, respectively, the first and the
second variable forming the contingency table. Required only when the data
is a stretched contingency table.}

\item{size}{point size. By default, the points size reflects the relative
magnitude of the value of the corresponding cell (\code{size = "value"}).
Can be also numeric (\code{size = 4}).}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping variables for
faceting the plot into multiple panels. Should be in the data.}

\item{size.range}{a numeric vector of length 2 that specifies the minimum and
maximum size of the plotting symbol. Default values are \code{size.range =
c(1, 10)}.}

\item{shape}{points shape. The default value is 21. Alternative values include
22, 23, 24, 25.}

\item{color}{point border line color.}

\item{fill}{point fill color. Default is "lightgray". Considered only for
points 21 to 25.}

\item{show.label}{logical. If TRUE, show the data cell values as point
labels.}

\item{font.label}{a vector of length 3 indicating respectively the size
(e.g.: 14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and
the color (e.g.: "red") of point labels. For example font.label = c(14,
"bold", "red"). To specify only the size and the style, use font.label =
c(14, "plain").}

\item{rotate.x.text}{logical. If TRUE (default), rotate the x axis text.}

\item{ggtheme}{function, ggplot2 theme name. Default value is
\code{theme_minimal()}. Set \code{ggtheme = NULL} to skip applying a ggpubr
theme, so the plot keeps ggplot2 default theme or the theme set globally via
\code{theme_set()}.}

\item{...}{other arguments passed to the function \code{\link{ggpar}}}
}
\description{
Plot a graphical matrix where each cell contains a dot whose
  size reflects the relative magnitude of the corresponding component. Useful
  to visualize contingency table formed by two categorical variables.
}
\examples{
# Define color palette
my_cols <- c(
  "#0D0887FF", "#6A00A8FF", "#B12A90FF",
  "#E16462FF", "#FCA636FF", "#F0F921FF"
)

# Standard contingency table
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
# Read a contingency table: housetasks
# Repartition of 13 housetasks in the couple
data <- read.delim(
  system.file("demo-data/housetasks.txt", package = "ggpubr"),
  row.names = 1
)
data

# Basic ballon plot
ggballoonplot(data)

# Change color and fill
ggballoonplot(data, color = "#0073C2FF", fill = "#0073C2FF")


# Change color according to the value of table cells
ggballoonplot(data, fill = "value") +
  scale_fill_gradientn(colors = my_cols)

# Change the plotting symbol shape
ggballoonplot(data, fill = "value", shape = 23) +
  gradient_fill(c("blue", "white", "red"))


# Set points size to 8, but change fill color by values
# Sow labels
ggballoonplot(data,
  fill = "value", color = "lightgray",
  size = 10, show.label = TRUE
) +
  gradient_fill(c("blue", "white", "red"))

# Stretched contingency table
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::

# Create an Example Data Frame Containing Car x Color data
carnames <- c("bmw", "renault", "mercedes", "seat")
carcolors <- c("red", "white", "silver", "green")
datavals <- round(rnorm(16, mean = 100, sd = 60), 1)
car_data <- data.frame(
  Car = rep(carnames, 4),
  Color = rep(carcolors, c(4, 4, 4, 4)),
  Value = datavals
)

car_data

ggballoonplot(car_data,
  x = "Car", y = "Color",
  size = "Value", fill = "Value"
) +
  scale_fill_gradientn(colors = my_cols) +
  guides(size = "none")


# Grouped frequency table
# :::::::::::::::::::::::::::::::::::::::::::::::::::::::::
data("Titanic")
dframe <- as.data.frame(Titanic)
head(dframe)
ggballoonplot(
  dframe,
  x = "Class", y = "Sex",
  size = "Freq", fill = "Freq",
  facet.by = c("Survived", "Age"),
  ggtheme = theme_bw()
) +
  scale_fill_gradientn(colors = my_cols)

# Hair and Eye Color of Statistics Students
data(HairEyeColor)
ggballoonplot(as.data.frame(HairEyeColor),
  x = "Hair", y = "Eye", size = "Freq",
  ggtheme = theme_gray()
) \%>\%
  facet("Sex")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_point_shapes.R
\name{show_point_shapes}
\alias{show_point_shapes}
\title{Point shapes available in R}
\usage{
show_point_shapes()
}
\value{
a ggplot.
}
\description{
Show point shapes available in R.
}
\examples{
show_point_shapes() +
  theme_minimal()
}
\seealso{
\code{\link{ggpar}} and \code{\link{ggline}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_means.R
\name{compare_means}
\alias{compare_means}
\title{Comparison of Means}
\usage{
compare_means(
  formula,
  data,
  method = "wilcox.test",
  paired = FALSE,
  id = NULL,
  group.by = NULL,
  ref.group = NULL,
  symnum.args = list(),
  p.adjust.method = "holm",
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  ...
)
}
\arguments{
\item{formula}{a formula of the form \code{x ~ group} where \code{x} is a
 numeric variable giving the data values and \code{group} is a factor with
 one or multiple levels giving the corresponding groups. For example,
 \code{formula = TP53 ~ cancer_group}.

 It's also possible to perform the test for multiple response variables at
 the same time. For example, \code{formula = c(TP53, PTEN) ~ cancer_group}.}

\item{data}{a data.frame containing the variables in the formula.}

\item{method}{the type of test. Default is \link[stats]{wilcox.test}. Allowed
values include: \itemize{ \item \code{\link[stats]{t.test}} (parametric) and
\code{\link[stats]{wilcox.test}} (non-parametric). Perform comparison
between two groups of samples. If the grouping variable contains more than
two levels, then a pairwise comparison is performed. \item
\code{\link[stats]{anova}} (parametric) and
\code{\link[stats]{kruskal.test}} (non-parametric). Perform one-way ANOVA
test comparing multiple groups. }}

\item{paired}{a logical indicating whether you want a paired test. Used only
in \code{\link[stats]{t.test}} and in \link[stats]{wilcox.test}.}

\item{id}{optional character string naming a column that identifies matched
subjects for a \strong{paired} comparison (\code{method = "t.test"} or
\code{"wilcox.test"}). By default (\code{id = NULL}) a paired test pairs
observations by \emph{row order}, so a p-value can be wrong if the data are
not sorted so that the compared groups align. Providing \code{id} pairs the
observations by subject id instead (row-order independent), using only the
complete pairs (per-comparison pairwise deletion, via \pkg{rstatix}). It
works for a two-group, a pairwise (more than two groups) and a
\code{ref.group} comparison; it is an error to combine \code{id} with
\code{anova}/\code{kruskal.test} or with \code{ref.group = ".all."}.}

\item{group.by}{a character vector containing the name of grouping variables.}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 \code{ref.group} can be also \code{".all."}. In this case, each of the
 grouping variable levels is compared to all (i.e. basemean).}

\item{symnum.args}{a list of arguments to pass to the function
 \code{\link[stats]{symnum}} for symbolic number coding of p-values. For
 example, \code{symnum.args <- list(cutpoints = c(0, 0.0001, 0.001,
 0.01, 0.05, Inf), symbols = c("****", "***", "**", "*",  "ns"))}.

 In other words, we use the following convention for symbols indicating
 statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
 \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001 \item \code{****}:  p <= 0.0001 }

 Note: If \code{signif.cutoffs} is provided, it takes precedence over
 \code{symnum.args}.}

\item{p.adjust.method}{method for adjusting p values (see
 \code{\link[stats]{p.adjust}}). Has impact only in a situation, where
 multiple pairwise tests are performed; or when there are multiple grouping
 variables. Allowed values include "holm", "hochberg", "hommel",
 "bonferroni", "BH", "BY", "fdr", "none". If you don't want to adjust the p
 value (not recommended), use p.adjust.method = "none".

 Note that, when the \code{formula} contains multiple variables, the p-value
 adjustment is done independently for each variable.}

\item{p.format.style}{character string specifying the p-value formatting style.
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
See \code{\link{list_p_format_styles}} for details.}

\item{p.digits}{integer specifying the number of decimal places for p-values.
If provided, overrides the style default.}

\item{p.leading.zero}{logical indicating whether to include leading zero before
decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display exactly.
Values below this threshold are shown as "< threshold". If NULL, the selected
style's default threshold is used; styles without a threshold show exact
values. If provided, overrides the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{signif.cutoffs}{numeric vector of p-value cutoffs in descending order
for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
If \code{use.four.stars = TRUE}, can include a fourth level (e.g.,
\code{c(0.10, 0.05, 0.01, 0.001)} for "****" at p < 0.001).
Default is NULL, which uses the package defaults (backward compatible).}

\item{signif.symbols}{character vector of symbols corresponding to
\code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
(and "****" if \code{use.four.stars = TRUE}). Must have the same length
as \code{signif.cutoffs}.}

\item{ns.symbol}{character string for non-significant results. Default is "ns".
Use "" (empty string) to show nothing for non-significant results.}

\item{use.four.stars}{logical. If TRUE and \code{signif.symbols} is NULL,
allows four stars (****) for the most significant level when
\code{signif.cutoffs} has 4 levels. Default is FALSE.}

\item{...}{Other arguments to be passed to the test function.}
}
\value{
a data frame with the following columns:
\itemize{
\item \code{.y.}: the y variable used in the test.
\item \code{group1,group2}: the compared groups in the pairwise tests.
Available only when \code{method = "t.test"} or \code{method = "wilcox.test"}.
\item \code{p}: the p-value.
\item \code{p.adj}: the adjusted p-value. Default for \code{p.adjust.method = "holm"}.
\item \code{p.format}: the formatted p-value.
\item \code{p.format.signif}: the formatted p-value with significance symbols.
\item \code{p.signif}: the significance level.
\item \code{method}: the statistical test used to compare groups.


}
}
\description{
Performs one or multiple mean comparisons.
}
\section{Significance letters (compact letter display)}{

To label groups with letters instead of p-values or stars - groups that share a
letter are not significantly different - compute the pairwise comparisons and
derive the letters with \code{rstatix::add_cld()} (available without any extra
package, as \code{rstatix} is already a dependency), then place them with
\code{geom_text()}:

\preformatted{
library(ggpubr)
library(rstatix)
library(dplyr)
df <- ToothGrowth
df$dose <- factor(df$dose)

# all-pairwise comparisons, then compact letters (columns: group, cld)
cld <- df \%>\% tukey_hsd(len ~ dose) \%>\% add_cld()

# one letter per group, placed above each box
ypos <- df \%>\% group_by(dose) \%>\%
  summarise(y.position = max(len) + 2, .groups = "drop")
cld$y.position <- ypos$y.position[match(cld$group, as.character(ypos$dose))]

ggboxplot(df, "dose", "len") +
  geom_text(data = cld, aes(x = group, y = y.position, label = cld))
}

\code{add_cld()} expects an all-pairwise result (\code{tukey_hsd()},
\code{dunn_test()}, \code{games_howell_test()}, pairwise
\code{wilcox_test()}/\code{t_test()}, or \code{compare_means()}); it is not
defined for a single \code{ref.group} comparison.

To instead flag each treatment by which of several controls it differs from
(e.g. \code{"a"} for a negative control and \code{"b"} for a positive control),
run one comparison per control and assemble the letters:

\preformatted{
trts <- c("trtA", "trtB", "trtC")
cn <- compare_means(value ~ group, df, ref.group = "neg.ctrl")
cp <- compare_means(value ~ group, df, ref.group = "pos.ctrl")
pv <- function(cc, g) dplyr::filter(cc, group1 == g | group2 == g)$p[1]
lab <- sapply(trts, function(g)
  paste0(if (pv(cn, g) < .05) "a" else "",
         if (pv(cp, g) < .05) "b" else ""))
# then place `lab` above each treatment with geom_text()
}
}

\examples{
# Load data
# :::::::::::::::::::::::::::::::::::::::
data("ToothGrowth")
df <- ToothGrowth

# One-sample test
# :::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ 1, df, mu = 0)

# Two-samples unpaired test
# :::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ supp, df)

# Two-samples paired test
# :::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ supp, df, paired = TRUE)

# Paired test pairing by a subject id column (row-order independent)
# :::::::::::::::::::::::::::::::::::::::::
df$id <- rep(1:30, 2) # pairs the two supp levels by subject
compare_means(len ~ supp, df, paired = TRUE, id = "id")

# Compare supp levels after grouping the data by "dose"
# ::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ supp, df, group.by = "dose")

# pairwise comparisons
# ::::::::::::::::::::::::::::::::::::::::
# As dose contains more thant two levels ==>
# pairwise test is automatically performed.
compare_means(len ~ dose, df)

# Comparison against reference group
# ::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ dose, df, ref.group = "0.5")

# Comparison against all
# ::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ dose, df, ref.group = ".all.")

# Anova and kruskal.test
# ::::::::::::::::::::::::::::::::::::::::
compare_means(len ~ dose, df, method = "anova")
compare_means(len ~ dose, df, method = "kruskal.test")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_pvalue_manual.R
\name{stat_pvalue_manual}
\alias{stat_pvalue_manual}
\title{Add P-values Manually to a ggplot}
\usage{
stat_pvalue_manual(
  data,
  label = NULL,
  y.position = "y.position",
  xmin = "group1",
  xmax = "group2",
  x = NULL,
  size = 3.88,
  label.size = size,
  p.digits = 3,
  p.format.style = "default",
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  bracket.size = 0.3,
  bracket.nudge.y = 0,
  bracket.shorten = 0,
  color = "black",
  linetype = 1,
  tip.length = 0.03,
  tip.length.ref = c("data", "axis"),
  remove.bracket = FALSE,
  step.increase = 0,
  step.group.by = NULL,
  hide.ns = FALSE,
  vjust = 0,
  coord.flip = FALSE,
  position = "identity",
  inherit.aes = FALSE,
  ...
)
}
\arguments{
\item{data}{a data frame containing statistical test results. The expected
default format should contain the following columns: \code{group1 | group2 |
p | y.position | etc}. \code{group1} and \code{group2} are the groups that
have been compared. \code{p} is the resulting p-value. \code{y.position} is
the y coordinates of the p-values in the plot.}

\item{label}{the column containing the label (e.g.: label = "p" or label =
"p.adj"), where \code{p} is the p-value. Can be also an expression that can
be formatted by the \code{\link[glue]{glue}()} package. For example, when
specifying label = "t-test, p = \{p\}", the expression \{p\} will be
replaced by its value. Values inserted through a glue expression are taken
from the raw data column and are \strong{not} rounded by \code{p.digits}; to
round within a glue expression, wrap the value, e.g.
\code{label = "p = \{signif(p, 3)\}"} (or \code{\{format_p_value(p)\}} for the
p-value house style).}

\item{y.position}{column containing the coordinates (in data units) to be used
for absolute positioning of the label. Default value is "y.position". Can be
also a numeric vector.}

\item{xmin}{column containing the position of the left sides of the brackets.
Default value is "group1".}

\item{xmax}{(optional) column containing the position of the right sides of
the brackets. Default value is "group2". If NULL, the p-values are plotted
as a simple text.}

\item{x}{x position of the p-value. Should be used only when you want plot the
p-value as text (without brackets).}

\item{size, label.size}{size of label text.}

\item{p.digits}{integer indicating the number of digits used to
format recognized \strong{numeric} p-value label columns (\code{label} is one
of \code{"p"}, \code{"p.adj"}, \code{"p.value"}, \code{"p.val"},
\code{"pval"}, \code{"padj"}). Default is 3. Set to \code{NULL} to print the
raw value without rounding (this also disables \code{p.format.style} and the
related formatting arguments). All other labels are left unchanged: other
numeric columns (e.g. \code{"statistic"}, \code{"n"}, effect sizes),
significance symbols (\code{"p.signif"}, \code{"p.adj.signif"}),
already-formatted strings, and \code{\link[glue]{glue}} expressions. A
p-named column whose values fall outside \code{[0, 1]} is also left as-is.
Uses the same formatting engine (\code{\link{format_p_value}()}) as
\code{\link{stat_anova_test}()} for consistency across layers. The style of
this formatting is further controlled by \code{p.format.style},
\code{p.leading.zero}, \code{p.min.threshold} and \code{p.decimal.mark}
(same arguments as \code{\link{stat_compare_means}()}).}

\item{p.format.style}{character string specifying the p-value formatting style
applied to recognized numeric p-value label columns (see \code{p.digits}).
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS). See
\code{\link{list_p_format_styles}} for details. Default is \code{"default"},
which leaves the rendered p-value labels unchanged.}

\item{p.leading.zero}{logical indicating whether to include the leading zero
before the decimal point (e.g., "0.05" vs ".05"). If provided, overrides the
style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display
exactly. Values below this threshold are shown as "< threshold" (e.g.
\code{p.min.threshold = 0.001} renders very small p-values as "< 0.001"). If
provided, overrides the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If
\code{NULL}, uses \code{getOption("OutDec")}.}

\item{bracket.size}{Width of the lines of the bracket.}

\item{bracket.nudge.y}{Vertical adjustment to nudge brackets by. Useful to
move up or move down the bracket. If positive value, brackets will be moved
up; if negative value, brackets are moved down.}

\item{bracket.shorten}{a small numeric value in [0-1] for shortening the with
of bracket.}

\item{color}{text and line color. Can be variable name in the data for coloring by groups.}

\item{linetype}{linetype. Can be variable name in the data for changing linetype by groups.}

\item{tip.length}{numeric vector with the fraction that the bracket tips go
down to indicate the precise column. Interpreted relative to the reference set
by \code{tip.length.ref}. Default is 0.03.}

\item{tip.length.ref}{character string specifying what \code{tip.length} is a
fraction of. Either \code{"data"} (default): fraction of the trained data
range (tips scale with the data; existing plots unchanged); or \code{"axis"}:
fraction of the y-axis range (\code{ylim}/\code{scale_y_*}), giving visually
constant tip lengths across plots with different scales (#362).}

\item{remove.bracket}{logical, if \code{TRUE}, brackets are removed from the
plot. Considered only in the situation, where comparisons are performed
against reference group or against "all".}

\item{step.increase}{numeric vector with the increase in fraction of total
height for every additional comparison to minimize overlap.}

\item{step.group.by}{a variable name for grouping brackets before adding
step.increase. Useful to group bracket by facet panel.}

\item{hide.ns}{can be logical value or a character vector.
\itemize{
\item Case when logical value. If TRUE, hide ns symbol when displaying
 significance levels. Filter is done by checking the column
 \code{p.adj.signif}, \code{p.signif}, \code{p.adj} and \code{p}.
 \item Case when character value. Possible values are "p" or "p.adj", for filtering out non significant.
 }}

\item{vjust}{move the text up or down relative to the bracket. Can be also a
column name available in the data.}

\item{coord.flip}{logical. If \code{TRUE}, flip x and y coordinates so that
horizontal becomes vertical, and vertical, horizontal. When adding the
p-values to a horizontal ggplot (generated using
\code{\link[ggplot2]{coord_flip}()}), you need to specify the option
\code{coord.flip = TRUE}.}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function.}

\item{inherit.aes}{If \code{FALSE} (the default), overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{stat_pvalue_manual()}. Most layers should use
\code{TRUE} (the \pkg{ggplot2} default), but \code{stat_pvalue_manual()} uses
its own data with different column names than the parent plot data, so
\code{FALSE} avoids conflicts between parent plot aesthetics (like \code{fill} or
\code{color}) and the p-value annotation data.}

\item{...}{other arguments passed to the function \code{geom_bracket()} or
\code{geom_text()}}
}
\description{
Add p-values manually to a ggplot, such as box plots, dot plots
 and stripcharts. Frequently asked questions are available on \href{https://www.datanovia.com/en/blog/tag/ggpubr/}{Datanovia ggpubr FAQ page}, for example:
 \itemize{
 \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-basic-ggplots/}{How to Add P-Values onto Basic GGPlots}
 \item \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-adjusted-p-values-to-a-multi-panel-ggplot/}{How to Add Adjusted P-values to a Multi-Panel GGPlot}
 \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-to-ggplot-facets/}{How to Add P-values to GGPlot Facets}
 \item \href{https://www.datanovia.com/en/blog/ggpubr-how-to-add-p-values-generated-elsewhere-to-a-ggplot/}{How to Add P-Values Generated Elsewhere to a GGPlot}
 \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-a-grouped-ggplot-using-the-ggpubr-r-package/}{How to Add P-Values onto a Grouped GGPlot using the ggpubr R Package}
 \item \href{https://www.datanovia.com/en/blog/how-to-create-stacked-bar-plots-with-error-bars-and-p-values/}{How to Create Stacked Bar Plots with Error Bars and P-values}
 \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-onto-horizontal-ggplots/}{How to Add P-Values onto Horizontal GGPlots}
 }
}
\examples{

# T-test
stat.test <- compare_means(
  len ~ dose,
  data = ToothGrowth,
  method = "t.test"
)
stat.test

# Create a simple box plot
p <- ggboxplot(ToothGrowth, x = "dose", y = "len")
p

# Perform a t-test between groups
stat.test <- compare_means(
  len ~ dose,
  data = ToothGrowth,
  method = "t.test"
)
stat.test

# Add p-values manually from stat.test data
# First specify the y.position of each comparison
stat.test <- stat.test \%>\%
  mutate(y.position = c(29, 35, 39))
p + stat_pvalue_manual(stat.test, label = "p.adj")

# Customize the label with glue expression
# (https://github.com/tidyverse/glue)
p + stat_pvalue_manual(stat.test, label = "p = {p.adj}")


# Grouped bar plots
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
ToothGrowth$dose <- as.factor(ToothGrowth$dose)
# Comparisons against reference
stat.test <- compare_means(
  len ~ dose,
  data = ToothGrowth, group.by = "supp",
  method = "t.test", ref.group = "0.5"
)
stat.test
# Plot
bp <- ggbarplot(ToothGrowth,
  x = "supp", y = "len",
  fill = "dose", palette = "jco",
  add = "mean_sd", add.params = list(group = "dose"),
  position = position_dodge(0.8)
)
bp + stat_pvalue_manual(
  stat.test,
  x = "supp", y.position = 33,
  label = "p.signif",
  position = position_dodge(0.8)
)

}
\seealso{
\code{\link{stat_compare_means}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggarrange.R
\name{ggarrange}
\alias{ggarrange}
\title{Arrange Multiple ggplots}
\usage{
ggarrange(
  ...,
  plotlist = NULL,
  ncol = NULL,
  nrow = NULL,
  labels = NULL,
  label.x = 0,
  label.y = 1,
  hjust = -0.5,
  vjust = 1.5,
  font.label = list(size = 14, color = "black", face = "bold", family = NULL),
  align = c("none", "h", "v", "hv"),
  widths = 1,
  heights = 1,
  byrow = TRUE,
  legend = NULL,
  common.legend = FALSE,
  legend.grob = NULL,
  spacing = 0
)
}
\arguments{
\item{...}{list of plots to be arranged into the grid. The plots can be
either ggplot2 plot objects or arbitrary gtables.}

\item{plotlist}{(optional) list of plots to display.}

\item{ncol}{(optional) number of columns in the plot grid.}

\item{nrow}{(optional) number of rows in the plot grid.}

\item{labels}{(optional) list of labels to be added to the plots. You can
also set labels="AUTO" to auto-generate upper-case labels or labels="auto"
to auto-generate lower-case labels.}

\item{label.x}{(optional) Single value or vector of x positions for plot
labels, relative to each subplot. Defaults to 0 for all labels. (Each label
is placed all the way to the left of each plot.)}

\item{label.y}{(optional) Single value or vector of y positions for plot
labels, relative to each subplot. Defaults to 1 for all labels. (Each label
is placed all the way to the top of each plot.)}

\item{hjust}{Adjusts the horizontal position of each label. More negative values move the label further
to the right on the plot canvas. Can be a single value (applied to all labels) or a vector of values
(one for each label). Default is -0.5.}

\item{vjust}{Adjusts the vertical position of each label. More positive values move the label further
down on the plot canvas. Can be a single value (applied to all labels) or a vector of values
(one for each label). Default is 1.5.}

\item{font.label}{a list of arguments for customizing labels. Allowed values
are the combination of the following elements: size (e.g.: 14), face (e.g.:
"plain", "bold", "italic", "bold.italic"), color (e.g.: "red") and family.
For example font.label = list(size = 14, face = "bold", color ="red").}

\item{align}{(optional) Specifies whether graphs in the grid should be horizontally ("h") or
vertically ("v") aligned. Options are "none" (default), "hv" (align in both directions), "h", and "v".}

\item{widths}{(optional) numerical vector of relative columns widths. For
example, in a two-column grid, widths = c(2, 1) would make the first column
twice as wide as the second column.}

\item{heights}{same as \code{widths} but for column heights.}

\item{byrow}{logical. If \code{TRUE} (default), the plots are filled into the
grid by row; set to \code{FALSE} to fill by column. Passed to
\code{\link[cowplot]{plot_grid}()}.}

\item{legend}{character specifying legend position. Allowed values are one of
c("top", "bottom", "left", "right", "none"). To remove the legend use
legend = "none".}

\item{common.legend}{logical value, or one or several plot indices. Default is FALSE. If
\code{TRUE}, a single shared legend is used for all the arranged plots.
Note that this legend is \strong{not} merged or validated across plots: it
is simply the legend of the \emph{first} plot, and the other legends are
dropped. It is therefore only correct when every plot shares the same scale
(same groups/levels, order and color range). If the first plot's legend is
not representative - for example a group is missing in the first plot, or a
continuous color scale spans a different range - the shared legend will
misrepresent the other plots. In that case you can: (i) give the plots a
consistent scale yourself (e.g. \code{scale_fill_manual(limits = ...)} or
\code{scale_color_continuous(limits = ...)}) so a single legend is valid,
and/or (ii) choose which plot's legend is shown by passing that plot's index,
e.g. \code{common.legend = 2} to use the second plot's legend (equivalent to
\code{legend.grob = get_legend(plots[[2]])}). You can also pass several indices,
e.g. \code{common.legend = c(1, 2)}, to keep and combine the legends of those
plots into a single shared block (side by side for \code{legend = "top"}/
\code{"bottom"}, stacked for \code{"left"}/\code{"right"}) - useful when the
plots genuinely need different legends. Note that (ii) only changes
\emph{which} legend is displayed; it does not re-map the other plots' color
scales, so for the legend keys to match every panel you still need a
consistent scale as in (i). When the plots genuinely cannot be described by a
single legend (e.g. a discrete fill in one plot and a continuous color bar in
another), use \code{common.legend = FALSE} to keep a separate legend per plot.}

\item{legend.grob}{a legend grob as returned by the function
\code{\link{get_legend}()}. If provided, it will be used as the common
legend.}

\item{spacing}{numeric value giving the margin, in text-line units, set
uniformly around each plot to increase the gap between the arranged plots.
Default is 0, which leaves each plot's own margins untouched (existing
arrangements are unchanged). A positive value sets a uniform margin of that
many lines around every plot, replacing the plots' default margin; e.g.
\code{spacing = 1} puts a one-line margin around each plot.}
}
\value{
an object of class \code{ggarrange}, which is a ggplot or a
  list of ggplots.
}
\description{
Arrange multiple ggplots on the same page. Wrapper around
  \code{\link[cowplot]{plot_grid}()}. Can arrange multiple ggplots over
  multiple pages, compared to the standard
  \code{\link[cowplot]{plot_grid}()}. Can also create a common unique legend
  for multiple plots.
}
\examples{
data("ToothGrowth")
df <- ToothGrowth
df$dose <- as.factor(df$dose)

# Create some plots
# ::::::::::::::::::::::::::::::::::::::::::::::::::
# Box plot
bxp <- ggboxplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
# Dot plot
dp <- ggdotplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
# Density plot
dens <- ggdensity(df, x = "len", fill = "dose", palette = "jco")

# Arrange
# ::::::::::::::::::::::::::::::::::::::::::::::::::
ggarrange(bxp, dp, dens, ncol = 2, nrow = 2)
# Use a common legend for multiple plots
ggarrange(bxp, dp, common.legend = TRUE)

}
\seealso{
\code{\link{annotate_figure}()}
}
\author{
Laszlo Erdey \email{erdey.laszlo@econ.unideb.hu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\name{format_p_value}
\alias{format_p_value}
\title{Format P-Values According to Style or Custom Settings}
\usage{
format_p_value(
  p,
  style = "default",
  digits = NULL,
  leading.zero = NULL,
  min.threshold = NULL,
  decimal.mark = NULL,
  use.scientific = NULL
)
}
\arguments{
\item{p}{Numeric vector of p-values to format.}

\item{style}{Character string specifying the formatting style. One of:
\itemize{
  \item \code{"default"}: Current behavior with scientific notation (backward compatible)
  \item \code{"apa"}: APA Style - no leading zero, 3 decimals, "< .001" threshold
  \item \code{"nejm"}: NEJM Style - leading zero, 3 decimals, "< 0.001" threshold
  \item \code{"lancet"}: Lancet Style - leading zero, 4 decimals, "< 0.0001" threshold
  \item \code{"ama"}: AMA Style - no leading zero, 3 decimals, "< .001" threshold
  \item \code{"graphpad"}: GraphPad Style - leading zero, 4 decimals, "< 0.0001" threshold
  \item \code{"scientific"}: Scientific notation for genomics/GWAS studies
}}

\item{digits}{Integer specifying the number of decimal places. If provided,
overrides the style default.}

\item{leading.zero}{Logical indicating whether to include leading zero before
decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.}

\item{min.threshold}{Numeric specifying the minimum p-value to display exactly.
Values below this threshold are shown as "< threshold" (e.g., "< 0.001").
If NULL, the selected style's default threshold is used; styles without a
threshold show exact values. If provided, overrides the style default.
Must be a single positive finite number.}

\item{decimal.mark}{Character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{use.scientific}{Logical indicating whether to force scientific notation.
If NULL, uses the style default.}
}
\value{
Character vector of formatted p-values.
}
\description{
Formats p-values according to major scientific publication standards
  (APA, AMA, NEJM, Lancet, etc.) or custom user-defined settings. This function
  provides flexible control over decimal places, leading zeros, and threshold
  notation for very small p-values.
}
\details{
P-value formatting conventions vary across scientific disciplines and journals:

\strong{APA Style} (Psychology, Social Sciences):
\itemize{
  \item No leading zero (write ".05" not "0.05")
  \item 3 decimal places
  \item Report as "p < .001" for very small values
}

\strong{NEJM/Medical Journals}:
\itemize{
  \item Leading zero (write "0.05" not ".05")
  \item 3 decimal places
  \item P < 0.001: report as "< 0.001"
}

\strong{Scientific Notation} (GWAS, Genomics):
\itemize{
  \item Used when very small p-values are meaningful (e.g., 5e-8 threshold)
  \item Appropriate for high-dimensional data analyses
}
}
\examples{
# Test p-values
p_vals <- c(0.76404, 0.0432, 0.0043, 0.00018, 1.7e-11)

# Different styles
format_p_value(p_vals, style = "default")
format_p_value(p_vals, style = "apa")
format_p_value(p_vals, style = "nejm")
format_p_value(p_vals, style = "lancet")

# Custom formatting
format_p_value(p_vals, digits = 2, leading.zero = FALSE, min.threshold = 0.01)

# Override style defaults
format_p_value(p_vals, style = "nejm", digits = 4)

}
\seealso{
\code{\link{get_p_format_style}}, \code{\link{list_p_format_styles}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotate_axis_text.R
\name{rotate_axis_text}
\alias{rotate_axis_text}
\alias{rotate_x_text}
\alias{rotate_y_text}
\title{Rotate Axes Text}
\usage{
rotate_x_text(angle = 90, hjust = NULL, vjust = NULL, ...)

rotate_y_text(angle = 90, hjust = NULL, vjust = NULL, ...)
}
\arguments{
\item{angle}{numeric value specifying the rotation angle. Default is 90 for vertical x-axis text.}

\item{hjust}{horizontal justification (in [0, 1]).}

\item{vjust}{vertical justification (in [0, 1]).}

\item{...}{other arguments to pass to the function
\code{\link[ggplot2:element]{element_text}()}.}
}
\description{
Rotate the x-axis text (tick mark labels).
\itemize{
\item \code{rotate_x_text}(): Rotate x axis text.
\item \code{rotate_y_text}(): Rotate y axis text.
}
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
p <- ggboxplot(df, x = "dose", y = "len")
p
# Vertical x axis text
p + rotate_x_text()
# Set rotation angle to 45
p + rotate_x_text(45)
p + rotate_y_text(45)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text_grob.R
\name{text_grob}
\alias{text_grob}
\title{Create a Text Graphical object}
\usage{
text_grob(
  label,
  just = "centre",
  hjust = NULL,
  vjust = NULL,
  rot = 0,
  color = "black",
  face = "plain",
  size = NULL,
  lineheight = NULL,
  family = NULL,
  ...
)
}
\arguments{
\item{label}{A character or \link{expression} vector.  Other
    objects are coerced by \code{\link{as.graphicsAnnot}}.}

\item{just}{The justification of the text
    relative to its (x, y) location.  If there are two values, the first
    value specifies horizontal justification and the second value specifies
    vertical justification.  Possible string values are: \code{"left"},
    \code{"right"}, \code{"centre"}, \code{"center"}, \code{"bottom"},
    and \code{"top"}.  For numeric values, 0 means left (bottom) alignment
    and 1 means right (top) alignment.
  }

\item{hjust}{A numeric vector specifying horizontal justification.
    If specified, overrides the \code{just} setting.}

\item{vjust}{A numeric vector specifying vertical justification.
    If specified, overrides the \code{just} setting.}

\item{rot}{The angle to rotate the text.}

\item{color}{text font color.}

\item{face}{font face. Allowed values include one of \code{"plain", "bold",
"italic", "bold.italic"}.}

\item{size}{font size (e.g.: size = 12)}

\item{lineheight}{line height (e.g.: lineheight = 2).}

\item{family}{font family.}

\item{...}{other arguments passed to \link[grid:grid.text]{textGrob}.}
}
\value{
a text grob.
}
\description{
Create easily a customized text grob (graphical object). Wrapper around
  \code{\link[grid:grid.text]{textGrob}}.
}
\examples{
text <- paste("iris data set gives the measurements in cm",
  "of the variables sepal length and width",
  "and petal length and width, respectively,",
  "for 50 flowers from each of 3 species of iris.",
  "The species are Iris setosa, versicolor, and virginica.",
  sep = "\n"
)

# Create a text grob
tgrob <- text_grob(text, face = "italic", color = "steelblue")
# Draw the text
as_ggplot(tgrob)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggadjust_pvalue.R
\name{ggadjust_pvalue}
\alias{ggadjust_pvalue}
\title{Adjust p-values Displayed on a GGPlot}
\usage{
ggadjust_pvalue(
  p,
  layer = NULL,
  p.adjust.method = "holm",
  label = "p.adj",
  hide.ns = NULL,
  symnum.args = list(),
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  output = c("plot", "stat_test")
)
}
\arguments{
\item{p}{a ggplot}

\item{layer}{An integer indicating the statistical layer rank in the ggplot
(in the order added to the plot).}

\item{p.adjust.method}{method for adjusting p values (see
\code{\link[stats]{p.adjust}}).  Has impact only in a situation, where
multiple pairwise tests are performed; or when there are multiple grouping
variables. Ignored when the specified method is \code{"tukey_hsd"} or
\code{"games_howell_test"} because they come with internal p adjustment
method. Allowed values include "holm", "hochberg", "hommel", "bonferroni",
"BH", "BY", "fdr", "none". If you don't want to adjust the p value (not
recommended), use p.adjust.method = "none".}

\item{label}{character string specifying label. Can be: \itemize{ \item the
column containing the label (e.g.: \code{label = "p"} or \code{label =
"p.adj"}), where \code{p} is the p-value. Other possible values are
\code{"p.signif", "p.adj.signif", "p.format", "p.format.signif", "p.adj.format"}.
\item an
expression that can be formatted by the \code{\link[glue]{glue}()} package.
For example, when specifying \code{label = "Wilcoxon, p = \{p\}"}, the
expression \{p\} will be replaced by its value. \item a combination of
plotmath expressions and glue expressions. You may want some of the
statistical parameter in italic; for example:\code{label = "Wilcoxon,
italic(p)= {p}"}}.}

\item{hide.ns}{can be logical value (\code{TRUE} or \code{FALSE}) or a character vector (\code{"p.adj"} or \code{"p"}).}

\item{symnum.args}{a list of arguments to pass to the function
 \code{\link[stats]{symnum}} for symbolic number coding of p-values. For
 example, \code{symnum.args = list(cutpoints = c(0, 0.0001, 0.001, 0.01,
 0.05, Inf), symbols = c("****", "***", "**", "*",  "ns"))}.

 In other words, we use the following convention for symbols indicating
 statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
 \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001
 \item \code{****}:  p <= 0.0001 }

 Note: If \code{symnum.args} is provided, it takes precedence over
 \code{signif.cutoffs}.}

\item{p.format.style}{character string specifying the p-value formatting style.
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
See \code{\link{list_p_format_styles}} for details.}

\item{p.digits}{integer specifying the number of decimal places for p-values.
If provided, overrides the style default.}

\item{p.leading.zero}{logical indicating whether to include leading zero before
decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display exactly.
Values below this threshold are shown as "< threshold". If provided, overrides
the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{signif.cutoffs}{numeric vector of p-value cutoffs in descending order
for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
Default is NULL, which uses the package defaults.}

\item{signif.symbols}{character vector of symbols corresponding to
\code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
(and "****" if \code{use.four.stars = TRUE}).}

\item{ns.symbol}{character string for non-significant results. Default is "ns".
Use "" (empty string) to show nothing.}

\item{use.four.stars}{logical. If TRUE, allows four stars (****) for the most
significant level. Default is FALSE.}

\item{output}{character. Possible values are one of \code{c("plot",
"stat_test")}. Default is "plot".}
}
\description{
Adjust p-values produced by \code{\link{geom_pwc}()} on a ggplot.
 This is mainly useful when using facet, where p-values are generally
 computed and adjusted by panel without taking into account the other panels.
 In this case, one might want to adjust after the p-values of all panels together.
}
\examples{
# Data preparation
# :::::::::::::::::::::::::::::::::::::::
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Add a random grouping variable
df$group <- factor(rep(c("grp1", "grp2"), 30))
head(df, 3)

# Boxplot: Two groups by panel
# :::::::::::::::::::::::::::::::::::::::
# Create a box plot
bxp <- ggboxplot(
  df,
  x = "supp", y = "len", fill = "#00AFBB",
  facet.by = "dose"
)
# Make facet and add p-values
bxp <- bxp + geom_pwc(method = "t_test")
bxp
# Adjust all p-values together after
ggadjust_pvalue(
  bxp,
  p.adjust.method = "bonferroni",
  label = "{p.adj.format}{p.adj.signif}", hide.ns = TRUE
)


# Boxplot: Three groups by panel
# :::::::::::::::::::::::::::::::::::::::
# Create a box plot
bxp <- ggboxplot(
  df,
  x = "dose", y = "len", fill = "#00AFBB",
  facet.by = "supp"
)
# Make facet and add p-values
bxp <- bxp + geom_pwc(method = "t_test")
bxp
# Adjust all p-values together after
ggadjust_pvalue(
  bxp,
  p.adjust.method = "bonferroni",
  label = "{p.adj.format}{p.adj.signif}"
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggerrorplot.R
\name{ggerrorplot}
\alias{ggerrorplot}
\title{Visualizing Error}
\usage{
ggerrorplot(
  data,
  x,
  y,
  desc_stat = "mean_se",
  numeric.x.axis = FALSE,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = "white",
  palette = NULL,
  size = NULL,
  width = NULL,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  select = NULL,
  remove = NULL,
  order = NULL,
  add = "none",
  add.params = list(),
  error.plot = "pointrange",
  ci = 0.95,
  position = position_dodge(),
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x, y}{x and y variables for drawing.}

\item{desc_stat}{descriptive statistics to be used for visualizing errors. Default value is "mean_se".
Allowed values are one of , "mean", "mean_se", "mean_sd", "mean_ci", "mean_range",
"median", "median_iqr", "median_hilow", "median_q1q3", "median_mad", "median_range"; see \code{\link{desc_statby}} for more details.}

\item{numeric.x.axis}{logical. If TRUE, x axis will be treated as numeric. Default is FALSE.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color, fill}{outline and fill colors.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{width}{numeric value between 0 and 1 specifying the width of the plot elements.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items. Considered only when x axis is a factor variable.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{ci}{the percent range of the confidence interval (default is 0.95).}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function (e.g. \code{position_dodge(0.8)}).
Used to control the spacing between grouped elements.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to be passed to ggpar().}
}
\description{
Visualizing error.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{

# Data: ToothGrowth data set we'll be used.
df <- ToothGrowth
head(df, 10)

# Plot mean_se
ggerrorplot(df, x = "dose", y = "len")


# Change desc_stat to mean_sd
# (other values include: mean_sd, mean_ci, median_iqr, ....)
# Add labels
ggerrorplot(df,
  x = "dose", y = "len",
  desc_stat = "mean_sd"
)

# Change error.plot to "errorbar" and add mean point
# Visualize the mean of each group
ggerrorplot(df,
  x = "dose", y = "len",
  add = "mean", error.plot = "errorbar"
)

# Horizontal plot
ggerrorplot(df,
  x = "dose", y = "len",
  add = "mean", error.plot = "errorbar",
  orientation = "horizontal"
)


# Change error.plot to "crossbar"
ggerrorplot(df,
  x = "dose", y = "len",
  error.plot = "crossbar", width = 0.5
)


# Add jitter points and errors (mean_se)
ggerrorplot(df,
  x = "dose", y = "len",
  add = "jitter"
)

# Add dot and errors (mean_se)
ggerrorplot(df,
  x = "dose", y = "len",
  add = "dotplot"
)

# Multiple groups with error bars and jitter point
ggerrorplot(df,
  x = "dose", y = "len",
  color = "supp", palette = "Paired",
  error.plot = "pointrange",
  position = position_dodge(0.5)
)
#

}
\seealso{
\code{\link{ggpar}}, \code{\link{ggline}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdensity.R
\name{ggdensity}
\alias{ggdensity}
\title{Density plot}
\usage{
ggdensity(
  data,
  x,
  y = "density",
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = NA,
  palette = NULL,
  size = NULL,
  linewidth = NULL,
  linetype = "solid",
  alpha = 0.5,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  add = c("none", "mean", "median"),
  add.params = list(linetype = "dashed"),
  rug = FALSE,
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{variable to be drawn.}

\item{y}{one of "density" or "count".}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color, fill}{density line color and fill color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{linewidth}{numeric value specifying line width. This is the preferred
parameter for ggplot2 3.4.0+. If \code{size} is also specified, \code{linewidth}
takes precedence.}

\item{linetype}{line type. See \code{\link{show_line_types}}.}

\item{alpha}{numeric value specifying fill color transparency. Value should
be in [0, 1], where 0 is full transparency and 1 is no transparency.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{add}{allowed values are one of "mean" or "median" (for adding mean or
median line, respectively).}

\item{add.params}{parameters (color, size, linetype) for the argument 'add';
e.g.: add.params = list(color = "red").}

\item{rug}{logical value. If TRUE, add marginal rug.}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variables values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to
\code{\link[ggplot2]{geom_density}} and \code{\link{ggpar}}.}
}
\description{
Create a density plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Create some data format
set.seed(1234)
wdata <- data.frame(
  sex = factor(rep(c("F", "M"), each = 200)),
  weight = c(rnorm(200, 55), rnorm(200, 58))
)

head(wdata, 4)

# Basic density plot
# Add mean line and marginal rug
ggdensity(wdata,
  x = "weight", fill = "lightgray",
  add = "mean", rug = TRUE
)

# Change outline colors by groups ("sex")
# Use custom palette
ggdensity(wdata,
  x = "weight",
  add = "mean", rug = TRUE,
  color = "sex", palette = c("#00AFBB", "#E7B800")
)


# Change outline and fill colors by groups ("sex")
# Use custom palette
ggdensity(wdata,
  x = "weight",
  add = "mean", rug = TRUE,
  color = "sex", fill = "sex",
  palette = c("#00AFBB", "#E7B800")
)

}
\seealso{
\code{\link{gghistogram}} and \code{\link{ggpar}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_breaks.R
\name{get_breaks}
\alias{get_breaks}
\title{Easy Break Creation for Numeric Axes}
\usage{
get_breaks(n = NULL, by = NULL, from = NULL, to = NULL)
}
\arguments{
\item{n}{number of breaks.}

\item{by}{number: the step between breaks.}

\item{from}{the starting value of breaks. By default, 0 is used for positive
variables}

\item{to}{the end values of breaks. This corresponds generally to the maximum
limit of the axis.}
}
\value{
a break function
}
\description{
Creates breaks for numeric axes to be used in the functions
  \code{\link[ggplot2:scale_continuous]{scale_x_continuous}()} and
  \code{\link[ggplot2:scale_continuous]{scale_y_continuous}()}. Can be used to increase the
  number of x and y ticks by specifying the option \code{n}. It's also
  possible to control axis breaks by specifying a step between ticks.  For
  example, if \code{by = 5}, a tick mark is shown on every 5.
}
\examples{

# Generate 5 breaks for a variable x
get_breaks(n = 5)(x = 1:100)

# Generate breaks using an increasing step
get_breaks(by = 10)(x = 1:100)

# Combine with ggplot scale_xx functions
library(ggplot2)

# Create a basic plot
p <- ggscatter(mtcars, x = "wt", y = "mpg")
p

# Increase the number of ticks
p +
  scale_x_continuous(breaks = get_breaks(n = 10)) +
  scale_y_continuous(breaks = get_breaks(n = 10))

# Set ticks according to a specific step, starting from 0
p + scale_x_continuous(
  breaks = get_breaks(by = 1.5, from = 0),
  limits = c(0, 6)
) +
  scale_y_continuous(
    breaks = get_breaks(by = 10, from = 0),
    limits = c(0, 40)
  )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_regline_equation.R
\name{stat_regline_equation}
\alias{stat_regline_equation}
\title{Add Regression Line Equation and R-Square to a GGPLOT.}
\usage{
stat_regline_equation(
  mapping = NULL,
  data = NULL,
  formula = y ~ x,
  label.x.npc = "left",
  label.y.npc = "top",
  label.x = NULL,
  label.y = NULL,
  output.type = "expression",
  geom = "text",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}} or
\code{\link[ggplot2:aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{formula}{a formula object}

\item{label.x.npc, label.y.npc}{can be \code{numeric} or \code{character}
 vector of the same length as the number of groups and/or panels. If too
 short they will be recycled. \itemize{ \item If \code{numeric}, value should
 be between 0 and 1. Coordinates to be used for positioning the label,
 expressed in "normalized parent coordinates". \item If \code{character},
 allowed values include: i) one of c('right', 'left', 'center', 'centre',
 'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
 'middle') for y-axis.}

 If too short they will be recycled.}

\item{label.x, label.y}{\code{numeric} Coordinates (in data units) to be used
for absolute positioning of the label. If too short they will be recycled.}

\item{output.type}{character One of "expression", "latex" or "text".}

\item{geom}{The geometric object to use display the data}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{na.rm}{If FALSE (the default), removes missing values with a warning. If
TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:borders]{borders()}}.}

\item{...}{other arguments to pass to \code{\link[ggplot2]{geom_text}} or
\code{\link[ggplot2:geom_text]{geom_label}}.}
}
\description{
Add regression line equation and R^2 to a ggplot. Regression
 model is fitted using the function \code{\link[stats]{lm}}.
}
\section{Computed variables}{

  \describe{ \item{x}{x position for left edge}
  \item{y}{y position near upper edge}
  \item{eq.label}{equation for the
  fitted polynomial as a character string to be parsed}
  \item{rr.label}{\eqn{R^2} of the fitted model as a character string to be parsed}
  \item{adj.rr.label}{Adjusted \eqn{R^2} of the fitted model as a character string
  to be parsed}
  \item{AIC.label}{AIC for the fitted model.}
  \item{BIC.label}{BIC for the fitted model.}
  \item{hjust}{Set to zero to override the default of the "text" geom.}}
}

\examples{

# Simple scatter plot with correlation coefficient and
# regression line
#::::::::::::::::::::::::::::::::::::::::::::::::::::
ggscatter(mtcars, x = "wt", y = "mpg", add = "reg.line") +
  stat_cor(label.x = 3, label.y = 34) +
  stat_regline_equation(label.x = 3, label.y = 32)


# Groupped scatter plot
#::::::::::::::::::::::::::::::::::::::::::::::::::::
ggscatter(
  iris, x = "Sepal.Length", y = "Sepal.Width",
  color = "Species", palette = "jco",
  add = "reg.line"
  ) +
  facet_wrap(~Species) +
  stat_cor(label.y = 4.4) +
  stat_regline_equation(label.y = 4.2)

# Polynomial equation
#::::::::::::::::::::::::::::::::::::::::::::::::::::

# Demo data
set.seed(4321)
x <- 1:100
y <- (x + x^2 + x^3) + rnorm(length(x), mean = 0, sd = mean(x^3) / 4)
my.data <- data.frame(x, y, group = c("A", "B"),
                      y2 = y * c(0.5,2), block = c("a", "a", "b", "b"))

# Fit polynomial regression line and add labels
formula <- y ~ poly(x, 3, raw = TRUE)
p <- ggplot(my.data, aes(x, y2, color = group)) +
  geom_point() +
  stat_smooth(aes(fill = group, color = group), method = "lm", formula = formula) +
  stat_regline_equation(
    aes(label =  paste(..eq.label.., ..adj.rr.label.., sep = "~~~~")),
    formula = formula
  ) +
  theme_bw()
ggpar(p, palette = "jco")

}
\references{
the source code of the function \code{stat_regline_equation()} is
 inspired from the code of the function \code{stat_poly_eq()} (in ggpmisc
 package).
}
\seealso{
\code{\link{ggscatter}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggadd.R
\name{ggadd}
\alias{ggadd}
\title{Add Summary Statistics or a Geom onto a ggplot}
\usage{
ggadd(
  p,
  add = NULL,
  color = "black",
  fill = "white",
  group = 1,
  width = 1,
  shape = 19,
  size = NULL,
  alpha = 1,
  jitter = 0.2,
  seed = 123,
  binwidth = NULL,
  dotsize = size,
  linetype = 1,
  linewidth = size,
  show.legend = NA,
  error.plot = "pointrange",
  ci = 0.95,
  outliers = TRUE,
  outlier.shape = 19,
  data = NULL,
  position = position_dodge(0.8),
  p_geom = ""
)
}
\arguments{
\item{p}{a ggplot}

\item{add}{character vector specifying other plot elements to be added.
Allowed values are one or the combination of: "none", "dotplot", "jitter",
"boxplot", "point", "mean", "mean_se", "mean_sd", "mean_ci", "mean_range",
"median", "median_iqr", "median_hilow", "median_q1q3", "median_mad",
"median_range".}

\item{color}{point or outline color.}

\item{fill}{fill color. Used only when \code{error.plot = "crossbar"}.}

\item{group}{grouping variable. Allowed values are 1 (for one group) or a
character vector specifying the name of the grouping variable. Used only for
adding statistical summary per group.}

\item{width}{numeric value between 0 and 1 specifying bar or box width.
Example width = 0.8. Used only when \code{error.plot} is one of
c("crossbar", "errorbar").}

\item{shape}{point shape. Allowed values can be displayed using the function
\code{\link{show_point_shapes}()}.}

\item{size}{numeric value in [0-1] specifying point and line size.}

\item{alpha}{numeric value specifying fill color transparency. Value should be
in [0, 1], where 0 is full transparency and 1 is no transparency.}

\item{jitter}{a numeric value specifying the amount of jittering. Used only
when \code{add} contains "jitter".}

\item{seed}{A random seed to make the jitter reproducible. Default is `123`. Useful if you need
to apply the same jitter twice, e.g., for a point and a corresponding label.
The random seed is reset after jittering. If `NA`, the
seed is initialized with a random value; this makes sure that two subsequent
calls start with a different seed. Use NULL to use the current random seed
and also avoid resetting (the behaviour of ggplot 2.2.1 and earlier).}

\item{binwidth}{numeric value specifying bin width. use value between 0 and 1
when you have a strong dense dotplot. For example binwidth = 0.2. Used only
when \code{add} contains "dotplot".}

\item{dotsize}{as \code{size} but applied only to dotplot.}

\item{linetype}{line type.}

\item{linewidth}{numeric value in [0-1] specifying line width. Used only when
\code{add} contains "line" or for adding error bars. Default is \code{size}.}

\item{show.legend}{logical. Should this layer be included in the legends? NA,
the default, includes if any aesthetics are mapped. \code{FALSE} never
includes, and TRUE always includes. It can also be a named logical vector to
finely select the aesthetics to display.}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
\code{c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange")}. Default value is "pointrange".}

\item{ci}{the percent range of the confidence interval (default is 0.95).}

\item{outliers}{logical. If TRUE (default), outliers are displayed in boxplots.
If FALSE, outliers are not displayed.}

\item{outlier.shape}{numeric value specifying the shape of outliers in boxplots.
Default is 19 (filled circle). if set to NA, outliers are not displayed.}

\item{data}{a \code{data.frame} to be displayed. If \code{NULL}, the default,
the data is inherited from the plot data as specified in the call to
\link[ggplot2]{ggplot}.}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function. Used to adjust position for multiple
groups.}

\item{p_geom}{the geometry of the main plot. Ex: p_geom = "geom_line". If
NULL, the geometry is extracted from p. Used only by \link{ggline}().}
}
\description{
Add summary statistics or a geometry onto a ggplot.
}
\examples{
# Basic violin plot
data("ToothGrowth")
p <- ggviolin(ToothGrowth, x = "dose", y = "len", add = "none")

# Add mean +/- SD and jitter points
p \%>\% ggadd(c("mean_sd", "jitter"), color = "dose")

# Add box plot
p \%>\% ggadd(c("boxplot", "jitter"), color = "dose")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggstripchart.R
\name{ggstripchart}
\alias{ggstripchart}
\title{Stripcharts}
\usage{
ggstripchart(
  data,
  x,
  y,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = "white",
  palette = NULL,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  shape = 19,
  size = NULL,
  select = NULL,
  remove = NULL,
  order = NULL,
  add = "mean_se",
  add.params = list(),
  error.plot = "pointrange",
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  jitter = 0.2,
  position = position_jitter(jitter, seed = 123),
  ggtheme = theme_pubr(),
  show.n = FALSE,
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{character string containing the name of x variable.}

\item{y}{character vector containing one or more variables to plot}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{outline color.}

\item{fill}{fill color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{shape}{point shape}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variables values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{jitter}{the amount of jitter.}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function. Used to adjust position for
multiple groups.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{show.n}{logical. If TRUE, displays the number of observations
(\code{"n = <count>"}) at the top of each group. Off by default. When the
groups are dodged (a \code{color}/\code{fill} grouping with a dodging
\code{position}), one count is shown per group; otherwise a single count is
shown per x-axis tick. Counts respect \code{select}/\code{remove} and are
computed per facet.}

\item{...}{other arguments to be passed to
\code{\link[ggplot2]{geom_jitter}}, \code{\link{ggpar}} and
\code{\link{facet}}.}
}
\description{
Create a stripchart, also known as one dimensional scatter
  plots. These plots are suitable compared to box plots when sample sizes are
  small.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot with summary statistics: mean_se
# +++++++++++++++++++++++++++
# Change point shapes by groups: "dose"
ggstripchart(df,
  x = "dose", y = "len",
  shape = "dose", size = 3,
  add = "mean_se"
)

# Use mean_sd
# Change error.plot to "crossbar"
ggstripchart(df,
  x = "dose", y = "len",
  shape = "dose", size = 3,
  add = "mean_sd", add.params = list(width = 0.5),
  error.plot = "crossbar"
)


# Add summary statistics
# ++++++++++++++++++++++++++

# Add box plot
ggstripchart(df,
  x = "dose", y = "len",
  shape = "dose", add = "boxplot"
)

# Add violin + mean_sd
ggstripchart(df,
  x = "dose", y = "len",
  shape = "dose", add = c("violin", "mean_sd")
)


# Change colors
# +++++++++++++++++++++++++++
# Change colors by groups: dose
# Use custom color palette
ggstripchart(df, "dose", "len",
  shape = "dose",
  color = "dose", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  add = "mean_sd"
)


# Plot with multiple groups
# +++++++++++++++++++++
# Change shape and color by a second group : "supp"
ggstripchart(df, "dose", "len",
  shape = "supp",
  color = "supp", palette = c("#00AFBB", "#E7B800")
)

# Adjust point position
ggstripchart(df, "dose", "len",
  shape = "supp",
  color = "supp", palette = c("#00AFBB", "#E7B800"),
  position = position_dodge(0.8)
)

# You can also use position_jitterdodge()
# but fill aesthetic is required
ggstripchart(df, "dose", "len",
  shape = "supp",
  color = "supp", palette = c("#00AFBB", "#E7B800"),
  position = position_jitterdodge()
)

# Add boxplot
ggstripchart(df, "dose", "len",
  shape = "supp",
  color = "supp", palette = c("#00AFBB", "#E7B800"),
  add = "boxplot", add.params = list(color = "black")
)

}
\seealso{
\code{\link{ggpar}}, \code{\link{ggviolin}}, \code{\link{ggdotplot}}
 and \code{\link{ggboxplot}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_aes.R
\name{create_aes}
\alias{create_aes}
\title{Create Aes Mapping from a List}
\usage{
create_aes(.list, parse = TRUE)
}
\arguments{
\item{.list}{a list of aesthetic arguments; for example .list = list(x = "dose", y = "len", color = "dose").}

\item{parse}{logical. If TRUE, parse the input as an expression.}
}
\description{
Create aes mapping to make programming easy with ggplot2.
}
\examples{
# Simple aes creation
create_aes(list(x = "Sepal.Length", y = "Petal.Length"))

# Parse an expression
x <- "log2(Sepal.Length)"
y <- "log2(Petal.Length)"
create_aes(list(x = x, y = y), parse = TRUE)

# Create a ggplot
mapping <- create_aes(list(x = x, y = y), parse = TRUE)
ggplot(iris, mapping) +
  geom_point()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rotate.R
\name{rotate}
\alias{rotate}
\title{Rotate a ggplot Horizontally}
\usage{
rotate(...)
}
\arguments{
\item{...}{other arguments to pass to \code{\link[ggplot2]{coord_flip}}.}
}
\description{
Rotate a ggplot to create horizontal plots. Wrapper around
 \code{\link[ggplot2]{coord_flip}}.
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
p <- ggboxplot(df,
  x = "dose", y = "len",
  color = "dose", palette = "jco"
)
p
# Create horizontal plots
p + rotate()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpubr_args.R
\name{ggpubr_args}
\alias{ggpubr_args}
\title{ggpubr General Arguments Description}
\arguments{
\item{data}{a data frame}

\item{x}{character string containing the name of x variable.}

\item{y}{character vector containing one or more variables to plot}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{outline color.}

\item{fill}{fill color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{linetype}{line types.}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_mad",
"median_range"; see ?desc_statby for more details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{font.label}{a list which can contain the combination of the following elements: the size
(e.g.: 14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and
the color (e.g.: "red") of labels. For example font.label = list(size = 14,
face = "bold", color ="red"). To specify only the size and the style, use font.label =
list(size = 14, face = "plain").}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab
= FALSE to hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}
}
\description{
ggpubr General Arguments Description
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggscatter.R
\name{ggscatter}
\alias{ggscatter}
\title{Scatter plot}
\usage{
ggscatter(
  data,
  x,
  y,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = "lightgray",
  palette = NULL,
  shape = 19,
  size = 2,
  point = TRUE,
  rug = FALSE,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  add = c("none", "reg.line", "loess"),
  add.params = list(),
  conf.int = FALSE,
  conf.int.level = 0.95,
  fullrange = FALSE,
  ellipse = FALSE,
  ellipse.level = 0.95,
  ellipse.type = "norm",
  ellipse.alpha = 0.1,
  ellipse.border.remove = FALSE,
  mean.point = FALSE,
  mean.point.size = ifelse(is.numeric(size), 2 * size, size),
  star.plot = FALSE,
  star.plot.lty = 1,
  star.plot.lwd = NULL,
  label = NULL,
  font.label = c(12, "plain"),
  font.family = "",
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  parse = FALSE,
  cor.coef = FALSE,
  cor.coeff.args = list(),
  cor.method = "pearson",
  cor.coef.coord = c(NULL, NULL),
  cor.coef.size = 4,
  ggp = NULL,
  show.legend.text = NA,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{x variables for drawing.}

\item{y}{y variables for drawing.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color, fill}{point colors.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{shape}{point shape. See \code{\link{show_point_shapes}}.}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{point}{logical value. If TRUE, show points.}

\item{rug}{logical value. If TRUE, add marginal rug.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{add}{allowed values are one of "none", "reg.line" (for adding linear
regression line) or "loess" (for adding local regression fitting).}

\item{add.params}{parameters (color, size, linetype) for the argument 'add';
e.g.: add.params = list(color = "red").}

\item{conf.int}{logical value. If TRUE, adds confidence interval.}

\item{conf.int.level}{Level controlling confidence region. Default is 95\%.
Used only when add != "none" and conf.int = TRUE.}

\item{fullrange}{should the fit span the full range of the plot, or just the
data. Used only when add != "none".}

\item{ellipse}{logical value. If TRUE, draws ellipses around points.}

\item{ellipse.level}{the size of the concentration ellipse in normal
probability.}

\item{ellipse.type}{Character specifying frame type. Possible values are
 \code{"convex"}, \code{"confidence"} or types supported by
 \code{\link[ggplot2]{stat_ellipse}()} including one of \code{c("t", "norm",
 "euclid")} for plotting concentration ellipses.

 \itemize{ \item \code{"convex"}: plot convex hull of a set o points. \item
 \code{"confidence"}: plot confidence ellipses around group mean points as
 \code{FactoMineR::coord.ellipse()}. \item \code{"t"}:
 assumes a multivariate t-distribution. \item \code{"norm"}: assumes a
 multivariate normal distribution. \item \code{"euclid"}: draws a circle with
 the radius equal to level, representing the euclidean distance from the
 center. This ellipse probably won't appear circular unless
 \code{\link[ggplot2]{coord_fixed}()} is applied.}}

\item{ellipse.alpha}{Alpha for ellipse specifying the transparency level of
fill color. Use alpha = 0 for no fill color.}

\item{ellipse.border.remove}{logical value. If TRUE, remove ellipse border lines.}

\item{mean.point}{logical value. If TRUE, group mean points are added to the
plot.}

\item{mean.point.size}{numeric value specifying the size of mean points.}

\item{star.plot}{logical value. If TRUE, a star plot is generated.}

\item{star.plot.lty, star.plot.lwd}{line type and line width (size) for star
plot, respectively.}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a vector of length 3 indicating respectively the size
(e.g.: 14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and
the color (e.g.: "red") of point labels. For example \emph{font.label =
c(14, "bold", "red")}. To specify only the size and the style, use
font.label = c(14, "plain").}

\item{font.family}{character vector specifying font family.}

\item{label.select}{character vector specifying some labels to show.}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{parse}{If \code{TRUE}, the labels will be parsed into expressions and
displayed as described in \code{?plotmath}.}

\item{cor.coef}{logical value. If TRUE, correlation coefficient with the
p-value will be added to the plot.}

\item{cor.coeff.args}{a list of arguments to pass to the function
\code{\link{stat_cor}} for customizing the displayed correlation
coefficients. For example: \code{cor.coeff.args = list(method = "pearson",
label.x.npc = "right", label.y.npc = "top")}.}

\item{cor.method}{method for computing correlation coefficient. Allowed
values are one of "pearson", "kendall", or "spearman".}

\item{cor.coef.coord}{numeric vector, of length 2, specifying the x and y
coordinates of the correlation coefficient. Default values are NULL.}

\item{cor.coef.size}{correlation coefficient text font size.}

\item{ggp}{a ggplot. If not NULL, points are added to an existing plot.}

\item{show.legend.text}{logical. Should text be included in the legends? NA,
the default, includes if any aesthetics are mapped. FALSE never includes,
and TRUE always includes.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to \code{\link[ggplot2]{geom_point}}
and \code{\link{ggpar}}.}
}
\description{
Create a scatter plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Load data
data("mtcars")
df <- mtcars
df$cyl <- as.factor(df$cyl)
head(df[, c("wt", "mpg", "cyl")], 3)

# Basic plot
# +++++++++++++++++++++++++++
ggscatter(df,
  x = "wt", y = "mpg",
  color = "black", shape = 21, size = 3, # Points color, shape and size
  add = "reg.line", # Add regressin line
  add.params = list(color = "blue", fill = "lightgray"), # Customize reg. line
  conf.int = TRUE, # Add confidence interval
  cor.coef = TRUE, # Add correlation coefficient. see ?stat_cor
  cor.coeff.args = list(method = "pearson", label.x = 3, label.sep = "\n")
)

# loess method: local regression fitting
ggscatter(df,
  x = "wt", y = "mpg",
  add = "loess", conf.int = TRUE
)


# Control point size by continuous variable values ("qsec")
ggscatter(df,
  x = "wt", y = "mpg",
  color = "#00AFBB", size = "qsec"
)


# Change colors
# +++++++++++++++++++++++++++
# Use custom color palette
# Add marginal rug
ggscatter(df,
  x = "wt", y = "mpg", color = "cyl",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)


# Add group ellipses and mean points
# Add stars
# +++++++++++++++++++
ggscatter(df,
  x = "wt", y = "mpg",
  color = "cyl", shape = "cyl",
  palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  ellipse = TRUE, mean.point = TRUE,
  star.plot = TRUE
)


# Textual annotation
# +++++++++++++++++
df$name <- rownames(df)
ggscatter(df,
  x = "wt", y = "mpg",
  color = "cyl", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  label = "name", repel = TRUE
)

}
\seealso{
\code{\link{stat_cor}}, \code{\link{stat_stars}}, \code{\link{stat_conf_ellipse}} and \code{\link{ggpar}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpar.R
\name{ggpar}
\alias{ggpar}
\title{Graphical parameters}
\usage{
ggpar(
  p,
  palette = NULL,
  gradient.cols = NULL,
  main = NULL,
  submain = NULL,
  caption = NULL,
  xlab = NULL,
  ylab = NULL,
  title = NULL,
  subtitle = NULL,
  font.main = NULL,
  font.submain = NULL,
  font.x = NULL,
  font.y = NULL,
  font.caption = NULL,
  font.title = NULL,
  font.subtitle = NULL,
  font.family = "",
  xlim = NULL,
  ylim = NULL,
  xscale = c("none", "log2", "log10", "sqrt"),
  yscale = c("none", "log2", "log10", "sqrt"),
  format.scale = FALSE,
  legend = NULL,
  legend.direction = NULL,
  legend.title = NULL,
  font.legend = NULL,
  ticks = TRUE,
  tickslab = TRUE,
  font.tickslab = NULL,
  font.xtickslab = font.tickslab,
  font.ytickslab = font.tickslab,
  x.text.angle = NULL,
  y.text.angle = NULL,
  xtickslab.rt = x.text.angle,
  ytickslab.rt = y.text.angle,
  xticks.by = NULL,
  yticks.by = NULL,
  rotate = FALSE,
  orientation = c("vertical", "horizontal", "reverse"),
  ggtheme = NULL,
  ...
)
}
\arguments{
\item{p}{an object of class ggplot or a list of ggplots}

\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector of length(groups); in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}

\item{gradient.cols}{vector of colors to use for n-colour gradient. Allowed
values include brewer and ggsci color palettes.}

\item{main}{plot main title.}

\item{submain, subtitle}{plot subtitle.}

\item{caption}{plot caption.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{title}{plot main title.}

\item{font.main, font.submain, font.caption, font.x, font.y}{a vector of length 3
indicating respectively the size (e.g.: 14), the style (e.g.: "plain",
"bold", "italic", "bold.italic") and the color (e.g.: "red") of main title,
subtitle, caption, xlab and ylab, respectively. For example \emph{font.x =
c(14, "bold", "red")}. Use font.x = 14, to change only font size; or use
font.x = "bold", to change only font face.}

\item{font.title, font.subtitle}{alias of font.submain and font.submain,
respectively.}

\item{font.family}{character vector specifying font family.}

\item{xlim, ylim}{a numeric vector of length 2, specifying  x and y axis
limits (minimum and maximum), respectively. e.g.: ylim = c(0, 50).}

\item{xscale, yscale}{x and y axis scale, respectively. Allowed values are one
of c("none", "log2", "log10", "sqrt"); e.g.: yscale="log2".}

\item{format.scale}{logical value. If TRUE, axis tick mark labels will be
formatted when xscale or yscale = "log2" or "log10".}

\item{legend}{character specifying legend position. Allowed values are one of
c("top", "bottom", "left", "right", "none"). To remove the legend use
legend = "none". Legend position can be also specified using a numeric
vector c(x, y); see details section.}

\item{legend.direction}{character specifying the layout of legend items.
Allowed values are "horizontal" or "vertical". Useful to override the
default direction, e.g. to stack items vertically with
\code{legend = "bottom"}.}

\item{legend.title}{legend title, e.g.: \code{legend.title = "Species"}. Can
be also a list, \code{legend.title = list(color = "Species", linetype = "Species",
shape = "Species")}.}

\item{font.legend}{legend text font style; e.g.: font.legend = c(10, "plain",
"black").}

\item{ticks}{logical value. Default is TRUE. If FALSE, hide axis tick marks.}

\item{tickslab}{logical value. Default is TRUE. If FALSE, hide axis tick
labels.}

\item{font.tickslab, font.xtickslab, font.ytickslab}{Font style (size, face,
color) for tick labels, e.g.: c(14, "bold", "red").}

\item{x.text.angle, y.text.angle}{Numeric value specifying the rotation angle
of x and y axis tick labels, respectively. Default value is NULL. For
vertical x axis texts use x.text.angle = 90.}

\item{xtickslab.rt, ytickslab.rt}{Same as x.text.angle and y.text.angle,
respectively. Will be deprecated in the near future.}

\item{xticks.by, yticks.by}{numeric value controlling x and y axis breaks,
respectively. For example, if yticks.by = 5, a tick mark is shown on every
5. Default value is NULL.}

\item{rotate}{logical value. If TRUE, rotate the graph by setting the plot
orientation to horizontal.}

\item{orientation}{change the orientation of the plot. Allowed values are one
of c( "vertical", "horizontal", "reverse"). Partial match is allowed.}

\item{ggtheme}{function, ggplot2 theme name. The default is set by each
function's \code{ggtheme} argument; see the function usage for the actual
default. Set \code{ggtheme = NULL} to skip applying a ggpubr theme, so the
plot keeps ggplot2 default theme or the theme set globally via
\code{theme_set()}.
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{not used}
}
\description{
Graphical parameters
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic box plot
# +++++++++++++++++++++++++++

p <- ggboxplot(df, x = "dose", y = "len")

# Change the plot orientation: horizontal
ggpar(p, orientation = "horiz")


# Change main title and axis labels
# ++++++++++++++++++++++++++++

ggpar(p,
  main = "Plot of length \n by dose",
  xlab = "Dose (mg)", ylab = "Length"
)

# Title font styles: 'plain', 'italic', 'bold', 'bold.italic'
ggpar(p,
  main = "Length by dose",
  font.main = c(14, "bold.italic", "red"),
  font.x = c(14, "bold", "#2E9FDF"),
  font.y = c(14, "bold", "#E7B800")
)

# Hide axis labels
ggpar(p, xlab = FALSE, ylab = FALSE)


# Change colors
# ++++++++++++++++++++++

# Change outline colors by groups: dose
p2 <- ggboxplot(df, "dose", "len", color = "dose")
p2

# Use custom color palette
ggpar(p2, palette = c("#00AFBB", "#E7B800", "#FC4E07"))

# Use brewer palette
ggpar(p2, palette = "Dark2")

# Use grey palette
ggpar(p2, palette = "grey")

# Use scientific journal palette from ggsci package
ggpar(p2, palette = "npg") # nature

# Axis ticks, limits, scales
# +++++++++++++++++++++++++

# Axis ticks labels and rotation
ggpar(p,
  font.tickslab = c(14, "bold", "#993333"),
  xtickslab.rt = 45, ytickslab.rt = 45
)
# Hide axis ticks and tick labels
ggpar(p, ticks = FALSE, tickslab = FALSE)

# Axis limits
ggpar(p, ylim = c(0, 50))

# Axis scale
ggpar(p, yscale = "log2")

# Format axis scale
ggpar(p, yscale = "log2", format.scale = TRUE)

# Legends
# ++++++++++++++++++
# Change legend position and title
ggpar(p2,
  legend = "right", legend.title = "Dose (mg)",
  font.legend = c(10, "bold", "red")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene_expression.R
\docType{data}
\name{gene_expression}
\alias{gene_expression}
\title{Gene Expression Data}
\format{
A data frame with 1305 rows and 5 columns. \describe{
 \item{\code{bcr_patient_barcode}}{sample ID} \item{\code{dataset}}{cancer type} \item{\code{GATA3}}{GATA3 gene expression}
 \item{\code{PTEN}}{PTEN gene expression}\item{\code{XBP1}}{XBP1 gene expression.}}
}
\usage{
data("gene_expression")
}
\description{
Gene expression data extracted from TCGA using the `RTCGA` and
 `RTCGA.mRNA` R packages. It contains the mRNA expression for 3 genes -
 GATA3, PTEN and XBP1- from 3 different datasets: Breast invasive carcinoma
 (BRCA), Ovarian serous cystadenocarcinoma (OV) and Lung squamous cell
 carcinoma (LUSC)
}
\examples{
data(gene_expression)

ggboxplot(gene_expression,
  x = "dataset",
  y = c("GATA3", "PTEN", "XBP1"),
  combine = TRUE,
  ylab = "Expression",
  color = "dataset", palette = "jco"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_mean.R
\name{stat_mean}
\alias{stat_mean}
\title{Draw group mean points}
\usage{
stat_mean(
  mapping = NULL,
  data = NULL,
  geom = "point",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{geom}{The geometric object to use to display the data for this layer.
When using a \verb{stat_*()} function to construct a layer, the \code{geom} argument
can be used to override the default coupling between stats and geoms. The
\code{geom} argument accepts the following:
\itemize{
\item A \code{Geom} ggproto subclass, for example \code{GeomPoint}.
\item A string naming the geom. To give the geom as a string, strip the
function name of the \code{geom_} prefix. For example, to use \code{geom_point()},
give the geom as \code{"point"}.
\item For more information and other ways to specify the geom, see the
\link[ggplot2:layer_geoms]{layer geom} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If FALSE (the default), removes missing values with a warning.
If TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{...}{other arguments to pass to \code{\link[ggplot2]{geom_point}}.}
}
\description{
Draw the mean point of each group.
}
\examples{
# Load data
data("mtcars")
df <- mtcars
df$cyl <- as.factor(df$cyl)

# Scatter plot with ellipses and group mean points
ggscatter(df,
  x = "wt", y = "mpg",
  color = "cyl", shape = "cyl", ellipse = TRUE
) +
  stat_mean(aes(color = cyl, shape = cyl), size = 4)

}
\seealso{
\code{\link{stat_conf_ellipse}}, \code{\link{stat_chull}} and
  \code{\link{ggscatter}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_anova_test.R
\name{stat_anova_test}
\alias{stat_anova_test}
\title{Add Anova Test P-values to a GGPlot}
\usage{
stat_anova_test(
  mapping = NULL,
  data = NULL,
  method = c("one_way", "one_way_repeated", "two_way", "two_way_repeated",
    "two_way_mixed"),
  wid = NULL,
  group.by = NULL,
  type = NULL,
  effect.size = "ges",
  error = NULL,
  correction = c("auto", "GG", "HF", "none"),
  label = "{method}, p = {p.format}",
  label.x.npc = "left",
  label.y.npc = "top",
  label.x = NULL,
  label.y = NULL,
  step.increase = 0.1,
  p.adjust.method = "holm",
  significance = list(),
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  geom = "text",
  position = "identity",
  na.rm = FALSE,
  show.legend = FALSE,
  inherit.aes = TRUE,
  parse = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{method}{ANOVA test methods. Possible values are one of
\code{c("one_way", "one_way_repeated", "two_way", "two_way_repeated",
"two_way_mixed")}.}

\item{wid}{(factor) column name containing individuals/subjects identifier.
Should be unique per individual. Required only for repeated measure tests
(\code{"one_way_repeated", "two_way_repeated", "friedman_test", etc}).}

\item{group.by}{(optional) character vector specifying the grouping variable;
it should be used only for grouped plots. Possible values are : \itemize{
\item \code{"x.var"}: Group by the x-axis variable and perform the test
between legend groups. In other words, the p-value is compute between legend
groups at each x position \item \code{"legend.var"}: Group by the legend
variable and perform the test between x-axis groups. In other words, the
test is performed between the x-groups for each legend level. }}

\item{type}{the type of sums of squares for ANOVA. Allowed values are either
1, 2 or 3. \code{type = 2} is the default because this will yield identical
ANOVA results as type = 1 when data are balanced but type = 2 will
additionally yield various assumption tests where appropriate. When the data
are unbalanced the \code{type = 3} is used by popular commercial software
including SPSS.}

\item{effect.size}{the effect size to compute and to show in the ANOVA
results. Allowed values can be either "ges" (generalized eta squared) or
"pes" (partial eta squared) or both. Default is "ges".}

\item{error}{(optional) for a linear model, an lm model object from which the
overall error sum of squares and degrees of freedom are to be calculated.
Read more in \code{\link[car]{Anova}()} documentation.}

\item{correction}{character. Used only in repeated measures ANOVA test to
specify which correction of the degrees of freedom should be reported for
the within-subject factors. Possible values are: \itemize{ \item{"GG"}:
applies Greenhouse-Geisser correction to all within-subjects factors even if
the assumption of sphericity is met (i.e., Mauchly's test is not
significant, p > 0.05). \item{"HF"}: applies Huynh-Feldt correction to all
within-subjects factors even if the assumption of sphericity is met,
\item{"none"}: returns the ANOVA table without any correction and
\item{"auto"}: apply automatically GG correction to only within-subjects
factors violating the sphericity assumption (i.e., Mauchly's test p-value is
significant, p <= 0.05). }}

\item{label}{character string specifying label. Can be: \itemize{ \item the
column containing the label (e.g.: \code{label = "p"} or \code{label =
"p.adj"}), where \code{p} is the p-value. Other possible values are
\code{"p.signif", "p.adj.signif", "p.format", "p.format.signif", "p.adj.format"}.
\item an
expression that can be formatted by the \code{\link[glue]{glue}()} package.
For example, when specifying \code{label = "Anova, p = \{p\}"}, the
expression \{p\} will be replaced by its value. \item a combination of
plotmath expressions and glue expressions. You may want some of the
statistical parameter in italic; for example:\code{label = "Anova, italic(p)
= {p}"}. \item a constant: \code{label = "as_italic"}: display statistical
parameters in italic; \code{label = "as_detailed"}: detailed plain text;
\code{label = "as_detailed_expression"} or \code{label =
"as_detailed_italic"}: detailed plotmath expression. Statistical parameters
will be displayed in italic.}.}

\item{label.x.npc, label.y.npc}{can be \code{numeric} or \code{character}
vector of the same length as the number of groups and/or panels. If too
short they will be recycled. \itemize{ \item If \code{numeric}, value should
be between 0 and 1. Coordinates to be used for positioning the label,
expressed in "normalized parent coordinates". \item If \code{character},
allowed values include: i) one of c('right', 'left', 'center', 'centre',
'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
'middle') for y-axis.}}

\item{label.x, label.y}{\code{numeric} Coordinates (in data units) to be used
for absolute positioning of the label. If too short they will be recycled.}

\item{step.increase}{numeric value in with the increase in fraction of total
height for every additional comparison to minimize overlap. The step value
can be negative to reverse the order of groups.}

\item{p.adjust.method}{method for adjusting p values (see
\code{\link[stats]{p.adjust}}).  Has impact only in a situation, where
multiple pairwise tests are performed; or when there are multiple grouping
variables. Allowed values include "holm", "hochberg", "hommel",
"bonferroni", "BH", "BY", "fdr", "none". If you don't want to adjust the p
value (not recommended), use p.adjust.method = "none".}

\item{significance}{a list of arguments specifying the significance cutpoints
 and symbols. For example, \code{significance <- list(cutpoints = c(0,
 0.0001, 0.001, 0.01, 0.05, Inf), symbols = c("****", "***", "**", "*",
 "ns"))}.

 In other words, we use the following convention for symbols indicating
 statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
 \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001
 \item \code{****}:  p <= 0.0001 }

 Note: \code{significance} is kept for backward compatibility. For functions
 that also expose \code{signif.cutoffs} and related parameters,
 \code{significance} takes precedence when provided.}

\item{signif.cutoffs}{numeric vector of p-value cutoffs in descending order
for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
If \code{use.four.stars = TRUE}, can include a fourth level.
Default is NULL, which uses the package defaults.}

\item{signif.symbols}{character vector of symbols corresponding to
\code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
(and "****" if \code{use.four.stars = TRUE}).}

\item{ns.symbol}{character string for non-significant results. Default is "ns".
Use "" (empty string) to show nothing.}

\item{use.four.stars}{logical. If TRUE, allows four stars (****) for the most
significant level. Default is FALSE.}

\item{p.format.style}{character string specifying the p-value formatting style.
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
See \code{\link{list_p_format_styles}} for details.}

\item{p.digits}{integer specifying the number of decimal places for p-values.
If provided, overrides the style default.}

\item{p.leading.zero}{logical indicating whether to include leading zero before
decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display exactly.
Values below this threshold are shown as "< threshold". If provided, overrides
the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{geom}{The geometric object to use to display the data for this layer.
When using a \verb{stat_*()} function to construct a layer, the \code{geom} argument
can be used to override the default coupling between stats and geoms. The
\code{geom} argument accepts the following:
\itemize{
\item A \code{Geom} ggproto subclass, for example \code{GeomPoint}.
\item A string naming the geom. To give the geom as a string, strip the
function name of the \code{geom_} prefix. For example, to use \code{geom_point()},
give the geom as \code{"point"}.
\item For more information and other ways to specify the geom, see the
\link[ggplot2:layer_geoms]{layer geom} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If FALSE (the default), removes missing values with a warning. If
TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{parse}{If TRUE, the labels will be parsed into expressions and displayed
as described in \code{?plotmath}.}

\item{...}{other arguments to pass to
\code{\link[ggplot2:geom_text]{geom_text}}, such as:\itemize{ \item
\code{hjust}: horizontal justification of the text. Move the text left or
right and \item \code{vjust}: vertical justification of the text. Move the
text up or down. }}
}
\description{
Adds automatically one-way and two-way ANOVA test p-values to a
 ggplot, such as box plots, dot plots and stripcharts.
}
\section{Computed variables}{
 \itemize{ \item{DFn}: Degrees of Freedom in the
 numerator (i.e. DF effect). \item{DFd}:	Degrees of Freedom in the
 denominator (i.e., DF error). \item{ges}:	Generalized Eta-Squared measure of
 effect size. Computed only when the option \code{effect.size = "ges"}.
 \item{pes}:	Partial Eta-Squared measure of effect size. Computed only when
 the option \code{effect.size = "pes"}. \item{F}:	F-value. \item{p}:	p-value.
 \item{p.adj}: Adjusted p-values. \item{p.signif}: P-value significance.
 \item{p.adj.signif}: Adjusted p-value significance. \item{p.format}:
 Formatted p-value. \item{p.format.signif}: Formatted p-value with significance symbols.
 \item{p.adj.format}: Formatted adjusted p-value. \item{n}:
 number of samples. }
}

\examples{
# Data preparation
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Transform `dose` into factor variable
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Add individuals id
df$id <- rep(1:10, 6)
# Add a random grouping variable
set.seed(123)
df$group <- sample(factor(rep(c("grp1", "grp2", "grp3"), 20)))
df$len <- ifelse(df$group == "grp2", df$len + 2, df$len)
df$len <- ifelse(df$group == "grp3", df$len + 7, df$len)
head(df, 3)


# Basic boxplot
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Create a basic boxplot
# Add 5\% and 10\% space to the plot bottom and the top, respectively
bxp <- ggboxplot(df, x = "dose", y = "len") +
  scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))

# Add the p-value to the boxplot
bxp + stat_anova_test()

\dontrun{
# Change the label position
# Using coordinates in data units
bxp + stat_anova_test(label.x = "1", label.y = 10, hjust = 0)
}

# Format the p-value differently
custom_p_format <- function(p) {
  rstatix::p_format(p, accuracy = 0.0001, digits = 3, leading.zero = FALSE)
}
bxp + stat_anova_test(
  label = "Anova, italic(p) = {custom_p_format(p)}{p.signif}"
)

# Show a detailed label in italic
bxp + stat_anova_test(label = "as_detailed_italic")


# Faceted plots
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Create a ggplot facet
bxp <- ggboxplot(df, x = "dose", y = "len", facet.by = "supp") +
  scale_y_continuous(expand = expansion(mult = c(0.05, 0.1)))
# Add p-values
bxp + stat_anova_test()


# Grouped plots
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
bxp2 <- ggboxplot(df, x = "group", y = "len", color = "dose", palette = "npg")

# For each x-position, computes tests between legend groups
bxp2 + stat_anova_test(aes(group = dose), label = "p = {p.format}{p.signif}")

#  For each legend group, computes tests between x variable groups
bxp2 + stat_anova_test(aes(group = dose, color = dose), group.by = "legend.var")

\dontrun{
# Two-way ANOVA: Independent measures
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Visualization: box plots with p-values
# Two-way interaction p-values between x and legend (group) variables
bxp3 <- ggboxplot(
  df,
  x = "supp", y = "len",
  color = "dose", palette = "jco"
)
bxp3 + stat_anova_test(aes(group = dose), method = "two_way")

# One-way repeatead measures ANOVA
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
df$id <- as.factor(c(rep(1:10, 3), rep(11:20, 3)))
ggboxplot(df, x = "dose", y = "len") +
  stat_anova_test(method = "one_way_repeated", wid = "id")

# Two-way repeatead measures ANOVA
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
df$id <- as.factor(rep(1:10, 6))
ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
  stat_anova_test(aes(group = supp), method = "two_way_repeated", wid = "id")

# Grouped one-way repeated measures ANOVA
ggboxplot(df, x = "dose", y = "len", color = "supp", palette = "jco") +
  stat_anova_test(aes(group = supp, color = supp),
    method = "one_way_repeated", wid = "id", group.by = "legend.var"
  )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggtext.R
\name{ggtext}
\alias{ggtext}
\title{Text}
\usage{
ggtext(
  data,
  x = NULL,
  y = NULL,
  label = NULL,
  color = "black",
  palette = NULL,
  size = 11,
  face = "plain",
  family = "",
  show.legend = NA,
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  parse = FALSE,
  grouping.vars = NULL,
  position = "identity",
  ggp = NULL,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x, y}{x and y variables for drawing.}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{color}{text font color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{text font size.}

\item{face}{text font style. Allowed values are one of c("plain", "bold",
"italic", "bold.italic").}

\item{family}{character vector specifying font family.}

\item{show.legend}{logical. Should text be included in the legends? NA, the
default, includes if any aesthetics are mapped. FALSE never includes, and
TRUE always includes.}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variables values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{parse}{If \code{TRUE}, the labels will be parsed into expressions and
displayed as described in \code{?plotmath}.}

\item{grouping.vars}{grouping variables to sort the data by, when the user
wants to display the top n up/down labels.}

\item{position}{Position adjustment, either as a string, or the result of a
call to a position adjustment function.}

\item{ggp}{a ggplot. If not NULL, points are added to an existing plot.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to \code{\link{ggpar}}.}
}
\description{
Add text to a plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right"  }
}
\examples{
# Load data
data("mtcars")
df <- mtcars
df$cyl <- as.factor(df$cyl)
df$name <- rownames(df)
head(df[, c("wt", "mpg", "cyl")], 3)

# Textual annotation
# +++++++++++++++++
ggtext(df,
  x = "wt", y = "mpg",
  color = "cyl", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  label = "name", repel = TRUE
)

# Add rectangle around label
ggtext(df,
  x = "wt", y = "mpg",
  color = "cyl", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  label = "name", repel = TRUE, label.rectangle = TRUE
)

}
\seealso{
\code{\link{ggpar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\name{create_p_label}
\alias{create_p_label}
\title{Create P-Value Label with Proper Prefix}
\usage{
create_p_label(p.format, p.signif = NULL)
}
\arguments{
\item{p.format}{Character string of the formatted p-value (e.g., "0.05", "< 0.001").}

\item{p.signif}{Optional character string of significance symbol (e.g., "*", "**", "ns").
If provided, it will be appended after the p-value.}
}
\value{
A character string with the properly formatted p-value label. Missing
  values in \code{p.format} are preserved as \code{NA_character_}.
}
\description{
Creates a p-value label string with proper handling of inequality
  symbols. When the formatted p-value starts with "<" or ">", uses "p <value"
  or "p >value" format. Otherwise uses "p = value" format.
}
\examples{
\dontrun{
create_p_label("0.05")
# Returns: "p = 0.05"

create_p_label("< 0.001")
# Returns: "p < 0.001"

create_p_label("0.01", "**")
# Returns: "p = 0.01 **"

create_p_label("< 0.001", "****")
# Returns: "p < 0.001 ****"

create_p_label(c("0.05", NA_character_))
# Returns: c("p = 0.05", NA)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gghistogram.R
\name{gghistogram}
\alias{gghistogram}
\title{Histogram plot}
\usage{
gghistogram(
  data,
  x,
  y = "..count..",
  combine = FALSE,
  merge = FALSE,
  weight = NULL,
  color = "black",
  fill = NA,
  palette = NULL,
  size = NULL,
  linetype = "solid",
  alpha = 0.5,
  bins = NULL,
  binwidth = NULL,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  add = c("none", "mean", "median"),
  add.params = list(linetype = "dashed"),
  rug = FALSE,
  add_density = FALSE,
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  position = position_identity(),
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{variable to be drawn.}

\item{y}{one of "..density.." or "..count..".}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{weight}{a variable name available in the input data for creating a weighted histogram.}

\item{color, fill}{histogram line color and fill color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{linetype}{line type. See \code{\link{show_line_types}}.}

\item{alpha}{numeric value specifying fill color transparency. Value should
be in [0, 1], where 0 is full transparency and 1 is no transparency.}

\item{bins}{Number of bins. Defaults to 30.}

\item{binwidth}{numeric value specifying bin width. use value between 0 and 1
when you have a strong dense dotplot. For example binwidth = 0.2.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{add}{allowed values are one of "mean" or "median" (for adding mean or
median line, respectively).}

\item{add.params}{parameters (color, size, linetype) for the argument 'add';
e.g.: add.params = list(color = "red").}

\item{rug}{logical value. If TRUE, add marginal rug.}

\item{add_density}{logical value. If TRUE, add density curves.}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variabes values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{position}{Position adjustment, either as a string, or the result of a
call to a position adjustment function. Allowed values include "identity",
"stack", "dodge".}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to
\code{\link[ggplot2]{geom_histogram}} and \code{\link{ggpar}}.}
}
\description{
Create a histogram plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Create some data format
set.seed(1234)
wdata = data.frame(
   sex = factor(rep(c("F", "M"), each=200)),
   weight = c(rnorm(200, 55), rnorm(200, 58)))

head(wdata, 4)

# Basic density plot
# Add mean line and marginal rug
gghistogram(wdata, x = "weight", fill = "lightgray",
   add = "mean", rug = TRUE)

# Change outline colors by groups ("sex")
# Use custom color palette
gghistogram(wdata, x = "weight",
   add = "mean", rug = TRUE,
   color = "sex", palette = c("#00AFBB", "#E7B800"))

# Change outline and fill colors by groups ("sex")
# Use custom color palette
gghistogram(wdata, x = "weight",
   add = "mean", rug = TRUE,
   color = "sex", fill = "sex",
   palette = c("#00AFBB", "#E7B800"))



# Combine histogram and density plots
gghistogram(wdata, x = "weight",
   add = "mean", rug = TRUE,
   fill = "sex", palette = c("#00AFBB", "#E7B800"),
   add_density = TRUE)

# Weighted histogram
gghistogram(iris, x = "Sepal.Length", weight = "Petal.Length")
}
\seealso{
\code{\link{ggdensity}} and \code{\link{ggpar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils_stat_test_label.R
\name{build_symnum_args}
\alias{build_symnum_args}
\title{Build symnum.args from Significance Parameters}
\usage{
build_symnum_args(
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  symnum.args = list()
)
}
\arguments{
\item{signif.cutoffs}{Numeric vector of p-value cutoffs in descending order
(e.g., \code{c(0.10, 0.05, 0.01)} or \code{c(0.10, 0.05, 0.01, 0.001)}).
Values smaller than each cutoff receive the corresponding symbol.}

\item{signif.symbols}{Character vector of symbols matching signif.cutoffs.
If NULL, auto-generated based on length: 3 cutoffs -> c("*", "**", "***"),
4 cutoffs -> c("*", "**", "***", "****").}

\item{ns.symbol}{Character string for non-significant results. Default is "ns".
Use "" (empty string) to show nothing for non-significant results.}

\item{use.four.stars}{Logical. If TRUE and signif.symbols is NULL, include
four stars (****) for the most significant level. Default is FALSE.}

\item{symnum.args}{Existing symnum.args list. If provided and non-empty,
it takes precedence over other parameters (for backward compatibility).}
}
\value{
A list suitable for use as symnum.args parameter.
}
\description{
Internal helper function to build a symnum.args list from
  user-friendly significance parameters. This allows users to specify
  custom significance cutoffs and symbols without using the complex
  symnum.args format directly.
}
\details{
Priority order:
1. If symnum.args is provided (non-empty), use it directly
2. If signif.cutoffs is provided, build symnum.args from it
3. Otherwise, use package defaults
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_palette.R
\name{set_palette}
\alias{set_palette}
\alias{change_palette}
\alias{color_palette}
\alias{fill_palette}
\title{Set Color Palette}
\usage{
set_palette(p, palette)

change_palette(p, palette)

color_palette(palette = NULL, ...)

fill_palette(palette = NULL, ...)
}
\arguments{
\item{p}{a ggplot}

\item{palette}{Color palette. Allowed values include: \itemize{ \item
\strong{Grey color palettes}: "grey" or "gray"; \item \strong{RColorBrewer
palettes}, see \code{\link[RColorBrewer:ColorBrewer]{brewer.pal}} and details section.
Examples of palette names include: "RdBu", "Blues", "Dark2", "Set2", ...;
\item \strong{Custom color palettes}. For example, palette = c("#00AFBB",
"#E7B800", "#FC4E07"); \item \strong{ggsci scientific journal palettes},
e.g.: "npg", "aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". }}

\item{...}{other arguments passed to ggplot2 scale_color_xxx() and
scale_fill_xxx() functions.}
}
\description{
\itemize{ \item \code{change_palette(), set_palette()}: Change
both color and fill palettes. \item \code{color_palette()}: change color
palette only. \item \code{fill_palette()}: change fill palette only.

}
}
\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
p <- ggboxplot(df,
  x = "dose", y = "len",
  color = "dose"
)
p

# Change the color palette
set_palette(p, "jco")
}
\seealso{
\link{get_palette}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggboxplot.R
\name{ggboxplot}
\alias{ggboxplot}
\title{Box plot}
\usage{
ggboxplot(
  data,
  x,
  y,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = "white",
  palette = NULL,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  bxp.errorbar = FALSE,
  bxp.errorbar.width = 0.4,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  linetype = "solid",
  size = NULL,
  linewidth = NULL,
  width = 0.7,
  notch = FALSE,
  outliers = TRUE,
  outlier.shape = 19,
  select = NULL,
  remove = NULL,
  order = NULL,
  add = "none",
  add.params = list(),
  error.plot = "pointrange",
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  position = position_dodge(0.8),
  ggtheme = theme_pubr(),
  show.n = FALSE,
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{character string containing the name of x variable.}

\item{y}{character vector containing one or more variables to plot}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color}{outline color.}

\item{fill}{fill color.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{bxp.errorbar}{logical value. If TRUE, shows error bars of box plots.}

\item{bxp.errorbar.width}{numeric value specifying the width of box plot error
bars. Default is 0.4.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{linetype}{line types.}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{linewidth}{constant value specifying the line width.}

\item{width}{numeric value between 0 and 1 specifying the width of the plot elements.}

\item{notch}{If \code{FALSE} (default) make a standard box plot. If
\code{TRUE}, make a notched box plot. Notches are used to compare groups;
if the notches of two boxes do not overlap, this suggests that the medians
are significantly different.}

\item{outliers}{Whether to display (\code{TRUE}) or discard (\code{FALSE}) outliers
from the plot. Hiding or discarding outliers can be useful when, for
example, raw data points need to be displayed on top of the boxplot.
By discarding outliers, the axis limits will adapt to the box and whiskers
only, not the full data range. If outliers need to be hidden and the axes
needs to show the full data range, please use \code{outlier.shape = NA} instead.}

\item{outlier.shape}{point shape of outlier. Default is 19. To hide outlier,
specify \code{outlier.shape = NA}. When jitter is added, then outliers will
be automatically hidden.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{label}{the name of the column containing point labels. Can be also a
character vector with length = nrow(data).}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variables values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function (e.g. \code{position_dodge(0.8)}).
Used to control the spacing between grouped elements.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{show.n}{logical. If TRUE, displays the number of observations
(\code{"n = <count>"}) at the top of each group. Off by default. When the
groups are dodged (a \code{color}/\code{fill} grouping with a dodging
\code{position}), one count is shown per group; otherwise a single count is
shown per x-axis tick. Counts respect \code{select}/\code{remove} and are
computed per facet.}

\item{...}{other arguments to be passed to
\code{\link[ggplot2]{geom_boxplot}}, \code{\link{ggpar}} and
\code{\link{facet}}.}
}
\description{
Create a box plot with points. Box plots display a group of
 numerical data through their quartiles.
}
\details{
The plot can be easily customized using the function ggpar(). Read
 ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
 ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
 scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
 = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
 and position: legend = "right" \item plot orientation : orientation =
 c("vertical", "horizontal", "reverse") }
}
\section{Suggestions for the argument "add"}{
 Suggested values are one of
 c("dotplot", "jitter").
}

\examples{
# Load data
data("ToothGrowth")
df <- ToothGrowth

# Basic plot
# +++++++++++++++++++++++++++
# width: change box plots width
ggboxplot(df, x = "dose", y = "len", width = 0.8)

# Change orientation: horizontal
ggboxplot(df, "dose", "len", orientation = "horizontal")

# Notched box plot
ggboxplot(df,
  x = "dose", y = "len",
  notch = TRUE
)

# Add dots
# ++++++++++++++++++++++++++
ggboxplot(df,
  x = "dose", y = "len",
  add = "dotplot"
)

# Add jitter points and change the shape by groups
ggboxplot(df,
  x = "dose", y = "len",
  add = "jitter", shape = "dose"
)


# Select and order items
# ++++++++++++++++++++++++++++++

# Select which items to display: "0.5" and "2"
ggboxplot(df, "dose", "len",
  select = c("0.5", "2")
)

# Change the default order of items
ggboxplot(df, "dose", "len",
  order = c("2", "1", "0.5")
)


# Change colors
# +++++++++++++++++++++++++++
# Change outline and fill colors
ggboxplot(df, "dose", "len",
  color = "black", fill = "gray"
)

# Change outline colors by groups: dose
# Use custom color palette
# Add jitter points and change the shape by groups
ggboxplot(df, "dose", "len",
  color = "dose", palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  add = "jitter", shape = "dose"
)

# Change fill color by groups: dose
ggboxplot(df, "dose", "len",
  fill = "dose", palette = c("#00AFBB", "#E7B800", "#FC4E07")
)


# Box plot with multiple groups
# +++++++++++++++++++++
# fill or color box plot by a second group : "supp"
ggboxplot(df, "dose", "len",
  color = "supp",
  palette = c("#00AFBB", "#E7B800")
)

}
\seealso{
\code{\link{ggpar}}, \code{\link{ggviolin}}, \code{\link{ggdotplot}}
 and \code{\link{ggstripchart}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggtexttable.R
\name{ggtexttable}
\alias{ggtexttable}
\alias{ttheme}
\alias{colnames_style}
\alias{rownames_style}
\alias{tbody_style}
\alias{table_cell_font}
\alias{table_cell_bg}
\alias{tab_cell_crossout}
\alias{tab_ncol}
\alias{tab_nrow}
\alias{tab_add_hline}
\alias{tab_add_vline}
\alias{tab_add_border}
\alias{tbody_add_border}
\alias{thead_add_border}
\alias{tab_add_title}
\alias{tab_add_footnote}
\title{Draw a Textual Table}
\usage{
ggtexttable(
  x,
  rows = rownames(x),
  cols = colnames(x),
  vp = NULL,
  theme = ttheme(),
  ...
)

ttheme(
  base_style = "default",
  base_size = 11,
  base_colour = "black",
  padding = unit(c(4, 4), "mm"),
  colnames.style = colnames_style(size = base_size),
  rownames.style = rownames_style(size = base_size),
  tbody.style = tbody_style(size = base_size)
)

colnames_style(
  color = "black",
  face = "bold",
  size = 12,
  fill = "grey80",
  linewidth = 1,
  linecolor = "white",
  parse = FALSE,
  ...
)

rownames_style(
  color = "black",
  face = "italic",
  size = 12,
  fill = NA,
  linewidth = 1,
  linecolor = "white",
  parse = FALSE,
  ...
)

tbody_style(
  color = "black",
  face = "plain",
  size = 12,
  fill = c("grey95", "grey90"),
  linewidth = 1,
  linecolor = "white",
  parse = FALSE,
  ...
)

table_cell_font(tab, row, column, face = NULL, size = NULL, color = NULL)

table_cell_bg(
  tab,
  row,
  column,
  fill = NULL,
  color = NULL,
  linewidth = NULL,
  alpha = NULL
)

tab_cell_crossout(
  tab,
  row,
  column,
  linetype = 1,
  linewidth = 1,
  linecolor = "black",
  reduce.size.by = 0
)

tab_ncol(tab)

tab_nrow(tab)

tab_add_hline(
  tab,
  at.row = 2:tab_nrow(tab),
  row.side = c("bottom", "top"),
  from.column = 1,
  to.column = tab_ncol(tab),
  linetype = 1,
  linewidth = 1,
  linecolor = "black"
)

tab_add_vline(
  tab,
  at.column = 2:tab_ncol(tab),
  column.side = c("left", "right"),
  from.row = 1,
  to.row = tab_nrow(tab),
  linetype = 1,
  linewidth = 1,
  linecolor = "black"
)

tab_add_border(
  tab,
  from.row = 2,
  to.row = tab_nrow(tab),
  from.column = 1,
  to.column = tab_ncol(tab),
  linetype = 1,
  linewidth = 1,
  linecolor = "black"
)

tbody_add_border(
  tab,
  from.row = 2,
  to.row = tab_nrow(tab),
  from.column = 1,
  to.column = tab_ncol(tab),
  linetype = 1,
  linewidth = 1,
  linecolor = "black"
)

thead_add_border(
  tab,
  from.row = 1,
  to.row = 1,
  from.column = 1,
  to.column = tab_ncol(tab),
  linetype = 1,
  linewidth = 1,
  linecolor = "black"
)

tab_add_title(
  tab,
  text,
  face = NULL,
  size = NULL,
  color = NULL,
  family = NULL,
  padding = unit(1.5, "line"),
  just = "left",
  hjust = NULL,
  vjust = NULL
)

tab_add_footnote(
  tab,
  text,
  face = NULL,
  size = NULL,
  color = NULL,
  family = NULL,
  padding = unit(1.5, "line"),
  just = "right",
  hjust = NULL,
  vjust = NULL
)
}
\arguments{
\item{x}{a \code{data.frame} or \code{matrix}.}

\item{rows}{optional vector to specify row names}

\item{cols}{optional vector to specify column names}

\item{vp}{optional viewport}

\item{theme}{a list, as returned by the function \code{ttheme()}, defining the
parameters of the table theme. Allowed values include one of \code{ttheme()}
and \code{ttheme_clean()}.}

\item{...}{extra parameters for text justification, e.g.: hjust and x. Default
is "centre" for the body and header, and "right" for the row names. Left
justification: \code{hjust = 0, x = 0.1}. Right justification: \code{hjust = 1, x = 0.9}.}

\item{base_style}{character string the table style/theme. The available themes
are illustrated in the
\href{https://rpkgs.datanovia.com/ggpubr/files/ggtexttable-theme.pdf}{ggtexttable-theme.pdf}
 file. Allowed values include one of \code{c("default", "blank", "classic",
"minimal", "light", "lBlack", "lBlue", "lRed", "lGreen", "lViolet", "lCyan",
"lOrange", "lBlackWhite", "lBlueWhite", "lRedWhite", "lGreenWhite",
"lVioletWhite", "lCyanWhite", "lOrangeWhite", "mBlack", "mBlue", "mRed",
"mGreen", "mViolet", "mCyan", "mOrange", "mBlackWhite", "mBlueWhite",
"mRedWhite", "mGreenWhite", "mVioletWhite", "mCyanWhite", "mOrangeWhite" )}.
Note that, l = "light"; m = "medium".}

\item{base_size}{default font size}

\item{base_colour}{default font colour}

\item{padding}{length-2 unit vector specifying the horizontal and vertical padding of text within each cell}

\item{colnames.style}{a list, as returned by the function
\code{colnames_style()}, defining the style of the table column names.
Considered only when \code{base_size = "default"}.}

\item{rownames.style}{a list, as returned by the function
\code{rownames_style()}, defining the style of the table row names.
Considered only when \code{base_size = "default"}.}

\item{tbody.style}{a list, as returned by the function \code{tbody_style()},
defining the style of the table body. Considered only when \code{base_size =
"default"}.}

\item{color, face, size}{text font color, face and size, respectively. Allowed
values for face include c("plain", "bold", "italic", "bold.italic").}

\item{fill}{background color.}

\item{linewidth, linecolor}{line width and color, respectively.}

\item{parse}{logical, default behaviour for parsing text as plotmath}

\item{tab}{an object from \code{ggtexttable} or from \code{gridExtra::tableGrob()}.}

\item{row, column}{an integer specifying the row and the column numbers for the cell of interest.}

\item{alpha}{numeric value specifying fill color transparency.
Value should be in [0, 1], where 0 is full transparency and 1 is no transparency.}

\item{linetype}{line type}

\item{reduce.size.by}{Numeric value in [0, 1] to reduce the size by.}

\item{at.row}{a numeric vector of row indexes; for example \code{at.row = c(1, 2)}.}

\item{row.side}{row side to which the horizontal line should be added. Can be one of \code{c("bottom", "top")}.}

\item{from.column}{integer indicating the column from which to start drawing the horizontal line.}

\item{to.column}{integer indicating the column to which the horizontal line should end.}

\item{at.column}{a numeric vector of column indexes; for example \code{at.column = c(1, 2)}.}

\item{column.side}{column side to which the vertical line should be added. Can be one of \code{c("left", "right")}.}

\item{from.row}{integer indicating the row from which to start drawing the horizontal line.}

\item{to.row}{integer indicating the row to which the vertical line should end.}

\item{text}{text to be added as title or footnote.}

\item{family}{font family}

\item{just}{The justification of the text relative to its (x, y) location. If
there are two values, the first value specifies horizontal justification
and the second value specifies vertical justification. Possible string
values are: "left", "right", "centre", "center", "bottom", and "top". For
numeric values, 0 means left (bottom) alignment and 1 means right (top)
alignment.}

\item{hjust}{A numeric vector specifying horizontal justification. If
specified, overrides the just setting.}

\item{vjust}{A numeric vector specifying vertical justification. If
specified, overrides the just setting.}
}
\value{
an object of class ggplot.
}
\description{
Draw a textual table. \itemize{ \item \code{ggtexttable()}: draw
 a textual table. \item \code{ttheme()}: customize table theme. \item
 \code{rownames_style(), colnames_style(), tbody_style()}: helper functions
 to customize the table row names, column names and body.
 \item \code{table_cell_font()}: access to a table cell for changing the text font (size and face).
 \item \code{table_cell_bg()}: access to a table cell for changing the background (fill, color, linewidth).
 \item \code{tab_cell_crossout()}: cross out a table cell.
 \item \code{tab_ncol(), tab_nrow()}: returns, respectively, the number of columns and rows in a ggtexttable.
 \item \code{tab_add_hline()}: Creates horizontal lines or separators at the top or the bottom side of a given specified row.
 \item \code{tab_add_vline()}: Creates vertical lines or separators at the right or the left side of a given specified column.
 \item \code{tab_add_border(), tbody_add_border(), thead_add_border()}: Add borders to table; tbody is for table body and thead is for table head.
 \item \code{tab_add_title(),tab_add_footnote()}: Add title, subtitle and footnote to a table.
  }
}
\examples{
# data
df <- head(iris)

# Default table
# Remove row names using rows = NULL
ggtexttable(df, rows = NULL)

# Text justification for individual cells/rows/columns (#335)
# First column is left justified i.e., hjust = 0 , x = 0.1
# Remaining columns are right justified i.e., hjust = 1 , x = 0.9
table_theme <- ttheme(
  tbody.style = tbody_style(
    hjust = as.vector(matrix(c(0, 1, 1, 1, 1), ncol = 5, nrow = nrow(df), byrow = TRUE)),
    x = as.vector(matrix(c(.1, .9, .9, .9, .9), ncol = 5, nrow = nrow(df), byrow = TRUE))
  )
)
ggtexttable(df, rows = NULL, theme = table_theme)

# Blank theme
ggtexttable(df, rows = NULL, theme = ttheme("blank"))

# light theme
ggtexttable(df, rows = NULL, theme = ttheme("light"))

# Column names border only
ggtexttable(df, rows = NULL, theme = ttheme("blank")) \%>\%
  tab_add_hline(at.row = 1:2, row.side = "top", linewidth = 2)

# classic theme
ggtexttable(df, rows = NULL, theme = ttheme("classic"))

# minimal theme
ggtexttable(df, rows = NULL, theme = ttheme("minimal"))

# Medium blue (mBlue) theme
ggtexttable(df, rows = NULL, theme = ttheme("mBlue"))


# Customize the table as you want
ggtexttable(df,
  rows = NULL,
  theme = ttheme(
    colnames.style = colnames_style(color = "white", fill = "#8cc257"),
    tbody.style = tbody_style(color = "black", fill = c("#e8f3de", "#d3e8bb"))
  )
)

# Use RColorBrewer palette
# Provide as many fill color as there are rows in the table body, here nrow = 6
ggtexttable(df,
  theme = ttheme(
    colnames.style = colnames_style(fill = "white"),
    tbody.style = tbody_style(fill = get_palette("RdBu", 6))
  )
)

# Text justification
# ::::::::::::::::::::::::::::::::::::::::::::::
# Default is "centre" for the body and header, and "right" for the row names.
# Left justification: hjust=0, x=0.1
# Right justification: hjust=1, x=0.9
tbody.style <- tbody_style(
  color = "black",
  fill = c("#e8f3de", "#d3e8bb"), hjust = 1, x = 0.9
)
ggtexttable(head(iris),
  rows = NULL,
  theme = ttheme(
    colnames.style = colnames_style(color = "white", fill = "#8cc257"),
    tbody.style = tbody.style
  )
)

# Access and modify the font and
# the background of table cells
# :::::::::::::::::::::::::::::::::::::::::::::
tab <- ggtexttable(head(iris),
  rows = NULL,
  theme = ttheme("classic")
)
tab <- table_cell_font(tab,
  row = 3, column = 2,
  face = "bold"
)
tab <- table_cell_bg(tab,
  row = 4, column = 3, linewidth = 5,
  fill = "darkolivegreen1", color = "darkolivegreen4"
)
tab

# Change table cells background and font for column 3,
# Spaning from row 2 to the last row in the data
tab <- ggtexttable(df, rows = NULL, theme = ttheme("classic"))
tab \%>\%
  table_cell_bg(row = 2:tab_nrow(tab), column = 3, fill = "darkblue") \%>\%
  table_cell_font(row = 2:tab_nrow(tab), column = 3, face = "italic", color = "white")

# Add separators and borders
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Table with blank theme
tab <- ggtexttable(df, theme = ttheme("blank"), rows = NULL)
# Add horizontal and vertical lines
tab \%>\%
  tab_add_hline(at.row = c(1, 2), row.side = "top", linewidth = 3, linetype = 1) \%>\%
  tab_add_hline(at.row = c(7), row.side = "bottom", linewidth = 3, linetype = 1) \%>\%
  tab_add_vline(at.column = 2:tab_ncol(tab), column.side = "left", from.row = 2, linetype = 2)

# Add borders to table body and header
# Cross out some cells
tab \%>\%
  tbody_add_border() \%>\%
  thead_add_border() \%>\%
  tab_cell_crossout(
    row = c(2, 4), column = 3, linecolor = "red",
    reduce.size.by = 0.6
  )

# Add titles andd footnote
# :::::::::::::::::::::::::::::::::::::::::::::::::::
# Add titles and footnote
# Wrap subtitle into multiple lines using strwrap()
main.title <- "Edgar Anderson's Iris Data"
subtitle <- paste0(
  "This famous (Fisher's or Anderson's) iris data set gives the measurements",
  " in centimeters of the variables sepal length and width and petal length and width,",
  " respectively, for 50 flowers from each of 3 species of iris.",
  " The species are Iris setosa, versicolor, and virginica."
) \%>\%
  strwrap(width = 80) \%>\%
  paste(collapse = "\n")

tab <- ggtexttable(head(iris), theme = ttheme("light"))
tab \%>\%
  tab_add_title(text = subtitle, face = "plain", size = 10) \%>\%
  tab_add_title(text = main.title, face = "bold", padding = unit(0.1, "line")) \%>\%
  tab_add_footnote(text = "*Table created using ggpubr", size = 10, face = "italic")


# Combine density plot and summary table
# :::::::::::::::::::::::::::::::::::::
# Density plot of "Sepal.Length"
density.p <- ggdensity(iris,
  x = "Sepal.Length",
  fill = "Species", palette = "jco"
)

# Draw the summary table of Sepal.Length
# Descriptive statistics by groups
stable <- desc_statby(iris,
  measure.var = "Sepal.Length",
  grps = "Species"
)
stable <- stable[, c("Species", "length", "mean", "sd")]
stable.p <- ggtexttable(stable,
  rows = NULL,
  theme = ttheme("mOrange")
)

# Arrange the plots on the same page
ggarrange(density.p, stable.p,
  ncol = 1, nrow = 2,
  heights = c(1, 0.5)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_pwc.R
\name{stat_pwc}
\alias{stat_pwc}
\alias{geom_pwc}
\title{Add Pairwise Comparisons P-values to a GGPlot}
\usage{
stat_pwc(
  mapping = NULL,
  data = NULL,
  method = "wilcox_test",
  method.args = list(),
  ref.group = NULL,
  label = "p.format",
  y.position = NULL,
  group.by = NULL,
  dodge = 0.8,
  bracket.nudge.y = 0.05,
  bracket.shorten = 0,
  bracket.group.by = c("x.var", "legend.var"),
  step.increase = 0.12,
  tip.length = 0.03,
  size = 0.3,
  label.size = 3.88,
  family = "",
  vjust = 0,
  hjust = 0.5,
  p.adjust.method = "holm",
  p.adjust.by = c("group", "panel"),
  p.adjust.n = NULL,
  symnum.args = list(),
  hide.ns = FALSE,
  remove.bracket = FALSE,
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  parse = FALSE,
  ...
)

geom_pwc(
  mapping = NULL,
  data = NULL,
  stat = "pwc",
  method = "wilcox_test",
  method.args = list(),
  ref.group = NULL,
  label = "p.format",
  y.position = NULL,
  group.by = NULL,
  dodge = 0.8,
  stack = FALSE,
  step.increase = 0.12,
  tip.length = 0.03,
  bracket.nudge.y = 0.05,
  bracket.shorten = 0,
  bracket.group.by = c("x.var", "legend.var"),
  size = 0.3,
  label.size = 3.88,
  family = "",
  vjust = 0,
  hjust = 0.5,
  p.adjust.method = "holm",
  p.adjust.by = c("group", "panel"),
  p.adjust.n = NULL,
  symnum.args = list(),
  hide.ns = FALSE,
  remove.bracket = FALSE,
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  parse = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{method}{a character string indicating which method to be used for
pairwise comparisons. Default is \code{"wilcox_test"}. Allowed methods
include pairwise comparisons methods implemented in the \code{rstatix} R
package. These methods are: \code{"wilcox_test", "t_test", "sign_test",
"dunn_test", "emmeans_test", "tukey_hsd", "games_howell_test"}.}

\item{method.args}{a list of additional arguments used for the test method.
For example one might use \code{method.args = list(alternative = "greater")}
for wilcoxon test.}

\item{ref.group}{a character string or a numeric value specifying the
 reference group. If specified, for a given grouping variable, each of the
 group levels will be compared to the reference group (i.e. control group).

 \code{ref.group} can be also \code{"all"}. In this case, each of the
 grouping variable levels is compared to all (i.e. basemean).

 Allowed values can be: \itemize{ \item \strong{numeric value}: specifying
 the rank of the reference group. For example, use \code{ref.group = 1} when
 the first group is the reference; use \code{ref.group = 2} when the second
 group is the reference, and so on. This works for all situations, including
 i) when comparisons are performed between x-axis groups and ii) when
 comparisons are performed between legend groups. \item \strong{character
 value}: For example, you can use \code{ref.group = "ctrl"} instead of using
 the numeric rank value of the "ctrl" group. \item \strong{"all"}: In this
 case, each of the grouping variable levels is compared to all (i.e.
 basemean). }}

\item{label}{character string specifying label. Can be: \itemize{ \item the
column containing the label (e.g.: \code{label = "p"} or \code{label =
"p.adj"}), where \code{p} is the p-value. Other possible values are
\code{"p.signif", "p.adj.signif", "p.format", "p.format.signif", "p.adj.format"}.
\item an
expression that can be formatted by the \code{\link[glue]{glue}()} package.
For example, when specifying \code{label = "Wilcoxon, p = \{p\}"}, the
expression \{p\} will be replaced by its value. \item a combination of
plotmath expressions and glue expressions. You may want some of the
statistical parameter in italic; for example:\code{label = "Wilcoxon,
italic(p)= {p}"}}.}

\item{y.position}{numeric vector with the y positions of the brackets}

\item{group.by}{(optional) character vector specifying the grouping variable;
it should be used only for grouped plots. Possible values are : \itemize{
\item \code{"x.var"}: Group by the x-axis variable and perform the test
between legend groups. In other words, the p-value is compute between legend
groups at each x position \item \code{"legend.var"}: Group by the legend
variable and perform the test between x-axis groups. In other words, the
test is performed between the x-groups for each legend level. }}

\item{dodge}{dodge width for grouped ggplot/test. Default is 0.8. It's used to
dodge the brackets position when \code{group.by = "legend.var"}.}

\item{bracket.nudge.y}{Vertical adjustment to nudge brackets by (in fraction
of the total height). Useful to move up or move down the bracket. If
positive value, brackets will be moved up; if negative value, brackets are
moved down.}

\item{bracket.shorten}{a small numeric value in [0-1] for shortening the width
of bracket.}

\item{bracket.group.by}{(optional); a variable name for grouping brackets
before adding step.increase. Useful for grouped plots. Possible values
include \code{"x.var"} and \code{"legend.var"}.}

\item{step.increase}{numeric vector with the increase in fraction of total
height for every additional comparison to minimize overlap.}

\item{tip.length}{numeric vector with the fraction of total height that the
bar goes down to indicate the precise column/}

\item{size}{change the width of the lines of the bracket}

\item{label.size}{change the size of the label text}

\item{family}{change the font used for the text}

\item{vjust}{move the text up or down relative to the bracket.}

\item{hjust}{move the text left or right relative to the bracket.}

\item{p.adjust.method}{method for adjusting p values (see
\code{\link[stats]{p.adjust}}).  Has impact only in a situation, where
multiple pairwise tests are performed; or when there are multiple grouping
variables. Ignored when the specified method is \code{"tukey_hsd"} or
\code{"games_howell_test"} because they come with internal p adjustment
method. Allowed values include "holm", "hochberg", "hommel", "bonferroni",
"BH", "BY", "fdr", "none". If you don't want to adjust the p value (not
recommended), use p.adjust.method = "none".}

\item{p.adjust.by}{possible value is one of \code{c("group", "panel")}.
Default is \code{"group"}: for a grouped data, if pairwise test is
performed, then the p-values are adjusted for each group level
independently. P-values are adjusted by panel when \code{p.adjust.by =
"panel"}.}

\item{p.adjust.n}{optional single number giving the number of comparisons to
use for the p-value adjustment, passed as \code{n} to
\code{\link[stats]{p.adjust}}. Default is \code{NULL}, which uses the number
of p-values actually computed (the standard behavior). Set it when the
displayed comparisons are a subset of a larger family and the adjustment
should reflect that larger family size. As required by
\code{\link[stats]{p.adjust}}, \code{p.adjust.n} must be greater than or
equal to the number of p-values being adjusted. It affects the adjustment
that \code{geom_pwc()} performs itself: the panel-level adjustment
(\code{p.adjust.by = "panel"}) and cases where the test returns a single set
of p-values (a single comparison, or grouped comparisons with two levels per
group). It does not alter the adjustment that pairwise tests already perform
internally per group.}

\item{symnum.args}{a list of arguments to pass to the function
 \code{\link[stats]{symnum}} for symbolic number coding of p-values. For
 example, \code{symnum.args = list(cutpoints = c(0, 0.0001, 0.001, 0.01,
 0.05, Inf), symbols = c("****", "***", "**", "*",  "ns"))}.

 In other words, we use the following convention for symbols indicating
 statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
 \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001
 \item \code{****}:  p <= 0.0001 }

 Note: If \code{symnum.args} is provided, it takes precedence over
 \code{signif.cutoffs}.}

\item{hide.ns}{can be logical value (\code{TRUE} or \code{FALSE}) or a character vector (\code{"p.adj"} or \code{"p"}).}

\item{remove.bracket}{logical, if \code{TRUE}, brackets are removed from the
plot. \itemize{ \item Case when logical value. If TRUE, hide ns symbol when
displaying significance levels. Filter is done by checking the column
\code{p.adj.signif}, \code{p.signif}, \code{p.adj} and \code{p}. \item Case
when character value. Possible values are "p" or "p.adj", for filtering out
non significant. }}

\item{p.format.style}{character string specifying the p-value formatting style.
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
See \code{\link{list_p_format_styles}} for details.}

\item{p.digits}{integer specifying the number of decimal places for p-values.
If provided, overrides the style default.}

\item{p.leading.zero}{logical indicating whether to include leading zero before
decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display exactly.
Values below this threshold are shown as "< threshold". If provided, overrides
the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{signif.cutoffs}{numeric vector of p-value cutoffs in descending order
for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
Default is NULL, which uses the package defaults.}

\item{signif.symbols}{character vector of symbols corresponding to
\code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
(and "****" if \code{use.four.stars = TRUE}).}

\item{ns.symbol}{character string for non-significant results. Default is "ns".
Use "" (empty string) to show nothing.}

\item{use.four.stars}{logical. If TRUE, allows four stars (****) for the most
significant level. Default is FALSE.}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If \code{FALSE} (the default), removes missing values with a
warning.  If \code{TRUE} silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{parse}{logical for parsing plotmath expression.}

\item{...}{other arguments passed on to \code{\link[ggplot2]{layer}()}). These are often
aesthetics, used to set an aesthetic to a fixed value, like \code{color =
"red"} or \code{size = 3}. They may also be parameters to the paired
geom/stat.}

\item{stat}{The statistical transformation to use on the data for this layer.
When using a \verb{geom_*()} function to construct a layer, the \code{stat}
argument can be used to override the default coupling between geoms and
stats. The \code{stat} argument accepts the following:
\itemize{
\item A \code{Stat} ggproto subclass, for example \code{StatCount}.
\item A string naming the stat. To give the stat as a string, strip the
function name of the \code{stat_} prefix. For example, to use \code{stat_count()},
give the stat as \code{"count"}.
\item For more information and other ways to specify the stat, see the
\link[ggplot2:layer_stats]{layer stat} documentation.
}}

\item{stack}{logical value. Default is FALSE; should be set to TRUE for
stacked bar plots or line plots. If TRUE, then the brackets are
automatically removed and the \code{dodge} value is set to zero.}
}
\description{
add pairwise comparison p-values to a ggplot such as box plots,
 dot plots and stripcharts.
}
\details{
\bold{Notes on adjusted p-values and facet}. When using the ggplot facet functions, the p-values are computed and adjusted by panel, without taking into account the other panels. This is by design in ggplot2.

In this case, when there is only one computed p-value by panel, then using `label = "p"` or `label = "p.adj"` will give the same results using `geom_pwc()`. Again, p-value computation and adjustment in a given facet panel is done independently to the other panels.

For grouped plots with sparse data, if some subsets do not contain enough
levels or observations for the requested test, those subsets are skipped while
preserving valid inferential comparisons from comparable subsets.
Informative messages list skipped grouped subsets and their skip reasons
(for example, missing \code{ref.group} or fewer than two comparison levels).

One might want to adjust the p-values of all the facet panels together. There are two solutions for that:

\itemize{
\item Using \code{\link{ggadjust_pvalue}(p)} after creating the plot \code{p}
\item or adding the adjusted p-value manually using \code{\link{stat_pvalue_manual}()}. Read more at:
\itemize{
  \item \href{https://www.datanovia.com/en/blog/how-to-add-p-values-to-ggplot-facets/}{How to Add P-values to GGPlot Facets}
  \item \href{https://www.datanovia.com/en/blog/add-p-values-to-ggplot-facets-with-different-scales/}{Add P-values to GGPlot Facets with Different Scales}
}
}
}
\examples{
df <- ToothGrowth
df$dose <- factor(df$dose)

# Data preparation
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# Transform `dose` into factor variable
df <- ToothGrowth
df$dose <- as.factor(df$dose)
# Add a random grouping variable
df$group <- factor(rep(c("grp1", "grp2"), 30))
head(df, 3)


# Two groups by x position
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%

# Create a box plot
# Add 10\% spaces between the p-value labels and the plot border
bxp <- ggboxplot(
  df,
  x = "dose", y = "len",
  color = "supp", palette = c("#00AFBB", "#E7B800")
) +
  scale_y_continuous(expand = expansion(mult = c(0.05, 0.10)))


# Add p-values onto the box plots
# label can be "p.format"  or "p.adj.format"
bxp + geom_pwc(
  aes(group = supp),
  tip.length = 0,
  method = "t_test", label = "p.format"
)

# Show adjusted p-values and significance levels
# Hide ns (non-significant)
bxp + geom_pwc(
  aes(group = supp),
  tip.length = 0,
  method = "t_test", label = "{p.adj.format}{p.adj.signif}",
  p.adjust.method = "bonferroni", p.adjust.by = "panel",
  hide.ns = TRUE
)

# Complex cases
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%
# 1. Add p-values of OJ vs VC at each dose group
bxp.complex <- bxp +
  geom_pwc(
    aes(group = supp),
    tip.length = 0,
    method = "t_test", label = "p.adj.format",
    p.adjust.method = "bonferroni", p.adjust.by = "panel"
  )
# 2. Add pairwise comparisons between dose levels
# Nudge up the brackets by 20\% of the total height
bxp.complex <- bxp.complex +
  geom_pwc(
    method = "t_test", label = "p.adj.format",
    p.adjust.method = "bonferroni",
    bracket.nudge.y = 0.2
  )
# 3. Display the plot
bxp.complex


# Three groups by x position
# \%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%\%

# Simple plots
# _____________________________________

# Box plots with p-values
bxp <- ggboxplot(
  df,
  x = "supp", y = "len", fill = "dose",
  palette = "npg"
)
bxp +
  geom_pwc(
    aes(group = dose),
    tip.length = 0,
    method = "t_test", label = "p.adj.format",
    bracket.nudge.y = -0.08
  ) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.1)))

# Bar plots with p-values
bp <- ggbarplot(
  df,
  x = "supp", y = "len", fill = "dose",
  palette = "npg", add = "mean_sd",
  position = position_dodge(0.8)
)
bp +
  geom_pwc(
    aes(group = dose),
    tip.length = 0,
    method = "t_test", label = "p.adj.format",
    bracket.nudge.y = -0.08
  ) +
  scale_y_continuous(expand = expansion(mult = c(0, 0.1)))

}
\seealso{
\code{\link{ggadjust_pvalue}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\name{list_p_format_styles}
\alias{list_p_format_styles}
\title{List Available P-Value Format Styles}
\usage{
list_p_format_styles()
}
\value{
A data frame with columns: style, digits, leading.zero, min.threshold,
  use.scientific, description.
}
\description{
Returns a data frame describing all available p-value formatting styles.
}
\examples{
list_p_format_styles()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene_citation.R
\docType{data}
\name{gene_citation}
\alias{gene_citation}
\title{Gene Citation Index}
\format{
A data frame with 66 rows and 2 columns. \describe{
 \item{\code{gene}}{gene names} \item{\code{citation_index}}{mean citation index} }
}
\usage{
data("gene_citation")
}
\description{
Contains the mean citation index of 66 genes obtained by
 assessing PubMed abstracts and annotations using two key words i) Gene name
 + b cell differentiation and ii) Gene name + plasma cell differentiation.
}
\examples{
data(gene_citation)

# Some key genes of interest to be highlighted
key.gns <- c(
  "MYC", "PRDM1", "CD69", "IRF4", "CASP3", "BCL2L1", "MYB", "BACH2", "BIM1", "PTEN",
  "KRAS", "FOXP1", "IGF1R", "KLF4", "CDK6", "CCND2", "IGF1", "TNFAIP3", "SMAD3", "SMAD7",
  "BMPR2", "RB1", "IGF2R", "ARNT"
)
# Density distribution
ggdensity(gene_citation,
  x = "citation_index", y = "..count..",
  xlab = "Number of citation",
  ylab = "Number of genes",
  fill = "lightgray", color = "black",
  label = "gene", label.select = key.gns, repel = TRUE,
  font.label = list(color = "citation_index"),
  xticks.by = 20, # Break x ticks by 20
  gradient.cols = c("blue", "red"),
  legend = "bottom",
  legend.title = "" # Hide legend title
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\name{resolve_p_format_params}
\alias{resolve_p_format_params}
\title{Resolve P-Format Parameters}
\usage{
resolve_p_format_params(
  style = "default",
  digits = NULL,
  leading.zero = NULL,
  min.threshold = NULL,
  decimal.mark = NULL,
  use.scientific = NULL
)
}
\arguments{
\item{style}{Character string specifying the style.}

\item{digits}{Override for digits parameter.}

\item{leading.zero}{Override for leading.zero parameter.}

\item{min.threshold}{Override for min.threshold parameter.}

\item{decimal.mark}{Override for decimal.mark parameter.}

\item{use.scientific}{Override for use.scientific parameter.}
}
\value{
A list with resolved parameters: digits, leading.zero, min.threshold, use.scientific.
}
\description{
Internal helper function to resolve p-value formatting parameters
  from style and individual overrides. Used by other functions to maintain
  consistent parameter handling.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggscatterhist.R
\name{ggscatterhist}
\alias{ggscatterhist}
\alias{print.ggscatterhist}
\title{Scatter Plot with Marginal Histograms}
\usage{
ggscatterhist(
  data,
  x,
  y,
  group = NULL,
  color = "black",
  fill = NA,
  palette = NULL,
  shape = 19,
  size = 2,
  linetype = "solid",
  bins = 30,
  margin.plot = c("density", "histogram", "boxplot"),
  margin.params = list(),
  margin.ggtheme = theme_void(),
  margin.space = FALSE,
  main.plot.size = 2,
  margin.plot.size = 1,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  legend = "top",
  ggtheme = theme_pubr(),
  print = TRUE,
  ...
)

\method{print}{ggscatterhist}(
  x,
  margin.space = FALSE,
  main.plot.size = 2,
  margin.plot.size = 1,
  title = NULL,
  legend = "top",
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{an object of class \code{ggscatterhist}.}

\item{y}{y variables for drawing.}

\item{group}{a grouping variable. Change points color and shape by groups if
the options \code{color} and \code{shape} are missing. Should be also
specified when you want to create a marginal box plot that is grouped.}

\item{color, fill}{point colors.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{shape}{point shape. See \code{\link{show_point_shapes}}.}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{linetype}{line type ("solid", "dashed", ...)}

\item{bins}{Number of histogram bins. Defaults to 30. Pick a better value that
fit to your data.}

\item{margin.plot}{the type of the marginal plot. Default is "hist".}

\item{margin.params}{parameters to be applied to the marginal plots.}

\item{margin.ggtheme}{the theme of the marginal plot. Default is
\code{\link[ggplot2:ggtheme]{theme_void}()}.}

\item{margin.space}{logical value. If TRUE, adds space between the main plot
and the marginal plot.}

\item{main.plot.size}{the width of the main plot. Default is 2.}

\item{margin.plot.size}{the width of the marginal plot. Default is 1.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{legend}{specify the legend position. Allowed values include: "top",
"bottom", "left", "right".}

\item{ggtheme}{the theme to be used for the scatter plot. Default is
\code{\link{theme_pubr}()}.}

\item{print}{logical value. If \code{TRUE} (default), print the plot.}

\item{...}{other arguments passed to the function \code{\link{ggscatter}()}.}
}
\value{
an object of class \code{ggscatterhist}, which is a list of ggplots,
 including the following elements: \itemize{\item sp: main scatter plot;
 \item xplot: marginal x-axis plot; \item yplot: marginal y-axis plot. }.

 User can modify each of plot before printing.
}
\description{
Create a scatter plot with marginal histograms, density plots or
 box plots.
}
\examples{
# Basic scatter plot with marginal density plot
ggscatterhist(iris,
  x = "Sepal.Length", y = "Sepal.Width",
  color = "#00AFBB",
  margin.params = list(fill = "lightgray")
)


# Grouped data
ggscatterhist(
  iris,
  x = "Sepal.Length", y = "Sepal.Width",
  color = "Species", size = 3, alpha = 0.6,
  palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  margin.params = list(fill = "Species", color = "black", size = 0.2)
)

# Use boxplot as marginal
ggscatterhist(
  iris,
  x = "Sepal.Length", y = "Sepal.Width",
  color = "Species", size = 3, alpha = 0.6,
  palette = c("#00AFBB", "#E7B800", "#FC4E07"),
  margin.plot = "boxplot",
  ggtheme = theme_bw()
)

# Add vertical and horizontal line to a ggscatterhist
plots <- ggscatterhist(iris, x = "Sepal.Length", y = "Sepal.Width", print = FALSE)
plots$sp <- plots$sp +
  geom_hline(yintercept = 3, linetype = "dashed", color = "blue") +
  geom_vline(xintercept = 6, linetype = "dashed", color = "red")
plots

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gradient_color.R
\name{gradient_color}
\alias{gradient_color}
\alias{gradient_fill}
\title{Set Gradient Color}
\usage{
gradient_color(palette)

gradient_fill(palette)
}
\arguments{
\item{palette}{the color palette to be used for coloring or filling by
groups. Allowed values include "grey" for grey color palettes; brewer
palettes e.g. "RdBu", "Blues", ...; or custom color palette e.g. c("blue",
"red"); and scientific journal palettes from ggsci R package, e.g.: "npg",
"aaas", "lancet", "jco", "ucscgb", "uchicago", "simpsons" and
"rickandmorty". Can be also a numeric vector; in this
case a basic color palette is created using the function
\link[grDevices]{palette}.}
}
\description{
Change gradient color.
\itemize{
\item \code{gradient_color()}: Change gradient color.
\item \code{gradient_fill()}: Change gradient fill.
}
}
\examples{
df <- mtcars
p <- ggscatter(df,
  x = "wt", y = "mpg",
  color = "mpg"
)

# Change gradient color
# Use one custom color
p + gradient_color("red")

# Two colors
p + gradient_color(c("blue", "red"))

# Three colors
p + gradient_color(c("blue", "white", "red"))

# Use RColorBrewer palette
p + gradient_color("RdYlBu")

# Use ggsci color palette
p + gradient_color("npg")
}
\seealso{
\link{set_palette}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_npc.R
\name{as_npc}
\alias{as_npc}
\alias{as_npcx}
\alias{as_npcy}
\title{Convert Character Coordinates into Normalized Parent Coordinates (NPC)}
\usage{
as_npc(
  value,
  group = 1L,
  step = 0.1,
  margin.npc = 0.05,
  axis = c("xy", "x", "y")
)

as_npcx(value, group = 1L, step = 0.1, margin.npc = 0.05)

as_npcy(value, group = 1L, step = 0.1, margin.npc = 0.05)
}
\arguments{
\item{value}{numeric (in [0-1]) or character vector of coordinates. If
character, should be one of \code{c('right', 'left', 'bottom', 'top',
'center', 'centre', 'middle')}.}

\item{group}{integer ggplot's group id. Used to shift coordinates to avoid
overlaps.}

\item{step}{numeric value in [0-1]. The step size for shifting coordinates
in npc units. Considered as horizontal step for x-axis and vertical step
for y-axis. For y-axis, the step value can be negative to reverse the order of groups.}

\item{margin.npc}{numeric [0-1] The margin added towards the nearest
plotting area edge when converting character coordinates into npc.}

\item{axis}{the concerned axis . Should be one of \code{c("xy", "x", "y")}.}
}
\value{
A numeric vector with values in the range [0-1] representing npc
  coordinates.
}
\description{
Convert character coordinates to npc units and shift positions to avoid
overlaps when grouping is active. If numeric validate npc values.
}
\details{
the \code{as_npc()} function is an adaptation from
  \code{ggpmisc::compute_npc()}.
}
\section{Functions}{
\itemize{
\item \code{as_npc()}: converts x or y coordinate values into npc. Input values
should be numeric or one of the following values \code{c('right', 'left',
'bottom', 'top', 'center', 'centre', 'middle')}.

\item \code{as_npcx()}: converts x coordinate values into npc. Input values should
be numeric or one of the following values \code{c('right', 'left',
'center', 'centre', 'middle')}. Wrapper around \code{as_npc(axis = "x")}.

\item \code{as_npcy()}: converts y coordinate values into npc. Input values should
be numeric or one of the following values \code{c( 'bottom', 'top',
'center', 'centre', 'middle')}. Wrapper around \code{as_npc(axis = "y")}.

}}
\examples{
as_npc(c("left", "right"))
as_npc(c("top", "right"))

}
\seealso{
\code{\link{npc_to_data_coord}}, \code{\link{get_coord}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_bracket.R
\name{stat_bracket}
\alias{stat_bracket}
\alias{geom_bracket}
\title{Add Brackets with Labels to a GGPlot}
\usage{
stat_bracket(
  mapping = NULL,
  data = NULL,
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  label = NULL,
  type = c("text", "expression"),
  y.position = NULL,
  xmin = NULL,
  xmax = NULL,
  step.increase = 0,
  step.group.by = NULL,
  tip.length = 0.03,
  tip.length.ref = c("data", "axis"),
  bracket.nudge.y = 0,
  bracket.shorten = 0,
  size = 0.3,
  linewidth = size,
  label.size = 3.88,
  family = "",
  vjust = 0,
  ...
)

geom_bracket(
  mapping = NULL,
  data = NULL,
  stat = "bracket",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  label = NULL,
  type = c("text", "expression"),
  y.position = NULL,
  xmin = NULL,
  xmax = NULL,
  step.increase = 0,
  step.group.by = NULL,
  tip.length = 0.03,
  tip.length.ref = c("data", "axis"),
  bracket.nudge.y = 0,
  bracket.shorten = 0,
  size = 0.3,
  linewidth = size,
  label.size = 3.88,
  family = "",
  vjust = 0,
  coord.flip = FALSE,
  orientation = c("horizontal", "vertical"),
  ymin = NULL,
  ymax = NULL,
  x.position = NULL,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If \code{FALSE} (the default), removes missing values with a
warning.  If \code{TRUE} silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{label}{character vector with alternative label, if not null test is
ignored}

\item{type}{the label type. Can be one of "text" and "expression" (for
parsing plotmath expression).}

\item{y.position}{numeric vector with the y positions of the brackets}

\item{xmin}{numeric vector with the positions of the left sides of the
brackets}

\item{xmax}{numeric vector with the positions of the right sides of the
brackets}

\item{step.increase}{numeric vector with the increase in fraction of total
height for every additional comparison to minimize overlap.}

\item{step.group.by}{a variable name for grouping brackets before adding
step.increase. Useful to group bracket by facet panel.}

\item{tip.length}{numeric vector with the fraction that the bracket tips go
down to indicate the precise column. Interpreted relative to the reference
set by \code{tip.length.ref}. Default is 0.03. Can be of the same length as
the number of brackets to adjust each tip specifically, e.g.
\code{tip.length = c(0.01, 0.03)}. If too short it is recycled.}

\item{tip.length.ref}{character string specifying what \code{tip.length} is a
fraction of. Allowed values are: \itemize{ \item \code{"data"} (default):
fraction of the trained data range. Tips scale with the data, so plots with
different data ranges get different absolute tip lengths (historical
behavior; keeps existing plots unchanged). \item \code{"axis"}: fraction of
the y-axis range (the limits set via \code{ylim}/\code{scale_y_*}). This
renders at the same physical fraction across plots and therefore gives
visually constant tip lengths regardless of the data - useful to keep tips
consistent across facets or across separate plots with different scales
(#362).}}

\item{bracket.nudge.y}{Vertical adjustment to nudge brackets by. Useful to
move up or move down the bracket. If positive value, brackets will be moved
up; if negative value, brackets are moved down.}

\item{bracket.shorten}{a small numeric value in [0-1] for shortening the with
of bracket.}

\item{size}{change the width of the lines of the bracket. Deprecated, use \code{linewidth} instead.}

\item{linewidth}{change the width of the lines of the bracket}

\item{label.size}{change the size of the label text}

\item{family}{change the font used for the text}

\item{vjust}{move the text up or down relative to the bracket}

\item{...}{other arguments passed on to \code{\link[ggplot2]{layer}()}). These are often
aesthetics, used to set an aesthetic to a fixed value, like \code{color =
"red"} or \code{size = 3}. They may also be parameters to the paired
geom/stat.}

\item{stat}{The statistical transformation to use on the data for this layer.
When using a \verb{geom_*()} function to construct a layer, the \code{stat}
argument can be used to override the default coupling between geoms and
stats. The \code{stat} argument accepts the following:
\itemize{
\item A \code{Stat} ggproto subclass, for example \code{StatCount}.
\item A string naming the stat. To give the stat as a string, strip the
function name of the \code{stat_} prefix. For example, to use \code{stat_count()},
give the stat as \code{"count"}.
\item For more information and other ways to specify the stat, see the
\link[ggplot2:layer_stats]{layer stat} documentation.
}}

\item{coord.flip}{logical. If \code{TRUE}, flip x and y coordinates so that
horizontal becomes vertical, and vertical, horizontal. When adding the
p-values to a horizontal ggplot (generated using
\code{\link[ggplot2]{coord_flip}()}), you need to specify the option
\code{coord.flip = TRUE}.}

\item{orientation}{the bracket orientation. Either \code{"horizontal"} (the
default, a bracket spanning the x axis with the bar at \code{y.position}) or
\code{"vertical"} (a bracket spanning the y axis with the bar at
\code{x.position}, tips pointing left and the label rotated to the side).
Vertical brackets are useful to annotate plots where the comparison runs
along the y axis, e.g. Kaplan-Meier curves. They are designed for a
continuous x axis; specify the span with \code{ymin}/\code{ymax} and the
position with \code{x.position}. Cannot be combined with
\code{coord.flip = TRUE}.}

\item{ymin, ymax}{numeric vectors with the positions, along the y axis, of the
two ends of each vertical bracket. Used only when
\code{orientation = "vertical"} (the vertical-bracket analogue of
\code{xmin}/\code{xmax}).}

\item{x.position}{numeric vector with the x positions of the vertical
brackets. Used only when \code{orientation = "vertical"} (the
vertical-bracket analogue of \code{y.position}).}
}
\description{
add brackets with label annotation to a ggplot. Helpers for
  adding p-value or significance levels to a plot.
}
\examples{
df <- ToothGrowth
df$dose <- factor(df$dose)

# Add bracket with labels
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    xmin = "0.5", xmax = "1", y.position = 30,
    label = "t-test, p < 0.05"
  )

# Customize bracket tip.length tip.length
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    xmin = "0.5", xmax = "1", y.position = 30,
    label = "t-test, p < 0.05", tip.length = c(0.2, 0.02)
  )

# Using plotmath expression
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    xmin = "0.5", xmax = "1", y.position = 30,
    label = "list(~italic(p)<=0.001)", type = "expression",
    tip.length = c(0.2, 0.02)
  )

# Specify multiple brackets manually
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    xmin = c("0.5", "1"), xmax = c("1", "2"),
    y.position = c(30, 35), label = c("***", "**"),
    tip.length = 0.01
  )

# Compute statistical tests and add p-values
stat.test <- compare_means(len ~ dose, ToothGrowth, method = "t.test")
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    aes(xmin = group1, xmax = group2, label = signif(p, 2)),
    data = stat.test, y.position = 35
  )

# Increase step length between brackets
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    aes(xmin = group1, xmax = group2, label = signif(p, 2)),
    data = stat.test, y.position = 35, step.increase = 0.1
  )

# Or specify the positions of each comparison
ggboxplot(df, x = "dose", y = "len") +
  geom_bracket(
    aes(xmin = group1, xmax = group2, label = signif(p, 2)),
    data = stat.test, y.position = c(32, 35, 38)
  )

# Vertical bracket (e.g. to annotate a plot with a continuous x axis)
ggscatter(df, x = "len", y = "len") +
  geom_bracket(
    orientation = "vertical",
    ymin = 10, ymax = 25, x.position = 34,
    label = "p < 0.05"
  )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdonutchart.R
\name{ggdonutchart}
\alias{ggdonutchart}
\title{Donut chart}
\usage{
ggdonutchart(
  data,
  x,
  label = x,
  lab.pos = c("out", "in"),
  lab.adjust = 0,
  lab.font = c(4, "plain", "black"),
  label.repel = FALSE,
  font.family = "",
  color = "black",
  fill = "white",
  palette = NULL,
  size = NULL,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x}{variable containing values for drawing.}

\item{label}{variable specifying the label of each slice.}

\item{lab.pos}{character specifying the position for labels. Allowed values
are "out" (for outside) or "in" (for inside).}

\item{lab.adjust}{numeric value, used to adjust label position when lab.pos =
"in". Increase or decrease this value to see the effect.}

\item{lab.font}{a vector of length 3 indicating respectively the size (e.g.:
14), the style (e.g.: "plain", "bold", "italic", "bold.italic") and the
color (e.g.: "red") of label font. For example \emph{lab.font= c(4, "bold",
"red")}.}

\item{label.repel}{logical. Default is FALSE. If TRUE, the slice labels are
placed around the chart with \code{ggrepel::geom_text_repel()} and connected
to their slice with leader lines, so that the labels of many small slices no
longer overlap. When TRUE, \code{lab.pos} is ignored.}

\item{font.family}{character vector specifying font family.}

\item{color, fill}{outline and fill colors.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to be passed to ggpar().}
}
\description{
Create a donut chart.
}
\details{
The plot can be easily customized using the function ggpar(). Read
 ?ggpar for changing: \itemize{ \item main title and axis labels: main, xlab,
 ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
 scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes: palette
 = "Dark2" or palette = c("gray", "blue", "red") \item legend title, labels
 and position: legend = "right" \item plot orientation : orientation =
 c("vertical", "horizontal", "reverse") }
}
\examples{

# Data: Create some data
# +++++++++++++++++++++++++++++++

df <- data.frame(
  group = c("Male", "Female", "Child"),
  value = c(25, 25, 50)
)

head(df)


# Basic pie charts
# ++++++++++++++++++++++++++++++++

ggdonutchart(df, "value", label = "group")


# Change color
# ++++++++++++++++++++++++++++++++

# Change fill color by group
# set line color to white
# Use custom color palette
ggdonutchart(df, "value",
  label = "group",
  fill = "group", color = "white",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)


# Change label
# ++++++++++++++++++++++++++++++++

# Show group names and value as labels
labs <- paste0(df$group, " (", df$value, "\%)")
ggdonutchart(df, "value",
  label = labs,
  fill = "group", color = "white",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)

# Change the position and font color of labels
ggdonutchart(df, "value",
  label = labs,
  lab.pos = "in", lab.font = "white",
  fill = "group", color = "white",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)

}
\seealso{
\code{\link{ggpar}}, \code{\link{ggpie}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stat_compare_means.R
\name{stat_compare_means}
\alias{stat_compare_means}
\title{Add Mean Comparison P-values to a ggplot}
\usage{
stat_compare_means(
  mapping = NULL,
  data = NULL,
  method = NULL,
  paired = FALSE,
  id = NULL,
  method.args = list(),
  ref.group = NULL,
  comparisons = NULL,
  hide.ns = FALSE,
  label.sep = ", ",
  label = NULL,
  label.x.npc = "left",
  label.y.npc = "top",
  label.x = NULL,
  label.y = NULL,
  vjust = 0,
  tip.length = 0.03,
  bracket.size = 0.3,
  step.increase = 0,
  symnum.args = list(),
  p.format.style = "default",
  p.digits = NULL,
  p.leading.zero = NULL,
  p.min.threshold = NULL,
  p.decimal.mark = NULL,
  signif.cutoffs = NULL,
  signif.symbols = NULL,
  ns.symbol = "ns",
  use.four.stars = FALSE,
  show.signif = TRUE,
  geom = "text",
  position = "identity",
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{method}{a character string indicating which method to be used for
comparing means.}

\item{paired}{a logical indicating whether you want a paired test. Used only
in \code{\link[stats]{t.test}} and in \link[stats]{wilcox.test}.}

\item{id}{optional character string naming a column that identifies matched
subjects for a \strong{paired} comparison (\code{paired = TRUE}, with
\code{method = "t.test"} or \code{"wilcox.test"}, and without
\code{comparisons}). Without it the paired test pairs observations by row
order, so the p-value is wrong when the data are not sorted so that the
compared groups align by subject. Providing \code{id} pairs the observations
by subject id instead (row-order independent); see \code{\link{compare_means}}.}

\item{method.args}{a list of additional arguments used for the test method.
For example one might use \code{method.args = list(alternative = "greater")}
for wilcoxon test.}

\item{ref.group}{a character string specifying the reference group. If
 specified, for a given grouping variable, each of the group levels will be
 compared to the reference group (i.e. control group).

 \code{ref.group} can be also \code{".all."}. In this case, each of the
 grouping variable levels is compared to all (i.e. basemean).}

\item{comparisons}{A list of length-2 vectors. The entries in the vector are
 either the names of 2 values on the x-axis or the 2 integers that correspond
 to the index of the groups of interest, to be compared.

 Note: when \code{comparisons} is specified, each pairwise test is computed
 independently and the displayed p-values are \strong{not} adjusted for
 multiple comparisons. For p-values corrected for multiple testing, use
 \code{\link{geom_pwc}()}, or \code{\link{stat_pvalue_manual}()} together with
 \code{\link{compare_means}(..., p.adjust.method = )}.}

\item{hide.ns}{logical value. If TRUE, hide ns symbol when displaying
significance levels.}

\item{label.sep}{a character string to separate the terms. Default is ", ", to
separate the test method name and the p-value.}

\item{label}{character string specifying label type. Allowed values include
\code{"p.signif"} (shows the significance levels), \code{"p.format"} (shows
the formatted p-value), and \code{"p.format.signif"} (shows the formatted
p-value followed by significance stars, e.g., "p = 0.01 **").}

\item{label.x.npc, label.y.npc}{can be \code{numeric} or \code{character}
vector of the same length as the number of groups and/or panels. If too
short they will be recycled. \itemize{ \item If \code{numeric}, value should
be between 0 and 1. Coordinates to be used for positioning the label,
expressed in "normalized parent coordinates". \item If \code{character},
allowed values include: i) one of c('right', 'left', 'center', 'centre',
'middle') for x-axis; ii) and one of c( 'bottom', 'top', 'center', 'centre',
'middle') for y-axis.}}

\item{label.x, label.y}{\code{numeric} Coordinates (in data units) to be used
for absolute positioning of the label. If too short they will be recycled.}

\item{vjust}{move the text up or down relative to the bracket.}

\item{tip.length}{numeric vector with the fraction that the bracket tips go
 down to indicate the precise column. Default is 0.03. Can be of
 same length as the number of comparisons to adjust specifically the tip
 length of each comparison. For example tip.length = c(0.01, 0.03).

 If too short they will be recycled.

 Note: when \code{comparisons} is set, brackets are drawn via
 \code{ggsignif::geom_signif()} and \code{tip.length} is a fraction of the
 trained data range, so the absolute tip length scales with your data (plots
 with different scales get different tip lengths). To obtain visually constant
 tip lengths across plots, compute the tests with \code{\link{compare_means}()}
 and draw them with \code{\link{stat_pvalue_manual}(..., tip.length.ref = "axis")}
 (which makes \code{tip.length} a fraction of the y-axis range). Passing
 \code{tip.length.ref} directly to \code{stat_compare_means()} has no effect
 and emits an informative message.}

\item{bracket.size}{Width of the lines of the bracket.}

\item{step.increase}{numeric vector with the increase in fraction of total
height for every additional comparison to minimize overlap.}

\item{symnum.args}{a list of arguments to pass to the function
 \code{\link[stats]{symnum}} for symbolic number coding of p-values. For
 example, \code{symnum.args <- list(cutpoints = c(0, 0.0001, 0.001,
 0.01, 0.05, Inf), symbols = c("****", "***", "**", "*",  "ns"))}.

 In other words, we use the following convention for symbols indicating
 statistical significance: \itemize{ \item \code{ns}: p > 0.05 \item
 \code{*}: p <= 0.05 \item \code{**}: p <= 0.01 \item \code{***}: p <= 0.001 \item \code{****}:  p <= 0.0001 }

 Note: If \code{signif.cutoffs} is provided, it takes precedence over
 \code{symnum.args}.}

\item{p.format.style}{character string specifying the p-value formatting style.
One of: \code{"default"} (backward compatible, uses scientific notation),
\code{"apa"} (APA style, no leading zero), \code{"nejm"} (NEJM style),
\code{"lancet"} (Lancet style), \code{"ama"} (AMA style), \code{"graphpad"}
(GraphPad style), or \code{"scientific"} (scientific notation for GWAS).
See \code{\link{list_p_format_styles}} for details.}

\item{p.digits}{integer specifying the number of decimal places for p-values.
If provided, overrides the style default.}

\item{p.leading.zero}{logical indicating whether to include leading zero before
decimal point (e.g., "0.05" vs ".05"). If provided, overrides the style default.}

\item{p.min.threshold}{numeric specifying the minimum p-value to display exactly.
Values below this threshold are shown as "< threshold". If provided, overrides
the style default.}

\item{p.decimal.mark}{character string to use as the decimal mark. If NULL,
uses \code{getOption("OutDec")}.}

\item{signif.cutoffs}{numeric vector of p-value cutoffs in descending order
for assigning significance symbols. For example, \code{c(0.10, 0.05, 0.01)}
means p < 0.10 gets "*", p < 0.05 gets "**", p < 0.01 gets "***".
If \code{use.four.stars = TRUE}, can include a fourth level.
Default is NULL, which uses the package defaults.}

\item{signif.symbols}{character vector of symbols corresponding to
\code{signif.cutoffs}. If NULL, auto-generated as "*", "**", "***"
(and "****" if \code{use.four.stars = TRUE}).}

\item{ns.symbol}{character string for non-significant results. Default is "ns".
Use "" (empty string) to show nothing.}

\item{use.four.stars}{logical. If TRUE, allows four stars (****) for the most
significant level. Default is FALSE.}

\item{show.signif}{logical. If TRUE (default), shows significance symbols when
using \code{label = "p.format.signif"}. If FALSE, falls back to showing only
the p-value (equivalent to \code{label = "p.format"}) with a warning.}

\item{geom}{The geometric object to use to display the data for this layer.
When using a \verb{stat_*()} function to construct a layer, the \code{geom} argument
can be used to override the default coupling between stats and geoms. The
\code{geom} argument accepts the following:
\itemize{
\item A \code{Geom} ggproto subclass, for example \code{GeomPoint}.
\item A string naming the geom. To give the geom as a string, strip the
function name of the \code{geom_} prefix. For example, to use \code{geom_point()},
give the geom as \code{"point"}.
\item For more information and other ways to specify the geom, see the
\link[ggplot2:layer_geoms]{layer geom} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{na.rm}{If FALSE (the default), removes missing values with a warning. If
TRUE silently removes missing values.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE} (the default for most ggpubr functions),
overrides the default aesthetics, rather than combining with them. This is
most useful for helper functions that define both data and aesthetics and
shouldn't inherit behaviour from the default plot specification. Set to
\code{TRUE} to inherit aesthetics from the parent ggplot layer.}

\item{...}{other arguments to pass to \code{\link[ggplot2]{geom_text}} or
\code{\link[ggplot2:geom_text]{geom_label}}.}
}
\description{
Add mean comparison p-values to a ggplot, such as box plots, dot
 plots and stripcharts.
}
\details{
For grouped plots, if one or more subsets do not contain enough levels
 to identify the requested comparison, those subsets are skipped and valid
 subsets are still tested. This avoids fatal layer failures in sparse grouped
 data settings (Issue #663).
}
\examples{
# Load data
data("ToothGrowth")
head(ToothGrowth)

# Two independent groups
# :::::::::::::::::::::::::::::::::::::::::::::::::
p <- ggboxplot(ToothGrowth,
  x = "supp", y = "len",
  color = "supp", palette = "npg", add = "jitter"
)

#  Add p-value
p + stat_compare_means()
# Change method
p + stat_compare_means(method = "t.test")

# Paired samples
# :::::::::::::::::::::::::::::::::::::::::::::::::
ggpaired(ToothGrowth,
  x = "supp", y = "len",
  color = "supp", line.color = "gray", line.size = 0.4,
  palette = "npg"
) +
  stat_compare_means(paired = TRUE)

# Paired samples matched by a subject id column (row-order independent):
# `id` gives the correct p-value even when the rows are not in subject order.
# :::::::::::::::::::::::::::::::::::::::::::::::::
tg <- ToothGrowth
tg$id <- c(1:30, 30:1) # subjects, with the second group in reverse order
ggpaired(tg, x = "supp", y = "len", color = "supp",
  line.color = "gray", palette = "npg") +
  stat_compare_means(paired = TRUE, id = "id")

# More than two groups
# :::::::::::::::::::::::::::::::::::::::::::::::::
# Pairwise comparisons: Specify the comparisons you want
my_comparisons <- list(c("0.5", "1"), c("1", "2"), c("0.5", "2"))
ggboxplot(ToothGrowth,
  x = "dose", y = "len",
  color = "dose", palette = "npg"
) +
  # Add pairwise comparisons p-value
  stat_compare_means(comparisons = my_comparisons, label.y = c(29, 35, 40)) +
  stat_compare_means(label.y = 45) # Add global Anova p-value

# Multiple pairwise test against a reference group
ggboxplot(ToothGrowth,
  x = "dose", y = "len",
  color = "dose", palette = "npg"
) +
  stat_compare_means(method = "anova", label.y = 40) + # Add global p-value
  stat_compare_means(aes(label = after_stat(p.signif)),
    method = "t.test", ref.group = "0.5"
  )

# Multiple grouping variables
# :::::::::::::::::::::::::::::::::::::::::::::::::
# Box plot facetted by "dose"
p <- ggboxplot(ToothGrowth,
  x = "supp", y = "len",
  color = "supp", palette = "npg",
  add = "jitter",
  facet.by = "dose", short.panel.labs = FALSE
)
# Use only p.format as label. Remove method name.
p + stat_compare_means(
  aes(label = paste0("p = ", after_stat(p.format)))
)

}
\seealso{
\code{\link{compare_means}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{mutate}
\alias{group_by}
\alias{get_summary_stats}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{dplyr}{\code{\link[dplyr]{group_by}}, \code{\link[dplyr]{mutate}}}

  \item{rstatix}{\code{\link[rstatix]{get_summary_stats}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpubr-package.R
\docType{package}
\name{ggpubr-package}
\alias{ggpubr}
\alias{ggpubr-package}
\title{ggpubr: 'ggplot2' Based Publication Ready Plots}
\description{
The 'ggplot2' package is excellent and flexible for elegant data visualization in R. However the default generated plots require some formatting before we can send them for publication. Furthermore, to customize a 'ggplot', the syntax is opaque and this raises the level of difficulty for researchers with no advanced R programming skills. 'ggpubr' provides some easy-to-use functions for creating and customizing 'ggplot2'-based publication ready plots. This version includes modern R ecosystem compatibility updates and customizable p-value formatting presets (APA, AMA, NEJM, Lancet, GraphPad, and scientific notation) for publication workflows, plus robust sparse-subset handling in statistical annotation layers such as 'stat_compare_means()' and 'geom_pwc()', with informative per-group skip diagnostics for non-comparable subsets.
}
\details{
General resources:

\itemize{
\item \href{https://rpkgs.datanovia.com/ggpubr/}{ggpubr documentation}
\item \href{https://www.datanovia.com/en/blog/tag/ggpubr/}{ggpubr tutorials}
}
}
\section{P-Value Formatting}{


ggpubr provides customizable p-value formatting with predefined style presets
to match different journal and publication requirements. Use \code{\link{format_p_value}()}
for direct formatting, or set \code{p.format.style} in statistical functions.

Available styles: \code{"default"}, \code{"apa"}, \code{"nejm"}, \code{"lancet"},
\code{"ama"}, \code{"graphpad"}, \code{"scientific"}.

See \code{\link{list_p_format_styles}()} for details on each style.
}

\section{Package options}{


\describe{
\item{ggpubr.parse_aes}{logical indicating whether to parse aesthetic variable names.
Default is \code{TRUE}. For example, if you want \code{ggpubr} to handle non-standard column names, like \code{A-A},
without parsing, then set this option to \code{FALSE} using \code{options(ggpubr.parse_aes = FALSE)}.}
\item{ggpubr.null_device}{A function that creates an appropriate null device.
These include: \code{\link[cowplot:pdf_null_device]{cowplot::pdf_null_device}},
\code{\link[cowplot:png_null_device]{cowplot::png_null_device}},
\code{\link[cowplot:cairo_null_device]{cowplot::cairo_null_device}} and
\code{\link[cowplot:agg_null_device]{cowplot::agg_null_device}}. Default is
\code{\link[cowplot:pdf_null_device]{cowplot::pdf_null_device}}. This is used in
functions like \code{\link{as_ggplot}()}, which need to open a graphics
device to render ggplot objects into grid graphics objects. This function is
used to open a null device to avoid displaying an unnecessary blank page
when calling \code{\link{ggarrange}()} or \code{\link{as_ggplot}()}} }
}

\seealso{
Useful links:
\itemize{
  \item \url{https://rpkgs.datanovia.com/ggpubr/}
  \item \url{https://github.com/kassambara/ggpubr}
  \item Report bugs at \url{https://github.com/kassambara/ggpubr/issues}
}

}
\author{
\strong{Maintainer}: Alboukadel Kassambara \email{alboukadel.kassambara@gmail.com}

Other contributors:
\itemize{
  \item Laszlo Erdey \email{erdey.laszlo@econ.unideb.hu} (Faculty of Economics and Business, University of Debrecen, Hungary) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/npc_to_data_coord.R
\name{npc_to_data_coord}
\alias{npc_to_data_coord}
\title{Convert NPC to Data Coordinates}
\usage{
npc_to_data_coord(npc, data.ranges)
}
\arguments{
\item{npc}{a numeric vector. Each value should be in [0-1]}

\item{data.ranges}{a numeric vector of length 2 containing the data ranges (minimum and the maximum)}
}
\value{
a numeric vector representing data coordinates.
}
\description{
Convert NPC (Normalized Parent Coordinates) into data coordinates.
}
\examples{
npc_to_data_coord(npc = c(0.2, 0.95), data.ranges = c(1, 20))
as_npc(c("top", "right")) \%>\%
  npc_to_data_coord(data.ranges = c(1, 20))

}
\seealso{
\code{\link{as_npc}}, \code{\link{get_coord}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/p_format_utils.R
\name{get_p_format_style}
\alias{get_p_format_style}
\title{Get P-Value Format Style Settings}
\usage{
get_p_format_style(style = "default")
}
\arguments{
\item{style}{Character string specifying the style. One of: "default", "apa",
"nejm", "lancet", "ama", "graphpad", "scientific".}
}
\value{
A list containing: digits, leading.zero, min.threshold, use.scientific, description.
}
\description{
Returns the settings for a given p-value format style.
}
\examples{
get_p_format_style("apa")
get_p_format_style("nejm")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rremove.R
\name{rremove}
\alias{rremove}
\title{Remove a ggplot Component}
\usage{
rremove(object)
}
\arguments{
\item{object}{character string specifying the plot components. Allowed values
 include: \itemize{
 \item \code{"grid"} for both x and y grids
 \item \code{"x.grid"} for x axis grids
 \item \code{"y.grid"} for y axis grids
 \item \code{"axis"} for both x and y axes
 \item \code{"x.axis"} for x axis
 \item \code{"y.axis"} for y axis

 \item \code{"xlab", or "x.title"}
 for x axis label
 \item \code{"ylab", or "y.title"} for y axis label
 \item \code{"xylab", "xy.title" or "axis.title"} for both x and y axis
 labels
 \item \code{"x.text"} for x axis texts (x axis tick labels)
 \item \code{"y.text"} for y axis texts (y axis tick labels)
 \item \code{"xy.text"} or \code{"axis.text"} for both x and y axis texts

 \item \code{"ticks"} for both x and y ticks
 \item \code{"x.ticks"} for x ticks
 \item \code{"y.ticks"} for y ticks

 \item \code{"legend.title"} for the legend title
 \item \code{"legend"} for the legend
 }}
}
\description{
Remove a specific component from a ggplot.
}
\examples{
# Load data
data("ToothGrowth")

# Basic plot
p <- ggboxplot(ToothGrowth,
  x = "dose", y = "len",
  ggtheme = theme_gray()
)
p

# Remove all grids
p + rremove("grid")

# Remove only x grids
p + rremove("x.grid")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggbarplot.R
\name{ggbarplot}
\alias{ggbarplot}
\title{Bar plot}
\usage{
ggbarplot(
  data,
  x,
  y,
  combine = FALSE,
  merge = FALSE,
  color = "black",
  fill = "white",
  palette = NULL,
  size = NULL,
  width = NULL,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  select = NULL,
  remove = NULL,
  order = NULL,
  add = "none",
  add.params = list(),
  error.plot = "errorbar",
  label = FALSE,
  lab.col = "black",
  lab.size = 4,
  lab.pos = c("out", "in"),
  lab.vjust = NULL,
  lab.hjust = NULL,
  lab.nb.digits = NULL,
  sort.val = c("none", "desc", "asc"),
  sort.by.groups = TRUE,
  top = Inf,
  position = position_stack(),
  numeric.x.axis = FALSE,
  ggtheme = theme_pubr(),
  ...
)
}
\arguments{
\item{data}{a data frame}

\item{x, y}{x and y variables for drawing.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{merge}{logical or character value. Default is FALSE. Used only when y is
a vector containing multiple variables to plot. If TRUE, merge multiple y
variables in the same plotting area. Allowed values include also "asis"
(TRUE) and "flip". If merge = "flip", then y variables are used as x tick
labels and the x variable is used as grouping variable.}

\item{color, fill}{outline and fill colors.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{size}{Numeric value (e.g.: size = 1). change the size of points and
outlines.}

\item{width}{numeric value between 0 and 1 specifying the width of the plot elements.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{error.plot}{plot type used to visualize error. Allowed values are one of
c("pointrange", "linerange", "crossbar", "errorbar", "upper_errorbar",
"lower_errorbar", "upper_pointrange", "lower_pointrange", "upper_linerange",
"lower_linerange"). Default value is "pointrange" or "errorbar". Used only
when add != "none" and add contains one "mean_*" or "med_*" where "*" = sd,
se, ....}

\item{label}{specify whether to add labels on the bar plot. Allowed values
are: \itemize{ \item \strong{logical value}: If TRUE, y values are added as
labels on the bar plot \item \strong{character vector}: Used as text
labels; must be the same length as y. }}

\item{lab.col, lab.size}{text color and size for labels.}

\item{lab.pos}{character specifying the position for labels. Allowed values
are "out" (for outside) or "in" (for inside). Ignored when lab.vjust !=
NULL.}

\item{lab.vjust}{numeric, vertical justification of labels. Provide negative
value (e.g.: -0.4) to put labels outside the bars or positive value to put
labels inside (e.g.: 2).}

\item{lab.hjust}{numeric, horizontal justification of labels.}

\item{lab.nb.digits}{integer indicating the number of decimal places (round) to be used.}

\item{sort.val}{a string specifying whether the value should be sorted.
Allowed values are "none" (no sorting), "asc" (for ascending) or "desc" (for descending).}

\item{sort.by.groups}{logical value. If TRUE the data are sorted by groups.
Used only when sort.val != "none".}

\item{top}{a numeric value specifying the number of top elements to be shown.}

\item{position}{position adjustment, either as a string, or the result of a
call to a position adjustment function (e.g. \code{position_dodge(0.8)}).
Used to control the spacing between grouped elements.}

\item{numeric.x.axis}{logical. If TRUE, x axis will be treated as numeric.
Default is FALSE. Useful, for example, to plot bars at their numeric x
positions (e.g. a time axis) instead of at equally-spaced discrete
categories. Ignored when \code{order} is set or \code{sort.val != "none"},
which require a discrete x axis.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to be passed to ggpar().}
}
\description{
Create a bar plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\section{Faceting a summarized bar plot}{

When the bars show a computed summary (e.g. \code{add = "mean_se"}), facet the
plot with the \code{facet.by} argument - \strong{not} by appending
\code{+ facet_wrap()} / \code{+ facet_grid()}. The summaries are pre-computed,
grouping by \code{x}, \code{color}/\code{fill} and \code{facet.by}; a facet added
afterwards is not part of that grouping, so the bars (and, for stacked bars, the
error bars) are pooled over the whole data set and repeated identically in every
panel. Use \code{ggbarplot(..., facet.by = "group")} for correct per-panel
summaries.
}

\examples{
# Data
df <- data.frame(
  dose = c("D0.5", "D1", "D2"),
  len = c(4.2, 10, 29.5)
)
print(df)

# Basic plot with label outsite
# +++++++++++++++++++++++++++
ggbarplot(df,
  x = "dose", y = "len",
  label = TRUE, label.pos = "out"
)

# Change width
ggbarplot(df, x = "dose", y = "len", width = 0.5)

# Change the plot orientation: horizontal
ggbarplot(df, "dose", "len", orientation = "horiz")

# Change the default order of items
ggbarplot(df, "dose", "len",
  order = c("D2", "D1", "D0.5")
)


# Change colors
# +++++++++++++++++++++++++++

# Change fill and outline color
# add labels inside bars
ggbarplot(df, "dose", "len",
  fill = "steelblue", color = "steelblue",
  label = TRUE, lab.pos = "in", lab.col = "white"
)

# Change colors by groups: dose
# Use custom color palette
ggbarplot(df, "dose", "len",
  color = "dose",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)

# Change fill and outline colors by groups
ggbarplot(df, "dose", "len",
  fill = "dose", color = "dose",
  palette = c("#00AFBB", "#E7B800", "#FC4E07")
)


# Plot with multiple groups
# +++++++++++++++++++++

# Create some data
df2 <- data.frame(
  supp = rep(c("VC", "OJ"), each = 3),
  dose = rep(c("D0.5", "D1", "D2"), 2),
  len = c(6.8, 15, 33, 4.2, 10, 29.5)
)
print(df2)

# Plot "len" by "dose" and change color by a second group: "supp"
# Add labels inside bars
ggbarplot(df2, "dose", "len",
  fill = "supp", color = "supp", palette = "Paired",
  label = TRUE, lab.col = "white", lab.pos = "in"
)

# Change position: Interleaved (dodged) bar plot
ggbarplot(df2, "dose", "len",
  fill = "supp", color = "supp", palette = "Paired",
  label = TRUE,
  position = position_dodge(0.9)
)

# Add points and errors
# ++++++++++++++++++++++++++

# Data: ToothGrowth data set we'll be used.
df3 <- ToothGrowth
head(df3, 10)

# It can be seen that for each group we have
# different values
ggbarplot(df3, x = "dose", y = "len")

# Visualize the mean of each group
ggbarplot(df3,
  x = "dose", y = "len",
  add = "mean"
)

# Add error bars: mean_se
# (other values include: mean_sd, mean_ci, median_iqr, ....)
# Add labels
ggbarplot(df3,
  x = "dose", y = "len",
  add = "mean_se", label = TRUE, lab.vjust = -1.6
)

# Use only "upper_errorbar"
ggbarplot(df3,
  x = "dose", y = "len",
  add = "mean_se", error.plot = "upper_errorbar"
)

# Change error.plot to "pointrange"
ggbarplot(df3,
  x = "dose", y = "len",
  add = "mean_se", error.plot = "pointrange"
)

# Add jitter points and errors (mean_se)
ggbarplot(df3,
  x = "dose", y = "len",
  add = c("mean_se", "jitter")
)

# Add dot and errors (mean_se)
ggbarplot(df3,
  x = "dose", y = "len",
  add = c("mean_se", "dotplot")
)

# Multiple groups with error bars and jitter point
ggbarplot(df3,
  x = "dose", y = "len", color = "supp",
  add = "mean_se", palette = c("#00AFBB", "#E7B800"),
  position = position_dodge()
)
#

}
\seealso{
\code{\link{ggpar}}, \code{\link{ggline}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdotchart.R
\name{ggdotchart}
\alias{ggdotchart}
\alias{theme_cleveland}
\title{Cleveland's Dot Plots}
\usage{
ggdotchart(
  data,
  x,
  y,
  group = NULL,
  combine = FALSE,
  color = "black",
  palette = NULL,
  shape = 19,
  size = NULL,
  dot.size = size,
  sorting = c("ascending", "descending", "none"),
  add = c("none", "segment"),
  add.params = list(),
  x.text.col = TRUE,
  rotate = FALSE,
  title = NULL,
  xlab = NULL,
  ylab = NULL,
  facet.by = NULL,
  panel.labs = NULL,
  short.panel.labs = TRUE,
  select = NULL,
  remove = NULL,
  order = NULL,
  label = NULL,
  font.label = list(size = 11, color = "black"),
  label.select = NULL,
  repel = FALSE,
  label.rectangle = FALSE,
  position = "identity",
  ggtheme = theme_pubr(),
  ...
)

theme_cleveland(rotate = TRUE)
}
\arguments{
\item{data}{a data frame}

\item{x, y}{x and y variables for drawing.}

\item{group}{an optional column name indicating how the elements of x are
grouped.}

\item{combine}{logical value. Default is FALSE. Used only when y is a vector
containing multiple variables to plot. If TRUE, create a multi-panel plot by
combining the plot of y variables.}

\item{color, size}{points color and size.}

\item{palette}{the color palette to be used for coloring or filling by groups.
Allowed values include "grey" for grey color palettes; brewer palettes e.g.
"RdBu", "Blues", ...; or custom color palette e.g. c("blue", "red"); and
scientific journal palettes from ggsci R package, e.g.: "npg", "aaas",
"lancet", "jco", "ucscgb", "uchicago", "simpsons" and "rickandmorty".}

\item{shape}{point shape. See \code{\link{show_point_shapes}}.}

\item{dot.size}{numeric value specifying the dot size.}

\item{sorting}{a character vector for sorting into ascending or descending
order. Allowed values are one of "descending", "ascending" and "none". Partial
match are allowed (e.g. sorting = "desc" or "asc"). Default is
"descending".}

\item{add}{character vector for adding another plot element (e.g.: dot plot or
error bars). Allowed values are one or the combination of: "none",
"dotplot", "jitter", "boxplot", "point", "mean", "mean_se", "mean_sd",
"mean_ci", "mean_range", "median", "median_iqr", "median_hilow",
"median_q1q3", "median_mad", "median_range"; see ?desc_statby for more
details.}

\item{add.params}{parameters (color, shape, size, fill, linetype) for the
argument 'add'; e.g.: add.params = list(color = "red").}

\item{x.text.col}{logical. If TRUE (default), x axis texts are colored by
groups.}

\item{rotate}{logical value. If TRUE, rotate the graph by setting the plot
orientation to horizontal.}

\item{title}{plot main title.}

\item{xlab}{character vector specifying x axis labels. Use xlab = FALSE to
hide xlab.}

\item{ylab}{character vector specifying y axis labels. Use ylab = FALSE to
hide ylab.}

\item{facet.by}{character vector, of length 1 or 2, specifying grouping
variables for faceting the plot into multiple panels. Should be in the data.}

\item{panel.labs}{a list of one or two character vectors to modify facet panel
labels. For example, panel.labs = list(sex = c("Male", "Female")) specifies
the labels for the "sex" variable. For two grouping variables, you can use
for example panel.labs = list(sex = c("Male", "Female"), rx = c("Obs",
"Lev", "Lev2") ).}

\item{short.panel.labs}{logical value. Default is TRUE. If TRUE, create short
labels for panels by omitting variable names; in other words panels will be
labelled only by variable grouping levels.}

\item{select}{character vector specifying which items to display.}

\item{remove}{character vector specifying which items to remove from the plot.}

\item{order}{character vector specifying the order of items.}

\item{label}{the name of the column containing point labels.}

\item{font.label}{a list which can contain the combination of the following
elements: the size (e.g.: 14), the style (e.g.: "plain", "bold", "italic",
"bold.italic") and the color (e.g.: "red") of labels. For example font.label
= list(size = 14, face = "bold", color ="red"). To specify only the size and
the style, use font.label = list(size = 14, face = "plain").}

\item{label.select}{can be of two formats: \itemize{ \item a character vector
specifying some labels to show. \item a list containing one or the
combination of the following components: \itemize{ \item \code{top.up} and
\code{top.down}: to display the labels  of the top up/down points. For
example, \code{label.select = list(top.up = 10, top.down = 4)}. \item
\code{criteria}: to filter, for example, by x and y variables values, use
this: \code{label.select = list(criteria = "`y` > 2 & `y` < 5 & `x` \%in\%
c('A', 'B')")}. } }}

\item{repel}{a logical value, whether to use ggrepel to avoid overplotting
text labels or not.}

\item{label.rectangle}{logical value. If TRUE, add rectangle underneath the
text, making it easier to read.}

\item{position}{Position adjustment, either as a string, or the result of a
call to a position adjustment function.}

\item{ggtheme}{function, ggplot2 theme name. Default value is theme_pubr(). Set ggtheme = NULL to skip applying a ggpubr theme, so the plot keeps ggplot2 default theme or the theme set globally via theme_set().
Allowed values include ggplot2 official themes: theme_gray(), theme_bw(),
theme_minimal(), theme_classic(), theme_void(), ....}

\item{...}{other arguments to be passed to \code{\link[ggplot2]{geom_point}}
and \code{\link{ggpar}}.}
}
\description{
Draw a Cleveland dot plot.
}
\details{
The plot can be easily customized using the function ggpar(). Read
  ?ggpar for changing: \itemize{ \item main title and axis labels: main,
  xlab, ylab \item axis limits: xlim, ylim (e.g.: ylim = c(0, 30)) \item axis
  scales: xscale, yscale (e.g.: yscale = "log2") \item color palettes:
  palette = "Dark2" or palette = c("gray", "blue", "red") \item legend title,
  labels and position: legend = "right" \item plot orientation : orientation
  = c("vertical", "horizontal", "reverse") }
}
\examples{
# Load data
data("mtcars")
df <- mtcars
df$cyl <- as.factor(df$cyl)
df$name <- rownames(df)
head(df[, c("wt", "mpg", "cyl")], 3)

# Basic plot
ggdotchart(df,
  x = "name", y = "mpg",
  ggtheme = theme_bw()
)

# Change colors by  group cyl
ggdotchart(df,
  x = "name", y = "mpg",
  group = "cyl", color = "cyl",
  palette = c("#999999", "#E69F00", "#56B4E9"),
  rotate = TRUE,
  sorting = "descending",
  ggtheme = theme_bw(),
  y.text.col = TRUE
)


# Plot with multiple groups
# +++++++++++++++++++++
# Create some data
df2 <- data.frame(
  supp = rep(c("VC", "OJ"), each = 3),
  dose = rep(c("D0.5", "D1", "D2"), 2),
  len = c(6.8, 15, 33, 4.2, 10, 29.5)
)
print(df2)

ggdotchart(df2,
  x = "dose", y = "len",
  color = "supp", size = 3,
  add = "segment",
  add.params = list(color = "lightgray", linewidth = 1.5),
  position = position_dodge(0.3),
  palette = "jco",
  ggtheme = theme_pubclean()
)

}
\seealso{
\code{\link{ggpar}}
}
